@@ -20,8 +20,6 @@
 #ifndef BOXPLOTS_H
 #define BOXPLOTS_H
 
-extern char boxplottmp[MAXLEN];
-
 int boxplots (int *list, char **bools, 
 	      double ***pZ, const DATAINFO *pdinfo, 
 	      gretlopt opt);
@@ -37,12 +35,10 @@ int ps_print_plots (const char *fname, int flag, gpointer data);
 int plot_to_xpm (const char *fname, gpointer data);
 #endif
 
-int augment_boxplot_count (void);
-
-void zero_boxplot_count (void);
-
 const char *get_boxplots_string (void);
 
+const char *get_boxdump_name (void);
+
 #endif /* BOXPLOTS_H */
 
 
