@@ -41,8 +41,6 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget);
 
 void model_genr_callback (gpointer data, guint u, GtkWidget *widget);
 
-void run_script_callback (GtkWidget *w, gpointer data);
-
 void gp_send_callback (GtkWidget *w, gpointer data);
 
 void file_save_callback (GtkWidget *w, gpointer data);
@@ -51,6 +49,8 @@ void add_rand_callback (gpointer data, guint r, GtkWidget *widget);
 
 void newdata_callback (gpointer data, guint pd_code, GtkWidget *widget);
 
+void xcorrgm_callback (gpointer p, guint v, GtkWidget *w);
+
 void do_nistcheck (gpointer p, guint u, GtkWidget *w);
 
 #if defined (ENABLE_MAILER) && !defined(G_OS_WIN32)
