@@ -22,8 +22,9 @@
 #include "lib_private.h"
 #include "cmd_private.h"
 #include "fileselect.h"
+#include "session.h"
 
-#undef CMD_DEBUG
+#define CMD_DEBUG 0
 
 typedef struct {
     int ID;
@@ -67,6 +68,7 @@ void free_command_stack (void)
 
 int add_command_to_stack (const char *str)
 {
+    int n = strlen(str);
     char **cmds;
 
     cmds = myrealloc(cmd_stack, (n_cmds + 1) * sizeof *cmds);
@@ -76,12 +78,26 @@ int add_command_to_stack (const char *str)
 
     cmd_stack = cmds;
 
-    cmd_stack[n_cmds] = gretl_strdup(str);
+    if (n > 0 && str[n-1] == '\n') {
+	cmd_stack[n_cmds] = gretl_strdup(str);
+    } else {
+	cmd_stack[n_cmds] = g_strdup_printf("%s\n", str);
+    }
+
     if (cmd_stack[n_cmds] == NULL) {
 	return 1;
     }
 
-#ifdef CMD_DEBUG
+    if (strlen(str) > 2 && 
+	strncmp(str, "help", 4) &&
+	strncmp(str, "info", 4) &&
+	strncmp(str, "list", 4) &&
+	strncmp(str, "open", 4) &&
+	strncmp(str, "quit", 4)) {
+	mark_session_changed();
+    }
+
+#if CMD_DEBUG
     fprintf(stderr, "added to stack as cmd_stack[%d]:\n"
 	    " %s\n", n_cmds, cmd_stack[n_cmds]);
 #endif
@@ -93,8 +109,10 @@ int add_command_to_stack (const char *str)
 
 void delete_last_command (void)
 {
-    free(cmd_stack[n_cmds - 1]);
-    n_cmds--;
+    if (n_cmds > 0) {
+	free(cmd_stack[n_cmds - 1]);
+	n_cmds--;
+    }
 }
 
 static model_stack *add_model_stack (int model_id)
@@ -114,7 +132,7 @@ static model_stack *add_model_stack (int model_id)
     mstacks[nm].n = 0;
     mstacks[nm].cmds = NULL;
 
-#ifdef CMD_DEBUG
+#if CMD_DEBUG
     fprintf(stderr, "add_model_stack:\n"
 	    " mstacks[%d]: ID=%d\n", nm, model_id);
 #endif
@@ -122,6 +140,24 @@ static model_stack *add_model_stack (int model_id)
     return &mstacks[nm];
 }
 
+static int mstack_delete_last_command (model_stack *mstack)
+{
+    int nc = mstack->n;
+    char **tmp;
+
+    free(mstack->cmds[nc-1]);
+    mstack->n -= 1;
+
+    tmp = realloc(mstack->cmds, (nc - 1) * sizeof *tmp);
+    if (tmp == NULL) {
+	return 1;
+    }
+
+    mstack->cmds = tmp;
+
+    return 0;
+}
+
 static int add_command_to_mstack (model_stack *mstack, const char *str)
 {
     int nc = mstack->n;
@@ -141,7 +177,9 @@ static int add_command_to_mstack (model_stack *mstack, const char *str)
 
     mstack->n += 1;
 
-#ifdef CMD_DEBUG
+
+
+#if CMD_DEBUG
     fprintf(stderr, "add_command_to_mstack, with ID=%d:\n"
 	    " %s\n", mstack->ID, str);
 #endif
@@ -195,7 +233,7 @@ int model_command_init (int model_ID)
 	return 1;
     }
 
-    echo_cmd(libcmd, datainfo, line, 0, echo);
+    echo_cmd(libcmd, datainfo, line, CMD_RECORDING, echo);
 
     if (add_command_to_mstack(mstack, gretl_print_get_buffer(echo))) {
 	err = 1;
@@ -206,11 +244,21 @@ int model_command_init (int model_ID)
     return err;
 }
 
+void model_command_delete (int model_ID) 
+{
+    model_stack *mstack;
+
+    mstack = mstack_from_model_id(model_ID);
+    if (mstack != NULL && mstack->n > 0) {
+	mstack_delete_last_command(mstack);
+    }
+}
+
 static void dump_model_cmds (const model_stack *mstack, FILE *fp)
 {
     int i;
 
-    fprintf(fp, "(* commands pertaining to model %d *)\n", mstack->ID);
+    fprintf(fp, "/* commands pertaining to model %d */\n", mstack->ID);
 
     for (i=0; i<mstack->n; i++) {
 	fprintf(fp, "%s", mstack->cmds[i]);
@@ -218,11 +266,12 @@ static void dump_model_cmds (const model_stack *mstack, FILE *fp)
 }
 
 /* get the ID number of a variable operated upon by 
-   "genr" or "label" 
+   "genr" or "setinfo" 
 */
 
 static int vnum_from_data_command (const char *s)
 {
+    
     char vname[VNAMELEN];
     char format[8];
     int offset = 6;
@@ -261,8 +310,8 @@ static int parse_store_cmd (const char *sline, CMD *scmd)
     return err;
 }
 
-/* check if, so far as we can tell, a given modified variable has been
-   saved to the current datafile
+/* So far as we can tell, has a given modified variable been
+   saved to the current datafile?
 */
 
 static int var_is_stored (int v, int pos)
@@ -305,15 +354,12 @@ static char *mark_redundant_commands (void)
 
     for (i=0; i<n_cmds; i++) {
 	s = cmd_stack[i];
-	if (!strncmp(s, "genr", 4) || !strncmp(s, "label", 5)) {
+	if (!strncmp(s, "genr", 4) || !strncmp(s, "setinfo", 7)) {
 	    int v = vnum_from_data_command(s);
 
 	    if (v > 0 && var_is_stored(v, i)) {
 		drop[i] = 1;
 	    }
-	} else if (!strcmp(s, "corr") || !strcmp(s, "summary")) {
-	    /* available as session objects by default */
-	    drop[i] = 1;
 	}
     }
 
@@ -354,6 +400,17 @@ static int maybe_prepend_open_data (FILE *fp)
     return cancel;
 }
 
+static int creates_model (const char *cmdword)
+{
+    if (is_model_cmd(cmdword) ||
+	!strcmp(cmdword, "addto") ||
+	!strcmp(cmdword, "omitfrom")) {
+	return 1;
+    }
+
+    return 0;
+}
+
 /* ship out the stack of commands entered in the current session */
 
 int dump_command_stack (const char *fname, int insert_open_data)
@@ -410,8 +467,8 @@ int dump_command_stack (const char *fname, int insert_open_data)
 	    sscanf(cmd_stack[i], "%8s", cmdword);
 	}
 	
-	if (is_model_cmd(cmdword)) {
-#ifdef CMD_DEBUG
+	if (creates_model(cmdword)) {
+#if CMD_DEBUG
 	    fprintf(stderr, "cmd_stack[%d]: looking for model commands\n", i);
 #endif
 	    mstack = mstack_from_model_id(++modnum);
@@ -434,21 +491,21 @@ int dump_command_stack (const char *fname, int insert_open_data)
 
 void view_command_log (void)
 {
-    char fname[MAXLEN];
-    
+    char logfile[MAXLEN];
+
     if (n_cmds == 0) {
 	errbox(_("The command log is empty"));
 	return;
     }
 
-    strcpy(fname, paths.userdir);
-    strcat(fname, "session.inp");
+    strcpy(logfile, paths.userdir);
+    strcat(logfile, "session.inp");
 
-    if (dump_command_stack(fname, 0)) {
+    if (dump_command_stack(logfile, 0)) {
 	return;
     }
 
-    view_file(fname, 0, 0, 78, 370, VIEW_LOG);
+    view_file(logfile, 0, 0, 78, 370, VIEW_LOG);
 }
 
 /* See whether user has done any work, to determine whether or not to
