@@ -24,11 +24,14 @@
 #include "menustate.h"
 
 #include "libset.h"
+#include "monte_carlo.h"
 #include "gretl_func.h"
+#include "cmd_private.h"
 
 static GtkWidget *console_view;
 static PRN *console_prn;
 static gchar *cbuf;
+static ExecState cstate;
 
 #define DEFAULT_HLINES 32
 
@@ -67,6 +70,9 @@ static int gretl_console_init (void)
 
     set_gretl_echo(1);
 
+    gretl_exec_state_init(&cstate, CONSOLE_EXEC, NULL, 
+			  get_lib_cmd(), models, NULL);
+
     return 0;
 }
 
@@ -142,9 +148,7 @@ static char *pop_history_line (int keyval)
 	} else {
 	    ret = cmd_history[hl];
 	}
-    }
-
-    else if (keyval == GDK_Down) {
+    } else if (keyval == GDK_Down) {
 	if (hl >= 0) hl--;
 	if (hl < 0) {
 	    if (beeptime) {
@@ -212,22 +216,23 @@ int console_sample_changed (const DATAINFO *pdinfo)
     return console_sample_handler(pdinfo, SAMPLE_CHECK);
 }
 
-static int console_function_exec (char *execline)
+static int console_vars_changed (const char *line, int oldv, int err)
 {
-    char *gotline = NULL;
-    int err = 0;
+    int ret = 0;
 
-    while (!gretl_execute_loop()) {
-	gotline = gretl_function_get_line(execline, MAXLINE, &Z, &datainfo, &err);
-	if (gotline == NULL || *gotline == '\0') {
-	    break;
-	}
-	if (!err) {
-	    err = gui_exec_line(execline, console_prn, SCRIPT_EXEC, NULL);
+    if (datainfo->v != oldv) {
+	ret = 1;
+    } else if (!err) {
+	if (!strncmp(line, "rename", 6) ||
+	    !strncmp(line, "setinfo", 7) ||
+	    !strncmp(line, "genr", 4) ||
+	    !strncmp(line, "series", 6) ||
+	    !strncmp(line, "scalar", 6)) {
+	    ret = 1;
 	}
     }
 
-    return err;
+    return ret;
 }
 
 static void console_exec (void)
@@ -237,6 +242,7 @@ static void console_exec (void)
     static int redirected;
     int oldv = datainfo->v;
     char execline[MAXLINE];
+    int coding = 0;
     int err = 0;
 
     buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(console_view));
@@ -248,8 +254,7 @@ static void console_exec (void)
 
     if (strcmp(cbuf, "quit") == 0 || 
 	strcmp(cbuf, "q") == 0 ||
-	strcmp(cbuf, "exit") == 0 ||
-	strcmp(cbuf, "x") == 0) {
+	strcmp(cbuf, "exit") == 0) {
 	gtk_widget_destroy(console_view->parent->parent->parent);
 	g_free(cbuf);
 	cbuf = NULL;
@@ -269,17 +274,17 @@ static void console_exec (void)
 
     console_record_sample(datainfo);
 
-    push_history_line(execline); 
+    push_history_line(execline);
+
+    cstate.line = execline;
+    cstate.flags = CONSOLE_EXEC;
+    cstate.prn = console_prn;
 
     /* actually execute the command line */
-    err = gui_exec_line(execline, console_prn, CONSOLE_EXEC, NULL);
+    err = gui_exec_line(&cstate, &Z, &datainfo);
 
-    while (!err && (gretl_execute_loop() || gretl_executing_function())) {
-	if (gretl_execute_loop()) { 
-	    err = gretl_loop_exec(execline, &Z, &datainfo, models, console_prn);
-	} else if (gretl_executing_function()) {
-	    err = console_function_exec(execline);
-	} 
+    while (!err && gretl_execute_loop()) {
+	err = gretl_loop_exec(&cstate, &Z, &datainfo);
     }
 
     redirected = printing_is_redirected(console_prn);
@@ -295,8 +300,10 @@ static void console_exec (void)
 	    gchar *trbuf = my_locale_to_utf8(cbuf);
 
 	    fprintf(stderr, "console text did not validate as utf8\n");
-	    gtk_text_buffer_insert(buf, &start, trbuf, strlen(trbuf));
-	    g_free(trbuf);
+	    if (trbuf != NULL) {
+		gtk_text_buffer_insert(buf, &start, trbuf, strlen(trbuf));
+		g_free(trbuf);
+	    }
 	} else {
 	    gtk_text_buffer_insert(buf, &start, cbuf, strlen(cbuf));
 	}
@@ -306,15 +313,18 @@ static void console_exec (void)
 	gretl_print_reset_buffer(console_prn);
     }
 
+    coding = gretl_compiling_loop() || gretl_compiling_function();
+
     gtk_text_buffer_insert_with_tags_by_name(buf, &start, 
-					     (gretl_compiling_loop())? "\n> " : "\n? ", 
-					     3, "redtext", NULL);
+					     (coding)? "> " : "? ", 
+					     2, "redtext", NULL);
 
     /* scroll to end of buffer */
     console_scroll_to_end(buf, &start);
 
     /* update variable listing in main window if needed */
-    if (datainfo->v != oldv || !strncmp(execline, "rename", 6)) {
+    if (console_vars_changed(execline, oldv, err)) {
+	mark_dataset_as_modified();
 	populate_varlist();
     }
 
@@ -418,7 +428,9 @@ gint console_key_handler (GtkWidget *w, GdkEventKey *key, gpointer d)
     last_line = gtk_text_buffer_get_line_count(buf) - 1;
 
     /* if at start of command line, backspacing does nothing */
-    if (IS_BACKKEY(key->keyval) && line_pos < 3) return TRUE;
+    if (IS_BACKKEY(key->keyval) && line_pos < 3) {
+	return TRUE;
+    }
 
     /* if not on prompt line, return to (the end of) it */
     if (curr_line < last_line) {
