@@ -23,7 +23,7 @@
 #include "boxplots.h"
 #include "database.h"
 #include "datafiles.h"
-#include "webget.h"
+#include "gretl_www.h"
 #include "menustate.h"
 #include "treeutils.h"
 
@@ -35,9 +35,14 @@
 #include <errno.h>
 
 #if G_BYTE_ORDER == G_BIG_ENDIAN
-#include <netinet/in.h>
+# include <netinet/in.h>
 #endif
 
+#ifdef G_OS_WIN32
+# include "gretlwin32.h"
+#endif
+
+
 /* private functions */
 static GtkWidget *database_window (windata_t *vwin);
 static int make_local_db_series_list (windata_t *vwin);
@@ -52,7 +57,7 @@ enum db_data_actions {
     DB_IMPORT
 };
 
-int validate_string (char *orig)
+static int utf8_correct (char *orig)
 {
     int err = 0;
 
@@ -116,9 +121,16 @@ float retrieve_float (netfloat nf)
 }
 #endif
 
-void show_network_error (windata_t *vwin, char *buf)
+void show_network_error (windata_t *vwin)
 {
-    if (*buf != '\0') {
+    const char *msg = gretl_errmsg_get();
+    char *buf = NULL;
+
+    if (*msg != '\0') {
+	buf = gretl_strdup(msg);
+    }
+
+    if (buf != NULL) {
 	size_t n = strlen(buf);
 
 	if (buf[n-1] == '\n') {
@@ -129,6 +141,7 @@ void show_network_error (windata_t *vwin, char *buf)
 	} else {
 	    errbox(buf);
 	}
+	free(buf);
     } else {
 	if (vwin != NULL) {
 	    update_statusline(vwin, _("Error retrieving data from server"));
@@ -138,62 +151,24 @@ void show_network_error (windata_t *vwin, char *buf)
     }
 }
 
-static int get_remote_db_data (windata_t *vwin, SERIESINFO *sinfo, 
-			       double **Z)
+static int gui_get_remote_db_data (windata_t *vwin, SERIESINFO *sinfo, 
+				   double **Z)
 {
-    char *getbuf = NULL;
-    char errbuf[80];
     char *dbbase = vwin->fname;
-    int t, err, n = sinfo->nobs;
-    dbnumber val;
-    size_t offset;
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-    netfloat nf;
-#endif
-
-    *errbuf = '\0';
+    int err;
 
     update_statusline(vwin, _("Retrieving data..."));
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-    err = retrieve_remote_db_data(dbbase, sinfo->varname, &getbuf,
-				  errbuf, GRAB_NBO_DATA);
-#else
-    err = retrieve_remote_db_data(dbbase, sinfo->varname, &getbuf,
-				  errbuf, GRAB_DATA);
-#endif
+    err = get_remote_db_data(dbbase, sinfo, Z);
 
     if (err) {
-	show_network_error(vwin, errbuf);
-	free(getbuf);
+	show_network_error(vwin);
 	return E_FOPEN;
-    } 
-
-    offset = 0L;
-    for (t=0; t<n; t++) {
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-	/* go via network byte order */
-	memcpy(&(nf.frac), getbuf + offset, sizeof nf.frac);
-	offset += sizeof nf.frac;
-	memcpy(&(nf.exp), getbuf + offset, sizeof nf.exp);
-	offset += sizeof nf.exp;
-	val = retrieve_float(nf);
-#else
-	/* just read floats */
-	memcpy(&val, getbuf + offset, sizeof val);
-	offset += sizeof val;
-#endif
-	if (val == -999.0) {
-	    Z[1][t] = NADBL;
-	} else {
-	    Z[1][t] = val;
-	}
+    } else {
+	update_statusline(vwin, "OK");
     }
 
-    update_statusline(vwin, "OK");
-    free(getbuf);
-
-    return 0;
+    return err;
 }
 
 static void display_dbdata (const double **dbZ, DATAINFO *dbdinfo)
@@ -204,14 +179,13 @@ static void display_dbdata (const double **dbZ, DATAINFO *dbdinfo)
 	return;
     }
 
-    printdata(NULL, dbZ, dbdinfo, OPT_O, prn);
+    printdata(NULL, NULL, dbZ, dbdinfo, OPT_O, prn);
     view_buffer(prn, 36, 350, _("gretl: display database series"), PRINT,
 		NULL); 
 }
 
 static void graph_dbdata (double ***dbZ, DATAINFO *dbdinfo)
 {
-    int lines[1] = {1};
     int list[2] = {1, 1};
     int err;
 
@@ -223,8 +197,8 @@ static void graph_dbdata (double ***dbZ, DATAINFO *dbdinfo)
 	return;
     }
 
-    err = gnuplot(list, lines, NULL, dbZ, dbdinfo,
-		  &plot_count, GP_GUI | GP_IDX);
+    err = gnuplot(list, NULL, (const double **) *dbZ, dbdinfo,
+		  OPT_G | OPT_O | OPT_T);
 
     if (err) {
 	errbox(_("gnuplot command failed"));
@@ -256,7 +230,7 @@ static void add_dbdata (windata_t *vwin, double **dbZ, SERIESINFO *sinfo)
 	int dbv;
 
 	if (check_db_import(sinfo, datainfo)) {
-	    errbox(get_gretl_errmsg());
+	    gui_errmsg(1);
 	    return;
 	}
 
@@ -459,7 +433,7 @@ void gui_get_db_series (gpointer p, guint action, GtkWidget *w)
     if (dbcode == NATIVE_SERIES) { 
 	err = get_native_db_data(vwin->fname, sinfo, dbZ);
     } else if (dbcode == REMOTE_SERIES) {
-	err = get_remote_db_data(vwin, sinfo, dbZ);
+	err = gui_get_remote_db_data(vwin, sinfo, dbZ);
     } else if (dbcode == RATS_SERIES) {
 	err = get_rats_db_data(vwin->fname, sinfo, dbZ);
     } else if (dbcode == PCGIVE_SERIES) {
@@ -804,7 +778,7 @@ static int make_local_db_series_list (windata_t *vwin)
 	    continue;
 	}
 
-	err = validate_string(line1);
+	err = utf8_correct(line1);
 
 	end_trim(line1);
 	charsub(line1, '\t', ' ');
@@ -870,7 +844,7 @@ static int make_remote_db_series_list (windata_t *vwin, char *buf)
 	end_trim(line1);
 	charsub(line1, '\t', ' ');
 
-	err = validate_string(line1);
+	err = utf8_correct(line1);
 
 	if (sscanf(line1, "%15s", sername) != 1) {
 	    break;
@@ -897,6 +871,8 @@ static int make_remote_db_series_list (windata_t *vwin, char *buf)
 	offset += nobs;
     }
 
+    bufgets_finalize(buf);
+
     db_drag_connect(vwin);
 
     return 0;
@@ -1010,14 +986,12 @@ static int make_rats_db_series_list (windata_t *vwin)
     fclose(fp);
 
     if (tbl == NULL) {
-	errbox(get_gretl_errmsg());
+	gui_errmsg(1);
 	return 1;
     }
 
     insert_and_free_db_table(tbl, vwin->listbox);
-
     vwin->active_var = 0;
-
     db_drag_connect(vwin);
 
     return 0;
@@ -1044,7 +1018,7 @@ static int make_pcgive_db_series_list (windata_t *vwin)
     fclose(fp);
 
     if (tbl == NULL) {
-	errbox(get_gretl_errmsg());
+	gui_errmsg(1);
 	return 1;
     }
 
@@ -1163,42 +1137,14 @@ static SERIESINFO *get_series_info (windata_t *vwin, int action)
     return sinfo;
 }
 
-static int has_rats_suffix (const char *dbname)
-{
-    const char *p = strrchr(dbname, '.');
-    int ret = 0;
-
-    if (p != NULL) {
-	if (!strcmp(p, ".rat") || 
-	    !strcmp(p, ".Rat") || 
-	    !strcmp(p, ".RAT")) {
-	    ret = 1;
-	}
-    }
-
-    return ret;
-}
-
-static int has_pcgive_suffix (const char *dbname)
-{
-    const char *p = strrchr(dbname, '.');
-    int ret = 0;
-
-    if (p != NULL && !strcmp(p, ".bn7")) {
-	ret = 1;
-    }
-
-    return ret;
-}
-
 void open_named_db_index (char *dbname)
 {
     int action;
     FILE *fp;
 
-    if (has_rats_suffix(dbname)) {
+    if (has_suffix(dbname, ".rat")) {
 	action = RATS_SERIES;
-    } else if (has_pcgive_suffix(dbname)) {
+    } else if (has_suffix(dbname, ".bn7")) {
 	action = PCGIVE_SERIES;
     } else {
 	action = NATIVE_SERIES;
@@ -1219,6 +1165,8 @@ void open_named_db_index (char *dbname)
     } 
 }
 
+#define KEEP_BROWSER_OPEN 1
+
 void open_db_index (GtkWidget *w, gpointer data)
 {
     gchar *fname = NULL, *dbdir = NULL;
@@ -1229,7 +1177,7 @@ void open_db_index (GtkWidget *w, gpointer data)
     tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
 			 vwin->active_var, 0, &fname);
 
-    if (has_rats_suffix(fname)) {
+    if (has_suffix(fname, ".rat")) {
 	action = RATS_SERIES;
     }
 
@@ -1253,15 +1201,12 @@ void open_db_index (GtkWidget *w, gpointer data)
 void open_named_remote_db_index (char *dbname)
 {
     char *getbuf = NULL;
-    char errbuf[80];
     int err;
 
-    *errbuf = '\0';
-
-    err = retrieve_remote_db_index(dbname, &getbuf, errbuf);
+    err = retrieve_remote_db_index(dbname, &getbuf);
 
     if (err) {
-	show_network_error(NULL, errbuf);
+	show_network_error(NULL);
     } else if (getbuf != NULL && !strncmp(getbuf, "Couldn't open", 13)) {
 	errbox(getbuf);
     } else {
@@ -1274,22 +1219,30 @@ void open_named_remote_db_index (char *dbname)
 
 void open_remote_db_index (GtkWidget *w, gpointer data)
 {
+    GtkTreeIter iter;
+    GtkTreeModel *model;
+    GtkTreeSelection *sel;
     char *getbuf = NULL;
-    char errbuf[80];
-    gchar *fname;
+    gchar *fname = NULL;
     windata_t *vwin = (windata_t *) data;
     int err;
 
-    *errbuf = '\0';
+    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(vwin->listbox));
+    if (!gtk_tree_selection_get_selected(sel, &model, &iter)) {
+	return;
+    }
 
-    tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
-			 vwin->active_var, 0, &fname);
+    gtk_tree_model_get(model, &iter, 0, &fname, -1);
+    if (fname == NULL || *fname == '\0') {
+	g_free(fname);
+	return;
+    }
 
     update_statusline(vwin, _("Retrieving data..."));
-    err = retrieve_remote_db_index(fname, &getbuf, errbuf);
+    err = retrieve_remote_db_index(fname, &getbuf);
 
     if (err) {
-	show_network_error(vwin, errbuf);
+	show_network_error(vwin);
     } else {
 	update_statusline(vwin, "OK");
 	make_db_series_list(REMOTE_SERIES, fname, getbuf);
@@ -1411,6 +1364,7 @@ static int ggz_extract (char *errbuf, char *ggzname)
     }
 
     bytesleft = datalen;
+
     while (bytesleft > 0) {
 #if G_BYTE_ORDER == G_BIG_ENDIAN
         if ((bgot = gzread(fgz, gzbuf, sizeof(long) + sizeof(short))) > 0) {
@@ -1438,6 +1392,7 @@ static int ggz_extract (char *errbuf, char *ggzname)
     }
 
     bytesleft = cblen;
+
     while (bytesleft > 0) {
 	memset(gzbuf, 0, GRETL_BUFSIZE);
 	bgot = gzread(fgz, gzbuf, (bytesleft > GRETL_BUFSIZE)? 
@@ -1477,12 +1432,29 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 {
     gchar *objname;
     char fndir[MAXLEN];
-    char *target, errbuf[80];
+    char *target;
     FILE *fp;
     int err = 0;
 
-    tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
-			 vwin->active_var, 0, &objname);
+    if (vwin->role == REMOTE_DB) {
+	GtkTreeSelection *sel;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(vwin->listbox));
+	if (!gtk_tree_selection_get_selected(sel, &model, &iter)) {
+	    return 1;
+	}
+
+	gtk_tree_model_get(model, &iter, 0, &objname, -1);
+	if (objname == NULL || *objname == '\0') {
+	    g_free(objname);
+	    return 1;
+	}
+    } else {
+	tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
+			     vwin->active_var, 0, &objname);
+    }
     
     target = mymalloc(MAXLEN);
     if (target == NULL) {
@@ -1506,8 +1478,10 @@ static int real_install_file_from_server (windata_t *vwin, int op)
     }
 
     /* try test write to target file */
+
     errno = 0;
     fp = gretl_fopen(target, "w");
+
     if (fp == NULL) {
 	if (errno == EACCES && vwin->role != REMOTE_FUNC_FILES) { 
 	    /* write to user dir instead? */
@@ -1521,20 +1495,18 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 	fclose(fp);
     }
 
-    *errbuf = '\0';
-
     if (vwin->role == REMOTE_FUNC_FILES) {
-	err = retrieve_remote_function_package(objname, target, errbuf);
+	err = retrieve_remote_function_package(objname, target);
     } else {
 #if G_BYTE_ORDER == G_BIG_ENDIAN
-	err = retrieve_remote_db(objname, target, errbuf, GRAB_NBO_DATA);
+	err = retrieve_remote_db(objname, target, GRAB_NBO_DATA);
 #else
-	err = retrieve_remote_db(objname, target, errbuf, GRAB_DATA);
+	err = retrieve_remote_db(objname, target, GRAB_DATA);
 #endif
     }
 
     if (err) {
-	show_network_error(NULL, errbuf);
+	show_network_error(NULL);
 	free(objname);
 	free(target);
 	return err;
@@ -1548,6 +1520,8 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 	    gui_show_function_info(target, VIEW_FUNC_INFO);
 	}
     } else {
+	char errbuf[80];
+
 	err = ggz_extract(errbuf, target);
 	if (err) {
 	    if (*errbuf == '\0') {
@@ -1620,7 +1594,7 @@ real_get_db_description (const char *fullname, const char *binname,
 	    char *s = tmp + 2;
 
 	    tailstrip(s);
-	    validate_string(s);
+	    utf8_correct(s);
 	    descrip = g_strdup(s);
 	}
     }
@@ -1784,7 +1758,7 @@ static int read_remote_filetime (char *line, char *fname, time_t *date)
        and <year> is 4-digit year; <day> is not used.
     */
 
-    if (sscanf(line, "%*s%*s%3s%2d%8s%4d%16s", 
+    if (sscanf(line, "%*s%*s%3s%2d%8s%4d%31s", 
 	       mon, &mday, hrs, &yr, fname) != 5) {
 	return 1;
     }
@@ -1812,42 +1786,145 @@ static int read_remote_filetime (char *line, char *fname, time_t *date)
     return 0;
 }
 
-/* fill a list box with names and short descriptions of either
-   databases or function packages, retrieved from server */
+/* below: mechanism for tucking individual databases under a twisty,
+   when there's a large number of databases from a single source.
+*/
 
-gint populate_remote_object_list (windata_t *vwin)
+static char *get_source_string (char *src, const char *s)
 {
-    GtkListStore *store;
-    GtkTreeIter iter;  
-    char *getbuf = NULL;
-    char line[1024];
-    char fname[16], errbuf[80], status[20];
-    gchar *row[3];
-    gint i;
-    time_t remtime;
-    int err = 0;
+    char *p;
 
-    *errbuf = '\0';
+    *src = 0;
+    strncat(src, s, 95);
+    
+    p = strchr(src, '(');
+    if (p == NULL) {
+	p = strstr(src, "--");
+    }
+    if (p != NULL) {
+	*p = '\0';
+    }
+    tailstrip(src);
 
-    if (vwin->role == REMOTE_DB) {
-	err = list_remote_dbs(&getbuf, errbuf);
-    } else {
-	err = list_remote_function_packages(&getbuf, errbuf);
+    return src;
+}
+
+struct src_info {
+    int start;
+    int ndb;
+};
+
+static struct src_info *dbsrc;
+static int n_src;
+
+static int push_src_info (int start, int ndb)
+{
+    struct src_info *tmp;
+
+    tmp = realloc(dbsrc, (n_src + 1) * sizeof *tmp);
+    if (tmp == NULL) {
+	return E_ALLOC;
     }
 
+    dbsrc = tmp;
+    dbsrc[n_src].start = start;
+    dbsrc[n_src].ndb = ndb;
+    n_src++;
+
+    return 0;
+}
+
+static int get_ndbs (int lineno)
+{
+    int i;
+
+    for (i=0; i<n_src; i++) {
+	if (dbsrc[i].start == lineno) {
+	    return dbsrc[i].ndb;
+	}
+    }
+    
+    return 1;
+}
+
+static void free_src_info (void)
+{
+    free(dbsrc);
+    dbsrc = NULL;
+    n_src = 0;
+}
+
+gint populate_remote_db_list (windata_t *vwin)
+{
+    GtkTreeStore *store;
+    GtkTreeIter iter, child_iter; 
+    char *getbuf = NULL;
+    char line[1024];
+    char fname[32], status[20];
+    char src[96], srcbak[96];
+    gchar *row[3];
+    time_t remtime;
+    int start, parent, kids;
+    int i, ndb, err = 0;
+
+    err = list_remote_dbs(&getbuf);
     if (err) {
-	show_network_error(NULL, errbuf);
+	show_network_error(NULL);
 	free(getbuf);
 	return err;
     }
 
-    store = GTK_LIST_STORE(gtk_tree_view_get_model 
-			   (GTK_TREE_VIEW(vwin->listbox)));
-    gtk_list_store_clear(store);
-    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+    i = 0;
+    ndb = start = 0;
+    src[0] = srcbak[0] = '\0';
+
+    /* first pass: figure "parentage" of databases */
 
     bufgets_init(getbuf);
+
+    while (bufgets(line, sizeof line, getbuf)) {
+	if (strstr(line, "idx")) {
+	    continue;
+	}
+	if (read_remote_filetime(line, fname, &remtime)) {
+	    continue;
+	}
+	if (bufgets(line, sizeof line, getbuf)) {
+	    get_source_string(src, line + 2);
+	    if (strcmp(src, srcbak)) {
+		if (ndb > 3) {
+		    push_src_info(start, ndb);
+		}
+		start = i;
+		ndb = 1;
+	    } else {
+		ndb++;
+	    }
+	    strcpy(srcbak, src);
+	}
+	i++;
+    }
+
+    bufgets_finalize(getbuf);
+
+    if (i == 0) {
+	errbox(_("No database files found"));
+	free_src_info();
+	free(getbuf);
+	return 1;
+    }
+
+    /* second pass: insert databases into tree view */
+
+    store = GTK_TREE_STORE(gtk_tree_view_get_model 
+			   (GTK_TREE_VIEW(vwin->listbox)));
+    gtk_tree_store_clear(store);
+    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+
     i = 0;
+    parent = kids = 0;
+
+    bufgets_init(getbuf);
 
     while (bufgets(line, sizeof line, getbuf)) {
 	if (strstr(line, "idx")) {
@@ -1859,29 +1936,126 @@ gint populate_remote_object_list (windata_t *vwin)
 
 	get_local_object_status(fname, vwin->role, status, remtime);
 	row[0] = strip_extension(fname);
+	row[2] = status;
 
 	if (bufgets(line, sizeof line, getbuf)) {
-	    validate_string(line);
+	    utf8_correct(line);
 	    row[1] = line + 2;
+	    ndb = get_ndbs(i);
+	    if (ndb > 1) {
+		get_source_string(src, row[1]);
+		parent = 1;
+		kids = ndb;
+	    }	
 	} else {
 	    row[1] = NULL;
 	}
 
-	row[2] = status;
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, 0, row[0], 1, row[1],
+	if (parent) {
+	    /* header for child databases */
+	    gtk_tree_store_append(store, &iter, NULL);
+	    gtk_tree_store_set(store, &iter, 0, "", 1, src, -1);
+	    parent = 0;
+	}
+	
+	if (kids > 0) {
+	    /* insert child under heading */
+	    gtk_tree_store_insert_before(store, &child_iter, 
+					 &iter, NULL);
+	    gtk_tree_store_set(store, &child_iter, 0, row[0], 
+			       1, row[1], 2, row[2], -1);
+	    kids--;
+	} else {
+	    /* insert at top level */
+	    gtk_tree_store_append(store, &iter, NULL);
+	    gtk_tree_store_set(store, &iter, 0, row[0], 1, row[1],
 			   2, row[2], -1);
+	}	    
+
 	i++;
     }
 
+    bufgets_finalize(getbuf);
+    free(getbuf);
+    free_src_info();
+
+    return err;
+}
+
+/* Fill a list box with names, version numbers, and short descriptions
+   of function packages, retrieved from server.
+*/
+
+gint populate_remote_func_list (windata_t *vwin)
+{
+    GtkListStore *store;
+    GtkTreeIter iter;  
+    char *getbuf = NULL;
+    char line[1024];
+    char fname[32], status[20];
+    char *basename;
+    char *descrip;
+    char *version;
+    time_t remtime;
+    int n, err = 0;
+
+    err = list_remote_function_packages(&getbuf);
+    if (err) {
+	show_network_error(NULL);
+	free(getbuf);
+	return err;
+    }
+
+    store = GTK_LIST_STORE(gtk_tree_view_get_model 
+			   (GTK_TREE_VIEW(vwin->listbox)));
+    gtk_list_store_clear(store);
+    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+
+    n = 0;
+    bufgets_init(getbuf);
+
+    while (bufgets(line, sizeof line, getbuf)) {
+	descrip = NULL;
+	version = NULL;
+
+	if (read_remote_filetime(line, fname, &remtime)) {
+	    continue;
+	}
+
+	get_local_object_status(fname, vwin->role, status, remtime);
+	basename = strip_extension(fname);
+
+	if (bufgets(line, sizeof line, getbuf)) {
+	    utf8_correct(line);
+	    descrip = gretl_strdup(line + 2);
+	} 
+
+	if (bufgets(line, sizeof line, getbuf)) {
+	    version = line + 2;
+	} 
+
+	if (descrip == NULL || version == NULL) {
+	    free(descrip);
+	    continue;
+	}
+
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 
+			   0, basename, 
+			   1, version,
+			   2, descrip, 
+			   3, status,
+			   -1);
+
+	free(descrip);
+	n++;
+    }
+
+    bufgets_finalize(getbuf);
     free(getbuf);
 
-    if (i == 0) {
-	if (vwin->role == REMOTE_DB) {
-	    errbox(_("No database files found"));
-	} else {
-	    errbox(_("No function packages found"));
-	}
+    if (n == 0) {
+	errbox(_("No function packages found"));
 	err = 1;
     }
 
@@ -1899,16 +2073,10 @@ gint populate_dbfilelist (windata_t *vwin)
 
     while (dir == NULL) {
 #ifdef G_OS_WIN32 
-	/* opendir doesn't work on e.g. c:\foo\ !! */
-	if (strlen(dbdir) > 3 && dbdir[strlen(dbdir) - 1] == '\\') {
-	    dbdir[strlen(dbdir) - 1] = '\0';
-	}
-	/* but neither does it work on e.g. f: */
-	if (dbdir[strlen(dbdir) - 1] == ':') {
-	    strcat(dbdir, "\\");
-	}
-#endif
+	dir = win32_opendir(dbdir);
+#else
 	dir = opendir(dbdir);
+#endif
 	if (dir == NULL) {
 	    errbox(_("Can't open folder %s"), dbdir);
 	    if (++tries == 2 || options_dialog(1)) {
