@@ -15,7 +15,9 @@ void file_info_from_server (GtkWidget *w, gpointer data);
 
 gint populate_dbfilelist (windata_t *ddata);
 
-gint populate_remote_object_list (windata_t *win);
+gint populate_remote_db_list (windata_t *vwin);
+
+gint populate_remote_func_list (windata_t *win);
 
 void gui_get_db_series (gpointer p, guint action, GtkWidget *w);
 
@@ -29,7 +31,7 @@ gchar *get_db_description (const char *binname);
 
 int write_db_description (const char *binname, const char *descrip);
 
-void show_network_error (windata_t *vwin, char *buf);
+void show_network_error (windata_t *vwin);
 
 void open_rats_window (char *fname);
 
