@@ -25,7 +25,7 @@
 #include "datafiles.h"
 #include "database.h"
 #include "filelists.h"
-#include "webget.h"
+#include "gretl_www.h"
 #include "menustate.h"
 #include "fnsave.h"
 #include "fncall.h"
@@ -68,6 +68,8 @@ enum {
     STACK_POP_PS,
     STACK_RESET_DATA,
     STACK_RESET_PS,
+    STACK_SORT_DATA,
+    STACK_SORT_PS,
     STACK_DESTROY
 };
 
@@ -227,14 +229,16 @@ static file_collection *file_collection_new (const char *path,
 	return NULL;
     }
 
-    coll->path = malloc(strlen(path) + 1);
+    coll->title = NULL;
+    
+    coll->path = gretl_strdup(path);
     if (coll->path == NULL) {
 	free(coll);
 	*err = 1;
 	return NULL;
     }
 
-    coll->descfile = malloc(strlen(descfile) + 1);
+    coll->descfile = gretl_strdup(descfile);
     if (coll->descfile == NULL) {
 	free(coll->path);
 	free(coll);
@@ -242,11 +246,6 @@ static file_collection *file_collection_new (const char *path,
 	return NULL;
     }  
 
-    strcpy(coll->path, path);
-    strcpy(coll->descfile, descfile);
-
-    coll->title = NULL;
-
     if (strstr(coll->descfile, "ps_")) {
 	coll->which = COLL_PS;
     } else {
@@ -270,6 +269,33 @@ static file_collection *file_collection_new (const char *path,
     return coll;
 }
 
+static int compare_colls (const void *a, const void *b)
+{
+    const file_collection *ca = *(const file_collection **) a;
+    const file_collection *cb = *(const file_collection **) b;
+     
+    return strcmp(ca->title, cb->title);
+}
+
+static void collection_stack_sort (file_collection **colls, int n)
+{
+    file_collection *tmp;
+    int i;
+
+    for (i=0; i<n; i++) {
+	if (!strcmp(colls[i]->title, "Gretl")) {
+	    if (i > 0) {
+		tmp = colls[0];
+		colls[0] = colls[i];
+		colls[i] = tmp;
+	    }
+	    break;
+	}
+    }
+
+    qsort(colls + 1, n - 1, sizeof *colls, compare_colls);
+}
+
 static file_collection *collection_stack (file_collection *coll, int op)
 {
     static file_collection **datacoll;
@@ -303,8 +329,11 @@ static file_collection *collection_stack (file_collection *coll, int op)
 	n_data_popped = 0;
     } else if (op == STACK_RESET_PS) {
 	n_ps_popped = 0;
+    } else if (op == STACK_SORT_DATA) {
+	collection_stack_sort(datacoll, n_data);
+    } else if (op == STACK_SORT_PS) {
+	collection_stack_sort(pscoll, n_ps);
     } else if (op == STACK_DESTROY) {
-
         for (j=0; j<n_data; j++) {
 	    free_file_collection(datacoll[j]);
 	}
@@ -355,7 +384,17 @@ static void reset_ps_stack (void)
     collection_stack(NULL, STACK_RESET_PS);
 }
 
-static int test_dir_for_file_collections (const char *dname, DIR *dir)
+static void sort_data_stack (void)
+{
+    collection_stack(NULL, STACK_SORT_DATA);
+}
+
+static void sort_ps_stack (void)
+{
+    collection_stack(NULL, STACK_SORT_PS);
+}
+
+static int get_file_collections_from_dir (const char *dname, DIR *dir)
 {
     file_collection *coll;
     struct dirent *dirent;
@@ -421,7 +460,7 @@ static int seek_file_collections (const char *topdir)
 #ifdef COLL_DEBUG
 		fprintf(stderr, " trying in subdir '%s'\n", subdir);
 #endif
-		err = test_dir_for_file_collections(subdir, try);
+		err = get_file_collections_from_dir(subdir, try);
 #ifdef COLL_DEBUG
 		fprintf(stderr, " result: err = %d\n", err);
 #endif
@@ -484,6 +523,10 @@ static int build_file_collections (void)
 	if (!err) {
 	    err = seek_file_collections(paths.userdir);
 	}
+	if (!err) {
+	    sort_data_stack();
+	    sort_ps_stack();
+	}
 	built = 1;
     }
 
@@ -622,24 +665,24 @@ void browser_open_ps (GtkWidget *w, gpointer data)
     build_path(scriptfile, coll->path, fname, ".inp");
     g_free(fname);
 
+    /* close the calling window */
     gtk_widget_destroy(GTK_WIDGET(vwin->w));
 
-    mkfilelist(FILE_LIST_SCRIPT, scriptfile);
-
     set_scriptpage(coll->title);
 
     view_file(scriptfile, 0, 0, 78, 370, VIEW_SCRIPT);
 } 
 
 enum {
-    VIEW_FN_PKG,
+    VIEW_FN_PKG_INFO,
+    VIEW_FN_PKG_CODE,
     LOAD_FN_PKG,
     EDIT_FN_PKG,
     DELETE_FN_PKG,
     CALL_FN_PKG
 };
 
-static void gui_load_user_functions (const char *fname)
+static int gui_load_user_functions (const char *fname)
 {
     int err;
 
@@ -647,15 +690,17 @@ static void gui_load_user_functions (const char *fname)
     if (err) {
 	gui_errmsg(err);
     } 
+
+    return err;
 }
 
-static void gui_delete_fn_pkg (const char *fname, windata_t *vwin)
+static int gui_delete_fn_pkg (const char *fname, windata_t *vwin)
 {
     char *msg = g_strdup_printf(_("Really delete %s?"), fname);
     int err;
     
     if (yes_no_dialog(_("gretl: delete"), msg, 0) != GRETL_YES) {
-        return;
+        return 0;
     }
 
     err = remove(fname);
@@ -674,6 +719,8 @@ static void gui_delete_fn_pkg (const char *fname, windata_t *vwin)
 	}
 	gtk_list_store_remove(GTK_LIST_STORE(mod), &iter);
     }
+
+    return err;
 }
 
 windata_t *gui_show_function_info (const char *fname, int role)
@@ -720,11 +767,10 @@ static void browser_functions_handler (windata_t *vwin, int task)
     gchar *fname;
     gchar *dir;
     int dircol = 0;
+    int err = 0;
 
-    if (vwin->role == FUNC_EDIT) {
-	dircol = 2;
-    } else if (vwin->role == FUNC_FILES) {
-	dircol = 3;
+    if (vwin->role == FUNC_FILES) {
+	dircol = 4;
     } else if (vwin->role != REMOTE_FUNC_FILES) {
 	dummy_call();
 	return;
@@ -745,15 +791,21 @@ static void browser_functions_handler (windata_t *vwin, int task)
     g_free(fname);
 
     if (task == LOAD_FN_PKG) {
-	gui_load_user_functions(fnfile);
+	err = gui_load_user_functions(fnfile);
     } else if (task == DELETE_FN_PKG) {
-	gui_delete_fn_pkg(fnfile, vwin);
-    } else if (task == VIEW_FN_PKG) {
+	err = gui_delete_fn_pkg(fnfile, vwin);
+    } else if (task == VIEW_FN_PKG_INFO) {
 	gui_show_function_info(fnfile, VIEW_FUNC_INFO);
+    } else if (task == VIEW_FN_PKG_CODE) {
+	gui_show_function_info(fnfile, VIEW_FUNC_CODE);
     } else if (task == EDIT_FN_PKG) {
-	edit_function_package(fnfile);
+	edit_function_package(fnfile, &err);
     } else if (task == CALL_FN_PKG) {
-	call_function_package(fnfile, vwin->w);
+	call_function_package(fnfile, vwin->w, &err);
+    }
+
+    if (!err && (task == EDIT_FN_PKG || task == CALL_FN_PKG)) {
+	maybe_update_func_files_window(0);
     }
 }
 
@@ -782,7 +834,14 @@ static void display_function_info (GtkWidget *w, gpointer data)
 {
     windata_t *vwin = (windata_t *) data;
 
-    browser_functions_handler(vwin, VIEW_FN_PKG);
+    browser_functions_handler(vwin, VIEW_FN_PKG_INFO);
+}
+
+static void display_function_code (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    browser_functions_handler(vwin, VIEW_FN_PKG_CODE);
 }
 
 static void browser_del_func (GtkWidget *w, gpointer data)
@@ -794,10 +853,12 @@ static void browser_del_func (GtkWidget *w, gpointer data)
 
 static void set_browser_status (windata_t *vwin, int status)
 {
+    int i = vwin->role - TEXTBOOK_DATA;
+
     if (status == BROWSER_BUSY) {
-	browsers[vwin->role - TEXTBOOK_DATA] = vwin->w;
+	browsers[i] = vwin->w;
     } else {
-	browsers[vwin->role - TEXTBOOK_DATA] = NULL;
+	browsers[i] = NULL;
     }
 } 
 
@@ -811,21 +872,16 @@ static void free_browser (GtkWidget *w, gpointer data)
 
 static gpointer get_browser_ptr (int role)
 {
-    return (gpointer) &(browsers[role - TEXTBOOK_DATA]);
+    int i = role - TEXTBOOK_DATA;
+
+    return (gpointer) &(browsers[i]);
 }
 
-static void build_datafiles_popup (windata_t *vwin)
+static gpointer get_browser (int role)
 {
-    if (vwin->popup == NULL) {
-	vwin->popup = gtk_menu_new();
+    int i = role - TEXTBOOK_DATA;
 
-	add_popup_item(_("Info"), vwin->popup, 
-		       G_CALLBACK(display_datafile_info), 
-		       vwin);
-	add_popup_item(_("Open"), vwin->popup, 
-		       G_CALLBACK(browser_open_data), 
-		       vwin);
-    }
+    return browsers[i];
 }
 
 static int browser_busy (guint code)
@@ -845,15 +901,194 @@ static int browser_busy (guint code)
     return ret;
 }
 
+static void new_package_callback (GtkWidget *w , gpointer p)
+{
+    file_save(NULL, SAVE_FUNCTIONS, NULL);
+}
+
+static void close_files_viewer (GtkWidget *w, windata_t *vwin)
+{
+    gtk_widget_destroy(vwin->w);
+} 
+
+static void build_datafiles_popup (windata_t *vwin)
+{
+    if (vwin->popup == NULL) {
+	vwin->popup = gtk_menu_new();
+
+	add_popup_item(_("Info"), vwin->popup, 
+		       G_CALLBACK(display_datafile_info), 
+		       vwin);
+	add_popup_item(_("Open"), vwin->popup, 
+		       G_CALLBACK(browser_open_data), 
+		       vwin);
+    }
+}   
+
+static void build_funcfiles_popup (windata_t *vwin)
+{
+    if (vwin->popup == NULL) {
+	vwin->popup = gtk_menu_new();
+
+	if (vwin->role == FUNC_FILES) {
+	    add_popup_item(_("Edit"), vwin->popup, 
+			   G_CALLBACK(browser_edit_func), 
+			   vwin);
+	    add_popup_item(_("Info"), vwin->popup, 
+			   G_CALLBACK(display_function_info), 
+			   vwin);
+	    add_popup_item(_("View code"), vwin->popup, 
+			   G_CALLBACK(display_function_code), 
+			   vwin);
+	    add_popup_item(_("Execute"), vwin->popup, 
+			   G_CALLBACK(browser_call_func), 
+			   vwin);
+	    add_popup_item(_("Delete"), vwin->popup, 
+			   G_CALLBACK(browser_del_func), 
+			   vwin);
+	    add_popup_item(_("New"), vwin->popup, 
+			   G_CALLBACK(new_package_callback), 
+			   vwin);
+	} else {
+	    add_popup_item(_("Info"), vwin->popup, 
+			   G_CALLBACK(file_info_from_server), 
+			   vwin);
+	    add_popup_item(_("Install"), vwin->popup, 
+			   G_CALLBACK(install_file_from_server), 
+			   vwin);
+	    add_popup_item(_("Execute"), vwin->popup, 
+			   G_CALLBACK(browser_call_func), 
+			   vwin);
+	}
+    }
+}
+
+enum {
+    BTN_EDIT = 1,
+    BTN_INFO,
+    BTN_CODE,
+    BTN_INDX,
+    BTN_INST,
+    BTN_EXEC,
+    BTN_DEL,
+    BTN_NEW,
+    BTN_FIND,
+    BTN_OPEN,
+    BTN_CLOSE
+};
+
+struct files_item {
+    const char *str;
+    int action;
+    const gchar *icon;
+};
+
+#ifndef GTK_STOCK_INFO
+# define GTK_STOCK_INFO GRETL_STOCK_INFO
+#endif
+#ifndef GTK_STOCK_EDIT
+# define GTK_STOCK_EDIT GRETL_STOCK_EDIT
+#endif
+
+static struct files_item files_items[] = {
+    { N_("Open"),      BTN_OPEN,  GTK_STOCK_OK },
+    { N_("Edit"),      BTN_EDIT,  GTK_STOCK_EDIT },
+    { N_("Info"),      BTN_INFO,  GTK_STOCK_INFO },
+    { N_("View code"), BTN_CODE,  GTK_STOCK_PROPERTIES },
+    { N_("Index"),     BTN_INDX,  GTK_STOCK_INDEX },
+    { N_("Install"),   BTN_INST,  GTK_STOCK_SAVE },
+    { N_("Execute"),   BTN_EXEC,  GTK_STOCK_EXECUTE },
+    { N_("Delete"),    BTN_DEL,   GTK_STOCK_DELETE },
+    { N_("New"),       BTN_NEW,   GTK_STOCK_NEW },
+    { N_("Find"),      BTN_FIND,  GTK_STOCK_FIND },
+    { N_("Close"),     BTN_CLOSE, GTK_STOCK_CLOSE },
+    { NULL, 0, NULL }
+};
+
+static void make_filesbar (windata_t *vwin)
+{
+    GtkWidget *hbox, *button;
+    int i;
+
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
+
+    vwin->mbar = gtk_toolbar_new();
+    gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
+
+    for (i=0; files_items[i].str != NULL; i++) {
+	void (*toolfunc)() = NULL;
+
+	if (files_items[i].action == BTN_CLOSE) {
+	    toolfunc = close_files_viewer;
+	} else if (files_items[i].action == BTN_FIND) {
+	    toolfunc = datafile_find;
+	} else if (vwin->role == FUNC_FILES) {
+	    if (files_items[i].action == BTN_EDIT) {
+		toolfunc = browser_edit_func;
+	    } else if (files_items[i].action == BTN_INFO) {
+		toolfunc = display_function_info;
+	    } else if (files_items[i].action == BTN_CODE) {
+		toolfunc = display_function_code;
+	    } else if (files_items[i].action == BTN_EXEC) {
+		toolfunc = browser_call_func;
+	    } else if (files_items[i].action == BTN_DEL) {
+		toolfunc = browser_del_func;
+	    } else if (files_items[i].action == BTN_NEW) {
+		toolfunc = new_package_callback;
+	    } 
+	} else if (vwin->role == REMOTE_FUNC_FILES) {
+	    if (files_items[i].action == BTN_INFO) {
+		toolfunc = file_info_from_server;
+	    } else if (files_items[i].action == BTN_INST) {
+		toolfunc = install_file_from_server;
+	    } else if (files_items[i].action == BTN_EXEC) {
+		toolfunc = browser_call_func;
+	    } 
+	} else if (vwin->role == NATIVE_DB) {
+	    if (files_items[i].action == BTN_INDX) {
+		toolfunc = open_db_index;
+	    }
+	} else if (vwin->role == REMOTE_DB) {
+	    if (files_items[i].action == BTN_INDX) {
+		toolfunc = open_remote_db_index;
+	    } else if (files_items[i].action == BTN_INST) {
+		toolfunc = install_file_from_server;
+	    }
+	} else if (vwin->role == TEXTBOOK_DATA) {
+	    if (files_items[i].action == BTN_OPEN) {
+		toolfunc = browser_open_data;
+	    } else if (files_items[i].action == BTN_INFO) {
+		toolfunc = display_datafile_info;
+	    }
+	} else if (vwin->role == PS_FILES) {
+	    if (files_items[i].action == BTN_OPEN) {
+		toolfunc = browser_open_ps;
+	    }
+	}
+
+	if (toolfunc == NULL) {
+	    continue;
+	}
+
+	button = gtk_image_new();
+	gtk_image_set_from_stock(GTK_IMAGE(button), files_items[i].icon, 
+				 GTK_ICON_SIZE_MENU);
+        gtk_toolbar_append_item(GTK_TOOLBAR(vwin->mbar),
+				NULL, _(files_items[i].str), NULL,
+				button, toolfunc, vwin);
+    }
+
+    gtk_widget_show(vwin->mbar);
+    gtk_widget_show(hbox);
+}
+
 void display_files (gpointer p, guint code, GtkWidget *w)
 {
     GtkWidget *filebox, *button;
-    GtkWidget *main_vbox, *button_box;
+    GtkWidget *button_box;
     windata_t *vwin;
-    const gchar *label;
-    void (*browse_func)() = NULL;
-    void (*delete_func)() = NULL;
-    void (*call_func)() = NULL;
+    int col1w = 0;
     int err = 0;
 
     if (browser_busy(code)) {
@@ -880,43 +1115,37 @@ void display_files (gpointer p, guint code, GtkWidget *w)
     case PS_FILES:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: practice files"));
-	browse_func = browser_open_ps;
 	break;
     case FUNC_FILES:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: function packages"));
-	browse_func = browser_edit_func;
-	call_func = browser_call_func;
-	delete_func = browser_del_func;
 	break;
     case TEXTBOOK_DATA:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: data files"));
-	browse_func = browser_open_data;
 	break;
     case NATIVE_DB:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: database files"));
-	browse_func = open_db_index;
 	break;
     case REMOTE_DB:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: databases on server"));
-	browse_func = open_remote_db_index;
 	gtk_widget_set_usize(vwin->w, 640, 480);
 	break;
     case REMOTE_FUNC_FILES:
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: function packages on server"));
-	call_func = browser_call_func;
-	browse_func = file_info_from_server;
 	break;
     }
 
     /* set up grids */
-    main_vbox = gtk_vbox_new(FALSE, 5);
-    gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 10);
-    gtk_container_add(GTK_CONTAINER(vwin->w), main_vbox);
+    vwin->vbox = gtk_vbox_new(FALSE, 1);
+    gtk_box_set_spacing(GTK_BOX(vwin->vbox), 4);
+    gtk_container_set_border_width(GTK_CONTAINER(vwin->vbox), 4);
+    gtk_container_add(GTK_CONTAINER(vwin->w), vwin->vbox);
+
+    make_filesbar(vwin);
 
     if (code == TEXTBOOK_DATA || code == PS_FILES) {
 	filebox = files_notebook(vwin, code);
@@ -924,7 +1153,9 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 	filebox = files_window(vwin);
     }
 
-    gtk_box_pack_start(GTK_BOX(main_vbox), filebox, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(vwin->vbox), filebox, TRUE, TRUE, 0);
+
+    g_object_set_data(G_OBJECT(vwin->w), "vwin", vwin);
 
     if (code == TEXTBOOK_DATA) { 
 	file_collection *coll;
@@ -938,11 +1169,18 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 			     vwin->popup);
 	}
 	reset_data_stack();
-    } else if (REMOTE_ACTION(code)) {
+    } else if (code == FUNC_FILES || code == REMOTE_FUNC_FILES) {
+	build_funcfiles_popup(vwin);
+	g_signal_connect(G_OBJECT(vwin->listbox), "button_press_event",
+			 G_CALLBACK(popup_menu_handler), 
+			 vwin->popup);
+    }
+
+    if (REMOTE_ACTION(code)) {
 	GtkWidget *hbox;
 
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(main_vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
 	vwin->status = gtk_label_new(_("Network status: OK"));
 	gtk_label_set_justify(GTK_LABEL(vwin->status), GTK_JUSTIFY_LEFT);
 	gtk_box_pack_start(GTK_BOX(hbox), vwin->status, FALSE, FALSE, 0);
@@ -950,57 +1188,7 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 
     button_box = gtk_hbox_new(FALSE, 5);
     gtk_box_set_homogeneous(GTK_BOX(button_box), TRUE);
-    gtk_box_pack_start(GTK_BOX(main_vbox), button_box, FALSE, FALSE, 0);
-
-    if (browse_func != NULL) {
-	label = (code == REMOTE_DB)? N_("Get series listing") :
-	    (code == REMOTE_FUNC_FILES)? N_("Info") :
-	    (code == FUNC_FILES)? N_("Edit") :
-	    N_("Open");
-
-	button = gtk_button_new_with_label(_(label));
-	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
-
-	g_signal_connect(G_OBJECT(button), "clicked",
-			 G_CALLBACK(browse_func), vwin);
-
-	if (code != NATIVE_DB && code != FUNC_FILES && !REMOTE_ACTION(code)) {
-	    g_signal_connect(G_OBJECT(button), "clicked", 
-			     G_CALLBACK(delete_widget), vwin->w); 
-	}
-    }
-
-    if (code == TEXTBOOK_DATA || code == FUNC_FILES || REMOTE_ACTION(code)) {
-	label = (REMOTE_ACTION(code))? N_("Install") : N_("Info");
-	button = gtk_button_new_with_label(_(label));
-	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
-	g_signal_connect(G_OBJECT(button), "clicked",
-			 (REMOTE_ACTION(code))?
-			 G_CALLBACK(install_file_from_server) :
-			 (code == FUNC_FILES)? 
-			 G_CALLBACK(display_function_info) :
-			 G_CALLBACK(display_datafile_info), vwin);
-	if (code != FUNC_FILES && code != REMOTE_FUNC_FILES) {
-	    button = gtk_button_new_with_label(_("Find"));
-	    gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
-	    g_signal_connect(G_OBJECT(button), "clicked",
-			     G_CALLBACK(datafile_find), vwin);
-	}
-    }
-
-    if (call_func != NULL) {
-	button = gtk_button_new_with_label(_("Execute"));
-	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
-	g_signal_connect(G_OBJECT(button), "clicked",
-			 G_CALLBACK(call_func), vwin);
-    }
-
-    if (delete_func != NULL) {
-	button = gtk_button_new_with_label(_("Delete"));
-	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
-	g_signal_connect(G_OBJECT(button), "clicked",
-			 G_CALLBACK(delete_func), vwin);
-    }
+    gtk_box_pack_start(GTK_BOX(vwin->vbox), button_box, FALSE, FALSE, 0);
 
     button = gtk_button_new_with_label(_("Close"));
     gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 0);
@@ -1010,6 +1198,16 @@ void display_files (gpointer p, guint code, GtkWidget *w)
     /* put stuff into list box(es) */
     if (code == TEXTBOOK_DATA || code == PS_FILES) {
 	err = populate_notebook_filelists(vwin, filebox, code);
+    } else if (code == FUNC_FILES) {
+	err = populate_filelist(vwin, &col1w);
+	if (col1w > 15) {
+	    /* widen the file box if need be */
+	    gint w, h;
+
+	    gtk_widget_get_size_request(filebox, &w, &h);
+	    w += 100;
+	    gtk_widget_set_size_request(filebox, w, h);
+	}
     } else {
 	err = populate_filelist(vwin, NULL);
     }
@@ -1018,57 +1216,68 @@ void display_files (gpointer p, guint code, GtkWidget *w)
 	gtk_widget_destroy(vwin->w);
     } else {
 	gtk_widget_show_all(vwin->w); 
+	gtk_widget_grab_focus(vwin->listbox);
     }
 }
 
-static char *get_func_description (const char *fname, const char *fndir)
+static int get_func_info (const char *fname, const char *fndir,
+			  char **pdesc, char **pver)
 {
     char fullname[FILENAME_MAX];
-    char *descrip = NULL;
     int err = 0;
 
     build_path(fullname, fndir, fname, NULL);
-    descrip = get_function_file_header(fullname, &err);
+    *pdesc = get_function_file_header(fullname, pver, &err);
     if (err) {
 	gui_errmsg(err);
     }
 
-    return descrip;
+    return err;
 }
 
 static int
 read_fn_files_in_dir (int role, DIR *dir, const char *fndir, 
-		      GtkListStore *store, GtkTreeIter *iter)
+		      GtkListStore *store, GtkTreeIter *iter,
+		      int *maxlen)
 {
     struct dirent *dirent;
     char fullname[MAXLEN];
     gboolean loaded;
     char *fname;
     char *descrip;
+    char *version;
     int n, nfn = 0;
+    int err;
 
     while ((dirent = readdir(dir)) != NULL) {
+	if (!strcmp(dirent->d_name, ".") ||
+	    !strcmp(dirent->d_name, "..")) {
+	    continue;
+	}
 	fname = g_strdup(dirent->d_name);
 	if (fname == NULL) {
 	    break;
 	}
 	n = strlen(fname);
 	if (!g_ascii_strcasecmp(fname + n - 4, ".gfn")) {
-	    descrip = get_func_description(fname, fndir);
-	    if (descrip != NULL) {
+	    err = get_func_info(fname, fndir, &descrip, &version);
+	    if (!err) {
 		gtk_list_store_append(store, iter);
-		if (role == FUNC_EDIT) {
-		    fname[n - 4] = '\0';
-		    gtk_list_store_set(store, iter, 0, fname, 1, descrip,
-				       2, fndir, -1);
-		} else {
-		    build_path(fullname, fndir, fname, NULL);
-		    fname[n - 4] = '\0';
-		    loaded = user_function_file_is_loaded(fullname);
-		    gtk_list_store_set(store, iter, 0, fname, 1, descrip,
-				       2, loaded, 3, fndir, -1);
+		build_path(fullname, fndir, fname, NULL);
+		n -= 4;
+		fname[n] = '\0';
+		if (n > *maxlen) {
+		    *maxlen = n;
 		}
-		g_free(descrip);
+		loaded = function_package_is_loaded(fullname);
+		gtk_list_store_set(store, iter, 
+				   0, fname, 
+				   1, version,
+				   2, descrip, 
+				   3, loaded, 
+				   4, fndir, -1);
+		free(descrip);
+		free(version);
 		nfn++;
 	    }
 	}
@@ -1078,52 +1287,114 @@ read_fn_files_in_dir (int role, DIR *dir, const char *fndir,
     return nfn;
 }
 
-gint populate_func_list (windata_t *vwin)
+gint populate_func_list (windata_t *vwin, int *wid)
 {
     GtkListStore *store;
     GtkTreeIter iter;
     char fndir[FILENAME_MAX];
     DIR *dir;
+    int maxlen = 0;
     int nfn = 0;
 
     store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
     /* pick up any function files in system dir */
+
     build_path(fndir, paths.gretldir, "functions", NULL);
     dir = opendir(fndir);
-
     if (dir != NULL) {
-	nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter);
+	nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
+				    &maxlen);
 	closedir(dir);
     }
 
     /* pick up any function files in the user's personal dir */
+
     build_path(fndir, paths.userdir, "functions", NULL);
     dir = opendir(fndir);
-
     if (dir != NULL) {
-	nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter);
+	nfn += read_fn_files_in_dir(vwin->role, dir, fndir, store, &iter,
+				    &maxlen);
 	closedir(dir);
     }
 
     if (nfn == 0) {
 	errbox(_("No function files found"));
-	/* FIXME don't leak list store */
+	/* FIXME don't leak list store? */
 	return 1;
+    } 
+
+    if (wid != NULL) {
+	*wid = maxlen;
     }
 
     return 0;
 }
 
+/* update loaded status after run, edit calls */
+
+void maybe_update_func_files_window (int editing)
+{
+    GtkWidget *w = get_browser(FUNC_FILES);
+    windata_t *vwin = NULL;
+    GtkWidget *lbox = NULL;
+
+    if (w != NULL) {
+	vwin = g_object_get_data(G_OBJECT(w), "vwin");
+    }
+
+    if (vwin != NULL) {
+	lbox = vwin->listbox;
+    }
+
+    if (editing && lbox != NULL) {
+	GtkListStore *store;
+
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(lbox)));	
+	gtk_list_store_clear(store);
+	populate_func_list(vwin, NULL);
+	return;
+    }
+
+    if (lbox != NULL) {
+	GtkListStore *store = 
+	    GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(lbox)));
+	GtkTreeIter iter;
+	char fullname[MAXLEN];
+	gchar *fname;
+	gchar *fdir;
+	gboolean loaded;
+
+	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter)) {
+	    return;
+	}
+
+	while (1) {
+	    gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &fname, 
+			       4, &fdir, -1);
+	    sprintf(fullname, "%s%c%s.gfn", fdir, SLASH, fname);
+	    g_free(fname);
+	    g_free(fdir);
+	    loaded = function_package_is_loaded(fullname);
+	    gtk_list_store_set(store, &iter, 3, loaded, -1);
+	    if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter)) {
+		break;
+	    }
+	}
+    }
+}
+
 gint populate_filelist (windata_t *vwin, gpointer p)
 {
     if (vwin->role == NATIVE_DB) {
 	return populate_dbfilelist(vwin);
-    } else if (REMOTE_ACTION(vwin->role)) {
-	return populate_remote_object_list(vwin);
+    } else if (vwin->role == REMOTE_DB) {
+	return populate_remote_db_list(vwin);
+    } else if (vwin->role == REMOTE_FUNC_FILES) {
+	return populate_remote_func_list(vwin);
     } else if (vwin->role == FUNC_FILES) {
-	return populate_func_list(vwin);
+	return populate_func_list(vwin, p);
     } else {
 	return read_file_descriptions(vwin, p);
     }
@@ -1151,18 +1422,16 @@ static GtkWidget *files_window (windata_t *vwin)
     };
     const char *func_titles[] = {
 	N_("Package"), 
+	N_("Version"),
 	N_("Summary"), 
 	N_("Loaded?")
     };
     const char *remote_func_titles[] = {
 	N_("Package"), 
+	N_("Version"),
 	N_("Summary"), 
 	N_("Local status")
     };
-    const char *func_edit_titles[] = {
-	N_("Package"), 
-	N_("Summary") 
-    };
 
     GType types_2[] = {
 	G_TYPE_STRING,
@@ -1174,6 +1443,13 @@ static GtkWidget *files_window (windata_t *vwin)
 	G_TYPE_STRING
     };
     GType types_4[] = {
+	G_TYPE_STRING,
+	G_TYPE_STRING,
+	G_TYPE_STRING,
+	G_TYPE_STRING
+    };
+    GType types_5[] = {
+	G_TYPE_STRING,
 	G_TYPE_STRING,
 	G_TYPE_STRING,
 	G_TYPE_BOOLEAN,
@@ -1185,6 +1461,7 @@ static GtkWidget *files_window (windata_t *vwin)
 
     int full_width = 500, file_height = 260;
     int hidden_col = 0;
+    int use_tree = 0;
 
     GtkWidget *box;
     int cols = 2;
@@ -1198,28 +1475,28 @@ static GtkWidget *files_window (windata_t *vwin)
     case REMOTE_DB:
 	titles = remote_db_titles;
 	cols = 3;
-	full_width = 560;
+	full_width = 580;
+	use_tree = 1;
 	break;
     case PS_FILES:
 	titles = ps_titles;
 	cols = 3;
-	full_width = 480;
+	full_width = 560;
+	file_height = 300;
 	break;
     case FUNC_FILES:
 	titles = func_titles;
-	cols = 4;
-	types = types_4;
+	cols = 5;
+	types = types_5;
 	hidden_col = TRUE;
+	full_width = 560;
 	break;
     case REMOTE_FUNC_FILES:
 	titles = remote_func_titles;
-	cols = 3;
+	cols = 4;
+	types = types_4;
+	full_width = 580;
 	break;
-    case FUNC_EDIT:
-	titles = func_edit_titles;
-	cols = 3;
-	hidden_col = TRUE;
-	break;	
     default:
 	break;
     }
@@ -1234,7 +1511,7 @@ static GtkWidget *files_window (windata_t *vwin)
     box = gtk_vbox_new(FALSE, 0);
     gtk_widget_set_size_request(box, full_width, file_height);
     vwin_add_list_box(vwin, GTK_BOX(box), cols, hidden_col, 
-		      types, titles, 0);
+		      types, titles, use_tree);
     gtk_widget_show(box);
 
     return box;
