@@ -30,7 +30,7 @@ void browser_load_func (GtkWidget *w, gpointer data);
 
 void browser_edit_func (GtkWidget *w, gpointer data);
 
-windata_t *gui_show_function_info (const char *fname, int role);
+void browser_call_func (GtkWidget *w, gpointer data);
 
 void destroy_file_collections (void);
 
@@ -40,4 +40,8 @@ gint populate_filelist (windata_t *fdata, gpointer p);
 
 char *strip_extension (char *s);
 
+windata_t *gui_show_function_info (const char *fname, int role);
+
+void maybe_update_func_files_window (int code);
+
 #endif /* DATAFILES_H */
