@@ -43,9 +43,9 @@ gint yes_no_dialog (char *title, char *msg, int cancel);
 
 int make_default_storelist (void);
 
-gint exit_check (GtkWidget *widget, GdkEvent *event, gpointer data);
+gboolean exit_check (void);
 
-void menu_exit_check (GtkWidget *w, gpointer data);
+void menu_exit_check (void);
 
 void delimiter_dialog (gretlopt *optp);
 
@@ -77,12 +77,14 @@ int radio_dialog_with_spinner (const char *title, const char **opts,
 			       int *spinvar, const char *spintxt,
 			       int spinmin, int spinmax);
 
-int checks_dialog (const char *title, const char **opts, int nopts, 
+int checks_dialog (const char *title, const char *blurb,
+		   const char **opts, int nopts, 
 		   int *active, int nradios, int *rvar,
 		   int *spinvar, const char *spintxt, 
 		   int spinmin, int spinmax, int helpcode);
 
-int spin_dialog (const char *title, int *spinvar, const char *spintxt, 
+int spin_dialog (const char *title, const char *blurb,
+		 int *spinvar, const char *spintxt, 
 		 int spinmin, int spinmax, int helpcode);
 
 int get_obs_dialog (const char *title, const char *text,
@@ -95,7 +97,14 @@ int add_obs_dialog (const char *blurb, int addmin);
 int forecast_dialog (int t1min, int t1max, int *t1, 
 		     int t2min, int t2max, int *t2,
 		     int pmin, int pmax, int *p,
-		     int dyn);
+		     int dyn, MODEL *pmod);
+
+int freq_dialog (const char *title, const char *blurb,
+		 int *nbins, int nbmax, double *f0, double *fwid,
+		 double xmin, double xmax);
+
+void bootstrap_dialog (windata_t *vwin, int *pp, int *pB,
+		       gretlopt *popt, int *cancelled);
 
 void compute_default_ts_info (DATAINFO *dwinfo, int newdata);
 
@@ -103,4 +112,6 @@ void data_structure_wizard (gpointer p, guint create, GtkWidget *w);
 
 void lmax_dialog (double *lmax, double ymax);
 
+void tex_format_dialog (gpointer p, guint u, GtkWidget *w);
+
 #endif /* DIALOGS_H */
