@@ -40,7 +40,7 @@ GtkWidget *context_help_button (GtkWidget *hbox, int cmdcode)
 {
     GtkWidget *w;
 
-    w = standard_button(GTK_STOCK_HELP);
+    w = gtk_button_new_from_stock(GTK_STOCK_HELP);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
     gtk_container_add(GTK_CONTAINER(hbox), w);
     gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(hbox),
@@ -65,7 +65,7 @@ GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ,
 {
     GtkWidget *w;
 
-    w = standard_button(GTK_STOCK_CANCEL);
+    w = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
     if (canceled != NULL) {
@@ -97,7 +97,7 @@ static void maybe_opt_invalid (GtkDialog *d, int resp, int *opt)
 GtkWidget *cancel_options_button (GtkWidget *hbox, GtkWidget *targ,
 				  int *opt)
 {
-    GtkWidget *w = standard_button(GTK_STOCK_CANCEL);
+    GtkWidget *w = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
 
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
@@ -123,7 +123,7 @@ GtkWidget *ok_button (GtkWidget *hbox)
 {
     GtkWidget *w;
 
-    w = standard_button(GTK_STOCK_OK);
+    w = gtk_button_new_from_stock(GTK_STOCK_OK);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
     gtk_container_add(GTK_CONTAINER(hbox), w);
 
@@ -134,7 +134,7 @@ GtkWidget *apply_button (GtkWidget *hbox)
 {
     GtkWidget *w;
 
-    w = standard_button(GTK_STOCK_APPLY);
+    w = gtk_button_new_from_stock(GTK_STOCK_APPLY);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
     gtk_container_add(GTK_CONTAINER(hbox), w);
 
@@ -145,7 +145,7 @@ GtkWidget *cancel_button (GtkWidget *hbox)
 {
     GtkWidget *w;
 
-    w = standard_button(GTK_STOCK_CANCEL);
+    w = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
     gtk_container_add(GTK_CONTAINER(hbox), w);
 
@@ -156,7 +156,7 @@ GtkWidget *next_button (GtkWidget *hbox)
 {
     GtkWidget *w;
 
-    w = standard_button(GTK_STOCK_GO_FORWARD);
+    w = gtk_button_new_from_stock(GTK_STOCK_GO_FORWARD);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
     gtk_container_add(GTK_CONTAINER(hbox), w);
 
@@ -167,7 +167,7 @@ GtkWidget *back_button (GtkWidget *hbox)
 {
     GtkWidget *w;
 
-    w = standard_button(GTK_STOCK_GO_BACK);
+    w = gtk_button_new_from_stock(GTK_STOCK_GO_BACK);
     GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
     gtk_container_add(GTK_CONTAINER(hbox), w);
 
@@ -289,11 +289,6 @@ static void destroy_dialog_data (GtkWidget *w, gpointer data)
 	gtk_main_quit();
     }
 
-    if (d->code == GENR_RANDOM) {
-	/* pointer to double */
-	free(d->data);
-    } 
-
     g_free(d); 
 
     open_edit_dialog = NULL;
@@ -311,7 +306,7 @@ static void cancel_on_delete (GtkDialog *d, int resp, int *c)
 }
 
 static dialog_t *
-dialog_data_new (gpointer data, gint code, const char *title,
+dialog_data_new (gpointer p, gint code, const char *title,
 		 int *canceled)
 {
     dialog_t *d = mymalloc(sizeof *d);
@@ -320,7 +315,7 @@ dialog_data_new (gpointer data, gint code, const char *title,
 	return NULL;
     }
 
-    d->data = data;
+    d->data = p;
     d->code = code;
     d->opt = OPT_NONE;
     d->dialog = gtk_dialog_new();
@@ -385,6 +380,25 @@ static int dlg_text_set_previous (dialog_t *d)
     }
 }
 
+static int dlg_text_set_gmm_skel (dialog_t *d)
+{
+    const char *skel = 
+	"# initializations go here\n\n\n"
+	"gmm\n\n"
+	"orthog\n"
+	"weights\n"
+	"params\n"
+	"end gmm\n";
+
+    if (d->code == GMM && edit_save_buf == NULL) {
+	textview_set_text(d->edit, skel);
+	textview_set_cursor_at_line(d->edit, 1);
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
 /* end edit saver apparatus */
 
 gchar *edit_dialog_special_get_text (dialog_t *dlg)
@@ -630,7 +644,7 @@ static void set_replace_restrictions (GtkWidget *w, gpointer p)
     dialog_t *d = (dialog_t *) p;
     
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
-	d->opt = OPT_C;
+	d->opt = OPT_P;
     } else {
 	d->opt = OPT_NONE;
     }
@@ -701,6 +715,17 @@ static gboolean opt_r_callback (GtkWidget *w, dialog_t *dlg)
     return FALSE;
 }
 
+static gboolean opt_b_callback (GtkWidget *w, dialog_t *dlg)
+{
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
+	dlg->opt |= OPT_B;
+    } else {
+	dlg->opt &= ~OPT_B;
+    }
+
+    return FALSE;
+}
+
 static void dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
 				  gretlopt opt)
 {
@@ -714,6 +739,10 @@ static void dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
 	b = gtk_check_button_new_with_label(_("Robust standard errors"));
 	g_signal_connect(G_OBJECT(b), "toggled", 
 			 G_CALLBACK(opt_r_callback), dlg);
+    } else if (opt == OPT_B) {
+	b = gtk_check_button_new_with_label(_("Use bootstrap"));
+	g_signal_connect(G_OBJECT(b), "toggled", 
+			 G_CALLBACK(opt_b_callback), dlg);
     } else {
 	return;
     }
@@ -726,6 +755,55 @@ static void dialog_option_switch (GtkWidget *vbox, dialog_t *dlg,
     gtk_widget_show(hbox);
 }
 
+static void gmm_option_callback (GtkWidget *w, dialog_t *d)
+{
+    gint i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "i"));
+
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
+	switch (i) {
+	case 0:
+	    d->opt &= ~OPT_T;
+	    d->opt &= ~OPT_I;
+	    break;
+	case 1:
+	    d->opt |= OPT_T;
+	    d->opt &= ~OPT_I;
+	    break;
+	case 2:
+	    d->opt &= ~OPT_T;
+	    d->opt |= OPT_I;
+	    break;
+	}
+    }
+}
+
+static void build_gmm_radios (GtkWidget *vbox, dialog_t *d)
+{
+    GtkWidget *b, *hbox;
+    GSList *group = NULL;
+    const char *strs[] = {
+	N_("One-step estimation"),
+	N_("Two-step estimation"),
+	N_("Iterated estimation")
+    };
+    int i;
+
+    for (i=0; i<3; i++) {
+	b = gtk_radio_button_new_with_label(group, _(strs[i]));
+	g_object_set_data(G_OBJECT(b), "i", GINT_TO_POINTER(i));
+	g_signal_connect(G_OBJECT(b), "toggled", 
+			 G_CALLBACK(gmm_option_callback), d);
+	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(b));
+	hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(hbox), b, TRUE, TRUE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show_all(hbox);
+	if (i == 0) {
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), TRUE);
+	}
+    }
+}
+
 static void system_estimator_list (GtkWidget *vbox, dialog_t *d)
 {
     gretl_equation_system *sys = NULL;
@@ -835,7 +913,20 @@ static void edit_dialog_ok (GtkWidget *w, dialog_t *d)
     gtk_widget_destroy(d->dialog);
 }
 
-void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext, 
+static int ols_model_window (windata_t *vwin)
+{
+    if (vwin->role == VIEW_MODEL) {
+	MODEL *pmod = (MODEL *) vwin->data;
+	
+	if (pmod->ci == OLS) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+void edit_dialog (const char *title, const char *info, const char *deflt, 
 		  void (*okfunc)(), void *okptr,
 		  guint cmdcode, guint varclick, 
 		  int *canceled)
@@ -851,7 +942,7 @@ void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext,
 	return;
     }
 
-    d = dialog_data_new(okptr, cmdcode, diagtxt, canceled);
+    d = dialog_data_new(okptr, cmdcode, title, canceled);
     if (d == NULL) return;
 
     open_edit_dialog = d->dialog;
@@ -863,17 +954,17 @@ void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext,
     button_box = GTK_DIALOG(d->dialog)->action_area;
     gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box), 
 			      GTK_BUTTONBOX_END);
-    
 
     if (cmdcode == SYSTEM && d->data != NULL) {
 	/* revisiting saved equation system */
 	dlg_display_sys(d);
     } else if (cmdcode == NLS || cmdcode == MLE || 
-	cmdcode == SYSTEM || cmdcode == RESTRICT) {
+	       cmdcode == SYSTEM || cmdcode == RESTRICT ||
+	       cmdcode == GMM) {
 	int hsize = 62;
 	gchar *lbl;
 
-	lbl = g_strdup_printf("%s\n%s", infotxt,
+	lbl = g_strdup_printf("%s\n%s", info,
 			      _("(Please refer to Help for guidance)"));
 	w = gtk_label_new(lbl);
 	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
@@ -885,18 +976,21 @@ void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext,
 	dialog_table_setup(d, hsize);
 
 	/* insert previous text, if any and if the command
-	   is the same as previously */
-	if (dlg_text_set_previous(d)) {
+	   is the same as previously -- or insert skeleton
+	   of command
+	*/
+	if (dlg_text_set_previous(d) ||
+	    dlg_text_set_gmm_skel(d)) {
 	    clear = 1;
 	}
 
-	if (cmdcode != RESTRICT) {
+	if (cmdcode != RESTRICT && cmdcode != GMM) {
 	    g_signal_connect(G_OBJECT(d->edit), "button_press_event", 
 			     G_CALLBACK(edit_dialog_popup_handler), d);
 	}
     } else {
-	if (infotxt != NULL) {
-	    w = gtk_label_new(infotxt);
+	if (info != NULL) {
+	    w = gtk_label_new(info);
 	    gtk_box_pack_start(GTK_BOX(top_vbox), w, TRUE, TRUE, 5);
 	    gtk_widget_show(w);
 	}
@@ -910,9 +1004,9 @@ void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext,
 			     G_CALLBACK(okfunc), d);
 	}
 
-	if (deftext != NULL && *deftext != '\0') {
-	    gtk_entry_set_text(GTK_ENTRY(d->edit), deftext);
-	    gtk_editable_select_region(GTK_EDITABLE(d->edit), 0, strlen(deftext));
+	if (deflt != NULL && *deflt != '\0') {
+	    gtk_entry_set_text(GTK_ENTRY(d->edit), deflt);
+	    gtk_editable_select_region(GTK_EDITABLE(d->edit), 0, strlen(deflt));
 	} 
 
 	gtk_widget_show(d->edit);
@@ -928,7 +1022,12 @@ void edit_dialog (const char *diagtxt, const char *infotxt, const char *deftext,
     } else if (cmdcode == NLS || cmdcode == MLE) {
 	dialog_option_switch(top_vbox, d, OPT_V);
 	dialog_option_switch(top_vbox, d, OPT_R);
-    } 
+    } else if (cmdcode == GMM) {
+	dialog_option_switch(top_vbox, d, OPT_V);
+	build_gmm_radios(top_vbox, d);
+    } else if (cmdcode == RESTRICT && ols_model_window(okptr)) {
+	dialog_option_switch(top_vbox, d, OPT_B);
+    }
 
     if (varclick == VARCLICK_INSERT_ID) { 
 	active_edit_id = d->edit; 
@@ -1015,4 +1114,3 @@ char *entry_box_get_trimmed_text (GtkWidget *w)
 
     return ret;
 }
-
