@@ -30,6 +30,7 @@
 #include "fileselect.h"
 #include "fnsave.h"
 #include "database.h"
+#include "bootstrap.h"
 
 #define IS_DAT_ACTION(i) (i == SAVE_DATA || \
                           i == SAVE_DATA_AS || \
@@ -57,8 +58,6 @@
                                i == APPEND_JMULTI)
 
 #define SAVE_GRAPH_ACTION(i) (i == SAVE_GNUPLOT || \
-                              i == SAVE_THIS_GRAPH || \
-                              i == SAVE_LAST_GRAPH || \
                               i == SAVE_BOXPLOT_EPS || \
                               i == SAVE_BOXPLOT_PS || \
                               i == SAVE_BOXPLOT_XPM)
@@ -69,14 +68,18 @@
                              a == EXPORT_DAT || \
                              a == EXPORT_JM) && s != FSEL_DATA_PRN)
 
+#define GDT_ACTION(i) (i == SAVE_DATA || \
+                       i == SAVE_DATA_AS || \
+                       i == SAVE_BOOT_DATA || \
+                       i == OPEN_DATA || \
+                       i == APPEND_DATA)
+
 struct extmap {
     int action;
     char *ext;
 };
 
 static struct extmap action_map[] = {
-    { SAVE_DATA,         ".gdt" },
-    { SAVE_DATA_AS,      ".gdt" },
     { SAVE_DBDATA,       ".bin" },
     { SAVE_SCRIPT,       ".inp" },
     { SAVE_CONSOLE,      ".inp" },
@@ -95,8 +98,6 @@ static struct extmap action_map[] = {
     { SAVE_OUTPUT,       ".txt" },
     { SAVE_TEX,          ".tex" },
     { SAVE_RTF,          ".rtf" },
-    { OPEN_DATA,         ".gdt" },
-    { APPEND_DATA,       ".gdt" },    
     { OPEN_SCRIPT,       ".inp" },
     { OPEN_SESSION,      ".gretl" },
     { OPEN_CSV,          ".csv" },
@@ -124,14 +125,15 @@ static gretlopt save_action_to_opt (int action, gpointer p)
     gretlopt opt = OPT_NONE;
 
     switch (action) {
-    case SAVE_DATA:     opt = OPT_Z; break;
-    case SAVE_DATA_AS:  opt = OPT_Z; break;
-    case SAVE_DBDATA:   opt = OPT_D; break;
-    case EXPORT_OCTAVE: opt = OPT_M; break;
-    case EXPORT_R:      opt = OPT_R; break;
-    case EXPORT_CSV:    opt = OPT_C; break;
-    case EXPORT_DAT:    opt = OPT_G; break; /* PcGive */
-    case EXPORT_JM:     opt = OPT_J; break; /* JMulti */
+    case SAVE_DATA:
+    case SAVE_DATA_AS:
+    case SAVE_BOOT_DATA: opt = OPT_Z; break;
+    case SAVE_DBDATA:    opt = OPT_D; break;
+    case EXPORT_OCTAVE:  opt = OPT_M; break;
+    case EXPORT_R:       opt = OPT_R; break;
+    case EXPORT_CSV:     opt = OPT_C; break;
+    case EXPORT_DAT:     opt = OPT_G; break; /* PcGive */
+    case EXPORT_JM:      opt = OPT_J; break; /* JMulti */
     default: break;
     }
 
@@ -142,15 +144,16 @@ static gretlopt save_action_to_opt (int action, gpointer p)
     return opt;
 }
 
-static const char *get_gp_ext (const char *termtype)
+static const char *get_gp_ext (int ttype)
 {
-    if (!strncmp(termtype, "postscript", 10))    return ".eps";
-    else if (!strcmp(termtype, "PDF"))           return ".pdf";
-    else if (!strcmp(termtype, "fig"))           return ".fig";
-    else if (!strcmp(termtype, "latex"))         return ".tex";
-    else if (!strncmp(termtype, "png", 3))       return ".png";
-    else if (!strncmp(termtype, "emf", 3))       return ".emf";
-    else if (!strcmp(termtype, "plot commands")) return ".plt";
+    if (ttype == GP_TERM_EPS)      return ".eps";
+    else if (ttype == GP_TERM_PDF) return ".pdf";
+    else if (ttype == GP_TERM_FIG) return ".fig";
+    else if (ttype == GP_TERM_TEX) return ".tex";
+    else if (ttype == GP_TERM_PNG) return ".png";
+    else if (ttype == GP_TERM_EMF) return ".emf";
+    else if (ttype == GP_TERM_SVG) return ".svg";
+    else if (ttype == GP_TERM_PLT) return ".plt";
     else return "*";
 }
 
@@ -180,11 +183,12 @@ static const char *get_ext (int action, gpointer data)
 {
     const char *s = NULL;
 
-    if (action == SAVE_GNUPLOT || action == SAVE_THIS_GRAPH) {
-	GPT_SPEC *plot = (GPT_SPEC *) data;
-	s = get_gp_ext(plot->termtype);
-    } else if (action == SAVE_LAST_GRAPH) {
-	s = get_gp_ext(data);
+    if (GDT_ACTION(action)) {
+	return ".gdt";
+    } else if (action == SAVE_GNUPLOT) {
+	int ttype = gp_term_code(data);
+
+	s = get_gp_ext(ttype);
     } else {
 	size_t i;
 
@@ -424,6 +428,17 @@ static char *suggested_exportname (const char *fname, int action)
     return s;
 }
 
+static void bootstrap_save_callback (const char *fname)
+{
+    int err = bootstrap_save_data(fname);
+
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	infobox(_("Saved data as %s"), fname);
+    }
+}
+
 static void
 file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 			      gpointer data)
@@ -487,19 +502,7 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
     } else if (SAVE_DATA_ACTION(action)) {
 	do_store(fname, save_action_to_opt(action, data));
     } else if (action == SAVE_GNUPLOT) {
-	int err = 0;
-	GPT_SPEC *plot = (GPT_SPEC *) data;
-
-	err = go_gnuplot(plot, fname);
-	if (err == 1) {
-	    errbox(_("gnuplot command failed"));
-	} else if (err == 2) {
-	    infobox(_("There were missing observations"));
-	}
-    } else if (action == SAVE_THIS_GRAPH) {
-	GPT_SPEC *plot = (GPT_SPEC *) data;
-
-	save_this_graph(plot, fname);
+	save_graph_to_file(data, fname);
     } else if (action == SAVE_BOXPLOT_EPS || action == SAVE_BOXPLOT_PS) {
 	int err;
 
@@ -511,6 +514,8 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	save_session(fname);
     } else if (action == SAVE_FUNCTIONS) {
 	save_user_functions(fname, data);
+    } else if (action == SAVE_BOOT_DATA) {
+	bootstrap_save_callback(fname);
     } else if (action == SET_PROG || action == SET_DIR) {
 	char *strvar = (char *) data;
 
@@ -544,10 +549,11 @@ struct win32_filtermap {
     struct winfilter filter;
 };
 
-static struct winfilter get_gp_filter (const char *termtype)
+static struct winfilter get_gp_filter (int ttype)
 {
-    static struct winfilter gpfilters[] = {
+    static struct winfilter gpfilt[] = {
 	{ N_("postscript files (*.eps)"), "*.eps" },
+	{ N_("PDF files (*.pdf)"), "*.pdf" },
 	{ N_("xfig files (*.fig)"), "*.fig" },
 	{ N_("LaTeX files (*.tex)"), "*.tex" },
 	{ N_("PNG files (*.png)"), "*.png" },
@@ -556,29 +562,28 @@ static struct winfilter get_gp_filter (const char *termtype)
 	{ N_("all files (*.*)"), "*.*" }
     };
 
-    if (!strncmp(termtype, "postscript", 10)) 
-	return gpfilters[0];
-    else if (!strcmp(termtype, "fig")) 
-	return gpfilters[1];
-    else if (!strcmp(termtype, "latex")) 
-	return gpfilters[2];
-    else if (!strncmp(termtype, "png", 3)) 
-	return gpfilters[3];
-    else if (!strncmp(termtype, "emf", 3)) 
-	return gpfilters[4];
-    else if (!strcmp(termtype, "plot commands")) 
-	return gpfilters[5];
-    else 
-	return gpfilters[6];
+    if (ttype == GP_TERM_EPS) 
+	return gpfilt[0];
+    else if (ttype == GP_TERM_PDF) 
+	return gpfilt[1];
+    else if (ttype == GP_TERM_FIG) 
+	return gpfilt[2];
+    else if (ttype == GP_TERM_TEX) 
+	return gpfilt[3];
+    else if (ttype == GP_TERM_PNG) 
+	return gpfilt[4];
+    else if (ttype == GP_TERM_EMF) 
+	return gpfilt[5];
+    else if (ttype == GP_TERM_PLT)
+	return gpfilt[6];
+    else
+	return gpfilt[7];
 }
 
 static struct winfilter get_filter (int action, gpointer data)
 {
-    int i;
-    struct winfilter filter;
     static struct win32_filtermap map[] = {
 	{ SAVE_DATA,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
-	{ SAVE_DATA_AS, { N_("gretl data files (*.gdt)"), "*.gdt" }},
 	{ SAVE_DBDATA,  { N_("gretl database files (*.bin)"), "*.bin" }},
 	{ SAVE_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
 	{ SAVE_CONSOLE, { N_("gretl command files (*.inp)"), "*.inp" }},
@@ -597,8 +602,6 @@ static struct winfilter get_filter (int action, gpointer data)
 	{ SAVE_OUTPUT,  { N_("text files (*.txt)"), "*.txt" }},
 	{ SAVE_TEX,     { N_("TeX files (*.tex)"), "*.tex" }},
 	{ SAVE_RTF,     { N_("RTF files (*.rtf)"), "*.rtf" }},
-	{ OPEN_DATA,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
-	{ APPEND_DATA,  { N_("gretl data files (*.gdt)"), "*.gdt" }},
 	{ OPEN_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
 	{ OPEN_SESSION, { N_("session files (*.gretl)"), "*.gretl" }},
 	{ OPEN_CSV,     { N_("CSV files (*.csv)"), "*.csv" }},
@@ -620,44 +623,47 @@ static struct winfilter get_filter (int action, gpointer data)
 	{ OPEN_PCGIVE_DB, { N_("PcGive data files (*.bn7)"), "*.bn7" }},
 	{ SET_PROG,     { N_("program files (*.exe)"), "*.exe" }}
     };
-
     static struct winfilter default_filter = {
 	N_("all files (*.*)"), "*.*" 
     };
+    struct winfilter filt;
+    int i;
 
-    if (action == SAVE_GNUPLOT || action == SAVE_THIS_GRAPH) {
-	GPT_SPEC *plot = (GPT_SPEC *) data;
+    if (action == SAVE_GNUPLOT) {
+	int ttype = gp_term_code(data);
 
-	return get_gp_filter(plot->termtype);
-    }
+	filt = get_gp_filter(ttype);
+    } else {
+	filt = default_filter;
 
-    if (action == SAVE_LAST_GRAPH) {
-	return get_gp_filter(data);
-    }
-
-    filter = default_filter;
-
-    for (i=0; i< sizeof map / sizeof *map; i++) {
-	if (action == map[i].action) {
-	    filter = map[i].filter;
-	    break;
+	for (i=0; i< sizeof map / sizeof *map; i++) {
+	    if (action == map[i].action) {
+		filt = map[i].filter;
+		break;
+	    }
 	}
     }
 
-    return filter;
+    return filt;
 }
 
 static char *make_winfilter (int action, gpointer data)
 {
-    char *p = mymalloc(128);
-    char *start = p;
     struct winfilter filter;
+    char *ret, *p;
 
-    if (p == NULL) {
+    ret = calloc(128, 1);
+    if (ret == NULL) {
 	return NULL;
     }
 
-    filter = get_filter(action, data);
+    p = ret;
+
+    if (GDT_ACTION(action)) {
+	filter = get_filter(SAVE_DATA, data);
+    } else {
+	filter = get_filter(action, data);
+    }
 
     strcpy(p, I_(filter.descrip));
     p += strlen(p) + 1;
@@ -670,10 +676,7 @@ static char *make_winfilter (int action, gpointer data)
 	strcpy(p, "*.*");
     }
 
-    p += strlen(p) + 1;
-    *p = '\0';
-
-    return start;
+    return ret;
 }
 
 static int select_dirname (char *fname, char *trmsg)
@@ -741,7 +744,9 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 		strcpy(fname, strvar + slashpos(strvar) + 1);
 	    } 
 	}
-    }	
+    } else if (action == SAVE_FUNCTIONS) {
+	get_default_package_name(fname, data);
+    }
 
     if (doing_nls()) {
 	trmsg = my_locale_from_utf8(msg);
@@ -851,7 +856,7 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
     }
 
     /* FIXME: parent window below should not always be mdata->w,
-       in particular when action == SAVE_THIS_GRAPH
+       in particular when action == SAVE_GNUPLOT
     */
 
     filesel = gtk_file_chooser_dialog_new(msg, GTK_WINDOW(parent), fa,
@@ -876,6 +881,7 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
     }
 
+    /* FIXME session dir */
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), startdir);
 
     /* special cases */
@@ -892,9 +898,6 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
 					  savename);
 	g_free(savename);
-    } else if (action == SAVE_SESSION && *scriptfile != '\0') {
-	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
-				      scriptfile);
     } else if (action == SET_PROG) {
 	char *strvar = (char *) data;
 
@@ -912,6 +915,13 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 	    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
 					  strvar);
 	} 
+    } else if (action == SAVE_FUNCTIONS) {
+	char fname[MAXLEN];
+
+	*fname = '\0';
+	get_default_package_name(fname, data);
+	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
+					  fname);
     }	
 
     if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
