@@ -21,9 +21,13 @@
 #include "dlgutils.h"
 #include "selector.h"
 #include "gretl_func.h"
+#include "monte_carlo.h"
 #include "usermat.h"
-#include "webget.h"
+#include "cmd_private.h"
+#include "gretl_www.h"
 #include "database.h"
+#include "guiprint.h"
+#include "ssheet.h"
 
 #define FCDEBUG 0
 
@@ -32,34 +36,42 @@ typedef struct call_info_ call_info;
 struct call_info_ {
     GtkWidget *dlg;
     GList *lsels;
-    int *publist;
+    GList *msels;
     int iface;
-    int need_list;
+    int extracol;
     const ufunc *func;
     int n_params;
-    int n_returns;
-    char *return_types;
+    char rettype;
     char **args;
-    char **rets;
-    int canceled;
+    char *ret;
+    int ok;
 };
 
-static void cinfo_init (call_info *cinfo)
+static call_info *cinfo_new (void)
 {
+    call_info *cinfo = mymalloc(sizeof *cinfo);
+
+    if (cinfo == NULL) {
+	return NULL;
+    }
+
+    cinfo->iface = -1;
+
     cinfo->lsels = NULL;
-    cinfo->publist = NULL;
+    cinfo->msels = NULL;
 
     cinfo->func = NULL;
     cinfo->n_params = 0;
 
-    cinfo->n_returns = 0;
-    cinfo->return_types = NULL;
+    cinfo->rettype = ARG_NONE;
 
     cinfo->args = NULL;
-    cinfo->rets = NULL;
+    cinfo->ret = NULL;
 
-    cinfo->need_list = 0;
-    cinfo->canceled = 0;
+    cinfo->extracol = 0;
+    cinfo->ok = 0;
+
+    return cinfo;
 }
 
 static int cinfo_args_init (call_info *cinfo)
@@ -67,7 +79,7 @@ static int cinfo_args_init (call_info *cinfo)
     int err = 0;
 
     cinfo->args = NULL;
-    cinfo->rets = NULL;
+    cinfo->ret = NULL;
 
     if (cinfo->n_params > 0) {
 	cinfo->args = strings_array_new(cinfo->n_params);
@@ -76,52 +88,68 @@ static int cinfo_args_init (call_info *cinfo)
 	}
     }
 
-    if (!err && cinfo->n_returns > 0) {
-	cinfo->rets = strings_array_new(cinfo->n_returns);
-	if (cinfo->rets == NULL) {
-	    err = E_ALLOC;
-	}
-    } 
-
     return err;
 }
 
 static void cinfo_free (call_info *cinfo)
 {
-    free(cinfo->publist);
-    free(cinfo->return_types);
     free_strings_array(cinfo->args, cinfo->n_params);
-    free_strings_array(cinfo->rets, cinfo->n_returns);
+    free(cinfo->ret);
     g_list_free(cinfo->lsels);
+    g_list_free(cinfo->msels);
+    free(cinfo);
 }
 
+#define scalar_type(t) (t == ARG_SCALAR || t == ARG_REF_SCALAR)
+#define series_type(t) (t == ARG_SERIES || t == ARG_REF_SERIES)
+#define matrix_type(t) (t == ARG_MATRIX || t == ARG_REF_MATRIX)
+
 static const char *arg_type_string (int type)
 {
     if (type == ARG_BOOL)   return "boolean";
     if (type == ARG_INT)    return "int";
-    if (type == ARG_SCALAR) return "scalar";
-    if (type == ARG_SERIES) return "series";
     if (type == ARG_LIST)   return "list";
-    if (type == ARG_MATRIX) return "matrix";
+    
+    if (scalar_type(type)) return "scalar";
+    if (series_type(type)) return "series";
+    if (matrix_type(type)) return "matrix";
+
     return "";
 }
 
+static int check_args (call_info *cinfo)
+{
+    int i;
+
+    /* FIXME optional args? */
+
+    if (cinfo->args != NULL) {
+	for (i=0; i<cinfo->n_params; i++) {
+	    if (cinfo->args[i] == NULL) {
+		errbox(_("Argument %d (%s) is missing"), i + 1,
+		       fn_param_name(cinfo->func, i));
+		return 1;
+	    }
+	}
+    }
+
+    return 0;
+}
+
 static void fncall_finalize (GtkWidget *w, call_info *cinfo)
 {
+    if (check_args(cinfo)) {
+	return;
+    }
+    cinfo->ok = 1;
     gtk_widget_destroy(cinfo->dlg);
 }
 
 static void fncall_cancel (GtkWidget *w, call_info *cinfo)
 {
-    cinfo->canceled = 1;
     gtk_widget_destroy(cinfo->dlg);
 }
 
-static void fncall_delete (GtkWidget *w, call_info *cinfo)
-{
-    cinfo->canceled = 1;
-}
-
 static GtkWidget *label_hbox (GtkWidget *w, const char *txt, int center)
 {
     GtkWidget *hbox, *label;
@@ -172,9 +200,16 @@ static gboolean update_arg (GtkEditable *entry,
 {
     int i = 
 	GPOINTER_TO_INT(g_object_get_data(G_OBJECT(entry), "argnum"));
+    char *s;
 
     free(cinfo->args[i]);
-    cinfo->args[i] = entry_box_get_trimmed_text(GTK_WIDGET(entry));
+    s = cinfo->args[i] = entry_box_get_trimmed_text(GTK_WIDGET(entry));
+
+    if (s != NULL && fn_param_type(cinfo->func, i) == ARG_SCALAR) {
+	if (isdigit(*s) || *s == '-' || *s == '+' || *s == ',') {
+	    charsub(s, ',', '.');
+	}
+    }
 
     return FALSE;
 }
@@ -182,28 +217,31 @@ static gboolean update_arg (GtkEditable *entry,
 static gboolean update_return (GtkEditable *entry, 
 			       call_info *cinfo)
 {
-    int i = 
-	GPOINTER_TO_INT(g_object_get_data(G_OBJECT(entry), "retnum"));
-
-    free(cinfo->rets[i]);
-    cinfo->rets[i] = entry_box_get_trimmed_text(GTK_WIDGET(entry));
+    free(cinfo->ret);
+    cinfo->ret = entry_box_get_trimmed_text(GTK_WIDGET(entry));
 
     return FALSE;
 }
 
-static GList *get_selection_list (int type)
+static GList *get_selection_list (call_info *cinfo, int i, int type)
 {
     GList *list = NULL;
     const char *name;
-    int i;
+    int optional = 0;
 
-    if (type == ARG_SERIES || type == ARG_SCALAR) {
+    /* FIXME int, bool arguments */
+
+    if (i >= 0) {
+	optional = fn_param_optional(cinfo->func, i);
+    }
+
+    if (series_type(type) || scalar_type(type)) {
 	for (i=1; i<datainfo->v; i++) {
 	    if (var_is_hidden(datainfo, i)) {
 		continue;
 	    }
-	    if ((type == ARG_SERIES && var_is_series(datainfo, i)) ||
-		(type == ARG_SCALAR && var_is_scalar(datainfo, i))) {
+	    if ((series_type(type) && var_is_series(datainfo, i)) ||
+		(scalar_type(type) && var_is_scalar(datainfo, i))) {
 		list = g_list_append(list, (gpointer) datainfo->varname[i]);
 	    } 
 	}
@@ -213,11 +251,15 @@ static GList *get_selection_list (int type)
     } else if (type == ARG_LIST) {
 	int nl = n_saved_lists();
 
+	if (optional) {
+	    list = g_list_append(list, "null");
+	}
+
 	for (i=0; i<nl; i++) {
 	    name = get_list_name_by_index(i);
 	    list = g_list_append(list, (gpointer) name);
 	}
-    } else if (type == ARG_MATRIX) {
+    } else if (matrix_type(type)) {
 	int nm = n_user_matrices();
 
 	for (i=0; i<nm; i++) {
@@ -226,6 +268,10 @@ static GList *get_selection_list (int type)
 	}	
     }
 
+    if (optional && type != ARG_LIST) {
+	list = g_list_append(list, "null");
+    }
+
     return list;
 }
 
@@ -245,10 +291,44 @@ static void fncall_help (GtkWidget *w, call_info *cinfo)
 	gretl_print_destroy(prn);
 	dummy_call();
     } else {
-	view_buffer(prn, 80, 400, "help", PRINT, NULL);
+	view_buffer(prn, 80, 400, fnname, VIEW_FUNC_INFO, NULL);
     }
 }
 
+static void update_matrix_selectors (call_info *cinfo)
+{
+    GList *slist = cinfo->msels;
+    GList *mlist = NULL;
+    GtkWidget *sel;
+    const char *mname;
+    const gchar *txt;
+    gchar *saved;
+    int i, nm = n_user_matrices();
+
+    for (i=0; i<nm; i++) {
+	mname = get_matrix_name_by_index(i);
+	mlist = g_list_append(mlist, (gpointer) mname);
+    }
+
+    while (slist != NULL) {
+	sel = GTK_WIDGET(slist->data);
+	saved = NULL;
+	txt = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(sel)->entry));
+	if (*txt != 0) {
+	    saved = g_strdup(txt);
+	}
+	gtk_combo_set_popdown_strings(GTK_COMBO(sel), mlist);
+	if (saved != NULL) {
+	    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(sel)->entry), 
+			       saved);
+	    free(saved);
+	}
+	slist = slist->next;
+    }
+
+    g_list_free(mlist);
+}
+
 static void update_list_selectors (call_info *cinfo)
 {
     GList *slist = cinfo->lsels;
@@ -294,17 +374,26 @@ int do_make_list (selector *sr)
     int *list;
     int err;
 
-    if (buf == NULL || *buf == 0) {
-	return 1;
-    }
-
     if (lname == NULL || *lname == 0) {
 	errbox(_("No name was given for the list"));
 	return 1;
-    }    
+    }   
+
+    if (buf == NULL || *buf == 0) {
+	int resp;
+
+	resp = yes_no_dialog("gretl", _("Really create an empty list?"), 0);
+	if (resp == GRETL_YES) {
+	    list = gretl_null_list();
+	} else {
+	    return 0;
+	}
+    } else {
+	list = gretl_list_from_string(buf);
+    }
 
-    list = gretl_list_from_string(buf);
     if (list == NULL) {
+	nomem();
 	return 1;
     }
 
@@ -335,6 +424,19 @@ static void launch_list_maker (GtkWidget *w, GtkWidget *entry)
 		     entry);
 }
 
+static void launch_matrix_maker (GtkWidget *w, call_info *cinfo)
+{
+    int n = n_user_matrices();
+
+    gui_new_matrix();
+
+    if (n_user_matrices() > n) {
+	update_matrix_selectors(cinfo);
+    }
+
+    gdk_window_raise(cinfo->dlg->window);
+}
+
 static int spinner_arg (call_info *cinfo, int i)
 {
     double x = fn_param_minval(cinfo->func, i);
@@ -393,12 +495,17 @@ static GtkWidget *combo_arg_selector (call_info *cinfo, int ptype, int i)
     g_object_set_data(G_OBJECT(GTK_COMBO(combo)->entry), "cinfo", cinfo);
     g_signal_connect(G_OBJECT(GTK_COMBO(combo)->entry), "changed",
 		     G_CALLBACK(update_arg), cinfo);
-    list = get_selection_list(ptype);
+    gtk_entry_set_activates_default(GTK_ENTRY(GTK_COMBO(combo)->entry), 
+				    TRUE);
+
+    list = get_selection_list(cinfo, i, ptype);
     if (list != NULL) {
 	gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);
 	g_list_free(list);
     } 
 
+    /* FIXME bool etc */
+
     if (ptype == ARG_INT || ptype == ARG_SCALAR) {
 	double x = fn_param_default(cinfo->func, i);
 
@@ -425,16 +532,12 @@ static void function_call_dialog (call_info *cinfo)
     err = cinfo_args_init(cinfo);
     if (err) {
 	gui_errmsg(err);
-	cinfo->canceled = 1;
 	return;
     }
 
     cinfo->dlg = gretl_dialog_new(_("gretl: function call"), NULL, 
 				  GRETL_DLG_BLOCK | GRETL_DLG_RESIZE);
 
-    g_signal_connect(G_OBJECT(cinfo->dlg), "delete_event",
-		     G_CALLBACK(fncall_delete), cinfo);
-
     fnname = user_function_name_by_index(cinfo->iface);
     txt = g_strdup_printf(_("Call to function %s"), fnname);
     hbox = label_hbox(GTK_DIALOG(cinfo->dlg)->vbox, txt, 1);
@@ -443,9 +546,9 @@ static void function_call_dialog (call_info *cinfo)
     /* function argument selection */
 
     if (cinfo->n_params > 0) {
-	int tcols = (cinfo->need_list)? 4 : 3;
+	int tcols = (cinfo->extracol)? 4 : 3;
 
-	hbox = label_hbox(GTK_DIALOG(cinfo->dlg)->vbox, _("Required arguments:"), 0);
+	hbox = label_hbox(GTK_DIALOG(cinfo->dlg)->vbox, _("Arguments:"), 0);
 	gtk_widget_show(hbox);
 
 	tbl = gtk_table_new(cinfo->n_params + 1, tcols, FALSE);
@@ -495,20 +598,29 @@ static void function_call_dialog (call_info *cinfo)
 
 	    if (ptype == ARG_LIST) {
 		cinfo->lsels = g_list_append(cinfo->lsels, sel);
-		button = gtk_button_new_with_label("More...");
+		button = gtk_button_new_with_label(_("More..."));
 		gtk_table_attach(GTK_TABLE(tbl), button, 3, 4, i+1, i+2,
 				 GTK_EXPAND, GTK_FILL, 5, 5);
 		g_signal_connect(G_OBJECT(button), "clicked", 
 				 G_CALLBACK(launch_list_maker), 
 				 GTK_COMBO(sel)->entry);
 		gtk_widget_show(button);
-	    }
+	    } else if (ptype == ARG_MATRIX) {
+		cinfo->msels = g_list_append(cinfo->msels, sel);
+		button = gtk_button_new_with_label(_("New..."));
+		gtk_table_attach(GTK_TABLE(tbl), button, 3, 4, i+1, i+2,
+				 GTK_EXPAND, GTK_FILL, 5, 5);
+		g_signal_connect(G_OBJECT(button), "clicked", 
+				 G_CALLBACK(launch_matrix_maker), 
+				 cinfo);
+		gtk_widget_show(button);
+	    }		
 	}
 
 	gtk_widget_show(tbl);
     }
 
-    if (cinfo->n_params > 0 && cinfo->n_returns > 0) {
+    if (cinfo->n_params > 0 && cinfo->rettype != ARG_NONE) {
 	GtkWidget *hsep = gtk_hseparator_new();
 
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cinfo->dlg)->vbox),
@@ -518,13 +630,14 @@ static void function_call_dialog (call_info *cinfo)
 
     /* function return(s) assignment */
 
-    if (cinfo->n_returns > 0) {
+    if (cinfo->rettype != ARG_NONE) {
+	GList *list = NULL;
 
 	hbox = label_hbox(GTK_DIALOG(cinfo->dlg)->vbox, 
-			  _("Assign return values (optional):"), 0);
+			  _("Assign return value (optional):"), 0);
 	gtk_widget_show(hbox);
 
-	tbl = gtk_table_new(cinfo->n_returns + 1, 2, FALSE);
+	tbl = gtk_table_new(1, 2, FALSE);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cinfo->dlg)->vbox),
 			   tbl, FALSE, FALSE, 5);
 
@@ -538,30 +651,24 @@ static void function_call_dialog (call_info *cinfo)
 			 GTK_EXPAND, GTK_FILL, 5, 5);
 	gtk_widget_show(label);
 
-	for (i=0; i<cinfo->n_returns; i++) {
-	    GList *list = NULL;
+	label = gtk_label_new(arg_type_string(cinfo->rettype));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.75, 0.5);
+	gtk_table_attach(GTK_TABLE(tbl), label, 0, 1, 1, 2,
+			 GTK_EXPAND, GTK_FILL, 5, 5);
+	gtk_widget_show(label);
 
-	    label = gtk_label_new(arg_type_string(cinfo->return_types[i]));
-	    gtk_misc_set_alignment(GTK_MISC(label), 0.75, 0.5);
-	    gtk_table_attach(GTK_TABLE(tbl), label, 0, 1, i+1, i+2,
-			     GTK_EXPAND, GTK_FILL, 5, 5);
-	    gtk_widget_show(label);
-
-	    sel = gtk_combo_new();
-	    g_object_set_data(G_OBJECT(GTK_COMBO(sel)->entry), "retnum",
-			      GINT_TO_POINTER(i));
-	    g_signal_connect(G_OBJECT(GTK_COMBO(sel)->entry), "changed",
-			     G_CALLBACK(update_return), cinfo);
-	    list = get_selection_list(cinfo->return_types[i]);
-	    if (list != NULL) {
-		gtk_combo_set_popdown_strings(GTK_COMBO(sel), list); 
-		g_list_free(list);
-	    }
-	    gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(sel)->entry), TRUE);
-	    gtk_table_attach(GTK_TABLE(tbl), sel, 1, 2, i+1, i+2,
-			     GTK_EXPAND, GTK_FILL, 5, 5);
-	    gtk_widget_show(sel);
+	sel = gtk_combo_new();
+	g_signal_connect(G_OBJECT(GTK_COMBO(sel)->entry), "changed",
+			 G_CALLBACK(update_return), cinfo);
+	list = get_selection_list(cinfo, -1, cinfo->rettype);
+	if (list != NULL) {
+	    gtk_combo_set_popdown_strings(GTK_COMBO(sel), list); 
+	    g_list_free(list);
 	}
+	gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(sel)->entry), TRUE);
+	gtk_table_attach(GTK_TABLE(tbl), sel, 1, 2, 1, 2,
+			 GTK_EXPAND, GTK_FILL, 5, 5);
+	gtk_widget_show(sel);
 
 	gtk_widget_show(tbl);
     }
@@ -579,90 +686,34 @@ static void function_call_dialog (call_info *cinfo)
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
 
-    /* Help button */
-    button = context_help_button(GTK_DIALOG(cinfo->dlg)->action_area, -1);
-    g_signal_connect(G_OBJECT(button), "clicked", 
-		     G_CALLBACK(fncall_help), cinfo);
-    gtk_widget_show(button);    
+    /* Help button? */
+    if (cinfo->n_params > 0 || cinfo->rettype != ARG_NONE) {
+	button = context_help_button(GTK_DIALOG(cinfo->dlg)->action_area, -1);
+	g_signal_connect(G_OBJECT(button), "clicked", 
+			 G_CALLBACK(fncall_help), cinfo);
+	gtk_widget_show(button);
+    }  
 
     gtk_widget_show(cinfo->dlg);
 }
 
-static int check_args_and_rets (call_info *cinfo)
-{
-    int i;
-
-    if (cinfo->args != NULL) {
-	for (i=0; i<cinfo->n_params; i++) {
-	    if (cinfo->args[i] == NULL) {
-		errbox(_("Argument %d (%s) is missing"), i + 1,
-		       fn_param_name(cinfo->func, i));
-		return 1;
-	    }
-	}
-    }
-
-    if (cinfo->rets != NULL) {
-	int nr = 0;
-
-	for (i=0; i<cinfo->n_returns; i++) {
-	    if (cinfo->rets[i] != NULL) {
-		nr++;
-	    }
-	}
-	if (nr > 0 && nr < cinfo->n_returns) {
-	    errbox(_("You should assign all return values, or none"));
-	    return E_DATA;
-	}
-    }    
-
-    return 0;
-}
-
-static int package_function_exec (char *fnline, PRN *prn)
-{
-    char *gotline = NULL;
-    int err = 0;
-
-    while (!gretl_execute_loop()) {
-	gotline = gretl_function_get_line(fnline, MAXLINE, &Z, &datainfo, &err);
-	if (gotline == NULL || *gotline == '\0') {
-	    break;
-	}
-	if (!err) {
-#if FCDEBUG
-	    fprintf(stderr, "package_function_exec: '%s'\n", fnline); 
-#endif	    
-	    err = gui_exec_line(fnline, prn, SCRIPT_EXEC, NULL);
-	}
-    }
-
-    return err;
-}
-
-static int fn_executor (char *fnline, PRN *prn)
-{
-    int err = 0;
-
-    while (!err && (gretl_execute_loop() || gretl_executing_function())) {
-	if (gretl_execute_loop()) { 
-	    err = gretl_loop_exec(fnline, &Z, &datainfo, models, prn);
-	} else if (gretl_executing_function()) {
-	    err = package_function_exec(fnline, prn);
-	}
-    }
-
-    return err;
-} 
-
 static int function_data_check (call_info *cinfo)
 {
     int i, err = 0;
 
+    /* FIXME provide a way for a function to signal that
+       it doesn't need data loaded? */
+
+    if (datainfo == NULL || datainfo->v == 0) {
+	errbox(_("Please open a data file first"));
+	return 1;
+    }
+
     for (i=0; i<cinfo->n_params; i++) {
 	int type = fn_param_type(cinfo->func, i);
 
-	if (type == ARG_SERIES || type == ARG_LIST) {
+	if (type == ARG_SERIES || type == ARG_LIST ||
+	    type == ARG_REF_SERIES) {
 	    if (datainfo == NULL || datainfo->v == 0) {
 		errbox(_("Please open a data file first"));
 		err = 1;
@@ -670,14 +721,9 @@ static int function_data_check (call_info *cinfo)
 	    }
 	}
 	if (type == ARG_LIST) {
-	    cinfo->need_list = 1;
-	} else if (type == ARG_MATRIX) {
-	    if (n_user_matrices() == 0) {
-		errbox(_("This function takes a matrix argument\n"
-			 "but no matrices are currently defined"));
-		err = 1;
-		break;
-	    }
+	    cinfo->extracol = 1;
+	} else if (type == ARG_MATRIX || type == ARG_REF_MATRIX) {
+	    cinfo->extracol = 1;
 	}
     }
 
@@ -686,7 +732,6 @@ static int function_data_check (call_info *cinfo)
 
 static int temp_install_remote_fnpkg (const char *fname, char *target)
 {
-    char errbuf[80];
     int err = 0;
 
     build_path(target, paths.userdir, "dltmp", NULL);
@@ -695,9 +740,9 @@ static int temp_install_remote_fnpkg (const char *fname, char *target)
 	return err;
     }
 
-    err = retrieve_remote_function_package(fname, target, errbuf);
+    err = retrieve_remote_function_package(fname, target);
     if (err) {
-	show_network_error(NULL, errbuf);
+	show_network_error(NULL);
 	return err;
     } 
 
@@ -714,15 +759,45 @@ static int temp_install_remote_fnpkg (const char *fname, char *target)
     return err;
 }
 
-void call_function_package (const char *fname, GtkWidget *w)
+static int addressify_var (call_info *cinfo, int i)
 {
+    char *numchars = "0123456789+-.";
+    int t = fn_param_type(cinfo->func, i);
+    char *s = cinfo->args[i];
+
+    return (t == ARG_REF_SCALAR && strchr(numchars, *s)) ||
+	(t == ARG_REF_MATRIX && *s == '{');
+}
+
+static int needs_amp (call_info *cinfo, int i)
+{
+    int t = fn_param_type(cinfo->func, i);
+    char *s = cinfo->args[i];
+
+    if (*s == '&') {
+	return 0;
+    }
+
+    if (t != ARG_REF_SCALAR && t != ARG_REF_SERIES &&
+	t != ARG_REF_MATRIX) {
+	return 0;
+    }
+
+    return 1;
+}
+
+void call_function_package (const char *fname, GtkWidget *w,
+			    int *loaderr)
+{
+    ExecState state;
     char tmpfile[FILENAME_MAX];
     char fnline[MAXLINE];
+    char auxline[MAXLINE];
     const char *fnname;
     FuncDataReq dreq;
     float minver;
     PRN *prn;
-    call_info cinfo;
+    call_info *cinfo;
     int i, err = 0;
 
     *tmpfile = 0;
@@ -731,11 +806,12 @@ void call_function_package (const char *fname, GtkWidget *w)
 	/* not a full filename -> a function package on server */
 	err = temp_install_remote_fnpkg(fname, tmpfile);
     } else {
-	if (!user_function_file_is_loaded(fname)) {
+	if (!function_package_is_loaded(fname)) {
 	    err = load_user_function_file(fname);
 	    if (err) {
 		fprintf(stderr, "load_user_function_file: failed on %s\n", fname);
 		errbox(_("Couldn't open %s"), fname);
+		*loaderr = 1;
 	    }
 	}
     }
@@ -744,12 +820,15 @@ void call_function_package (const char *fname, GtkWidget *w)
 	return;
     }
 
-    cinfo_init(&cinfo);
+    cinfo = cinfo_new();
+    if (cinfo == NULL) {
+	return;
+    }
 
-    /* get interface(s) for package */
+    /* get interface for package */
     err = function_package_get_info((*tmpfile)? tmpfile : fname,
 				    NULL,
-				    &cinfo.publist,
+				    &cinfo->iface,
 				    NULL,
 				    NULL,
 				    NULL,
@@ -762,56 +841,49 @@ void call_function_package (const char *fname, GtkWidget *w)
 	remove(tmpfile);
     }
 
-    if (cinfo.publist == NULL || cinfo.publist[0] == 0) {
-	free(cinfo.publist);
+    if (cinfo->iface < 0) {
 	errbox(_("Function package is broken"));
+	free(cinfo);
 	return;
     }
 
     err = check_function_needs(datainfo, dreq, minver);
     if (err) {
 	gui_errmsg(err);
+	free(cinfo);
 	return;
     }
 
-    /* FIXME selection of interface, if there's more than one
-       available?
-    */
-    cinfo.iface = cinfo.publist[1];
-    cinfo.func = get_user_function_by_index(cinfo.iface);
+    cinfo->func = get_user_function_by_index(cinfo->iface);
 
-    if (cinfo.func == NULL) {
+    if (cinfo->func == NULL) {
 	fprintf(stderr, "get_user_function_by_index: got NULL for idx = %d\n", 
-		cinfo.iface);
+		cinfo->iface);
 	errbox(_("Couldn't get function package information"));
+	free(cinfo);
 	return;
     }
 
-    cinfo.n_params = fn_n_params(cinfo.func);
+    cinfo->n_params = fn_n_params(cinfo->func);
 
-    if (function_data_check(&cinfo)) {
-	cinfo_free(&cinfo);
+    if (function_data_check(cinfo)) {
+	cinfo_free(cinfo);
 	return;
     }
 
-    err = user_func_get_return_types(cinfo.func,
-				     &cinfo.n_returns,
-				     &cinfo.return_types);
+    cinfo->rettype = user_func_get_return_type(cinfo->func);
 
     if (err) {
 	fprintf(stderr, "user_func_get_return_types: failed for idx = %d\n",
-		cinfo.iface);
+		cinfo->iface);
 	errbox(_("Couldn't get function package information"));
 	return;
     }
 
-    function_call_dialog(&cinfo);
-    if (cinfo.canceled) {
-	return;
-    }
+    function_call_dialog(cinfo);
 
-    if (check_args_and_rets(&cinfo)) {
-	cinfo_free(&cinfo);
+    if (!cinfo->ok || check_args(cinfo)) {
+	cinfo_free(cinfo);
 	return;
     }
 
@@ -820,36 +892,46 @@ void call_function_package (const char *fname, GtkWidget *w)
     gtk_widget_destroy(w);
 #endif
 
-    fnname = user_function_name_by_index(cinfo.iface);
+    fnname = user_function_name_by_index(cinfo->iface);
     *fnline = 0;
 
-    if (cinfo.rets != NULL) {
-	for (i=0; i<cinfo.n_returns; i++) {
-	    if (cinfo.rets[i] == NULL) {
-		break;
-	    }
-	    strcat(fnline, cinfo.rets[i]);
-	    if (i < cinfo.n_returns - 1) {
-		strcat(fnline, ", ");
-	    } else {
-		strcat(fnline, " = ");
-	    }
-	}
+    if (cinfo->ret != NULL) {
+	strcat(fnline, cinfo->ret);
+	strcat(fnline, " = ");
     }    
 
     strcat(fnline, fnname);
 
-    if (cinfo.args != NULL) {
-	strcat(fnline, " ");
-	for (i=0; i<cinfo.n_params; i++) {
-	    strcat(fnline, cinfo.args[i]);
-	    if (i < cinfo.n_params - 1) {
+    if (cinfo->args != NULL) {
+	strcat(fnline, "(");
+	for (i=0; i<cinfo->n_params; i++) {
+	    char auxname[VNAMELEN];
+
+	    if (addressify_var(cinfo, i)) {
+		sprintf(auxname, "FNARG%d", i + 1);
+		sprintf(auxline, "genr %s=%s", auxname, cinfo->args[i]);
+		err = generate(auxline, &Z, datainfo, OPT_NONE, NULL);
+		if (!err) {
+		    free(cinfo->args[i]);
+		    cinfo->args[i] = g_strdup(auxname);
+		}
+	    } 
+	    if (needs_amp(cinfo, i)) {
+		strcat(fnline, "&");
+	    }
+	    strcat(fnline, cinfo->args[i]);
+	    if (i < cinfo->n_params - 1) {
 		strcat(fnline, ", ");
 	    }
 	}
+	strcat(fnline, ")");
+    } else {
+	strcat(fnline, "()");
     }
 
-    cinfo_free(&cinfo);
+    /* FIXME destroy any "ARG" vars or matrices that were created? */
+
+    cinfo_free(cinfo);
 
     if (bufopen(&prn)) {
 	return;
@@ -859,13 +941,19 @@ void call_function_package (const char *fname, GtkWidget *w)
     fprintf(stderr, "fnline: '%s'\n", fnline);
 #endif
 
-    err = gui_exec_line(fnline, prn, SCRIPT_EXEC, NULL);
-    if (!err) {
-	err = fn_executor(fnline, prn);
-    }
+    gretl_exec_state_init(&state, SCRIPT_EXEC, fnline, get_lib_cmd(),
+			  models, prn);
+
+    err = gui_exec_line(&state, &Z, &datainfo);
 
     if (err) {
-	gui_errmsg(err);
+	const char *msg = get_funcerr_message();
+
+	if (*msg != 0) {
+	    errbox(msg);
+	} else {
+	    gui_errmsg(err);
+	}
     } else {
 	view_buffer(prn, 80, 400, fnname, PRINT, NULL);
     }
