@@ -19,9 +19,10 @@
 
 #include "gretl.h"
 #include "dlgutils.h"
+#include "datafiles.h"
 #include "textbuf.h"
 #include "fileselect.h"
-#include "webget.h"
+#include "gretl_www.h"
 #include "fnsave.h"
 
 #ifdef G_OS_WIN32
@@ -48,10 +49,9 @@ struct function_info_ {
     char *version;
     char *date;
     char *pkgdesc;
-    char **help;
-    int *publist;
+    char *help;
+    int pub;
     int *privlist;
-    int n_public;
     int iface;
     FuncDataReq dreq;
     float minver;
@@ -87,29 +87,15 @@ function_info *finfo_new (void)
     finfo->saveas = 0;
     finfo->iface = -1;
 
-    finfo->n_public = 0;
     finfo->help = NULL;
-    finfo->publist = NULL;
+    finfo->pub = -1;
     finfo->privlist = NULL;
     finfo->dreq = 0;
-    finfo->minver = 1.5;
+    finfo->minver = 1.6;
 
     return finfo;
 }
 
-static int finfo_init (function_info *finfo)
-{
-    finfo->n_public = finfo->publist[0];
-
-    finfo->help = strings_array_new(finfo->n_public);
-    if (finfo->help == NULL) {
-	nomem();
-	return E_ALLOC;
-    }
-
-    return 0;
-}
-
 static void finfo_free (function_info *finfo)
 {
     free(finfo->fname);
@@ -117,10 +103,8 @@ static void finfo_free (function_info *finfo)
     free(finfo->version);
     free(finfo->date);
     free(finfo->pkgdesc);
-    free(finfo->publist);
     free(finfo->privlist);
-
-    free_strings_array(finfo->help, finfo->n_public);
+    free(finfo->help);
 
     free(finfo);
 }
@@ -177,14 +161,6 @@ static char *trim_text (const char *s)
     return ret;
 }
 
-static int help_text_index (function_info *finfo)
-{
-    int i;
-
-    i = gtk_option_menu_get_history(GTK_OPTION_MENU(finfo->ifsel));
-    return finfo->publist[i+1];
-}
-
 static void login_finalize (GtkWidget *w, login_info *linfo)
 {
     linfo->login = entry_box_get_trimmed_text(linfo->login_entry);
@@ -193,6 +169,62 @@ static void login_finalize (GtkWidget *w, login_info *linfo)
     gtk_widget_destroy(linfo->dlg);
 }
 
+/* for use in File, Save dialog */
+
+void get_default_package_name (char *fname, gpointer p)
+{
+    function_info *finfo = (function_info *) p;
+    const char *pubname;
+
+    *fname = '\0';
+    pubname = user_function_name_by_index(finfo->pub);  
+
+    if (pubname != NULL) {
+	strcpy(fname, pubname);
+	strcat(fname, ".gfn");	
+    }
+}
+
+static int check_version_string (const char *s)
+{
+    int dc = 0;
+    int err = 0;
+
+    if (!isdigit(*s) || (*s && !isdigit(s[strlen(s) - 1]))) {
+	err = 1;
+    }
+
+    while (*s && !err) {
+	if (*s == '.' && ++dc > 2) {
+	    err = 1;
+	} else if (!isdigit(*s) && strspn(s, ".") != 1) {
+	    err = 1;
+	}
+	s++;
+    }
+
+    return err;
+}
+
+static int maybe_revise_package_name (function_info *finfo)
+{
+    gchar *base = g_strdup(finfo->fname);
+    const char *pubname;
+    char *p;
+
+    p = strrchr(base, SLASH);
+    if (p != NULL) {
+	*(p + 1) = '\0';
+    }
+
+    free(finfo->fname);
+    pubname = user_function_name_by_index(finfo->pub);
+    finfo->fname = g_strdup_printf("%s%s.gfn", base, pubname);
+    g_free(base);
+
+    return 0;
+}
+
 static void real_finfo_save (function_info *finfo)
 {
     char **fields[] = {
@@ -217,17 +249,18 @@ static void real_finfo_save (function_info *finfo)
 	return;
     }
 
+    if (check_version_string(finfo->version)) {
+	errbox(_("Invalid version string: use numbers and '.' only"));
+	return;
+    }
+
     finfo->upload = 
 	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(finfo->check));
 
-    if (finfo->n_public > 1) {
-	hidx = help_text_index(finfo);
-    } 
-
     if (hidx >= 0) {
 	char *tmp = textview_get_text(finfo->text);
 
-	finfo->help[hidx] = trim_text(tmp);
+	finfo->help = trim_text(tmp);
 	free(tmp);
     }
 
@@ -235,20 +268,17 @@ static void real_finfo_save (function_info *finfo)
 	file_selector(_("Save function package"), SAVE_FUNCTIONS, 
 		      FSEL_DATA_MISC, finfo);
     } else {
-	save_user_functions(finfo->fname, finfo);
+	maybe_revise_package_name(finfo);
+	err = save_user_functions(finfo->fname, finfo);
+	if (!err) {
+	    infobox(_("Saved package as %s"), finfo->fname);
+	}
     }
 }
 
-static void finfo_save_as (GtkWidget *w, function_info *finfo)
-{
-    finfo->saveas = 1;
-    real_finfo_save(finfo);
-}
-
 static void finfo_save (GtkWidget *w, function_info *finfo)
 {
     finfo->saveas = (finfo->fname == NULL);
-
     real_finfo_save(finfo);
 }
 
@@ -268,65 +298,17 @@ enum {
     HIDX_SWITCH
 };
 
-static void 
-set_dialog_info_from_fn (function_info *finfo, int idx, int code)
+const char *print_today (void)
 {
-    const char *attrib = NULL;
-    const char *keys[] = {
-	"author",
-	"version",
-	"date",
-	"pkgdesc"
-    };
-    const char *etxt;
+    static char timestr[16];
+    struct tm *local;
+    time_t t;
 
-    static int old_hidx;
-    int i, new_hidx;
+    t = time(NULL);
+    local = localtime(&t);
+    strftime(timestr, 15, "%Y-%m-%d", local);
 
-    if (code == HIDX_INIT) {
-	old_hidx = new_hidx = 0;
-    } else {
-	new_hidx = help_text_index(finfo);
-    }
-
-    for (i=0; i<NENTRIES; i++) {
-	etxt = gtk_entry_get_text(GTK_ENTRY(finfo->entries[i]));
-	if (*etxt == '\0') {
-	    gretl_function_get_info(idx, keys[i], &attrib);
-	    if (attrib != NULL) {
-		etxt = gtk_entry_get_text(GTK_ENTRY(finfo->entries[i]));
-	    }
-	}
-    }
-
-    if (new_hidx != old_hidx) {
-	/* we're switching the "active" interface, so save the 
-	   help text for the previous interface */
-	char *old_help = textview_get_text(finfo->text);
-
-	free(finfo->help[old_hidx]);
-	finfo->help[old_hidx] = old_help;
-    }
-
-    if (code == HIDX_INIT || new_hidx != old_hidx) {
-	/* initializing or switching: insert new help text */
-	const char *new_help;
-
-	gretl_function_get_info(idx, "help", &new_help);
-	textview_set_text(finfo->text, new_help);
-    }
-
-    old_hidx = new_hidx;
-}
-
-static gboolean update_public (GtkOptionMenu *menu, 
-			       function_info *finfo)
-{
-    int i = gtk_option_menu_get_history(menu);
-
-    set_dialog_info_from_fn(finfo, finfo->publist[i+1], HIDX_SWITCH);
-
-    return FALSE;
+    return timestr;
 }
 
 static gboolean update_iface (GtkOptionMenu *menu, 
@@ -334,11 +316,10 @@ static gboolean update_iface (GtkOptionMenu *menu,
 {
     int i = gtk_option_menu_get_history(menu);
 
-    if (i < finfo->publist[0]) {
-	finfo->iface = finfo->publist[i+1];
+    if (i == 0) {
+	finfo->iface = finfo->pub;
     } else {
-	i -= finfo->publist[0];
-	finfo->iface = finfo->privlist[i+1];
+	finfo->iface = finfo->privlist[i];
     }
 
     return FALSE;
@@ -430,16 +411,14 @@ static GtkWidget *interface_selector (function_info *finfo, int iface)
     const char *fnname;
     int i;
 
-    finfo->iface = finfo->publist[1];
+    finfo->iface = finfo->pub;
 
     ifmenu = gtk_option_menu_new();
     menu = gtk_menu_new();
 
-    for (i=1; i<=finfo->publist[0]; i++) {
-	fnname = user_function_name_by_index(finfo->publist[i]);
-	tmp = gtk_menu_item_new_with_label(fnname);
-	gtk_menu_shell_append(GTK_MENU_SHELL(menu), tmp);
-    }
+    fnname = user_function_name_by_index(finfo->pub);
+    tmp = gtk_menu_item_new_with_label(fnname);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), tmp);
 
     if (iface == IFACE_ALL && finfo->privlist != NULL) {
 	for (i=1; i<=finfo->privlist[0]; i++) {
@@ -573,6 +552,33 @@ static void add_minver_selector (GtkWidget *tbl, int i,
     gtk_widget_show_all(hbox);
 }
 
+static GtkWidget *editable_text_box (void)
+{
+    GtkTextBuffer *tbuf = gretl_text_buf_new();
+    GtkWidget *w = gtk_text_view_new_with_buffer(tbuf);
+
+    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(w), GTK_WRAP_WORD);
+    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(w), 4);
+    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(w), 4);
+    gtk_widget_modify_font(GTK_WIDGET(w), fixed_font);
+    gtk_text_view_set_editable(GTK_TEXT_VIEW(w), TRUE);
+    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(w), TRUE);
+
+    return w;
+}
+
+static const gchar *get_user_string (void)
+{
+    const gchar *name;
+
+    name = g_get_real_name();
+    if (name == NULL) {
+	name = g_get_user_name();
+    }
+
+    return name;
+}
+
 static void finfo_dialog (function_info *finfo)
 {
     GtkWidget *button, *label;
@@ -590,12 +596,11 @@ static void finfo_dialog (function_info *finfo)
 	finfo->pkgdesc
     };
     const char *fnname;
+    int focused = 0;
+    const char *hlp;
+    gchar *ltxt;
     int i;
 
-    if (finfo_init(finfo)) {
-	return;
-    }
-
     finfo->dlg = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
     g_signal_connect(G_OBJECT(finfo->dlg), "destroy", 
@@ -603,7 +608,7 @@ static void finfo_dialog (function_info *finfo)
 
     gtk_window_set_title(GTK_WINDOW(finfo->dlg), 
 			 _("gretl: function package editor")); 
-    gtk_window_set_default_size(GTK_WINDOW(finfo->dlg), 640, 480);
+    gtk_window_set_default_size(GTK_WINDOW(finfo->dlg), 640, 500);
 
     vbox = gtk_vbox_new(FALSE, 5);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
@@ -631,9 +636,22 @@ static void finfo_dialog (function_info *finfo)
 
 	if (entry_texts[i] != NULL) {
 	    gtk_entry_set_text(GTK_ENTRY(entry), entry_texts[i]);
+	} else if (i == 0) {
+	    const gchar *s = get_user_string();
+
+	    if (s != NULL) {
+		gtk_entry_set_text(GTK_ENTRY(entry), s);
+	    }
+	} else if (i == 1) {
+	    gtk_entry_set_text(GTK_ENTRY(entry), "1.0");
+	} else if (i == 2) {
+	    gtk_entry_set_text(GTK_ENTRY(entry), print_today());
 	}
 
-	if (i == 1) {
+	if (i == 0 && entry_texts[i] == NULL) {
+	    gtk_widget_grab_focus(entry);
+	    focused = 1;
+	} else if (i == 1 && !focused) {
 	    gtk_widget_grab_focus(entry);
 	}
     }
@@ -642,29 +660,17 @@ static void finfo_dialog (function_info *finfo)
     add_data_requirement_menu(tbl, i, finfo);
     gtk_widget_show(tbl);
 
-    if (finfo->n_public > 1) {
-	/* drop-down selector for public interfaces */
-	hbox = label_hbox(vbox, _("Help text for"));
-	finfo->ifsel = interface_selector(finfo, IFACE_PUBLIC);
-	gtk_box_pack_start(GTK_BOX(hbox), finfo->ifsel, FALSE, FALSE, 5);
-	g_signal_connect(G_OBJECT(GTK_OPTION_MENU(finfo->ifsel)), "changed",
-			 G_CALLBACK(update_public), finfo);
-	gtk_widget_show(hbox);
-    } else {
-	/* only one public interface */
-	gchar *ltxt;
+    fnname = user_function_name_by_index(finfo->pub);
+    ltxt = g_strdup_printf(_("Help text for %s:"), fnname);
+    hbox = label_hbox(vbox, ltxt);
+    gtk_widget_show(hbox);
+    g_free(ltxt);
 
-	fnname = user_function_name_by_index(finfo->publist[1]);
-	ltxt = g_strdup_printf(_("Help text for %s:"), fnname);
-	hbox = label_hbox(vbox, ltxt);
-	gtk_widget_show(hbox);
-	g_free(ltxt);
-    }
-
-    finfo->text = create_text(NULL, -1, -1, TRUE);
+    finfo->text = editable_text_box();
     text_table_setup(vbox, finfo->text);
 
-    set_dialog_info_from_fn(finfo, finfo->publist[1], HIDX_INIT);
+    gretl_function_get_info(finfo->pub, "help", &hlp);
+    textview_set_text(finfo->text, hlp);
 
     /* edit code button, possibly with selector */
     hbox = gtk_hbox_new(FALSE, 5);
@@ -675,13 +681,13 @@ static void finfo_dialog (function_info *finfo)
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(edit_code_callback), finfo);
 
-    if (finfo->publist[0] > 1 || finfo->privlist != NULL) {
+    if (finfo->privlist != NULL) {
 	finfo->codesel = interface_selector(finfo, IFACE_ALL);
 	gtk_box_pack_start(GTK_BOX(hbox), finfo->codesel, FALSE, FALSE, 5);
 	g_signal_connect(G_OBJECT(GTK_OPTION_MENU(finfo->codesel)), "changed",
 			 G_CALLBACK(update_iface), finfo);
     } else {
-	finfo->iface = finfo->publist[1];
+	finfo->iface = finfo->pub;
     }
 
     gtk_widget_show_all(hbox);
@@ -696,13 +702,6 @@ static void finfo_dialog (function_info *finfo)
     gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 10);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-    /* SaveAs button */
-    button = gtk_button_new_from_stock(GTK_STOCK_SAVE_AS);
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_container_add(GTK_CONTAINER(hbox), button);
-    g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(finfo_save_as), finfo);
-
     /* Save button */
     button = gtk_button_new_from_stock(GTK_STOCK_SAVE);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
@@ -743,7 +742,8 @@ static void login_dialog (login_info *linfo)
     gtk_widget_show(hbox);
 
     tbl = gtk_table_new(2, 2, FALSE);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(linfo->dlg)->vbox), tbl, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(linfo->dlg)->vbox), tbl, 
+		       FALSE, FALSE, 5);
 
     for (i=0; i<2; i++) {
 	char *src = (i == 0)? linfo->login : linfo->pass;
@@ -758,6 +758,7 @@ static void login_dialog (login_info *linfo)
 	entry = gtk_entry_new();
 	gtk_entry_set_width_chars(GTK_ENTRY(entry), 34);
 	gtk_entry_set_editable(GTK_ENTRY(entry), TRUE);
+	gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), entry, 1, 2, i, i+1);
 	if (src != NULL) {
 	    gtk_entry_set_text(GTK_ENTRY(entry), src);
@@ -811,39 +812,180 @@ static void login_dialog (login_info *linfo)
     gtk_widget_show(linfo->dlg);
 }
 
+#define url_reserved(c) (strchr(";/?:@&=+$,<>%#\t\r\n\v\0", c) != NULL)
+
+static int count_specials (const char *s)
+{
+    int n = 0;
+
+    while (*s) {
+	if (url_reserved(*s) || !isprint(*s)) {
+	    n++;
+	}
+	s++;
+    }
+
+    return n;
+}
+
+static char *url_encode_string (const char *s)
+{
+    char *encstr, *p;
+    int n;
+
+    if (s == NULL) {
+	return NULL;
+    }
+
+    n = count_specials(s);
+    if (n == 0) {
+	return gretl_strdup(s);
+    }
+
+    encstr = malloc(strlen(s) + n * 2 + 1);
+
+    if (encstr != NULL) {
+	p = encstr;
+	while (*s) {
+	    if (*s == ' ') {
+		*p++ = '+';
+	    } else if (url_reserved(*s) || !isprint(*s)) {
+		sprintf(p, "%%%.2X", *s);
+		p += 3;
+	    } else {
+		*p++ = *s;
+	    } 
+	    s++;
+	}
+	*p = '\0';
+    }
+
+    return encstr;
+}
+
 static void do_upload (const char *fname)
 {
+    char *ulogin = NULL;
+    char *upass = NULL;
+    char *ufname = NULL;
+    char *ubuf = NULL;
+    char *buf = NULL;
+    char *retbuf = NULL;
     login_info linfo;
+    GdkDisplay *disp;
+    GdkCursor *cursor;
+    GdkWindow *w1;
+    gint x, y;
+    int err = 0;
 
     login_dialog(&linfo);
 
-    if (!linfo.canceled) {
-	char *errbuf = NULL;
-	int err = upload_function_package(linfo.login,
-					  linfo.pass,
-					  fname,
-					  &errbuf);
-	if (err) {
-	    errbox(errbuf);
-	}
-	free(errbuf);
+    if (linfo.canceled) {
+	linfo_free(&linfo);
+	return;
     }
 
+    /* set waiting cursor */
+    disp = gdk_display_get_default();
+    cursor = gdk_cursor_new(GDK_WATCH);
+    w1 = gdk_display_get_window_at_pointer(disp, &x, &y);
+    gdk_window_set_cursor(w1, cursor);
+    gdk_display_sync(disp);
+    gdk_cursor_unref(cursor);
+
+    g_file_get_contents(fname, &buf, NULL, NULL);
+
+    if (buf == NULL) {
+	err = E_ALLOC;
+    } else {  
+	ubuf = url_encode_string(buf);
+	ulogin = url_encode_string(linfo.login);
+	upass = url_encode_string(linfo.pass);
+	ufname = url_encode_string(path_last_element(fname));
+	
+	if (ubuf == NULL || ulogin == NULL || upass == NULL || ufname == NULL) {
+	    err = E_ALLOC;
+	}
+    }
+
+    if (!err) {
+	err = upload_function_package(ulogin, upass, ufname, ubuf,
+				      &retbuf);
+    }
+
+    /* reset default cursor */
+    gdk_window_set_cursor(w1, NULL);
+
+    if (err) {
+	gui_errmsg(err);
+    } else if (retbuf != NULL && *retbuf != '\0') {
+	infobox(retbuf);
+    }
+
+    free(ulogin);
+    free(upass);
+    free(ufname);
+    free(ubuf);
+    free(buf);
+    free(retbuf);
+
     linfo_free(&linfo);
 }
 
-void save_user_functions (const char *fname, gpointer p)
+static int 
+fnpkg_check_filename (function_info *finfo, const char *fname)
+{
+    const char *p = strrchr(fname, SLASH);
+    char funname[FN_NAMELEN];
+    int i, n, err = 0;
+
+    if (p == NULL) {
+	p = fname;
+    } else {
+	p++;
+    }
+
+    n = 0;
+    for (i=0; p[i] != '\0'; i++) {
+	if (p[i] == '-' || !strcmp(p + i, ".gfn")) {
+	    break;
+	}
+	n++;
+    }
+
+    if (n >= FN_NAMELEN) {
+	err = 1;
+    } else {
+	const char *iname = user_function_name_by_index(finfo->pub);
+
+	*funname = '\0';
+	strncat(funname, p, n);
+	if (strcmp(funname, iname)) {
+	    err = 1;
+	}
+    }
+
+    if (err) {
+	errbox(_("The package filename must match the name of\n"
+		 "the package's public interface"));
+    }
+
+    return err;
+}
+
+int save_user_functions (const char *fname, gpointer p)
 {
     function_info *finfo = p;
     int i, err;
 
-    /* sync filename with functions editor */
+    /* sync/check filename with functions editor */
     if (finfo->fname == NULL) {
+	err = fnpkg_check_filename(finfo, fname);
+	if (err) {
+	    return 1;
+	}
 	finfo->fname = g_strdup(fname);
-    } else if (strcmp(fname, finfo->fname)) {
-	g_free(finfo->fname);
-	finfo->fname = g_strdup(fname);
-    }
+    } 
 
     if (finfo->privlist != NULL) {
 	for (i=1; i<=finfo->privlist[0]; i++) {
@@ -851,17 +993,15 @@ void save_user_functions (const char *fname, gpointer p)
 	}
     }
 
-    for (i=1; i<=finfo->publist[0]; i++) {
-	gretl_function_set_info(finfo->publist[i], finfo->help[i-1]);
-	gretl_function_set_private(finfo->publist[i], FALSE);
-    }
+    gretl_function_set_info(finfo->pub, finfo->help);
+    gretl_function_set_private(finfo->pub, FALSE);
 
 #if 0
     fprintf(stderr, "author='%s'\n", finfo->author);
     fprintf(stderr, "version='%s'\n", finfo->version);
     fprintf(stderr, "date='%s'\n", finfo->date);
     fprintf(stderr, "pkgdesc='%s'\n", finfo->pkgdesc);
-    printlist(finfo->publist, "finfo->publist");
+    fprintf(stderr, "finfo->pub = %d\n", finfo->pub);
     printlist(finfo->privlist, "finfo->privlist");
     fprintf(stderr, "dreq=%d\n", finfo->dreq);
     fprintf(stderr, "minver=%.2f\n", (double) finfo->minver);
@@ -869,7 +1009,7 @@ void save_user_functions (const char *fname, gpointer p)
 		
     err = write_function_package(finfo->pkg,
 				 fname,
-				 finfo->publist, 
+				 finfo->pub, 
 				 finfo->privlist,
 				 finfo->author,
 				 finfo->version,
@@ -880,9 +1020,14 @@ void save_user_functions (const char *fname, gpointer p)
 
     if (err) {
 	gui_errmsg(err);
-    } else if (finfo->upload) {
-	do_upload(fname);
+    } else {
+	maybe_update_func_files_window(1);
+	if (finfo->upload) {
+	    do_upload(fname);
+	}
     }
+
+    return err;
 }
 
 /* called from function selection dialog: a set of functions has been
@@ -893,6 +1038,7 @@ void prepare_functions_save (void)
 {
     function_info *finfo;
     int *list = NULL;
+    int i;
 
     if (storelist == NULL) {
 	return;
@@ -910,18 +1056,22 @@ void prepare_functions_save (void)
 	return;
     }
 
-    if (gretl_list_has_separator(list)) {
-	if (gretl_list_split_on_separator(list, &finfo->publist, 
-					  &finfo->privlist)) {
+    finfo->pub = list[1];
+
+    if (list[0] > 1) {
+	finfo->privlist = gretl_list_new(list[0] - 1);
+	if (finfo->privlist == NULL) {
 	    nomem();
 	    free(finfo);
 	    free(list);
 	    return;
 	} else {
+	    for (i=1; i<=finfo->privlist[0]; i++) {
+		finfo->privlist[i] = list[i+1];
+	    }
 	    free(list);
 	}
     } else {
-	finfo->publist = list;
 	finfo->privlist = NULL;
     }
 
@@ -929,16 +1079,18 @@ void prepare_functions_save (void)
     finfo_dialog(finfo);
 }
 
-void edit_function_package (const char *fname)
+void edit_function_package (const char *fname, int *loaderr)
 {
     function_info *finfo;
+    const char *p;
     int err = 0;
 
-    if (!user_function_file_is_loaded(fname)) {
+    if (!function_package_is_loaded(fname)) {
 	err = load_user_function_file(fname);
 	if (err) {
 	    fprintf(stderr, "load_user_function_file: failed on %s\n", fname);
 	    errbox(_("Couldn't open %s"), fname);
+	    *loaderr = 1;
 	    return;
 	}
     }
@@ -950,7 +1102,7 @@ void edit_function_package (const char *fname)
 
     err = function_package_get_info(fname,
 				    &finfo->pkg,
-				    &finfo->publist,
+				    &finfo->pub,
 				    &finfo->privlist,
 				    &finfo->author,
 				    &finfo->version,
@@ -966,9 +1118,14 @@ void edit_function_package (const char *fname)
 	return;
     }
 
+    p = strrchr(fname, SLASH);
+    if (p == NULL) {
+	p = fname;
+    } else {
+	p++;
+    }
+
     finfo->fname = g_strdup(fname);
 
     finfo_dialog(finfo);
 }
-
-
