@@ -20,10 +20,12 @@
 /* gpt_control.c for gretl -- gnuplot controller */
 
 #include "gretl.h"
+#include "plotspec.h"
 #include "gpt_control.h"
 #include "session.h"
 #include "gpt_dialog.h"
 #include "fileselect.h"
+#include "calculator.h"
 
 #define GPDEBUG 0
 #define POINTS_DEBUG 0
@@ -65,7 +67,7 @@ enum {
     PLOT_POLAR          = 1 << 6
 } plot_format_flags;
 
-#define MAX_MARKERS 100
+#define MAX_MARKERS 120
 
 #define plot_is_saved(p)        (p->status & PLOT_SAVED)
 #define plot_has_controller(p)  (p->status & PLOT_HAS_CONTROLLER)
@@ -92,7 +94,7 @@ enum {
 #define plot_is_roots(p)        (p->spec->code == PLOT_VAR_ROOTS)
 
 #define plot_has_regression_list(p) (p->spec->reglist != NULL)
-#define plot_show_all_markers(p)    (p->spec->flags & GPTSPEC_ALL_MARKERS)
+#define plot_show_all_markers(p)    (p->spec->flags & GPT_ALL_MARKERS)
 
 enum {
     PNG_START,
@@ -201,10 +203,8 @@ void set_plot_has_y2_axis (png_plot *plot, gboolean s)
     }
 }
 
-void plot_label_position_click (GtkWidget *w, GPT_SPEC *spec)
+void plot_label_position_click (GtkWidget *w, png_plot *plot)
 {
-    png_plot *plot = (png_plot *) spec->ptr;
-
     if (plot != NULL) {
 	GtkWidget *entry;
 	GdkCursor* cursor;
@@ -220,12 +220,17 @@ void plot_label_position_click (GtkWidget *w, GPT_SPEC *spec)
     }
 }
 
-static void line_to_file (const char *s, FILE *fp, int l2)
+/* do or undo conversion between latin-2 and HTML representation,
+   if needed; otherwise pass the line through straight */
+
+static void line_to_file (const char *s, FILE *fp, int lv)
 {
 #ifdef ENABLE_NLS
-    if (l2 == -1) {
+    if (lv == -2) {
+	fprintf(stderr, "line_to_file: lv=-2, doing print_as_html\n");
 	print_as_html(s, fp);
-    } else if (l2 == 1) {
+    } else if (lv == 2) {
+	fprintf(stderr, "line_to_file: lv=2, doing print_as_locale\n");
 	print_as_locale(s, fp);
     } else {
 	fputs(s, fp);
@@ -268,9 +273,9 @@ add_or_remove_png_term (const char *fname, int add, GPT_SPEC *spec)
     char restore_line[MAXLEN] = {0};
     int png_line_saved = 0;
 #ifdef ENABLE_NLS
-    int l2 = use_latin_2();
+    int lv = iso_latin_version();
 #else
-    int l2 = 0;
+    int lv = 0;
 #endif
 
     sprintf(temp, "%sgpttmp", paths.userdir);
@@ -296,6 +301,8 @@ add_or_remove_png_term (const char *fname, int add, GPT_SPEC *spec)
 	rewind(fsrc);
     }
 
+    fprintf(stderr, "add_or_remove_png_term: add = %d\n", add);
+
     if (add) {
 	int need_term_line = 1;
 
@@ -310,7 +317,8 @@ add_or_remove_png_term (const char *fname, int add, GPT_SPEC *spec)
 	}
 	if (need_term_line) {
 	    int ptype = (spec != NULL)? spec->code : PLOT_REGULAR;
-	    const char *pline = get_gretl_png_term_line(ptype);
+	    int flags = (spec != NULL)? spec->flags : 0;
+	    const char *pline = get_gretl_png_term_line(ptype, flags);
 
 	    fprintf(ftmp, "%s\n", pline);
 	}	    
@@ -322,7 +330,7 @@ add_or_remove_png_term (const char *fname, int add, GPT_SPEC *spec)
     while (fgets(fline, sizeof fline, fsrc)) {
 	if (add) {
 	    if (!commented_term_line(fline) && !set_output_line(fline)) {
-		line_to_file(fline, ftmp, -l2);
+		line_to_file(fline, ftmp, -lv);
 	    }
 	} else {
 	    /* we're removing the png term line */
@@ -335,25 +343,24 @@ add_or_remove_png_term (const char *fname, int add, GPT_SPEC *spec)
 		    png_line_saved = 1;
 		} 
 		printit = 0;
-	    }
-	    else if (commented_term_line(fline)) {
+	    } else if (commented_term_line(fline)) {
 		printit = 0;
 	    } else if (set_output_line(fline)) {
 		printit = 0;
-	    } else if (spec != NULL && (spec->flags & GPTSPEC_OLS_HIDDEN)
-		       && is_auto_ols_string(fline)) {
+	    } else if (spec != NULL && (spec->flags & GPT_FIT_HIDDEN)
+		       && is_auto_fit_string(fline)) {
 		printit = 0;
 	    }
 	    if (printit) {
-		line_to_file(fline, ftmp, l2);
+		line_to_file(fline, ftmp, lv);
 	    }
 	}
     }
 
     fclose(fsrc);
     fclose(ftmp);
-
     remove(fname);
+
     return rename(temp, fname);
 }
 
@@ -362,14 +369,19 @@ static int add_png_term_to_plotfile (const char *fname)
     return add_or_remove_png_term(fname, 1, NULL);
 }
 
-/* public because called from session.c when saving a graph file */
-
-int remove_png_term_from_plotfile (const char *fname, GPT_SPEC *spec)
+static int remove_png_term_from_plotfile (const char *fname, GPT_SPEC *spec)
 {
     return add_or_remove_png_term(fname, 0, spec);
 }
 
-void mark_plot_as_saved (GPT_SPEC *spec)
+/* public because called from session.c when saving a graph file */
+
+int remove_png_term_from_plotfile_by_name (const char *fname)
+{
+    return add_or_remove_png_term(fname, 0, NULL);
+}
+
+static void mark_plot_as_saved (GPT_SPEC *spec)
 {
     png_plot *plot = (png_plot *) spec->ptr;
 
@@ -385,109 +397,136 @@ static int gnuplot_png_init (GPT_SPEC *spec, FILE **fpp)
 	return 1;
     }
 
-    fprintf(*fpp, "%s\n", get_gretl_png_term_line(spec->code));
+    fprintf(*fpp, "%s\n", get_gretl_png_term_line(spec->code, spec->flags));
     fprintf(*fpp, "set output '%sgretltmp.png'\n", paths.userdir);
 
     return 0;
 }
 
-/* take saved plot source file and make PNG from it, then display
-   the PNG */
-
-void display_session_graph_png (const char *fname) 
+int gp_term_code (gpointer p)
 {
-    char fullname[MAXLEN];
-    gchar *plotcmd;
-    int err = 0;
+    GPT_SPEC *spec = (GPT_SPEC *) p;
+    const char *s = spec->termtype;
 
-    if (g_path_is_absolute(fname)) {
-	strcpy(fullname, fname);
-    } else {
-	sprintf(fullname, "%s%s", paths.userdir, fname);
-    }
-
-    if (add_png_term_to_plotfile(fullname)) {
-	return;
-    }
-
-    plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, fullname);
-    err = gretl_spawn(plotcmd);
-    g_free(plotcmd);
-
-    if (err) {
-	errbox(_("Gnuplot error creating graph"));
-    } else {
-	gnuplot_show_png(fullname, NULL, 1);
-    }
+    if (!strncmp(s, "postscript", 10)) 
+	return GP_TERM_EPS;
+    else if (!strncmp(s, "PDF", 3)) 
+	return GP_TERM_PDF;
+    else if (!strcmp(s, "fig")) 
+	return GP_TERM_FIG;
+    else if (!strcmp(s, "latex")) 
+	return GP_TERM_TEX;
+    else if (!strncmp(s, "png", 3)) 
+	return GP_TERM_PNG;
+    else if (!strncmp(s, "emf", 3)) 
+	return GP_TERM_EMF;
+    else if (!strncmp(s, "svg", 3)) 
+	return GP_TERM_SVG;
+    else if (!strcmp(s, "plot commands")) 
+	return GP_TERM_PLT;
+    else 
+	return GP_TERM_NONE;
 }
 
-int maybe_switch_emf_point_style (char *s, PRN *prn)
+static int get_full_term_string (const GPT_SPEC *spec, char *termstr)
 {
-    char *p = strstr(s, "w points");
-    int do_pt2 = 0;
+    int cmds = 0;
 
-    if (p != NULL) {
-	if (strncmp(p + 8, " pt", 3)) {
-	    do_pt2 = 1;
-	}
-    }
+    if (!strcmp(spec->termtype, "postscript color")) {
+	strcpy(termstr, "postscript eps color"); 
+    } else if (!strcmp(spec->termtype, "postscript")) {
+	strcpy(termstr, "postscript eps"); 
+    } else if (!strcmp(spec->termtype, "PDF")) {
+	strcpy(termstr, "pdf");
+    } else if (!strcmp(spec->termtype, "fig")) {
+	strcpy(termstr, "fig");
+    } else if (!strcmp(spec->termtype, "latex")) {
+	strcpy(termstr, "latex");
+    } else if (!strcmp(spec->termtype, "png")) { 
+	const char *png_str = 
+	    get_gretl_png_term_line(spec->code, 0);
 
-    if (do_pt2) {
-	int i, len = p + 8 - s;
+	strcpy(termstr, png_str + 9);
+    } else if (!strcmp(spec->termtype, "emf color")) {
+	const char *emf_str = 
+	    get_gretl_emf_term_line(spec->code, 1);
 
-	for (i=0; i<len; i++) {
-	    pputc(prn, s[i]);
-	}
-	pputs(prn, " pt 2");
-	pputs(prn, p + 8);
+	strcpy(termstr, emf_str + 9);
+    } else if (!strcmp(spec->termtype, "plot commands")) { 
+	cmds = 1;
     } else {
-	pputs(prn, s);
+	strcpy(termstr, spec->termtype);
     }
 
-    return do_pt2;
+    return cmds;
 }
 
-void save_this_graph (GPT_SPEC *plot, const char *fname)
+void save_graph_to_file (gpointer data, const char *fname)
 {
-    FILE *fq;
-    PRN *prn;
+    GPT_SPEC *spec = (GPT_SPEC *) data;
     char plottmp[MAXLEN], plotline[MAXLEN], termstr[MAXLEN];
     gchar *plotcmd = NULL;
+    FILE *fq;
+    PRN *prn;
+    int mono = 0, png = 0, emf = 0;
     int cmds, err;
 
     if (user_fopen("gptout.tmp", plottmp, &prn)) {
 	return;
     }
 
-    fq = gretl_fopen(plot->fname, "r");
+    fq = gretl_fopen(spec->fname, "r");
     if (fq == NULL) {
 	errbox(_("Couldn't access graph info"));
 	gretl_print_destroy(prn);
 	return;
     }
  
-    cmds = get_termstr(plot, termstr);
-  
+    cmds = get_full_term_string(spec, termstr);
+
+    if (strstr(termstr, " mono")) {
+	mono = 1;
+    }
+
+    if (!strncmp(termstr, "png", 3)) {
+	png = 1;
+    } else if (!strncmp(termstr, "emf", 3)) {
+	emf = 1;
+    }
+
     if (cmds) {
-	if (copyfile(plot->fname, fname)) { 
+	if (copyfile(spec->fname, fname)) { 
 	    errbox(_("Failed to copy graph file"));
 	}
 	return;
     } else {
-	int done_pt2 = strncmp(termstr, "emf", 3);
-
 #ifdef ENABLE_NLS
 	pprint_gnuplot_encoding(termstr, prn);
 #endif
 	pprintf(prn, "set term %s\n", termstr);
 	pprintf(prn, "set output '%s'\n", fname);
 	while (fgets(plotline, MAXLEN-1, fq)) {
-	    if (!done_pt2 && strstr(plotline, "using 1:2")) {
-		done_pt2 = maybe_switch_emf_point_style(plotline, prn);
-	    } else if (strncmp(plotline, "set term", 8) && 
-		       strncmp(plotline, "set output", 10)) {
+	    if (!strncmp(plotline, "set term", 8) ||
+		!strncmp(plotline, "set output", 10)) {
+		continue;
+	    }
+	    if (mono && strstr(plotline, "set style fill solid")) {
+		pputs(prn, "set style fill solid 0.3\n");
+	    } else if (!png && html_encoded(plotline)) {
+		pprint_as_latin(prn, plotline, emf);
+	    } else {
 		pputs(prn, plotline);
 	    }
+#if 0
+	    if (!gretl_is_ascii(plotline)) {
+		fprintf(stderr, "non-ascii line: '%s'\n", plotline);
+		if (g_utf8_validate(plotline, -1, NULL)) {
+		    fprintf(stderr, " valid UTF-8\n");
+		} else {
+		    fprintf(stderr, " not valid UTF-8\n");
+		}
+	    }
+#endif
 	}
     }
 
@@ -557,79 +596,15 @@ static int get_gpt_marker (const char *line, char *label)
                       p == PLOT_VAR_ROOTS || \
 		      p == PLOT_ELLIPSE)
 
-static void plot_label_init (GPT_LABEL *lbl)
-{
-    lbl->text[0] = '\0';
-    lbl->just = GP_JUST_LEFT;
-    lbl->pos[0] = NADBL;
-    lbl->pos[1] = NADBL;
-}
+/* graphs where we don't attempt to find data coordinates */
 
-static GPT_SPEC *plotspec_new (void)
-{
-    GPT_SPEC *spec;
-    int i;
-
-    spec = mymalloc(sizeof *spec);
-    if (spec == NULL) {
-	return NULL;
-    }
-
-    spec->lines = mymalloc(MAX_PLOT_LINES * sizeof *spec->lines);
-
-    if (spec->lines == NULL) {
-	free(spec);
-	return NULL;
-    }
-
-    for (i=0; i<MAX_PLOT_LINES; i++) {
-	spec->lines[i].varnum = 0;
-	spec->lines[i].title[0] = 0;
-	spec->lines[i].formula[0] = 0;
-	spec->lines[i].style[0] = 0;
-	spec->lines[i].scale[0] = 0;
-	spec->lines[i].yaxis = 1;
-	spec->lines[i].type = 0;
-	spec->lines[i].width = 1;
-	spec->lines[i].ncols = 0;
-    }
-
-    for (i=0; i<4; i++) {
-	spec->titles[i][0] = 0;
-	spec->literal[i] = NULL;
-    }
-
-    for (i=0; i<MAX_PLOT_LABELS; i++) {
-	plot_label_init(&(spec->labels[i]));
-    }
-
-    spec->xtics[0] = 0;
-    spec->mxtics[0] = 0;
-    spec->fname[0] = 0;
-    strcpy(spec->keyspec, "left top");
-    spec->xzeroaxis = 0;
-
-    for (i=0; i<3; i++) {
-	spec->range[i][0] = NADBL;
-	spec->range[i][1] = NADBL;
-    }
-
-    spec->code = PLOT_REGULAR;
-    spec->flags = 0;
-    spec->fp = NULL;
-    spec->data = NULL;
-    spec->markers = NULL;
-    spec->n_markers = 0;
-    spec->labeled = NULL;
-    spec->ptr = NULL;
-    spec->reglist = NULL;
-    spec->n_lines = 0;
-    spec->nobs = 0;
-
-    spec->termtype[0] = 0;
-
-    return spec;
-}
+#define no_readback(p) (p == PLOT_CORRELOGRAM || \
+                        p == PLOT_LEVERAGE || \
+                        p == PLOT_MULTI_IRF || \
+                        p == PLOT_MULTI_SCATTER || \
+                        p == PLOT_PANEL || \
+                        p == PLOT_TRI_GRAPH || \
+                        p == PLOT_BI_GRAPH)
 
 static int get_gpt_data (GPT_SPEC *spec, int do_markers, FILE *fp)
 {
@@ -641,9 +616,12 @@ static int get_gpt_data (GPT_SPEC *spec, int do_markers, FILE *fp)
     int i, j, t;
     int err = 0;
 
+    spec->okobs = spec->nobs;
+
     gretl_push_c_numeric_locale();
 
     for (i=0; i<spec->n_lines && !err; i++) {
+	int okobs = spec->nobs;
 	int offset = 1;
 
 	if (spec->lines[i].ncols == 0) {
@@ -661,6 +639,7 @@ static int get_gpt_data (GPT_SPEC *spec, int do_markers, FILE *fp)
 	x[3] = x[2] + spec->nobs;	
 
 	for (t=0; t<spec->nobs; t++) {
+	    int missing = 0;
 	    int nf = 0;
 
 	    got = fgets(s, sizeof s, fp);
@@ -685,16 +664,25 @@ static int get_gpt_data (GPT_SPEC *spec, int do_markers, FILE *fp)
 	    for (j=offset; j<nf; j++) {
 		if (test[j][0] == '?') {
 		    x[j][t] = NADBL;
+		    missing++;
 		} else {
 		    x[j][t] = atof(test[j]);
 		}
 	    }
 
+	    if (missing) {
+		okobs--;
+	    }
+
 	    if (i == 0 && do_markers) {
 		get_gpt_marker(s, spec->markers[t]);
 	    }
 	}
 
+	if (okobs < spec->okobs) {
+	    spec->okobs = okobs;
+	} 
+
 	/* trailer line for data block */
 	fgets(s, sizeof s, fp);
 
@@ -723,7 +711,7 @@ static int parse_label_line (GPT_SPEC *spec, const char *line, int i)
 	return 1;
     }
 
-    plot_label_init(&(spec->labels[i]));
+    plotspec_label_init(&(spec->labels[i]));
 
     /* find first single or double quote */
     p = strchr(line, '\'');
@@ -830,12 +818,40 @@ read_plotspec_range (const char *obj, const char *s, GPT_SPEC *spec)
     return err;
 }
 
+static void catch_value (char *targ, const char *src, int maxlen)
+{
+    int i, n;
+
+    src += strspn(src, " \t\r\n");
+
+    if (*src == '\'' || *src == '"') {
+	src++;
+    }
+
+    *targ = '\0';
+    strncat(targ, src, maxlen - 1);
+    n = strlen(targ);
+    
+    for (i=n-1; i>=0; i--) {
+	if (isspace((unsigned char) targ[i])) {
+	    targ[i] = '\0';
+	} else {
+	    break;
+	}
+    }    
+
+    if (targ[i] == '\'' || targ[i] == '"') {
+	targ[i] = '\0';
+    }
+}
+
 static int parse_gp_set_line (GPT_SPEC *spec, const char *s, int *labelno)
 {
     char variable[16] = {0};
     char value[MAXLEN] = {0};
 
     if (strstr(s, "encoding") != NULL) {
+	fprintf(stderr, "Got encoding: '%s'\n", s);
 	return 0;
     }
 
@@ -846,15 +862,14 @@ static int parse_gp_set_line (GPT_SPEC *spec, const char *s, int *labelno)
     }
 
     if (strcmp(variable, "y2tics") == 0) {
-	spec->flags |= GPTSPEC_Y2AXIS;
+	spec->flags |= GPT_Y2AXIS;
 	return 0;
     } else if (strcmp(variable, "border 3") == 0) {
-	spec->flags |= GPTSPEC_MINIMAL_BORDER;
+	spec->flags |= GPT_MINIMAL_BORDER;
 	return 0;
     }    
 
-    strcpy(value, s + 4 + strlen(variable));
-    top_n_tail(value);
+    catch_value(value, s + 4 + strlen(variable), MAXLEN);
 
     if (strstr(variable, "range")) {
 	if (read_plotspec_range(variable, value, spec)) {
@@ -866,8 +881,12 @@ static int parse_gp_set_line (GPT_SPEC *spec, const char *s, int *labelno)
 	strcpy(spec->titles[0], value);
     } else if (!strcmp(variable, "xlabel")) {
 	strcpy(spec->titles[1], value);
+	*spec->xvarname = '\0';
+	strncat(spec->xvarname, value, MAXDISP-1);
     } else if (!strcmp(variable, "ylabel")) {
 	strcpy(spec->titles[2], value);
+	*spec->yvarname = '\0';
+	strncat(spec->yvarname, value, MAXDISP-1);
     } else if (!strcmp(variable, "y2label")) {
 	strcpy(spec->titles[3], value);
     } else if (!strcmp(variable, "key")) {
@@ -880,6 +899,8 @@ static int parse_gp_set_line (GPT_SPEC *spec, const char *s, int *labelno)
 	safecpy(spec->mxtics, value, 3);
     } else if (!strcmp(variable, "xzeroaxis")) {
 	spec->xzeroaxis = 1;
+    } else if (!strcmp(variable, "boxwidth")) {
+	spec->boxwidth = (float) atof(value);
     } else if (!strcmp(variable, "label")) {
 	parse_label_line(spec, s, *labelno);
 	*labelno += 1;
@@ -963,6 +984,79 @@ static int get_plot_nobs (FILE *fp, PlotType *ptype, int *do_markers)
     return n;
 }
 
+static int grab_fit_coeffs (GPT_SPEC *spec, const char *s)
+{
+    int n, err = 0;
+
+    if (spec->fit == PLOT_FIT_OLS) {
+	spec->b_ols = gretl_column_vector_alloc(2);
+	if (spec->b_ols == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    gretl_push_c_numeric_locale();
+	    n = sscanf(s, "%lf + %lf", &spec->b_ols->val[0],
+		       &spec->b_ols->val[1]);
+	    gretl_pop_c_numeric_locale();
+	    if (n != 2) {
+		gretl_matrix_free(spec->b_ols);
+		spec->b_ols = NULL;
+		err = E_DATA;
+	    }
+	}
+    } else if (spec->fit == PLOT_FIT_INVERSE) {
+	spec->b_inv = gretl_column_vector_alloc(2);
+	if (spec->b_inv == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    gretl_push_c_numeric_locale();
+	    n = sscanf(s, "%lf + %lf", &spec->b_inv->val[0],
+		       &spec->b_inv->val[1]);
+	    gretl_pop_c_numeric_locale();
+	    if (n != 2) {
+		gretl_matrix_free(spec->b_inv);
+		spec->b_inv = NULL;
+		err = E_DATA;
+	    }
+	}
+    } else if (spec->fit == PLOT_FIT_QUADRATIC) {
+	spec->b_quad = gretl_column_vector_alloc(3);
+	if (spec->b_quad == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    gretl_push_c_numeric_locale();
+	    n = sscanf(s, "%lf + %lf*X + %lf", &spec->b_quad->val[0],
+		       &spec->b_quad->val[1], &spec->b_quad->val[2]);
+	    gretl_pop_c_numeric_locale();
+	    if (n != 3) {
+		gretl_matrix_free(spec->b_quad);
+		spec->b_quad = NULL;
+		err = E_DATA;
+	    }
+	}
+    }	
+
+    if (err) {
+	spec->flags &= ~GPT_AUTO_FIT;
+    }
+
+    return err;
+}
+
+/* scan the stuff after "title '" or 'title "' */
+
+static void grab_line_title (char *targ, const char *src)
+{
+    char *fmt;
+
+    if (*src == '\'') {
+	fmt = "%79[^']'";
+    } else {
+	fmt = "%79[^\"]\"";
+    }
+
+    sscanf(src + 1, fmt, targ);
+}
+
 /* parse the "using..." portion of plot specification for a
    given plot line: full form is like:
   
@@ -1001,6 +1095,9 @@ static int parse_gp_line_line (const char *s, GPT_SPEC *spec, int i)
 	}
 	if (p != NULL) {
 	    strncat(spec->lines[i].formula, s, p - s);
+	    if (i == 1 && spec->flags & GPT_AUTO_FIT) {
+		grab_fit_coeffs(spec, spec->lines[i].formula);
+	    }
 	}
     }
 
@@ -1009,7 +1106,7 @@ static int parse_gp_line_line (const char *s, GPT_SPEC *spec, int i)
     } 
 
     if ((p = strstr(s, " title "))) {
-	sscanf(p + 8, "%79[^']'", spec->lines[i].title);
+	grab_line_title(spec->lines[i].title, p + 7);
     }
 
     if ((p = strstr(s, " w "))) {
@@ -1047,17 +1144,26 @@ static void maybe_set_all_markers_ok (GPT_SPEC *spec)
 {
     if (spec->n_lines <= 2 &&
 	spec->lines[0].ncols == 2 &&
-	spec->lines[1].ncols == 0 &&
+	(spec->n_lines == 1 || spec->lines[1].ncols == 0) &&
 	spec->n_markers > 0) {
-	spec->flags |= GPTSPEC_ALL_MARKERS_OK;
-#if GPDEBUG > 1
-	fprintf(stderr, "set GPTSPEC_ALL_MARKERS_OK\n");
-#endif
+	spec->flags |= GPT_ALL_MARKERS_OK;
     } else {
-	spec->flags &= ~GPTSPEC_ALL_MARKERS_OK;
-#if GPDEBUG > 1
-	fprintf(stderr, "unset GPTSPEC_ALL_MARKERS_OK\n");
-#endif
+	spec->flags &= ~GPT_ALL_MARKERS_OK;
+    }
+}
+
+static void maybe_set_add_fit_ok (GPT_SPEC *spec)
+{
+    if (spec->n_lines == 2 && spec->fit != PLOT_FIT_NONE) {
+	; /* already OK */
+    } else if (spec->data != NULL &&
+	spec->code == PLOT_REGULAR &&
+	spec->n_lines == 1 &&
+	spec->lines[0].ncols == 2 &&
+	!(spec->flags & (GPT_IMPULSES|GPT_LINES|GPT_RESIDS|GPT_TS))) {
+	spec->fit = PLOT_FIT_NONE;
+    } else {
+	spec->fit = PLOT_FIT_NA;
     }
 }
 
@@ -1139,6 +1245,23 @@ static int uneditable_get_markers (GPT_SPEC *spec, FILE *fp, int *polar)
     return err;
 }
 
+static FitType recognize_fit_string (const char *s)
+{
+    if (strstr(s, "OLS")) {
+	return PLOT_FIT_OLS;
+    } else if (strstr(s, "quadratic")) {
+	return PLOT_FIT_QUADRATIC;
+    } else if (strstr(s, "inverse")) {
+	return PLOT_FIT_INVERSE;
+    } else if (strstr(s, "loess")) {
+	return PLOT_FIT_LOESS;
+    } else {
+	return PLOT_FIT_NONE;
+    }
+}
+
+#define plot_needs_obs(c) (c != PLOT_ELLIPSE && c != PLOT_PROB_DIST)
+
 /* Read plotspec struct from gnuplot command file.  This is _not_ a
    general parser for gnuplot files; it is designed specifically for
    files auto-generated by gretl.
@@ -1177,7 +1300,7 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 
     /* get the number of data-points, plot type, and check for markers */
     spec->nobs = get_plot_nobs(fp, &spec->code, &do_markers);
-    if (spec->nobs == 0 && spec->code != PLOT_ELLIPSE) {
+    if (spec->nobs == 0 && plot_needs_obs(spec->code)) {
 	/* failed reading plot data */
 #if GPDEBUG
 	fprintf(stderr, " got spec->nobs = 0\n");
@@ -1204,8 +1327,8 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
     /* get the preamble and "set" lines */
     labelno = 0;
     while ((got = fgets(gpline, sizeof gpline, fp))) {
-	int v, litlines = 0;
-	char vname[9];
+	char vname[VNAMELEN];
+	int v;
 
 	if (!strncmp(gpline, "# timeseries", 12)) {
 	    int pd;
@@ -1213,16 +1336,19 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 	    if (sscanf(gpline, "# timeseries %d", &pd)) {
 		*plot_pd = pd;
 	    }
-	    spec->flags |= GPTSPEC_TS;
+	    spec->flags |= GPT_TS;
+	    if (strstr(gpline, "letterbox")) {
+		spec->flags |= GPT_LETTERBOX;
+	    }
 	    continue;
 	}
 
-	if (sscanf(gpline, "# X = '%8[^\']' (%d)", vname, &v) == 2) {
+	if (sscanf(gpline, "# X = '%15[^\']' (%d)", vname, &v) == 2) {
 	    if (plot_ols_var_ok(vname, v)) {
 		reglist[2] = v;
 	    }
 	    continue;
-	} else if (sscanf(gpline, "# Y = '%8[^\']' (%d)", vname, &v) == 2) {
+	} else if (sscanf(gpline, "# Y = '%15[^\']' (%d)", vname, &v) == 2) {
 	    if (reglist[2] > 0 && plot_ols_var_ok(vname, v)) {
 		reglist[0] = 3;
 		reglist[1] = v;
@@ -1230,8 +1356,13 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 	    continue;
 	}
 	
-	if (sscanf(gpline, "# literal lines = %d", &litlines)) {
-	    for (i=0; i<litlines; i++) {
+	if (sscanf(gpline, "# literal lines = %d", &spec->n_literal)) {
+	    spec->literal = strings_array_new(spec->n_literal);
+	    if (spec->literal == NULL) {
+		err = E_ALLOC;
+		goto plot_bailout;
+	    }
+	    for (i=0; i<spec->n_literal; i++) {
 		if (!fgets(gpline, MAXLEN - 1, fp)) {
 		    errbox(_("Plot file is corrupted"));
 		} else {
@@ -1242,13 +1373,14 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 	    continue;
 	}
 
-	if (strstr(gpline, "automatic OLS")) {
-	    spec->flags |= GPTSPEC_AUTO_OLS;
+	if (strstr(gpline, "automatic fit")) {
+	    spec->flags |= GPT_AUTO_FIT;
+	    spec->fit = recognize_fit_string(gpline);
 	    continue;
 	}
 
 	if (strstr(gpline, "printing data labels")) {
-	    spec->flags |= GPTSPEC_ALL_MARKERS;
+	    spec->flags |= GPT_ALL_MARKERS;
 	    continue;
 	}	
 
@@ -1275,7 +1407,7 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 
     for (i=0; i<4; i++) {
 	if (spec->titles[i][0] != '\0') {
-	    delchar('\'', spec->titles[i]);
+	    delchar('"', spec->titles[i]);
 	}
     }
 
@@ -1343,7 +1475,7 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 
     err = plot_get_data_and_markers(spec, fp, datacols, do_markers);
 
-    if (!err && reglist[0] > 0) {
+    if (!err && reglist != NULL && reglist[0] > 0) {
 	spec->reglist = gretl_list_copy(reglist);
     }
 
@@ -1351,6 +1483,10 @@ static int read_plotspec_from_file (GPT_SPEC *spec, int *plot_pd, int *polar)
 	maybe_set_all_markers_ok(spec);
     }
 
+    if (!err && spec->fit == PLOT_FIT_NONE) {
+	maybe_set_add_fit_ok(spec);
+    }
+
  plot_bailout:
 
     fclose(fp);
@@ -1432,7 +1568,9 @@ static void x_to_date (double x, int pd, char *str)
     int subper = (int) ((x - yr + frac) * pd);
     static int decpoint;
 
-    if (decpoint == 0) decpoint = get_local_decpoint();
+    if (decpoint == 0) {
+	decpoint = get_local_decpoint();
+    }
 
     t = yr + subper / ((pd < 10)? 10.0 : 100.0);
     sprintf(str, "%.*f", (pd < 10)? 1 : 2, t);
@@ -1653,6 +1791,8 @@ identify_point (png_plot *plot, int pixel_x, int pixel_y,
     return TRUE;
 }
 
+#define float_fmt(i,x) ((i) && fabs(x) < 1.0e7)
+
 static gint
 motion_notify_event (GtkWidget *widget, GdkEventMotion *event, png_plot *plot)
 {
@@ -1679,7 +1819,9 @@ motion_notify_event (GtkWidget *widget, GdkEventMotion *event, png_plot *plot)
 	double data_x, data_y;
 
 	get_data_xy(plot, x, y, &data_x, &data_y);
-	if (na(data_x)) return TRUE;
+	if (na(data_x)) {
+	    return TRUE;
+	}
 
 	if (!plot_has_no_markers(plot) && !plot_show_all_markers(plot) &&
 	    !plot_is_zooming(plot) &&
@@ -1690,18 +1832,21 @@ motion_notify_event (GtkWidget *widget, GdkEventMotion *event, png_plot *plot)
 	if (plot->pd == 4 || plot->pd == 12) {
 	    x_to_date(data_x, plot->pd, label);
 	} else {
-	    sprintf(label, (plot->xint)? "%7.0f" : "%7.4g", data_x);
+	    sprintf(label, (float_fmt(plot->xint, data_x))? "%7.0f" : 
+		    "%7.4g", data_x);
 	}
 
 	if (!na(data_y)) {
 	    if (plot_has_png_coords(plot)) {
-		sprintf(label_y, (plot->yint)? " %-7.0f" : " %-7.4g", data_y);
+		sprintf(label_y, (float_fmt(plot->yint, data_y))? " %-7.0f" : 
+			" %-7.4g", data_y);
 	    } else {
 		/* pretty much guessing at y coordinate here */
-		sprintf(label_y, (plot->yint)? " %-7.0f" : " %-6.3g", data_y);
+		sprintf(label_y, (float_fmt(plot->yint, data_y))? " %-7.0f" : 
+				  " %-6.3g", data_y);
+			}
+		strcat(label, label_y);
 	    }
-	    strcat(label, label_y);
-	}
 
 	if (plot_is_zooming(plot) && (state & GDK_BUTTON1_MASK)) {
 	    draw_selection_rectangle(plot, x, y);
@@ -1729,7 +1874,7 @@ static void set_plot_format_flags (png_plot *plot)
     if (!string_is_blank(plot->spec->titles[3])) {
 	plot->format |= PLOT_Y2LABEL;
     }
-    if (plot->spec->flags & GPTSPEC_Y2AXIS) {
+    if (plot->spec->flags & GPT_Y2AXIS) {
 	plot->format |= PLOT_Y2AXIS;
     }
 }
@@ -1793,7 +1938,7 @@ static gint color_popup_activated (GtkWidget *w, gpointer data)
 	if (color) {
 	    strcat(plot->spec->termtype, " color");
 	} 
-	file_selector(_("Save gnuplot graph"), SAVE_THIS_GRAPH, 
+	file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
 		      FSEL_DATA_MISC, plot->spec);
     } else if (!strcmp(parent_item, _("Save as Windows metafile (EMF)..."))) {
 	strcpy(plot->spec->termtype, "emf");
@@ -1802,7 +1947,7 @@ static gint color_popup_activated (GtkWidget *w, gpointer data)
 	} else {
 	    strcat(plot->spec->termtype, " mono");
 	}
-	file_selector(_("Save gnuplot graph"), SAVE_THIS_GRAPH, 
+	file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
 		      FSEL_DATA_MISC, plot->spec);
     } 
 #ifdef G_OS_WIN32
@@ -1871,6 +2016,19 @@ static void show_numbers_from_markers (GPT_SPEC *spec)
     }
 }
 
+static void add_to_session_callback (GPT_SPEC *spec)
+{
+    char fullname[MAXLEN] = {0};
+    int err;
+
+    err = add_graph_to_session(spec->fname, fullname);
+
+    if (!err) {
+	remove_png_term_from_plotfile(fullname, spec);
+	mark_plot_as_saved(spec);
+    }
+}
+
 static gint plot_popup_activated (GtkWidget *w, gpointer data)
 {
     gchar *item = (gchar *) data;
@@ -1881,22 +2039,24 @@ static gint plot_popup_activated (GtkWidget *w, gpointer data)
     gtk_widget_destroy(plot->popup);
     plot->popup = NULL;
 
-    if (!strcmp(item, _("Save as PNG..."))) {
+    if (!strcmp(item, _("Add another curve..."))) {
+	stats_calculator(plot, CALC_GRAPH_ADD, NULL);
+    } else if (!strcmp(item, _("Save as PNG..."))) {
 	strcpy(plot->spec->termtype, "png");
-        file_selector(_("Save gnuplot graph"), SAVE_THIS_GRAPH, 
+        file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
 		      FSEL_DATA_MISC, plot->spec);
     } else if (!strcmp(item, _("Save as PDF..."))) {
 	strcpy(plot->spec->termtype, "PDF");
-        file_selector(_("Save gnuplot graph"), SAVE_THIS_GRAPH, 
+        file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
 		      FSEL_DATA_MISC, plot->spec);
     } else if (!strcmp(item, _("Save to session as icon"))) { 
-	add_graph_to_session(plot->spec, GRETL_OBJ_GRAPH, NULL);
+	add_to_session_callback(plot->spec);
     } else if (plot_is_range_mean(plot) && !strcmp(item, _("Help"))) { 
 	context_help(NULL, GINT_TO_POINTER(RMPLOT));
     } else if (plot_is_hurst(plot) && !strcmp(item, _("Help"))) { 
 	context_help(NULL, GINT_TO_POINTER(HURST));
     } else if (!strcmp(item, _("Freeze data labels"))) {
-	plot->spec->flags |= GPTSPEC_ALL_MARKERS;
+	plot->spec->flags |= GPT_ALL_MARKERS;
 	redisplay_edited_png(plot);
     } else if (!strcmp(item, _("Clear data labels"))) { 
 	zoom_unzoom_png(plot, PNG_REDISPLAY);
@@ -1919,7 +2079,9 @@ static gint plot_popup_activated (GtkWidget *w, gpointer data)
     }
 #endif 
     else if (!strcmp(item, _("OLS estimates"))) { 
-	do_graph_model(plot->spec);
+	if (plot->spec != NULL) {
+	    do_graph_model(plot->spec->reglist, plot->spec->fit);
+	}
     } else if (!strcmp(item, _("Numerical values"))) {
 	show_numbers_from_markers(plot->spec);
     } else if (!strcmp(item, _("Edit"))) { 
@@ -1959,10 +2121,15 @@ static void attach_color_popup (GtkWidget *w, png_plot *plot)
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(w), cpopup);
 }
 
+#define graph_model_ok(f) (f == PLOT_FIT_OLS || \
+                           f == PLOT_FIT_QUADRATIC || \
+                           f == PLOT_FIT_INVERSE)
+
 static void build_plot_menu (png_plot *plot)
 {
     GtkWidget *item;    
     const char *regular_items[] = {
+	N_("Add another curve..."),
 #ifdef G_OS_WIN32
 	N_("Save as Windows metafile (EMF)..."),
 #endif
@@ -2015,6 +2182,11 @@ static void build_plot_menu (png_plot *plot)
 
     i = 0;
     while (plot_items[i]) {
+	if (plot->spec->code != PLOT_PROB_DIST &&
+	    !strcmp(plot_items[i], "Add another curve...")) {
+	    i++;
+	    continue;
+	}
 	if (plot_not_zoomable(plot) &&
 	    !strcmp(plot_items[i], "Zoom...")) {
 	    i++;
@@ -2045,7 +2217,8 @@ static void build_plot_menu (png_plot *plot)
 	    i++;
 	    continue;
 	}
-	if (!plot_has_regression_list(plot) &&
+	if ((!plot_has_regression_list(plot) || 
+	     !graph_model_ok(plot->spec->fit)) && 
 	    !strcmp(plot_items[i], "OLS estimates")) {
 	    i++;
 	    continue;
@@ -2099,7 +2272,7 @@ int redisplay_edited_png (png_plot *plot)
 
     /* dump the edited plot details to file */
     set_png_output(plot->spec);
-    print_plotspec_details(plot->spec, fp);
+    plotspec_print(plot->spec, fp);
     fclose(fp);
 
     /* get gnuplot to create a new PNG graph */
@@ -2280,7 +2453,7 @@ plot_key_handler (GtkWidget *w, GdkEventKey *key, png_plot *plot)
 	break;
     case GDK_s:
     case GDK_S:
-	add_graph_to_session(plot->spec, GRETL_OBJ_GRAPH, NULL);
+	add_to_session_callback(plot->spec);
 	break;
 #ifdef G_OS_WIN32
     case GDK_c:
@@ -2395,7 +2568,7 @@ static void destroy_png_plot (GtkWidget *w, png_plot *plot)
     } else {
 	/* no controller: take responsibility for freeing the
 	   plot specification */
-	free_plotspec(plot->spec);
+	plotspec_destroy(plot->spec);
     }
 
     if (plot->invert_gc != NULL) {
@@ -2621,6 +2794,11 @@ static int get_plot_ranges (png_plot *plot)
     plot->xint = plot->yint = 0;
     plot->pd = 0;
 
+    if (no_readback(plot->spec->code)) {
+	plot->status |= (PLOT_DONT_ZOOM | PLOT_DONT_MOUSE);
+	return 1;
+    }
+
     fp = gretl_fopen(plot->spec->fname, "r");
     if (fp == NULL) {
 	plot->status |= (PLOT_DONT_ZOOM | PLOT_DONT_MOUSE);
@@ -2658,15 +2836,17 @@ static int get_plot_ranges (png_plot *plot)
 		" xmin=%d xmax=%d ymin=%d ymax=%d\n", 
 		plot->pixel_xmin, plot->pixel_xmax,
 		plot->pixel_ymin, plot->pixel_ymax);
+	fprintf(stderr, "using px_height %d, px_width %d\n",
+		plot->pixel_height, plot->pixel_width);
 # endif
     } else {
 	fprintf(stderr, "get_png_bounds_info(): failed\n");
     }
 #endif /* PNG_COMMENTS */
 
-    /* If got_x = 0 at this point, we didn't an x-range out of
-       the gnuplot source file OR the PNG file, so we might as
-       well give up.
+    /* If got_x = 0 at this point, we didn't get an x-range out of the
+       gnuplot source file OR the PNG file, so we might as well give
+       up.
     */
 
     if (got_x) {
@@ -2741,7 +2921,8 @@ static png_plot *png_plot_new (void)
     return plot;
 }
 
-int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
+static png_plot *
+gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
 {
     GtkWidget *vbox;
     GtkWidget *canvas_hbox;
@@ -2758,7 +2939,7 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
 
     plot = png_plot_new();
     if (plot == NULL) {
-	return 1;
+	return NULL;
     }
 
     if (spec != NULL) {
@@ -2767,7 +2948,7 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
 	plot->spec = plotspec_new();
 	if (plot->spec == NULL) {
 	    free(plot);
-	    return 1;
+	    return NULL;
 	}
 	strcpy(plot->spec->fname, plotfile);
     }
@@ -2791,7 +2972,6 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
     } else if (cant_edit(plot->spec->code)) {
 	if (plot->spec->n_markers > 0) {
 	    plot->status |= (PLOT_DONT_EDIT | PLOT_DONT_ZOOM);
-	    get_plot_ranges(plot);
 	    if (polar) {
 		plot->format |= PLOT_POLAR;
 	    }
@@ -2800,13 +2980,22 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
 	}
     } else {
 	set_plot_format_flags(plot);
-	get_plot_ranges(plot);
     } 
 
     if (plot->spec->code == PLOT_VAR_ROOTS) {
 	plot->pixel_width = plot->pixel_height;
     }
 
+    if (plot->spec->flags & GPT_LETTERBOX) {
+	plot->pixel_width = 680;
+	plot->pixel_height = 400;
+    }
+
+    if (!err) {
+	get_plot_ranges(plot);
+    }
+	
+
     plot->shell = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
     /* note need for corresponding unref */
@@ -2828,7 +3017,7 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
     gtk_box_pack_start(GTK_BOX(vbox), canvas_hbox, TRUE, TRUE, 0);
     gtk_widget_show(canvas_hbox);
 
-    /*  eventbox and hbox for status area  */
+    /* eventbox and hbox for status area  */
     plot->statusarea = gtk_event_box_new();
     gtk_box_pack_start(GTK_BOX(vbox), plot->statusarea, FALSE, FALSE, 0);
 
@@ -2930,7 +3119,39 @@ int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
 
     render_pngfile(plot, PNG_START);
 
-    return 0;
+    return plot;
+}
+
+void gnuplot_show_png_by_name (const char *fname)
+{
+    gnuplot_show_png(fname, NULL, 0);
+}
+
+void display_session_graph_png (const char *fname) 
+{
+    char fullname[MAXLEN];
+    gchar *plotcmd;
+    int err = 0;
+
+    if (g_path_is_absolute(fname)) {
+	strcpy(fullname, fname);
+    } else {
+	sprintf(fullname, "%s%s", paths.userdir, fname);
+    }
+
+    if (add_png_term_to_plotfile(fullname)) {
+	return;
+    }
+
+    plotcmd = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, fullname);
+    err = gretl_spawn(plotcmd);
+    g_free(plotcmd);
+
+    if (err) {
+	errbox(_("Gnuplot error creating graph"));
+    } else {
+	gnuplot_show_png(fullname, NULL, 1);
+    }
 }
 
 /* apparatus for getting coordinate info out of PNG files created using
@@ -3075,3 +3296,158 @@ static int get_png_bounds_info (png_bounds *bounds)
 }
 
 #endif /* PNG_COMMENTS */
+
+/* for graphs in the stats calculator */
+
+static int get_dist_and_df (const char *s, int *d, int *m, int *n)
+{
+    int ret = 1;
+
+    if (!strncmp(s, "# standard", 10)) {
+	*d = NORMAL_DIST;
+    } else if (sscanf(s, "# t(%d)", m) == 1) {
+	*d = T_DIST;
+    } else if (sscanf(s, "# chi-square(%d)", m) == 1) {
+	*d = CHISQ_DIST;
+    } else if (sscanf(s, "# F(%d,%d)", m, n) == 2) {
+	*d = F_DIST;
+    } else {
+	ret = 0;
+    }
+
+    return ret;
+}
+
+void revise_distribution_plotspec (png_plot *plot, int d, int df1, int df2)
+{
+    GPT_SPEC *spec = plot->spec;
+    gchar *title = NULL;
+    const char *s;
+    char dfstr[16];
+    int got[5] = {0};
+    int bigchi = 0;
+    int m, n, prevd, dfmax = 0;
+    int i, err = 0;
+
+    for (i=0; i<spec->n_literal; i++) {
+	int dfid;
+
+	s = spec->literal[i];
+	m = n = 0;
+	prevd = -1;
+	if (*s == '#') {
+	    if (get_dist_and_df(s, &prevd, &m, &n)) {
+		if (prevd == d && m == df1 && n == df2) {
+		    /* line is already present */
+		    return;
+		}
+		if (prevd == CHISQ_DIST && m > 69) {
+		    got[4] += 1;
+		} else {
+		    got[prevd] += 1;
+		}
+	    }
+	} else if (sscanf(s, "df%d=", &dfid) == 1) {
+	    if (dfid > dfmax) {
+		dfmax = dfid;
+	    }
+	}
+    }
+
+    /* adjust x-range if needed */
+    if (d == CHISQ_DIST || d == F_DIST) {
+	double x = dist_xmax(d, df1, df2);
+
+	if (x > spec->range[0][1]) {
+	    spec->range[0][1] = x;
+	}
+    }
+
+    /* add comment for current plot */
+    title = dist_marker_line(d, df1, df2);
+    err = strings_array_add(&spec->literal, &spec->n_literal, title);
+    g_free(title);
+
+    if (!err) {
+	/* add df line(s) if needed */
+	m = dfmax + 1;
+	if (d == T_DIST || d == CHISQ_DIST || d == F_DIST) {
+	    sprintf(dfstr, "df%d=%.1f", m, (double) df1);
+	    err = strings_array_add(&spec->literal, &spec->n_literal, dfstr);
+	}
+	if (d == F_DIST && !err) {
+	    sprintf(dfstr, "df%d=%.1f", m + 1, (double) df2);
+	    err = strings_array_add(&spec->literal, &spec->n_literal, dfstr);
+	}
+    }
+
+    if (err) {
+	gui_errmsg(err);
+	return;
+    }  
+
+    if (d == CHISQ_DIST && df1 > 69) {
+	bigchi = 1;
+    }
+
+    /* add any required formula lines */
+
+    if (d == T_DIST) {
+	if (!got[T_DIST] && !got[F_DIST]) {
+	    err = strings_array_add(&spec->literal, &spec->n_literal, 
+				    dist_formula(F_BINV));
+	}
+    } else if (d == CHISQ_DIST) {
+	if (bigchi && !got[4]) {
+	    err = strings_array_add(&spec->literal, &spec->n_literal, 
+				    dist_formula(F_LOG2));
+	    if (!err) {
+		err = strings_array_add(&spec->literal, &spec->n_literal, 
+					dist_formula(F_BIGCHI));
+	    }
+	} else if (!bigchi && !got[CHISQ_DIST]) {
+	    err = strings_array_add(&spec->literal, &spec->n_literal, 
+				    dist_formula(F_CHI));
+	}
+    } else if (d == F_DIST) {
+	if (!got[F_DIST] && !got[T_DIST]) {
+	    err = strings_array_add(&spec->literal, &spec->n_literal, 
+				    dist_formula(F_BINV));
+	} 
+	if (!err && !got[F_DIST]) {
+	    err = strings_array_add(&spec->literal, &spec->n_literal, 
+				    dist_formula(F_F));
+	}
+    }
+
+    if (!err) {
+	/* add new plot line */
+	err = plotspec_add_line(spec);
+    }
+
+    if (err) {
+	gui_errmsg(err);
+	return;
+    }
+
+    n = spec->n_lines - 1;
+
+    strcpy(spec->lines[n].scale, "NA");
+    strcpy(spec->lines[n].style, "lines");
+    title = dist_graph_title(d, NADBL, df1, df2);
+    strcpy(spec->lines[n].title, title);
+    g_free(title);
+
+    if (d == NORMAL_DIST) {
+	strcpy(spec->lines[n].formula, "(1/(sqrt(2*pi))*exp(-(x)**2/2))");
+    } else if (d == T_DIST) {
+	sprintf(spec->lines[n].formula, "Binv(0.5*df%d,0.5)/sqrt(df%d)*(1.0+(x*x)/df%d)"
+		"**(-0.5*(df%d+1.0))", m, m, m, m);
+    } else if (d == CHISQ_DIST) {
+	sprintf(spec->lines[n].formula, "%s(x,df%d)", (bigchi)? "bigchi" : "chi", m);
+    } else if (d == F_DIST) {
+	sprintf(spec->lines[n].formula, "f(x,df%d,df%d)", m, m + 1);
+    }
+
+    redisplay_edited_png(plot);
+}
