@@ -20,6 +20,7 @@
 /* gpt_dialog.c for gretl -- GUI gnuplot controller dialog */
 
 #include "gretl.h"
+#include "plotspec.h"
 #include "gpt_control.h"
 #include "session.h"
 #include "dlgutils.h"
@@ -52,8 +53,8 @@ GtkWidget *labelpos[MAX_PLOT_LABELS];
 
 static GtkWidget *gpt_control;
 static GtkWidget *keycombo;
+static GtkWidget *fitcombo;
 static GtkWidget *termcombo;
-static GtkWidget *fitline_check;
 static GtkWidget *border_check;
 static GtkWidget *markers_check;
 static GtkWidget *y2_check;
@@ -76,10 +77,14 @@ struct gpt_titles_t gpt_titles[] = {
     { N_("Title for axis"), 3, NULL },
 };
 
-#define frequency_plot(s)       (s->code == PLOT_FREQ_SIMPLE || \
-			         s->code == PLOT_FREQ_NORMAL || \
-			         s->code == PLOT_FREQ_GAMMA || \
-                                 s->code == PLOT_KERNEL)
+const gchar *fittype_strings[] = {
+    N_("none"),
+    N_("linear: y = a + b*x"),
+    N_("quadratic: y = a + b*x + c*x^2"),
+    N_("inverse: y = a + b*(1/x)"),
+    N_("loess (locally weighted fit)"),
+    NULL
+};
 
 static const char *get_font_filename (const char *showname);
 
@@ -99,7 +104,7 @@ static void close_plot_controller (GtkWidget *widget, gpointer data)
 #endif
 	plot_remove_controller(plot);
     } else {
-	free_plotspec(spec); 
+	plotspec_destroy(spec); 
     }
 } 
 
@@ -131,15 +136,90 @@ static void entry_to_gp_string (GtkWidget *w, char *targ, size_t n)
     wstr = gtk_entry_get_text(GTK_ENTRY(w));
 
     if (wstr != NULL && *wstr != '\0') {
-	gchar *trstr = force_locale_from_utf8(wstr);
+	gchar *trstr = gp_locale_from_utf8(wstr);
 
 	if (trstr != NULL) {
 	    strncat(targ, trstr, n-1);
 	    g_free(trstr);
+	} else {
+	    strncat(targ, wstr, n-1);
 	}
     }
 }
 
+static FitType fit_type_from_string (const char *s)
+{
+    FitType f = PLOT_FIT_NONE;
+    int i;
+
+    if (s != NULL && *s != '\0') {
+	for (i=0; fittype_strings[i] != NULL; i++) {
+	    if (!strcmp(s, _(fittype_strings[i]))) {
+		f = i;
+		break;
+	    }
+	}
+    }
+
+    return f;
+}
+
+static void fittype_from_combo (GtkWidget *w, GPT_SPEC *spec)
+{
+    FitType f = PLOT_FIT_NONE;
+    const char *s;
+
+    g_return_if_fail(GTK_IS_ENTRY(w));
+    s = gtk_entry_get_text(GTK_ENTRY(w));
+    f = fit_type_from_string(s);
+
+    if (f == PLOT_FIT_OLS || f == PLOT_FIT_QUADRATIC || 
+	f == PLOT_FIT_INVERSE || f == PLOT_FIT_LOESS) {
+	plotspec_add_fit(spec, f);
+	spec->flags &= ~GPT_FIT_HIDDEN;
+    } else if (f == PLOT_FIT_NONE) {
+	if (spec->n_lines == 2) {
+	    spec->flags |= GPT_FIT_HIDDEN;
+	}
+	spec->fit = f;
+    }
+}
+
+static gboolean fit_type_changed (GtkEditable *entry, GPT_SPEC *spec)
+{
+    const gchar *s = gtk_entry_get_text(GTK_ENTRY(entry));
+    const char *s1 = spec->yvarname;
+    const char *s2 = spec->xvarname;
+    char title[128];
+    FitType f;
+
+    if (*s1 == '\0' || *s2 == '\0') {
+	return FALSE;
+    }
+
+    f = fit_type_from_string(s);
+
+    *title = '\0';
+
+    if (f == PLOT_FIT_OLS) {
+	sprintf(title, _("%s versus %s (with least squares fit)"),
+		s1, s2);
+    } else if (f == PLOT_FIT_QUADRATIC) {
+	sprintf(title, _("%s versus %s (with quadratic fit)"),
+		s1, s2);
+    } else if (f == PLOT_FIT_INVERSE) {
+	sprintf(title, _("%s versus %s (with inverse fit)"),
+		s1, s2);
+    } else if (f == PLOT_FIT_LOESS) {
+	sprintf(title, _("%s versus %s (with loess fit)"),
+		s1, s2);
+    }
+
+    gtk_entry_set_text(GTK_ENTRY(gpt_titles[0].widget), title);
+    
+    return FALSE;
+}
+
 /* take a double (which might be NA) and format it for
    a gtkentry widget */
 
@@ -177,7 +257,7 @@ static double entry_to_gp_double (GtkWidget *w)
 	    const gchar *tmp = s;
 #endif
 	    if (check_atof(tmp)) {
-		errbox(get_gretl_errmsg());
+		errbox(gretl_errmsg_get());
 	    } else {
 		ret = atof(tmp);
 	    }
@@ -192,25 +272,32 @@ static double entry_to_gp_double (GtkWidget *w)
 }
 
 /* Take text from a gnuplot spec string and put it into a gtkentry.
-   Under gtk2, we have to ensure that the text is put into utf-8.
+   We have to ensure that the text is put into utf-8.
 */
 
 static void gp_string_to_entry (GtkWidget *w, const char *str)
 {
 #ifdef ENABLE_NLS
-    int l2 = use_latin_2();
+    int lv = iso_latin_version();
 #else
-    int l2 = 0;
+    int lv = 0;
 #endif
-    gchar *trstr;
+    gchar *trstr = NULL;
 
-    if (l2) {
+    if (*str == '\0') {
+	gtk_entry_set_text(GTK_ENTRY(w), str);
+	return;
+    }
+
+    if (lv == 2) {
 	char lstr[MAXTITLE];
 	
 	sprint_html_to_l2(lstr, str);
-	trstr = my_locale_to_utf8(lstr);
+	trstr = latin2_to_utf8(lstr);
+    } else if (!g_utf8_validate(str, -1, NULL)) {
+	trstr = latin1_to_utf8(str);
     } else {
-	trstr = my_locale_to_utf8(str);
+	trstr = g_strdup(str);
     }
 
     if (trstr != NULL) {
@@ -261,7 +348,7 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 {
     const gchar *yaxis;
     int supress_y2 = 0;
-    int i, k, save = 0;
+    int i, k, fsave = 0;
     int err = 0;
 
     /* entry_to_gp_string translates from utf-8 to the locale, if
@@ -270,7 +357,9 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
     if (widget == filesavebutton) {
 	entry_to_gp_string(GTK_COMBO(termcombo)->entry, spec->termtype, 
 			   sizeof spec->termtype);
-	if (strcmp(spec->termtype, "screen")) save = 1;
+	if (strcmp(spec->termtype, "screen")) {
+	    fsave = 1;
+	}
     }
    
     for (i=0; i<NTITLES; i++) {
@@ -283,7 +372,7 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
     entry_to_gp_string(GTK_COMBO(keycombo)->entry, spec->keyspec, 
 		       sizeof spec->keyspec);
 
-    spec->flags &= ~GPTSPEC_Y2AXIS;
+    spec->flags &= ~GPT_Y2AXIS;
 
     if (y2_check != NULL) {
 	if (GTK_TOGGLE_BUTTON(y2_check)->active) {
@@ -302,12 +391,12 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 	    spec->lines[i].yaxis = 2;	
 	}
 	if (spec->lines[i].yaxis == 2) {
-	    spec->flags |= GPTSPEC_Y2AXIS;
+	    spec->flags |= GPT_Y2AXIS;
 	}
     }
 
     if (spec->code == PLOT_REGULAR) {
-	k = (spec->flags & GPTSPEC_Y2AXIS)? 3 : 2;
+	k = (spec->flags & GPT_Y2AXIS)? 3 : 2;
 	for (i=0; i<k; i++) {
 	    if (axis_range[i].isauto != NULL) {
 		if (GTK_TOGGLE_BUTTON(axis_range[i].isauto)->active) {
@@ -362,27 +451,19 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 
     if (!err && border_check != NULL) {
 	if (GTK_TOGGLE_BUTTON(border_check)->active) {
-	    spec->flags &= ~GPTSPEC_MINIMAL_BORDER;
+	    spec->flags &= ~GPT_MINIMAL_BORDER;
 	} else {
-	    spec->flags |= GPTSPEC_MINIMAL_BORDER;
+	    spec->flags |= GPT_MINIMAL_BORDER;
 	}
     } 
 
-    if (!err && fitline_check != NULL) {
-	if (GTK_TOGGLE_BUTTON(fitline_check)->active) {
-	    spec->flags |= GPTSPEC_OLS_HIDDEN;
-	} else {
-	    spec->flags &= ~GPTSPEC_OLS_HIDDEN;
-	}
-    }
-
     if (!err && markers_check != NULL) {
 	if (GTK_TOGGLE_BUTTON(markers_check)->active) {
 	    free(spec->labeled);
 	    spec->labeled = NULL;
-	    spec->flags |= GPTSPEC_ALL_MARKERS;
+	    spec->flags |= GPT_ALL_MARKERS;
 	} else {
-	    spec->flags &= ~GPTSPEC_ALL_MARKERS;
+	    spec->flags &= ~GPT_ALL_MARKERS;
 	}
     }
 
@@ -403,17 +484,22 @@ static void apply_gpt_changes (GtkWidget *widget, GPT_SPEC *spec)
 	}
     }
 
+    if (fitcombo != NULL) {
+	fittype_from_combo(GTK_COMBO(fitcombo)->entry, spec);
+    }
+
     if (!err) {
-	if (save) { /* do something other than a screen graph? */
+	if (fsave) { 
+	    /* saving to file */
 	    file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
 			  FSEL_DATA_MISC, spec);
 	} else { 
 	    png_plot *plot = (png_plot *) spec->ptr;
 
-	    set_plot_has_y2_axis(plot, spec->flags & GPTSPEC_Y2AXIS);
+	    set_plot_has_y2_axis(plot, spec->flags & GPT_Y2AXIS);
 	    redisplay_edited_png(plot);
 	}
-	session_changed(1);
+	mark_session_changed();
     }
 }
 
@@ -554,16 +640,106 @@ static void toggle_axis_selection (GtkWidget *w, GPT_SPEC *spec)
 	if (i >= MAX_PLOT_LINES) {
 	    break;
 	}
-	gtk_widget_set_sensitive(yaxiscombo[i], !no_y2);
+	if (yaxiscombo[i] != NULL) {
+	    gtk_widget_set_sensitive(yaxiscombo[i], !no_y2);
+	}
+    }
+}
+
+/* re-establish the default plot colors and reset the
+   color selection buttons accordingly */
+
+static void color_default_callback (GtkWidget *w, GtkWidget *book)
+{
+    GtkWidget *button;
+    char id[32];
+    int i;
+
+    sprintf(id, "color-button%d", BOXCOLOR);
+    button = g_object_get_data(G_OBJECT(book), id);
+
+    if (button != NULL) {
+	graph_palette_reset(BOXCOLOR);
+	color_patch_button_reset(button, BOXCOLOR);
+    } else {
+	for (i=0; i<BOXCOLOR; i++) {
+	    sprintf(id, "color-button%d", i);
+	    button = g_object_get_data(G_OBJECT(book), id);
+	    if (button != NULL) {
+		if (i == 0) {
+		    graph_palette_reset(i);
+		}
+		color_patch_button_reset(button, i);
+	    }
+	}
+    }
+}
+
+static void table_add_row (GtkWidget *tbl, int *rows, int cols)
+{
+    *rows += 1;
+    gtk_table_resize(GTK_TABLE(tbl), *rows, cols);    
+}
+
+static void add_color_selector (int i, GtkWidget *tbl, int *rows,
+				GtkWidget *notebook)
+{
+    GtkWidget *button, *hbox;
+    GtkWidget *label;
+    char str[32];
+
+    table_add_row(tbl, rows, TAB_MAIN_COLS);
+
+    hbox = gtk_hbox_new(FALSE, 2);
+
+    if (i == BOXCOLOR) {
+	strcpy(str, _("Fill color"));
+    } else {
+	sprintf(str, _("Color %d"), i + 1);
+    }
+
+    label = gtk_label_new(str);
+    gtk_container_add(GTK_CONTAINER(hbox), label);
+    gtk_table_attach_defaults(GTK_TABLE(tbl), hbox, 0, 1, 
+			      *rows - 1, *rows);
+    gtk_widget_show(label);
+    gtk_widget_show(hbox);
+
+    hbox = gtk_hbox_new(FALSE, 2);
+    button = color_patch_button(i);
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    gtk_table_attach_defaults(GTK_TABLE(tbl), hbox, 1, 2, 
+			      *rows - 1, *rows);
+    g_signal_connect(G_OBJECT(button), "clicked", 
+		     G_CALLBACK(graph_color_selector), 
+		     GINT_TO_POINTER(i));
+
+    gtk_widget_show_all(button);
+    gtk_widget_show(hbox);
+
+    sprintf(str, "color-button%d", i);
+    g_object_set_data(G_OBJECT(notebook), str, button);
+
+    if (i == BOXCOLOR || i == BOXCOLOR - 1) {
+	table_add_row(tbl, rows, TAB_MAIN_COLS);
+	hbox = gtk_hbox_new(FALSE, 2);
+	button = gtk_button_new_with_label(_("Reset to default"));
+	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 10);
+	g_signal_connect(G_OBJECT(button), "clicked", 
+			 G_CALLBACK(color_default_callback), 
+			 notebook);
+	gtk_table_attach(GTK_TABLE(tbl), hbox, 0, 2, *rows - 1, *rows,
+			 GTK_FILL, 0, 0, 5);
+	gtk_widget_show(button);
+	gtk_widget_show(hbox);
     }
 }
 
 static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec) 
 {
     GtkWidget *label, *vbox, *tbl;
-    int i, tbl_len;
+    int i, rows = 1;
     GList *keypos_list = NULL;
-
     gchar *keypos[] = {
 	"left top",
 	"right top",
@@ -586,8 +762,7 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
     gtk_widget_show(label);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);   
 
-    tbl_len = 1;
-    tbl = gtk_table_new(tbl_len, TAB_MAIN_COLS, FALSE);
+    tbl = gtk_table_new(rows, TAB_MAIN_COLS, FALSE);
     gtk_table_set_row_spacings(GTK_TABLE(tbl), 5);
     gtk_table_set_col_spacings(GTK_TABLE(tbl), 5);
     gtk_box_pack_start(GTK_BOX(vbox), tbl, FALSE, FALSE, 0);
@@ -597,17 +772,19 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	if (gpt_titles[i].tab == 0) {
 	    GtkWidget *entry;
 
-	    tbl_len++;
-	    gtk_table_resize(GTK_TABLE(tbl), tbl_len, TAB_MAIN_COLS);
+	    if (i > 0) {
+		table_add_row(tbl, &rows, TAB_MAIN_COLS);
+	    }
+
 	    label = gtk_label_new(_(gpt_titles[i].description));
 	    gtk_table_attach_defaults(GTK_TABLE (tbl), 
-				      label, 0, 1, tbl_len-1, tbl_len);
+				      label, 0, 1, rows-1, rows);
 	    gtk_widget_show(label);
 
 	    entry = gtk_entry_new();
 	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				      entry, 1, TAB_MAIN_COLS, 
-				      tbl_len-1, tbl_len);
+				      rows-1, rows);
 				      
             if (spec->titles[i] != NULL && *spec->titles[i] != '\0') {
 		gp_string_to_entry(entry, spec->titles[i]);
@@ -623,39 +800,69 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
     }
 
     /* specify position of plot key or legend */
-    tbl_len++;
+    table_add_row(tbl, &rows, TAB_MAIN_COLS);
     label = gtk_label_new(_("key position"));
     gtk_table_attach_defaults(GTK_TABLE(tbl), 
-			      label, 0, 1, tbl_len-1, tbl_len);
+			      label, 0, 1, rows-1, rows);
     gtk_widget_show(label);
 
     keycombo = gtk_combo_new();
     gtk_table_attach_defaults(GTK_TABLE(tbl), 
-			      keycombo, 1, TAB_MAIN_COLS, tbl_len-1, tbl_len);
+			      keycombo, 1, TAB_MAIN_COLS, rows-1, rows);
     gtk_combo_set_popdown_strings(GTK_COMBO(keycombo), keypos_list); 
     gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(keycombo)->entry), spec->keyspec);
-    gtk_widget_show(keycombo);	
+    gtk_widget_show(keycombo);
+    g_list_free(keypos_list);
+
+    /* choice of fitted line type, if appropriate */
+    if (spec->fit != PLOT_FIT_NA) {
+	GList *fitlist = NULL;
+
+	for (i=0; fittype_strings[i] != NULL; i++) {
+	    fitlist = g_list_append(fitlist, _(fittype_strings[i]));
+	}
+
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
+	label = gtk_label_new(_("fitted line"));
+	gtk_table_attach_defaults(GTK_TABLE(tbl), 
+				  label, 0, 1, rows-1, rows);
+	gtk_widget_show(label);
+
+	fitcombo = gtk_combo_new();
+	gtk_table_attach_defaults(GTK_TABLE(tbl), 
+				  fitcombo, 1, TAB_MAIN_COLS, rows-1, rows);
+	gtk_combo_set_popdown_strings(GTK_COMBO(fitcombo), fitlist); 
+	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(fitcombo)->entry), 
+			   _(fittype_strings[spec->fit]));
+	gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(fitcombo)->entry), FALSE);
+	g_signal_connect(G_OBJECT(GTK_COMBO(fitcombo)->entry), "changed",
+			 G_CALLBACK(fit_type_changed), spec);
+	gtk_widget_show(fitcombo);
+	g_list_free(fitlist);
+    } else {
+	fitcombo = NULL;
+    }
 
     /* give option of removing top & right border */
-    if (!(spec->flags & GPTSPEC_Y2AXIS)) { 
+    if (!(spec->flags & GPT_Y2AXIS)) { 
 	y2_check = NULL;
-	tbl_len++;
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
 	border_check = gtk_check_button_new_with_label(_("Show full border"));
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  border_check, 0, TAB_MAIN_COLS, 
-				  tbl_len-1, tbl_len);
-	if (!(spec->flags & GPTSPEC_MINIMAL_BORDER)) {
+				  rows-1, rows);
+	if (!(spec->flags & GPT_MINIMAL_BORDER)) {
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(border_check),
 					 TRUE);
 	}	
 	gtk_widget_show(border_check);
     } else {
 	border_check = NULL;
-	tbl_len++;
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
 	y2_check = gtk_check_button_new_with_label(_("Use only one y axis"));
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  y2_check, 0, TAB_MAIN_COLS, 
-				  tbl_len-1, tbl_len);
+				  rows-1, rows);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(y2_check),
 				     FALSE);
 	g_signal_connect(G_OBJECT(y2_check), "clicked", 
@@ -663,30 +870,14 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	gtk_widget_show(y2_check);
     }
 
-    /* give option of removing an auto-fitted line */
-    if (spec->flags & GPTSPEC_AUTO_OLS) { 
-	tbl_len++;
-	fitline_check = gtk_check_button_new_with_label(_("Hide fitted line"));
-	gtk_table_attach_defaults(GTK_TABLE(tbl), 
-				  fitline_check, 0, TAB_MAIN_COLS, 
-				  tbl_len-1, tbl_len);
-	if (spec->flags & GPTSPEC_OLS_HIDDEN) {
-	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fitline_check),
-					 TRUE);
-	}	
-	gtk_widget_show(fitline_check);
-    } else {
-	fitline_check = NULL;
-    }
-
     /* give option of showing all case markers */
-    if (spec->flags & GPTSPEC_ALL_MARKERS_OK) { 
-	tbl_len++;
+    if (spec->flags & GPT_ALL_MARKERS_OK) { 
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
 	markers_check = gtk_check_button_new_with_label(_("Show all data labels"));
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  markers_check, 0, TAB_MAIN_COLS, 
-				  tbl_len-1, tbl_len);
-	if (spec->flags & GPTSPEC_ALL_MARKERS) {
+				  rows-1, rows);
+	if (spec->flags & GPT_ALL_MARKERS) {
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(markers_check),
 					 TRUE);
 	}	
@@ -696,7 +887,7 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
     }
 
     /* set TT font (if gnuplot uses libgd and freetype) */
-    if (gnuplot_has_ttf()) {
+    if (gnuplot_has_ttf(0)) {
 	GtkWidget *ebox, *hsep;
 	GList *fontnames = NULL;
 	struct font_info *ttflist;
@@ -718,18 +909,18 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	}
 
 	/* first a separator */
-	tbl_len++;
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);	
 	hsep = gtk_hseparator_new();
 	gtk_table_attach_defaults(GTK_TABLE(tbl), hsep, 0, TAB_MAIN_COLS, 
-				  tbl_len-1, tbl_len);  
+				  rows-1, rows);  
 	gtk_widget_show(hsep);
 
-	tbl_len++;
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
 	ebox = gtk_event_box_new();
 	label = gtk_label_new(_("TrueType font"));
 	gtk_container_add(GTK_CONTAINER(ebox), label);
 	gtk_table_attach_defaults(GTK_TABLE (tbl), ebox, 0, 1, 
-				  tbl_len-1, tbl_len);
+				  rows-1, rows);
 	gtk_widget_show(label);
 	gtk_widget_show(ebox);
 
@@ -738,20 +929,21 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
 	ttfcombo = gtk_combo_new();
 	gtk_entry_set_max_length(GTK_ENTRY(GTK_COMBO(ttfcombo)->entry), 15);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), ttfcombo, 1, 2, 
-				  tbl_len-1, tbl_len);
+				  rows-1, rows);
 	gtk_combo_set_popdown_strings(GTK_COMBO(ttfcombo), fontnames); 
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(ttfcombo)->entry), default_font);
 	gtk_entry_set_width_chars(GTK_ENTRY(GTK_COMBO(ttfcombo)->entry), 15);
 	g_signal_connect(G_OBJECT(GTK_COMBO(ttfcombo)->entry), "activate", 
 			 G_CALLBACK(apply_gpt_changes), 
 			 spec);
-	gtk_widget_show (ttfcombo);
+	gtk_widget_show(ttfcombo);
+	g_list_free(fontnames);
 
 	ttfspin = gtk_spin_button_new_with_range(6, 24, 1);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(ttfspin), 
 				  get_point_size(gretl_png_font()));
 	gtk_table_attach_defaults(GTK_TABLE(tbl), ttfspin, 2, 3, 
-				  tbl_len-1, tbl_len);
+				  rows - 1, rows);
 	gtk_widget_show(ttfspin);
     } else {
 	ttfcombo = NULL;
@@ -759,55 +951,19 @@ static void gpt_tab_main (GtkWidget *notebook, GPT_SPEC *spec)
     }
 
     if (gnuplot_has_specified_colors()) { 
-	GtkWidget *hsep;
-	int colmax;
+	GtkWidget *hsep = gtk_hseparator_new();
 
-	tbl_len++;
-	hsep = gtk_hseparator_new();
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), hsep, 0, TAB_MAIN_COLS, 
-				  tbl_len-1, tbl_len);  
+				  rows - 1, rows);  
 	gtk_widget_show(hsep);
 
-	if (frequency_plot(spec)) {
-	    colmax = 1;
+	if (frequency_plot_code(spec->code)) {
+	    add_color_selector(BOXCOLOR, tbl, &rows, notebook);
 	} else {
-	    colmax = COLOR_MAX;
-	}
-
-	for (i=0; i<colmax; i++) {
-	    GtkWidget *button, *hbox;
-	    char labstr[16];
-
-	    if (frequency_plot(spec)) {
-		i = COLOR_MAX;
+	    for (i=0; i<BOXCOLOR; i++) {
+		add_color_selector(i, tbl, &rows, notebook);
 	    }
-
-	    tbl_len++;
-	    hbox = gtk_hbox_new(FALSE, 2);
-
-	    if (i == COLOR_MAX) {
-		sprintf(labstr, _("Fill color"));
-	    } else {
-		sprintf(labstr, _("Color %d"), i + 1);
-	    }
-
-	    label = gtk_label_new(labstr);
-	    gtk_container_add(GTK_CONTAINER(hbox), label);
-	    gtk_table_attach_defaults(GTK_TABLE(tbl), hbox, 0, 1, 
-				      tbl_len-1, tbl_len);
-	    gtk_widget_show(label);
-	    gtk_widget_show(hbox);
-
-	    hbox = gtk_hbox_new(FALSE, 2);
-	    button = color_patch_button(i);
-	    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-	    gtk_table_attach_defaults(GTK_TABLE(tbl), hbox, 1, 2, 
-				      tbl_len-1, tbl_len);
-	    g_signal_connect(G_OBJECT(button), "clicked", 
-			     G_CALLBACK(gnuplot_color_selector), 
-			     GINT_TO_POINTER(i));
-	    gtk_widget_show_all(button);
-	    gtk_widget_show(hbox);
 	}
     }
 }
@@ -861,11 +1017,12 @@ static void gpt_tab_output (GtkWidget *notebook, GPT_SPEC *spec)
     termcombo = gtk_combo_new();
     gtk_table_attach_defaults(GTK_TABLE(tbl), termcombo, 1, 2, 
 			      tbl_len-1, tbl_len);
-    gtk_combo_set_popdown_strings(GTK_COMBO(termcombo), termlist);   
+    gtk_combo_set_popdown_strings(GTK_COMBO(termcombo), termlist); 
     gtk_widget_show(termcombo);
+    g_list_free(termlist); 
 
     /* button to generate output to file */
-    filesavebutton = gtk_button_new_with_label(_("Save to file..."));
+    filesavebutton = gtk_button_new_from_stock(GTK_STOCK_SAVE_AS);
     GTK_WIDGET_SET_FLAGS(filesavebutton, GTK_CAN_DEFAULT);
     tbl_len++;
     gtk_table_attach_defaults(GTK_TABLE(tbl), 
@@ -874,7 +1031,7 @@ static void gpt_tab_output (GtkWidget *notebook, GPT_SPEC *spec)
 		      G_CALLBACK(apply_gpt_changes), 
 		      spec);
     gtk_widget_grab_default(filesavebutton);
-    gtk_widget_show(filesavebutton);    
+    gtk_widget_show(filesavebutton);  
 }
 
 static void linetitle_callback (GtkWidget *w, GPT_SPEC *spec)
@@ -888,26 +1045,30 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
     GtkWidget *vbox, *hbox;
     int i, tbl_len, tbl_num, tbl_col;
     char label_text[32];
-    GList *plot_types = NULL;
+    GList *stylist = NULL;
     GList *yaxis_loc = NULL;
     int do_scale_axis = 0;
 
-    if (spec->code == PLOT_REGULAR && (spec->flags & GPTSPEC_TS)) {
+    if (spec->code == PLOT_REGULAR && (spec->flags & GPT_TS)) {
 	do_scale_axis = 1;
     }
 
-    if (spec->flags & GPTSPEC_TS) {
-	plot_types = g_list_append(plot_types, "lines");
-	plot_types = g_list_append(plot_types, "points");
-    } else {
-	plot_types = g_list_append(plot_types, "points");
-	plot_types = g_list_append(plot_types, "lines");
+    if (frequency_plot_code(spec->code)) {
+	stylist = g_list_append(stylist, "boxes");
     }
 
-    plot_types = g_list_append(plot_types, "linespoints"); 
-    plot_types = g_list_append(plot_types, "impulses");
-    plot_types = g_list_append(plot_types, "dots");
-    plot_types = g_list_append(plot_types, "steps");
+    if (spec->flags & GPT_TS) {
+	stylist = g_list_append(stylist, "lines");
+	stylist = g_list_append(stylist, "points");
+    } else {
+	stylist = g_list_append(stylist, "points");
+	stylist = g_list_append(stylist, "lines");
+    }
+
+    stylist = g_list_append(stylist, "linespoints"); 
+    stylist = g_list_append(stylist, "impulses");
+    stylist = g_list_append(stylist, "dots");
+    stylist = g_list_append(stylist, "steps");
 
     if (do_scale_axis) {
 	yaxis_loc = g_list_append(yaxis_loc, "left");
@@ -996,7 +1157,7 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 			       spec->lines[i].style); 
 	    gtk_widget_set_sensitive(stylecombo[i], FALSE);
 	} else {
-	    gtk_combo_set_popdown_strings(GTK_COMBO(stylecombo[i]), plot_types); 
+	    gtk_combo_set_popdown_strings(GTK_COMBO(stylecombo[i]), stylist); 
 	    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(stylecombo[i])->entry), 
 			       spec->lines[i].style); 
 	} 
@@ -1041,7 +1202,7 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 	    gtk_combo_set_popdown_strings(GTK_COMBO(yaxiscombo[i]), yaxis_loc); 
 	    gtk_entry_set_text (GTK_ENTRY(GTK_COMBO(yaxiscombo[i])->entry), 
 				(spec->lines[i].yaxis == 1)? "left" : "right");  
-	    gtk_widget_show(yaxiscombo[i]);	
+	    gtk_widget_show(yaxiscombo[i]);
 	}
 
 	/* line-width adjustment */
@@ -1068,6 +1229,12 @@ static void gpt_tab_lines (GtkWidget *notebook, GPT_SPEC *spec)
 				  tbl_len-1, tbl_len);
 	gtk_widget_show(hbox);
     }
+
+    g_list_free(stylist);
+
+    if (yaxis_loc != NULL) {    
+	g_list_free(yaxis_loc);
+    }
 }
 
 static void label_pos_to_entry (double *pos, GtkWidget *w)
@@ -1167,7 +1334,7 @@ static void gpt_tab_labels (GtkWidget *notebook, GPT_SPEC *spec)
 	    button = gtk_button_new();
 	    g_object_set_data(G_OBJECT(button), "labelpos_entry", labelpos[i]);
 	    g_signal_connect(G_OBJECT(button), "clicked",
-			     G_CALLBACK(plot_label_position_click), spec);
+			     G_CALLBACK(plot_label_position_click), spec->ptr);
 	    icon = gdk_pixbuf_new_from_xpm_data((const char **) mini_mouse_xpm);
 	    image = gtk_image_new_from_pixbuf(icon);
 	    gtk_widget_set_size_request(button, 32, 26);
@@ -1398,7 +1565,7 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
     gpt_tab_XY(notebook, spec, 0);
     gpt_tab_XY(notebook, spec, 1);
 
-    if (spec->flags & GPTSPEC_Y2AXIS) {
+    if (spec->flags & GPT_Y2AXIS) {
 	gpt_tab_XY(notebook, spec, 2);
     }
 
@@ -1424,7 +1591,7 @@ int show_gnuplot_dialog (GPT_SPEC *spec)
     gtk_widget_show(button);
 
     /* Close button (do not apply changes) */
-    button = standard_button(GTK_STOCK_CLOSE);
+    button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_container_add(GTK_CONTAINER(hbox), button);
     g_signal_connect(G_OBJECT(button), "clicked",
