@@ -19,9 +19,11 @@
 
 #include "gretl.h"
 #include "graph_page.h"
+#include "session.h"
 
 #ifdef G_OS_WIN32 
 # include <io.h>
+# include "gretlwin32.h"
 #else
 # include <unistd.h>
 # include <sys/stat.h>
@@ -156,11 +158,18 @@ static int oddgraph (int ng, int i)
     return (ng % 2) && (i == ng - 1);
 }
 
+enum {
+    SCALE_LARGE,
+    SCALE_REGULAR,
+    SCALE_MEDIUM,
+    SCALE_SMALL
+};
+
 static int tex_graph_setup (int ng, FILE *fp)
 {
     char fname[FILENAME_MAX];
-    double scale = 1.0;
-    double vspace = 1.0;
+    double scale[] = { 1.2, 1.0, 0.9, 0.85 };
+    double s, vspace = 1.0;
     int i;
 
     if (ng > GRAPHS_MAX) {
@@ -171,29 +180,30 @@ static int tex_graph_setup (int ng, FILE *fp)
     gretl_push_c_numeric_locale();
 
     if (ng == 1) {
+	s = scale[SCALE_LARGE];
 	sprintf(fname, "%s_1", gpage_tex_base);
-	fprintf(fp, "\\includegraphics[scale=1.2]{%s}\n", fname);
+	fprintf(fp, "\\includegraphics[scale=%g]{%s}\n", s, fname);
     } else if (ng == 2) {
+	s = scale[SCALE_REGULAR];
 	sprintf(fname, "%s_%d", gpage_tex_base, 1);
-	fprintf(fp, "\\includegraphics{%s}\n\n", fname);
+	fprintf(fp, "\\includegraphics[scale=%g]{%s}\n\n", s, fname);
 	fprintf(fp, "\\vspace{%gin}\n\n", vspace);
 	sprintf(fname, "%s_%d", gpage_tex_base, 2);
 	fprintf(fp, "\\includegraphics{%s}\n\n", fname);
     } else if (ng == 3) {
-	scale = 0.9;
+	s = scale[SCALE_MEDIUM];
 	vspace = 0.25;
 	for (i=0; i<3; i++) {
 	    sprintf(fname, "%s_%d", gpage_tex_base, i + 1);
-	    fprintf(fp, "\\includegraphics[scale=%g]{%s}\n\n",
-		    scale, fname);
+	    fprintf(fp, "\\includegraphics[scale=%g]{%s}\n\n", s, fname);
 	    fprintf(fp, "\\vspace{%gin}\n", vspace);
 	}
     } else {
 	if (ng > 6) {
-	    scale = 0.85;
+	    s = scale[SCALE_SMALL];
 	    vspace = 0.20;
 	} else {
-	    scale = 0.9;
+	    s = scale[SCALE_MEDIUM];
 	    vspace = 0.25;
 	}	    
 	fputs("\\begin{tabular}{cc}\n", fp);
@@ -201,10 +211,9 @@ static int tex_graph_setup (int ng, FILE *fp)
 	    sprintf(fname, "%s_%d", gpage_tex_base, i + 1);
 	    if (oddgraph(ng, i)) {
 		fprintf(fp, "\\multicolumn{2}{c}{\\includegraphics[scale=%g]{%s}}",
-			scale, fname);
+			s, fname);
 	    } else {
-		fprintf(fp, "\\includegraphics[scale=%g]{%s}",
-			scale, fname);
+		fprintf(fp, "\\includegraphics[scale=%g]{%s}", s, fname);
 		if (i % 2 == 0) {
 		    fputs(" &\n  ", fp);
 		} else if (i < ng - 1) {
@@ -269,12 +278,16 @@ int graph_page_add_file (const char *fname)
     }
 
     fnames = myrealloc(gpage.fnames, ng * sizeof *fnames);
-    if (fnames == NULL) return 1;
+    if (fnames == NULL) {
+	return E_ALLOC;
+    }
 
     fnames[ng - 1] = g_strdup(fname);
     gpage.fnames = fnames;
     gpage.ngraphs = ng;
 
+    mark_session_changed();
+
     return 0;
 }
 
@@ -315,29 +328,35 @@ static int gp_make_outfile (const char *gfname, int i, double scale)
     int err = 0;
 
     fp = gretl_fopen(gfname, "r");
-    if (fp == NULL) return 1;
+    if (fp == NULL) {
+	fprintf(stderr, "Couldn't read from %s\n", gfname);
+	return E_FOPEN;
+    }
 
     fname = gpage_fname(".plt", 0);
 
     fq = gretl_fopen(fname, "w");
     if (fq == NULL) {
+	fprintf(stderr, "Couldn't write to %s\n", fname);
 	fclose(fp);
-	return 1;
+	return E_FOPEN;
     }
 
     gretl_push_c_numeric_locale();
     
     if (gpage.output == PDF_OUTPUT) {
-	fputs("set term pdf\n", fq);
+	fprintf(fq, "set term pdf%s", (gpage.color)? " color" : " monochrome dashed");
 	if (scale != 1.0) {
-	    fprintf(fq, "set size %g,%g\n", scale, scale);
+	    fprintf(fq, " size %g,%g\n", scale * 5.0, scale * 3.0);
+	} else {
+	    fputc('\n', fq);
 	}	
 	fname = gpage_fname(".pdf", i);
     } else {
 #ifdef ENABLE_NLS
 	fprint_gnuplot_encoding("postscript", fq);
 #endif
-	fprintf(fq, "set term postscript eps%s\n", (gpage.color)? " color" : "");
+	fprintf(fq, "set term postscript eps%s\n", (gpage.color)? " color" : " monochrome");
 	if (scale != 1.0) {
 	    fprintf(fq, "set size %g,%g\n", scale, scale);
 	}
@@ -442,8 +461,7 @@ int dvips_compile (char *texshort)
 
 #if defined(G_OS_WIN32)
     if (*dvips_path == 0 && get_dvips_path(dvips_path)) {
-	DWORD dw = GetLastError();
-	win_show_error(dw);
+	win_show_last_error();
 	return 1;
     }
 
@@ -471,7 +489,7 @@ static int latex_compile_graph_page (void)
     }
     remove(fname);
 
-    err = latex_compile(gpage_base);
+    err = latex_compile(gpage_tex_base);
 
     if (err == LATEX_ERROR) {
 	char *fname = gpage_fname(".log", 0);
@@ -524,11 +542,7 @@ static int real_display_gpage (void)
     }
 
 #if defined(G_OS_WIN32)
-    if ((int) ShellExecute(NULL, "open", fname, NULL, NULL, SW_SHOW) <= 32) {
-	DWORD dw = GetLastError();
-	win_show_error(dw);
-	err = 1;
-    }
+    err = win32_open_file(fname);
 #elif defined(OSX_BUILD)
     err = osx_open_file(fname);
 #else
@@ -563,16 +577,33 @@ static void gpage_cleanup (void)
     remove(fname);
 }
 
+/* FIXME color/mono selection */
+
 int display_graph_page (void)
 {
+    const char *opts[] = {
+	N_("color"),
+	N_("monochrome")
+    };
+    const char *sdir = get_session_dirname();
     char *latex_orig = NULL;
-    int err = 0;
+    int resp, err = 0;
 
     if (gpage.ngraphs == 0) {
 	gpage_errmsg(_("The graph page is empty"), 1);
 	return 1;
     }
 
+    resp = radio_dialog(_("graph page options"), NULL, opts, 
+			2, 0, 0);
+    if (resp < 0) {
+	return 0;
+    }
+
+    gpage.color = (resp == 0);
+
+    chdir(sdir);
+
     gpage_filenames_init(NULL);
 
     if (!strncmp(latex, "pdf", 3)) {
@@ -612,6 +643,10 @@ int display_graph_page (void)
 
     gpage_cleanup();
 
+    if (err) {
+	gui_errmsg(err);
+    }
+
     return err;
 }
 
@@ -633,10 +668,13 @@ int graph_page_get_n_graphs (void)
 }
 
 int save_graph_page (const char *fname)
-{
+{    
+    const char *sdir = get_session_dirname();
     char *latex_orig = NULL;
     int err = 0;
 
+    chdir(sdir);
+
     gpage_filenames_init(fname);
 
     if (!strncmp(latex, "pdf", 3)) {
@@ -667,6 +705,3 @@ int save_graph_page (const char *fname)
 
     return err;
 }
-
-
-
