@@ -28,7 +28,6 @@
 #include "ssheet.h"
 #include "console.h"
 #include "session.h"
-#include "webget.h"
 #include "database.h"
 #include "datafiles.h"
 #include "cmdstack.h"
@@ -37,6 +36,7 @@
 #include "menustate.h"
 #include "fileselect.h"
 #include "filters.h"
+#include "calculator.h"
 
 #include <dirent.h>
 
@@ -49,15 +49,13 @@
 # include "gretlwin32.h"
 #endif
 
-/* #define WINDEBUG 1 */
-
-#ifdef WINDEBUG
-FILE *dbg;
-#endif
+/* update.c */
+extern int silent_update_query (void);
+extern int update_query (void); 
 
 /* functions private to gretl.c */
 static void sort_varlist (gpointer p, guint col, GtkWidget *w);
-static GtkWidget *make_main_window (int gui_get_data);
+static GtkWidget *make_main_window (void);
 
 static gboolean main_popup_handler (GtkWidget *w, GdkEventButton *event,
 				    gpointer data);
@@ -107,7 +105,6 @@ static const struct poptOption options[] = {
 windata_t *mdata;
 DATAINFO *datainfo;
 
-char cmdfile[MAXLEN];
 char scriptfile[MAXLEN];
 char sessionfile[MAXLEN];
 char tryfile[MAXLEN];
@@ -116,7 +113,7 @@ PATHS paths;                /* useful paths */
 double **Z;                 /* data set */
 MODEL **models;             /* gretl models structs */
 
-int plot_count, data_status, orig_vars;
+int data_status, orig_vars;
 float gui_scale;
 
 /* defaults for some options */
@@ -129,8 +126,6 @@ int main_y = -1;
 int mainwin_width = 520;
 int mainwin_height = 420;
 
-char dbproxy[21];
-
 #if defined(G_OS_WIN32)
 char Rcommand[MAXSTR] = "RGui.exe";
 char calculator[MAXSTR] = "calc.exe";
@@ -151,58 +146,21 @@ char Rcommand[MAXSTR] = "xterm -e R";
 char Browser[MAXSTR] = "mozilla";
 #endif
 
-#ifdef HAVE_TRAMO
-char tramo[MAXSTR] = "tramo";
-char tramodir[MAXSTR] = "";
-#endif
-
 static void spreadsheet_edit (gpointer p, guint u, GtkWidget *w) 
 {
     show_spreadsheet(SHEET_EDIT_VARLIST);
 }
 
-#if 0 /* not yet */
 static void matrix_edit_callback (gpointer p, guint u, GtkWidget *w) 
 {
-    edit_matrix(NULL);
+    gui_new_matrix();
 }
-#endif
 
 static void manual_update_query (gpointer p, guint u, GtkWidget *w)
 {
     update_query();
 }
 
-#ifdef USE_GNOME_HELP
-static void gnome_help (void)
-{
-    GError *error = NULL;
-
-    gnome_help_display ("gretl.xml", NULL, &error);
-        
-    if (error != NULL) {
-	g_warning (error->message);
-	g_error_free (error);
-    }
-}
-#endif 
-
-#ifndef G_OS_WIN32
-static void root_check (void)
-{
-    if (getuid() == 0) {
-	int resp;
-
-	resp = yes_no_dialog ("gretl", _("You seem to be running gretl " 
-			      "as root.  Do you really want to do this?"), 
-			      0);
-	if (resp == GRETL_NO) {
-	    exit(EXIT_FAILURE);
-	}
-    }
-}
-#endif
-
 extern void find_var (gpointer p, guint u, GtkWidget *w); /* gui_utils.c */
 
 static void varinfo_callback (gpointer p, guint u, GtkWidget *w)
@@ -328,6 +286,10 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Tools/_P-value finder"), NULL, stats_calculator, CALC_PVAL, NULL, GNULL },
     { N_("/Tools/_Test statistic calculator"), NULL, stats_calculator, CALC_TEST, 
       NULL, GNULL },
+    { N_("/Tools/_Nonparametric tests"), NULL, stats_calculator, CALC_NPTEST, 
+      NULL, GNULL },
+    { N_("/Tools/_Probability distributions"), NULL, stats_calculator, CALC_GRAPH, 
+      NULL, GNULL },
     { N_("/Tools/_Seed for random numbers"), NULL, rand_seed_dialog, 0, 
       NULL, GNULL },
     { "/Tools/sep1", NULL, NULL, 0, "<Separator>", GNULL },
@@ -376,9 +338,8 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Data/_Compact data..."), NULL, do_compact_data_set, 0, NULL, GNULL },
     { N_("/Data/_Expand data..."), NULL, do_expand_data_set, 0, NULL, GNULL },
     { N_("/Data/_Transpose data..."), NULL, gui_transpose_data, 0, NULL, GNULL },
-#if 0 /* not ready */
-    { N_("/Data/_Matrix..."), NULL, matrix_edit_callback, 0, NULL, GNULL },
-#endif
+    { "/Data/sep3", NULL, NULL, 0, "<Separator>", GNULL },
+    { N_("/Data/_Refresh window"), NULL, refresh_data, 0, NULL, GNULL },
 
     /* View menu */
     { N_("/_View"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -409,8 +370,10 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/View/_Summary statistics"), NULL, do_menu_op, SUMMARY, NULL, GNULL },
     { "/View/sep2", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/View/_Correlation matrix"), NULL, do_menu_op, CORR, NULL, GNULL },
+    { N_("/View/Cross _Tabulation"), NULL, do_menu_op, XTAB, NULL, GNULL },
     { N_("/View/_Principal components"), NULL, do_menu_op, PCA, NULL, GNULL },
     { N_("/View/_Mahalanobis distances"), NULL, do_menu_op, MAHAL, NULL, GNULL },
+    { N_("/View/C_ross-correlogram"), NULL, xcorrgm_callback, 0, NULL, GNULL },
 
     /* "Add" (variables) menu */
     { N_("/_Add"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -441,6 +404,10 @@ GtkItemFactoryEntry data_items[] = {
       RANDOM_CHISQ, NULL, GNULL },
     { N_("/Add/Random variable/Student's _t..."), NULL, add_rand_callback, 
       RANDOM_ST, NULL, GNULL },
+    { N_("/Add/Random variable/Binomial..."), NULL, add_rand_callback, 
+      RANDOM_BIN, NULL, GNULL },
+    { N_("/Add/Random variable/Poisson..."), NULL, add_rand_callback, 
+      RANDOM_POIS, NULL, GNULL },
     { "/Add/sep3", NULL, NULL, 0, "<Separator>", GNULL },
     /* dummies */
     { N_("/Add/_Periodic dummies"), NULL, add_dummies, TS_DUMMIES, NULL, GNULL },
@@ -452,6 +419,7 @@ GtkItemFactoryEntry data_items[] = {
     /* genr */
     { N_("/Add/Define _new variable..."), NULL, gretl_callback, 
       GENR, NULL, GNULL },
+    { N_("/Add/_Define matrix..."), NULL, matrix_edit_callback, 0, NULL, GNULL },
 
     /* Sample menu */
     { N_("/_Sample"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -511,7 +479,6 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Variable/_TRAMO analysis"), NULL, do_tramo_x12a, TRAMO, NULL, GNULL },
 #endif
     { N_("/Variable/_Hurst exponent"), NULL, do_hurst, 0, NULL, GNULL }, 
-    { N_("/Variable/_Runs test"), NULL, do_menu_op, RUNS, NULL, GNULL }, 
     { "/Variable/sep2", NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Variable/_Edit attributes"), NULL, varinfo_callback, 0, NULL, GNULL },
     { N_("/Variable/Set missing _value code..."), NULL, gretl_callback, 
@@ -553,17 +520,8 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Model/Robust estimation/_Rank correlation..."), NULL, 
       selector_callback, SPEARMAN, NULL, GNULL },
     { "/Model/sep5",  NULL, NULL, 0, "<Separator>", GNULL },
-    { N_("/Model/_Bivariate tests"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Model/Bivariate tests/Difference of _means"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Model/Bivariate tests/Difference of means/Assuming _equal variances..."), NULL, 
-      selector_callback, MEANTEST, NULL, GNULL },
-    { N_("/Model/Bivariate tests/Difference of means/Assuming _unequal variances..."), NULL, 
-      selector_callback, MEANTEST2, NULL, GNULL },
-    { N_("/Model/Bivariate tests/Difference of _variances..."), NULL, selector_callback, 
-      VARTEST, NULL,  GNULL },
-    { "/Model/sep6",  NULL, NULL, 0, "<Separator>", GNULL },
     { N_("/Model/_Maximum likelihood..."), NULL, gretl_callback, MLE, NULL, GNULL },
-    { "/Model/sep7",  NULL, NULL, 0, "<Separator>", GNULL },
+    { N_("/Model/_GMM..."), NULL, gretl_callback, GMM, NULL, GNULL },
     { N_("/Model/_Simultaneous equations..."), NULL, gretl_callback, SYSTEM, NULL, GNULL },
 
     /* Help menu */
@@ -589,11 +547,18 @@ GtkItemFactoryEntry data_items[] = {
 static void gui_usage (void)
 {
     gui_logo(NULL);
-    printf(I_("You may supply the name of a data file on the command line.\n"));
-    printf(I_("Or you may do \"gretl -r script_file\" to open a script.\n"));
-    printf(I_("Or you may do \"gretl -d database\" to open a gretl database.\n"));
-    printf(I_("You may do \"gretl -e\" to force use of English.\n"));
-    printf(I_("You may do \"gretl -q\" to force use of Basque.\n"));
+
+    printf(I_("\nYou may supply the name of a data file on the command line.\n"
+	      "Options:\n"
+	      " -c or --dump      Dump the configuration file.\n"
+	      " -d or --db        Open a local database.\n"
+	      " -e or --english   Force use of English rather than translation.\n"
+	      " -q or --basque    Force use of Basque translation.\n"
+	      " -r or --run       Open a command script.\n"
+	      " -w or --webdb     Open a remote database.\n"
+	      " -h or --help      Print this info and exit.\n"
+	      " -v or --version   Print version info and exit.\n"));
+
     exit(0);
 }
 
@@ -652,11 +617,6 @@ static void fix_dbname (char *db)
     }
 }
 
-static void destroy (GtkWidget *widget, gpointer data)
-{
-    gtk_main_quit();
-}
-
 #ifdef ENABLE_NLS
 
 # if defined(G_OS_WIN32)
@@ -666,7 +626,7 @@ static void real_nls_init (void)
     char gretldir[MAXSTR], localedir[MAXSTR];
     char *loc;
 
-    if (read_reg_val(HKEY_CLASSES_ROOT, "gretl", "gretldir", gretldir)) {
+    if (read_reg_val(HKEY_LOCAL_MACHINE, "gretl", "gretldir", gretldir)) {
 	return;
     }
 
@@ -686,7 +646,7 @@ static void real_nls_init (void)
 
     loc = setlocale(LC_ALL, "");
     set_gretl_charset(loc);
-    bindtextdomain (PACKAGE, LOCALEDIR);
+    bindtextdomain(PACKAGE, LOCALEDIR);
     textdomain(PACKAGE);
     bind_textdomain_codeset(PACKAGE, "UTF-8");
 }
@@ -708,11 +668,12 @@ void nls_init (void)
 static void force_language (int f)
 {
     if (f == ENGLISH) {
+	putenv("LANGUAGE=english");
 	setlocale(LC_ALL, "C");
     } else if (f == BASQUE) {
 # ifdef G_OS_WIN32
 	setlocale(LC_ALL, "eu");
-#else
+# else
 	setlocale(LC_ALL, "eu_ES");
 # endif
     }
@@ -751,19 +712,25 @@ static void record_filearg (char *targ, const char *src)
 }
 #endif
 
+static int have_data (void)
+{
+    return datainfo != NULL && datainfo->v > 0;
+}
+
 int main (int argc, char *argv[])
 {
-    int err = 0, gui_get_data = 0;
+    int open_datafile = 0;
     int ftype = 0;
     char dbname[MAXLEN];
     char filearg[MAXLEN];
+    int opt = 0;
 #ifdef USE_GNOME
     GnomeProgram *program;
 #endif
-
-#ifdef WINDEBUG
-    dbg = fopen("debug.txt", "w");
-#endif    
+#ifdef G_OS_WIN32
+    int debug = 0;
+#endif
+    int err = 0;
 
 #ifdef ENABLE_NLS
     nls_init();
@@ -793,20 +760,25 @@ int main (int argc, char *argv[])
 #endif /* USE_GNOME */
 
     libgretl_init();
-    set_paths(&paths, OPT_D | OPT_X); /* defaults, gui */
+    gretl_set_paths(&paths, OPT_D | OPT_X); /* defaults, gui */
+    set_program_startdir(argv[0]);
 
 #ifdef G_OS_WIN32
     gretl_win32_init(argv[0]);
 #else 
-    set_rcfile(); /* also calls read_rc() */
-    if (!expert) {
-	root_check();
-    }
-#endif/* G_OS_WIN32 */
+    gretl_config_init();
+#endif
 
     if (argc > 1) {
 	int force_lang = 0;
-	int opt = parseopt((const char **) argv, argc, filearg, &force_lang);
+
+	opt = parseopt((const char **) argv, argc, filearg, &force_lang);
+
+#ifdef G_OS_WIN32
+	if (opt & OPT_DEBUG) {
+	    debug = 1;
+	}
+#endif
 
 	switch (opt) {
 	case OPT_HELP:
@@ -825,7 +797,6 @@ int main (int argc, char *argv[])
 	    }
 	    get_runfile(filearg);
 #endif
-	    gui_get_data = 1;
 	    break;
 	case OPT_DBOPEN:
 	case OPT_WEBDB:
@@ -840,31 +811,25 @@ int main (int argc, char *argv[])
 	    if (opt == OPT_DBOPEN) {
 		fix_dbname(dbname);
 	    }
-	    gui_get_data = opt;
 	    break;
 	case OPT_DUMP:
 	    dump_rc();
 	    exit(EXIT_SUCCESS);
 	    break;
 	default:
-	    /* let's suppose the argument is a data file */
+	    /* let's suppose any string argument is a data file */
+	    if (*filearg != '\0') {
+		open_datafile = 1;
+	    }
 	    break;
 	}
 
 #ifdef ENABLE_NLS
 	if (force_lang) {
 	    force_language(force_lang);
-	    if (argc == 2) {
-		gui_get_data = 1;
-	    }	
 	}
 #endif
-    } else {
-	gui_get_data = 1;
-    }
-
-    strcpy(cmdfile, paths.userdir);
-    strcat(cmdfile, "session.inp");
+    } 
 
     /* allocate data information struct */
     datainfo = datainfo_new();
@@ -873,16 +838,10 @@ int main (int argc, char *argv[])
     }
 
     /* allocate memory for models */
-    models = malloc(3 * sizeof *models);
-    if (models == NULL) noalloc(_("models")); 
-
-    models[0] = gretl_model_new_protected();
-    models[1] = gretl_model_new_protected();
-    models[2] = gretl_model_new_protected();
-
-    if (models[0] == NULL || models[1] == NULL || models[2] == NULL) {
-	noalloc(_("models")); 
-    }
+    models = allocate_working_models(3);
+    if (models == NULL) {
+	noalloc(_("models"));
+    } 
 
     library_command_init();
 
@@ -891,7 +850,7 @@ int main (int argc, char *argv[])
     init_fileptrs();
 
     /* get the data file, if specified on the command line */
-    if (!gui_get_data) {
+    if (open_datafile) {
 	PRN *prn; 
 
 	prn = gretl_print_new(GRETL_PRINT_STDERR);
@@ -936,11 +895,10 @@ int main (int argc, char *argv[])
 	case GRETL_WF1:
 	case GRETL_DTA:
 	case GRETL_JMULTI:
-	    err = get_worksheet_data(paths.datfile, ftype, 0, &gui_get_data);
+	    err = get_worksheet_data(paths.datfile, ftype, 0);
 	    break;
 	case GRETL_SCRIPT:
 	case GRETL_SESSION:
-	    gui_get_data = 1;
 	    get_runfile(paths.datfile);
 	    *paths.datfile = '\0';
 	    break;
@@ -951,7 +909,7 @@ int main (int argc, char *argv[])
 	    *tryfile = '\0';
 	    *paths.datfile = '\0';
 	    fix_dbname(dbname);
-	    gui_get_data = OPT_DBOPEN;
+	    opt = OPT_DBOPEN;
 	    break;
 	case GRETL_UNRECOGNIZED:
 	    gui_usage();
@@ -966,18 +924,21 @@ int main (int argc, char *argv[])
 	}
 #endif
 
+	if (err == E_CANCEL) {
+	    err = 0;
+	    ftype = 0;
+	    *tryfile = '\0';
+	    *paths.datfile = '\0';
+	}
+
 	if (ftype != GRETL_SCRIPT && err) {
 	    errmsg(err, prn);
 	    exit(EXIT_FAILURE);
 	}
+
 	gretl_print_destroy(prn);
     }
 
-#ifdef WINDEBUG
-    fprintf(dbg, "starting on GUI building\n");
-    fclose(dbg);
-#endif
-
     /* create the GUI */
     gretl_tooltips_init();
     gretl_stock_icons_init();
@@ -985,9 +946,9 @@ int main (int argc, char *argv[])
     /* create main window */
     if ((mdata = mymalloc(sizeof *mdata)) == NULL)
 	noalloc(_("GUI"));
-    if (make_main_window(gui_get_data) == NULL) 
+    if (make_main_window() == NULL) 
 	noalloc(_("main window"));
-    if (!gui_get_data) {
+    if (have_data()) {
 	set_sample_label(datainfo);
     }
 
@@ -1002,21 +963,17 @@ int main (int argc, char *argv[])
     restore_sample_state(FALSE);
     main_menubar_state(FALSE);
 
-#ifdef HAVE_TRAMO
-    set_tramo_ok(-1);
-#endif
-#ifdef HAVE_X12A
-    set_x12a_ok(-1);
-#endif
-
-    if (!gui_get_data) {
+    if (have_data()) {
 	register_data(paths.datfile, tryfile, 1);
 	maybe_display_string_table();
 	*tryfile = 0;
     }
 
-    /* opening a script from the command line? */
+    /* opening a script or session from the command line? */
     if (tryfile[0] != '\0') { 
+	if (gretl_is_pkzip_file(tryfile)) {
+	    ftype = GRETL_SESSION;
+	}
 	if (ftype == GRETL_SESSION) {
 	    do_open_session();
 	} else {
@@ -1025,18 +982,23 @@ int main (int argc, char *argv[])
     }
 
     /* check for program updates? */
-    proxy_init(dbproxy);
     if (updater) {
 	silent_update_query(); 
     }
 
     /* try opening specified database */
-    if (gui_get_data == OPT_DBOPEN) {
+    if (opt == OPT_DBOPEN) {
 	open_named_db_index(dbname);
-    } else if (gui_get_data == OPT_WEBDB) {
+    } else if (opt == OPT_WEBDB) {
 	open_named_remote_db_index(dbname);
     }
 
+#ifdef G_OS_WIN32
+    if (debug) {
+	gretl_win32_debug();
+    }
+#endif
+
     /* Enter the event loop */
     gtk_main();
 
@@ -1047,10 +1009,7 @@ int main (int argc, char *argv[])
 	free_Z(Z, datainfo);
     }
 
-    gretl_model_free_on_exit(models[0]);
-    gretl_model_free_on_exit(models[1]);
-    gretl_model_free_on_exit(models[2]);
-    free(models);
+    destroy_working_models(models, 3);
 
     library_command_free();
     libgretl_cleanup();
@@ -1067,7 +1026,12 @@ int main (int argc, char *argv[])
 static void check_varmenu_state (GtkTreeSelection *select, gpointer p)
 {
     if (mdata->ifac != NULL) {
-	int sc = selection_count(select, NULL);
+	int vnum = 0;
+	int sc = selection_count(select, &vnum);
+
+	if (sc == 1 && vnum > 0) {
+	    mdata->active_var = vnum;
+	}
 
 	variable_menu_state(sc == 1);
     }
@@ -1184,30 +1148,61 @@ static void mdata_select_all (void)
     gtk_tree_selection_select_all(select);
 }
 
+static int get_line_pos (GtkTreeModel *mod)
+{
+    GtkTreeIter iter;
+    gchar *idstr;
+    int i = 1, pos = 0;
+
+    if (gtk_tree_model_get_iter_first(mod, &iter)) {
+	while (gtk_tree_model_iter_next(mod, &iter)) {
+	    gtk_tree_model_get(mod, &iter, 0, &idstr, -1);
+	    if (idstr != NULL && atoi(idstr) == mdata->active_var) {
+		pos = i;
+	    }
+	    g_free(idstr);
+	    if (pos) {
+		break;
+	    }
+	    i++;
+	}
+    }
+
+    return pos;
+}
+
 void populate_varlist (void)
 {
     GtkTreeStore *store;
     GtkTreeSelection *select;
     GtkTreeIter iter;    
     char id[4];
+    int pos = 0;
     int i;
 
     static gint check_connected;
     static gint click_connected;
 
-    /* find and clear the existing tree */
     store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(mdata->listbox)));
+
+    if (store != NULL) {
+	/* record line position? */
+	pos = get_line_pos(GTK_TREE_MODEL(store));
+    }
+    
     gtk_tree_store_clear(store);
+    sort_varlist(NULL, 0, NULL);
 
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
     for (i=0; i<datainfo->v; i++) {
-	int pv;
+	int pv = 0;
 
 	if (var_is_hidden(datainfo, i)) {
 	    continue;
 	}
-	if (i > 0 && is_standard_lag(i, datainfo, &pv)) {
+	if (i > 0 && (is_standard_lag(i, datainfo, &pv) ||
+		      is_dummy_child(i, datainfo, &pv))) {
 	    if (pv > 0) {
 		GtkTreeIter child_iter, parent_iter;
 
@@ -1222,27 +1217,51 @@ void populate_varlist (void)
 				       -1);	
 		}	
 	    }
-	    continue;
 	}
-	gtk_tree_store_append(store, &iter, NULL);
-	sprintf(id, "%d", i);
-	gtk_tree_store_set(store, &iter, 
-			   0, id, 
-			   1, datainfo->varname[i],
-			   2, VARLABEL(datainfo, i),
-			   -1);
+
+	if (pv == 0) {
+	    gtk_tree_store_append(store, &iter, NULL);
+	    sprintf(id, "%d", i);
+	    gtk_tree_store_set(store, &iter, 
+			       0, id, 
+			       1, datainfo->varname[i],
+			       2, VARLABEL(datainfo, i),
+			       -1);
+	}
     } 
 
-    mdata->active_var = 1;
-
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-    gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+
+    if (pos != 0) {
+	/* return to previous position? */
+	GtkTreeIter last;
+
+	i = 1;
+	while (1) {
+	    last = iter;
+	    if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter)) {
+		iter = last;
+		break;
+	    } else if (i == pos) {
+		break;
+	    }
+	    i++;
+	}
+	pos = i;
+    } else {
+	pos = 1;
+	gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+    }
+
+    mdata->active_var = pos;
     select = gtk_tree_view_get_selection(GTK_TREE_VIEW(mdata->listbox));
     gtk_tree_selection_select_iter(select, &iter);
 
     if (datainfo->v > 1) {
-	GtkTreePath *path = gtk_tree_path_new_from_string("1");
+	GtkTreePath *path;
 
+	sprintf(id, "%d", pos);
+	path = gtk_tree_path_new_from_string(id);
 	gtk_tree_view_set_cursor(GTK_TREE_VIEW(mdata->listbox), path, NULL, FALSE);
 	gtk_tree_path_free(path);
     }
@@ -1396,7 +1415,7 @@ static void scale_main_window (void)
     }
 }
 
-static GtkWidget *make_main_window (int gui_get_data) 
+static GtkWidget *make_main_window (void) 
 {
     GtkWidget *main_vbox;
     GtkWidget *box, *dlabel, *align;
@@ -1441,7 +1460,7 @@ static GtkWidget *make_main_window (int gui_get_data)
     g_signal_connect(G_OBJECT(mdata->w), "delete_event",
 		     G_CALLBACK(exit_check), NULL);
     g_signal_connect(G_OBJECT(mdata->w), "destroy",
-		     G_CALLBACK(destroy), NULL);
+		     G_CALLBACK(gtk_main_quit), NULL);
 
     gtk_window_set_title(GTK_WINDOW(mdata->w), "gretl");
     gtk_window_set_default_size(GTK_WINDOW(mdata->w), 
@@ -1497,7 +1516,7 @@ static GtkWidget *make_main_window (int gui_get_data)
     gtk_box_pack_start(GTK_BOX(main_vbox), mdata->status, FALSE, TRUE, 0);
 
     /* put stuff into list box, activate menus */
-    if (!gui_get_data) {
+    if (have_data()) {
 	populate_varlist();
     }
 
@@ -1544,11 +1563,11 @@ static void set_up_main_menu (void)
     mdata->mbar = gtk_item_factory_get_widget(mdata->ifac, "<main>");
 }
 
-int gui_restore_sample (void)
+int gui_restore_sample (double ***pZ, DATAINFO **ppdinfo)
 {
     int err;
 
-    err = restore_full_sample(&Z, &datainfo);
+    err = restore_full_sample(pZ, ppdinfo, NULL);
     if (err) {
 	gui_errmsg(err);
     } else {
@@ -1560,7 +1579,7 @@ int gui_restore_sample (void)
 
 static void restore_sample_callback (gpointer p, int verbose, GtkWidget *w)
 {
-    int err = gui_restore_sample(); 
+    int err = gui_restore_sample(&Z, &datainfo); 
 
     if (verbose && !err) {
 	set_sample_label(datainfo);    
@@ -1672,10 +1691,10 @@ drag_data_received  (GtkWidget *widget,
 
     strcpy(tryfile, tmp);
 
-    if (probably_script_file(tmp)) {
-	do_open_script();
-    } else if (probably_session_file(tmp)) {
+    if (gretl_is_pkzip_file(tmp)) {
 	verify_open_session();
+    } else if (probably_script_file(tmp)) {
+	do_open_script();
     } else {
 	verify_open_data(NULL, 0);
     }
