@@ -41,9 +41,7 @@
 
 #define GNULL (gconstpointer) NULL
 
-#ifdef USE_GTKSOURCEVIEW
-# include <gtksourceview/gtksourceview.h>
-#endif
+#include <gtksourceview/gtksourceview.h>
 
 #include "gretltypes.h"
 #include "gui_utils.h"
@@ -68,7 +66,6 @@ extern PATHS paths;
 extern MODEL **models;
 
 /* global counters */
-extern int plot_count;
 extern int orig_vars;
 
 /* global state variables */
@@ -77,7 +74,6 @@ extern char *storelist;
 extern float gui_scale;
 
 /* global filenames */
-extern char cmdfile[MAXLEN];
 extern char scriptfile[MAXLEN];
 extern char sessionfile[MAXLEN];
 extern char tryfile[MAXLEN];
@@ -127,10 +123,9 @@ int mdata_selection_count (void);
 int mdata_active_var (void);
 void populate_varlist (void);
 void clear_varlist (GtkWidget *widget);
-int gui_restore_sample (void);
+int gui_restore_sample (double ***pZ, DATAINFO **ppdinfo);
 
 /* functions defined in files other than gretl.c */
 void about_dialog (gpointer data);
-void stats_calculator (gpointer p, guint code, GtkWidget *w);
 
 #endif /* GRETL_H */
