@@ -72,6 +72,7 @@ enum extra_cmds {
     VAR_DECOMP,
     IRF_BOOT,
     HTEST, 
+    HTESTNP,
     MODEL_RESTR,
     SYS_RESTR,
     VECM_RESTR,
@@ -82,10 +83,15 @@ enum extra_cmds {
     VLAGSEL,
     VAROMIT,
     DEFINE_LIST,
+    DEFINE_MATRIX,
     PANEL_WLS,
     PANEL_MODE,
     TSPLOTS,
     ITERATIONS,
+    CUSUMSQ,
+    PANEL_B,
+    IMPORT,
+    BOOTSTRAP,
     GUI_CMD_MAX
 };
 
@@ -130,15 +136,14 @@ enum file_ops {
     SAVE_SCRIPT,
     SAVE_OUTPUT,
     SAVE_SESSION,
-    SAVE_GNUPLOT,
     SAVE_BOXPLOT_EPS,
     SAVE_BOXPLOT_PS,
     SAVE_BOXPLOT_XPM,
-    SAVE_LAST_GRAPH,
-    SAVE_THIS_GRAPH,
+    SAVE_GNUPLOT,
     SAVE_GP_CMDS,
     SAVE_CONSOLE,
     SAVE_FUNCTIONS,
+    SAVE_BOOT_DATA,
     SET_PROG,
     SET_DIR,
     FILE_OP_MAX
@@ -151,7 +156,6 @@ enum browser_codes {
     PS_FILES,
     FUNC_FILES,
     REMOTE_FUNC_FILES,
-    FUNC_EDIT,
     NATIVE_DB,
     REMOTE_DB,
     NATIVE_SERIES,
@@ -173,13 +177,6 @@ enum stat_codes {
     HQC
 };
 
-enum exec_codes {
-    CONSOLE_EXEC      = 1 << 0,
-    SCRIPT_EXEC       = 1 << 1,
-    SESSION_EXEC      = 1 << 2,
-    INCLUDE_EXEC      = 1 << 3
-};
-
 enum clipstuff {
     TARGET_STRING,
     TARGET_TEXT,
@@ -187,7 +184,7 @@ enum clipstuff {
     TARGET_RTF
 };
 
-enum data_status {
+enum data_status_flags {
     HAVE_DATA     = 1 << 0,
     BOOK_DATA     = 1 << 1,
     USER_DATA     = 1 << 2,
@@ -225,7 +222,10 @@ enum font_selections {
 enum calc_functions {
     CALC_PVAL,
     CALC_DIST,
-    CALC_TEST
+    CALC_TEST,
+    CALC_NPTEST,
+    CALC_GRAPH,
+    CALC_GRAPH_ADD
 };
 
 enum auto_dummies {
@@ -244,9 +244,21 @@ enum random_types {
     RANDOM_UNIFORM,
     RANDOM_NORMAL,
     RANDOM_CHISQ,
-    RANDOM_ST
+    RANDOM_ST,
+    RANDOM_BIN,
+    RANDOM_POIS
 };
 
+enum pdists {
+    NORMAL_DIST,
+    T_DIST,
+    CHISQ_DIST,
+    F_DIST,
+    DW_DIST,
+    BINOMIAL_DIST,
+    POISSON_DIST
+};    
+
 #define MULTI_FORMAT_ENABLED(c) (c == SUMMARY || c == VAR_SUMMARY || \
                                  c == ALL_SUMMARY || \
 	                         c == CORR || c == ALL_CORR || c == FCASTERR || \
