@@ -20,9 +20,7 @@
 #ifndef GRETLTYPES_H
 #define GRETLTYPES_H
 
-#ifdef USE_GTKSOURCEVIEW
-# include <gtksourceview/gtksourceview.h>
-#endif
+#include <gtksourceview/gtksourceview.h>
 
 #define GRETL_STOCK_TEX    "gretl-tex"
 #define GRETL_STOCK_MAIL   "gretl-mail"
@@ -30,6 +28,8 @@
 #define GRETL_STOCK_BOX    "gretl-boxplot"
 #define GRETL_STOCK_PDF    "gretl-pdf"
 #define GRETL_STOCK_BOOK   "gretl-book"
+#define GRETL_STOCK_INFO   "gretl-info"
+#define GRETL_STOCK_EDIT   "gretl-edit"
 
 enum windata_flags {
     VWIN_HELP_ACTIVE  = 1 << 0,
@@ -57,9 +57,7 @@ struct _windata_t {
     int n_gretl_children;
     unsigned char flags;
     char fname[MAXLEN];
-#ifdef USE_GTKSOURCEVIEW
     GtkSourceBuffer *sbuf;
-#endif
 };
 
 #define window_is_busy(w)    (w->flags & VWIN_BUSY)
