@@ -25,22 +25,84 @@
 #include "guiprint.h"
 #include "gpt_control.h"
 #include "menustate.h"
+#include "gretl_www.h"
 
 #include <gdk/gdkwin32.h>
 #include <dirent.h>
 #include <mapi.h>
 #include <shlobj.h>
+#include <shellapi.h>
+#include <fcntl.h>
 
-#define HUSH_RUNTIME_WARNINGS
+#define MAX_CONSOLE_LINES 500
 
 extern int wimp; /* settings.c */
-extern int ws_startup (void);
 
-int create_child_process (char *prog, char *env) 
+void redirect_io_to_console (void)
+{
+    CONSOLE_SCREEN_BUFFER_INFO coninfo;
+    int conhandle;
+    long stdhandle;
+    FILE *fp;
+
+    AllocConsole();
+
+    GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE),
+			       &coninfo);
+
+    coninfo.dwSize.Y = MAX_CONSOLE_LINES;
+    SetConsoleScreenBufferSize(GetStdHandle(STD_OUTPUT_HANDLE),
+			       coninfo.dwSize);
+
+    /* redirect unbuffered STDOUT to the console */
+    stdhandle = (long) GetStdHandle(STD_OUTPUT_HANDLE);
+    conhandle = _open_osfhandle(stdhandle, _O_TEXT);
+    fp = _fdopen(conhandle, "w");
+    *stdout = *fp;
+    setvbuf(stdout, NULL, _IONBF, 0);
+
+    /* redirect unbuffered STDERR to the console */
+    stdhandle = (long) GetStdHandle(STD_ERROR_HANDLE);
+    conhandle = _open_osfhandle(stdhandle, _O_TEXT);
+    fp = _fdopen(conhandle, "w");
+    *stderr = *fp;
+    setvbuf(stderr, NULL, _IONBF, 0);
+}
+
+static void ws_cleanup (void)
+{
+    WSACleanup();
+}
+
+static int ws_startup (void)
+{
+    WORD requested;
+    WSADATA data;
+
+    requested = MAKEWORD(1, 1);
+
+    if (WSAStartup(requested, &data)) {
+	fprintf(stderr, I_("Couldn't find usable socket driver\n"));
+	return 1;
+    }
+
+    if (LOBYTE (requested) < 1 || (LOBYTE (requested) == 1 &&
+				   HIBYTE (requested) < 1)) {
+	fprintf(stderr, I_("Couldn't find usable socket driver\n"));
+	WSACleanup();
+	return 1;
+    }
+
+    atexit(ws_cleanup);
+
+    return 0;
+}
+
+int real_create_child_process (char *prog, int showerr) 
 { 
     PROCESS_INFORMATION proc_info; 
     STARTUPINFO start_info; 
-    int ret;
+    int ret, err = 0;
 
     ZeroMemory(&proc_info, sizeof proc_info);
     ZeroMemory(&start_info, sizeof start_info);
@@ -52,40 +114,42 @@ int create_child_process (char *prog, char *env)
 			NULL,          /* primary thread security attributes */ 
 			FALSE,         /* handles are inherited?  */
 			0,             /* creation flags  */
-			(LPVOID) env,  /* NULL => use parent's environment  */
+			(LPVOID) NULL, /* NULL => use parent's environment  */
 			NULL,          /* use parent's current directory  */
 			&start_info,   /* receives STARTUPINFO */ 
 			&proc_info);   /* receives PROCESS_INFORMATION  */
 
     if (ret == 0) {
-	DWORD dw = GetLastError();
-	win_show_error(dw);
+	if (showerr) {
+	    win_show_last_error();
+	}
+	err = 1;
     }
 
 #ifdef CHILD_DEBUG
-    if (1) {
-	FILE *fp = fopen("gretlbug.txt", "w");
-
-	if (fp != NULL) {
-	    fprintf(fp, "gretl: create_child_process():\n"
-		    " prog='%s'\n env='%s'\n", prog, env);
-	    fprintf(fp, " return from CreateProcess() = %d\n", ret);
-	    fclose(fp);
-	}
+    if (err) {
+	fprintf(stderr, "gretl: create_child_process():\n"
+		" prog='%s'\n\n", prog);
+	fprintf(stderr, " return from CreateProcess() = %d\n", ret);
     }	
 #endif
 
-    return ret;
+    return err;
 }
 
-void startR (const char *Rcommand)
+int create_child_process (char *prog) 
+{
+    return real_create_child_process(prog, 1);
+}
+
+void startR (char *Rcommand)
 {
     char Rprofile[MAXLEN], Rdata[MAXLEN], Rline[MAXLEN];
     const char *supp1 = "--no-init-file";
     const char *supp2 = "--no-restore-data";
     int *list;
     FILE *fp;
-    int enverr;
+    int err = 0;
 
     if (!data_status) {
 	errbox(_("Please open a data file first"));
@@ -99,8 +163,8 @@ void startR (const char *Rcommand)
 	return;
     }
 
-    enverr = ! SetEnvironmentVariable("R_PROFILE", Rprofile);
-    if (enverr) {
+    err = ! SetEnvironmentVariable("R_PROFILE", Rprofile);
+    if (err) {
 	errbox(_("Couldn't set R_PROFILE environment variable"));
 	fclose(fp);
 	return;
@@ -122,6 +186,7 @@ void startR (const char *Rcommand)
     free(list);
 
     if (dataset_is_time_series(datainfo)) {
+	fputs("# load data from gretl\n", fp);
 	fputs("vnum <- as.double(R.version$major) + (as.double(R.version$minor) / 10.0)\n", fp);
 	fputs("if (vnum > 1.89) library(stats) else library(ts)\n", fp);
 	fprintf(fp, "source(\"%s\", echo=TRUE)\n", 
@@ -133,7 +198,9 @@ void startR (const char *Rcommand)
 	build_path(Rtmp, paths.userdir, "Rtmp", NULL);
 	fq = gretl_fopen(Rtmp, "w");
 	if (fq != NULL) {
-	    fprintf(fq, "gretldata <- read.table(\"%s\")\n", 
+	    fputs("# load data from gretl\n", fq);
+	    fputs("library(stats)\n", fq);
+	    fprintf(fq, "gretldata <- read.table(\"%s\", header=TRUE)\n", 
 		    slash_convert(Rdata, FROM_BACKSLASH));
 	    fprintf(fq, "attach(gretldata)\n");
 	    fclose(fq);
@@ -145,7 +212,27 @@ void startR (const char *Rcommand)
     fclose(fp);
 
     sprintf(Rline, "\"%s\" %s %s", Rcommand, supp1, supp2);
-    create_child_process(Rline, NULL);
+    err = real_create_child_process(Rline, 0);
+
+    if (err) {
+	/* failed: try registry for Rgui.exe path? */
+	char tmp[MAX_PATH] = {0}; 
+
+	err = read_reg_val(HKEY_LOCAL_MACHINE, "R-core\\R", "InstallPath", tmp);
+	if (err) {
+	    err = read_reg_val(HKEY_LOCAL_MACHINE, "R", "InstallPath", tmp);
+	}
+	if (!err) {
+	    strcat(tmp, "\\bin\\Rgui.exe");
+	    sprintf(Rline, "\"%s\" %s %s", tmp, supp1, supp2);
+	    err = real_create_child_process(Rline, 1);
+	    if (!err) {
+		/* got a good R path, so record it */
+		*Rcommand = '\0';
+		strncat(Rcommand, tmp, MAXSTR - 1);
+	    }
+	}
+    }
 }
 
 char *slash_convert (char *str, int which)
@@ -245,8 +332,6 @@ void win_help (void)
     }
 }
 
-#ifdef HUSH_RUNTIME_WARNINGS
-
 static void dummy_output_handler (const gchar *log_domain,
 				  GLogLevelFlags log_level,
 				  const gchar *message,
@@ -275,8 +360,6 @@ static void hush_warnings (void)
 		       NULL);
 }
 
-#endif /* HUSH_RUNTIME_WARNINGS */
-
 char *default_windows_menu_fontspec (void)
 {
     gchar *fontspec = NULL;
@@ -425,76 +508,13 @@ void gretl_win32_init (const char *progname)
 
     read_rc(); /* get config info from registry */
     set_gd_fontpath();
-
-# ifdef HUSH_RUNTIME_WARNINGS
     hush_warnings();
-# endif 
-
     ws_startup(); 
-    atexit(write_rc);
 }
 
-int gretl_mkdir (const char *path)
+void gretl_win32_debug (void)
 {
-    DIR *test;
-    int done;
-
-    test = opendir(path);
-    if (test != NULL) {
-	closedir(test);
-	return 0;
-    }
-
-    done = CreateDirectory(path, NULL);
-    
-    return !done;
-}
-
-void win32_make_user_dirs (void)
-{
-    char dirname[MAXLEN];
-    extern char *tramodir;
-    size_t n;
-
-    strcpy(dirname, paths.userdir);
-    n = strlen(dirname);
-
-    if (n > 0 && (dirname[n-1] == '\\' || dirname[n-1] == '/')) {
-	dirname[n-1] = '\0';
-    }
-
-    if (gretl_mkdir(dirname)) {
-	gchar *msg;
-
-	msg = g_strdup_printf("Couldn't open or create gretl "
-			      "user directory\n%s", paths.userdir);
-	errbox(msg);
-	g_free(msg);
-	return;
-    }
-
-    build_path(paths.x12adir, paths.userdir, "x12arima", NULL);
-    gretl_mkdir(paths.x12adir);
-
-    build_path(tramodir, paths.userdir, "tramo", NULL);
-    if (gretl_mkdir(tramodir)) return;
-
-    sprintf(dirname, "%s\\output", tramodir);
-    gretl_mkdir(dirname);
-
-    sprintf(dirname, "%s\\graph", tramodir);
-    if (gretl_mkdir(dirname)) return;
-
-    sprintf(dirname, "%s\\graph\\acf", tramodir);
-    gretl_mkdir(dirname);
-    sprintf(dirname, "%s\\graph\\filters", tramodir);
-    gretl_mkdir(dirname);
-    sprintf(dirname, "%s\\graph\\forecast", tramodir);
-    gretl_mkdir(dirname);
-    sprintf(dirname, "%s\\graph\\series", tramodir);
-    gretl_mkdir(dirname);
-    sprintf(dirname, "%s\\graph\\spectra", tramodir);
-    gretl_mkdir(dirname);
+    redirect_io_to_console();
 }
 
 static int win_copy_buf (const char *buf, int fmt, size_t buflen)
@@ -763,7 +783,7 @@ void win32_process_graph (GPT_SPEC *spec, int color, int dest)
     char plottmp[MAXLEN], plotline[MAXLEN];
     gchar *plotcmd = NULL;
     gchar *emfname = NULL;
-    int err, done_pt2 = 0;
+    int err;
 
     /* create temporary file to hold the special gnuplot commands */
     if (user_fopen("gptout.tmp", plottmp, &prn)) {
@@ -782,11 +802,17 @@ void win32_process_graph (GPT_SPEC *spec, int color, int dest)
     pprintf(prn, "%s\n", get_gretl_emf_term_line(spec->code, color));
     emfname = g_strdup_printf("%sgpttmp.emf", paths.userdir);
     pprintf(prn, "set output '%s'\n", emfname);
+
     while (fgets(plotline, MAXLEN-1, fq)) {
-	if (!done_pt2 && strstr(plotline, "using 1:2")) {
-	    done_pt2 = maybe_switch_emf_point_style(plotline, prn);
-	} else if (strncmp(plotline, "set term", 8) && 
-	    strncmp(plotline, "set output", 10)) {
+	if (!strncmp(plotline, "set term", 8) ||
+	    !strncmp(plotline, "set output", 10)) {
+	    continue;
+	}
+	if (!color && strstr(plotline, "set style fill solid")) {
+	    pputs(prn, "set style fill solid 0.3\n");
+	} else if (html_encoded(plotline)) {
+	    pprint_as_latin(prn, plotline, 1);
+	} else {
 	    pputs(prn, plotline);
 	}
     }
@@ -836,15 +862,15 @@ static long GetRegKey (HKEY key, char *subkey, char *retdata)
     return err;
 }
 
-int browser_open (const char *url)
+static int win32_open_arg (const char *arg, char *ext)
 {
     char key[MAX_PATH + MAX_PATH];
     int err = 0;
 
-    if ((long) ShellExecute(NULL, "open", url, NULL, NULL, SW_SHOW) <= 32) {
-	/* if the above fails, get the .htm regkey and 
+    if ((long) ShellExecute(NULL, "open", arg, NULL, NULL, SW_SHOW) <= 32) {
+	/* if the above fails, get the appropriate fileext regkey and 
 	   look up the program */
-	if (GetRegKey(HKEY_CLASSES_ROOT, ".htm", key) == ERROR_SUCCESS) {
+	if (GetRegKey(HKEY_CLASSES_ROOT, ext, key) == ERROR_SUCCESS) {
 	    lstrcat(key,"\\shell\\open\\command");
 	    if (GetRegKey(HKEY_CLASSES_ROOT, key, key) == ERROR_SUCCESS) {
 		char *p;
@@ -864,7 +890,7 @@ int browser_open (const char *url)
 		}
 
 		lstrcat(p, " ");
-		lstrcat(p, url);
+		lstrcat(p, arg);
 		if (WinExec(key, SW_SHOW) < 32) err = 1;
 	    }
 	}
@@ -872,3 +898,60 @@ int browser_open (const char *url)
 
     return err;
 }
+
+int browser_open (const char *url)
+{
+    char sfx[5] = ".htm";
+
+    return win32_open_arg(url, sfx);
+}
+
+int win32_open_file (const char *fname)
+{
+    char sfx[5];
+    int err = 0;
+
+    if (has_suffix(fname, ".pdf")) {
+	strcpy(sfx, ".pdf");
+    } else {
+	strcpy(sfx, ".ps");
+    }
+
+    err = win32_open_arg(fname, sfx);
+
+    if (err) {
+	win_show_last_error();
+    }
+
+    return err;
+}
+
+int win32_delete_dir (const char *path)
+{
+    SHFILEOPSTRUCT op;
+    char *from;
+    int err = 0;
+
+    from = calloc(strlen(path) + 2, 1);
+    if (from == NULL) {
+	return E_ALLOC;
+    }
+
+    strcpy(from, path);
+
+    op.hwnd = NULL;
+    op.wFunc = FO_DELETE;
+    op.pFrom = from;
+    op.pTo = NULL;
+    op.fFlags = FOF_SILENT | FOF_NOCONFIRMATION | FOF_NOERRORUI;
+    op.fAnyOperationsAborted = FALSE;
+    op.hNameMappings = NULL;
+    op.lpszProgressTitle = NULL;
+
+    err = SHFileOperation(&op);
+
+    free(from);
+
+    return err;
+}
+
