@@ -20,6 +20,11 @@
 #ifndef GRETLWIN32_H
 #define GRETLWIN32_H
 
+#include <sys/types.h>
+#include <dirent.h>
+
+#include "plotspec.h"
+
 enum {
     TO_BACKSLASH,
     FROM_BACKSLASH
@@ -30,9 +35,9 @@ enum {
     WIN32_TO_PRINTER
 };
 
-int create_child_process (char *prog, char *env);
+int create_child_process (char *prog);
 
-void startR (const char *Rcommand);
+void startR (char *Rcommand);
 
 char *slash_convert (char *str, int which);
 
@@ -46,9 +51,9 @@ void win_help (void);
 
 void gretl_win32_init (const char *progname);
 
-const char *get_network_cfg_filename (void);
+void gretl_win32_debug (void);
 
-void win32_make_user_dirs (void);
+const char *get_network_cfg_filename (void);
 
 int prn_to_clipboard (PRN *prn, int copycode);
 
@@ -62,4 +67,8 @@ void win32_process_graph (GPT_SPEC *spec, int color, int dest);
 
 int browser_open (const char *url);
 
+int win32_open_file (const char *fname);
+
+int win32_delete_dir (const char *path);
+
 #endif /* GRETLWIN32_H */
