@@ -130,36 +130,18 @@ gchar *my_filename_from_utf8 (char *fname)
     return fname;
 }
 
-#define TR_DEBUG 0
-
-static gchar *
-real_locale_from_utf8 (const gchar *src, int force)
+gchar *my_locale_from_utf8 (const gchar *src)
 {
+    const gchar *cset;
     gchar *trstr;
     gsize bytes;
     GError *err = NULL;
-    const gchar *cset = NULL;
 
-#if TR_DEBUG
-    gchar *msg;
-    int u = g_get_charset(&cset);
-    msg = g_strdup_printf("real_locale_from_utf8: force=%d, "
-			  "g_get_charset returned %d and gave "
-			  "'%s'\n", force, u, 
-			  (cset != NULL)? cset : "NULL");
-    infobox(msg);
-    g_free(msg);
-
-    if (!force && u) {
+    if (g_get_charset(&cset)) {
+	/* g_get_charset returns TRUE if the returned 
+	   charset is UTF-8 */ 
 	return g_strdup(src);
     }
-#else
-    if (!force && g_get_charset(&cset)) {
-	/* According to the glib manual, g_get_charset returns TRUE if 
-	   the returned charset is UTF-8 */ 
-	return g_strdup(src);
-    }
-#endif
 
     trstr = g_locale_from_utf8(src, -1, NULL, &bytes, &err);
 
@@ -175,16 +157,6 @@ real_locale_from_utf8 (const gchar *src, int force)
     return trstr;
 }
 
-gchar *my_locale_from_utf8 (const gchar *src)
-{
-    return real_locale_from_utf8(src, 0);
-}
-
-gchar *force_locale_from_utf8 (const gchar *src)
-{
-    return real_locale_from_utf8(src, 1);
-}
-
 #define FNAME_DEBUG 0
 
 gchar *my_filename_to_utf8 (char *fname)
@@ -224,13 +196,22 @@ gchar *my_filename_to_utf8 (char *fname)
     return fname;
 }
 
-gchar *my_locale_to_utf8 (const gchar *src)
+static gchar *real_my_locale_to_utf8 (const gchar *src,
+				      int starting)
 {
+    static int errcount;
     gchar *trstr;
     gsize bytes;
     GError *err = NULL;
 
-    trstr = g_locale_to_utf8(src, -1, NULL, &bytes, &err);
+    if (starting) {
+	errcount = 0;
+	trstr = g_locale_to_utf8(src, -1, NULL, &bytes, &err);
+    } else if (errcount == 0) {
+	trstr = g_locale_to_utf8(src, -1, NULL, &bytes, &err);
+    } else {
+	trstr = g_locale_to_utf8(src, -1, NULL, &bytes, NULL);
+    }
 
     if (err != NULL) {
 	const gchar *cset = NULL;
@@ -242,7 +223,132 @@ gchar *my_locale_to_utf8 (const gchar *src)
 	    errbox("g_locale_to_utf8 failed; so did g_get_charset");
 	}
 	g_error_free(err);
+	errcount++;
     }
 
     return trstr;
 }
+
+static const char *gp_cset (void)
+{
+    if (iso_latin_version() == 2) {
+#ifdef GO_OS_WIN32
+	return "CP1250";
+#else
+	return "ISO-8859-2";
+#endif
+    } else {
+	return "ISO-8859-1";
+    } 
+}
+
+static gchar *real_gp_locale_to_utf8 (const gchar *src,
+				      int starting)
+{
+    static int errcount;
+    static const char *cset;
+    gchar *trstr;
+    gsize read;
+    gsize wrote;
+    GError *err = NULL;
+
+    if (cset == NULL) {
+	cset = gp_cset();
+    }
+
+    if (starting) {
+	errcount = 0;
+	trstr = g_convert(src, -1, "UTF-8", cset,
+			  &read, &wrote, &err);
+    } else if (errcount == 0) {
+	trstr = g_convert(src, -1, "UTF-8", cset,
+			  &read, &wrote, &err);
+    } else {
+	trstr = g_convert(src, -1, "UTF-8", cset,
+			  &read, &wrote, NULL);
+    }
+
+    if (err != NULL) {
+	errbox("g_convert (character set conversion) failed");
+	g_error_free(err);
+	errcount++;
+    }
+
+    return trstr;
+}
+
+gchar *my_locale_to_utf8 (const gchar *src)
+{
+    return real_my_locale_to_utf8(src, 1);
+}
+
+gchar *my_locale_to_utf8_next (const gchar *src)
+{
+    return real_my_locale_to_utf8(src, 0);
+}
+
+gchar *gp_locale_to_utf8 (const gchar *src)
+{
+    return real_gp_locale_to_utf8(src, 1);
+}
+
+gchar *gp_locale_to_utf8_next (const gchar *src)
+{
+    return real_gp_locale_to_utf8(src, 0);
+}
+
+gchar *gp_locale_from_utf8 (const gchar *src)
+{
+    static const char *cset;
+    gchar *trstr;
+    gsize read;
+    gsize wrote;
+
+    if (gretl_is_ascii(src)) {
+	return NULL;
+    }
+
+    if (cset == NULL) {
+	cset = gp_cset();
+    }
+
+    trstr = g_convert(src, -1, cset, "UTF-8",
+		      &read, &wrote, NULL);
+
+    return trstr;
+}
+
+gchar *latin1_to_utf8 (const gchar *src)
+{
+    gsize read, wrote;
+
+    return g_convert(src, -1, "UTF-8", "ISO-8859-1",
+		     &read, &wrote, NULL);
+}
+
+gchar *latin2_to_utf8 (const gchar *src)
+{
+    gsize read, wrote;
+
+#ifdef G_OS_WIN32
+    return g_convert(src, -1, "UTF-8", "CP1250",
+		     &read, &wrote, NULL);
+#else
+    return g_convert(src, -1, "UTF-8", "ISO-8859-2",
+		     &read, &wrote, NULL);
+#endif
+}
+
+int html_encoded (const char *s)
+{
+    while (*s) {
+	if (*s == '&' && *(s+1) == '#' && isdigit(*(s+2))) {
+	    return 1;
+	}
+	s++;
+    }
+
+    return 0;
+}
+
+
