@@ -21,11 +21,13 @@
 
 #include "gretl.h"
 #include "textbuf.h"
-#include "webget.h"
+#include "gretl_www.h"
 #include "treeutils.h"
 #include "dlgutils.h"
 
-#ifndef G_OS_WIN32
+#ifdef G_OS_WIN32
+# include "gretlwin32.h"
+#else
 # include <sys/stat.h>
 # include <sys/types.h>
 # include <fcntl.h>
@@ -46,7 +48,8 @@ struct help_head_t {
     int *topics;       /* array of topics under heading by command number */
     char **topicnames; /* array of descriptive topic names (GUI help only) */
     int *pos;          /* array of byte offsets into file for topics */
-    int ntopics;       /* number of topics under this heading */
+    int ntalloc;       /* number of topics initally allocated */
+    int ntopics;       /* actual number of topics under this heading */
 };
 
 static help_head **cli_heads, **gui_heads;
@@ -104,6 +107,7 @@ static struct gui_help_item gui_help_items[] = {
     { HCCME,          "hccme" },
     { IRF_BOOT,       "irfboot" },
     { HTEST,          "gui-htest" },
+    { HTESTNP,        "gui-htest-np" },
     { MODEL_RESTR,    "restrict-model" },
     { SYS_RESTR,      "restrict-system" },
     { VECM_RESTR,     "restrict-vecm" },
@@ -114,6 +118,8 @@ static struct gui_help_item gui_help_items[] = {
     { VAROMIT,        "VAR-omit" },
     { PANEL_MODE,     "panel-mode" },
     { PANEL_WLS,      "panel-wls" },
+    { PANEL_B,        "panel-between" },
+    { BOOTSTRAP,      "bootstrap" },
     { -1,          NULL },
 };
 
@@ -357,7 +363,7 @@ static void free_help_head (help_head *head)
     free(head->pos);
     
     if (head->topicnames != NULL) {
-	for (i=0; i<head->ntopics; i++) {
+	for (i=0; i<head->ntalloc; i++) {
 	    free(head->topicnames[i]);
 	}
 	free(head->topicnames);
@@ -368,17 +374,19 @@ static void free_help_head (help_head *head)
 
 static int head_allocate_topicnames (help_head *head)
 {
-    int i;
+    int i, n = head->ntopics;
 
-    head->topicnames = malloc(head->ntopics * sizeof *head->topicnames);
+    head->topicnames = malloc(n * sizeof *head->topicnames);
     
     if (head->topicnames == NULL) {
 	return 1;
     }
 
-    for (i=0; i<head->ntopics; i++) {
+    for (i=0; i<n; i++) {
 	head->topicnames[i] = NULL;
     }
+    
+    head->ntalloc = n;
 
     return 0;
 }
@@ -411,15 +419,15 @@ static help_head *help_head_new (const char *name, int nt, int gui)
     return head;
 }
 
-static help_head **allocate_heads (int nh)
+static help_head **allocate_heads (int n)
 {
     help_head **heads;
     int i;
 
-    heads = malloc(nh * sizeof *heads);
+    heads = malloc(n * sizeof *heads);
 
     if (heads != NULL) {
-	for (i=0; i<nh; i++) {
+	for (i=0; i<n; i++) {
 	    heads[i] = NULL;
 	}
     }
@@ -431,7 +439,9 @@ static void free_help_heads (help_head **heads, int nh)
 {
     int i;
 
-    if (heads == NULL) return;
+    if (heads == NULL) {
+	return;
+    }
 
     for (i=0; i<nh; i++) {
 	if (heads[i] != NULL) {
@@ -442,31 +452,28 @@ static void free_help_heads (help_head **heads, int nh)
     free(heads);
 }
 
-static void add_topic_to_head (help_head *head, int j, const char *word,
+static void add_topic_to_head (help_head *head, int *i, const char *word,
 			       char *label, int gui)
 {
     int hnum = help_topic_number(word, gui);
 
     if (hnum > 0) {
-	head->topics[j] = hnum;
+	head->topics[*i] = hnum;
 	if (label != NULL && head->topicnames != NULL) {
-	    head->topicnames[j] = label;
+	    head->topicnames[*i] = label;
 	}
+	*i += 1;
+#if HDEBUG
+	fprintf(stderr, "add_topic_to_head: topic %d: word = %s: hnum = %d\n", 
+		*i, word, hnum);
+#endif
     } else {
 	fprintf(stderr, "helpfile error: word='%s' not recognized\n", word);
-#if 0
-	head->topics[j] = 0;
-	head->topicnames[j] = NULL;
-#endif
+	head->ntopics -= 1;
 	if (label != NULL) {
 	    free(label);
 	}
     }
-
-#if HDEBUG
-    fprintf(stderr, "add_topic_to_head: topic %d: word = %s: hnum = %d\n", 
-	    j+1, word, hnum);
-#endif
 }
 
 static int 
@@ -476,7 +483,7 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
     FILE *fp;
     char line[128];
     char tmp[32];
-    int i, j, nh2;
+    int i, j, k, nh2;
     int nh = 0;
     int err = 0;
 
@@ -504,7 +511,7 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
     }
 
 #if HDEBUG
-    fprintf(stderr, "Found got %d topic headings\n", nh);
+    fprintf(stderr, "Found %d topic headings\n", nh);
 #endif
 
     nh2 = 0;
@@ -536,7 +543,7 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
 	}
 
 #if HDEBUG
-	fprintf(stderr, "Heading %d (%s): got %d topics\n", nh2+1, tmp, nt);
+	fprintf(stderr, "Heading %d (\"%s\"): got %d topics\n", nh2+1, tmp, nt);
 #endif
 
 	/* heading with at least one topic */
@@ -546,6 +553,8 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
 	    err = 1;
 	}
 
+	k = 0;
+
 	/* get topics under heading */
 	for (j=0; j<nt && !err; j++) {
 	    char *label = NULL;
@@ -566,7 +575,7 @@ get_helpfile_structure (help_head ***pheads, int gui, const char *fname)
 	    }
 
 	    if (!err) {
-		add_topic_to_head(heads[nh2], j, tmp, label, gui);
+		add_topic_to_head(heads[nh2], &k, tmp, label, gui);
 	    }
 	}
 
@@ -725,7 +734,9 @@ static int gui_pos_from_cmd (int cmd, int en)
     }
 
     /* special for gui-specific help items */
+
     helpstr = help_string_from_cmd(cmd);
+
     if (helpstr != NULL) {
 	int altcode = extra_command_number(helpstr);
 
@@ -865,7 +876,7 @@ static void add_help_topics (windata_t *hwin, int cli, int en)
 				    hds[i]->topicnames[j]);
 #if HDEBUG
 		fprintf(stderr, "(1) Built help topic path from\n"
-			" '%s', '%s' and '%s'\n", mpath, headname),
+			" '%s', '%s' and '%s'\n", mpath, headname,
 			hds[i]->topicnames[j]);
 #endif
 	    } else {
@@ -877,7 +888,7 @@ static void add_help_topics (windata_t *hwin, int cli, int en)
 					gretl_command_word(tnum));
 #if HDEBUG
 		    fprintf(stderr, "(2) Built help topic path from\n"
-			    " '%s', '%s' and '%s'\n", mpath, headname),
+			    " '%s', '%s' and '%s'\n", mpath, headname,
 			    gretl_command_word(tnum));
 #endif
 		} else if (!cli) {
@@ -1104,7 +1115,23 @@ gint edit_script_help (GtkWidget *widget, GdkEventButton *b,
 	    if (!gtk_text_iter_ends_word(&iter)) {
 		gtk_text_iter_forward_word_end(&w_end);
 	    }
+
 	    text = gtk_text_buffer_get_text(buf, &w_start, &w_end, FALSE);
+
+	    /* "coint2" command! */
+	    if (text != NULL && !strcmp(text, "coint") && 
+		gtk_text_iter_forward_char(&w_end)) {
+		gchar *s = gtk_text_buffer_get_text(buf, &w_start, &w_end, FALSE);
+
+		if (s != NULL) {
+		    if (!strcmp(s, "coint2")) {
+			g_free(text);
+			text = s;
+		    } else {
+			g_free(s);
+		    }
+		}
+	    }
 	} 
 
 	if (text != NULL && *text != '\0') {
@@ -1208,7 +1235,7 @@ static gboolean real_find_in_text (GtkTextView *view, const gchar *s,
 	return 0;
     }
 
-    while (1) {
+    while (!found) {
 	got = gtk_text_buffer_get_text(buf, &start, &end, FALSE);
 	if (g_ascii_strcasecmp(got, s) == 0) {
 	    found = 1;
@@ -1245,7 +1272,7 @@ static void find_in_text (GtkWidget *widget, gpointer data)
     gboolean found;
 
     needle = gtk_editable_get_chars(GTK_EDITABLE(find_entry), 0, -1);
-    if (*needle == '\0') {
+    if (needle == NULL || *needle == '\0') {
 	return;
     }
 
@@ -1256,24 +1283,38 @@ static void find_in_text (GtkWidget *widget, gpointer data)
     }
 }
 
-static void find_in_listbox (GtkWidget *w, gpointer data)
+static void 
+get_tree_model_haystack (GtkTreeModel *mod, GtkTreeIter *iter, int col,
+			 char *haystack)
 {
-    int found = 0, wrapped = 0, minvar = 0;
-    gchar *tmp; 
-    char haystack[MAXLEN];
-    windata_t *win;
-    gchar *pstr;
-    GtkTreeModel *model;
-    GtkTreeIter iter, iterhere;
+    gchar *tmp;
 
-    win = (windata_t *) g_object_get_data(G_OBJECT(data), "windat");
-    if (win == mdata) {
-	/* searching in the main gretl window: start on line 1, not 0 */
-	minvar = 1;
+    gtk_tree_model_get(mod, iter, col, &tmp, -1);
+    if (tmp != NULL) {
+	strcpy(haystack, tmp);
+	lower(haystack);
+	g_free(tmp);
+    } else {
+	*haystack = '\0';
     }
+}
 
-    if (needle) {
+static void find_in_listbox (GtkWidget *w, gpointer p)
+{
+    windata_t *win = g_object_get_data(G_OBJECT(p), "windat");
+    int minvar, wrapped = 0;
+    char haystack[MAXLEN];
+    char pstr[16];
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    int found = -1;
+
+    /* if searching in the main gretl window, start on line 1 */
+    minvar = (win == mdata)? 1 : 0;
+
+    if (needle != NULL) {
 	g_free(needle);
+	needle = NULL;
     }
 
     needle = gtk_editable_get_chars(GTK_EDITABLE(find_entry), 0, -1);
@@ -1281,38 +1322,36 @@ static void find_in_listbox (GtkWidget *w, gpointer data)
 
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(win->listbox));
 
-    /* try searching downward from the current line plus one */
-    pstr = g_strdup_printf("%d", win->active_var);
-    gtk_tree_model_get_iter_from_string (model, &iter, pstr);
-    g_free(pstr);
-    iterhere = iter;
-    if (!gtk_tree_model_iter_next(model, &iter)) {
-	iter = iterhere;
+    /* if possible, try searching downward from the current line 
+       plus one; failing start, start from the top */
+    sprintf(pstr, "%d", win->active_var);
+    if (!gtk_tree_model_get_iter_from_string(model, &iter, pstr) ||
+	!gtk_tree_model_iter_next(model, &iter)) {
+	gtk_tree_model_get_iter_first(model, &iter);
     }
 
  search_wrap:
 
-    while (1) {
+    while (found < 0) {
 	/* try looking in column 1 first */
-	gtk_tree_model_get(model, &iter, 1, &tmp, -1);
-	strcpy(haystack, tmp);
-	g_free(tmp);
-	lower(haystack);
+	get_tree_model_haystack(model, &iter, 1, haystack);
+
 	found = look_for_string(haystack, needle, 0);
-	if (found >= 0) break;
-	if (win == mdata) {
-	    /* then column 2 */
-	    gtk_tree_model_get(model, &iter, 2, &tmp, -1);
-	} else {
-	    /* then column 0 */
-	    gtk_tree_model_get(model, &iter, 0, &tmp, -1);
+
+	if (found < 0) {
+	    if (win == mdata) {
+		/* then column 2 */
+		get_tree_model_haystack(model, &iter, 2, haystack);
+	    } else {
+		/* then column 0 */
+		get_tree_model_haystack(model, &iter, 0, haystack);
+	    }
+	    found = look_for_string(haystack, needle, 0);
+	}
+
+	if (found >= 0 || !gtk_tree_model_iter_next(model, &iter)) {
+	    break;
 	}
-	strcpy(haystack, tmp);
-	g_free(tmp);
-	lower(haystack);
-	found = look_for_string(haystack, needle, 0);
-	if (found >= 0) break;
-	if (!gtk_tree_model_iter_next(model, &iter)) break;
     }
 
     if (found < 0 && win->active_var > minvar && !wrapped) {
@@ -1335,7 +1374,9 @@ static void find_in_listbox (GtkWidget *w, gpointer data)
 				 path, NULL, FALSE);
 	win->active_var = tree_path_get_row_number(path);
 	gtk_tree_path_free(path);
-	if (wrapped) infobox(_("Search wrapped"));
+	if (wrapped) {
+	    infobox(_("Search wrapped"));
+	}
     } else {
 	infobox(_("String was not found."));
     }
@@ -1366,12 +1407,12 @@ static void parent_find (GtkWidget *finder, windata_t *caller)
     }
 }
 
-static void find_string_dialog (void (*findfunc)(), gpointer data)
+static void find_string_dialog (void (*findfunc)(), gpointer p)
 {
+    windata_t *mydat = (windata_t *) p;
     GtkWidget *label;
     GtkWidget *button;
     GtkWidget *hbox;
-    windata_t *mydat = (windata_t *) data;
 
     if (find_window != NULL) {
 	g_object_set_data(G_OBJECT(find_window), "windat", mydat);
@@ -1393,10 +1434,9 @@ static void find_string_dialog (void (*findfunc)(), gpointer data)
     gtk_widget_show (label);
     find_entry = gtk_entry_new();
 
-    if (needle) {
+    if (needle != NULL) {
 	gtk_entry_set_text(GTK_ENTRY(find_entry), needle);
-	gtk_editable_select_region(GTK_EDITABLE(find_entry), 0, 
-				   strlen(needle));
+	gtk_editable_select_region(GTK_EDITABLE(find_entry), 0, -1);
     }
 
     g_signal_connect(G_OBJECT(find_entry), "activate", 
@@ -1419,8 +1459,8 @@ static void find_string_dialog (void (*findfunc)(), gpointer data)
 		     G_CALLBACK(cancel_find), find_window);
     gtk_widget_show(button);
 
-    /* find button -- make this the default */
-    button = standard_button(GTK_STOCK_FIND);
+    /* find button */
+    button = gtk_button_new_from_stock(GTK_STOCK_FIND);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_container_add(GTK_CONTAINER(hbox), button);
     g_signal_connect(G_OBJECT(button), "clicked",
@@ -1515,12 +1555,9 @@ static int find_or_download_pdf (int uguide, int i, char *fullpath)
 	"gretl-ref-es.pdf"  
     };
     const char *fname;
-    char errtext[MAXLEN];
     FILE *fp;
     int err = 0;
 
-    *errtext = '\0';
-
     if (i < 0 || i > 3) {
 	i = 0;
     }
@@ -1548,12 +1585,14 @@ static int find_or_download_pdf (int uguide, int i, char *fullpath)
 
     /* do actual download */
     if (!err) {
-	err = retrieve_manfile(fname, fullpath, errtext);
+	err = retrieve_manfile(fname, fullpath);
     }
 
     if (err) {
-	if (*errtext) {
-	    errbox(errtext);
+	const char *buf = gretl_errmsg_get();
+
+	if (*buf) {
+	    errbox(buf);
 	} else {
 	    errbox(_("Failed to download file"));
 	}
@@ -1568,16 +1607,14 @@ void display_pdf_help (gpointer p, guint uguide, GtkWidget *w)
     int pref, err = 0;
 
     pref = get_manpref();
+
     err = find_or_download_pdf(uguide, pref, fname);
     if (err) {
 	return;
     }
 
 #if defined(G_OS_WIN32)
-    if ((int) ShellExecute(NULL, "open", fname, NULL, NULL, SW_SHOW) <= 32) {
-	DWORD dw = GetLastError();
-	win_show_error(dw);
-    }
+    win32_open_file(fname);
 #elif defined(OSX_BUILD)
     osx_open_file(fname);
 #else
