@@ -49,11 +49,7 @@ void library_command_init (void);
 
 void library_command_free (void);
 
-int replaying (void);
-
-void set_replay_on (void);
-
-void set_replay_off (void);
+CMD *get_lib_cmd (void);
 
 int gretl_command_sprintf (const char *template, ...);
 
@@ -65,11 +61,9 @@ int user_fopen (const char *fname, char *fullname, PRN **pprn);
 
 gint bufopen (PRN **pprn);
 
-void do_menu_op (gpointer data, guint action, GtkWidget *widget);
+void do_menu_op (gpointer data, guint action, GtkWidget *w);
 
-int do_two_var_test (selector *sr);
-
-void do_run_script (gpointer data, guint code, GtkWidget *w);
+void do_run_script (GtkWidget *w, gpointer data);
 
 void gui_errmsg (const int errcode);
 
@@ -77,11 +71,9 @@ void register_graph (void);
 
 /* sample-related functions */
 
-void set_sample_label_special (void);
-
 int bool_subsample (gretlopt opt);
 
-void do_samplebool (GtkWidget *widget, dialog_t *dlg);
+void do_samplebool (GtkWidget *w, dialog_t *dlg);
 
 int do_set_sample (void);
 
@@ -109,7 +101,9 @@ int get_original_n (void);
 
 int do_coint (selector *sr);
 
-void do_forecast (gpointer data, guint u, GtkWidget *w);
+void do_forecast (gpointer p, guint u, GtkWidget *w);
+
+void do_bootstrap (gpointer p, guint u, GtkWidget *w) ;
 
 int do_coeff_sum (selector *sr);
 
@@ -119,33 +113,35 @@ int do_VAR_omit (selector *sr);
 
 int do_confidence_region (selector *sr);
 
-void do_lmtest (gpointer data, guint aux_code, GtkWidget *widget);
+void do_lmtest (gpointer data, guint aux_code, GtkWidget *w);
 
-void do_autocorr (gpointer data, guint u, GtkWidget *widget);
+void do_autocorr (gpointer data, guint u, GtkWidget *w);
 
 void do_chow_cusum (gpointer data, guint action, GtkWidget *w);
 
-void do_reset (gpointer data, guint u, GtkWidget *widget);
+void do_reset (gpointer data, guint u, GtkWidget *w);
 
-void unit_root_test (gpointer data, guint u, GtkWidget *widget);
+void unit_root_test (gpointer data, guint u, GtkWidget *w);
 
-void do_arch (gpointer data, guint u, GtkWidget *widget);
+void do_arch (gpointer data, guint u, GtkWidget *w);
 
-void do_restrict (GtkWidget *widget, dialog_t *dlg);
+void do_restrict (GtkWidget *w, dialog_t *dlg);
 
-void do_nls_model (GtkWidget *widget, dialog_t *dlg);
+void do_nls_model (GtkWidget *w, dialog_t *dlg);
 
-void do_mle_model (GtkWidget *widget, dialog_t *dlg);
+void do_mle_model (GtkWidget *w, dialog_t *dlg);
 
-void do_eqn_system (GtkWidget *widget, dialog_t *dlg);
+void do_gmm_model (GtkWidget *w, dialog_t *dlg);
 
-void do_saved_eqn_system (GtkWidget *widget, dialog_t *dlg);
+void do_eqn_system (GtkWidget *w, dialog_t *dlg);
+
+void do_saved_eqn_system (GtkWidget *w, dialog_t *dlg);
 
 int do_model (selector *sr);
 
 int do_vector_model (selector *sr);
 
-void do_graph_model (GPT_SPEC *spec);
+void do_graph_model (const int *list, int fit);
 
 void do_gini (gpointer data, guint u, GtkWidget *w);
 
@@ -167,83 +163,79 @@ int out_of_sample_info (int add_ok, int *t2);
 
 /* variable-related functions */
 
-void do_minibuf (GtkWidget *widget, dialog_t *dlg);
+void do_minibuf (GtkWidget *w, dialog_t *dlg);
 
-void do_genr (GtkWidget *widget, dialog_t *dlg);
+void do_genr (GtkWidget *w, dialog_t *dlg);
 
-void do_model_genr (GtkWidget *widget, dialog_t *dlg);
+void do_model_genr (GtkWidget *w, dialog_t *dlg);
 
-void do_random_uniform (GtkWidget *widget, dialog_t *dlg);
+void add_rand_series (GtkWidget *w, dialog_t *dlg);
 
-void do_random_normal (GtkWidget *widget, dialog_t *dlg);
+void do_global_setmiss (GtkWidget *w, dialog_t *dlg);
 
-void do_random_chisq (GtkWidget *widget, dialog_t *dlg);
+void do_variable_setmiss (GtkWidget *w, dialog_t *dlg);
 
-void do_random_st (GtkWidget *widget, dialog_t *dlg);
-
-void do_global_setmiss (GtkWidget *widget, dialog_t *dlg);
-
-void do_variable_setmiss (GtkWidget *widget, dialog_t *dlg);
-
-void do_edit_label (GtkWidget *widget, dialog_t *dlg);
+void do_edit_label (GtkWidget *w, dialog_t *dlg);
 
 int do_rename_variable (int v, const char *newname, int full);
 
 int record_varlabel_change (int v);
 
-void do_resid_freq (gpointer data, guint action, GtkWidget *widget);
+void do_resid_freq (gpointer p, guint action, GtkWidget *w);
 
-void do_freqplot (gpointer data, guint gamma, GtkWidget *widget);
+void do_freqplot (gpointer p, guint gamma, GtkWidget *w);
 
-void do_corrgm (gpointer data, guint u, GtkWidget *widget);
+void do_corrgm (gpointer p, guint u, GtkWidget *w);
 
-void residual_correlogram (gpointer data, guint u, GtkWidget *widget);
+void residual_correlogram (gpointer p, guint u, GtkWidget *w);
 
-void do_pergm (gpointer data, guint opt, GtkWidget *widget);
+void do_pergm (gpointer p, guint opt, GtkWidget *w);
 
-void residual_periodogram (gpointer data, guint opt, GtkWidget *widget);
+void residual_periodogram (gpointer p, guint opt, GtkWidget *w);
 
 #if defined (HAVE_TRAMO) || defined (HAVE_X12A)
-void do_tramo_x12a (gpointer data, guint opt, GtkWidget *widget);
+void do_tramo_x12a (gpointer p, guint opt, GtkWidget *w);
 #endif
 
-void do_range_mean (gpointer data, guint opt, GtkWidget *widget);
+void do_range_mean (gpointer p, guint opt, GtkWidget *w);
 
-void do_hurst (gpointer data, guint opt, GtkWidget *widget);
+void do_hurst (gpointer p, guint opt, GtkWidget *w);
 
-void do_outcovmx (gpointer data, guint action, GtkWidget *widget);
+void do_outcovmx (gpointer p, guint u, GtkWidget *w);
 
-void add_dummies (gpointer data, guint action, GtkWidget *widget);
+void do_anova (gpointer p, guint u, GtkWidget *w);
 
-void add_index (gpointer data, guint tm, GtkWidget *widget);
+void add_dummies (gpointer p, guint action, GtkWidget *w);
 
-void do_add_obs (gpointer data, guint u, GtkWidget *widget);
+void add_index (gpointer p, guint tm, GtkWidget *w);
 
-void do_remove_obs (gpointer data, guint u, GtkWidget *widget);
+void do_add_obs (gpointer p, guint u, GtkWidget *w);
 
-void add_logs_etc (gpointer data, guint action, GtkWidget *widget);
+void do_remove_obs (gpointer p, guint u, GtkWidget *w);
 
-int add_system_resid (gpointer data, int eqnum, int ci);
+void add_logs_etc (gpointer p, guint action, GtkWidget *w);
+
+int add_system_resid (gpointer p, int eqnum, int ci);
 
 int add_fit_resid (MODEL *pmod, int code, int undo);
 
-int add_system_resid (gpointer data, int eqnum, int ci);
+int add_system_resid (gpointer p, int eqnum, int ci);
 
 void add_model_stat (MODEL *pmod, int which);
 
-void resid_plot (gpointer data, guint xvar, GtkWidget *widget);
+void resid_plot (gpointer p, guint xvar, GtkWidget *w);
 
-void fit_actual_plot (gpointer data, guint xvar, GtkWidget *widget);
+void fit_actual_plot (gpointer p, guint xvar, GtkWidget *w);
 
-void fit_actual_splot (gpointer data, guint u, GtkWidget *widget);
+void fit_actual_splot (gpointer p, guint u, GtkWidget *w);
 
-void display_fit_resid (gpointer data, guint code, GtkWidget *widget);
+void display_fit_resid (gpointer p, guint code, GtkWidget *w);
 
 void do_graph_var (int varnum);
 
 void do_boxplot_var (int varnum);
 
-void ts_plot_var (gpointer data, guint opt, GtkWidget *widget);
+void ts_plot_var (gpointer p, guint opt, GtkWidget *w);
 
 int do_scatters (selector *sr);
 
@@ -251,17 +243,19 @@ int do_graph_from_selector (selector *sr);
 
 int do_splot_from_selector (selector *sr);
 
-void plot_from_selection (gpointer data, guint action, GtkWidget *widget);
+void plot_from_selection (gpointer p, guint action, GtkWidget *w);
 
-void do_box_graph (GtkWidget *widget, dialog_t *dlg);
+void do_box_graph (GtkWidget *w, dialog_t *dlg);
 
 int do_dummy_graph (selector *sr);
 
+int do_xcorrgm (selector *sr);
+
 void delete_selected_vars (void);
 
 void delete_single_var (int id);
 
-void display_selected (gpointer data, guint action, GtkWidget *widget);
+void display_selected (gpointer p, guint action, GtkWidget *w);
 
 void display_var (void);
 
@@ -269,9 +263,9 @@ void display_var (void);
 
 void do_open_script (void);
 
-void open_info (gpointer data, guint edit, GtkWidget *widget);
+void open_info (gpointer p, guint edit, GtkWidget *w);
 
-void do_new_script (gpointer data, guint action, GtkWidget *widget);
+void do_new_script (gpointer p, guint action, GtkWidget *w);
 
 void do_open_csv_box (char *fname, int code, int append);
 
@@ -283,11 +277,7 @@ void save_latex (PRN *prn, const char *fname);
 
 void do_save_text (char *fname, MODEL *pmod);
 
-int execute_script (const char *runfile, const char *buf,
-		    PRN *prn, int exec_code);
-
-int gui_exec_line (char *line, PRN *prn, int exec_code, 
-		   const char *myname); 
+int gui_exec_line (ExecState *s, double ***pZ, DATAINFO **ppdinfo);
 
 int check_and_record_command (void);
 
