@@ -94,9 +94,8 @@ void variable_menu_state (gboolean s)
     if (mdata == NULL || mdata->ifac == NULL) return;
 
     flip(mdata->ifac, "/Variable", s);
-    flip(mdata->ifac, "/View/Correlation matrix", !s);
-    flip(mdata->ifac, "/View/Principal components", !s);
-    flip(mdata->ifac, "/View/Mahalanobis distances", !s);
+    flip(mdata->ifac, "/View/Cross-correlogram",  
+	 dataset_is_time_series(datainfo));
 }
 
 void main_menubar_state (gboolean s)
@@ -131,6 +130,7 @@ static GtkItemFactoryEntry time_series_model_items[] = {
     { N_("/Model/Time series/_Prais-Winsten..."), NULL, model_callback, PWE, NULL, GNULL },
     { N_("/Model/Time series/_Autoregressive estimation..."), NULL, model_callback, AR, NULL, GNULL },
     { N_("/Model/Time series/ARI_MA..."), NULL, model_callback, ARMA, NULL, GNULL },
+    { N_("/Model/Time series/_ARCH..."), NULL, model_callback, ARCH, NULL, GNULL },
     { N_("/Model/Time series/_GARCH..."), NULL, model_callback, GARCH, NULL, GNULL },
     { N_("/Model/Time series/_Vector Autoregression..."), NULL, selector_callback, VAR, NULL, GNULL },
     { N_("/Model/Time series/VAR _lag selection..."), NULL, selector_callback, VLAGSEL, NULL, GNULL },
@@ -144,7 +144,9 @@ static GtkItemFactoryEntry time_series_model_items[] = {
 
 static GtkItemFactoryEntry panel_model_items[] = {
     { N_("/Model/Panel/_Fixed or random effects..."), NULL, model_callback, PANEL, NULL, GNULL },
-    { N_("/Model/Panel/_Weighted least squares..."), NULL, model_callback, PANEL_WLS, NULL, GNULL }
+    { N_("/Model/Panel/_Weighted least squares..."), NULL, model_callback, PANEL_WLS, NULL, GNULL },
+    { N_("/Model/Panel/_Between model..."), NULL, model_callback, PANEL_B, NULL, GNULL },
+    { N_("/Model/Panel/_Arellano-Bond..."), NULL, model_callback, ARBOND, NULL, GNULL },
 };
 
 #define COMPACTABLE(d) (d->structure == TIME_SERIES && \
@@ -160,12 +162,15 @@ static GtkItemFactoryEntry panel_model_items[] = {
 			(d)->structure == SPECIAL_TIME_SERIES || \
                         (d)->structure == STACKED_TIME_SERIES)
 
-#define seasonal_ts(d) ((d)->structure == TIME_SERIES && (d->pd == 4 || d->pd == 12))
+#define tramo_ts(d) ((d)->structure == TIME_SERIES && \
+                     (d->pd == 1 || d->pd == 4 || d->pd == 12))
+
+#define x12_ts(d) ((d)->structure == TIME_SERIES && \
+                   (d->pd == 4 || d->pd == 12))
 
 void time_series_menu_state (gboolean s)
 {
     gboolean sx = extended_ts(datainfo);
-    gboolean ss = seasonal_ts(datainfo);
 
     if (mdata->ifac == NULL) {
 	return;
@@ -182,15 +187,14 @@ void time_series_menu_state (gboolean s)
     /* Variable menu */
     flip(mdata->ifac, "/Variable/Correlogram", s);
     flip(mdata->ifac, "/Variable/Spectrum", s);
-    flip(mdata->ifac, "/Variable/Runs test", s);
     flip(mdata->ifac, "/Variable/Augmented Dickey-Fuller test", s);
     flip(mdata->ifac, "/Variable/KPSS test", s);
     flip(mdata->ifac, "/Variable/Filter", s);
 #ifdef HAVE_X12A
-    flip(mdata->ifac, "/Variable/X-12-ARIMA analysis", ss);
+    flip(mdata->ifac, "/Variable/X-12-ARIMA analysis", x12_ts(datainfo));
 #endif
 #ifdef HAVE_TRAMO
-    flip(mdata->ifac, "/Variable/TRAMO analysis", ss);
+    flip(mdata->ifac, "/Variable/TRAMO analysis", tramo_ts(datainfo));
 #endif
     flip(mdata->ifac, "/Variable/Hurst exponent", s);
     /* Model menu */
@@ -338,8 +342,6 @@ static gint var_popup_click (GtkWidget *w, gpointer p)
 	unit_root_test(NULL, ADF, NULL);
     else if (!strcmp(item, _("KPSS test"))) 
 	unit_root_test(NULL, KPSS, NULL);
-    else if (!strcmp(item, _("Runs test"))) 
-	do_menu_op(NULL, RUNS, NULL);
     else if (!strcmp(item, _("Hurst exponent"))) 
 	do_hurst(NULL, 0, NULL);
     else if (!strcmp(item, _("Edit attributes")))  
@@ -409,13 +411,15 @@ static gint selection_popup_click (GtkWidget *w, gpointer p)
 	do_menu_op(NULL, SUMMARY, NULL);
     else if (!strcmp(item, _("Correlation matrix"))) 
 	do_menu_op(NULL, CORR, NULL);
+    else if (!strcmp(item, _("Cross-correlogram"))) 
+	xcorrgm_callback(NULL, 0, NULL);
     else if (!strcmp(item, _("Time series plot"))) 
 	plot_from_selection(NULL, GR_PLOT, NULL);
     else if (!strcmp(item, _("XY scatterplot"))) 
 	plot_from_selection(NULL, GR_XY, NULL);
     else if (!strcmp(item, _("Copy to clipboard"))) 
 	csv_selected_to_clipboard();
-    else if (!strcmp(item, _("Edit"))) 
+    else if (!strcmp(item, _("Edit values"))) 
 	show_spreadsheet(SHEET_EDIT_VARLIST);
     else if (!strcmp(item, _("Delete"))) 
 	delete_selected_vars();
@@ -431,10 +435,11 @@ GtkWidget *build_selection_popup (void)
 	N_("Display values"),
 	N_("Descriptive statistics"),
 	N_("Correlation matrix"),
+	N_("Cross-correlogram"),
 	N_("Time series plot"),
 	N_("XY scatterplot"),
 	N_("Copy to clipboard"),
-	N_("Edit"),
+	N_("Edit values"),
 	N_("Delete")
     };
 
@@ -445,7 +450,10 @@ GtkWidget *build_selection_popup (void)
     sel_menu = gtk_menu_new();
 
     for (i=0; i<n_items; i++) {
-	if (!extended_ts(datainfo) && i == 3) {
+	if (!dataset_is_time_series(datainfo) && i == 3) {
+	    continue;
+	}
+	if (!extended_ts(datainfo) && i == 4) {
 	    continue;
 	}
 	item = gtk_menu_item_new_with_label(_(items[i]));
@@ -514,8 +522,15 @@ void set_sample_label (DATAINFO *pdinfo)
     flip(mdata->ifac, "/Data/Transpose data...", 
 	 !dataset_is_panel(pdinfo));
 
-    sprintf(labeltxt, _("%s: Full range %s - %s"), 
-	    pdstr, stobs, endobs);
+    if (complex_subsampled() && pdinfo->t1 == 0 && 
+	pdinfo->t2 == pdinfo->n - 1 && 
+	datainfo->structure == CROSS_SECTION) {
+	sprintf(labeltxt, _("Undated: Full range n = %d; current sample"
+			    " n = %d"), get_full_length_n(), datainfo->n);
+    } else {
+	sprintf(labeltxt, _("%s: Full range %s - %s"), 
+		pdstr, stobs, endobs);
+    }
 
     if (pdinfo->t1 > 0 || pdinfo->t2 < pdinfo->n - 1) {
 	char t1str[OBSLEN], t2str[OBSLEN];
@@ -547,9 +562,12 @@ void set_sample_label (DATAINFO *pdinfo)
 	gtk_label_set_text(GTK_LABEL(dlabel), labeltxt);
     }
 
+    if (complex_subsampled() || pdinfo->t1 > 0 ||
+	pdinfo->t2 < pdinfo->n - 1) {
+	restore_sample_state(TRUE);
+    }
+
     console_record_sample(datainfo);
 }
 
 
-
-
