@@ -23,15 +23,20 @@
 #include "model_table.h"
 #include "session.h"
 #include "textutil.h"
+#include "texprint.h"
 
 static MODEL **table_models;
 static int n_models;
-static int *grand_list;
+static char **pnames;
+static int n_params;
 static int use_tstats;
+static int depvarnum;
+static int colwidth;
 
 static void print_rtf_row_spec (PRN *prn, int tall);
 
-#define MAX_TABLE_MODELS 6
+#define MAX_PORTRAIT_MODELS 6
+#define MAX_TABLE_MODELS 12
 
 static void mtable_errmsg (char *msg, int gui)
 {
@@ -69,8 +74,12 @@ int in_model_table (const MODEL *pmod)
 {
     int i;
 
+    if (pmod == NULL || n_models == 0) {
+	return 0;
+    }
+
     for (i=0; i<n_models; i++) {
-	if (pmod == table_models[i]) {
+	if (pmod == table_models[i] || pmod->ID == table_models[i]->ID) {
 	    return 1;
 	}
     }
@@ -83,6 +92,11 @@ int model_table_n_models (void)
     return n_models;
 }
 
+int model_table_landscape (void)
+{
+    return n_models > MAX_PORTRAIT_MODELS;
+}
+
 MODEL *model_table_model_by_index (int i)
 {
     if (i >= 0 && i < n_models) {
@@ -106,8 +120,10 @@ void clear_model_table (PRN *prn)
     free(table_models);
     table_models = NULL;
 
-    free(grand_list);
-    grand_list = NULL;
+    free_strings_array(pnames, n_params);
+    pnames = NULL;
+    n_params = 0;
+
     n_models = 0;
     
     if (prn != NULL) {
@@ -129,16 +145,17 @@ static int model_table_depvar (void)
     return -1;
 }
 
-int add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
+static int model_table_precheck (MODEL *pmod, int add_mode)
 {
     int gui = (add_mode != MODEL_ADD_BY_CMD);
 
     if (pmod == NULL) {
 	return 1;
-    }
+    }    
 
-    /* NLS and MLE models won't work */
-    if (pmod->ci == NLS || pmod->ci == MLE) {
+    /* NLS, MLE and GMM models won't work */
+    if (pmod->ci == NLS || pmod->ci == MLE || pmod->ci == GMM ||
+	pmod->ci == ARBOND) {
 	mtable_errmsg(_("Sorry, this model can't be put in the model table"),
 		      gui);
 	return 1;
@@ -158,16 +175,8 @@ int add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
 	return 1;
     }    
 
-    /* is the list started or not? */
-    if (n_models == 0) {
-	table_models = mymalloc(sizeof *table_models);
-	if (table_models == NULL) {
-	    return 1;
-	}
-	n_models = 1;
-    } else {
+    if (n_models > 0) {
 	int dv = model_table_depvar();
-	MODEL **mods;
 
 	/* check that the dependent variable is in common */
 	if (pmod->list[1] != dv) {
@@ -186,6 +195,22 @@ int add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
 	if (model_table_too_many(gui)) {
 	    return 1;
 	}
+    }
+
+    return 0;
+}
+
+static int real_add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
+{
+    /* is the list started or not? */
+    if (n_models == 0) {
+	table_models = mymalloc(sizeof *table_models);
+	if (table_models == NULL) {
+	    return 1;
+	}
+	n_models = 1;
+    } else {
+	MODEL **mods;
 
 	n_models++;
 	mods = myrealloc(table_models, n_models * sizeof *mods);
@@ -209,31 +234,26 @@ int add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
 	pputc(prn, '\n');
     }
 
+    mark_session_changed();
+
     return 0;
 }
 
-static int var_is_in_model (int v, const MODEL *pmod)
+int add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
 {
-    int i;
-
-    for (i=2; i<=pmod->list[0]; i++) {
-	if (pmod->list[i] == LISTSEP) {
-	    break;
-	}
-	if (v == pmod->list[i]) {
-	    return i;
-	}
+    if (model_table_precheck(pmod, add_mode)) {
+	return 1;
     }
 
-    return 0;    
+    return real_add_to_model_table(pmod, add_mode, prn);
 }
 
-static int on_grand_list (int v)
+static int on_param_list (const char *pname)
 {
     int i;
 
-    for (i=2; i<=grand_list[0]; i++) {
-	if (v == grand_list[i]) {
+    for (i=0; i<n_params; i++) {
+	if (!strcmp(pname, pnames[i])) {
 	    return 1;
 	}
     }
@@ -241,68 +261,43 @@ static int on_grand_list (int v)
     return 0;
 }
 
-static void add_to_grand_list (const int *list)
+static int add_to_param_list (const MODEL *pmod)
 {
-    int i, j = grand_list[0] + 1;
+    char pname[VNAMELEN];
+    int i, err = 0;
 
-    for (i=2; i<=list[0]; i++) {
-	if (!on_grand_list(list[i])) {
-	    grand_list[0] += 1;
-	    grand_list[j++] = list[i];
-	}
-    }
-}
-
-static int real_list_length (int *list)
-{
-    int i;
-
-    for (i=1; i<=list[0]; i++) {
-	if (list[i] == LISTSEP) {
-	    return i - 1;
+    for (i=0; i<pmod->ncoeff && !err; i++) {
+	gretl_model_get_param_name(pmod, datainfo, i, pname);
+	if (!on_param_list(pname)) {
+	    err = strings_array_add(&pnames, &n_params, pname);
 	}
     }
 
-    return list[0];
+    return err;
 }
 
-static int make_grand_varlist (void)
+static int make_full_param_list (void)
 {
-    int i, j, first = 1;
-    int l0 = 0;
     const MODEL *pmod;
+    int first = 1;
+    int i, err = 0;
 
-    free(grand_list);
+    free_strings_array(pnames, n_params);
+    pnames = NULL;
+    n_params = 0;
 
-    for (i=0; i<n_models; i++) {
-	if (table_models[i] != NULL) {
-	    l0 += real_list_length(table_models[i]->list);
-	}
-    }
-
-    grand_list = gretl_list_new(l0);
-    if (grand_list == NULL) {
-	return 1;
-    }
-
-    for (i=0; i<n_models; i++) {
+    for (i=0; i<n_models && !err; i++) {
 	if (table_models[i] != NULL) {
 	    pmod = table_models[i];
 	    if (first) {
-		for (j=0; j<=pmod->list[0]; j++) {
-		    if (pmod->list[j] == LISTSEP) {
-			break;
-		    }
-		    grand_list[j] = pmod->list[j];
-		}
+		depvarnum = gretl_model_get_depvar(pmod);
 		first = 0;
-	    } else {
-		add_to_grand_list(pmod->list);
-	    }
+	    } 
+	    err = add_to_param_list(pmod);
 	}
     }
 
-    return 0;
+    return err;
 }
 
 static int model_table_is_empty (void)
@@ -360,14 +355,30 @@ static int common_df (void)
     return 1;
 }
 
-static const char *short_estimator_string (int ci, PRN *prn)
+static const char *short_estimator_string (const MODEL *pmod, PRN *prn)
 {
-    if (ci == HSK) return N_("HSK");
-    else if (ci == CORC) return N_("CORC");
-    else if (ci == HILU) return N_("HILU");
-    else if (ci == PWE) return N_("PWE");
-    else if (ci == ARCH) return N_("ARCH");
-    else return estimator_string(ci, prn);
+    if (pmod->ci == HSK) return N_("HSK");
+    else if (pmod->ci == CORC) return N_("CORC");
+    else if (pmod->ci == HILU) return N_("HILU");
+    else if (pmod->ci == PWE) return N_("PWE");
+    else if (pmod->ci == ARCH) return N_("ARCH");
+    else if (pmod->ci == WLS) {
+	if (gretl_model_get_int(pmod, "iters")) {
+	    return N_("MLE");
+	} else {
+	    return N_("WLS");
+	}
+    } else if (pmod->ci == PANEL) {
+	if (gretl_model_get_int(pmod, "fixed-effects")) {
+	    return N_("Within");
+	} else if (gretl_model_get_int(pmod, "random-effects")) {
+	    return N_("GLS");
+	} else {
+	    return N_("Between");
+	}
+    } else {
+	return estimator_string(pmod, prn);
+    }
 }
 
 static const char *get_asts (double pval)
@@ -385,7 +396,7 @@ static const char *get_pre_asts (double pval)
     return (pval >= 0.1)? "" : (pval >= 0.05)? "$\\,$" : "$\\,\\,$";
 }
 
-static void print_model_table_coeffs (int nwidth, PRN *prn)
+static void print_model_table_coeffs (int namewidth, PRN *prn)
 {
     int i, j, k;
     const MODEL *pmod;
@@ -394,18 +405,19 @@ static void print_model_table_coeffs (int nwidth, PRN *prn)
     int rtf = rtf_format(prn);
 
     /* loop across all variables that appear in any model */
-    for (i=2; i<=grand_list[0]; i++) {
-	int v = grand_list[i];
-	int f = 1;
+
+    for (i=0; i<n_params; i++) {
+	char *pname = pnames[i];
+	int first = 1;
 
 	if (tex) {
-	    tex_escape(tmp, datainfo->varname[v]);
+	    tex_escape(tmp, pname);
 	    pprintf(prn, "%s ", tmp);
 	} else if (rtf) {
 	    print_rtf_row_spec(prn, 0);
-	    pprintf(prn, "\\intbl \\qc %s\\cell ", datainfo->varname[v]);
+	    pprintf(prn, "\\intbl \\qc %s\\cell ", pname);
 	} else {
-	    pprintf(prn, "%-*s ", nwidth, datainfo->varname[v]);
+	    pprintf(prn, "%-*s ", namewidth, pname);
 	}
 
 	/* print the coefficient estimates across a row */
@@ -414,9 +426,9 @@ static void print_model_table_coeffs (int nwidth, PRN *prn)
 	    if (pmod == NULL) {
 		continue;
 	    }
-	    if ((k = var_is_in_model(v, pmod))) {
-		double x = screen_zero(pmod->coeff[k-2]);
-		double s = screen_zero(pmod->sderr[k-2]);
+	    if ((k = gretl_model_get_param_number(pmod, datainfo, pname)) >= 0) {
+		double x = screen_zero(pmod->coeff[k]);
+		double s = screen_zero(pmod->sderr[k]);
 		double pval;
 		char numstr[32];
 
@@ -427,7 +439,7 @@ static void print_model_table_coeffs (int nwidth, PRN *prn)
 			pval = 0.0001;
 		    }
 		} else {
-		    pval = coeff_pval(pmod, x / s, pmod->dfd);
+		    pval = coeff_pval(pmod->ci, x / s, pmod->dfd);
 		}
 
 		sprintf(numstr, "%#.4g", x);
@@ -445,10 +457,10 @@ static void print_model_table_coeffs (int nwidth, PRN *prn)
 		    pprintf(prn, "\\qc %s%s\\cell ", numstr, get_asts(pval));
 		} else {
 		    /* note: strlen(asts) = 2 */
-		    pprintf(prn, "%*s%s", (f == 1)? 12 : 10,
+		    pprintf(prn, "%*s%s", (first)? colwidth : colwidth - 2,
 			    numstr, get_asts(pval));
 		}
-		f = 0;
+		first = 0;
 	    } else {
 		/* variable not present in this column */
 		if (tex) {
@@ -456,7 +468,7 @@ static void print_model_table_coeffs (int nwidth, PRN *prn)
 		} else if (rtf) {
 		    pputs(prn, "\\qc \\cell ");
 		} else {
-		    pputs(prn, "            "); /* 12 */
+		    bufspace(colwidth, prn);
 		}
 	    }
 	}
@@ -471,24 +483,24 @@ static void print_model_table_coeffs (int nwidth, PRN *prn)
 	    pputs(prn, "\\intbl ");
 	} else {
 	    pputc(prn, '\n');
-	    bufspace(nwidth + 2, prn);
+	    bufspace(namewidth + 2, prn);
 	}
 
 	/* print the t-stats or standard errors across a row */
-	f = 1;
+	first = 1;
 	for (j=0; j<n_models; j++) {
 	    pmod = table_models[j];
 	    if (pmod == NULL) {
 		continue;
 	    }
-	    if ((k = var_is_in_model(v, pmod))) {
+	    if ((k = gretl_model_get_param_number(pmod, datainfo, pname)) >= 0) {
 		char numstr[32];
 		double val;
 
 		if (use_tstats) {
-		    val = pmod->coeff[k-2] / pmod->sderr[k-2];
+		    val = pmod->coeff[k] / pmod->sderr[k];
 		} else {
-		    val = pmod->sderr[k-2];
+		    val = pmod->sderr[k];
 		}
 
 		sprintf(numstr, "%#.4g", val);
@@ -501,14 +513,14 @@ static void print_model_table_coeffs (int nwidth, PRN *prn)
 			pprintf(prn, "& \\footnotesize{(%s)} ", numstr);
 		    }
 		} else if (rtf) {
-		    if (f == 1) {
+		    if (first) {
 			pputs(prn, "\\qc \\cell ");
 		    }
 		    pprintf(prn, "\\qc (%s)\\cell ", numstr);
-		    f = 0;
+		    first = 0;
 		} else {
 		    sprintf(tmp, "(%s)", numstr);
-		    pprintf(prn, "%12s", tmp);
+		    pprintf(prn, "%*s", colwidth, tmp);
 		}
 	    } else {
 		/* variable not present in this column */
@@ -517,7 +529,7 @@ static void print_model_table_coeffs (int nwidth, PRN *prn)
 		} else if (rtf) {
 		    pputs(prn, "\\qc \\cell ");
 		} else {
-		    pputs(prn, "            "); /* 12 */
+		    bufspace(colwidth, prn);
 		}
 	    }
 	}
@@ -564,10 +576,10 @@ static int any_r_squared (void)
     return 0;
 }
 
-static void print_n_r_squared (int wid, PRN *prn, int *binary)
+static void print_n_r_squared (int width0, PRN *prn, int *binary)
 {
-    int same_df, any_R2, any_ll;
     const MODEL *pmod;
+    int same_df, any_R2, any_ll;
     int tex = tex_format(prn);
     int rtf = rtf_format(prn);
     int j;
@@ -581,7 +593,7 @@ static void print_n_r_squared (int wid, PRN *prn, int *binary)
     } else if (rtf) {
 	pprintf(prn, "\\intbl \\qc %s\\cell ", _("n"));
     } else {
-	pprintf(prn, "%*s", wid, _("n"));
+	pprintf(prn, "%*s", width0, _("n"));
     }
 
     for (j=0; j<n_models; j++) {
@@ -592,7 +604,7 @@ static void print_n_r_squared (int wid, PRN *prn, int *binary)
 	    } else if (rtf) {
 		pprintf(prn, "\\qc %d\\cell ", pmod->nobs);
 	    } else {
-		pprintf(prn, "%12d", pmod->nobs);
+		pprintf(prn, "%*d", colwidth, pmod->nobs);
 	    }
 	}
     }
@@ -617,7 +629,8 @@ static void print_n_r_squared (int wid, PRN *prn, int *binary)
 	    pprintf(prn, "\\qc %s\\cell ", 
 		    (same_df)? "R{\\super 2}" : _("Adj. R{\\super 2}"));
 	} else {
-	    pprintf(prn, "%*s", wid, (same_df)? _("R-squared") : _("Adj. R**2"));
+	    pprintf(prn, "%*s", width0, (same_df)? _("R-squared") : 
+		    _("Adj. R**2"));
 	}
 
 	for (j=0; j<n_models; j++) {
@@ -639,7 +652,7 @@ static void print_n_r_squared (int wid, PRN *prn, int *binary)
 		} else if (rtf) {
 		    pprintf(prn, "\\qc %.4f\\cell ", pmod->rsq);
 		} else {
-		    pprintf(prn, "%#12.4g", pmod->rsq);
+		    pprintf(prn, "%#*.4g", colwidth, pmod->rsq);
 		}
 	    } else {
 		double rsq = (same_df)? pmod->rsq : pmod->adjrsq;
@@ -649,7 +662,7 @@ static void print_n_r_squared (int wid, PRN *prn, int *binary)
 		} else if (rtf) {
 		    pprintf(prn, "\\qc %.4f\\cell ", rsq);
 		} else {
-		    pprintf(prn, "%#12.4g", rsq);
+		    pprintf(prn, "%#*.4g", colwidth, rsq);
 		}
 	    }
 	}
@@ -674,7 +687,7 @@ static void print_n_r_squared (int wid, PRN *prn, int *binary)
 	} else if (rtf) {
 	    pputs(prn, "\\qc lnL\\cell ");
 	} else {
-	    pprintf(prn, "%*s", wid, "lnL");
+	    pprintf(prn, "%*s", width0, "lnL");
 	}
 
 	for (j=0; j<n_models; j++) {
@@ -698,7 +711,7 @@ static void print_n_r_squared (int wid, PRN *prn, int *binary)
 		} else if (rtf) {
 		    pprintf(prn, "\\qc %.3f\\cell ", pmod->lnL);
 		} else {
-		    pprintf(prn, "%#12.6g", pmod->lnL);
+		    pprintf(prn, "%#*.6g", colwidth, pmod->lnL);
 		}
 	    }
 	}
@@ -713,12 +726,12 @@ static void print_n_r_squared (int wid, PRN *prn, int *binary)
     }
 }
 
-static int grand_list_namelen (void)
+static int max_namelen (void)
 {
     int i, len, maxlen = 8;
 
-    for (i=2; i<=grand_list[0]; i++) {
-	len = strlen(datainfo->varname[grand_list[i]]);
+    for (i=0; i<n_params; i++) {
+	len = strlen(pnames[i]);
 	if (len > maxlen) {
 	    maxlen = len;
 	}
@@ -740,11 +753,12 @@ int display_model_table (int gui)
 	return 1;
     }
 
-    if (make_grand_varlist()) {
+    if (make_full_param_list()) {
 	return 1;
     }
 
-    namelen = grand_list_namelen();
+    namelen = max_namelen();
+    colwidth = 13;
 
     if (bufopen(&prn)) {
 	clear_model_table(NULL);
@@ -756,12 +770,11 @@ int display_model_table (int gui)
     if (ci > 0) {
 	/* all models use same estimation procedure */
 	pprintf(prn, _("%s estimates"), 
-		_(estimator_string(ci, prn)));
+		_(estimator_string(table_models[0], prn)));
 	pputc(prn, '\n');
     }
 
-    pprintf(prn, _("Dependent variable: %s\n"),
-	    datainfo->varname[grand_list[1]]);
+    pprintf(prn, _("Dependent variable: %s\n"), datainfo->varname[depvarnum]);
 
     pputc(prn, '\n');
     bufspace(namelen + 4, prn);
@@ -769,10 +782,16 @@ int display_model_table (int gui)
     for (j=0; j<n_models; j++) {
 	char modhd[32];
 
-	if (table_models[j] != NULL) {
-	    sprintf(modhd, _("Model %d"), table_models[j]->ID);
-	    print_centered(modhd, 12, prn);
+	if (table_models[j] == NULL) {
+	    continue;
 	}
+	if (table_models[j]->name != NULL) {
+	    *modhd = '\0';
+	    strncat(modhd, table_models[j]->name, 31);
+	} else {
+	    sprintf(modhd, _("Model %d"), table_models[j]->ID);
+	}
+	print_centered(modhd, colwidth, prn);
     }
     pputc(prn, '\n');
     
@@ -783,9 +802,8 @@ int display_model_table (int gui)
 	for (j=0; j<n_models; j++) {
 	    if (table_models[j] != NULL) {
 		strcpy(est, 
-		       _(short_estimator_string(table_models[j]->ci,
-						prn)));
-		print_centered(est, 12, prn);
+		       _(short_estimator_string(table_models[j], prn)));
+		print_centered(est, colwidth, prn);
 	    }
 	}
 	pputc(prn, '\n');
@@ -831,12 +849,10 @@ static int tex_print_model_table (PRN *prn)
 	return 1;
     }
 
-    if (make_grand_varlist()) {
+    if (make_full_param_list()) {
 	return 1;
     }
 
-    gretl_print_set_format(prn, GRETL_FORMAT_TEX);
-
     ci = common_estimator();
 
     pputs(prn, "\\begin{center}\n");
@@ -844,27 +860,34 @@ static int tex_print_model_table (PRN *prn)
     if (ci > 0) {
 	/* all models use same estimation procedure */
 	pprintf(prn, I_("%s estimates"), 
-		I_(estimator_string(ci, prn)));
+		I_(estimator_string(table_models[0], prn)));
 	pputs(prn, "\\\\\n");
     }
 
-    tex_escape(tmp, datainfo->varname[grand_list[1]]);
+    tex_escape(tmp, datainfo->varname[depvarnum]);
     pprintf(prn, "%s: %s \\\\\n", I_("Dependent variable"), tmp);
 
     pputs(prn, "\\vspace{1em}\n\n");
-    pputs(prn, "\\begin{tabular}{l");
+    pputs(prn, "\\begin{longtable}{l");
     for (j=0; j<n_models; j++) {
 	pputs(prn, "c");
     }
     pputs(prn, "}\n");
 
     for (j=0; j<n_models; j++) {
-	char modhd[32];
+	char modhd[48];
 
 	if (table_models[j] == NULL) {
 	    continue;
 	}
-	sprintf(modhd, I_("Model %d"), table_models[j]->ID);
+	if (table_models[j]->name != NULL) {
+	    *modhd = '\0';
+	    strncat(modhd, table_models[j]->name, 16);
+	    tex_escape(tmp, modhd);
+	    strcpy(modhd, tmp);
+	} else {
+	    sprintf(modhd, I_("Model %d"), table_models[j]->ID);
+	}
 	pprintf(prn, " & %s ", modhd);
     }
     pputs(prn, "\\\\ ");
@@ -879,8 +902,7 @@ static int tex_print_model_table (PRN *prn)
 		continue;
 	    }
 	    strcpy(est, 
-		   I_(short_estimator_string(table_models[j]->ci,
-					     prn)));
+		   I_(short_estimator_string(table_models[j], prn)));
 	    pprintf(prn, " & %s ", est);
 	}
 	pputs(prn, "\\\\ ");
@@ -891,7 +913,7 @@ static int tex_print_model_table (PRN *prn)
     print_model_table_coeffs(0, prn);
     print_n_r_squared(0, prn, &binary);
 
-    pputs(prn, "\\end{tabular}\n\n");
+    pputs(prn, "\\end{longtable}\n\n");
     pputs(prn, "\\vspace{1em}\n");
 
     if (use_tstats) {
@@ -938,7 +960,9 @@ static int rtf_print_model_table (PRN *prn)
 	return 1;
     }
 
-    if (make_grand_varlist()) return 1;
+    if (make_full_param_list()) {
+	return 1;
+    }
 
     gretl_print_set_format(prn, GRETL_FORMAT_RTF);
 
@@ -950,13 +974,12 @@ static int rtf_print_model_table (PRN *prn)
 	/* all models use same estimation procedure */
 	pputs(prn, "\\par \\qc ");
 	pprintf(prn, I_("%s estimates"), 
-		I_(estimator_string(ci, prn)));
+		I_(estimator_string(table_models[0], prn)));
 	pputc(prn, '\n');
     }
 
     pprintf(prn, "\\par \\qc %s: %s\n\\par\n\\par\n{", 
-	    I_("Dependent variable"),
-	    datainfo->varname[grand_list[1]]);
+	    I_("Dependent variable"), datainfo->varname[depvarnum]);
 
     /* RTF row stuff */
     print_rtf_row_spec(prn, 1);
@@ -965,8 +988,15 @@ static int rtf_print_model_table (PRN *prn)
     for (j=0; j<n_models; j++) {
 	char modhd[32];
 
-	if (table_models[j] == NULL) continue;
-	sprintf(modhd, I_("Model %d"), table_models[j]->ID);
+	if (table_models[j] == NULL) {
+	    continue;
+	}
+	if (table_models[j]->name != NULL) {
+	    *modhd = '\0';
+	    strncat(modhd, table_models[j]->name, 31);
+	} else {
+	    sprintf(modhd, I_("Model %d"), table_models[j]->ID);
+	}
 	pprintf(prn, "\\qc %s\\cell ", modhd);
     }
     pputs(prn, "\\intbl \\row\n");
@@ -979,7 +1009,7 @@ static int rtf_print_model_table (PRN *prn)
 	for (j=0; j<n_models; j++) {
 	    if (table_models[j] == NULL) continue;
 	    strcpy(est, 
-		   I_(short_estimator_string(table_models[j]->ci, prn)));
+		   I_(short_estimator_string(table_models[j], prn)));
 	    pprintf(prn, "\\qc %s\\cell ", est);
 	}
 	pputs(prn, "\\intbl \\row\n");
@@ -1018,7 +1048,48 @@ int special_print_model_table (PRN *prn)
     }
 }
 
-int modeltab_parse_line (const char *line, MODEL *pmod, PRN *prn)
+static int cli_modeltab_add (PRN *prn)
+{
+    GretlObjType type;
+    void *ptr = get_last_model(&type);
+    int err = 0;
+
+    if (type != GRETL_OBJ_EQN) {
+	gretl_errmsg_set(_("No model is available"));
+	err = 1;
+    } else {
+	MODEL *pmod = (MODEL *) ptr;
+	MODEL *cpy = NULL;
+	int freeit = 0;
+
+	err = model_table_precheck(pmod, MODEL_ADD_BY_CMD);
+	if (err) {
+	    return err;
+	}
+
+	cpy = get_model_by_ID(pmod->ID);
+	if (cpy == NULL) {
+	    cpy = gretl_model_copy(pmod);
+	    if (cpy == NULL) {
+		err = E_ALLOC;
+	    } else {
+		freeit = 1;
+	    }
+	}
+
+	if (!err) {
+	    err = real_add_to_model_table(cpy, MODEL_ADD_BY_CMD, prn);
+	}
+
+	if (err && freeit) {
+	    gretl_model_free(cpy);
+	}
+    }
+
+    return err;
+}
+
+int modeltab_parse_line (const char *line, PRN *prn)
 {
     char cmdword[9];
     int err = 0;
@@ -1028,12 +1099,7 @@ int modeltab_parse_line (const char *line, MODEL *pmod, PRN *prn)
     }
 
     if (!strcmp(cmdword, "add")) {
-	if (pmod == NULL || pmod->ID == 0) {
-	    gretl_errmsg_set(_("No model is available"));
-	    err = 1;
-	} else {
-	    err = add_to_model_table(pmod, MODEL_ADD_BY_CMD, prn);
-	}
+	err = cli_modeltab_add(prn);
     } else if (!strcmp(cmdword, "show")) {
 	err = display_model_table(0);
     } else if (!strcmp(cmdword, "free")) {
