@@ -34,7 +34,7 @@ int display_model_table (int gui);
 
 int special_print_model_table (PRN *prn);
 
-int modeltab_parse_line (const char *line, MODEL *pmod, PRN *prn);
+int modeltab_parse_line (const char *line, PRN *prn);
 
 void model_table_dialog (void);
 
@@ -42,6 +42,8 @@ int in_model_table (const MODEL *pmod);
 
 int model_table_n_models (void);
 
+int model_table_landscape (void);
+
 MODEL *model_table_model_by_index (int i);
 
 #endif /* MODEL_TABLE_H */
