@@ -43,15 +43,17 @@ enum {
 
 int save_session (char *fname);
 
-int session_is_saved (void);
-
-void set_session_saved (int val);
+int session_is_modified (void);
 
 void session_menu_state (gboolean s);
 
+const char *get_session_dirname (void);
+
 int real_add_text_to_session (PRN *prn, const char *tname);
 
-void add_graph_to_session (gpointer data, guint code, GtkWidget *w);
+int add_graph_to_session (char *fname, char *fullname);
+
+void add_boxplot_to_session (const char *boxtmp);
 
 int cli_add_graph_to_session (const char *fname, const char *gname,
 			      GretlObjType type);
@@ -66,11 +68,13 @@ void session_model_callback (void *ptr, int action);
 
 void *get_session_object_by_name (const char *name, GretlObjType *type);
 
+int session_matrix_destroy_by_name (const char *name, PRN *prn);
+
 void delete_text_from_session (void *p);
 
 void display_saved_text (void *p);
 
-int session_changed (int set);
+void mark_session_changed (void);
 
 void session_init (void);
 
@@ -80,7 +84,7 @@ void gui_clear_dataset (void);
 
 void verify_clear_data (void);
 
-void close_session (void);
+void close_session (ExecState *s, double ***pZ, DATAINFO **ppdinfo);
 
 void free_session (void);
 
@@ -106,4 +110,6 @@ void disable_graph_page (void);
 
 void display_session_graph_by_data (void *p);
 
+void view_matrix_properties (const gretl_matrix *m, const char *name);
+
 #endif /* SESSION_H */
