@@ -60,16 +60,12 @@ static int restore_session_graphs (xmlNodePtr node)
 	}
 
 	if (!err) {
-	    SESSION_GRAPH *graph;
-	    
-	    graph = session_graph_new((const char *) name, 
-				      (const char *) fname, 
-				      ID, type);
-	    if (graph == NULL) {
-		err = 1;
-	    } else {
-		err = session_append_graph(graph);
-	    }
+	    SESSION_GRAPH *sg;
+
+	    sg = session_append_graph((const char *) name,
+				      (const char *) fname,
+				      type);
+	    err = (sg == NULL);
 	}
 
 	if (!err) {
@@ -110,15 +106,8 @@ static int restore_session_texts (xmlNodePtr node, xmlDocPtr doc)
 	    if (buf == NULL) {
 		err = 1;
 	    } else {
-		SESSION_TEXT *text;
-
-		text = session_text_new((const char *) name,
-					(char *) buf);
-		if (text == NULL) {
-		    err = 1;
-		} else {
-		    err = session_append_text(text);
-		}
+		err = session_append_text((const char *) name,
+					  (char *) buf);
 	    }
 	}
 
@@ -257,6 +246,9 @@ static int restore_session_models (xmlNodePtr node, xmlDocPtr doc)
 	if (!err) {
 	    fprintf(stderr, "reattaching model to session\n");
 	    err = reattach_model(ptr, type, (const char *) name, flags);
+	    if (!err) {
+		model_count_plus();
+	    }
 	}
 
 	free(fname);
@@ -360,12 +352,7 @@ read_session_xml (const char *fname, struct sample_info *sinfo)
 
 static int maybe_read_matrix_file (const char *fname) 
 {
-    xmlDocPtr doc = NULL;
-    xmlNodePtr cur = NULL;
-    gretl_matrix *m;
-    char *name;
     FILE *fp;
-    int err = 0;
 
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
@@ -374,37 +361,8 @@ static int maybe_read_matrix_file (const char *fname)
     }
 
     fclose(fp);
-    xmlKeepBlanksDefault(0);
 
-    err = gretl_xml_open_doc_root(fname, "gretl-matrices", &doc, &cur);
-    if (err) {
-	gui_errmsg(err);
-	return 1;
-    }
-
-    cur = cur->xmlChildrenNode;
-    while (cur != NULL && !err) {
-        if (!xmlStrcmp(cur->name, (XUC) "gretl-matrix")) {
-	    name = (char *) xmlGetProp(cur, (XUC) "name");
-	    if (name == NULL) {
-		err = 1;
-	    } else {
-		m = gretl_xml_get_matrix(cur, doc, &err);
-		if (m != NULL) {
-		    err = add_user_matrix(m, name);
-		}
-		free(name);
-	    }
-	}
-	cur = cur->next;
-    }
-
-    if (doc != NULL) {
-	xmlFreeDoc(doc);
-	xmlCleanupParser();
-    }
-
-    return err;
+    return load_user_matrix_file(fname);
 }
 
 static int maybe_read_functions_file (const char *fname) 
@@ -419,7 +377,7 @@ static int maybe_read_functions_file (const char *fname)
 
     fclose(fp);
 
-    return load_user_function_file(fname);
+    return read_session_functions_file(fname);
 }
 
 static int maybe_read_lists_file (const char *fname) 
@@ -476,14 +434,10 @@ static SavedObjectFlags model_save_flags (const void *ptr,
 
 static int maybe_write_matrix_file (void)
 {
-    int i, nmatrices = n_user_matrices();
-    gretl_matrix *m;
     char fullname[MAXLEN];
-    const char *name;
     FILE *fp;
-    int err = 0;
 
-    if (nmatrices == 0) {
+    if (n_user_matrices() == 0) {
 	return 0;
     }
 
@@ -493,31 +447,10 @@ static int maybe_write_matrix_file (void)
 	return E_FOPEN;
     }
 
-    gretl_xml_header(fp);
-    fprintf(fp, "<gretl-matrices count=\"%d\">\n", nmatrices);
-
-    gretl_push_c_numeric_locale();
-
-    for (i=0; i<nmatrices && !err; i++) {
-	m = user_matrix_by_index(i, &name);
-	if (m != NULL) {
-	    gretl_xml_put_matrix(m, name, fp);
-	} else {
-	    err = E_DATA;
-	}
-    }
-
-    gretl_pop_c_numeric_locale();
-
-    fputs("</gretl-matrices>\n", fp);
-
+    write_matrices_to_file(fp);
     fclose(fp);
 
-    if (err) {
-	remove(fullname);
-    }
-
-    return err;
+    return 0;
 }
 
 static int maybe_write_function_file (void)
