@@ -21,14 +21,16 @@
 
 #include "gretl.h"
 #include "filelists.h"
-#include "webget.h"
+#include "gretl_www.h"
 #include "toolbar.h"
 #include "dlgutils.h"
 #include "fileselect.h"
 #include "menustate.h"
+#include "session.h"
 
 #include "libset.h"
 #include "version.h"
+#include "texprint.h"
 
 #include <unistd.h>
 #include <sys/types.h>
@@ -55,13 +57,8 @@ char rcfile[MAXLEN];
 
 extern int want_toolbar;
 extern char Rcommand[MAXSTR];
-extern char dbproxy[21];
-
-#ifdef HAVE_TRAMO
-extern char tramo[MAXSTR];
-extern char tramodir[MAXSTR];
-#endif
 
+char dbproxy[21];
 int use_proxy;
 
 static void make_prefs_tab (GtkWidget *notebook, int tab);
@@ -104,6 +101,7 @@ static char scriptpage[24];
 static int hc_by_default;
 static char hc_xsect[5] = "HC1";
 static char hc_tseri[5] = "HAC";
+static char hc_panel[9] = "Arellano";
 static char hc_garch[5] = "QML";
 
 #ifdef G_OS_WIN32
@@ -137,7 +135,8 @@ typedef enum {
     RADIOSET = 1 << 5,
     INVISET  = 1 << 6,
     FIXSET   = 1 << 7,  /* setting fixed by admin (Windows network use) */
-    BROWSER  = 1 << 8   /* wants "Browse" button */
+    MACHSET  = 1 << 8,  /* "local machine" setting */
+    BROWSER  = 1 << 9   /* wants "Browse" button */
 } rcflags;
 
 typedef struct {
@@ -147,6 +146,7 @@ typedef struct {
     void *var;         /* pointer to variable */
     rcflags flags;     /* ROOTSET user string
 			  USERSET root string
+			  MACHSET local machine string
 			  BOOLSET boolean (user)
                           INTSET integer (user)
 			  LISTSET user string, from fixed menu
@@ -158,19 +158,16 @@ typedef struct {
     GtkWidget *widget;
 } RCVAR;
 
-/* Note: actually "len" above is overloaded: (1) if an rc_var is of
+/* Note: actually "len" above is overloaded: if an rc_var is of
    type BOOLSET and not part of a radio group, then a non-zero value
    for len will link the var's toggle button with the sensitivity of
    the preceding rc_var's entry field.  For example, the "use_proxy"
-   button controls the sensitivity of the "dbproxy" entry widget. (2)
-   if an rc_var is of type RADIOSET, len is used to represent the
-   number of radio (i.e. mutually incompatible) options for the
-   variable.
+   button controls the sensitivity of the "dbproxy" entry widget.
 */
 
 RCVAR rc_vars[] = {
     { "gretldir", N_("Main gretl directory"), NULL, paths.gretldir, 
-      ROOTSET | BROWSER, MAXLEN, TAB_MAIN, NULL },
+      MACHSET | BROWSER, MAXLEN, TAB_MAIN, NULL },
     { "userdir", N_("User's gretl directory"), NULL, paths.userdir, 
       USERSET | BROWSER, MAXLEN, TAB_MAIN, NULL },
     { "expert", N_("Expert mode (no warnings)"), NULL, &expert, 
@@ -193,23 +190,23 @@ RCVAR rc_vars[] = {
 #endif
 #if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
     { "browser", N_("Web browser"), NULL, Browser, 
-      ROOTSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
+      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
 #endif
     { "shellok", N_("Allow shell commands"), NULL, &shellok, 
       BOOLSET, 0, TAB_MAIN, NULL },
     { "gnuplot", N_("Command to launch gnuplot"), NULL, paths.gnuplot, 
-      ROOTSET | BROWSER, MAXLEN, TAB_PROGS, NULL },
+      MACHSET | BROWSER, MAXLEN, TAB_PROGS, NULL },
     { "Rcommand", N_("Command to launch GNU R"), NULL, Rcommand, 
-      ROOTSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
+      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
     { "latex", N_("Command to compile TeX files"), NULL, latex, 
-      ROOTSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
+      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
     { "viewdvi", N_("Command to view DVI files"), NULL, viewdvi, 
-      ROOTSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
+      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
 #if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
     { "viewps", N_("Command to view postscript files"), NULL, viewps, 
-      ROOTSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
+      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
     { "viewpdf", N_("Command to view PDF files"), NULL, viewpdf, 
-      ROOTSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
+      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
 #endif
 #if defined(HAVE_AUDIO) && !defined(G_OS_WIN32)
     { "midiplayer", N_("Program to play MIDI files"), NULL, midiplayer, 
@@ -222,13 +219,7 @@ RCVAR rc_vars[] = {
       ROOTSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
 #endif
 #ifdef HAVE_TRAMO
-    { "tramo", N_("path to tramo"), NULL, tramo, 
-      ROOTSET | BROWSER, MAXSTR, TAB_PROGS, NULL},
-#endif
-#ifdef G_OS_WIN32
-    { "x12adir", N_("X-12-ARIMA working directory"), NULL, paths.x12adir, 
-      ROOTSET | BROWSER, MAXSTR, TAB_PROGS, NULL},
-    { "tramodir", N_("TRAMO working directory"), NULL, tramodir, 
+    { "tramo", N_("path to tramo"), NULL, paths.tramo, 
       ROOTSET | BROWSER, MAXSTR, TAB_PROGS, NULL},
 #endif
     { "binbase", N_("gretl database directory"), NULL, paths.binbase, 
@@ -274,10 +265,12 @@ RCVAR rc_vars[] = {
       LISTSET, 5, TAB_VCV, NULL },
     { "HC_tseri", N_("For time-series data"), NULL, hc_tseri, 
       LISTSET, 5, TAB_VCV, NULL },
+    { "HC_panel", N_("For panel data"), NULL, hc_panel, 
+      LISTSET, 9, TAB_VCV, NULL },
     { "HC_garch", N_("For GARCH estimation"), NULL, hc_garch, 
       LISTSET, 5, TAB_VCV, NULL },
     { "manpref", N_("PDF manual preference"), NULL, &manpref, 
-      RADIOSET | INTSET, 4, TAB_MAN, NULL },
+      RADIOSET | INTSET, 0, TAB_MAN, NULL },
     { NULL, NULL, NULL, NULL, 0, 0, TAB_NONE, NULL }
 };
 
@@ -318,7 +311,7 @@ static gretlopt set_paths_opt = OPT_X;
 void force_english_help (void)
 {
     set_paths_opt |= OPT_N;
-    set_paths(&paths, set_paths_opt);
+    gretl_set_paths(&paths, set_paths_opt);
 }
 
 void set_fixed_font (void)
@@ -332,18 +325,17 @@ void set_fixed_font (void)
 
 #ifndef G_OS_WIN32
 
-/* remedial setting of font path for libgd, in case it's not
-   set right (which, it seems, is often the case on
-   Linux) 
+/* remedial setting of font path for libgd, in case it's not set right
+   (which, I'm sorry to say, seems often to be the case on Linux)
 */
 
-static void maybe_set_gd_fontpath (void)
+static void set_gd_fontpath (void)
 {
     char *gdpath = NULL;
     char *newpath = NULL;
 
-    if (gnuplot_has_ttf()) {
-	/* fonts already found, don't mess with this */
+    if (gnuplot_has_ttf(0)) {
+	/* we're OK, don't mess */
 	return;
     }
 
@@ -361,6 +353,7 @@ static void maybe_set_gd_fontpath (void)
     if (newpath != NULL) {
 	setenv("GDFONTPATH", newpath, 1);
 	g_free(newpath);
+	gnuplot_has_ttf(1);
     }
 }
 
@@ -437,37 +430,61 @@ static void get_functions_dir (char *dirname)
 {
     char *target = NULL;
     FILE *fp;
-    int err;
+    int err, ok = 0;
 
     sprintf(dirname, "%sfunctions", paths.gretldir);
     err = gretl_mkdir(dirname);
     if (!err) {
 	target = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
-	fp = gretl_fopen(target, "w");
-	if (fp != NULL) {
-	    fclose(fp);
-	    remove(target);
+	if (target != NULL) {
+	    fp = gretl_fopen(target, "w");
+	    if (fp != NULL) {
+		ok = 1;
+		fclose(fp);
+		remove(target);
+	    }
 	    free(target);
-	    return;
 	}
     } 
+    
+    if (ok) return;
 
     sprintf(dirname, "%sfunctions", paths.userdir);
     err = gretl_mkdir(dirname);
     if (!err) {
 	target = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
-	fp = gretl_fopen(target, "w");
-	if (fp != NULL) {
-	    fclose(fp);
-	    remove(target);
+	if (target != NULL) {
+	    fp = gretl_fopen(target, "w");
+	    if (fp != NULL) {
+		ok = 1;
+		fclose(fp);
+		remove(target);
+	    }
 	    free(target);
-	    return;
 	}
-    }	
+    }
+
+    if (ok) return;
 
     *dirname = '\0';
 }
 
+static char startdir[MAXLEN];
+
+void set_program_startdir (const char *callname)
+{
+    char *test = getcwd(startdir, MAXLEN);
+
+    if (test == NULL) {
+	*startdir = '\0';
+    }
+
+#if 0
+    fprintf(stderr, "Starting in '%s'\n", startdir);
+    fprintf(stderr, "Called as '%s'\n", callname);
+#endif
+}
+
 void get_default_dir (char *s, int action)
 {
     *s = '\0';
@@ -484,75 +501,35 @@ void get_default_dir (char *s, int action)
     }
 
     if (usecwd && action != SAVE_DBDATA) {
-	char *test = getcwd(s, MAXLEN);
+	if (*startdir != '\0') {
+	    strcpy(s, startdir);
+	} else {
+	    const char *sdir = get_session_dirname();
+	    char *test = getcwd(s, MAXLEN);
 
-	if (test == NULL) {
-	    strcpy(s, paths.userdir);
+	    if (test == NULL || (*sdir != '\0' && strstr(s, sdir))) {
+		strcpy(s, paths.userdir);
+	    }
 	} 
     } else {
 	strcpy(s, paths.userdir);   
-    } 
-    
+    }
+
     slash_terminate(s);
 }
 
-#if defined(HAVE_TRAMO) || defined(HAVE_X12A)
-
-# ifdef HAVE_TRAMO
-void set_tramo_ok (int set)
-{
-    static int ok;
-
-    if (set >= 0) {
-	ok = set;
-    }
-
-    if (mdata != NULL) {
-	flip(mdata->ifac, "/Variable/TRAMO analysis", ok);
-    }
-}
-# endif /* HAVE_TRAMO */
-
-# ifdef HAVE_X12A
-void set_x12a_ok (int set)
-{
-    static int ok;
-
-    if (set >= 0) {
-	ok = set;
-    }
-
-    if (mdata != NULL) {
-	flip(mdata->ifac, "/Variable/X-12-ARIMA analysis", ok);
-    }
-}
-# endif /* HAVE_X12A */
-
-# ifdef G_OS_WIN32
-
+#ifdef G_OS_WIN32
 static const char *get_reg_base (const char *key)
 {
-    if (strncmp(key, "x12a", 4) == 0) {
+    if (!strncmp(key, "x12a", 4)) {
         return "x12arima";
-    }
-    if (strncmp(key, "tramo", 5) == 0) {
+    } else if (!strncmp(key, "tramo", 5)) {
         return "tramo";
+    } else {
+	return "gretl";
     }
-    return "gretl";
 }
-
-static void set_tramo_x12a_dirs (void)
-{
-    int ok;
-    
-    ok = check_for_prog(tramo);
-    set_tramo_ok(ok);
-
-    ok = check_for_prog(paths.x12a);
-    set_x12a_ok(ok);
-}
-
-# else /* not G_OS_WIN32 */
+#endif
 
 #ifdef OSX_BUILD
 static int alt_ok (const char *prog)
@@ -577,7 +554,7 @@ static int alt_ok (const char *prog)
     
     if (ok) {
         if (tr) {
-            strcpy(tramo, test);
+            strcpy(paths.tramo, test);
 	} else {
 	    strcpy(paths.x12a, test);
 	}
@@ -587,73 +564,45 @@ static int alt_ok (const char *prog)
 }
 #endif
 
-static void set_tramo_x12a_dirs (void)
+#if defined(HAVE_TRAMO) || defined(HAVE_X12A)
+
+static void set_tramo_x12a_status (void)
 {
-    char dirname[MAXLEN];
-    DIR *test;
     int ok;
 
 #ifdef HAVE_TRAMO
-    ok =  check_for_prog(tramo);
+    ok = 0;
+    if (*paths.tramodir != '\0') {
+	ok = check_for_prog(paths.tramo);
 # ifdef OSX_BUILD
-    if (!ok) {
-        ok = alt_ok(tramo);
-    }
+	if (!ok) {
+	    ok = alt_ok(paths.tramo);
+	}
 # endif    
-    set_tramo_ok(ok); 
-       
-    if (*tramodir == '\0') {
-	build_path(tramodir, paths.userdir, "tramo", NULL);
     }
-#endif
+
+    if (mdata != NULL) {
+	flip(mdata->ifac, "/Variable/TRAMO analysis", ok);
+    }
+#endif /* TRAMO */
 
 #ifdef HAVE_X12A
-    ok = check_for_prog(paths.x12a);
+    ok = 0;
+    if (*paths.x12adir != '\0') {
+	ok = check_for_prog(paths.x12a);
 # ifdef OSX_BUILD    
-    if (!ok) {
-	ok = alt_ok(paths.x12a);
-    }
-# endif    
-    set_x12a_ok(ok);
-    
-    if (*paths.x12adir == '\0') {
-	build_path(paths.x12adir, paths.userdir, "x12arima", NULL);
+	if (!ok) {
+	    ok = alt_ok(paths.x12a);
+	}
+# endif  
     }
 
-    /* don't make dir structure (yet) if userdir doesn't exist */
-    test = opendir(paths.userdir);
-    if (test == NULL) {
-	return;
-    } else {
-	closedir(test);
-    }
-#endif
-    
-#ifdef HAVE_X12A
-    gretl_mkdir(paths.x12adir);
-#endif
-
-#ifdef HAVE_TRAMO
-    if (gretl_mkdir(tramodir)) return;
-    sprintf(dirname, "%s/output", tramodir);
-    gretl_mkdir(dirname);
-    sprintf(dirname, "%s/graph", tramodir);
-    if (gretl_mkdir(dirname)) return;
-    sprintf(dirname, "%s/graph/acf", tramodir);
-    gretl_mkdir(dirname);
-    sprintf(dirname, "%s/graph/filters", tramodir);
-    gretl_mkdir(dirname);
-    sprintf(dirname, "%s/graph/forecast", tramodir);
-    gretl_mkdir(dirname);
-    sprintf(dirname, "%s/graph/series", tramodir);
-    gretl_mkdir(dirname);
-    sprintf(dirname, "%s/graph/spectra", tramodir);
-    gretl_mkdir(dirname);
-#endif /* HAVE_TRAMO */
+    if (mdata != NULL) {
+	flip(mdata->ifac, "/Variable/X-12-ARIMA analysis", ok);
+    }    
+#endif /* X12A */
 }
 
-# endif /* G_OS_WIN32 */
-
 #endif /* tramo || x12a */
 
 #ifdef G_OS_WIN32
@@ -785,28 +734,35 @@ int check_for_prog (const char *prog)
     return found;
 }
 
-#endif
-
-#if !defined(G_OS_WIN32) && !defined(USE_GNOME)
-void set_rcfile (void) 
+static void root_check (void)
 {
-    char *tmp;
+    if (getuid() == 0) {
+	int resp;
 
-    tmp = getenv("HOME");
-    strcpy(rcfile, tmp);
-    strcat(rcfile, "/.gretl2rc");
-    read_rc(); 
-    maybe_set_gd_fontpath();
+	resp = yes_no_dialog ("gretl", _("You seem to be running gretl " 
+			      "as root.  Do you really want to do this?"), 
+			      0);
+	if (resp == GRETL_NO) {
+	    exit(EXIT_FAILURE);
+	}
+    }
 }
+
+void gretl_config_init (void)
+{
+#ifndef USE_GCONF
+    sprintf(rcfile, "%s/.gretl2rc", getenv("HOME"));
 #endif
 
-#ifdef USE_GNOME
-void set_rcfile (void)
-{
     read_rc();
-    maybe_set_gd_fontpath();
+    set_gd_fontpath();
+
+    if (!expert) {
+	root_check();
+    }
 }
-#endif
+
+#endif /* *nix versus Windows */
 
 static void option_dialog_canceled (GtkWidget *w, int *c)
 {
@@ -939,11 +895,16 @@ static gboolean takes_effect_on_restart (void)
     return FALSE;
 }
 
-static GList *get_settings_list (void *var)
+#define HIDE_SPANISH_MANUAL 1
+
+static GList *get_settings_list (void *var, int *nopt)
 {
     char *hc_strs[] = {
 	"HC0", "HC1", "HC2", "HC3", "HC3a", "HAC"
     };
+    char *hc_panel_strs[] = {
+	"Arellano", "PCSE"
+    };
     char *garch_strs[] = {
 	"QML", "BW"
     };
@@ -951,10 +912,10 @@ static GList *get_settings_list (void *var)
         N_("English (US letter paper)"),
         N_("English (A4 paper)"),
         N_("Italian"),
-	N_("Spanish"),
+	N_("Spanish")
     };
     GList *list = NULL;
-    int i, n;
+    int i, n = 0;
 
     if (var == hc_xsect || var == hc_tseri) {
 	n = sizeof hc_strs / sizeof hc_strs[0];
@@ -962,6 +923,11 @@ static GList *get_settings_list (void *var)
 	for (i=0; i<n; i++) {
 	    list = g_list_append(list, hc_strs[i]);
 	}
+    } else if (var == hc_panel) {
+	n = sizeof hc_panel_strs / sizeof hc_panel_strs[0];
+	for (i=0; i<n; i++) {
+	    list = g_list_append(list, hc_panel_strs[i]);
+	}
     } else if (var == hc_garch) {
 	n = sizeof garch_strs / sizeof garch_strs[0];
 	for (i=0; i<n; i++) {
@@ -969,10 +935,17 @@ static GList *get_settings_list (void *var)
 	}
     } else if (var == &manpref) {
 	n = sizeof man_strs / sizeof man_strs[0];
+#if HIDE_SPANISH_MANUAL
+	n--;
+#endif
 	for (i=0; i<n; i++) {
 	    list = g_list_append(list, _(man_strs[i]));
 	}
-    }	
+    }
+
+    if (nopt != NULL) {
+	*nopt = n;
+    }
 
     return list;
 }
@@ -1188,16 +1161,16 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 			     1, 2, s_len-1, s_len,
 			     0, 0, 0, 0);
 
-	    list = get_settings_list(rc->var);
+	    list = get_settings_list(rc->var, NULL);
 	    gtk_combo_set_popdown_strings(GTK_COMBO(rc->widget), list);
 	    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(rc->widget)->entry), strvar);
-	    gtk_entry_set_width_chars(GTK_ENTRY(GTK_COMBO(rc->widget)->entry), 
-				      rc->len - 1);
+	    gtk_entry_set_width_chars(GTK_ENTRY(GTK_COMBO(rc->widget)->entry), 8);
 	    gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(rc->widget)->entry), 
 				      FALSE);
 	    gtk_widget_show(rc->widget);
 	} else if (rc->flags & RADIOSET) {
 	    int i, rcval = *(int *) (rc->var);
+	    int nopt = 0;
 	    GtkWidget *b;
 	    GSList *group = NULL;
 	    GList *list, *mylist;
@@ -1209,9 +1182,9 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 				      b_len - 1, b_len);
 	    gtk_widget_show(b);
 
-	    mylist = list = get_settings_list(rc->var);
+	    mylist = list = get_settings_list(rc->var, &nopt);
 
-	    for (i=0; i<rc->len; i++) {
+	    for (i=0; i<nopt; i++) {
 		b_len++;
 		gtk_table_resize(GTK_TABLE(b_table), b_len, 2);
 		b = gtk_radio_button_new_with_label(group, mylist->data);
@@ -1271,7 +1244,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	gtk_box_pack_start(GTK_BOX(hb), w, TRUE, TRUE, 0);
 	gtk_widget_show(w);
 
-	w = standard_button(GTK_STOCK_HELP);
+	w = gtk_button_new_from_stock(GTK_STOCK_HELP);
 	gtk_box_pack_start(GTK_BOX(hb), w, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(w), "clicked", 
 			 G_CALLBACK(context_help), 
@@ -1285,23 +1258,23 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 
 #ifdef ENABLE_NLS
 
-#ifdef G_OS_WIN32
+# ifdef G_OS_WIN32
+
 struct langname {
     const char *abbr;
     const char *full;
 };
-#endif
 
 static void set_lcnumeric (void)
 {
     if (lcnumeric) {
-#ifdef G_OS_WIN32
 	struct langname names[] = {
 	    { "es", "Spanish" },
 	    { "fr", "French" },
 	    { "it", "Italian" },
 	    { "pl", "Polish" },
 	    { "de", "German" },
+	    { "pt", "Portuguese" },
 	    { NULL, NULL }
 	};
 	char *lang = getenv("LANG");
@@ -1310,45 +1283,105 @@ static void set_lcnumeric (void)
 
 	if (lang != NULL) {
 	    for (i=0; names[i].abbr != NULL; i++) {
-		if (!strcmp(lang, names[i].abbr)) {
+		if (!strncmp(lang, names[i].abbr, 2)) {
 		    set = setlocale(LC_NUMERIC, names[i].full);
 		    if (set == NULL) {
 			set = setlocale(LC_NUMERIC, names[i].abbr);
 		    }
+		    if (set != NULL) {
+			break;
+		    }
 		}
 	    }
 	}
 
 	if (set == NULL) {
 	    setlocale(LC_NUMERIC, "");
+	    putenv("LC_NUMERIC=");
 	}
-	putenv("LC_NUMERIC=");
-#else
-	putenv("LC_NUMERIC=");
-	setlocale(LC_NUMERIC, "");
-#endif
     } else {
-	putenv("LC_NUMERIC=C");
 	setlocale(LC_NUMERIC, "C");
+	putenv("LC_NUMERIC=C");
     }
 
     reset_local_decpoint();
 }
-#endif
+
+# else /* ! G_OS_WIN32 */
+
+static void set_lcnumeric (void)
+{
+    if (lcnumeric) {
+	char *lang = getenv("LANG");
+	char *set = NULL;
+
+	if (lang != NULL) {
+	    set = setlocale(LC_NUMERIC, lang);
+	    fprintf(stderr, "setlocale(LC_NUMERIC, \"%s\") returned %s\n", 
+		    lang, set);
+	} else {
+	    fprintf(stderr, "set_lcnumeric: getenv(\"LANG\") gave NULL\n");
+	}
+	if (set == NULL) {
+	    setlocale(LC_NUMERIC, "");
+	    putenv("LC_NUMERIC=");
+	}
+    } else {
+	setlocale(LC_NUMERIC, "C");
+	putenv("LC_NUMERIC=C");
+    }
+
+    reset_local_decpoint();
+}
+
+# endif
+
+#endif /* ENABLE_NLS */
 
 static void set_gp_colors (void)
 {
-    char cstr[4][8];
+    char cstr[N_GP_COLORS][8];
     int i, nc;
 
+    *cstr[0] = *cstr[1] = *cstr[2] = *cstr[3] = '\0';
+
     nc = sscanf(gpcolors, "%7s %7s %7s %7s", 
 		cstr[0], cstr[1], cstr[2], cstr[3]);
 
     for (i=0; i<nc; i++) {
-	set_gnuplot_pallette(i, cstr[i]);
+	set_graph_palette(i, cstr[i]);
     }
 }
 
+#if defined(HAVE_TRAMO) || defined(HAVE_X12A)
+
+static void maybe_revise_tramo_x12a_status (void)
+{
+    int doit = 0;
+
+# ifdef HAVE_TRAMO
+    if (strcmp(paths.tramo, gretl_tramo())) {
+	doit = 1;
+    } 
+# endif
+
+# ifdef HAVE_X12A
+    if (strcmp(paths.x12a, gretl_x12_arima())) {
+	doit = 1;
+    }
+# endif
+
+    fprintf(stderr, "maybe_revise_tramo_x12a_status: doit = %d\n", doit);
+
+    if (doit) {
+	set_tramo_x12a_status();
+    }
+}
+
+#endif
+
+/* register and react to changes from Preferences dialog */
+
 static void apply_changes (GtkWidget *widget, gpointer data) 
 {
     const gchar *str;
@@ -1363,7 +1396,9 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 		} else {
 		    *(int *) (rc_vars[i].var) = FALSE;
 		}
-	    } else if (rc_vars[i].flags & USERSET || rc_vars[i].flags & ROOTSET) {
+	    } else if ((rc_vars[i].flags & USERSET) || 
+		       (rc_vars[i].flags & ROOTSET) ||
+		       (rc_vars[i].flags & MACHSET)) {
 		str = gtk_entry_get_text(GTK_ENTRY(rc_vars[i].widget));
 		if (str != NULL && *str != '\0') { 
 		    strvar = (char *) rc_vars[i].var;
@@ -1382,23 +1417,25 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 	    }
 	}
     }
-    
-    write_rc();
+
+#if defined(HAVE_TRAMO) || defined(HAVE_X12A)
+    maybe_revise_tramo_x12a_status();
+#endif
+
+    write_rc(); /* note: calls gretl_set_paths */
 
     show_or_hide_toolbar(want_toolbar);
 
     /* register these for session using libset apparatus */
     set_use_qr(useqr);
+    set_use_cwd(usecwd);
     set_shell_ok(shellok);
     set_xsect_hccme(hc_xsect);
     set_tseries_hccme(hc_tseri);
+    set_panel_hccme(hc_panel);
     set_garch_robust_vcv(hc_garch);
 
-#if defined(HAVE_TRAMO) || defined(HAVE_X12A)
-    set_tramo_x12a_dirs();
-#endif
-
-    proxy_init(dbproxy);
+    gretl_www_init(paths.dbhost, dbproxy, use_proxy);
 }
 
 #ifndef USE_GCONF
@@ -1438,9 +1475,36 @@ static void boolvar_to_str (void *b, char *s)
     }
 }
 
-static void common_read_rc_setup (void)
+static int dir_exists (const char *dname, FILE *fp)
 {
+    DIR *test;
+    int ok = 0;
+
+#ifdef G_OS_WIN32
+    test = win32_opendir(dname);
+#else
+    test = opendir(dname);
+#endif
+
+    if (test != NULL) {
+	ok = 1;
+	if (fp != NULL) {
+	    fprintf(fp, "Directory '%s' exists, OK\n", dname);
+	}
+	closedir(test);
+    } else if (fp != NULL) {
+	fprintf(fp, "Directory '%s' does not exist\n", dname);
+    }
+
+    return ok;
+}
+
+static int common_read_rc_setup (void)
+{
+    int err = 0;
+
     set_use_qr(useqr);
+    set_use_cwd(usecwd);
     set_shell_ok(shellok);
     set_gp_colors();
     
@@ -1448,15 +1512,20 @@ static void common_read_rc_setup (void)
     set_tseries_hccme(hc_tseri);
     set_garch_robust_vcv(hc_garch);
 
-    set_paths(&paths, set_paths_opt);
+    err = gretl_set_paths(&paths, set_paths_opt);
+
+    gretl_www_init(paths.dbhost, dbproxy, use_proxy);
+    set_tex_use_pdf(latex);
 
 # if defined(HAVE_TRAMO) || defined(HAVE_X12A)
-    set_tramo_x12a_dirs();
+    set_tramo_x12a_status();
 # endif
 
 # ifdef ENABLE_NLS
     set_lcnumeric();
 # endif
+
+    return err;
 }
 
 /* next section: variant versions of write_rc and read_rc, depending
@@ -1492,7 +1561,7 @@ void write_rc (void)
 
     save_file_lists(client);
     g_object_unref(G_OBJECT(client));
-    set_paths(&paths, set_paths_opt);
+    gretl_set_paths(&paths, set_paths_opt);
     record_shell_opt();
 }
 
@@ -1516,7 +1585,7 @@ static void read_rc (void)
 		fprintf(stderr, "Error reading %s\n", rc_vars[i].key);
 		g_clear_error(&error);
 	    } else {
-		*(int *) rc_vars[i].var = bval;
+		*(int *) rc_vars[i].var = (int) bval;
 	    }
 	} else if (rc_vars[i].flags & INTSET) {
 	    ival = gconf_client_get_int(client, key, &error);
@@ -1544,9 +1613,7 @@ static void read_rc (void)
     }
 
     read_file_lists(client);
-
     g_object_unref(G_OBJECT(client));
-
     common_read_rc_setup();
 }
 
@@ -1559,41 +1626,50 @@ void write_rc (void)
     char bval[6];
     char ival[16];
     const char *strval;
-    int i = 0;
+    int i = 0, err = 0;
 
     for (i=0; rc_vars[i].key != NULL; i++) {
 
-	if (rc_vars[i].flags & FIXSET) continue;
+	if (rc_vars[i].flags & (FIXSET | ROOTSET)) {
+	    /* read-only variables */
+	    continue;
+	}
 
 	if (rc_vars[i].flags & BOOLSET) {
 	    boolvar_to_str(rc_vars[i].var, bval);
-	    write_reg_val(HKEY_CURRENT_USER, 
-			  "gretl", 
-			  rc_vars[i].key, 
-			  bval);
+	    err += write_reg_val(HKEY_CURRENT_USER, 
+				 "gretl", 
+				 rc_vars[i].key, 
+				 bval);
 	} else if (rc_vars[i].flags & INTSET) {
 	    sprintf(ival, "%d", *(int *) rc_vars[i].var);
-	    write_reg_val(HKEY_CURRENT_USER, 
-			  "gretl", 
-			  rc_vars[i].key, 
-			  ival);	    
-	} else if (rc_vars[i].flags & ROOTSET) {
+	    err += write_reg_val(HKEY_CURRENT_USER, 
+				 "gretl", 
+				 rc_vars[i].key, 
+				 ival);	    
+	} else if (rc_vars[i].flags & MACHSET) {
 	    strval = (char *) rc_vars[i].var;
-	    write_reg_val(HKEY_CLASSES_ROOT, 
-			  get_reg_base(rc_vars[i].key),
-			  rc_vars[i].key, 
-			  strval);
+	    err += write_reg_val(HKEY_LOCAL_MACHINE, 
+				 get_reg_base(rc_vars[i].key),
+				 rc_vars[i].key, 
+				 strval);
 	} else {
 	    strval = (char *) rc_vars[i].var;
-	    write_reg_val(HKEY_CURRENT_USER, 
-			  get_reg_base(rc_vars[i].key),
-			  rc_vars[i].key, 
-			  strval);
+	    err += write_reg_val(HKEY_CURRENT_USER, 
+				 get_reg_base(rc_vars[i].key),
+				 rc_vars[i].key, 
+				 strval);
 	}
     }
 
+#if 0
+    if (err) {
+	win_show_last_error();
+    }
+#endif
+
     save_file_lists();
-    set_paths(&paths, set_paths_opt);
+    gretl_set_paths(&paths, set_paths_opt);
 }
 
 static int get_network_settings (void)
@@ -1658,9 +1734,13 @@ void read_rc (void)
     int i;
 
     if (get_network_settings() && *paths.userdir != '\0') {
-	win32_make_user_dirs();
+	int err = set_gretl_user_dir(paths.userdir, &paths);
+
+	if (err) {
+	    gui_errmsg(err);
+	}
 	for (i=0; rc_vars[i].key != NULL; i++) {
-	    if (rc_vars[i].var == tramodir ||
+	    if (rc_vars[i].var == paths.tramodir ||
 		rc_vars[i].var == paths.x12adir) {
 		rc_vars[i].flags |= FIXSET;
 	    }
@@ -1677,15 +1757,21 @@ void read_rc (void)
 	*value = '\0';
 
 	if (rc_vars[i].flags & ROOTSET) {
-	    err = read_reg_val (HKEY_CLASSES_ROOT, 
-				get_reg_base(rc_vars[i].key),
-				rc_vars[i].key, 
-				value);
+	    err = read_reg_val_with_fallback(HKEY_LOCAL_MACHINE, 
+					     HKEY_CLASSES_ROOT,
+					     get_reg_base(rc_vars[i].key),
+					     rc_vars[i].key, 
+					     value);
+	} else if (rc_vars[i].flags & MACHSET) {
+	    err = read_reg_val(HKEY_LOCAL_MACHINE, 
+			       get_reg_base(rc_vars[i].key),
+			       rc_vars[i].key, 
+			       value);
 	} else {
-	    err = read_reg_val (HKEY_CURRENT_USER, 
-				get_reg_base(rc_vars[i].key),
-				rc_vars[i].key, 
-				value);
+	    err = read_reg_val(HKEY_CURRENT_USER, 
+			       get_reg_base(rc_vars[i].key),
+			       rc_vars[i].key, 
+			       value);
 	}
 	    
 	if (!err && *value != '\0') {
@@ -1702,9 +1788,7 @@ void read_rc (void)
     }
 
     read_file_lists();
-
     common_read_rc_setup();
-
     set_fixed_font();
     set_app_font(NULL);
 }
@@ -1718,6 +1802,8 @@ void write_rc (void)
     char *strvar;
     int i;
 
+    fprintf(stderr, "old write_rc\n");
+
     rc = fopen(rcfile, "w");
     if (rc == NULL) {
 	errbox(_("Couldn't open config file for writing"));
@@ -1741,7 +1827,7 @@ void write_rc (void)
 
     save_file_lists(rc);
     fclose(rc);
-    set_paths(&paths, set_paths_opt);
+    gretl_set_paths(&paths, set_paths_opt);
     record_shell_opt();
 }
 
@@ -1752,8 +1838,13 @@ static void read_rc (void)
     char *strvar;
     int i, j;
 
+    fprintf(stderr, "old read_rc\n");
+
     fp = fopen(rcfile, "r");
-    if (fp == NULL) return;
+    if (fp == NULL) {
+	fprintf(stderr, "Couldn't read %s\n", rcfile);
+	return;
+    }
 
     i = 0;
     while (rc_vars[i].var != NULL) {
@@ -1788,9 +1879,7 @@ static void read_rc (void)
     }
 
     read_file_lists(fp, line);
-
     fclose(fp);
-
     common_read_rc_setup();
 }
 
@@ -2008,14 +2097,11 @@ void font_selector (gpointer data, guint which, GtkWidget *widget)
 
 /* graph color selection apparatus */
 
-static double scale_round (double val)
-{
-    return val * 255.0 / 65535.0;
-}
-
 #define XPMROWS 19
 #define XPMCOLS 17
 
+#define scale_round(v) ((v) * 255.0 / 65535.0)
+
 static GtkWidget *get_image_for_color (const char *colstr)
 {
     GdkPixbuf *icon;
@@ -2024,17 +2110,9 @@ static GtkWidget *get_image_for_color (const char *colstr)
     int i;
 
     if (xpm == NULL) {
-	xpm = malloc(XPMROWS * sizeof *xpm);
+	xpm = strings_array_new_with_length(XPMROWS, XPMCOLS);
 	if (xpm != NULL) {
 	    for (i=0; i<XPMROWS; i++) {
-		xpm[i] = malloc(XPMCOLS * sizeof **xpm);
-		if (xpm[i] == NULL) {
-		    int j;
-
-		    for (j=0; j<i; j++) free(xpm[j]);
-		    free(xpm);
-		    xpm = NULL;
-		}
 		if (i == 0) {
 		    strcpy(xpm[i], "16 16 2 1");
 		} else if (i == 1) {
@@ -2050,7 +2128,9 @@ static GtkWidget *get_image_for_color (const char *colstr)
 	}
     }
 
-    if (xpm == NULL) return NULL;
+    if (xpm == NULL) {
+	return NULL;
+    }
 
     for (i=0; i<6; i++) {
 	xpm[1][10+i] = colstr[1+i];
@@ -2075,25 +2155,22 @@ static void color_select_callback (GtkWidget *button, GtkWidget *w)
 
     gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(csel), &color);
     sprintf(color_string, "x%02x%02x%02x",
-	    (guint) (scale_round (color.red)),
-	    (guint) (scale_round (color.green)),
-	    (guint) (scale_round (color.blue)));
+	    (guint) (scale_round(color.red)),
+	    (guint) (scale_round(color.green)),
+	    (guint) (scale_round(color.blue)));
 
     i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "colnum"));
 
-    set_gnuplot_pallette(i, color_string);
-
-    sprintf(gpcolors, "%s %s %s %s", 
-	    get_gnuplot_pallette(0, 0),
-	    get_gnuplot_pallette(1, 0),
-	    get_gnuplot_pallette(2, 0),
-	    get_gnuplot_pallette(3, 0));
+    set_graph_palette(i, color_string);
+    print_palette_string(gpcolors);
 
+    /* update the "image" widget */
     image = g_object_get_data(G_OBJECT(color_button), "image");
     gtk_widget_destroy(image);
     image = get_image_for_color(color_string);
     gtk_widget_show(image);
     gtk_container_add(GTK_CONTAINER(color_button), image);
+    g_object_set_data(G_OBJECT(color_button), "image", image);
   
     gtk_widget_destroy(w);
 }
@@ -2103,18 +2180,11 @@ static void color_cancel (GtkWidget *button, GtkWidget *w)
     gtk_widget_destroy(w);
 }
 
-GtkWidget *color_patch_button (int colnum)
+GtkWidget *color_patch_button (int cnum)
 {
     GtkWidget *image, *button;
-    const char *colstr;
 
-    if (colnum == COLOR_MAX) {
-	colstr = get_gnuplot_pallette(0, PLOT_FREQ_SIMPLE);
-    } else {
-	colstr = get_gnuplot_pallette(colnum, 0);
-    }
-
-    image = get_image_for_color(colstr);
+    image = get_image_for_color(graph_color_string(cnum));
 
     if (image == NULL) {
 	button = gtk_button_new_with_label(_("Select color"));
@@ -2127,26 +2197,38 @@ GtkWidget *color_patch_button (int colnum)
     return button;
 }
 
-void gnuplot_color_selector (GtkWidget *w, gpointer p)
+/* reset color patch button after selecting the option to
+   restore the default plot colors */
+
+void color_patch_button_reset (GtkWidget *button, int cnum)
+{
+    GtkWidget *image;
+
+    image = g_object_get_data(G_OBJECT(button), "image");
+    gtk_widget_destroy(image);
+    image = get_image_for_color(graph_color_string(cnum));
+    gtk_widget_show(image);
+    gtk_container_add(GTK_CONTAINER(button), image);
+    g_object_set_data(G_OBJECT(button), "image", image);
+
+    if (cnum == BOXCOLOR || cnum == BOXCOLOR - 1) {
+	print_palette_string(gpcolors);
+    }
+}
+
+void graph_color_selector (GtkWidget *w, gpointer p)
 {
     GtkWidget *cdlg;
     GtkWidget *button;
     gint i = GPOINTER_TO_INT(p);
-    const char *colstr;
-    char my_colstr[8];
+    char colstr[8];
     GdkColor color;
 
-    if (i == COLOR_MAX) {
-	colstr = get_gnuplot_pallette(0, PLOT_FREQ_SIMPLE);
-    } else {
-	colstr = get_gnuplot_pallette(i, 0); 
-    } 
+    strcpy(colstr, graph_color_string(i));
+    *colstr = '#';
+    gdk_color_parse(colstr, &color);
 
-    strcpy(my_colstr, colstr);
-    *my_colstr = '#';
-    gdk_color_parse(my_colstr, &color);
-
-    cdlg = gtk_color_selection_dialog_new("gretl color selection");
+    cdlg = gtk_color_selection_dialog_new(_("gretl: graph color selection"));
 
     g_object_set_data(G_OBJECT(cdlg), "colnum", GINT_TO_POINTER(i));
     g_object_set_data(G_OBJECT(cdlg), "color_button", w);
@@ -2162,62 +2244,29 @@ void gnuplot_color_selector (GtkWidget *w, gpointer p)
     button = GTK_COLOR_SELECTION_DIALOG(cdlg)->cancel_button;
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(color_cancel), cdlg);
-    
+
     gtk_widget_show(cdlg);
+    gtk_window_set_modal(GTK_WINDOW(cdlg), TRUE);
 }
 
 /* end graph color selection apparatus */
 
-static int dir_exists (const char *dname, FILE *fp)
-{
-    DIR *test;
-    int ok = 0;
-
-    test = opendir(dname);
-
-    if (test != NULL) {
-	ok = 1;
-	if (fp != NULL) {
-	    fprintf(fp, "Directory '%s' exists, OK\n", dname);
-	}
-	closedir(test);
-    } else if (fp != NULL) {
-	fprintf(fp, "Directory '%s' does not exist\n", dname);
-    }
-
-    return ok;
-}
-
 #ifndef G_OS_WIN32
 
-static int validate_dir (const char *dirname)
-{
-    int err = 0;
-
-    if (!dir_exists(dirname, NULL)) {
-	err = mkdir(dirname, 0755);
-	if (err) {
-	    errbox(_("Couldn't create directory '%s'"), dirname);
-	} else {
-	    infobox(_("Working directory created OK"));
-	}
-    }
-
-    return err;
-}
-
 static void real_set_userdir (GtkWidget *widget, dialog_t *dlg)
 {
     const gchar *dirname;
+    int err;
 
     dirname = edit_dialog_get_text(dlg);
+    err = set_gretl_user_dir(dirname, &paths);
 
-    if (validate_dir(dirname)) {
+    if (err) {
+	gui_errmsg(err);
 	return;
     } else {
-	set_gretl_user_dir(dirname, &paths);
 #if defined(HAVE_TRAMO) || defined(HAVE_X12A)
-	set_tramo_x12a_dirs();
+	set_tramo_x12a_status();
 #endif
 	close_dialog(dlg);
     }
