@@ -20,13 +20,12 @@
 #include "gretl.h"
 #include "textbuf.h"
 
-#ifdef USE_GTKSOURCEVIEW
-# include <gtksourceview/gtksourceview.h>
-# include <gtksourceview/gtksourcelanguage.h>
-# include <gtksourceview/gtksourcelanguagesmanager.h>
-#endif
+#include <gtksourceview/gtksourceview.h>
+#include <gtksourceview/gtksourcelanguage.h>
+#include <gtksourceview/gtksourcelanguagesmanager.h>
 
-#define GUIDE_PAGE 999
+#define GUIDE_PAGE  999
+#define SCRIPT_PAGE 998
 
 enum {
     PLAIN_TEXT,
@@ -98,6 +97,27 @@ gchar *textview_get_text (GtkWidget *view)
     return gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
 }
 
+gchar *textview_get_selection_or_all (GtkWidget *view,
+				      int *sel)
+{
+    GtkTextBuffer *tbuf;
+    GtkTextIter start, end;
+
+    g_return_val_if_fail(GTK_IS_TEXT_VIEW(view), NULL);
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+
+    if (gtk_text_buffer_get_selection_bounds(tbuf, &start, &end)) {
+	*sel = 1;
+	return gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
+    } else {
+	*sel = 0;
+	gtk_text_buffer_get_start_iter(tbuf, &start);
+	gtk_text_buffer_get_end_iter(tbuf, &end);
+	return gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
+    }
+}
+
 int textview_set_text (GtkWidget *view, const gchar *text)
 {
     GtkTextBuffer *tbuf;
@@ -116,6 +136,22 @@ int textview_set_text (GtkWidget *view, const gchar *text)
     return 0;
 }
 
+int textview_set_cursor_at_line (GtkWidget *view, int line)
+{
+    GtkTextBuffer *tbuf;
+    GtkTextIter iter;
+    
+    g_return_val_if_fail(GTK_IS_TEXT_VIEW(view), 1);
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+    g_return_val_if_fail(tbuf != NULL, 1);
+
+    gtk_text_buffer_get_iter_at_line(tbuf, &iter, line);
+    gtk_text_buffer_place_cursor(tbuf, &iter);
+
+    return 0;
+}
+
 int viewer_char_count (windata_t *vwin)
 {
     GtkTextBuffer *tbuf;
@@ -143,7 +179,6 @@ void text_undo (windata_t *vwin, guint u, GtkWidget *widget)
 {
     gchar *old = NULL;
 
-#ifdef USE_GTKSOURCEVIEW
     if (vwin->sbuf != NULL) {
 	if (gtk_source_buffer_can_undo(vwin->sbuf)) {
 	    gtk_source_buffer_undo(vwin->sbuf);
@@ -152,7 +187,6 @@ void text_undo (windata_t *vwin, guint u, GtkWidget *widget)
 	}
 	return;
     }
-#endif
     
     old = g_object_steal_data(G_OBJECT(vwin->w), "undo");
 
@@ -178,12 +212,12 @@ void text_undo (windata_t *vwin, guint u, GtkWidget *widget)
     }
 }
 
-#ifdef USE_GTKSOURCEVIEW
-
-static int source_buffer_load_file (GtkSourceBuffer *sbuf, FILE *fp)
+static int source_buffer_load_file (GtkSourceBuffer *sbuf, 
+				    int role, FILE *fp)
 {
     char readbuf[MAXSTR], *chunk = NULL;
-    GtkTextIter iter;   
+    GtkTextIter iter;
+    int i = 0;
 
     gtk_source_buffer_begin_not_undoable_action(sbuf);
 
@@ -195,22 +229,34 @@ static int source_buffer_load_file (GtkSourceBuffer *sbuf, FILE *fp)
     while (fgets(readbuf, sizeof readbuf, fp)) {
 	int len;
 
-# ifdef ENABLE_NLS
+#ifdef ENABLE_NLS
 	if (!g_utf8_validate(readbuf, -1, NULL)) {
-	    chunk = my_locale_to_utf8(readbuf);
+	    if (i == 0) {
+		if (role == GR_PLOT) {
+		    chunk = gp_locale_to_utf8(readbuf);
+		} else {
+		    chunk = my_locale_to_utf8(readbuf);
+		}
+		i++;
+	    } else {
+		if (role == GR_PLOT) {
+		    chunk = gp_locale_to_utf8_next(readbuf);
+		} else {
+		    chunk = my_locale_to_utf8_next(readbuf);
+		}
+	    }
 	    if (chunk == NULL) {
 		continue;
 	    }
 	} else {
 	    chunk = readbuf;
 	}
-# else
+#else
 	chunk = readbuf;
-# endif /* ENABLE_NLS */
+#endif /* ENABLE_NLS */
 
-	/* check that this works */
 	len = strlen(chunk);
-	if (chunk[len - 2] == '\r') {
+	if (len >= 2 && chunk[len - 2] == '\r') {
 	    chunk[len - 2] = '\n';
 	    chunk[len - 1] = '\0';
 	}
@@ -249,6 +295,8 @@ static int source_buffer_load_buf (GtkSourceBuffer *sbuf, const char *buf)
 	gtk_text_buffer_insert(GTK_TEXT_BUFFER(sbuf), &iter, "\n", 1);
     }
 
+    bufgets_finalize(buf);
+
     gtk_source_buffer_end_not_undoable_action(sbuf);
     gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(sbuf), FALSE);
 
@@ -294,7 +342,7 @@ real_sourceview_insert (windata_t *vwin, const char *fname, const char *buf)
     }
 
     if (fp != NULL) {
-	source_buffer_load_file(vwin->sbuf, fp);
+	source_buffer_load_file(vwin->sbuf, vwin->role, fp);
 	fclose(fp);
     } else {
 	source_buffer_load_buf(vwin->sbuf, buf);
@@ -346,7 +394,7 @@ void create_source (windata_t *vwin, int hsize, int vsize,
     vwin->w = gtk_source_view_new_with_buffer(sbuf);
     vwin->sbuf = sbuf;
 
-    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(vwin->w), GTK_WRAP_WORD);
+    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(vwin->w), GTK_WRAP_NONE);
     gtk_text_view_set_left_margin(GTK_TEXT_VIEW(vwin->w), 4);
     gtk_text_view_set_right_margin(GTK_TEXT_VIEW(vwin->w), 4);
 
@@ -362,8 +410,6 @@ void create_source (windata_t *vwin, int hsize, int vsize,
     g_object_unref(cmap);
 }
 
-#endif /* USE_GTKSOURCEVIEW */
-
 static gchar *my_utf_string (char *t)
 {
     static gchar *s = NULL;
@@ -453,7 +499,7 @@ static GtkTextTagTable *gretl_tags_new (void)
     gtk_text_tag_table_add(table, tag);
 
     tag = gtk_text_tag_new("sansbold");
-    g_object_set(tag, "family", "sans",
+    g_object_set(tag, "family", "sans", 
 		 "weight", PANGO_WEIGHT_BOLD, 
 		 NULL);
     gtk_text_tag_table_add(table, tag);
@@ -470,6 +516,13 @@ static GtkTextTagTable *gretl_tags_new (void)
 		 NULL);
     gtk_text_tag_table_add(table, tag);
 
+    tag = gtk_text_tag_new("superscript");
+    g_object_set(tag, "style", PANGO_STYLE_NORMAL,
+		 "rise", 4 * PANGO_SCALE,
+		 "size", 8 * PANGO_SCALE,
+		 NULL);
+    gtk_text_tag_table_add(table, tag);
+		 
     tag = gtk_text_tag_new("literal");
     g_object_set(tag, "family", "monospace", NULL);
     gtk_text_tag_table_add(table, tag);
@@ -479,7 +532,7 @@ static GtkTextTagTable *gretl_tags_new (void)
     gtk_text_tag_table_add(table, tag);
 
     tag = gtk_text_tag_new("indented");
-    g_object_set(tag, "left_margin", 30, NULL);
+    g_object_set(tag, "left_margin", 16, "indent", -12, NULL);
     gtk_text_tag_table_add(table, tag);
 
     tag = gtk_text_tag_new("code");
@@ -489,7 +542,7 @@ static GtkTextTagTable *gretl_tags_new (void)
     return table;
 }
 
-static GtkTextBuffer *gretl_text_buf_new (void)
+GtkTextBuffer *gretl_text_buf_new (void)
 {
     static GtkTextTagTable *tags = NULL;
     GtkTextBuffer *tbuf; 
@@ -503,19 +556,26 @@ static GtkTextBuffer *gretl_text_buf_new (void)
     return tbuf;
 }
 
-void textview_set_text_colorized (GtkWidget *view, const char *buf)
+static void 
+real_textview_add_colorized (GtkWidget *view, const char *buf,
+			     int append)
 {
     GtkTextBuffer *tbuf;
-    GtkTextIter iter;    
-    int thiscolor, nextcolor;
+    GtkTextIter iter; 
+    int thiscolor = PLAIN_TEXT;
+    int nextcolor;
     char readbuf[MAXSTR];
 
     g_return_if_fail(GTK_IS_TEXT_VIEW(view));
 
     tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
 
-    thiscolor = PLAIN_TEXT;
-    gtk_text_buffer_get_iter_at_offset(tbuf, &iter, 0);
+    if (append) {
+	gtk_text_buffer_get_end_iter(tbuf, &iter);
+    } else {
+	gtk_text_buffer_get_iter_at_offset(tbuf, &iter, 0);
+    }
+
     bufgets_init(buf);
 
     while (bufgets(readbuf, sizeof readbuf, buf)) {
@@ -543,6 +603,18 @@ void textview_set_text_colorized (GtkWidget *view, const char *buf)
 
 	thiscolor = nextcolor;
     }
+
+    bufgets_finalize(buf);
+}
+
+void textview_set_text_colorized (GtkWidget *view, const char *buf)
+{
+    real_textview_add_colorized(view, buf, 0);
+}
+
+void textview_append_text_colorized (GtkWidget *view, const char *buf)
+{
+    real_textview_add_colorized(view, buf, 1);
 }
 
 void textview_insert_file (windata_t *vwin, const char *fname)
@@ -608,7 +680,7 @@ void textview_insert_file (windata_t *vwin, const char *fname)
 }
 
 static void insert_link (GtkTextBuffer *tbuf, GtkTextIter *iter, 
-			 const gchar *text, gint page,
+			 const char *text, gint page, 
 			 const char *indent)
 {
     GtkTextTag *ltag;
@@ -616,6 +688,11 @@ static void insert_link (GtkTextBuffer *tbuf, GtkTextIter *iter,
     if (page == GUIDE_PAGE) {
 	ltag = gtk_text_buffer_create_tag(tbuf, NULL, "foreground", "blue", 
 					  "family", "sans", NULL);
+    } else if (page == SCRIPT_PAGE) {
+	ltag = gtk_text_buffer_create_tag(tbuf, NULL, "foreground", "blue", 
+					  "family", "monospace", NULL);
+	g_object_set_data_full(G_OBJECT(ltag), "fname", g_strdup(text), 
+			       g_free);
     } else if (indent != NULL) {
 	ltag = gtk_text_buffer_create_tag(tbuf, NULL, "foreground", "blue", 
 					  "left_margin", 30, NULL);
@@ -627,6 +704,16 @@ static void insert_link (GtkTextBuffer *tbuf, GtkTextIter *iter,
     gtk_text_buffer_insert_with_tags(tbuf, iter, text, -1, ltag, NULL);
 }
 
+static void link_open_script (GtkTextTag *tag)
+{
+    const char *fname = g_object_get_data(G_OBJECT(tag), "fname");
+    char fullname[MAXLEN];
+
+    sprintf(fullname, "%sscripts%cmisc%c%s", paths.gretldir, 
+	    SLASH, SLASH, fname);
+    view_file(fullname, 0, 0, 78, 370, VIEW_SCRIPT);
+}
+
 static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, gpointer p)
 {
     GSList *tags = NULL, *tagp = NULL;
@@ -640,6 +727,8 @@ static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, gpointer p)
 	if (page != 0) {
 	    if (page == GUIDE_PAGE) {
 		display_pdf_help(NULL, 1, NULL);
+	    } else if (page == SCRIPT_PAGE) {
+		link_open_script(tag);
 	    } else {
 		plain_text_cmdref(p, page, NULL);
 	    }
@@ -648,7 +737,7 @@ static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, gpointer p)
     }
 
     if (tags) {
-	g_slist_free (tags);
+	g_slist_free(tags);
     }
 }
 
@@ -955,8 +1044,10 @@ enum {
     INSERT_REPL,
     INSERT_LIT,
     INSERT_ITAL,
+    INSERT_SUP,
     INSERT_TEXT,
-    INSERT_PDFLINK
+    INSERT_PDFLINK,
+    INSERT_INPLINK
 };
 
 static void insert_help_figure (GtkTextBuffer *tbuf, GtkTextIter *iter,
@@ -991,6 +1082,9 @@ static void insert_tagged_text (GtkTextBuffer *tbuf, GtkTextIter *iter,
     case INSERT_LIT:
 	ftag = "literal";
 	break;
+    case INSERT_SUP:
+	ftag = "superscript";
+	break;
     default:
 	break;
     }
@@ -1016,8 +1110,12 @@ static int get_instruction_and_string (const char *p, char *str)
 	ins = INSERT_REPL;
     } else if (!strncmp(p, "lit", 3)) {
 	ins = INSERT_LIT;
+    } else if (!strncmp(p, "sup", 3)) {
+	ins = INSERT_SUP;
     } else if (!strncmp(p, "pdf", 3)) {
 	ins = INSERT_PDFLINK;
+    } else if (!strncmp(p, "inp", 3)) {
+	ins = INSERT_INPLINK;
     }
 
     if (ins != INSERT_NONE) {
@@ -1058,7 +1156,7 @@ static void
 insert_text_with_markup (GtkTextBuffer *tbuf, GtkTextIter *iter,
 			 const char *s)
 {
-    char targ[128];
+    static char targ[128];
     const char *p;
     int ins;
 
@@ -1085,6 +1183,8 @@ insert_text_with_markup (GtkTextBuffer *tbuf, GtkTextIter *iter,
 		insert_link(tbuf, iter, targ, gretl_command_number(targ), indent);
 	    } else if (ins == INSERT_PDFLINK) {
 		insert_link(tbuf, iter, targ, GUIDE_PAGE, indent);
+	    } else if (ins == INSERT_INPLINK) {
+		insert_link(tbuf, iter, targ, SCRIPT_PAGE, indent);
 	    } else if (ins == INSERT_FIG) {
 		insert_help_figure(tbuf, iter, targ);
 	    } else if (ins != INSERT_NONE) {
@@ -1139,28 +1239,31 @@ static char *grab_topic_buffer (const char *s)
 
 void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en)
 {
-    GtkTextBuffer *tbuf;
+    GtkTextBuffer *textb;
     GtkTextIter iter;
     char line[256];
     gchar *hbuf;
-    char *topicbuf;
+    char *buf;
 
-    tbuf = gretl_text_buf_new();
+    textb = gretl_text_buf_new();
 
     if (pos == 0) {
 	/* cli help with no topic selected */
-	cmdref_title_page(hwin, tbuf, en);
+	cmdref_title_page(hwin, textb, en);
 	cursor_to_top(hwin);
 	hwin->active_var = 0;
 	return;
     }
     
-    gtk_text_buffer_get_iter_at_offset(tbuf, &iter, 0);
+    gtk_text_buffer_get_iter_at_offset(textb, &iter, 0);
 
     hbuf = (gchar *) hwin->data + pos;
-    bufgets_init(hbuf);
 
-    if (bufgets(line, sizeof line, hbuf) == NULL) {
+    bufgets_init(hbuf);
+    buf = bufgets(line, sizeof line, hbuf);
+    bufgets_finalize(hbuf);
+
+    if (buf == NULL) {
 	return;
     }
 
@@ -1168,7 +1271,7 @@ void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en)
 	/* topic heading: descriptive string */
 	gchar *p = quoted_help_string(line);
 
-	gtk_text_buffer_insert_with_tags_by_name(tbuf, &iter,
+	gtk_text_buffer_insert_with_tags_by_name(textb, &iter,
 						 p, -1,
 						 "sansbold", NULL);
 	free(p);
@@ -1177,22 +1280,22 @@ void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en)
 	char hword[9];
 
 	sscanf(line + 2, "%8s", hword);
-	gtk_text_buffer_insert_with_tags_by_name(tbuf, &iter,
+	gtk_text_buffer_insert_with_tags_by_name(textb, &iter,
 						 hword, -1,
 						 "redtext", NULL);
     }
 
-    gtk_text_buffer_insert(tbuf, &iter, "\n", 1);
+    gtk_text_buffer_insert(textb, &iter, "\n", 1);
 
-    topicbuf = grab_topic_buffer(hbuf + strlen(line) + 1);
-    if (topicbuf == NULL) {
+    buf = grab_topic_buffer(hbuf + strlen(line) + 1);
+    if (buf == NULL) {
 	return;
     }
 
-    insert_text_with_markup(tbuf, &iter, topicbuf);
-    free(topicbuf);
+    insert_text_with_markup(textb, &iter, buf);
+    free(buf);
 
-    gtk_text_view_set_buffer(GTK_TEXT_VIEW(hwin->w), tbuf);
+    gtk_text_view_set_buffer(GTK_TEXT_VIEW(hwin->w), textb);
     g_object_set_data(G_OBJECT(hwin->w), "backpage", 
 		      GINT_TO_POINTER(hwin->active_var));
     maybe_connect_help_signals(hwin, en);
@@ -1200,39 +1303,14 @@ void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en)
     hwin->active_var = hcode;
 }
 
-#ifndef USE_GTKSOURCEVIEW
-
-void correct_line_color (windata_t *vwin)
-{
-    GtkTextBuffer *buf;
-    GtkTextIter start, end;
-    gint linelen;
-    gchar *txt;
-
-    buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
-    gtk_text_buffer_get_iter_at_mark(buf, &end, 
-				     gtk_text_buffer_get_insert(buf));
-    linelen = gtk_text_iter_get_chars_in_line(&end);
-    start = end;
-    gtk_text_iter_backward_chars(&start, linelen);
-
-    txt = gtk_text_buffer_get_text(buf, &start, &end, FALSE);
-
-    if (*txt == '#') {
-	gtk_text_buffer_apply_tag_by_name (buf, "bluetext",
-					   &start, &end);
-    }
-    g_free(txt);
-}
-
-#endif /* not USE_GTKSOURCEVIEW */
-
-GtkWidget *create_text (GtkWidget *dlg, int hsize, int vsize, 
-			gboolean editable)
+void create_text (windata_t *vwin, int hsize, int vsize, 
+		  gboolean editable)
 {
     GtkTextBuffer *tbuf = gretl_text_buf_new();
     GtkWidget *w = gtk_text_view_new_with_buffer(tbuf);
 
+    vwin->w = w;
+
     gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(w), GTK_WRAP_WORD);
     gtk_text_view_set_left_margin(GTK_TEXT_VIEW(w), 4);
     gtk_text_view_set_right_margin(GTK_TEXT_VIEW(w), 4);
@@ -1244,13 +1322,19 @@ GtkWidget *create_text (GtkWidget *dlg, int hsize, int vsize,
 	hsize += 48;
     }
 
-    if (dlg != NULL) {
-	gtk_window_set_default_size(GTK_WINDOW(dlg), hsize, vsize); 
-    }
+    gtk_window_set_default_size(GTK_WINDOW(vwin->dialog), hsize, vsize); 
+
     gtk_text_view_set_editable(GTK_TEXT_VIEW(w), editable);
     gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(w), editable);
+}
 
-    return w;
+void text_set_word_wrap (GtkWidget *w, gboolean wrap)
+{
+    if (wrap) {
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(w), GTK_WRAP_WORD);
+    } else {
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(w), GTK_WRAP_NONE);
+    }
 }
 
 void text_table_setup (GtkWidget *vbox, GtkWidget *w)
