@@ -25,7 +25,6 @@
 #include "datafiles.h"
 
 #ifdef G_OS_WIN32
-# include "webget.h"
 # include "gretlwin32.h"
 #endif
 
@@ -48,7 +47,7 @@ static GtkWidget *toolbar_box;
 static void show_calc (void)
 {
 #ifdef G_OS_WIN32
-    create_child_process(calculator, NULL);
+    create_child_process(calculator);
 #else
     gretl_fork(calculator, NULL);
 #endif 
