@@ -111,7 +111,6 @@ static void update_dialogs_from_varclick (int active_var)
 	edttext = gtk_entry_get_text(GTK_ENTRY(active_edit_name));
 	my_gtk_entry_append_text(GTK_ENTRY(active_edit_name), 
 				 datainfo->varname[active_var]);
-	my_gtk_entry_append_text(GTK_ENTRY(active_edit_name), " ");
     } else if (active_edit_text != NULL) {
 	GtkTextBuffer *tbuf;
 
@@ -288,7 +287,9 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
     GtkTreeSelection *select;
     int i, viscols = ncols;
 
-    if (hidden_col) viscols--;
+    if (hidden_col) {
+	viscols--;
+    }
 
     if (tree) {
 	tstore = gtk_tree_store_newv(ncols, types);
@@ -394,7 +395,6 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
 					GTK_SHADOW_IN);    
 
     gtk_container_add(GTK_CONTAINER(scroller), view);
-
     gtk_box_pack_start(box, scroller, TRUE, TRUE, TRUE);
 
     gtk_widget_show(view);
