@@ -19,6 +19,7 @@
 
 #include "libgretl.h"
 #include "libset.h"
+#include "matrix_extra.h"
 #include "bhhh_max.h"
 
 #define BHHH_DEBUG 0
@@ -462,28 +463,6 @@ static int expand_model_series (MODEL *pmod, model_info *minfo)
     return 0;
 }
 
-static int bhhh_iter_info (int iter, double *theta, int m, double ll,
-			   double steplength, PRN *prn)
-{
-    int i;
-
-    pprintf(prn, "\n*** %s %d: theta, ll ***\n", _("iteration"), iter);
-
-    for (i=0; i<m; i++) {
-	if (i && i % 5 == 0) pputc(prn, '\n');
-	if (na(theta[i]) || isnan(theta[i])) {
-	    pprintf(prn, "Invalid value for theta[%d]\n", i);
-	    return 1;
-	}
-	pprintf(prn, "%#12.5g ", theta[i]);
-    }
-
-    pprintf(prn, "\n    %s = %g, ll = %g\n", _("step length"),
-	    steplength, ll);
-
-    return 0;
-}
-
 /**
  * bhhh_max:
  * @loglik: pointer to function for calculating log-likelihood and
@@ -514,8 +493,8 @@ static int bhhh_iter_info (int iter, double *theta, int m, double ll,
  * member, %Z[0], represents the constant or intercept, and should
  * not be modified.  
  *
- * For examples of the use of this function, see arma.c and tobit.c
- * in the %plugin directory of the gretl source.
+ * For an example of the use of this function, see arma.c in the
+ * %plugin directory of the gretl source.
  * 
  * Returns: 0 on successful completion, non-zero error code otherwise.
  */
@@ -535,9 +514,9 @@ int bhhh_max (LL_FUNC loglik,
     DATAINFO *tinfo = NULL;
 
     int iters, itermax;
-    double minstep = 1.0e-06; /* in arma, was 1.0e-08 */
+    double minstep = 1.0e-06;
     double crit = 1.0;
-    double stepsize = 0.25;   /* in arma, was 0.125 */
+    double stepsize = 0.25;
 
     double *delta = NULL, *ctemp = NULL;
 
@@ -664,7 +643,8 @@ int bhhh_max (LL_FUNC loglik,
 	}
 
 	/* print interation info, if wanted */
-	bhhh_iter_info(iters, minfo->theta, k, minfo->ll, stepsize, prn);
+	print_iter_info(iters, minfo->ll, C_LOGLIK, k, minfo->theta, delta, 
+			stepsize, prn);
 
 	crit = minfo->ll2 - minfo->ll;  
     }
