@@ -26,11 +26,11 @@
 #include "gretl_panel.h"
 #include "var.h"
 #include "missing_private.h"
+#include "matrix_extra.h"
 
 #define WDEBUG 0
 
-#define print_add_omit_model(m,o) (m->ci != ARCH && \
-                                   !(opt & OPT_Q) && !(opt & OPT_I))
+#define print_add_omit_model(m,o) (!(opt & OPT_Q) && !(opt & OPT_I))
 
 enum {
     CHISQ_FORM,
@@ -40,7 +40,8 @@ enum {
 enum {
     ADD_TEST,
     OMIT_TEST,
-    OMIT_WALD
+    OMIT_WALD,
+    ADD_WALD
 };
 
 struct COMPARE {
@@ -100,6 +101,7 @@ static char *
 mask_from_test_list (const int *list, const MODEL *pmod, int *err)
 {
     char *mask;
+    int off1 = 2, off2 = 0;
     int nmask = 0;
     int i, j;
 
@@ -109,18 +111,30 @@ mask_from_test_list (const int *list, const MODEL *pmod, int *err)
 	return NULL;
     }
 
+    if (pmod->ci == ARBOND) {
+	/* find correct offset into independent vars in list */
+	for (i=2; i<=pmod->list[0]; i++) {
+	    if (pmod->list[i] == LISTSEP) {
+		off1 = i + 2;
+	    }
+	}
+	off2 = pmod->list[1];
+    }
+
     for (i=0; i<pmod->ncoeff; i++) {
 	if (list != NULL) {
 	    for (j=1; j<=list[0]; j++) {
-		if (pmod->list[i+2] == list[j]) {
-		    mask[i] = 1;
+		if (pmod->list[i + off1] == list[j]) {
+#if WDEBUG
+		    fprintf(stderr, "matched var %d at pmod->list[%d]: set mask[%d] = 1\n", 
+			    list[j], i + off1, i + off2);
+#endif
+		    mask[i + off2] = 1;
 		    nmask++;
 		}
 	    }
-	} else {
-	    if (pmod->list[i+2] != 0) {
-		mask[i] = 1;
-	    }
+	} else if (pmod->list[i + off1] != 0) {
+	    mask[i + off2] = 1;
 	}
     }
 
@@ -173,7 +187,7 @@ static double wald_test (const int *list, MODEL *pmod, int form,
     }
 
     if (!*err) {
-	w = gretl_scalar_b_X_b(b, GRETL_MOD_TRANSPOSE, C, err);
+	w = gretl_scalar_qform(b, C, err);
     }
 
     if (!*err && form == F_FORM) {
@@ -267,6 +281,11 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 
     stat_ok = !na(cmp->F) || !na(cmp->chisq);
 
+    if (!stat_ok && (!verbosity || cmp->score < 0)) {
+	/* nothing to show */
+	return;
+    }
+
     if (stat_ok && (opt & OPT_S)) {
 	test = model_test_new((cmp->cmd == OMIT)? GRETL_TEST_OMIT : GRETL_TEST_ADD);
     }
@@ -348,7 +367,7 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int flag,
 
     cmp.err = 0;
 
-    if (flag == ADD_TEST) {
+    if (flag == ADD_TEST || flag == ADD_WALD) {
 	umod = pmodB;
 	rmod = pmodA;
 	cmp.cmd = ADD;
@@ -377,9 +396,9 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int flag,
 
     cmp.dfd = umod->dfd;
 
-    /* FIXME TSLS (F or chi-square?) */
+    /* FIXME TSLS (use F or chi-square?) */
 
-    if (flag == OMIT_WALD) {
+    if (flag == OMIT_WALD || flag == ADD_WALD) {
 	cmp.chisq = wald_test(testvars, umod, CHISQ_FORM, &cmp.err);
     } else if (gretl_model_get_int(pmodA, "robust") || pmodA->ci == HCCM) {
 	cmp.F = robust_omit_F(testvars, umod);
@@ -392,16 +411,22 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int flag,
 	cmp.chisq = wald_test(testvars, umod, CHISQ_FORM, &cmp.err);
     }
 
-    if (pmodB != NULL) {
+    if (pmodB != NULL && flag != ADD_WALD) {
+	int miss = 0;
+
 	cmp.score = 0;
 	for (i=0; i<C_MAX; i++) { 
 	    if (na(pmodB->criterion[i]) || na(pmodA->criterion[i])) {
+		miss++;
 		continue;
 	    }
 	    if (pmodB->criterion[i] < pmodA->criterion[i]) {
 		cmp.score++;
 	    }
 	}
+	if (miss == C_MAX) {
+	    cmp.score = -1;
+	}
     }
 
     return cmp;
@@ -462,23 +487,64 @@ full_model_list (const MODEL *pmod, const int *inlist)
     return flist;
 }
 
+static gretlopt retrieve_arbond_opts (MODEL *pmod)
+{
+    gretlopt opt = OPT_NONE;
+
+    if (gretl_model_get_int(pmod, "time-dummies")) {
+	opt |= OPT_D;
+    }
+
+    if (gretl_model_get_int(pmod, "asy")) {
+	opt |= OPT_A;
+    }
+
+    if (gretl_model_get_int(pmod, "step") == 2) {
+	opt |=OPT_T;
+    }    
+
+    return opt;
+}
+
+static int obs_diff_ok (const MODEL *old, const MODEL *new)
+{
+    int tdiff, ndiff = new->nobs - old->nobs;
+
+    if (old->ci == CORC || old->ci == HILU || old->ci == PWE) {
+	return 0;
+    }
+
+    if (ndiff > 0) {
+	tdiff = (new->t2 - new->t1) - (old->t2 - old->t1);
+	if (ndiff == tdiff) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 #define be_quiet(o) ((o & OPT_A) || (o & OPT_Q))
 
-#undef SMPL_DEBUG
+#define SMPL_DEBUG 0
 
 static MODEL replicate_estimator (MODEL *orig, int **plist,
 				  double ***pZ, DATAINFO *pdinfo,
 				  gretlopt myopt, PRN *prn)
 {
     MODEL rep;
+    const char *param = NULL;
     double rho = 0.0;
     int *list = *plist;
     int mc = get_model_count();
     int repci = orig->ci;
     int order = 0;
+    int first = 1;
 
     gretl_model_init(&rep);
 
+    /* recreate options and auxiliary vars, if required */
+
     if (orig->ci == CORC || orig->ci == HILU || orig->ci == PWE) {
 	gretlopt hlopt = OPT_NONE;
 
@@ -500,29 +566,46 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	} else {
 	    list = *plist = full_list;
 	}
+    } else if (orig->ci == ARBOND) {
+	param = gretl_model_get_data(orig, "istr");
+	myopt |= retrieve_arbond_opts(orig);
+    } else if (orig->ci == ARCH) {
+	order = gretl_model_get_int(orig, "arch_order");
+    } else if (orig->ci == LOGIT || orig->ci == PROBIT) {
+	if (gretl_model_get_int(orig, "robust")) {
+	    myopt = OPT_R;
+	} else if (gretl_model_get_int(orig, "ordered")) {
+	    myopt = OPT_D;
+	} else {
+	    myopt = OPT_NONE;
+	}
+    } else if (orig->ci == PANEL) {
+	if (gretl_model_get_int(orig, "random-effects")) {
+	    myopt |= OPT_U;
+	}
     }
 
     if (rep.errcode) {
 	return rep;
     }
 
+ try_again:
+
     switch (orig->ci) {
 
     case AR:
 	rep = ar_func(list, pZ, pdinfo, myopt, prn);
 	break;
+    case ARBOND:
+	rep = arbond_model(list, param, (const double **) *pZ, 
+			   pdinfo, myopt, prn);
+	break;
     case ARCH:
-	order = gretl_model_get_int(orig, "arch_order");
 	rep = arch_model(list, order, pZ, pdinfo, myopt, prn);
 	break;
     case LOGIT:
     case PROBIT:
-	if (gretl_model_get_int(orig, "robust")) {
-	    myopt = OPT_R;
-	} else {
-	    myopt = OPT_NONE;
-	}
-	rep = logit_probit(list, pZ, pdinfo, orig->ci, myopt);
+	rep = logit_probit(list, pZ, pdinfo, orig->ci, myopt, NULL);
 	break;
     case TOBIT:
 	rep = tobit_model(list, pZ, pdinfo, NULL);
@@ -547,10 +630,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	}
 	break;
     case PANEL:
-	if (gretl_model_get_int(orig, "random-effects")) {
-	    myopt |= OPT_R;
-	}
-	rep = real_panel_model(list, pZ, pdinfo, myopt, NULL);
+	rep = real_panel_model(list, pZ, pdinfo, myopt, prn);
 	break;
     default:
 	/* handles OLS, WLS, HSK, HCCM, etc. */
@@ -578,10 +658,15 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 #endif
 
     /* check that we got the same sample as the original */
-    if (!rep.errcode) {
-	if (rep.nobs < orig->nobs) {
-	    rep.errcode = E_MISS;
-	} else if (rep.nobs > orig->nobs) {
+    if (!rep.errcode && rep.nobs != orig->nobs) {
+	if (first && obs_diff_ok(orig, &rep)) {
+	    pdinfo->t1 = orig->t1;
+	    pdinfo->t2 = orig->t2;
+	    clear_model(&rep);
+	    first = 0;
+	    goto try_again;
+	} else {
+	    fprintf(stderr, "Original obs = %d but new = %d\n", orig->nobs, rep.nobs);
 	    rep.errcode = E_DATA;
 	}
     } 
@@ -595,6 +680,17 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
     return rep;
 }
 
+static void nonlin_test_header (int code, PRN *prn)
+{
+    pputc(prn, '\n');
+    if (code == AUX_SQ) {
+	pputs(prn, _("Non-linearity test (squared terms)"));
+    } else {
+	pputs(prn, _("Non-linearity test (log terms)"));
+    }
+    pputs(prn, "\n\n");
+}
+
 static int
 real_nonlinearity_test (MODEL *pmod, int *list,
 			double ***pZ, DATAINFO *pdinfo,
@@ -626,9 +722,14 @@ real_nonlinearity_test (MODEL *pmod, int *list,
 	double pval = chisq_cdf_comp(trsq, df);
 
 	aux.aux = aux_code;
-	printmodel(&aux, pdinfo, opt, prn);
 
-	pprintf(prn, "\n%s: TR^2 = %g,\n", _("Test statistic"), trsq);
+	if (opt & OPT_Q) {
+	    nonlin_test_header(aux_code, prn);
+	} else {
+	    printmodel(&aux, pdinfo, opt, prn);
+	}
+
+	pprintf(prn, "%s: TR^2 = %g,\n", _("Test statistic"), trsq);
 	pprintf(prn, _("with p-value = prob(Chi-square(%d) > %g) = %g\n\n"), 
 		df, trsq, pval);
 
@@ -679,7 +780,7 @@ int nonlinearity_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     const int orig_nvar = pdinfo->v; 
     int err = 0;
 
-    if (!command_ok_for_model(ADD, pmod->ci)) {
+    if (!command_ok_for_model(ADD, 0, pmod->ci)) {
 	return E_NOTIMP;
     }
 
@@ -743,6 +844,9 @@ static void remove_special_flags (gretlopt *popt)
     if (opt & OPT_B) {
 	opt &= ~OPT_B;
     } 
+    if (opt & OPT_P) {
+	opt &= ~OPT_P;
+    }
 
     *popt = opt;
 }
@@ -783,7 +887,7 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
 	return 1;
     }
 
-    if (!command_ok_for_model(ADD, orig->ci)) {
+    if (!command_ok_for_model(ADD, 0, orig->ci)) {
 	return E_NOTIMP;
     }
 
@@ -796,7 +900,7 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
     /* create augmented regression list */
     if (orig->ci == TSLS) {
 	tmplist = tsls_list_add(orig->list, addvars, opt, &err);
-    } else if (orig->ci == PANEL) {
+    } else if (orig->ci == PANEL || orig->ci == ARBOND) {
 	tmplist = panel_list_add(orig, addvars, &err);
     } else {
 	tmplist = gretl_list_add(orig->list, addvars, &err);
@@ -821,11 +925,15 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
 			       (est_opt | OPT_Z), prn);
 
     if (new->errcode) {
-	pprintf(prn, "%s\n", gretl_errmsg);
-	err = new->errcode; 
+	err = new->errcode;
+	errmsg(err, prn);
     }
 
     if (!err) {
+	int flag = (orig->ci == OLS && new->nobs == orig->nobs)? 
+	    ADD_TEST : ADD_WALD;
+	struct COMPARE cmp;
+	int *addlist;
 
 	new->aux = AUX_ADD;
 
@@ -833,13 +941,9 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
 	    printmodel(new, pdinfo, est_opt, prn);
 	}
 
-	if (new->nobs == orig->nobs) {
-	    struct COMPARE cmp;
-	    int *addlist;
-
-	    addlist = gretl_list_diff_new(new->list, orig->list, 2);
-	    cmp = add_or_omit_compare(orig, new, ADD_TEST, addlist);
-
+	addlist = gretl_list_diff_new(new->list, orig->list, 2);
+	if (addlist != NULL) {
+	    cmp = add_or_omit_compare(orig, new, flag, addlist);
 	    gretl_make_compare(&cmp, addlist, orig, pdinfo, opt, prn);
 	    free(addlist);
 	}
@@ -866,7 +970,15 @@ static int wald_omit_test (const int *list, MODEL *pmod,
     int err = 0;
 
     /* test validity of omissions */
-    test = gretl_list_omit(pmod->list, list, 2, &err);
+
+    if (pmod->ci == TSLS) {
+	test = tsls_list_omit(pmod->list, list, opt, &err);
+    } else if (pmod->ci == PANEL || pmod->ci == ARBOND) {
+	test = panel_list_omit(pmod, list, &err);
+    } else {
+	test = gretl_list_omit(pmod->list, list, 2, &err);
+    }
+
     if (err) {
 	return err;
     }
@@ -928,12 +1040,11 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
     int smpl_t2 = pdinfo->t2;
     int omitlast = 0;
     int *tmplist = NULL;
-    int maxlag = 0;
     int err = 0;
 
     if (orig == NULL || orig->list == NULL) {
 	err = E_DATA;
-    } else if (!command_ok_for_model(OMIT, orig->ci)) {
+    } else if (!command_ok_for_model(OMIT, 0, orig->ci)) {
 	err = E_NOTIMP;
     } else if (omit_options_inconsistent(opt)) {
 	err = E_BADOPT;
@@ -963,7 +1074,7 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
     /* create list for test model */
     if (orig->ci == TSLS) {
 	tmplist = tsls_list_omit(orig->list, omitvars, opt, &err);
-    } else if (orig->ci == PANEL) {
+    } else if (orig->ci == PANEL || orig->ci == ARBOND) {
 	tmplist = panel_list_omit(orig, omitvars, &err);
     } else if (omitlast) {
 	/* special: just drop the last variable */
@@ -980,27 +1091,13 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 	return E_NOVARS;
     }
 
-    /* impose as sample range the estimation range of the 
-       original model */
-    pdinfo->t1 = orig->t1;
-    pdinfo->t2 = orig->t2;
+    /* impose the sample range used for the original model */ 
+    impose_model_smpl(orig, pdinfo);
 
     /* set the mask for missing obs within the sample range, based
        on the original model */
     set_reference_missmask(orig);
 
-    if (orig->ci == AR) { 
-	maxlag = orig->arinfo->arlist[orig->arinfo->arlist[0]];
-    } else if (orig->ci == ARCH) {
-	maxlag = gretl_model_get_int(orig, "arch_order");
-    }
-
-    pdinfo->t1 = orig->t1 - maxlag; /* FIXME: problem? */
-
-    if (orig->ci == CORC || orig->ci == HILU) {
-	pdinfo->t1 -= 1;
-    }
-
     /* extract option flags that should not be passed to estimator
        functions */
     remove_special_flags(&est_opt);
@@ -1008,8 +1105,8 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
     *new = replicate_estimator(orig, &tmplist, pZ, pdinfo, est_opt, prn);
 
     if (new->errcode) {
-	pprintf(prn, "%s\n", gretl_errmsg);
-	err = new->errcode; 
+	err = new->errcode;
+	errmsg(err, prn);
     }
 
     if (!err) {
@@ -1021,14 +1118,16 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 	    printmodel(new, pdinfo, est_opt, prn); 
 	}	
 
-	if (new->nobs == orig->nobs && !omitlast) {
+	if (!omitlast) {
 	    struct COMPARE cmp;
-	    int *omitlist = NULL;
+	    int *omitlist;
 
 	    omitlist = gretl_list_diff_new(orig->list, new->list, 2);
-	    cmp = add_or_omit_compare(orig, new, OMIT_TEST, omitlist);
-	    gretl_make_compare(&cmp, omitlist, orig, pdinfo, opt, prn); 
-	    free(omitlist);
+	    if (omitlist != NULL) {
+		cmp = add_or_omit_compare(orig, new, OMIT_TEST, omitlist);
+		gretl_make_compare(&cmp, omitlist, orig, pdinfo, opt, prn); 
+		free(omitlist);
+	    }
 	}
 
 	if (orig->ci == LOGIT || orig->ci == PROBIT) {
@@ -1040,55 +1139,29 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
     pdinfo->t1 = smpl_t1;
     pdinfo->t2 = smpl_t2;
 
-    /* reset the missing mask */
-    set_reference_missmask(NULL);
-
     free(tmplist);
 
     return err;
 }
 
-static int ljung_box (int varno, int order, const double **Z, 
-		      DATAINFO *pdinfo, double *lb)
+static int 
+ljung_box (int m, int t1, int t2, const double *y, double *pLB)
 {
-    double *x, *y, *acf;
-    int k, m, nobs, n = pdinfo->n; 
-    int t, t1 = pdinfo->t1, t2 = pdinfo->t2;
-    int list[2];
+    double acf, LB = 0.0;
+    int k, n = t2 - t1 + 1;
 
-    list[0] = 1;
-    list[1] = varno;
-
-    varlist_adjust_sample(list, &t1, &t2, Z);
-    nobs = t2 - t1 + 1;
-
-    x = malloc(n * sizeof *x);
-    y = malloc(n * sizeof *y);
-    acf = malloc((order + 1) * sizeof *acf);
-
-    if (x == NULL || y == NULL || acf == NULL) {
-	return E_ALLOC;
-    }
-
-    for (m=1; m<=order; m++) {
-	for (t=t1+m; t<=t2; t++) {
-	    k = t - (t1 + m);
-	    x[k] = Z[varno][t];
-	    y[k] = Z[varno][t-m];
+    /* calculate acf up to lag m, cumulating LB */
+    for (k=1; k<=m; k++) {
+	acf = gretl_acf(k, t1, t2, y);
+	if (na(acf)) {
+	    return E_MISSDATA;
 	}
-	acf[m] = gretl_corr(0, nobs - m - 1, x, y, NULL);
+	LB += acf * acf / (n - k);
     }
 
-    /* compute Ljung-Box statistic */
-    *lb = 0;
-    for (t=1; t<=order; t++) { 
-	*lb += acf[t] * acf[t] / (nobs - t);
-    }
-    *lb *= nobs * (nobs + 2.0);
+    LB *= n * (n + 2.0);
 
-    free(x);
-    free(y);
-    free(acf);
+    *pLB = LB;
 
     return 0;
 }
@@ -1193,6 +1266,18 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
+static void bg_test_header (int order, PRN *prn)
+{
+    pprintf(prn, "\n%s ", _("Breusch-Godfrey test for"));
+    if (order > 1) {
+	pprintf(prn, "%s %d\n", _("autocorrelation up to order"),
+		order);
+    } else {
+	pprintf(prn, "%s\n", _("first-order autocorrelation"));
+    }
+    pputc(prn, '\n');
+}
+
 /**
  * autocorr_test:
  * @pmod: pointer to model to be tested.
@@ -1222,8 +1307,9 @@ int autocorr_test (MODEL *pmod, int order,
     double trsq, LMF, lb, pval = 1.0;
     int err = 0;
 
-    if (pmod->ci == NLS || pmod->ci == ARMA || pmod->ci == LOGISTIC) 
+    if (pmod->ci != OLS && pmod->ci != VAR) { 
 	return E_NOTIMP;
+    }
 
     if (pmod->missmask != NULL) {
 	return E_DATA;
@@ -1300,19 +1386,24 @@ int autocorr_test (MODEL *pmod, int order,
 	pval = f_cdf_comp(LMF, order, aux.nobs - pmod->ncoeff - order);
 
 	if (pmod->aux != AUX_VAR) {
-	    printmodel(&aux, pdinfo, OPT_NONE, prn);
-	    pprintf(prn, "\n%s: LMF = %f,\n", _("Test statistic"), LMF);
+	    if (opt & OPT_Q) {
+		bg_test_header(order, prn);
+	    } else {
+		printmodel(&aux, pdinfo, OPT_NONE, prn);
+	    } 
+	    pprintf(prn, "%s: LMF = %f,\n", _("Test statistic"), LMF);
 	    pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
 		    order, aux.nobs - pmod->ncoeff - order, LMF, pval);
 	    pprintf(prn, "\n%s: TR^2 = %f,\n", 
 		    _("Alternative statistic"), trsq);
 	    pprintf(prn, "%s = P(%s(%d) > %g) = %.3g\n\n", 	_("with p-value"), 
 		    _("Chi-square"), order, trsq, chisq_cdf_comp(trsq, order));
-	    if (ljung_box(v, order, (const double **) *pZ, pdinfo, &lb) == 0) {
+	    if (ljung_box(order, pmod->t1, pmod->t2, (*pZ)[v], &lb) == 0) {
 		pprintf(prn, "Ljung-Box Q' = %g %s = P(%s(%d) > %g) = %.3g\n", 
 			lb, _("with p-value"), _("Chi-square"), order,
 			lb, chisq_cdf_comp(lb, order));
 	    }
+	    pputc(prn, '\n');
 	    record_test_result(LMF, pval, _("autocorrelation"));
 	}
 
@@ -1342,17 +1433,6 @@ int autocorr_test (MODEL *pmod, int order,
     return err;
 }
 
-#if 0
-/* shift the Chow split-point one period */
-
-static void shift_chow_split (int v, int n, int t, double **Z)
-{
-    for (i=0; i<n; i++) {
-	Z[v+i][t] = 0.0;
-    }
-}
-#endif
-
 /* compose list of variables to be added for Chow test and add
    them to the data set */
 
@@ -1415,6 +1495,34 @@ make_chow_list (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     return chowlist;
 }
 
+static int QLR_graph (const double *Ft, int t1, int t2, 
+		      const DATAINFO *pdinfo)
+{
+    const double *x = gretl_plotx(pdinfo);
+    FILE *fp = NULL;
+    int t;
+
+    if (gnuplot_init(PLOT_REGULAR, &fp)) {
+	return E_FOPEN;
+    }
+
+    gretl_push_c_numeric_locale();
+
+    fprintf(fp, "plot \\\n"
+	    "'-' using 1:2 title '%s' w lines\n",
+	    G_("Chow F-test for break"));
+    for (t=t1; t<=t2; t++) {
+	fprintf(fp, "%g %g\n", x[t], Ft[t-t1]);
+    }
+    fputs("e\n", fp);
+
+    gretl_pop_c_numeric_locale();
+    
+    fclose(fp);
+
+    return gnuplot_make_graph();
+}
+
 static void save_QLR_test (MODEL *pmod, char *datestr,
 			   double Fmax, double crit, double alpha,
 			   int dfn, int dfd)
@@ -1573,9 +1681,14 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
     if (!err && QLR) {
 	/* Quandt likelihood ratio */
 	double Fmax = 0.0;
+	double *Ft = NULL;
 	int dfn = 0, dfd = 0;
 	int tmax = 0;
 	int i, t;
+
+	if (gretl_in_gui_mode()) {
+	    Ft = malloc((smax - split + 1) * sizeof *Ft);
+	}
 	
 	for (t=split; t<=smax; t++) {
 	    chow_mod = lsq(chowlist, pZ, pdinfo, OLS, OPT_A);
@@ -1591,6 +1704,9 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 		tmax = t;
 		Fmax = F;
 	    }
+	    if (Ft != NULL) {
+		Ft[t - split] = F;
+	    }
 #if 0
 	    fprintf(stderr, "split at t=%d: F(%d,%d)=%g\n", t, 
 		    dfn, dfd, F);
@@ -1606,6 +1722,13 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	if (!err) {
 	    QLR_print_result(pmod, Fmax, tmax, dfn, dfd, pdinfo, opt, prn);
 	    record_test_result(Fmax, NADBL, "QLR");
+	    if (Ft != NULL) {
+		QLR_graph(Ft, split, smax, pdinfo);
+	    }
+	}
+
+	if (Ft != NULL) {
+	    free(Ft);
 	}
 
     } else if (!err) {
@@ -1679,6 +1802,139 @@ static double vprime_M_v (double *v, double *M, int n)
     return val;
 }
 
+static int cusum_compute (MODEL *pmod, double *cresid, int T, int k,
+			  double *wbar, double ***pZ, DATAINFO *pdinfo) 
+{
+    MODEL cmod;
+    gretlopt opt = OPT_X | OPT_A;
+    double *xvec;
+    double xx;
+    int n = T - k;
+    int i, j, t;
+    int err = 0;
+
+    xvec = malloc(k * sizeof *xvec);
+    if (xvec == NULL) {
+	return E_ALLOC;
+    }
+
+    for (j=0; j<n && !err; j++) {
+	cmod = lsq(pmod->list, pZ, pdinfo, OLS, opt);
+	if (cmod.errcode) {
+	    err = cmod.errcode;
+	} else {
+	    /* compute ex post prediction error */
+	    t = pdinfo->t2 + 1;
+	    xx = 0.0;
+	    for (i=0; i<cmod.ncoeff; i++) {
+		xvec[i] = (*pZ)[cmod.list[i+2]][t];
+		xx += cmod.coeff[i] * xvec[i];
+	    }
+	    cresid[j] = (*pZ)[pmod->list[1]][t] - xx;
+	    cmod.ci = CUSUM;
+	    err = makevcv(&cmod, 1.0);
+	    xx = vprime_M_v(xvec, cmod.vcv, cmod.ncoeff);
+	    cresid[j] /= sqrt(1.0 + xx);
+	    *wbar += cresid[j];
+	    pdinfo->t2 += 1;
+	}
+	clear_model(&cmod); 
+    }
+
+    free(xvec);
+
+    return err;
+}
+
+static int cusum_do_graph (double a, double b, const double *W, 
+			   int t1, int k, int m, 
+			   DATAINFO *pdinfo, gretlopt opt)
+{
+    FILE *fq = NULL;
+    const double *obs = NULL;
+    double x0 = 0.0;
+    int j, t;
+    int err = 0;
+
+    err = gnuplot_init(PLOT_CUSUM, &fq);
+    if (err) {
+	return err;
+    }
+
+    if (dataset_is_time_series(pdinfo)) { 
+        obs = gretl_plotx(pdinfo);
+	if (obs != NULL) {
+	    x0 = obs[t1+k];
+	}
+    }
+
+    gretl_push_c_numeric_locale();
+
+    fprintf(fq, "set xlabel '%s'\n", G_("Observation"));
+    fputs("set nokey\n", fq);
+
+    if (opt & OPT_R) {
+	fprintf(fq, "set title '%s'\n",
+		/* xgettext:no-c-format */
+		G_("CUSUMSQ plot with 95% confidence band"));
+	fprintf(fq, "plot \\\n%g*(x-%g) title '' w dots lt 2, \\\n", b, x0 - 1);
+	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", -a, b, x0 - 1);
+	fprintf(fq, "%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, b, x0 - 1);
+    } else {
+	fputs("set xzeroaxis\n", fq);
+	fprintf(fq, "set title '%s'\n",
+		/* xgettext:no-c-format */
+		G_("CUSUM plot with 95% confidence band"));
+	fprintf(fq, "plot \\\n%g+%g*(x-%g) title '' w lines lt 2, \\\n", a, b, x0);
+	fprintf(fq, "%g-%g*(x-%g) title '' w lines lt 2, \\\n", -a, b, x0);
+    }	
+
+    fputs("'-' using 1:2 w linespoints lt 1\n", fq);
+    for (j=0; j<m; j++) { 
+	t = t1 + k + j;
+	if (obs != NULL) {
+	    fprintf(fq, "%g %g\n", obs[t], W[j]);
+	} else {
+	    fprintf(fq, "%d %g\n", t, W[j]);
+	}
+    }
+    fputs("e\n", fq);
+
+    gretl_pop_c_numeric_locale();
+
+    fclose(fq);
+
+    err = gnuplot_make_graph();
+
+    return err;
+}
+
+static void cusum_harvey_collier (double wbar, double sigma, int m,
+				  MODEL *pmod, gretlopt opt,
+				  PRN *prn)
+{
+    double hct, pval;
+
+    hct = (sqrt((double) m) * wbar) / sigma;
+    pval = t_pvalue_2(hct, m - 1);
+    pprintf(prn, _("\nHarvey-Collier t(%d) = %g with p-value %.4g\n\n"), 
+	    m - 1, hct, pval);
+
+    if (opt & OPT_S) {
+	ModelTest *test = model_test_new(GRETL_TEST_CUSUM);
+
+	if (test != NULL) {
+	    model_test_set_teststat(test, GRETL_STAT_HARVEY_COLLIER);
+	    model_test_set_dfn(test, m - 1);
+	    model_test_set_value(test, hct);
+	    model_test_set_pvalue(test, pval);
+	    maybe_add_test_to_model(pmod, test);
+	}
+    }
+
+    record_test_result(hct, pval, "Harvey-Collier");
+}
+
 /**
  * cusum_test:
  * @pmod: pointer to model to be tested.
@@ -1687,7 +1943,8 @@ static double vprime_M_v (double *v, double *M, int n)
  * @opt: if flags include %OPT_S, save results of test to model.
  * @prn: gretl printing struct.
  *
- * Tests the given model for parameter stability (CUSUM test).
+ * Tests the given model for parameter stability via the CUSUM test,
+ * or if @opt includes %OPT_R, via the CUSUMSQ test.
  * 
  * Returns: 0 on successful completion, error code on error.
  */
@@ -1695,18 +1952,14 @@ static double vprime_M_v (double *v, double *M, int n)
 int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
 		gretlopt opt, PRN *prn) 
 {
-    int smpl_t1 = pdinfo->t1;
-    int smpl_t2 = pdinfo->t2;
-    int n_est, i, j, t;
-    int xno, yno = pmod->list[1];
-    const int T = pmod->t2 - pmod->t1 + 1;
-    const int K = pmod->ncoeff;
-    MODEL cum_mod;
-    gretlopt cmodopt = OPT_C | OPT_A;
+    int orig_t1 = pdinfo->t1;
+    int orig_t2 = pdinfo->t2;
+    int m, i, j;
+    int T = pmod->t2 - pmod->t1 + 1;
+    int k = pmod->ncoeff;
     char cumdate[OBSLEN];
-    double xx, yy, sigma, hct, wbar = 0.0;
-    double *cresid = NULL, *W = NULL, *xvec = NULL;
-    FILE *fq = NULL;
+    double wbar = 0.0;
+    double *cresid = NULL, *W = NULL;
     int err = 0;
 
     if (pmod->ci != OLS) {
@@ -1717,147 +1970,112 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	return E_DATA;
     }
 
-    n_est = T - K;
+    /* number of forecasts */
+    m = T - k; 
 
     /* set sample based on model to be tested */
     pdinfo->t1 = pmod->t1;
-    pdinfo->t2 = pmod->t1 + K - 1;    
+    pdinfo->t2 = pmod->t1 + k - 1;    
 
-    cresid = malloc(n_est * sizeof *cresid);
-    W = malloc(n_est * sizeof *W);
-    xvec = malloc(K * sizeof *xvec);
+    cresid = malloc(m * sizeof *cresid);
+    W = malloc(m * sizeof *W);
 
-    if (cresid == NULL || W == NULL || xvec == NULL) {
+    if (cresid == NULL || W == NULL) {
 	err = E_ALLOC;
     }
 
     if (!err) {
-	for (j=0; j<n_est && !err; j++) {
-	    cum_mod = lsq(pmod->list, pZ, pdinfo, OLS, cmodopt);
-	    err = cum_mod.errcode;
-	    if (err) {
-		errmsg(err, prn);
-	    } else {
-#if 0
-		printmodel(&cum_mod, pdinfo, OPT_NONE, prn);
-#endif
-		t = pdinfo->t2 + 1;
-		yy = 0.0;
-		for (i=0; i<cum_mod.ncoeff; i++) {
-		    xno = cum_mod.list[i+2];
-		    xvec[i] = (*pZ)[xno][t];
-		    yy += cum_mod.coeff[i] * xvec[i];
-		}
-		cresid[j] = (*pZ)[yno][t] - yy;
-		cum_mod.ci = CUSUM;
-		makevcv(&cum_mod, 1.0);
-		xx = vprime_M_v(xvec, cum_mod.vcv, cum_mod.ncoeff);
-		cresid[j] /= sqrt(1.0 + xx);
-#if 0
-		printf("cresid[%d] = %g\n", j, cresid[j]);
-#endif
-		wbar += cresid[j];
-		clear_model(&cum_mod);
-		pdinfo->t2 += 1;
-	    }
-	    clear_model(&cum_mod); 
+	err = cusum_compute(pmod, cresid, T, k, &wbar, pZ, pdinfo);
+	if (err) {
+	    errmsg(err, prn);
 	}
     }
 
     if (!err) {
-	double pval = NADBL;
+	double a, b, den = 0.0, sigma = 0.0;
+	int sig;
 
-	wbar /= T - K;
-	pprintf(prn, "\n%s\n\n",
-		_("CUSUM test for stability of parameters"));
-	pprintf(prn, _("mean of scaled residuals = %g\n"), wbar);
+	if (opt & OPT_R) {
+	    double a1, a2, a3;
+	    double n = 0.5 * m - 1;
 
-	sigma = 0;
-	for (j=0; j<n_est; j++) {
-	    xx = (cresid[j] - wbar);
-	    sigma += xx * xx;
+	    pprintf(prn, "\n%s\n\n", _("CUSUMSQ test for stability of parameters"));
+
+	    for (j=0; j<m; j++) {
+		den += cresid[j] * cresid[j];
+	    }
+
+	    /* see Edgerton and Wells, Oxford Bulletin of Economics and
+	       Statistics, 56, 1994, pp. 355-365 */
+	    a1 = 1.358015 / sqrt(n);
+	    a2 = -0.6701218 / n;
+	    a3 = -0.8858694 / pow(n, 1.5);
+
+	    /* 0.5 * total height of band */
+	    a = a1 + a2 + a3;
+	    /* slope of expectation wrt time */
+	    b = 1.0 / m;
+
+	    pputs(prn, _("Cumulated sum of squared residuals"));
+	} else {
+	    wbar /= T - k;
+	    pprintf(prn, "\n%s\n\n", _("CUSUM test for stability of parameters"));
+	    pprintf(prn, _("mean of scaled residuals = %g\n"), wbar);
+
+	    for (j=0; j<m; j++) {
+		sigma += (cresid[j] - wbar) * (cresid[j] - wbar);
+	    }
+	    sigma /= T - k - 1;
+	    sigma = sqrt(sigma);
+	    pprintf(prn, _("sigmahat                 = %g\n\n"), sigma);
+
+	    /* height of confidence band for first prediction */
+	    a = 0.948 * sqrt((double) m);
+	    /* slope of confidence band limit wrt time */
+	    b = 2.0 * a / m;
+
+	    pputs(prn, _("Cumulated sum of scaled residuals"));
 	}
-	sigma /= T - K - 1;
-	sigma = sqrt(sigma);
-	pprintf(prn, _("sigmahat                 = %g\n\n"), sigma);
 
-	xx = 0.948 * sqrt((double) (T-K));
-	yy = 2.0 * xx / (T-K);
-
-	pputs(prn, _("Cumulated sum of scaled residuals"));
 	pputc(prn, '\n');
 	pputs(prn, /* xgettext:no-c-format */
 	      _("('*' indicates a value outside of 95% confidence band)"));
 	pputs(prn, "\n\n");
     
-	for (j=0; j<n_est; j++) {
+	for (j=0; j<m; j++) {
 	    W[j] = 0.0;
-	    for (i=0; i<=j; i++) {
-		W[j] += cresid[i];
+	    if (opt & OPT_R) {
+		for (i=0; i<=j; i++) {
+		    W[j] += cresid[i] * cresid[i] / den;
+		}
+		sig = fabs(W[j] - (j+1) / (double) m) > a;
+	    } else {
+		for (i=0; i<=j; i++) {
+		    W[j] += cresid[i];
+		}
+		W[j] /= sigma;
+		sig = fabs(W[j]) > a + j * b;
 	    }
-	    W[j] /= sigma;
-	    t = pmod->t1 + K + j;
-	    ntodate(cumdate, t, pdinfo);
-	    /* FIXME printing of number below? */
-	    pprintf(prn, " %s %9.3f %s\n", cumdate, W[j],
-		    (fabs(W[j]) > xx + (j+1)*yy)? "*" : "");
+	    ntodate(cumdate, pmod->t1 + k + j, pdinfo);
+	    pprintf(prn, " %s %9.3f %s\n", cumdate, W[j], sig? "*" : "");
 	}
 
-	hct = (sqrt((double) (T-K)) * wbar) / sigma;
-	pval = t_pvalue_2(hct, T-K-1);
-	pprintf(prn, _("\nHarvey-Collier t(%d) = %g with p-value %.4g\n\n"), 
-		T-K-1, hct, pval);
-
-	if (opt & OPT_S) {
-	    ModelTest *test = model_test_new(GRETL_TEST_CUSUM);
-
-	    if (test != NULL) {
-		model_test_set_teststat(test, GRETL_STAT_HARVEY_COLLIER);
-		model_test_set_dfn(test, T-K-1);
-		model_test_set_value(test, hct);
-		model_test_set_pvalue(test, pval);
-		maybe_add_test_to_model(pmod, test);
-	    }
+	if (!(opt & OPT_R)) {
+	    cusum_harvey_collier(wbar, sigma, m, pmod, opt, prn);
 	}
 
-	record_test_result(hct, pval, "Harvey-Collier");
-
-	gretl_push_c_numeric_locale();
-
-	/* plot with 95% confidence bands, if not batch mode */
-	if (!gretl_in_batch_mode() &&
-	    gnuplot_init(PLOT_CUSUM, &fq) == 0) {
-	    fprintf(fq, "set xlabel '%s'\n", I_("Observation"));
-	    fputs("set xzeroaxis\n", fq);
-	    fprintf(fq, "set title '%s'\n",
-		    /* xgettext:no-c-format */
-		    I_("CUSUM plot with 95% confidence band"));
-	    fputs("set nokey\n", fq);
-	    fprintf(fq, "plot \\\n%g+%g*x title '' w lines lt 2, \\\n", xx - K*yy, yy);
-	    fprintf(fq, "%g-%g*x title '' w lines lt 2, \\\n", -xx + K*yy, yy);
-	    fputs("'-' using 1:2 w linespoints lt 1\n", fq);
-
-	    for (j=0; j<n_est; j++) { 
-		t = pmod->t1 + K + j;
-		fprintf(fq, "%d %g\n", t, W[j]);
-	    }
-	    fputs("e\n", fq);
-
-	    fclose(fq);
-
-	    err = gnuplot_make_graph();
+	/* plot with 95% confidence bands if not in batch mode */
+	if (!gretl_in_batch_mode()) {
+	    err = cusum_do_graph(a, b, W, pmod->t1, k, m, pdinfo, opt);
 	}
-
-	gretl_pop_c_numeric_locale();
     }
 
-    /* restore sample */
-    pdinfo->t1 = smpl_t1;
-    pdinfo->t2 = smpl_t2;
+    /* restore original sample range */
+    pdinfo->t1 = orig_t1;
+    pdinfo->t2 = orig_t2;
     
     free(cresid);
     free(W);
-    free(xvec);
 
     return err;
 }
@@ -1929,7 +2147,7 @@ int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
 	return E_ALLOC;
     }
 
-    t1 = gretl_matrix_get_int(m);
+    t1 = gretl_matrix_get_t1(m);
     t2 = t1 + gretl_matrix_rows(m);
 
     /* add leverage? */
@@ -2064,7 +2282,7 @@ int vif_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, PRN *prn)
     int (*print_vifs) (MODEL *, double ***, DATAINFO *, PRN *);
     int err;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     print_vifs = get_plugin_function("print_vifs", &handle);
     if (print_vifs == NULL) {
@@ -2075,172 +2293,6 @@ int vif_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, PRN *prn)
 
     close_plugin(handle);
 
-    if (err && *gretl_errmsg == '\0') {
-	gretl_errmsg_set(_("Command failed"));
-    }
-
-    return err;
-}
-
-static int varmatch (const int *sumvars, int test)
-{
-    int j;
-
-    for (j=1; j<=sumvars[0]; j++) {
-	if (sumvars[j] == test) {
-	    return 1;
-	}
-    }
-
-    return 0;
-}
-
-static 
-void fill_sum_var (double **Z, int n, int v, int vrepl, int vfirst)
-{
-    int t;
-
-    for (t=0; t<n; t++) {
-	Z[v][t] = Z[vrepl][t] - Z[vfirst][t];
-    }
-}
-
-static 
-int make_sum_test_list (MODEL *pmod, double **Z, DATAINFO *pdinfo,
-			int *tmplist, const int *sumvars, int newv)
-{
-    int repl = 0;
-    int testcoeff = 0;
-    int nnew = sumvars[0] - 1;
-    int i;
-
-    tmplist[0] = pmod->list[0];
-    tmplist[1] = pmod->list[1];
-
-    for (i=2; i<=pmod->list[0]; i++) {
-	if (nnew > 0 && varmatch(sumvars, pmod->list[i])) {
-	    if (repl) {
-		fill_sum_var(Z, pdinfo->n, newv, pmod->list[i], sumvars[1]);
-		tmplist[i] = newv++;
-		nnew--;
-	    } else {
-		tmplist[i] = pmod->list[i];
-		testcoeff = i;
-		repl = 1;
-	    }
-	} else {
-	    tmplist[i] = pmod->list[i];
-	}
-    }
-
-    return (nnew == 0)? testcoeff : -1;
-}
-
-/**
- * sum_test:
- * @sumvars: specification of variables to use.
- * @pmod: pointer to model.
- * @pZ: pointer to data matrix.
- * @pdinfo: information on the data set.
- * @prn: gretl printing struct.
- * 
- * Calculates the sum of the coefficients, relative to the given model, 
- * for the variables given in @sumvars.  Prints this estimate along 
- * with its standard error.
- * 
- * Returns: 0 on successful completion, error code on error.
- */
-
-int sum_test (const int *sumvars, MODEL *pmod, 
-	      double ***pZ, DATAINFO *pdinfo, 
-	      PRN *prn)
-{
-    int smpl_t1 = pdinfo->t1;
-    int smpl_t2 = pdinfo->t2;
-    int *tmplist = NULL;
-    const int oldv = pdinfo->v;
-    int testcoeff;
-    MODEL summod;
-    PRN *nullprn;
-    int err = 0;
-
-    if (sumvars[0] < 2) {
-	pprintf(prn, _("Invalid input\n"));
-	return E_DATA;
-    }
-
-    if (!command_ok_for_model(COEFFSUM, pmod->ci)) {
-	return E_NOTIMP;
-    }
-
-    tmplist = malloc((pmod->list[0] + 1) * sizeof *tmplist);
-    if (tmplist == NULL) {
-	return E_ALLOC;
-    }
-
-    if (dataset_add_series(sumvars[0] - 1, pZ, pdinfo)) {
-	free(tmplist);
-	return E_ALLOC;
-    }
-
-    nullprn = gretl_print_new(GRETL_PRINT_NULL);
-
-    testcoeff = make_sum_test_list(pmod, *pZ, pdinfo, tmplist, sumvars, oldv);
-
-    if (testcoeff < 0) {
-	pprintf(prn, _("Invalid input\n"));
-	free(tmplist);
-	dataset_drop_last_variables(pdinfo->v - oldv, pZ, pdinfo);
-	return E_DATA;
-    }
-
-    /* temporarily impose the sample that was in force when the
-       original model was estimated */
-    impose_model_smpl(pmod, pdinfo);
-
-    gretl_model_init(&summod);
-
-    if (!err) {
-	summod = replicate_estimator(pmod, &tmplist, pZ, pdinfo, OPT_A, 
-				     nullprn);
-	if (summod.errcode) {
-	    pprintf(prn, "%s\n", gretl_errmsg);
-	    err = summod.errcode; 
-	} else {
-	    int i;
-
-	    pprintf(prn, "\n%s: ", _("Variables"));
-	    for (i=1; i<=sumvars[0]; i++) {
-		pprintf(prn, "%s ", pdinfo->varname[sumvars[i]]);
-	    }
-	    /* FIXME: check indexing of summod.coeff[] below */
-	    pprintf(prn, "\n   %s = %g\n", _("Sum of coefficients"), 
-		    summod.coeff[testcoeff - 2]);
-	    if (!na(summod.sderr[testcoeff - 2])) {
-		double tval, pval;
-
-		pprintf(prn, "   %s = %g\n", _("Standard error"),
-			summod.sderr[testcoeff - 2]);
-		tval = summod.coeff[testcoeff - 2] / 
-		    summod.sderr[testcoeff - 2];
-		pval = coeff_pval(&summod, tval, summod.dfd);
-		pprintf(prn, "   t(%d) = %g ", summod.dfd, tval);
-		pprintf(prn, _("with p-value = %g\n"), pval);
-		record_test_result(tval, pval, _("sum"));
-	    }
-	}
-    }
-
-    free(tmplist);
-    clear_model(&summod);
-
-    dataset_drop_last_variables(pdinfo->v - oldv, pZ, pdinfo);
-    gretl_print_destroy(nullprn);
-
-    /* put back into pdinfo what was there on input */
-    pdinfo->t1 = smpl_t1;
-    pdinfo->t2 = smpl_t2;
-
     return err;
 }
 
@@ -2249,7 +2301,8 @@ int sum_test (const int *sumvars, MODEL *pmod,
  * @param: auxiliary parameter for some uses.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
- * @opt: controls which test(s) will be performed.
+ * @opt: controls which test(s) will be performed; %OPT_Q
+ * gives less verbose results.
  * @prn: gretl printing struct.
  * 
  * Performs some subset of gretl's "lmtest" tests on the
@@ -2263,11 +2316,12 @@ int lmtest_driver (const char *param,
 		   gretlopt opt, PRN *prn)
 {
     GretlObjType type;
+    gretlopt testopt;
     void *ptr;
     int k = 0;
     int err = 0;
 
-    if (opt == OPT_NONE) {
+    if (opt == OPT_NONE || opt == OPT_Q) {
 	pprintf(prn, "lmtest: no options selected\n");
 	return 0;
     }
@@ -2277,15 +2331,17 @@ int lmtest_driver (const char *param,
 	return E_DATA;
     }
 
-    if ((opt & OPT_A) || (opt & OPT_H)) {
+    if (opt & (OPT_A | OPT_H)) {
 	k = atoi(param);
     }
 
+    testopt = (opt & OPT_Q)? OPT_Q : OPT_NONE;
+
     /* non-linearity (squares) */
     if (!err && (opt & OPT_S)) {
 	if (type == GRETL_OBJ_EQN) {
 	    err = nonlinearity_test(ptr, pZ, pdinfo, 
-				    AUX_SQ, OPT_NONE, prn);
+				    AUX_SQ, testopt, prn);
 	} else {
 	    err = E_NOTIMP;
 	}
@@ -2295,7 +2351,7 @@ int lmtest_driver (const char *param,
     if (!err && (opt & OPT_L)) {
 	if (type == GRETL_OBJ_EQN) {
 	    err = nonlinearity_test(ptr, pZ, pdinfo, 
-				    AUX_LOG, OPT_NONE, prn);
+				    AUX_LOG, testopt, prn);
 	} else {
 	    err = E_NOTIMP;
 	}
@@ -2304,7 +2360,7 @@ int lmtest_driver (const char *param,
     /* heteroskedasticity, White */
     if (!err && (opt & OPT_W)) {
 	if (type == GRETL_OBJ_EQN) {
-	    err = whites_test(ptr, pZ, pdinfo, OPT_NONE, prn);
+	    err = whites_test(ptr, pZ, pdinfo, testopt, prn);
 	} else {
 	    err = E_NOTIMP;
 	}
@@ -2314,7 +2370,7 @@ int lmtest_driver (const char *param,
     if (!err && (opt & OPT_A)) {
 	if (type == GRETL_OBJ_EQN) {
 	    err = autocorr_test(ptr, k, pZ, pdinfo, 
-				OPT_NONE, prn);
+				testopt, prn);
 	} else if (type == GRETL_OBJ_VAR) {
 	    err = gretl_VAR_autocorrelation_test(ptr, k, pZ, pdinfo, prn);
 	} else {
@@ -2325,9 +2381,9 @@ int lmtest_driver (const char *param,
     /* ARCH */
     if (!err && (opt & OPT_H)) {
 	if (type == GRETL_OBJ_EQN) {
-	    err = arch_test_simple(ptr, k, pZ, pdinfo, OPT_NONE, prn);
+	    err = arch_test(ptr, k, pdinfo, testopt, prn);
 	} else if (type == GRETL_OBJ_VAR) {
-	    err = gretl_VAR_arch_test(ptr, k, pZ, pdinfo, prn);
+	    err = gretl_VAR_arch_test(ptr, k, pdinfo, prn);
 	} else {
 	    err = E_NOTIMP;
 	}
