@@ -28,9 +28,7 @@
 #include <time.h>
 #include <errno.h>
 
-#ifdef USE_GLIB2
-# include <glib.h>
-#endif
+#include <glib.h>
 
 #define QUOTE                  '\''
 #define CSVSTRLEN               72
@@ -146,7 +144,7 @@ static int readdata (FILE *fp, const DATAINFO *pdinfo, double **Z,
     char c, marker[OBSLEN];
     int err = 0;
 
-    gretl_errmsg[0] = '\0';
+    gretl_error_clear();
 
     if (binary == 1) { 
 	/* single-precision binary data */
@@ -253,9 +251,10 @@ static int gz_readdata (gzFile fz, const DATAINFO *pdinfo, double **Z,
     int i, t, n = pdinfo->n;
     int err = 0;
     
-    gretl_errmsg[0] = '\0';
+    gretl_error_clear();
 
-    if (binary == 1) { /* single-precision binary data */
+    if (binary == 1) { 
+	/* single-precision binary data */
 	float xx;
 
 	for (i=1; i<pdinfo->v; i++) {
@@ -268,7 +267,8 @@ static int gz_readdata (gzFile fz, const DATAINFO *pdinfo, double **Z,
 		Z[i][t] = (double) xx;
 	    }
 	}
-    } else if (binary == 2) { /* double-precision binary data */
+    } else if (binary == 2) { 
+	/* double-precision binary data */
 	for (i=1; i<pdinfo->v; i++) {
 	    if (!gzread(fz, &Z[i][0], n * sizeof(double))) {
 		sprintf(gretl_errmsg, 
@@ -276,7 +276,8 @@ static int gz_readdata (gzFile fz, const DATAINFO *pdinfo, double **Z,
 		return 1;
 	    }
 	}
-    } else { /* ascii data file */
+    } else { 
+	/* ascii data */
 	char *line, numstr[24], sformat[8];
 	int llen = pdinfo->v * 32;
 	size_t offset;
@@ -360,7 +361,7 @@ int check_varname (const char *varname)
     int testchar = 'a';
     int ret = 0;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     if (gretl_reserved_word(varname)) {
 	ret = VARNAME_RESERVED;
@@ -415,7 +416,7 @@ static int readhdr (const char *hdrfile, DATAINFO *pdinfo,
     int n, i = 0, panel = 0, descrip = 0;
     char str[MAXLEN], byobs[6], option[8];
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     fp = gretl_fopen(hdrfile, "r");
     if (fp == NULL) {
@@ -574,7 +575,7 @@ static int bad_date_string (const char *s)
 {
     int err = 0;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     while (*s && !err) {
 	if (!isdigit((unsigned char) *s) && !IS_DATE_SEP(*s)) {
@@ -1091,12 +1092,8 @@ format_from_opt_or_name (gretlopt opt, const char *fname)
 {
     GretlDataFormat fmt = 0;
     
-    if (opt & OPT_S) {
-	fmt = GRETL_DATA_FLOAT;	
-    } else if (opt & OPT_T) {
+    if (opt & OPT_T) {
 	fmt = GRETL_DATA_TRAD;
-    } else if (opt & OPT_O) {
-	fmt = GRETL_DATA_DOUBLE;
     } else if (opt & OPT_M) {
 	fmt = GRETL_DATA_OCTAVE;
     } else if (opt & OPT_R) {
@@ -1142,6 +1139,8 @@ static void date_maj_min (int t, const DATAINFO *pdinfo, int *maj, int *min)
     }
 }
 
+#define annual_data(p) (p->structure == TIME_SERIES && p->pd == 1)
+
 /**
  * write_data:
  * @fname: name of file to write.
@@ -1166,13 +1165,12 @@ int write_data (const char *fname, const int *list,
     GretlDataFormat fmt;
     char datfile[MAXLEN], hdrfile[MAXLEN], lblfile[MAXLEN];
     int tsamp = pdinfo->t2 - pdinfo->t1 + 1;
-    int omit_obs = 0;
     int n = pdinfo->n;
     FILE *fp = NULL;
     int *pmax = NULL;
     double xx;
 
-    *gretl_errmsg = 0;
+    gretl_error_clear();
 
     if (list == NULL || list[0] == 0) {
 	return 1;
@@ -1182,6 +1180,8 @@ int write_data (const char *fname, const int *list,
 
     fmt = format_from_opt_or_name(opt, fname);
 
+    gretl_maybe_switch_dir(fname);
+
     if (fmt == 0 || fmt == GRETL_DATA_GZIPPED) {
 	return gretl_write_gdt(fname, list, Z, pdinfo, fmt, ppaths);
     }
@@ -1203,11 +1203,6 @@ int write_data (const char *fname, const int *list,
 	fmt = GRETL_DATA_R_TS;
     }
 
-    if (fmt == GRETL_DATA_CSV && (opt & OPT_X)) {
-	/* don't print a first column with observations */
-	omit_obs = 1;
-    }
-
     /* write header and label files if not exporting to other formats */
     if (fmt != GRETL_DATA_R && fmt != GRETL_DATA_R_TS && 
 	fmt != GRETL_DATA_CSV && fmt != GRETL_DATA_OCTAVE &&
@@ -1230,52 +1225,11 @@ int write_data (const char *fname, const int *list,
     }
 
     /* open file for output */
-    if (fmt == GRETL_DATA_FLOAT || fmt == GRETL_DATA_DOUBLE) {
-	fp = gretl_fopen(datfile, "wb");
-    } else {
-	fp = gretl_fopen(datfile, "w");
-    }
-
+    fp = gretl_fopen(datfile, "w");
     if (fp == NULL) {
 	return E_FOPEN;
     }
 
-    if (fmt == GRETL_DATA_FLOAT) { 
-	/* single-precision binary */
-	float x;
-
-	for (i=1; i<=l0; i++) {
-	    v = list[i];
-	    if (na(Z[v][0])) {
-		x = -999.0;
-	    } else {
-		x = (float) Z[v][0];
-	    }
-	    for (t=0; t<n; t++) {
-		if (var_is_series(pdinfo, v)) {
-		    if (na(Z[v][0])) {
-			x = -999.0;
-		    } else {	
-			x = (float) Z[v][t];
-		    }
-		}
-		fwrite(&x, sizeof x, 1, fp);
-	    }
-	}
-    } else if (fmt == GRETL_DATA_DOUBLE) { 
-	/* double-precision binary */
-	for (i=1; i<=l0; i++) {
-	    v = list[i];
-	    if (var_is_series(pdinfo, v)) {
-		fwrite(&Z[v][0], sizeof(double), n, fp);
-	    } else {
-		for (t=0; t<n; t++) {
-		    fwrite(&Z[v][0], sizeof(double), 1, fp);
-		}
-	    }
-	}
-    }
-
     if (fmt == GRETL_DATA_CSV || fmt == GRETL_DATA_OCTAVE || 
 	GRETL_DATA_R || fmt == GRETL_DATA_TRAD || 
 	fmt == GRETL_DATA_DAT || fmt == GRETL_DATA_JM) { 
@@ -1295,8 +1249,9 @@ int write_data (const char *fname, const int *list,
 	}	
     }
 
-    if (fmt == GRETL_DATA_CSV && pdinfo->decpoint == ',') ;
-    else gretl_push_c_numeric_locale();
+    if (fmt != GRETL_DATA_CSV || pdinfo->decpoint != ',') {
+	gretl_push_c_numeric_locale();
+    }
 
     if (fmt == GRETL_DATA_TRAD) { 
 	/* plain ASCII */
@@ -1322,16 +1277,22 @@ int write_data (const char *fname, const int *list,
 	}
     } else if (fmt == GRETL_DATA_CSV || fmt == GRETL_DATA_R) { 
 	/* export CSV or GNU R (dataframe) */
+	int print_obs = 0;
 	char delim;
-	
+
 	if (fmt == GRETL_DATA_CSV) {
+	    if ((pdinfo->structure == TIME_SERIES || pdinfo->S != NULL)
+		&& !(opt & OPT_X)) {
+		print_obs = 1;
+	    }
 	    delim = get_csv_delim(pdinfo);
 	} else {
+	    print_obs = (pdinfo->S != NULL);
 	    delim = ' ';
 	}
 
 	/* variable names */
-	if (fmt == GRETL_DATA_CSV && !omit_obs && 
+	if (fmt == GRETL_DATA_CSV && print_obs && 
 	    (pdinfo->S != NULL || pdinfo->structure != CROSS_SECTION)) {
 	    fprintf(fp, "obs%c", delim);
 	}
@@ -1341,19 +1302,17 @@ int write_data (const char *fname, const int *list,
 	fprintf(fp, "%s\n", pdinfo->varname[list[l0]]);
 	
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	    if (!omit_obs) {
+	    if (print_obs) {
 		if (pdinfo->S != NULL) {
-		    fprintf(fp, "%s%c", pdinfo->S[t], delim);
-		} else if (pdinfo->structure != CROSS_SECTION) {
+		    fprintf(fp, "\"%s\"%c", pdinfo->S[t], delim);
+		} else {
 		    char tmp[OBSLEN];
 
 		    ntodate_full(tmp, t, pdinfo);
 		    if (quarterly_or_monthly(pdinfo)) {
 			modify_date_for_csv(tmp, pdinfo->pd);
-			fprintf(fp, "\"%s\"%c", tmp, delim);
-		    } else {
-			fprintf(fp, "\"'%s\"%c", tmp, delim);
 		    }
+		    fprintf(fp, "%s%c", tmp, delim);
 		}
 	    }
 	    for (i=1; i<=l0; i++) { 
@@ -1506,7 +1465,9 @@ int write_data (const char *fname, const int *list,
 	}
     }
 
-    gretl_pop_c_numeric_locale();
+    if (fmt != GRETL_DATA_CSV || pdinfo->decpoint != ','){
+	gretl_pop_c_numeric_locale();
+    }
 
     if (pmax != NULL) {
 	free(pmax);
@@ -1619,7 +1580,7 @@ static int readlbl (const char *lblfile, DATAINFO *pdinfo)
     char line[MAXLEN], *label, varname[VNAMELEN];
     int v;
     
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     fp = gretl_fopen(lblfile, "r");
     if (fp == NULL) return 0;
@@ -1771,18 +1732,17 @@ static void try_gdt (char *fname)
  * @ppdinfo: pointer to data information struct.
  * @datfile: name of file to try.
  * @ppaths: path information struct.
- * @code: option.
+ * @ocode: %DATA_NONE, %DATA_CLEAR or %DATA_APPEND.
  * @prn: where messages should be written.
  * 
  * Read data from file into gretl's work space, allocating space as
  * required.
  * 
  * Returns: 0 on successful completion, non-zero otherwise.
- *
  */
 
 int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppaths, 
-		    DataOpenCode code, PRN *prn) 
+		    DataOpenCode ocode, PRN *prn) 
 {
     DATAINFO *tmpdinfo = NULL;
     double **tmpZ = NULL;
@@ -1793,7 +1753,8 @@ int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppat
     int binary = 0, old_byvar = 0;
     int err = 0;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
+
     *hdrfile = '\0';
 
     gdtsuff = has_suffix(datfile, ".gdt");
@@ -1836,7 +1797,7 @@ int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppat
     /* catch XML files that have strayed in here? */
     if (gdtsuff && gretl_is_xml_file(datfile)) {
 	return gretl_read_gdt(pZ, ppdinfo, datfile, ppaths, 
-			      code, prn, 0);
+			      ocode, prn, 0);
     }
 
     tmpdinfo = datainfo_new();
@@ -1936,14 +1897,14 @@ int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, char *datfile, PATHS *ppat
     err = readlbl(lblfile, tmpdinfo);
     if (err) goto bailout;
 
-    if (code == DATA_APPEND) {
+    if (ocode == DATA_APPEND) {
 	err = merge_data(pZ, *ppdinfo, tmpZ, tmpdinfo, prn);
     } else {
 	if (ppaths != NULL && datfile != ppaths->datfile) {
 	    strcpy(ppaths->datfile, datfile);
 	}
 	free_Z(*pZ, *ppdinfo);
-	if (code == DATA_CLEAR) {
+	if (ocode == DATA_CLEAR) {
 	    clear_datainfo(*ppdinfo, CLEAR_FULL);
 	}
 	free(*ppdinfo);
@@ -2368,6 +2329,7 @@ static void make_endobs_string (char *endobs, const char *s)
 
 static int csv_time_series_check (DATAINFO *pdinfo, PRN *prn)
 {
+    const char *subchars = ".:QqMmPp";
     char year[5];
     char *lbl1 = pdinfo->S[0];
     char *lbl2 = pdinfo->S[pdinfo->n - 1];
@@ -2397,10 +2359,7 @@ static int csv_time_series_check (DATAINFO *pdinfo, PRN *prn)
 	    pputs(prn, M_("   but the dates are not complete and consistent\n"));
 	    return pd;
 	}
-    } else if (lbl1[4] == '.' || 
-	       lbl1[4] == ':' || 
-	       lbl1[4] == 'Q' || 
-	       lbl1[4] == 'P') {
+    } else if (strchr(subchars, lbl1[4])) {
 	char subper[3];
 
 	*subper = '\0';
@@ -2720,7 +2679,7 @@ int merge_data (double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    if (!err && (addvars || addobs)) {
+    if (!err && (addvars || addobs) && gretl_messages_on()) {
 	pputs(prn, _("Data appended OK\n"));
     }
 
@@ -2734,21 +2693,44 @@ int merge_data (double ***pZ, DATAINFO *pdinfo,
    file.  It also checks for extraneous binary data (the file is 
    supposed to be plain text), and checks whether the 'delim'
    character is present in the file, on a non-comment line (where
-   a comment line is one that starts with '#').  Finally, checks
-   whether the file has a trailing comma on every line.
+   a comment line is one that starts with '#').  
+
+   Optionally, we check whether the file has a trailing comma on every
+   line, and check for Mac-style '\r' line termination.  The heuristic
+   for the latter is that we find '\r' not followed by '\n'.
 */
 
 static int get_max_line_length (FILE *fp, char delim, int *gotdelim, 
-				int *gottab, int *trail, PRN *prn)
+				int *gottab, int *trail, int *mac,
+				PRN *prn)
 {
-    int c, cbak = 0, cc = 0;
+    int c, c1, cbak = 0, cc = 0;
     int comment = 0, maxlen = 0;
 
     if (trail != NULL) {
 	*trail = 1;
     }
 
+    if (mac != NULL) {
+	*mac = 0;
+    }
+
     while ((c = fgetc(fp)) != EOF) {
+	if (c == '\r') {
+	    c1 = fgetc(fp);
+	    if (c1 == EOF) {
+		break;
+	    } else if (c1 == '\n') {
+		c = c1;
+	    } else {
+		/* Mac-type file */
+		if (mac != NULL) {
+		    *mac = 1;
+		}
+		c = '\n';
+		ungetc(c1, fp);
+	    }
+	}
 	if (c == '\n') {
 	    if (cc > maxlen) {
 		maxlen = cc;
@@ -3019,10 +3001,9 @@ static int process_csv_obs (const char *str, int i, int t,
 	    if (ix >= 0) {
 		Z[i][t] = (double) ix;
 	    } else {
-		pprintf(prn, M_("At variable %d, observation %d:\n"), i, t+1);
-		pprintf(prn, " %s\n", gretl_errmsg);
-		*gretl_errmsg = '\0';
 		err = 1;
+		pprintf(prn, M_("At variable %d, observation %d:\n"), i, t+1);
+		errmsg(err, prn);
 	    }
 	} else {
 	    Z[i][t] = atof(str);
@@ -3069,6 +3050,33 @@ static char *get_csv_descrip (FILE *fp)
     return desc;
 }
 
+/* wrapper for fgets() that allows for handling a Mac-style
+   text file */
+
+static char *csv_fgets (char *s, int n, int mac, FILE *fp)
+{
+    if (!mac) {
+	return fgets(s, n, fp);
+    } else {
+	int c, i = 0;
+
+	for (i=0; i<n-1; i++) {
+	    c = fgetc(fp);
+	    if (c == EOF) {
+		break;
+	    } else if (c == '\r') {
+		s[i++] = '\n';
+		break;
+	    }
+	    s[i] = c;
+	}
+
+	s[i] = '\0';
+
+	return s;
+    }
+}
+
 static int 
 csv_reconfigure_for_markers (double ***pZ, DATAINFO *pdinfo)
 {
@@ -3109,6 +3117,7 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
     FILE *fp = NULL;
     DATAINFO *csvinfo = NULL;
     double **csvZ = NULL;
+    PRN *mprn = NULL;
     char *line = NULL, *p = NULL, *descrip = NULL;
 
     const char *msg = M_("\nPlease note:\n"
@@ -3122,6 +3131,7 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 
     char delim = '\t';
     int numcount, auto_name_vars = 0;
+    int mactext = 0;
     gretl_string_table *st = NULL;
 
     if (*ppdinfo != NULL) {
@@ -3132,6 +3142,10 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 	check_for_console(prn);
     }
 
+    if (gretl_messages_on()) {
+	mprn = prn;
+    }
+
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
 	pprintf(prn, M_("Couldn't open %s\n"), fname);
@@ -3147,21 +3161,18 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 
     csvinfo->delim = delim;
 
-#if USE_GLIB2
     if (!g_utf8_validate(fname, -1, NULL)) {
 	gchar *trfname = g_locale_to_utf8(fname, -1, NULL, NULL, NULL);
 
-	pprintf(prn, "%s %s...\n", M_("parsing"), trfname);
+	pprintf(mprn, "%s %s...\n", M_("parsing"), trfname);
 	g_free(trfname);
     } else {
-	pprintf(prn, "%s %s...\n", M_("parsing"), fname);
+	pprintf(mprn, "%s %s...\n", M_("parsing"), fname);
     }
-#else
-    pprintf(prn, "%s %s...\n", M_("parsing"), fname);
-#endif
 
     /* get line length, also check for binary data, etc. */
-    maxlen = get_max_line_length(fp, delim, &gotdelim, &gottab, &trail, prn);    
+    maxlen = get_max_line_length(fp, delim, &gotdelim, &gottab, &trail, 
+				 &mactext, prn);    
     if (maxlen <= 0) {
 	goto csv_bailout;
     } 
@@ -3175,12 +3186,13 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 	}
     }
 
-    pprintf(prn, M_("using delimiter '%c'\n"), delim);
-    pprintf(prn, M_("   longest line: %d characters\n"), maxlen - 1);
+    pprintf(mprn, M_("using delimiter '%c'\n"), delim);
+    pprintf(mprn, M_("   longest line: %d characters\n"), maxlen - 1);
 
     if (trail && delim != ',') trail = 0;
 
     /* create buffer to hold lines */
+
     line = malloc(maxlen);
     if (line == NULL) {
 	pputs(prn, M_("Out of memory\n"));
@@ -3190,8 +3202,10 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
     rewind(fp);
     
     /* read lines, check for consistency in number of fields */
+
     chkcols = ncols = nrows = gotdata = 0;
-    while (fgets(line, maxlen, fp)) {
+
+    while (csv_fgets(line, maxlen, mactext, fp)) {
 	/* skip comment lines */
 	if (*line == '#') {
 	    continue;
@@ -3211,13 +3225,13 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 	compress_csv_line(line, delim, trail);
 	if (!gotdata) {
 	    /* scrutinize first "real" line */
-	    check_first_field(line, delim, &blank_1, &obs_1, prn);
+	    check_first_field(line, delim, &blank_1, &obs_1, mprn);
 	    gotdata = 1;
 	} 
 	chkcols = count_csv_fields(line, delim);
 	if (ncols == 0) {
 	    ncols = chkcols;
-	    pprintf(prn, M_("   number of columns = %d\n"), ncols);	    
+	    pprintf(mprn, M_("   number of columns = %d\n"), ncols);	    
 	} else {
 	    if (chkcols != ncols) {
 		pprintf(prn, M_("   ...but row %d has %d fields: aborting\n"),
@@ -3232,8 +3246,8 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
     csvinfo->n = nrows - 1;
 
     csvinfo->v = (blank_1 || obs_1)? ncols : ncols + 1;
-    pprintf(prn, M_("   number of variables: %d\n"), csvinfo->v - 1);
-    pprintf(prn, M_("   number of non-blank lines: %d\n"), nrows);
+    pprintf(mprn, M_("   number of variables: %d\n"), csvinfo->v - 1);
+    pprintf(mprn, M_("   number of non-blank lines: %d\n"), nrows);
 
     /* end initial checking */
 
@@ -3260,9 +3274,9 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
     rewind(fp);
 
     /* parse the line containing variable names */
-    pputs(prn, M_("scanning for variable names...\n"));
+    pputs(mprn, M_("scanning for variable names...\n"));
 
-    while (fgets(line, maxlen, fp)) {
+    while (csv_fgets(line, maxlen, mactext, fp)) {
 	if (*line == '#' || string_is_blank(line)) {
 	    continue;
 	} else {
@@ -3275,7 +3289,7 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
     p = line;
     if (delim == ' ' && *p == ' ') p++;
     iso_to_ascii(p);
-    pprintf(prn, M_("   line: %s\n"), p);
+    pprintf(mprn, M_("   line: %s\n"), p);
     
     numcount = 0;
     for (k=0; k<ncols; k++) {
@@ -3309,8 +3323,7 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 		} else {
 		    iso_to_ascii(csvinfo->varname[nv]);
 		    if (check_varname(csvinfo->varname[nv])) {
-			pprintf(prn, "%s\n", gretl_errmsg);
-			*gretl_errmsg = '\0';
+			errmsg(1, prn);
 			goto csv_bailout;
 		    }
 		}
@@ -3340,8 +3353,7 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
     } else if (numcount > 0) {
 	for (i=1; i<csvinfo->v; i++) {
 	    if (check_varname(csvinfo->varname[i])) {
-		pprintf(prn, "%s\n", gretl_errmsg);
-		*gretl_errmsg = '\0';
+		errmsg(1, prn);
 		break;
 	    }
 	}	    
@@ -3352,10 +3364,10 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
 	gretl_push_c_numeric_locale();
     }
 
-    pputs(prn, M_("scanning for row labels and data...\n"));
+    pputs(mprn, M_("scanning for row labels and data...\n"));
 
     t = 0;
-    while (fgets(line, maxlen, fp)) {
+    while (csv_fgets(line, maxlen, mactext, fp)) {
 	int nv;
 
 	if (*line == '#' || string_is_blank(line)) {
@@ -3414,10 +3426,11 @@ int import_csv (double ***pZ, DATAINFO **ppdinfo,
     if (blank_1 || obs_1) {
 	markertest = test_markers_for_dates(&csvZ, csvinfo, prn);
     }
+
     if (markertest > 0) {
-	pputs(prn, M_("taking date information from row labels\n\n"));
+	pputs(mprn, M_("taking date information from row labels\n\n"));
     } else {
-	pputs(prn, M_("treating these as undated data\n\n"));
+	pputs(mprn, M_("treating these as undated data\n\n"));
 	dataset_obs_info_default(csvinfo);
     }
 
@@ -3624,7 +3637,7 @@ int import_octave (double ***pZ, DATAINFO **ppdinfo,
 
     pprintf(prn, "%s %s...\n", M_("parsing"), fname);
 
-    maxlen = get_max_line_length(fp, 0, NULL, NULL, NULL, prn);
+    maxlen = get_max_line_length(fp, 0, NULL, NULL, NULL, NULL, prn);
     if (maxlen <= 0) {
 	goto oct_bailout;
     }
@@ -4189,8 +4202,9 @@ GretlFileType detect_filetype (char *fname, PATHS *ppaths, PRN *prn)
     FILE *fp;
 
     /* might be a script file? (watch out for DOS-mangled names) */
-    if (has_suffix(fname, ".inp")) 
+    if (has_suffix(fname, ".inp")) { 
 	return GRETL_SCRIPT;
+    }
 
     if (has_suffix(fname, ".gretl")) {
 	if (gretl_is_pkzip_file(fname)) {
@@ -4221,6 +4235,10 @@ GretlFileType detect_filetype (char *fname, PATHS *ppaths, PRN *prn)
     if (has_suffix(fname, ".bn7"))
 	return GRETL_PCGIVE_DB;
 
+    if (ppaths == NULL) {
+	return GRETL_NATIVE_DATA; 
+    }
+
     addpath(fname, ppaths, 0); 
 
     if (gretl_is_xml_file(fname)) {
