@@ -50,6 +50,7 @@ typedef enum {
     GRETL_SCRIPT,         /* file containing gretl commands */
     GRETL_SESSION,        /* zipped session file */
     GRETL_NATIVE_DB,      /* gretl database */
+    GRETL_NATIVE_DB_WWW,  /* gretl database, accessed via internet */
     GRETL_RATS_DB,        /* RATS 4.0 database */
     GRETL_PCGIVE_DB,      /* PcGive bn7/in7 pair */
     GRETL_JMULTI,         /* JMulTi data file */
@@ -115,7 +116,7 @@ int merge_data (double ***pZ, DATAINFO *pdinfo,
 
 int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, 
 		    char *datfile, PATHS *ppaths, 
-		    DataOpenCode code, PRN *prn);
+		    DataOpenCode ocode, PRN *prn);
 
 int open_nulldata (double ***pZ, DATAINFO *pdinfo, 
 		   int data_status, int length,
