@@ -20,6 +20,8 @@
 #include "libgretl.h"
 #include "gretl_func.h"
 
+#define DDEBUG 0
+
 /**
  * free_Z:
  * @Z: data matrix.
@@ -33,6 +35,11 @@ void free_Z (double **Z, DATAINFO *pdinfo)
     if (Z != NULL && pdinfo != NULL) {
 	int i;
 
+#if DDEBUG
+	fprintf(stderr, "Freeing Z (%p): %d vars\n", (void *) Z, 
+		pdinfo->v);
+#endif
+
 	for (i=0; i<pdinfo->v; i++) {
 	    free(Z[i]);
 	}
@@ -261,7 +268,7 @@ int dataset_allocate_panel_info (DATAINFO *pdinfo)
     PANINFO *pan;
     int i;
 
-#if 0
+#if DDEBUG
     fprintf(stderr, "dataset_allocate_panel_info: pdinfo at %p\n",
 	    (void *) pdinfo);
 #endif
@@ -457,7 +464,7 @@ int dataset_finalize_panel_indices (DATAINFO *pdinfo)
 	return E_PDWRONG;
     }
 
-#if 0
+#if DDEBUG
     fprintf(stderr, "dataset_finalize_panel_indices:\n "
 	    "nunits=%d, Tmin=%d, Tmax=%d, olen=%d\n", pan->nunits,
 	    pan->Tmin, pan->Tmax, pan->olen);
@@ -470,15 +477,13 @@ static void gretl_varinfo_init (VARINFO *vinfo)
 {
     vinfo->label[0] = '\0';
     vinfo->display_name[0] = '\0';
+    vinfo->gp_display_name[0] = '\0';
     vinfo->flags = 0;
     vinfo->compact_method = COMPACT_NONE;
     vinfo->stack_level = 0;
     vinfo->line_width = 1;
     vinfo->sorted_markers = NULL;
-
-    if (gretl_executing_function()) {
-	vinfo->stack_level = gretl_function_stack_depth();
-    } 
+    vinfo->stack_level = gretl_function_depth();
 }
 
 /**
@@ -497,6 +502,7 @@ void copy_varinfo (VARINFO *targ, const VARINFO *src)
 
     strcpy(targ->label, src->label);
     strcpy(targ->display_name, src->display_name);
+    strcpy(targ->gp_display_name, src->gp_display_name);
     targ->flags = src->flags;
     targ->compact_method = src->compact_method;
     targ->stack_level = src->stack_level;
@@ -977,7 +983,7 @@ maybe_extend_dummies (double **Z, const DATAINFO *pdinfo, int oldn)
  * @pdinfo: dataset information.
  * @opt: use %OPT_A to attempt to recognize and
  * automatically extend simple deterministic variables such 
- * as a time trend and periodic dummy variables; also can
+ * as a time trend and periodic dummy variables; 
  * use %OPT_D to drop any observation markers rather than
  * expanding the set of markers and padding it out with
  * dummy values.
@@ -1226,14 +1232,27 @@ dataset_expand_varinfo (int newvars, DATAINFO *pdinfo)
     return 0;
 }
 
-static int real_dataset_add_series (int newvars, double *x,
-				    double ***pZ, DATAINFO *pdinfo)
+static int real_add_series (int newvars, double *x,
+			    double ***pZ, DATAINFO *pdinfo)
 {
     double **newZ;
-    int i, n = pdinfo->n, v = pdinfo->v;
+    int v = pdinfo->v;
+    int n = pdinfo->n;
+    int i, t;
     int err = 0;
 
-    newZ = realloc(*pZ, (v + newvars) * sizeof *newZ);  
+    if (newvars == 0) {
+	/* no-op */
+	return 0;
+    }
+
+    newZ = realloc(*pZ, (v + newvars) * sizeof *newZ); 
+
+#if DDEBUG
+    fprintf(stderr, "real_add_series: add %d vars, Z = %p\n",
+	    newvars, (void *) newZ);
+#endif
+	    
 
     if (newZ == NULL) {
 	err = E_ALLOC;
@@ -1250,6 +1269,10 @@ static int real_dataset_add_series (int newvars, double *x,
 		newZ[v+i] = malloc(n * sizeof **newZ);
 		if (newZ[v+i] == NULL) {
 		    err = E_ALLOC;
+		} else {
+		    for (t=0; t<n; t++) {
+			newZ[v+i][t] = 0.0;
+		    }
 		}
 	    }
 	}
@@ -1278,7 +1301,7 @@ static int real_dataset_add_series (int newvars, double *x,
 int 
 dataset_add_series (int newvars, double ***pZ, DATAINFO *pdinfo)
 {
-    return real_dataset_add_series(newvars, NULL, pZ, pdinfo);
+    return real_add_series(newvars, NULL, pZ, pdinfo);
 }
 
 /**
@@ -1297,7 +1320,7 @@ dataset_add_series (int newvars, double ***pZ, DATAINFO *pdinfo)
 int 
 dataset_add_allocated_series (double *x, double ***pZ, DATAINFO *pdinfo)
 {
-    return real_dataset_add_series(1, x, pZ, pdinfo);
+    return real_add_series(1, x, pZ, pdinfo);
 }
 
 /**
@@ -1369,60 +1392,25 @@ int dataset_add_scalar (double ***pZ, DATAINFO *pdinfo)
     return dataset_add_scalars(1, pZ, pdinfo);
 }
 
-/**
- * dataset_scalar_to_vector:
- * @v: index number of variable to process.
- * @pZ: pointer to data array.
- * @pdinfo: dataset information.
- *
- * Expands an existing scalar member of a dataset to a
- * full-length vector.  All values are initialized to
- * the missing value code, #NABDL.
- *
- * Returns: 0 on success, %E_ALLOC on error.
- */
-
-int dataset_scalar_to_vector (int v, double ***pZ, DATAINFO *pdinfo)
-{
-    double *tmp;
-    int t, err = 0;
-
-    tmp = realloc((*pZ)[v], pdinfo->n * sizeof *tmp);
-
-    if (tmp == NULL) {
-	err = E_ALLOC;
-    } else {
-	/* initialize all vals to missing */
-	for (t=0; t<pdinfo->n; t++) {
-	    tmp[t] = NADBL;
-	}
-	(*pZ)[v] = tmp;
-	set_var_scalar(pdinfo, v, 0);
-    }
-
-    return err;
-}
-
 /**
  * dataset_add_scalar_as:
- * @numstr: string representation of numeric value.
+ * @x: scalar value.
  * @newname: name to give the new variable.
  * @pZ: pointer to data array.
  * @pdinfo: dataset information.
  *
  * Adds to the dataset a new scalar with name @newname and
- * value given by @numstr.  The new variable is added at one
+ * value given by @x.  The new variable is added at one
  * level "deeper" (in terms of function execution) than the
- * current level.  This is for use with user-defined functions,
- * where a numeric string is given as a function argument.
+ * current level.  This is for use with user-defined functions.
  *
  * Returns: 0 on success, %E_ALLOC on error.
  */
 
-int dataset_add_scalar_as (const char *numstr, const char *newname,
+int dataset_add_scalar_as (double x, const char *newname,
 			   double ***pZ, DATAINFO *pdinfo)
 {
-    int vnew, err = 0;
+    int v, err = 0;
 
     if (pdinfo->varinfo == NULL) {
 	strcpy(gretl_errmsg, _("Please open a data file first"));
@@ -1432,10 +1420,54 @@ int dataset_add_scalar_as (const char *numstr, const char *newname,
     err = dataset_add_scalar(pZ, pdinfo);
 
     if (!err) {
-	vnew = pdinfo->v - 1;
-	(*pZ)[vnew][0] = dot_atof(numstr);
-	strcpy(pdinfo->varname[vnew], newname);
-	STACK_LEVEL(pdinfo, vnew) += 1;
+	v = pdinfo->v - 1;
+	(*pZ)[v][0] = x;
+	strcpy(pdinfo->varname[v], newname);
+	STACK_LEVEL(pdinfo, v) += 1;
+    }
+
+    return err;
+}
+
+/**
+ * dataset_add_series_as:
+ * @x: array to be added.
+ * @newname: name to give the new variable.
+ * @pZ: pointer to data array.
+ * @pdinfo: dataset information.
+ *
+ * Adds to the dataset a new series with name @newname and
+ * values given by @x.  The new variable is added at one
+ * level "deeper" (in terms of function execution) than the
+ * current level.  This is for use with user-defined functions.
+ *
+ * Returns: 0 on success, %E_ALLOC on error.
+ */
+
+int dataset_add_series_as (double *x, const char *newname,
+			   double ***pZ, DATAINFO *pdinfo)
+{
+    int v, t, err = 0;
+
+    if (pdinfo->varinfo == NULL) {
+	strcpy(gretl_errmsg, _("Please open a data file first"));
+	return 1;
+    }
+
+#if DDEBUG
+    fprintf(stderr, "dataset_add_series_as: incoming Z=%p, name='%s'\n",
+	    (void *) *pZ, newname);
+#endif
+
+    err = real_add_series(1, NULL, pZ, pdinfo);
+
+    if (!err) {
+	v = pdinfo->v - 1;
+	for (t=0; t<pdinfo->n; t++) {
+	    (*pZ)[v][t] = x[t];
+	}
+	strcpy(pdinfo->varname[v], newname);
+	STACK_LEVEL(pdinfo, v) += 1;
     }
 
     return err;
@@ -1465,7 +1497,7 @@ int dataset_copy_variable_as (int v, const char *newname,
     int t, err;
 
     if (var_is_series(pdinfo, v)) {
-	err = real_dataset_add_series(1, NULL, pZ, pdinfo);
+	err = real_add_series(1, NULL, pZ, pdinfo);
     } else {
 	err = dataset_add_scalar(pZ, pdinfo);
     }
@@ -1494,9 +1526,10 @@ enum {
 };
 
 /* DROP_SPECIAL is used when deleting variables from the "full" shadow
-   of a sub-sampled dataset: in that case we don't mess with the
+   of a sub-sampled dataset, after deleting those same variables from
+   the sub-sampled version: in that case we don't mess with the
    pointer elements of the DATAINFO struct, because these are shared
-   between the full and sub-sampled datasets.
+   between the full and sub-sampled versions.
 */
 
 static int 
@@ -1504,6 +1537,12 @@ shrink_dataset_to_size (double ***pZ, DATAINFO *pdinfo, int nv, int drop)
 {
     double **newZ;
 
+#if DDEBUG
+    fprintf(stderr, "shrink_dataset_to_size: nv=%d, got pZ at %p, pdinfo at %p\n"
+	    " drop = %s\n", nv, (void *) pZ, (void *) pdinfo, 
+	    (drop == DROP_NORMAL)? "DROP_NORMAL" : "DROP_SPECIAL");
+#endif
+
     if (drop == DROP_NORMAL) {
 	char **varname;
 	VARINFO **varinfo;
@@ -1532,42 +1571,65 @@ shrink_dataset_to_size (double ***pZ, DATAINFO *pdinfo, int nv, int drop)
     return 0;
 }
 
-#define DROPDBG 0
+static int vars_renumbered (const int *list, DATAINFO *pdinfo,
+			    int dmin)
+{
+    int i, ndel = 0;
 
-static int real_dataset_drop_listed_vars (const int *list, double ***pZ, 
-					  DATAINFO *pdinfo, int *renumber,
-					  int drop)
+    for (i=dmin; i<pdinfo->v; i++) {
+	if (in_gretl_list(list, i)) {
+	    ndel++;
+	} else if (ndel > 0 && !var_is_hidden(pdinfo, i)) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+int overwrite_err (const DATAINFO *pdinfo, int v)
+{
+    sprintf(gretl_errmsg, "The variable %s is read-only", 
+	    pdinfo->varname[v]);
+
+    return E_DATA;
+}
+
+static int real_drop_listed_vars (const int *list, double ***pZ, 
+				  DATAINFO *pdinfo, int *renumber,
+				  int drop)
 {
     double **Z = *pZ;
     int oldv = pdinfo->v, vmax = pdinfo->v;
+    int delmin = oldv;
     int i, v, ndel = 0; 
+    int err;
+
+    if (renumber != NULL) {
+	*renumber = 0;
+    }
 
     if (list == NULL) {
 	/* no-op */
 	return 0;
     }
 
-    if (renumber != NULL) {
-	*renumber = 0;
-    }
-
-#if DROPDBG
-    fprintf(stderr, "dataset_drop_listed_variables: dropping %d vars:\n",
+#if DDEBUG
+    fprintf(stderr, "real_drop_listed_variables: dropping %d vars:\n",
 	    list[0]);
 #endif
 
-    /* free and set to NULL all the vars to be deleted */
+    /* check that no vars to be deleted are marked "const", and do
+       some preliminary accounting while we're at it */
 
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
 	if (v > 0 && v < oldv) {
-	    free(Z[v]);
-	    Z[v] = NULL;
-	    if (drop == DROP_NORMAL) {
-		free(pdinfo->varname[v]);
-		if (pdinfo->varinfo[v] != NULL) {
-		    free(pdinfo->varinfo[v]);
-		}
+	    if (var_is_const(pdinfo, v)) {
+		return overwrite_err(pdinfo, v);
+	    }
+	    if (v < delmin) {
+		delmin = v;
 	    }
 	    ndel++;
 	}
@@ -1577,7 +1639,29 @@ static int real_dataset_drop_listed_vars (const int *list, double ***pZ,
 	return 0;
     }
 
-    /* rearrange pointers if necessary */
+    if (renumber != NULL) {
+	*renumber = vars_renumbered(list, pdinfo, delmin);
+    }    
+
+#if DDEBUG
+    fprintf(stderr, "real_drop_listed_variables: lowest ID of deleted var"
+	    " = %d\n", delmin);
+#endif
+
+    /* free and set to NULL all the vars to be deleted */
+    for (i=1; i<=list[0]; i++) {
+	v = list[i];
+	if (v > 0 && v < oldv) {
+	    free(Z[v]);
+	    Z[v] = NULL;
+	    if (drop == DROP_NORMAL) {
+		free(pdinfo->varname[v]);
+		free(pdinfo->varinfo[v]);
+	    }
+	}
+    }
+
+    /* repack pointers if necessary */
 
     for (v=1; v<vmax; v++) {
 	if (Z[v] == NULL) {
@@ -1595,23 +1679,39 @@ static int real_dataset_drop_listed_vars (const int *list, double ***pZ,
 		vmax -= gap;
 		for (i=v; i<vmax; i++) {
 		    if (drop == DROP_NORMAL) {
-			if (renumber != NULL && 
-			    !var_is_hidden(pdinfo, i + gap)) {
-			    *renumber = 1;
-			}
 			pdinfo->varname[i] = pdinfo->varname[i + gap];
 			pdinfo->varinfo[i] = pdinfo->varinfo[i + gap];
 		    }
 		    Z[i] = Z[i + gap];
 		}		    
 	    } else {
-		/* deleting all subsequent vars */
+		/* deleting all subsequent vars: done */
 		break;
 	    }
 	}
     }
 
-    return shrink_dataset_to_size(pZ, pdinfo, oldv - ndel, drop);
+    err = shrink_dataset_to_size(pZ, pdinfo, oldv - ndel, drop);
+
+    return err;
+}
+
+static int *make_dollar_list (DATAINFO *pdinfo, int *err)
+{
+    int *list = NULL;
+    int i;
+
+    for (i=1; i<pdinfo->v; i++) {
+	if (pdinfo->varname[i][0] == '$') {
+	    list = gretl_list_append_term(&list, i);
+	    if (list == NULL) {
+		*err = E_ALLOC;
+		break;
+	    }
+	}
+    }
+
+    return list;
 }
 
 /**
@@ -1631,23 +1731,54 @@ static int real_dataset_drop_listed_vars (const int *list, double ***pZ,
  * Returns: 0 on success, %E_ALLOC on error.
  */
 
-int dataset_drop_listed_variables (const int *list, double ***pZ, 
-				   DATAINFO *pdinfo, int *renumber)
+int dataset_drop_listed_variables (int *list, double ***pZ, 
+				   DATAINFO *pdinfo, 
+				   int *renumber)
 {
-    int err;
+    int *dlist = NULL;
+    int free_dlist = 0;
+    int lastvar[2];
+    int err = 0;
 
-    err = real_dataset_drop_listed_vars(list, pZ, pdinfo, renumber,
-					DROP_NORMAL);
+    if (list == NULL) {
+	/* signal to drop internal "$" variables */
+	dlist = make_dollar_list(pdinfo, &err);
+	if (err) {
+	    return err;
+	} else if (dlist == NULL) {
+	    /* no-op */
+	    return 0;
+	}
+	free_dlist = 1;
+    } else if (list[0] == 0) {
+	/* signal to drop last variable */
+	lastvar[0] = 1;
+	lastvar[1] = pdinfo->v - 1;
+	dlist = lastvar;
+    } else {
+	dlist = list;
+    }
+
+    err = real_drop_listed_vars(dlist, pZ, pdinfo, renumber,
+				DROP_NORMAL);
+
+    if (!err) {
+	err = gretl_lists_revise(dlist, 0);
+    }
 
     if (!err && complex_subsampled()) {
 	double ***fZ = fetch_full_Z();
 	DATAINFO *fdinfo = fetch_full_datainfo();
 
-	err = real_dataset_drop_listed_vars(list, fZ, fdinfo, NULL,
-					    DROP_SPECIAL);
+	err = real_drop_listed_vars(dlist, fZ, fdinfo, NULL,
+				    DROP_SPECIAL);
 	reset_full_Z(fZ);
     }
 
+    if (free_dlist) {
+	free(dlist);
+    }
+
     return err;
 }
 
@@ -1666,6 +1797,10 @@ int dataset_drop_variable (int v, double ***pZ, DATAINFO *pdinfo)
 {
     int list[2] = {1, v};
 
+    if (v <= 0 || v >= pdinfo->v) {
+	return E_DATA;
+    }
+
     return dataset_drop_listed_variables(list, pZ, pdinfo, NULL);
 }
 
@@ -1699,20 +1834,20 @@ int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo,
     }
 
     if (nhid > 0) {
-	int *hidlist = gretl_list_new(nhid);
+	int *list = gretl_list_new(nhid);
 
-	if (hidlist == NULL) {
+	if (list == NULL) {
 	    err = 1;
 	} else {
 	    int j = 1;
 
 	    for (i=vmin; i<pdinfo->v; i++) {
 		if (var_is_hidden(pdinfo, i)) {
-		    hidlist[j++] = i;
+		    list[j++] = i;
 		}
 	    }	    
-	    err = dataset_drop_listed_variables(hidlist, pZ, pdinfo, NULL);
-	    free(hidlist);
+	    err = dataset_drop_listed_variables(list, pZ, pdinfo, NULL);
+	    free(list);
 	}
     }
 
@@ -1733,29 +1868,60 @@ int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo,
 
 int dataset_drop_last_variables (int delvars, double ***pZ, DATAINFO *pdinfo)
 {
-    int i, v = pdinfo->v;   
+    int newv, v = pdinfo->v;
+    int i, err = 0;
 
     if (delvars <= 0) {
 	return 0;
     }
 
-    if (pdinfo->v <= 1) {
+    newv = pdinfo->v - delvars;
+
+    if (newv < 1) {
+	fprintf(stderr, "dataset_drop_last_vars: pdinfo->v = %d, delvars = %d "
+		" -> newv = %d\n (pdinfo = %p)\n", pdinfo->v, delvars,
+		newv, (void *) pdinfo);
 	return E_DATA;
     }
 
-    for (i=v-delvars; i<v; i++) {
-	if (pdinfo->varname[i] != NULL) {
-	    free(pdinfo->varname[i]);
-	}
-	if (pdinfo->varinfo[i] != NULL) {
-	    free_varinfo(pdinfo, i);
-	}
-	if ((*pZ)[i] != NULL) {
-	    free((*pZ)[i]);
+    for (i=newv; i<pdinfo->v; i++) {
+	free(pdinfo->varname[i]);
+	free_varinfo(pdinfo, i);
+	free((*pZ)[i]);
+	(*pZ)[i] = NULL;
+    }
+
+    err = shrink_dataset_to_size(pZ, pdinfo, newv, DROP_NORMAL);
+
+    if (!err) {
+	err = gretl_lists_revise(NULL, newv);
+    }
+
+    if (!err && complex_subsampled()) {
+	double ***fZ = fetch_full_Z();
+	DATAINFO *fdinfo = fetch_full_datainfo();
+
+	/* The following is required for the special case of undoing
+	   sub-sampling inside a function, when the dataset is already
+	   subsampled on entry to the function.  Note that we actually
+	   _do_ something here only if the number of variables in the
+	   full dataset equals that in the subsetted dataset (before
+	   the deletions in the latter).  Therefore, specifically, we
+	   do _not_ do anything if the vars deleted (above) from the
+	   subsetted dataset were not present in the full version.
+	*/
+
+	if (fdinfo->v == v) {
+	    for (i=newv; i<fdinfo->v; i++) {
+		free((*fZ)[i]);
+		(*fZ)[i] = NULL;
+	    }
+	    err = shrink_dataset_to_size(fZ, fdinfo, newv, DROP_SPECIAL);
+	    reset_full_Z(fZ);
 	}
     }
 
-    return shrink_dataset_to_size(pZ, pdinfo, v - delvars, DROP_NORMAL);
+    return err;
 }
 
 static void make_stack_label (char *label, char *s)
@@ -1879,10 +2045,10 @@ static int missing_tail (const double *x, int n)
 
 /**
  * dataset_stack_variables:
+ * @vname: name for new variable, to be produced by stacking.
+ * @line: instructions for stacking existing variables.
  * @pZ: pointer to data array.
  * @pdinfo: dataset information.
- * @newvar: name for new variable, produced by stacking
- * @s: instructions for stacking existing variables.
  * @prn: printing apparatus.
  *
  * Really for internal use.  Don't worry about it.
@@ -1890,12 +2056,13 @@ static int missing_tail (const double *x, int n)
  * Returns: 0 on success, non-zero code on error.
  */
 
-int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo, 
-			     char *newvar, char *s, PRN *prn)
+int dataset_stack_variables (const char *vname, const char *line,
+			     double ***pZ, DATAINFO *pdinfo, 
+			     PRN *prn)
 {
     char vn1[VNAMELEN], vn2[VNAMELEN];
     char format[16];
-    char *p, *scpy;
+    char *p, *s = NULL, *scpy = NULL;
     int *vnum = NULL;
     double *bigx = NULL;
     int i, v1 = 0, v2 = 0, nv = 0;
@@ -1903,18 +2070,36 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
     int oldn, bign, genv;
     int err = 0;
 
-    scpy = gretl_strdup(s);
-    if (scpy == NULL) return E_ALLOC;
+    /* copy full "stack(...)" spec for later reference */
+    scpy = gretl_strdup(line);
+    if (scpy == NULL) {
+	return E_ALLOC;
+    }
 
-    genv = varindex(pdinfo, newvar);
+    line += 6; /* skip "stack(" */
+    if (*line == ',') {
+	free(scpy);
+	return E_SYNTAX;
+    }
 
-    s += 6;
-    if (*s == ',') return E_SYNTAX;
+    /* copy active portion of line (we use strtok below) */
+    s = gretl_strdup(line);
+    if (s == NULL) {
+	free(scpy);
+	return E_ALLOC;
+    }
 
     p = strrchr(s, ')');
-    if (p == NULL) return E_SYNTAX;
+    if (p == NULL) {
+	err = E_SYNTAX;
+	goto bailout;
+    }
+
+    /* end of active portion of line */
     *p = '\0';
 
+    genv = varindex(pdinfo, vname);
+
     /* do we have a range of vars? */
     sprintf(format, "%%%d[^.]..%%%ds", VNAMELEN-1, VNAMELEN-1);
     if (sscanf(s, format, vn1, vn2) == 2) {
@@ -1932,7 +2117,7 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
 	    err = E_DATA;
 	}
     } else {
-	/* or do we have a comma separated list of vars? */
+	/* or a comma-separated list of vars? */
 	char *p = s;
 
 	while (*p) {
@@ -1941,7 +2126,9 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
 	}
 	nv++;
 
-	if (nv < 2) return E_SYNTAX;
+	if (nv < 2) {
+	    return E_SYNTAX;
+	}
 
 	vnum = malloc(nv * sizeof *vnum);
 	if (vnum == NULL) {
@@ -1950,6 +2137,7 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
 
 	for (i=0; i<nv && !err; i++) {
 	    p = strtok((i == 0)? s : NULL, ",");
+	    while (*p == ' ') p++;
 	    if (isdigit(*p)) {
 		v1 = atoi(p);
 	    } else {
@@ -2004,7 +2192,9 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
 	    } else {
 		ok = 1;
 	    }
-	    if (ok > maxok) maxok = ok;
+	    if (ok > maxok) {
+		maxok = ok;
+	    }
 	}
 
 	if (maxok * nv <= pdinfo->n && pdinfo->n % maxok == 0) {
@@ -2055,8 +2245,7 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
 	    } else {
 		bigx[bigt] = (*pZ)[j][0];
 	    }
-	    if (pdinfo->S != NULL && bigt != t && 
-		pdinfo->S[bigt][0] == '\0') {
+	    if (pdinfo->S != NULL && bigt != t) {
 		strcpy(pdinfo->S[bigt], pdinfo->S[t]);
 	    }
 	    bigt++;
@@ -2086,20 +2275,34 @@ int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo,
     
     /* complete the details */
     if (!err) {
-	strcpy(pdinfo->varname[genv], newvar);
+	strcpy(pdinfo->varname[genv], vname);
 	make_stack_label(VARLABEL(pdinfo, genv), scpy);
-	pprintf(prn, "%s %s %s (ID %d)", 
+	pprintf(prn, "%s %s %s (ID %d)\n", 
 		(genv == pdinfo->v - 1)? _("Generated") : _("Replaced"),
-		_("vector"), newvar, genv);
+		_("series"), vname, genv);
     }
 
  bailout:
 
     free(vnum);
+    free(s);
+    free(scpy);
 
     return err;
 }
 
+static int found_log_parent (const char *s, char *targ)
+{
+    int len = gretl_varchar_spn(s);
+
+    if (len < VNAMELEN && s[len] == ')') {
+	sscanf(s, "%15[^)]", targ);
+	return 1;
+    }
+
+    return 0;
+}
+
 /**
  * is_log_variable:
  * @i: ID number of variable.
@@ -2122,17 +2325,12 @@ int is_log_variable (int i, const DATAINFO *pdinfo, char *parent)
     if (s != NULL && *s != '\0') {
 	if (sscanf(s, "= log of %15s", parent) == 1) {
 	    return 1;
+	} else if (!strncmp(s, "log(", 4)) {
+	    return found_log_parent(s + 4, parent);
 	} else {
 	    s += strcspn(s, "=");
 	    if (!strncmp(s, "=log(", 5)) {
-		int len;
-
-		s += 5;
-		len = gretl_varchar_spn(s);
-		if (len < VNAMELEN && s[len] == ')') {
-		    sscanf(s, "%15[^)]", parent);
-		    return 1;
-		}
+		return found_log_parent(s + 5, parent);
 	    }
 	}
     }
@@ -2233,3 +2431,46 @@ int var_get_linewidth (const DATAINFO *pdinfo, int i)
 	return 0;
     }
 }
+
+int var_set_display_name (DATAINFO *pdinfo, int i,
+			  const char *s) 
+{
+    char *targ = pdinfo->varinfo[i]->display_name;
+
+    *targ = 0;
+    strncat(targ, s, MAXDISP - 1);
+
+    targ = pdinfo->varinfo[i]->gp_display_name;
+    *targ = 0;
+
+#ifdef ENABLE_NLS
+    if (*s != '\0') {
+	char *trs = utf8_to_latin(s);
+
+	if (trs != NULL) {
+	    strncat(targ, trs, MAXDISP - 1);
+	    free(trs);
+	}
+    }
+#else
+    strncat(targ, s, MAXDISP - 1);
+#endif
+
+    return 0;
+}
+
+const char *var_get_graph_name (const DATAINFO *pdinfo, int i)
+{
+    const char *ret = pdinfo->varname[i];
+
+    if (pdinfo->varinfo != NULL && pdinfo->varinfo[i] != NULL) {
+	ret = pdinfo->varinfo[i]->gp_display_name;
+	if (ret[0] == '\0') {
+	    ret = pdinfo->varname[i];
+	}
+    } 
+
+    return ret;
+}
+
+
