@@ -27,9 +27,11 @@ typedef enum {
 } DatasetMarkerType;
 
 typedef enum {
-    VAR_DISCRETE = 1 << 0,
-    VAR_SCALAR   = 1 << 1,
-    VAR_HIDDEN   = 1 << 2
+    VAR_DISCRETE   = 1 << 0,
+    VAR_SCALAR     = 1 << 1,
+    VAR_HIDDEN     = 1 << 2,
+    VAR_GENERATED  = 1 << 3,
+    VAR_SETCONST   = 1 << 4
 } VarinfoFlags;
 
 /**
@@ -204,6 +206,45 @@ typedef enum {
  */
 #define var_is_hidden(p, i) ((p)->varinfo[i]->flags & VAR_HIDDEN)
 
+/**
+ * var_is_generated:
+ * @p: pointer to data information struct.
+ * @i: index number of variable.
+ *
+ * Determine whether or not a variable was generated using
+ * a formula or transformation function.
+ */
+#define var_is_generated(p, i) ((p)->varinfo[i]->flags & VAR_GENERATED)
+
+/**
+ * var_is_const:
+ * @p: pointer to data information struct.
+ * @i: index number of variable.
+ *
+ * Determine whether or not a variable has been marked as
+ * "const".
+ */
+#define var_is_const(p, i) (i == 0 || ((p)->varinfo[i]->flags & VAR_SETCONST))
+
+/**
+ * set_var_const:
+ * @p: pointer to data information struct.
+ * @i: index number of variable.
+ *
+ * Set the "const" flag on the given variable.
+ */
+#define set_var_const(p, i) ((p)->varinfo[i]->flags |= VAR_SETCONST)
+
+/**
+ * unset_var_const:
+ * @p: pointer to data information struct.
+ * @i: index number of variable.
+ *
+ * Remove the "const" flag from the given variable.
+ */
+#define unset_var_const(p, i) ((p)->varinfo[i]->flags &= ~VAR_SETCONST)
+
+
 void free_Z (double **Z, DATAINFO *pdinfo);
 
 DATAINFO *datainfo_new (void);
@@ -266,15 +307,18 @@ int dataset_add_scalars (int n, double ***pZ, DATAINFO *pdinfo);
 
 int dataset_add_scalar (double ***pZ, DATAINFO *pdinfo);
 
-int dataset_add_scalar_as (const char *numstr, const char *newname,
+int dataset_add_scalar_as (double x, const char *newname,
+			   double ***pZ, DATAINFO *pdinfo);
+
+int dataset_add_series_as (double *x, const char *newname,
 			   double ***pZ, DATAINFO *pdinfo);
 
 int dataset_copy_variable_as (int v, const char *newname,
 			      double ***pZ, DATAINFO *pdinfo);
 
-int dataset_scalar_to_vector (int v, double ***pZ, DATAINFO *pdinfo);
+int overwrite_err (const DATAINFO *pdinfo, int v);
 
-int dataset_drop_listed_variables (const int *list, double ***pZ, 
+int dataset_drop_listed_variables (int *list, double ***pZ, 
 				   DATAINFO *pdinfo, int *renumber);
 
 int dataset_drop_variable (int v, double ***pZ, DATAINFO *pdinfo); 
@@ -284,8 +328,9 @@ int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo,
 
 int dataset_drop_last_variables (int delvars, double ***pZ, DATAINFO *pdinfo);
 
-int dataset_stack_variables (double ***pZ, DATAINFO *pdinfo, 
-			     char *newvar, char *s, PRN *prn);
+int dataset_stack_variables (const char *vname, const char *line,
+			     double ***pZ, DATAINFO *pdinfo, 
+			     PRN *prn);
 
 int is_log_variable (int i, const DATAINFO *pdinfo, char *parent);
 
@@ -299,4 +344,9 @@ void var_set_linewidth (DATAINFO *pdinfo, int i, int w);
 
 int var_get_linewidth (const DATAINFO *pdinfo, int i);
 
+int var_set_display_name (DATAINFO *pdinfo, int i,
+			  const char *s); 
+
+const char *var_get_graph_name (const DATAINFO *pdinfo, int i);
+
 #endif /* DATASET_H */
