@@ -21,6 +21,11 @@
 
 #include "libgretl.h"
 #include "swap_bytes.h"
+#include "gretl_www.h"
+
+#if WORDS_BIGENDIAN
+# include <netinet/in.h>
+#endif
 
 #define DB_DEBUG 0
 
@@ -83,10 +88,26 @@ typedef struct {
 static char db_name[MAXLEN];
 static int db_type;
 
+#if WORDS_BIGENDIAN
+typedef struct {
+    long frac;
+    short exp;
+} netfloat;
+
+float retrieve_float (netfloat nf)
+{
+    short exp = ntohs(nf.exp);
+    long frac = ntohl(nf.frac);
+    double receive = frac / 10e6;
+    
+    return ldexp(receive, exp);
+}
+#endif
+
 static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo, 
 			    double ***pZ, DATAINFO *pdinfo,
 			    CompactMethod method, int dbv,
-			    int *newdata);
+			    int *newdata, PRN *prn);
 
 static FILE *open_binfile (const char *dbbase, int code, int offset, int *err)
 {
@@ -147,6 +168,55 @@ int get_native_db_data (const char *dbbase, SERIESINFO *sinfo,
     return err;
 }
 
+int get_remote_db_data (const char *dbbase, SERIESINFO *sinfo, double **Z)
+{
+    char *getbuf = NULL;
+    int t, err, n = sinfo->nobs;
+    dbnumber x;
+    size_t offset;
+#if WORDS_BIGENDIAN
+    netfloat nf;
+#endif
+
+#if WORDS_BIGENDIAN
+    err = retrieve_remote_db_data(dbbase, sinfo->varname, &getbuf,
+				  GRAB_NBO_DATA);
+#else
+    err = retrieve_remote_db_data(dbbase, sinfo->varname, &getbuf,
+				  GRAB_DATA);
+#endif
+
+    if (err) {
+	free(getbuf);
+	return E_FOPEN;
+    } 
+
+    offset = 0L;
+    for (t=0; t<n; t++) {
+#if WORDS_BIGENDIAN
+	/* go via network byte order */
+	memcpy(&(nf.frac), getbuf + offset, sizeof nf.frac);
+	offset += sizeof nf.frac;
+	memcpy(&(nf.exp), getbuf + offset, sizeof nf.exp);
+	offset += sizeof nf.exp;
+	x = retrieve_float(nf);
+#else
+	/* just read floats */
+	memcpy(&x, getbuf + offset, sizeof x);
+	offset += sizeof x;
+#endif
+	if (x == -999.0) {
+	    Z[1][t] = NADBL;
+	} else {
+	    Z[1][t] = x;
+	}
+    }
+
+    free(getbuf);
+
+    return 0;
+}
+
 int get_pcgive_db_data (const char *dbbase, SERIESINFO *sinfo, 
 			double **Z)
 {
@@ -182,29 +252,12 @@ int get_pcgive_db_data (const char *dbbase, SERIESINFO *sinfo,
 
 static void get_native_series_comment (SERIESINFO *sinfo, const char *s)
 {
-    size_t n = strlen(sinfo->varname);
-    const char *p = s + n + 1;
-    int i;
-
-    while (*p) {
-	if (isspace(*p)) p++;
-	else break;
-    }
+    s += strcspn(s, " "); /* skip varname */
+    s += strspn(s, " ");  /* skip space */
 
     *sinfo->descrip = 0;
-    strncat(sinfo->descrip, p, MAXLABEL - 1);
-
-    n = strlen(sinfo->descrip) - 1;
-    
-    for (i=n; i>0; i--) {
-	if (isspace(sinfo->descrip[i])) {
-	    sinfo->descrip[i] = 0;
-	} else if (sinfo->descrip[i] == '\r' || sinfo->descrip[i] == '\n') {
-	    sinfo->descrip[i] = 0;
-	} else {
-	    break;
-	}
-    }
+    strncat(sinfo->descrip, s, MAXLABEL - 1);
+    tailstrip(sinfo->descrip);
 }
 
 static int get_native_series_pd (SERIESINFO *sinfo, char pdc)
@@ -278,9 +331,13 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
 
     while (fgets(line1, sizeof line1, fp) && !gotit) {
 
-	if (*line1 == '#') continue;
+	if (*line1 == '#') {
+	    continue;
+	}
 
-	if (sscanf(line1, "%15s", sername) != 1) break;
+	if (sscanf(line1, "%15s", sername) != 1) {
+	    break;
+	}
 
 	if (!strcmp(series, sername)) {
 	    gotit = 1;
@@ -320,6 +377,71 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
     return err;
 }
 
+static int 
+get_remote_series_info (const char *series, SERIESINFO *sinfo)
+{
+    char *buf = NULL;
+    char sername[VNAMELEN];
+    char line1[256], line2[72];
+    char stobs[11], endobs[11];
+    char pdc;
+    int gotit = 0;
+    int err = 0;
+
+    err = retrieve_remote_db_index(db_name, &buf);
+    if (err) {
+	return err;
+    }
+
+    sinfo->offset = 0;
+
+    bufgets_init(buf);
+
+    while (bufgets(line1, sizeof line1, buf) && !gotit) {
+	if (*line1 == '#') {
+	    continue;
+	}
+
+	if (sscanf(line1, "%15s", sername) != 1) {
+	    break;
+	}
+
+	if (strcmp(series, sername)) {
+	    continue;
+	}
+
+	gotit = 1;
+
+	if (bufgets(line2, sizeof line2, buf) == NULL) {
+	    err = DB_PARSE_ERROR;
+	    break;
+	} 
+
+	strcpy(sinfo->varname, sername);
+	get_native_series_comment(sinfo, line1);
+
+	if (sscanf(line2, "%c %10s %*s %10s %*s %*s %d", 
+		   &pdc, stobs, endobs, &(sinfo->nobs)) != 4) {
+	    strcpy(gretl_errmsg, _("Failed to parse series information"));
+	    err = DB_PARSE_ERROR;
+	} else {
+	    get_native_series_pd(sinfo, pdc);
+	    get_native_series_obs(sinfo, stobs, endobs);
+	}
+    }
+
+    bufgets_finalize(buf);
+
+    free(buf);
+
+    if (!gotit) {
+	sprintf(gretl_errmsg, _("Series not found, '%s'"), series);
+	err = DB_NO_SUCH_SERIES;
+    }
+
+    return err;
+}
+
 static int in7_nobs (int y0, int p0, int y1, int p1, int pd)
 {
     return  (y1 - y0 + 1) * pd - (p0 - 1) - (pd - p1);
@@ -821,7 +943,7 @@ db_table *read_pcgive_db (FILE *fp)
     db_table *tbl;
     int ns, err = 0;
 
-    *gretl_errmsg = 0;
+    gretl_error_clear();
 
     ns = count_in7_series(fp, &err);
     if (ns == 0) {
@@ -873,7 +995,7 @@ db_table *read_rats_db (FILE *fp)
     long forward;
     int i, err = 0;
 
-    *gretl_errmsg = 0;
+    gretl_error_clear();
     
     /* get into position */
     fseek(fp, 30L, SEEK_SET); /* skip unneeded fields */
@@ -1010,7 +1132,7 @@ static int get_rats_series_info (const char *series_name, SERIESINFO *sinfo)
     long forward;
     int err = 0;
 
-    *gretl_errmsg = 0;
+    gretl_error_clear();
 
     fp = gretl_fopen(db_name, "rb");
     if (fp == NULL) {
@@ -1219,8 +1341,8 @@ double *expand_db_series (const double *src, SERIESINFO *sinfo,
     return x;
 }
 
-int set_db_name (const char *fname, int filetype, const PATHS *ppaths, 
-		 PRN *prn)
+int 
+set_db_name (const char *fname, int filetype, const PATHS *ppaths, PRN *prn)
 {
     FILE *fp;
     int err = 0;
@@ -1228,6 +1350,23 @@ int set_db_name (const char *fname, int filetype, const PATHS *ppaths,
     *db_name = 0;
     strncat(db_name, fname, MAXLEN - 1);
 
+    if (filetype == GRETL_NATIVE_DB_WWW) {
+	int n = strlen(db_name);
+
+	if (n > 4) {
+	    n -= 4;
+	    if (!strcmp(db_name + n, ".bin")) {
+		db_name[n] = '\0';
+	    }
+	}
+	err = check_remote_db(db_name);
+	if (!err) {
+	    db_type = filetype;
+	    pprintf(prn, "%s\n", db_name);
+	} 
+	return err;
+    }
+
     fp = gretl_fopen(db_name, "rb");
 
     if (fp == NULL) {
@@ -1245,7 +1384,7 @@ int set_db_name (const char *fname, int filetype, const PATHS *ppaths,
     if (fp == NULL) {
 	*db_name = 0;
 	pprintf(prn, _("Couldn't open %s\n"), fname);
-	err = 1;
+	err = E_FOPEN;
     } else {
 	fclose(fp);
 	db_type = filetype;
@@ -1325,12 +1464,11 @@ get_compact_method_and_advance (const char *s, CompactMethod *method)
 
     *method = COMPACT_NONE;
 
-    if ((p = strstr(s, "(compact"))) {
+    if ((p = strstr(s, "(compact")) != NULL) {
 	char comp[8];
-	int i;
+	int i = 0;
 
 	p += 8;
-	i = 0;
 	while (*p && *p != ')' && i < 7) {
 	    if (!isspace(*p) && *p != '=') {
 		comp[i++] = *p;
@@ -1351,11 +1489,10 @@ get_compact_method_and_advance (const char *s, CompactMethod *method)
 
 	p = strchr(p, ')');
 	if (p != NULL) p++;
+    } else if ((p = strstr(s, "data ")) != NULL) {
+	p += 5;
     } else {
-	/* no compaction method given */
-	if ((p = strstr(s, "data "))) {
-	    p += 5;
-	}
+	p = s;
     }
 
     return p;
@@ -1386,7 +1523,7 @@ static double **new_dbZ (int n)
 int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo, 
 		   PRN *prn)
 {
-    char series[16];
+    char series[VNAMELEN];
     CompactMethod method;
     SERIESINFO sinfo;
     double **dbZ;
@@ -1430,6 +1567,8 @@ int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo,
 	    err = get_rats_series_info(series, &sinfo);
 	} else if (db_type == GRETL_PCGIVE_DB) {
 	    err = get_pcgive_series_info(series, &sinfo);
+	} else if (db_type == GRETL_NATIVE_DB_WWW) {
+	    err = get_remote_series_info(series, &sinfo);
 	} else {
 	    err = get_native_series_info(series, &sinfo);
 	} 
@@ -1447,13 +1586,16 @@ int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo,
 	}
 
 #if DB_DEBUG
-	fprintf(stderr, "db_get_series: offset=%d, nobs=%d\n", sinfo.offset, sinfo.nobs);
+	fprintf(stderr, "db_get_series: offset=%d, nobs=%d\n", 
+		sinfo.offset, sinfo.nobs);
 #endif
 
 	if (db_type == GRETL_RATS_DB) {
 	    err = get_rats_db_data(db_name, &sinfo, dbZ);
 	} else if (db_type == GRETL_PCGIVE_DB) {
 	    err = get_pcgive_db_data(db_name, &sinfo, dbZ);
+	} else if (db_type == GRETL_NATIVE_DB_WWW) {
+	    err = get_remote_db_data(db_name, &sinfo, dbZ);
 	} else {
 	    err = get_native_db_data(db_name, &sinfo, dbZ);
 	} 
@@ -1468,8 +1610,9 @@ int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	if (!err) {
-	    err = cli_add_db_data(dbZ, &sinfo, pZ, pdinfo, this_var_method, v,
-				  &newdata);
+	    err = cli_add_db_data(dbZ, &sinfo, pZ, pdinfo, 
+				  this_var_method, v, 
+				  &newdata, prn);
 	}
 
 	/* free up temp stuff */
@@ -1517,8 +1660,8 @@ int check_db_import (SERIESINFO *sinfo, DATAINFO *pdinfo)
 	sdn_new = get_date_x(sinfo->pd, sinfo->endobs);
 	sdn_old = get_date_x(pdinfo->pd, pdinfo->endobs);
 	if (sd0 > sdn_old || sdn_new < pdinfo->sd0) {
-	    strcpy(gretl_errmsg, _("Observation range does not overlap\nwith the working "
-				   "data set"));
+	    strcpy(gretl_errmsg, _("Observation range does not overlap\n"
+				   "with the working data set"));
 	    err = 1;
 	}
     }
@@ -1554,7 +1697,7 @@ void init_datainfo_from_sinfo (DATAINFO *pdinfo, SERIESINFO *sinfo)
 static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo, 
 			    double ***pZ, DATAINFO *pdinfo,
 			    CompactMethod method, int dbv,
-			    int *newdata)
+			    int *newdata, PRN *prn)
 {
     double *xvec = NULL;
     int pad1 = 0, pad2 = 0;
@@ -1590,7 +1733,8 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 
 #if DB_DEBUG
     fprintf(stderr, "Z=%p\n", (void *) *pZ);
-    fprintf(stderr, "pdinfo->n = %d, pdinfo->v = %d, dbv = %d\n", pdinfo->n, pdinfo->v, dbv);
+    fprintf(stderr, "pdinfo->n = %d, pdinfo->v = %d, dbv = %d\n", 
+	    pdinfo->n, pdinfo->v, dbv);
 #endif
 
     /* is the frequency of the new var higher? */
@@ -1604,12 +1748,10 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	    return 1;
 	}
 	if (method == COMPACT_NONE) {
-	    sprintf(gretl_errmsg, _("%s: you must specify a compaction method"), 
+	    method = COMPACT_AVG;
+	    pprintf(prn, _("%s: using default compaction method: averaging"), 
 		    sinfo->varname);
-	    if (new) {
-		dataset_drop_last_variables(1, pZ, pdinfo);
-	    }
-	    return 1;
+	    pputc(prn, '\n');
 	}
 	xvec = compact_db_series(dbZ[1], sinfo, pdinfo->pd, method);
 	if (xvec == NULL) {
@@ -2113,7 +2255,8 @@ static int get_n_ok_months (const DATAINFO *pdinfo,
 #define WEEKLY_DEBUG 0
 
 static int 
-weeks_to_months_exec (double **mZ, const double **Z, const DATAINFO *pdinfo, int mn)
+weeks_to_months_exec (double **mZ, const double **Z, const DATAINFO *pdinfo, 
+		      int mn)
 { 
     char obsstr[OBSLEN];
     int *den;
@@ -2573,7 +2716,7 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
     char stobs[OBSLEN];
     int i, err = 0;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     if (oldpd == 52) {
 	return weekly_dataset_to_monthly(pZ, pdinfo, default_method);
