@@ -38,6 +38,8 @@ struct Summary_ {
     double *low;
     double *high;
     double *cv;
+    double sw;
+    double sb;
 };
 
 struct FreqDist_ {
@@ -60,7 +62,7 @@ struct Xtab_ {
     char rvarname[VNAMELEN]; 
     char cvarname[VNAMELEN]; 
     int rows, cols;
-    int *rval, *cval;
+    double *rval, *cval;
     int *rtotal, *ctotal;
     int **f;
     int n, missing;
@@ -86,6 +88,8 @@ double gretl_restricted_mean (int t1, int t2, const double *x,
 			      const double *y, GretlOp yop, 
 			      double yval);
 
+double gretl_quantile (int t1, int t2, const double *x, double p);
+
 double gretl_median (int t1, int t2, const double *x);
 
 double gretl_sst (int t1, int t2, const double *x);
@@ -111,14 +115,23 @@ double gretl_corr (int t1, int t2, const double *x, const double *y,
 
 double gretl_corr_rsq (int t1, int t2, const double *x, const double *y);
 
+double gretl_acf (int k, int t1, int t2, const double *y);
+
+double gretl_xcf (int k, int t1, int t2, const double *x, const double *y);
+
 int gretl_moments (int t1, int t2, const double *x, 
 		   double *xbar, double *sd, 
 		   double *skew, double *kurt, int k);
 
 void free_freq (FreqDist *freq);
 
+int freq_setup (int v, const double **Z, const DATAINFO *pdinfo,
+		int *pn, double *pxmax, double *pxmin, int *nbins, 
+		double *binwidth);
+
 FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo, 
-		    int params, gretlopt opt);
+		    double fmin, double fwid, int nbins, int params, 
+		    gretlopt opt, int *err);
 
 int freqdist (int varno, const double **Z, const DATAINFO *pdinfo,
 	      int graph, gretlopt opt, PRN *prn);
@@ -132,11 +145,18 @@ int model_error_dist (const MODEL *pmod, double ***pZ,
 
 int auto_acf_order (int pd, int nobs);
 
+int auto_spectrum_order (int T, gretlopt opt);
+
 int corrgram (int varno, int order, int nparam,
 	      double ***pZ, DATAINFO *pdinfo, 
 	      PRN *prn, gretlopt opt);
 
-int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo, 
+int xcorrgram (const int *list, int order, 
+	      double ***pZ, DATAINFO *pdinfo, 
+	      PRN *prn, gretlopt opt);
+
+int periodogram (int varno, int width, 
+		 double ***pZ, const DATAINFO *pdinfo, 
 		 gretlopt opt, PRN *prn);
 
 Summary *summary (const int *list, const double **Z, 
@@ -149,14 +169,15 @@ void print_summary (const Summary *summ,
 
 void free_summary (Summary *summ);
 
-VMatrix *corrlist (int *list, const double **Z, const DATAINFO *pdinfo);
+VMatrix *corrlist (int *list, const double **Z, const DATAINFO *pdinfo,
+		   gretlopt opt, int *err);
 
 VMatrix *vmatrix_new (void);
 
 void free_vmatrix (VMatrix *vmat);
 
 int gretl_corrmx (int *list, const double **Z, const DATAINFO *pdinfo, 
-		  PRN *prn);
+		  gretlopt opt, PRN *prn);
 
 int means_test (const int *list, const double **Z, 
 		const DATAINFO *pdinfo, 
@@ -165,7 +186,7 @@ int means_test (const int *list, const double **Z,
 int vars_test (const int *list, const double **Z, 
 	       const DATAINFO *pdinfo, PRN *prn);
 
-void matrix_print_corr (VMatrix *corr, const DATAINFO *pdinfo, PRN *prn);
+void print_corrmat (VMatrix *corr, const DATAINFO *pdinfo, PRN *prn);
 
 double dh_root_b1_to_z1 (double rb1, double n);
 
@@ -177,7 +198,6 @@ int gretl_system_normality_test (const gretl_matrix *E,
 				 const gretl_matrix *Sigma, 
 				 PRN *prn);
 
-
 int mahalanobis_distance (const int *list, double ***pZ,
 			  DATAINFO *pdinfo, gretlopt opt,
 			  PRN *prn);
@@ -196,7 +216,7 @@ const int *mahal_dist_get_varlist(const MahalDist *md);
 
 double gretl_gini (int t1, int t2, const double *x);
 
-int gini (int vnum, double ***pZ, DATAINFO *pdinfo, 
+int gini (int vnum, const double **Z, DATAINFO *pdinfo, 
 	  gretlopt opt, PRN *prn);
 
 #endif /* DESCRIBE_H */
