@@ -221,7 +221,7 @@ compute_QML_vcv (MODEL *pmod, const double **Z)
     gretl_matrix *G = NULL;
     gretl_matrix *H = NULL;
     gretl_matrix *S = NULL;
-    gretl_matrix *tmp = NULL;
+    gretl_matrix *V = NULL;
 
     const double *y = Z[pmod->list[1]];
     const double *xi;
@@ -234,9 +234,9 @@ compute_QML_vcv (MODEL *pmod, const double **Z)
     G = gretl_matrix_alloc(k, T);
     H = gretl_matrix_alloc(k, k);
     S = gretl_matrix_alloc(k, k);
-    tmp = gretl_matrix_alloc(k, k);
+    V = gretl_matrix_alloc(k, k);
 
-    if (G == NULL || H == NULL || S == NULL || tmp == NULL) {
+    if (G == NULL || H == NULL || S == NULL || V == NULL) {
 	err = E_ALLOC;
 	goto bailout;
     }
@@ -278,11 +278,11 @@ compute_QML_vcv (MODEL *pmod, const double **Z)
     /* form S = GG' */
     gretl_matrix_multiply_mod(G, GRETL_MOD_NONE,
 			      G, GRETL_MOD_TRANSPOSE,
-			      S);
+			      S, GRETL_MOD_NONE);
 
-    /* form sandwich: H^{-1} S H^{-1} */
-    gretl_matrix_multiply(H, S, tmp);
-    gretl_matrix_multiply(tmp, H, S);
+    /* form sandwich: V = H^{-1} S H^{-1} */
+    gretl_matrix_qform(H, GRETL_MOD_NONE, S,
+		       V, GRETL_MOD_NONE);
 
     pmod->vcv = malloc((k * (k + 1) / 2) * sizeof *pmod->vcv);
     if (pmod->vcv == NULL) {
@@ -293,7 +293,7 @@ compute_QML_vcv (MODEL *pmod, const double **Z)
     /* set VCV and std err values */
     for (i=0; i<k; i++) {
 	for (j=0; j<=i; j++) {
-	    x = gretl_matrix_get(S, i, j);
+	    x = gretl_matrix_get(V, i, j);
 	    pmod->vcv[ijton(i, j, k)] = x;
 	    if (i == j) {
 		pmod->sderr[i] = sqrt(x);
@@ -308,75 +308,230 @@ compute_QML_vcv (MODEL *pmod, const double **Z)
     gretl_matrix_free(G);
     gretl_matrix_free(H);
     gretl_matrix_free(S);
-    gretl_matrix_free(tmp);
+    gretl_matrix_free(V);
 
     return err;
 }
 
-/**
- * logit_probit:
- * @list: dependent variable plus list of regressors.
- * @pZ: pointer to data matrix.
- * @pdinfo: information on the data set.
- * @ci: command index: if = %LOGIT, perform logit regression, otherwise
- * perform probit regression.
- * @opt: if includes %OPT_R form robust (QML) estimates of standard
- * errors and covariance matrix.
- *
- * Computes estimates of the discrete model specified by @list,
- * using an estimator determined by the value of @ci.  Uses the
- * BRMR auxiliary regression; see Davidson and MacKinnon.
- * 
- * Returns: a #MODEL struct, containing the estimates.
- */
+static MODEL ordered_model (const int *list, int ci, double ***pZ, 
+			    DATAINFO *pdinfo, gretlopt opt, PRN *prn)
+{
+    MODEL omod;
+    void *handle;
+    MODEL (* ordered_estimate) (const int *, int, double ***, DATAINFO *, 
+				gretlopt, PRN *);
 
-MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo, 
-		    int ci, gretlopt opt)
+    gretl_error_clear();
+
+    ordered_estimate = get_plugin_function("ordered_estimate", &handle);
+    if (ordered_estimate == NULL) {
+	gretl_model_init(&omod);
+	omod.errcode = E_FOPEN;
+	return omod;
+    }
+
+    omod = (*ordered_estimate) (list, ci, pZ, pdinfo, opt, prn);
+
+    close_plugin(handle);
+
+    set_model_id(&omod);
+
+    return omod;
+}
+
+/* calculate standard errors etc using the Hessian */
+
+static int logit_probit_vcv (MODEL *dmod, gretlopt opt, const double **Z)
+{
+    int i, err = 0;
+
+    if (dmod->vcv != NULL) {
+	free(dmod->vcv);
+	dmod->vcv = NULL;
+    }
+
+    if (dmod->xpx != NULL) {
+	free(dmod->xpx);
+    }
+
+    dmod->xpx = hessian(dmod, Z, dmod->ci);
+    if (dmod->xpx == NULL) {
+	strcpy(gretl_errmsg, _("Failed to construct Hessian matrix"));
+	return E_ALLOC;
+    } 
+
+    if (opt & OPT_R) {
+	err = compute_QML_vcv(dmod, Z);
+	if (err) {
+	    return err;
+	}
+    } else {    
+	/* obtain negative inverse of Hessian */
+	double *xpx = NULL, *diag = NULL;
+
+	cholesky_decomp(dmod->xpx, dmod->ncoeff); 
+
+	diag = malloc(dmod->ncoeff * sizeof *diag); 
+	if (diag == NULL) {
+	    return E_ALLOC;
+	}
+
+	xpx = copyvec(dmod->xpx, dmod->ncoeff * (dmod->ncoeff + 1) / 2);
+	if (xpx == NULL) {
+	    free(diag);
+	    return E_ALLOC;
+	}
+
+	neginv(xpx, diag, dmod->ncoeff);
+
+	for (i=0; i<dmod->ncoeff; i++) {
+	    dmod->sderr[i] = sqrt(diag[i]);
+	}
+
+	free(diag);
+	free(xpx);
+    }
+
+    return err;
+}
+
+/* BRMR, Davidson and MacKinnon, ETM, p. 461 */
+
+static int do_BRMR (const int *list, int *dmodlist, 
+		    MODEL *dmod, int ci, double *beta,
+		    double ***pZ, DATAINFO *pdinfo,
+		    PRN *prn)
+{
+    double lldiff, llbak = -1.0e9;
+    double xx, fx, Fx;
+    double tol = 1.0e-9;
+    int itermax = 250;
+    int v = dmodlist[1];
+    int depvar = list[1];
+    int i, t, iter;
+
+    for (iter=0; iter<itermax; iter++) {
+	double wt;
+
+	/* construct BRMR dataset */
+	for (t=dmod->t1; t<=dmod->t2; t++) {
+	    xx = dmod->yhat[t];
+	    if (na(xx)) {
+		(*pZ)[v][t] = NADBL;
+	    } else {
+		if (ci == LOGIT) {
+		    fx = logit_pdf(xx);
+		    Fx = logit(xx);
+		} else {
+		    fx = normal_pdf(xx);
+		    Fx = normal_cdf(xx);
+		}
+
+		if (Fx < 1.0) {
+		    wt = 1.0 / sqrt(Fx * (1.0 - Fx));
+		} else {
+		    wt = 0.0;
+		}
+
+		(*pZ)[v][t] = wt * ((*pZ)[depvar][t] - Fx);
+		wt *= fx;
+		for (i=2; i<=dmodlist[0]; i++) {
+		    (*pZ)[dmodlist[i]][t] = wt * (*pZ)[list[i]][t];
+		}
+	    }
+	}
+
+	dmod->lnL = logit_probit_llhood((*pZ)[depvar], dmod, ci);
+
+	lldiff = fabs(dmod->lnL - llbak);
+	if (lldiff < tol) {
+	    break; 
+	}
+
+	if (na(dmod->lnL)) {
+	    pprintf(prn, _("Iteration %d: log likelihood = NA"), iter);	
+	} else {
+	    pprintf(prn, _("Iteration %d: log likelihood = %#.12g"), iter, dmod->lnL);
+	}
+	pputc(prn, '\n');
+
+	llbak = dmod->lnL;
+	clear_model(dmod);
+	*dmod = lsq(dmodlist, pZ, pdinfo, OLS, OPT_A);
+	if (dmod->errcode) {
+	    fprintf(stderr, "logit_probit: dmod errcode = %d\n", dmod->errcode);
+	    if (iter > 0) {
+		dmod->errcode = E_NOCONV;
+	    }
+	    break;
+	}
+
+	/* update coefficient estimates */
+	for (i=0; i<dmod->ncoeff; i++) {
+	    if (isnan(dmod->coeff[i])) {
+		fprintf(stderr, "BRMR produced NaN coeff\n");
+		dmod->errcode = E_NOCONV;
+		break;
+	    }
+	    beta[i] += dmod->coeff[i];
+	}
+
+	if (dmod->errcode) {
+	    break;
+	}
+
+	/* calculate yhat */
+	for (t=dmod->t1; t<=dmod->t2; t++) {
+	    if (na(dmod->yhat[t])) {
+		continue;
+	    }
+	    dmod->yhat[t] = 0.0;
+	    for (i=0; i<dmod->ncoeff; i++) {
+		dmod->yhat[t] += beta[i] * (*pZ)[list[i+2]][t];
+	    }
+	}
+    }
+
+    if (lldiff > tol) {
+	dmod->errcode = E_NOCONV;
+    }
+
+    if (!dmod->errcode) {
+	gretl_model_set_int(dmod, "iters", iter);
+	pputc(prn, '\n');
+    }
+
+    return dmod->errcode;
+}
+
+static MODEL 
+binary_logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo, 
+		     int ci, gretlopt opt, PRN *prn)
 {
     int i, t, v, depvar = list[1];
     int nx = list[0] - 1;
     int oldt1 = pdinfo->t1;
     int oldt2 = pdinfo->t2;
     int oldv = pdinfo->v;
-    int itermax = 250;
-    double tol = 1.0e-9; /* ? 1.0e-9 */
     int *dmodlist = NULL;
     int *act_pred = NULL;
     MODEL dmod;
-    int dummy;
-    double xx, zz, fx, Fx, fbx;
-    double lldiff, llbak;
-    int iters;
+    double xx, fbx, f, F;
     double *xbar = NULL;
     double *beta = NULL;
 
     gretl_model_init(&dmod);
     
-    /* check whether depvar is binary */
-    dummy = gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[depvar]);
-    if (!dummy) {
-	dmod.errcode = E_UNSPEC;
-	sprintf(gretl_errmsg, _("The dependent variable '%s' is not a 0/1 "
-				"variable.\n"), pdinfo->varname[depvar]);
-	return dmod;
-    }
-
     dmodlist = gretl_list_new(list[0]);
     if (dmodlist == NULL) {
 	dmod.errcode = E_ALLOC;
 	return dmod;
     } 
 
-    /* allocate space for means of indep vars */
+    /* allocate space for means of indep vars, coeffs */
     xbar = malloc(nx * sizeof *xbar);
-    if (xbar == NULL) {
-	dmod.errcode = E_ALLOC;
-	return dmod;
-    }
-
-    /* space for coefficient vector */
     beta = malloc(nx * sizeof *beta);
-    if (beta == NULL) {
+    if (xbar == NULL || beta == NULL) {
 	dmod.errcode = E_ALLOC;
 	goto bailout;
     }
@@ -392,7 +547,7 @@ MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
     /* make room for full set of transformed vars */
     if (dataset_add_series(list[0], pZ, pdinfo)) {
 	dmod.errcode = E_ALLOC;
-	return dmod;
+	goto bailout;
     }
 
     v = oldv; /* the first newly created variable */
@@ -421,110 +576,11 @@ MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
     dmodlist[0] = list[0];
     dmodlist[1] = v; /* dep var is the newly created one */
 
-    llbak = -1.0e9;
-    
-    /* BRMR, Davidson and MacKinnon, ETM, p. 461 */
-
-    for (iters=0; iters<itermax; iters++) {
-	double wt;
-
-	/* construct BRMR dataset */
-	for (t=dmod.t1; t<=dmod.t2; t++) {
-	    xx = dmod.yhat[t];
-	    if (na(xx)) {
-		(*pZ)[v][t] = NADBL;
-	    } else {
-		if (ci == LOGIT) {
-		    fx = logit_pdf(xx);
-		    Fx = logit(xx);
-		} else {
-		    fx = normal_pdf(xx);
-		    Fx = normal_cdf(xx);
-		}
-
-		if (Fx < 1.0) {
-		    wt = 1.0 / sqrt(Fx * (1.0 - Fx));
-		} else {
-		    wt = 0.0;
-		}
-
-		(*pZ)[v][t] = wt * ((*pZ)[depvar][t] - Fx);
-#if LPDEBUG
-		fprintf(stderr, "creating Z[%d][%d] = %g * (%g - %g) = %g\n",
-			v, t, wt, (*pZ)[depvar][t], Fx, (*pZ)[v][t]);
-#endif
-		wt *= fx;
-		for (i=2; i<=dmodlist[0]; i++) {
-		    (*pZ)[dmodlist[i]][t] = wt * (*pZ)[list[i]][t];
-#if LPDEBUG
-		    fprintf(stderr, "creating Z[%d][%d] = %g * %g = %g\n",
-			    dmodlist[i], t, wt, (*pZ)[list[i]][t], 
-			    (*pZ)[dmodlist[i]][t]);
-#endif
-		}
-	    }
-	}
-
-	dmod.lnL = logit_probit_llhood((*pZ)[depvar], &dmod, ci);
-
-	lldiff = fabs(dmod.lnL - llbak);
-	if (lldiff < tol) {
-	    break; 
-	}
-
-#if LPDEBUG
-	fprintf(stderr, "\n*** iteration %d: log-likelihood = %g\n", iters, dmod.lnL);
-#endif
-
-	llbak = dmod.lnL;
-	clear_model(&dmod);
-	dmod = lsq(dmodlist, pZ, pdinfo, OLS, OPT_A);
-	if (dmod.errcode) {
-	    fprintf(stderr, "logit_probit: dmod errcode = %d\n", dmod.errcode);
-	    if (iters > 0) {
-		dmod.errcode = E_NOCONV;
-	    }
-#if LPDEBUG
-	    fputs("BRMR dataset\n", stderr);
-	    for (t=dmod.t1; t<=dmod.t2; t++) {
-		for (i=1; i<=dmodlist[0]; i++) {
-		    fprintf(stderr, "%g ", (*pZ)[dmodlist[i]][t]);
-		}
-		fputc('\n', stderr);
-	    }
-#endif
-	    goto bailout;
-	}
-
-	/* update coefficient estimates: FIXME stepsize? */
-	for (i=0; i<dmod.ncoeff; i++) {
-	    if (isnan(dmod.coeff[i])) {
-		fprintf(stderr, "BRMR produced NaN coeff\n");
-		dmod.errcode = E_NOCONV;
-		goto bailout;
-	    }
-	    beta[i] += dmod.coeff[i];
-	}
-
-	/* calculate yhat */
-	for (t=dmod.t1; t<=dmod.t2; t++) {
-	    if (na(dmod.yhat[t])) {
-		continue;
-	    }
-	    dmod.yhat[t] = 0.0;
-	    for (i=0; i<dmod.ncoeff; i++) {
-		dmod.yhat[t] += beta[i] * (*pZ)[list[i+2]][t];
-	    }
-	}
-    }
-
-    if (lldiff > tol) {
-	dmod.errcode = E_NOCONV;
+    do_BRMR(list, dmodlist, &dmod, ci, beta, pZ, pdinfo, prn);
+    if (dmod.errcode) {
 	goto bailout;
     }
 
-    gretl_model_set_int(&dmod, "iters", iters);
-
     /* re-establish original list in model */
     for (i=1; i<=list[0]; i++) {
 	dmod.list[i] = list[i];
@@ -539,55 +595,10 @@ MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
     Lr_chisq(&dmod, *pZ);
     dmod.ci = ci;
 
-    /* calculate standard errors etc using the Hessian */
-
-    if (dmod.vcv != NULL) {
-	free(dmod.vcv);
-	dmod.vcv = NULL;
-    }
-
-    if (dmod.xpx != NULL) {
-	free(dmod.xpx);
-    }
-
-    dmod.xpx = hessian(&dmod, (const double **) *pZ, ci);
-    if (dmod.xpx == NULL) {
-	dmod.errcode = E_ALLOC;
-	strcpy(gretl_errmsg, _("Failed to construct Hessian matrix"));
+    /* calculate standard errors etc */
+    dmod.errcode = logit_probit_vcv(&dmod, opt, (const double **) *pZ);
+    if (dmod.errcode) {
 	goto bailout;
-    } 
-
-    if (opt & OPT_R) {
-	dmod.errcode = compute_QML_vcv(&dmod, (const double **) *pZ);
-	if (dmod.errcode) {
-	    goto bailout;
-	}
-    } else {    
-	/* obtain negative inverse of Hessian */
-	double *xpx = NULL, *diag = NULL;
-
-	cholesky_decomp(dmod.xpx, dmod.ncoeff); 
-	diag = malloc(dmod.ncoeff * sizeof *diag); 
-	if (diag == NULL) {
-	    dmod.errcode = E_ALLOC;
-	    goto bailout;
-	}
-
-	xpx = copyvec(dmod.xpx, dmod.ncoeff * (dmod.ncoeff + 1) / 2);
-	if (xpx == NULL) {
-	    free(diag);
-	    dmod.errcode = E_ALLOC;
-	    goto bailout;
-	}
-
-	neginv(xpx, diag, dmod.ncoeff);
-
-	for (i=0; i<dmod.ncoeff; i++) {
-	    dmod.sderr[i] = sqrt(diag[i]);
-	}
-
-	free(diag);
-	free(xpx);
     }
 
     /* apparatus for calculating slopes at means */
@@ -614,24 +625,29 @@ MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
     xx = 0.0;
     for (t=dmod.t1; t<=dmod.t2; t++) {
 	double xb = dmod.yhat[t];
+	int yt;
 
 	if (model_missing(&dmod, t)) {
 	    continue;
 	}
-	zz = (*pZ)[depvar][t];
-	xx += zz;
+
+	yt = (int) (*pZ)[depvar][t];
+	xx += yt;
 
 	if (act_pred != NULL) {
-	    i = 2 * (floateq(zz, 1.0)) + (xb > 0.0);
+	    i = 2 * yt + (xb > 0.0);
 	    act_pred[i] += 1;
 	}
 
 	if (dmod.ci == LOGIT) {
-	   dmod.yhat[t] = exp(xb) / (1.0 + exp(xb)); 
+	    dmod.yhat[t] = exp(xb) / (1.0 + exp(xb)); 
+	    dmod.uhat[t] = yt - dmod.yhat[t];
 	} else {
-	   dmod.yhat[t] = normal_cdf(xb); 
+	    f = normal_pdf(xb);
+	    F = normal_cdf(xb);
+	    dmod.yhat[t] = F; 
+	    dmod.uhat[t] = yt ? f/F : -f/(1-F);
 	}
-	dmod.uhat[t] = zz - dmod.yhat[t];
     }
 
     xx /= dmod.nobs;
@@ -662,6 +678,68 @@ MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo,
     return dmod;
 }
 
+static int 
+ordered_model_ok (double **Z, const DATAINFO *pdinfo, int v)
+{
+    if (!var_is_discrete(pdinfo, v)) {
+	sprintf(gretl_errmsg, "The variable '%s' is not discrete",
+		pdinfo->varname[v]);
+	return 0;
+    } else {
+	double m = gretl_min(pdinfo->t1, pdinfo->t2, Z[v]);
+
+	if (m != 0.0) {
+	    sprintf(gretl_errmsg, "The minimum value of '%s' is not 0",
+		    pdinfo->varname[v]);
+	    return 0;
+	}
+    }
+
+    return 1;
+}
+
+/**
+ * logit_probit:
+ * @list: dependent variable plus list of regressors.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the data set.
+ * @ci: command index: if = %LOGIT, perform logit regression, otherwise
+ * perform probit regression.
+ * @opt: if includes %OPT_R form robust (QML) estimates of standard
+ * errors and covariance matrix, in binary case.
+ * @prn: printing struct in case additional information is
+ * wanted (%OPT_V).
+ *
+ * Computes estimates of the discrete model specified by @list,
+ * using an estimator determined by the value of @ci.  In the
+ * binary case, uses the BRMR auxiliary regression; see Davidson 
+ * and MacKinnon.  If the dependent variable is not binary but 
+ * is discrete and has a minimum value of 0, we do ordered 
+ * logit/probit.
+ * 
+ * Returns: a #MODEL struct, containing the estimates.
+ */
+
+MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo, 
+		    int ci, gretlopt opt, PRN *prn)
+{
+    int yv = list[1];
+
+    if (gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[yv])) {
+	return binary_logit_probit(list, pZ, pdinfo, ci, opt, prn);
+    } else if (ordered_model_ok(*pZ, pdinfo, yv)) {
+	return ordered_model(list, ci, pZ, pdinfo, opt, prn);
+    } else {
+	MODEL dmod;
+
+	gretl_model_init(&dmod);
+	dmod.errcode = E_UNSPEC;
+	sprintf(gretl_errmsg, _("The dependent variable '%s' is not a 0/1 "
+				"variable.\n"), pdinfo->varname[yv]);
+	return dmod;
+    }
+}
+
 /* Solves for diagonal elements of X'X inverse matrix.
    X'X must be Cholesky-decomposed already.
 */
