@@ -50,6 +50,10 @@ MODEL mp_ols (const int *list, const double **Z, DATAINFO *pdinfo);
 MODEL panel_model (const int *list, double ***pZ, DATAINFO *pdinfo,
 		   gretlopt opt, PRN *prn);
 
+MODEL arbond_model (const int *list, const char *istr, const double **Z, 
+		    const DATAINFO *pdinfo, gretlopt opt, 
+		    PRN *prn);
+
 int groupwise_hetero_test (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 			   PRN *prn);
 
@@ -63,12 +67,8 @@ MODEL ar_func (const int *list,
 	       double ***pZ, DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn);
 
-MODEL arch_test (MODEL *pmod, int order, 
-		 double ***pZ, DATAINFO *pdinfo, 
-		 gretlopt opt, PRN *prn);
-
-int arch_test_simple (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo, 
-		      gretlopt opt, PRN *prn);
+int arch_test (MODEL *pmod, int order, const DATAINFO *pdinfo, 
+	       gretlopt opt, PRN *prn);
 
 MODEL arch_model (const int *list, int order, 
 		  double ***pZ, DATAINFO *pdinfo, 
@@ -79,6 +79,10 @@ int makevcv (MODEL *pmod, double sigma);
 int *augment_regression_list (const int *orig, int aux, 
 			      double ***pZ, DATAINFO *pdinfo);
 
+int gretl_XTX_XTy (const int *list, int t1, int t2, 
+		   const double **Z, int nwt, double rho, int pwe,
+		   double *xpx, double *xpy, const char *mask);
+
 #endif /* ESTIMATE_H */
 
 
