@@ -18,6 +18,7 @@
  */
 
 #include "libgretl.h"
+#include "matrix_extra.h"
 #include "forecast.h"
 #include "var.h"
 
@@ -144,6 +145,7 @@ static FITRESID *fit_resid_new (int n)
 
     fr->model_ID = 0;
     fr->model_ci = 0;
+    fr->model_t1 = 0;
     fr->err = 0;
     fr->t0 = 0;
     fr->t1 = 0;
@@ -216,8 +218,6 @@ FITRESID *get_fit_resid (const MODEL *pmod, const double **Z,
     int depvar, t;
     FITRESID *fr;
 
-    depvar = gretl_model_get_depvar(pmod);
-
     fr = fit_resid_new(pdinfo->n);
     if (fr == NULL) {
 	return NULL;
@@ -226,9 +226,15 @@ FITRESID *get_fit_resid (const MODEL *pmod, const double **Z,
     if (LIMDEP(pmod->ci)) {
 	fr->sigma = NADBL;
     } else {
-	fr->sigma = pmod->sigma;
+	fr->sigma = gretl_model_get_double(pmod, "sigma_orig");
+
+	if (na(fr->sigma)) {
+	    fr->sigma = pmod->sigma;
+	}
     }
 
+    depvar = gretl_model_get_depvar(pmod);
+
     fr->t0 = pmod->t1;
     fr->t1 = pmod->t1;
     fr->t2 = pmod->t2;
@@ -406,6 +412,7 @@ fit_resid_init (int t0, int t1, int t2, const MODEL *pmod,
 
     fr->model_ID = pmod->ID;
     fr->model_ci = pmod->ci;
+    fr->model_t1 = pmod->t1;
 
     fr->pmax = PMAX_NOT_AVAILABLE;
 
@@ -534,7 +541,7 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 					       NULL);
 
 	/* forecast variance */
-	vyh = gretl_scalar_b_X_b(Xs, GRETL_MOD_NONE, V, NULL);
+	vyh = gretl_scalar_qform(Xs, V, &err);
 	if (na(vyh)) {
 	    err = 1;
 	} else {
@@ -2005,6 +2012,10 @@ int add_forecast (const char *str, MODEL *pmod, double ***pZ,
     char sdname[VNAMELEN];
     int nf, err = 0;
 
+    if (pmod->ci == ARBOND) {
+	return E_NOTIMP; /* FIXME */
+    }
+
     *t1str = '\0'; *t2str = '\0';
 
     /* Reject in case model was estimated using repacked daily
@@ -2138,6 +2149,10 @@ int display_forecast (const char *str, MODEL *pmod,
     int t1, t2;
     int err;
 
+    if (pmod->ci == ARBOND) {
+	return E_NOTIMP;
+    }
+
     err = parse_forecast_string(str, pmod, pdinfo, &t1, &t2);
     if (err) {
 	return err;
@@ -2389,3 +2404,90 @@ void forecast_options_for_model (MODEL *pmod, const double **Z,
 	free(dvlags);
     }
 }
+
+/* recursive one-step ahead forecasts and forecast errors, for models
+   estimated via OLS */
+
+FITRESID * 
+rolling_OLS_one_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
+			    int t0, int t1, int t2)
+{
+    FITRESID *fr;
+    int orig_t1 = pdinfo->t1;
+    int orig_t2 = pdinfo->t2;
+    double xit, yf;
+    int nf = t2 - t1 + 1;
+    MODEL mod;
+    int i, s, t;
+    int err = 0;
+
+    fr = fit_resid_new(0); 
+    if (fr == NULL) {
+	return NULL;
+    }
+
+    if (pmod->ci != OLS) {
+	fr->err = E_OLSONLY;
+	return fr;
+    }
+
+    if (gretl_model_get_int(pmod, "daily_repack")) {
+	fr->err = E_DATA;
+	return fr;
+    }
+
+    if (t1 - t0 < pmod->ncoeff || t2 < t1) {
+	fr->err = E_DATA;
+	return fr;
+    }
+
+    fit_resid_init(t1, t1, t2, pmod, pdinfo, fr); 
+    if (fr->err) {
+	return fr;
+    }
+
+    fr->method = FC_ONESTEP;
+
+    pdinfo->t1 = t0;
+    pdinfo->t2 = t1 - 1;
+
+    for (t=0; t<pdinfo->n; t++) {
+	fr->actual[t] = (*pZ)[pmod->list[1]][t];
+    }
+
+    for (s=0; s<nf; s++) {
+	mod = lsq(pmod->list, pZ, pdinfo, OLS, OPT_A);
+	if (mod.errcode) {
+	    err = mod.errcode;
+	    clear_model(&mod);
+	    break;
+	}
+
+	t = pdinfo->t2 + 1;
+	yf = 0.0;
+	for (i=0; i<mod.ncoeff; i++) {
+	    xit = (*pZ)[mod.list[i+2]][t];
+	    if (na(xit)) {
+		yf = NADBL;
+		break;
+	    } else {
+		yf += mod.coeff[i] * xit;
+	    }
+	}
+
+	fr->fitted[t] = yf;
+	
+	clear_model(&mod);
+	pdinfo->t2 += 1;
+    }
+
+    pdinfo->t1 = orig_t1;
+    pdinfo->t2 = orig_t2;
+
+    if (!fr->err) {
+	fit_resid_set_dec_places(fr);
+	strcpy(fr->depvar, pdinfo->varname[pmod->list[1]]);
+    }
+	
+    return fr;
+}
