@@ -37,7 +37,8 @@ static struct gretl_cmd gretl_cmds[] = {
     { ADDTO,    "addto" },
     { ADF,      "adf" }, 
     { APPEND,   "append" },
-    { AR,       "ar" },       
+    { AR,       "ar" },  
+    { ARBOND,   "arbond" },
     { ARCH,     "arch" },
     { ARMA,     "arima" },
     { BREAK,    "break" },
@@ -50,11 +51,11 @@ static struct gretl_cmd gretl_cmds[] = {
     { CORR,     "corr" },     
     { CORRGM,   "corrgm" },   
     { CRITERIA, "criteria" },
-    { CRITICAL, "critical" },
     { CUSUM,    "cusum" },
     { DATA,     "data" },
     { DELEET,   "delete" },
     { DIFF,     "diff" },
+    { DIFFTEST, "difftest" },
     { DISCRETE, "discrete" },
     { DUMMIFY,  "dummify" },
     { ELSE,     "else" },
@@ -67,12 +68,12 @@ static struct gretl_cmd gretl_cmds[] = {
     { FCAST,    "fcast" }, 
     { FCASTERR, "fcasterr" },
     { FIT,      "fit" }, 
-    { FNCALL,   "fncall" },
     { FREQ,     "freq" }, 
     { FUNC,     "function" },
     { FUNCERR,  "error" },
     { GARCH,    "garch" },
-    { GENR,     "genr" },     
+    { GENR,     "genr" },  
+    { GMM,      "gmm" },
     { GNUPLOT,  "gnuplot" },  
     { GRAPH,    "graph" }, 
     { HAUSMAN,  "hausman" },
@@ -82,7 +83,6 @@ static struct gretl_cmd gretl_cmds[] = {
     { HSK,      "hsk" }, 
     { HURST,    "hurst" },
     { IF,       "if" },
-    { IMPORT,   "import" },
     { INCLUDE,  "include" },
     { INFO,     "info" }, 
     { KPSS,     "kpss" },
@@ -97,7 +97,6 @@ static struct gretl_cmd gretl_cmds[] = {
     { LOGS,     "logs" },
     { LOOP,     "loop" },
     { MAHAL,    "mahal" },
-    { MATRIX,   "matrix" },
     { MEANTEST, "meantest" },
     { MLE,      "mle" },
     { MODELTAB, "modeltab" },
@@ -136,11 +135,13 @@ static struct gretl_cmd gretl_cmds[] = {
     { SETINFO,  "setinfo" },
     { SETOBS,   "setobs" },
     { SETMISS,  "setmiss" },
-    { SHELL,    "shell" },   
+    { SHELL,    "shell" },
     { SMPL,     "smpl" },
     { SPEARMAN, "spearman" },
+    { SPRINTF,  "sprintf" },
     { SQUARE,   "square" },  
     { STORE,    "store" },   
+    { STRING,   "string" },
     { SUMMARY,  "summary" },
     { SYSTEM,   "system" },
     { TABPRINT, "tabprint" },
@@ -154,6 +155,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { VECM,     "vecm" },
     { VIF,      "vif" },
     { WLS,      "wls" },
+    { XCORRGM,  "xcorrgm" },
     { XTAB,     "xtab" },
     { NC,       NULL}
 }; 
@@ -161,7 +163,8 @@ static struct gretl_cmd gretl_cmds[] = {
 static struct gretl_cmd gretl_cmd_aliases[] = {
     { GENR, "eval" },   
     { GENR, "series" },   
-    { GENR, "scalar" },   
+    { GENR, "scalar" },  
+    { GENR, "matrix" },  
     { LOOP, "for" },   
     { LOOP, "foreach" },
     { LOOP, "while" },
