@@ -27,7 +27,8 @@ typedef enum {
     ADDTO,
     ADF,
     APPEND,
-    AR,      
+    AR,  
+    ARBOND,
     ARCH,
     ARMA,
     BREAK,
@@ -40,11 +41,11 @@ typedef enum {
     CORR,     
     CORRGM,   
     CRITERIA,
-    CRITICAL,
     CUSUM,
     DATA,
     DELEET,
     DIFF,
+    DIFFTEST,
     DISCRETE,
     DUMMIFY,
     ELSE,
@@ -57,12 +58,12 @@ typedef enum {
     FCAST,   
     FCASTERR, 
     FIT,  
-    FNCALL,
     FREQ, 
     FUNC,
     FUNCERR,
     GARCH,
-    GENR,     
+    GENR,  
+    GMM,
     GNUPLOT, 
     GRAPH,
     HAUSMAN,
@@ -72,7 +73,6 @@ typedef enum {
     HSK,
     HURST,
     IF,
-    IMPORT,
     INCLUDE,
     INFO,
     KPSS,
@@ -87,7 +87,6 @@ typedef enum {
     LOGS,
     LOOP,
     MAHAL,
-    MATRIX,
     MEANTEST,
     MLE,
     MODELTAB,
@@ -126,11 +125,13 @@ typedef enum {
     SETINFO,
     SETOBS,
     SETMISS,
-    SHELL,   
+    SHELL,  
     SMPL,
     SPEARMAN,
+    SPRINTF,
     SQUARE,  
-    STORE,   
+    STORE, 
+    STRING,
     SUMMARY,
     SYSTEM,
     TABPRINT,
@@ -144,14 +145,15 @@ typedef enum {
     VECM,
     VIF,
     WLS,
+    XCORRGM,
     XTAB,
     NC
 } GretlCmdIndex;
 
 #define NEEDS_TWO_VARS(c)  ((c) == AR || (c) == ARCH || (c) == COINT || \
-                            (c) == CORC || (c) == HCCM || (c) == HILU || \
-                            (c) == HSK || (c) == LOGIT || (c) == SPEARMAN || \
-                            (c) == OLS || (c) == PROBIT || \
+                            (c) == CORC || (c) == CORR || (c) == HCCM || \
+                            (c) == HILU || (c) == HSK || (c) == LOGIT || \
+                            (c) == SPEARMAN || (c) == OLS || (c) == PROBIT || \
                             (c) == TSLS || (c) == VAR || (c) == WLS || \
 			    (c) == PWE || (c) == XTAB)
 
@@ -166,7 +168,6 @@ typedef enum {
                         c == CORR || \
                         c == CORRGM || \
                         c == CRITERIA || \
-                        c == CRITICAL || \
                         c == CUSUM || \
                         c == FCAST || \
                         c == FCASTERR || \
@@ -196,6 +197,7 @@ typedef enum {
                         c == TESTUHAT || \
                         c == VIF || \
                         c == VARTEST || \
+                        c == XCORRGM || \
                         c == XTAB)
 
 #define NEEDS_MODEL_CHECK(c) (c == ADD || \
