@@ -17,13 +17,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* errors.h -- prototypes of functions in errors.c */
+#ifndef GRETL_ERRORS_H
+#define GRETL_ERRORS_H
 
-#include <stdio.h>
-
-extern int gretl_errno;
 extern char gretl_errmsg[ERRLEN];
-extern char gretl_msg[ERRLEN];
 
 enum gretl_error_codes {
     E_DATA = 2,
@@ -78,18 +75,14 @@ enum gretl_error_codes {
     E_MAX          /* 51 */
 }; 
 
-/* functions follow */
- 
-char *get_errmsg (const int errcode, char *errtext, PRN *prn);
+void errmsg (int err, PRN *prn);
 
-void errmsg (const int errcode, PRN *prn);
+const char *errmsg_get_with_default (int err);
 
-int get_gretl_errno (void);
-
-const char *get_gretl_errmsg (void);
-
-int print_gretl_errmsg (PRN *prn);
+const char *gretl_errmsg_get (void);
 
 void gretl_errmsg_set (const char *str);
 
-void gretl_errmsg_clear (void);
+void gretl_error_clear (void);
+
+#endif /* GRETL_ERRORS_H */
