@@ -20,30 +20,44 @@
 #ifndef GRETL_FUNC_H
 #define GRETL_FUNC_H
 
-typedef enum {
-    ARG_NONE = 0,
-    ARG_SCALAR,
-    ARG_SERIES,
-    ARG_LIST,
-    ARG_MATRIX,
-    ARG_BOOL,
-    ARG_INT
-} FuncArgTypes;
+#include "usermat.h"
 
 typedef enum {
     FN_NEEDS_TS = 1, /* function requires time-series data */
     FN_NEEDS_QM,     /* function requires quarterly or monthly data */
-    FN_NEEDS_PANEL   /* function requires panel data */
+    FN_NEEDS_PANEL,  /* function requires panel data */
+    FN_NODATA_OK     /* function does not require a dataset */
 } FuncDataReq;
 
 #define NEEDS_TS    "needs-time-series-data"
 #define NEEDS_QM    "needs-qm-data"
 #define NEEDS_PANEL "needs-panel-data"
+#define NO_DATA_OK  "no-data-ok"
+
+#define FN_NAMELEN 32
 
 typedef struct ufunc_ ufunc;
 typedef struct fnpkg_ fnpkg;
+typedef struct fnargs_ fnargs;
 
-int n_user_functions (void);
+struct fnargs_ {
+    char *types;
+    int nx, nX, nM, nl;
+    int nrefv, nrefm;
+    int nnull;
+    int nnames;
+    double *x;
+    double **X;
+    gretl_matrix **M;
+    char **lists;
+    int *refv;
+    user_matrix **refm;
+    char **upnames;
+};
+
+int n_free_functions (void);
+
+ufunc *get_user_function_by_name (const char *name);
 
 const ufunc *get_user_function_by_index (int idx);
 
@@ -59,17 +73,23 @@ double fn_param_minval (const ufunc *fun, int i);
 
 double fn_param_maxval (const ufunc *fun, int i);
 
-int user_func_get_return_types (const ufunc *fun,
-				int *n_returns,
-				char **return_types);
+int fn_param_optional (const ufunc *fun, int i);
+
+int user_func_get_return_type (const ufunc *fun);
 
 const char *user_function_name_by_index (int i);
 
 int user_function_index_by_name (const char *name);
 
+void function_names_init (void);
+
+const char *next_free_function_name (void);
+
 int gretl_compiling_function (void);
 
-int gretl_executing_function (void);
+int gretl_function_depth (void);
+
+int repeating_function_exec (void);
 
 int gretl_start_compiling_function (const char *line, PRN *prn);
 
@@ -79,22 +99,16 @@ int gretl_is_user_function (const char *line);
 
 int gretl_is_public_user_function (const char *name);
 
-int gretl_get_user_function (const char *line, char **fnname);
+int gretl_get_user_function (const char *line);
 
-int is_user_matrix_function (const char *word);
+int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
+			 double ***pZ, DATAINFO *pdinfo,
+			 void *ret, char **descrip,
+			 PRN *prn);
 
-int gretl_function_start_exec (const char *line, const char *fname,
-			       double ***pZ, DATAINFO *pdinfo);
+void set_drop_function_vars (int s);
 
-char *gretl_function_get_line (char *line, int len,
-			       double ***pZ, DATAINFO **ppdinfo,
-			       int *err);
-
-int gretl_function_stack_depth (void);
-
-void gretl_function_stop_on_error (double ***pZ, DATAINFO **ppdinfo, PRN *prn);
-
-int gretl_function_flagged_error (const char *s, PRN *prn);
+const char *get_funcerr_message (void);
 
 int gretl_function_set_info (int i, const char *help);
 
@@ -106,7 +120,7 @@ void gretl_function_set_private (int i, int priv);
 
 int write_function_package (fnpkg *pkg,
 			    const char *fname,
-			    const int *publist, 
+			    int pub, 
 			    const int *privlist, 
 			    const char *author,
 			    const char *version,
@@ -117,7 +131,7 @@ int write_function_package (fnpkg *pkg,
 
 int function_package_get_info (const char *fname,
 			       fnpkg **ppkg,
-			       int **publist, 
+			       int *pub, 
 			       int **privlist,
 			       char **author,
 			       char **version,
@@ -131,7 +145,11 @@ int check_function_needs (const DATAINFO *pdinfo, FuncDataReq dreq,
 
 int write_user_function_file (const char *fname);
 
-int user_function_file_is_loaded (const char *fname);
+int function_package_is_loaded (const char *fname);
+
+const char *function_package_description (const char *fname);
+
+int read_session_functions_file (const char *fname);
 
 int load_user_function_file (const char *fname);
 
@@ -139,7 +157,8 @@ int get_function_file_info (const char *fname, PRN *prn, char **pname);
 
 int get_function_file_code (const char *fname, PRN *prn, char **pname);
 
-char *get_function_file_header (const char *fname, int *err);
+char *get_function_file_header (const char *fname, char **pver,
+				int *err);
 
 int update_function_from_script (const char *fname, int idx);
 
