@@ -18,10 +18,9 @@
  */
 
 #include "libgretl.h"
+#include "texprint.h"
 
-#ifdef USE_GLIB2
-# include <glib.h>
-#endif
+#include <glib.h>
 
 #ifdef ENABLE_NLS
 
@@ -93,7 +92,6 @@ void gretl_pop_c_numeric_locale (void)
  * doing_nls:
  *
  * Returns: 1 if NLS translation is in effect, 0 otherwise.
- *
  */
 
 int doing_nls (void)
@@ -113,15 +111,17 @@ int doing_nls (void)
 #endif
 }
 
-/**
- * get_local_decpoint:
- *
- * Returns: the character representing a decimal point in the current locale.
- *
- */
-
 static int decpoint;
 
+/**
+ * reset_local_decpoint:
+ *
+ * Uses localeconv() to determine the representation of the decimal
+ * point in the current locale.
+ *
+ * Returns: the decimal character for the current locale.
+ */
+
 int reset_local_decpoint (void)
 {
 #ifdef ENABLE_NLS
@@ -135,20 +135,29 @@ int reset_local_decpoint (void)
 #endif
 }
 
+/**
+ * get_local_decpoint:
+ *
+ * Returns: the decimal character for the current locale.
+ */
+
 int get_local_decpoint (void)
 {
 #ifdef ENABLE_NLS
-    if (decpoint == 0) decpoint = reset_local_decpoint();
+    if (decpoint == 0) {
+	decpoint = reset_local_decpoint();
+    }
     return decpoint;
 #else
     return '.';
 #endif
 }
 
-/* fudges for strings that should not be in utf-8 under some 
-   conditions: under gtk2, translations usually come out in
-   utf-8 in the GUI, but when we're sending stuff to stderr,
-   it should probably be in ISO-8859-N. */
+/* fudges for strings that should not be in utf-8 under some
+   conditions: under GTK translations always come out in utf-8 in the
+   GUI, but when we're sending stuff to stderr we may have to put it
+   into ISO-8859-N.
+*/
 
 #ifdef ENABLE_NLS
 
@@ -158,24 +167,23 @@ static int gretl_cset_min;
 static int gretl_cpage;
 # endif
 
+/* Use g_get_charset() to determine the current local character set,
+   and record this information.  If we get an "ISO-XXXX-Y" locale,
+   record the numerical elements as gretl_cset_maj and gretl_cset_min
+   respectively.  If we get a Windows "CPXXXX" reading, record the
+   codepage as gretl_cpage.
+ */
+
 void set_gretl_charset (const char *s)
 {
-# ifdef USE_GLIB2
     const char *charset = NULL;
-# else
-    char charset[16] = {0};
-#endif
     char gretl_charset[32];
     int using_utf8 = 0;
 
-# ifdef USE_GLIB2
     using_utf8 = g_get_charset(&charset);
-# else
-    if (*s == 'p' || *s == 'P') {
-	/* guessing this means Polish */
-	strcpy(charset, "iso-8859-2");
+    if (using_utf8) {
+	set_tex_use_utf(1);
     }
-# endif
 
     *gretl_charset = '\0';
 
@@ -212,7 +220,7 @@ void set_gretl_charset (const char *s)
     }
 }
 
-const char *get_gretl_charset (void)
+static const char *get_gretl_charset (void)
 {
     static char cset[12];
 
@@ -231,7 +239,7 @@ const char *get_gretl_charset (void)
     return NULL;
 }
 
-const char *get_gnuplot_charset (void)
+static const char *gnuplot_encoding_string (void)
 {
     static char gp_enc[12];
 
@@ -239,63 +247,129 @@ const char *get_gnuplot_charset (void)
 	(gretl_cset_min == 1 || 
 	 gretl_cset_min == 2 ||
 	 gretl_cset_min == 15)) {
-	sprintf(gp_enc, "iso_%d_%d\n", gretl_cset_maj, gretl_cset_min);
+	sprintf(gp_enc, "iso_%d_%d", gretl_cset_maj, gretl_cset_min);
 	return gp_enc;
     } 
 
     return NULL;
 }
 
-int use_latin_2 (void)
+void pprint_gnuplot_encoding (const char *termstr, PRN *prn)
 {
-    int l2 = gretl_cset_maj == 8859 && gretl_cset_min == 2;
+    if (strstr(termstr, "postscript")) {
+	const char *enc = gnuplot_encoding_string();
+
+	if (enc != NULL) {
+	    pprintf(prn, "set encoding %s\n", enc);
+	}
+    }
+}
+
+void fprint_gnuplot_encoding (const char *termstr, FILE *fp)
+{
+    if (strstr(termstr, "postscript")) {
+	const char *enc = gnuplot_encoding_string();
+
+	if (enc != NULL) {
+	    fprintf(fp, "set encoding %s\n", enc);
+	}
+    }
+}
+
+int iso_latin_version (void)
+{
+    char *lang = NULL;
+
+    if (gretl_cset_maj == 8859 &&
+	(gretl_cset_min == 1 || 
+	 gretl_cset_min == 2 ||
+	 gretl_cset_min == 15)) {
+	return gretl_cset_min;
+    }
 
 # ifdef WIN32
-    if (!l2) {
-	l2 = gretl_cpage == 1250;
+    if (gretl_cpage == 1252) {
+	return 1;
+    } else if (gretl_cpage == 1250) {
+	return 2;
     }
 # endif
 
-    return l2;
+    /* handle Polish UTF-8 locale? */
+    lang = getenv("LANG");
+    if (lang != NULL && !strncmp(lang, "pl", 2)) {
+	return 2;
+    } 
+
+    return 1;
 }
 
+static const char *get_gp_charset (void)
+{
+    if (iso_latin_version() == 2) {
+#ifdef WIN32
+	return "CP1250";
+#else
+	return "ISO-8859-2";
+#endif
+    } else {
+	return "ISO-8859-1";
+    }
+}
+
+char *gp_gettext (const char *msgid)
+{
+    static const char *cset;
+    char *ret;
+
+    if (cset == NULL) {
+	cset = get_gp_charset();
+    }
+
+    bind_textdomain_codeset(PACKAGE, cset);
+    ret = gettext(msgid);
+    bind_textdomain_codeset(PACKAGE, "UTF-8");
+
+    return ret;
+} 
+
 char *iso_gettext (const char *msgid)
 {
-   char *ret;
-   static int cli;
-   static int iso_ok = -1;
-   static const char *cset;
+    static int iso_ok = -1;
+    static const char *cset;
+    static int cli;
+    char *ret;
 
-   /* the command-line program is "special": it doesn't emit
-      utf-8 at all, so we omit the redundant switching of
-      codesets */
-   if (!strcmp(msgid, "@CLI_INIT")) {
-       cli = 1;
-       return NULL;
-   }
+    /* command line program: switching of codesets is not required */
+    if (!strcmp(msgid, "@CLI_INIT")) {
+	cli = 1;
+	return NULL;
+    }
 
-   if (cli) { /* command line program: switch not required */
-       return gettext(msgid);
-   }
+    if (cli) { 
+	return gettext(msgid);
+    }
 
-   if (iso_ok < 0) {
-       cset = get_gretl_charset();
-       if (cset == NULL) {
-	   iso_ok = 0;
-       } else {
-	   iso_ok = 1;
-       }
-   }
+    if (iso_ok < 0) {
+	/* not yet determined */
+	cset = get_gretl_charset();
+	fprintf(stderr, "get_gretl_charset gave %s\n", cset);
+	if (cset == NULL) {
+	    iso_ok = 0;
+	} else {
+	    iso_ok = 1;
+	}
+    }
 
-   if (iso_ok) {
-       bind_textdomain_codeset(PACKAGE, cset);
-       ret = gettext(msgid);
-       bind_textdomain_codeset(PACKAGE, "UTF-8");
-   } else {
-       ret = gettext(msgid);
-   }
+    if (iso_ok) {
+	bind_textdomain_codeset(PACKAGE, cset);
+	ret = gettext(msgid);
+	bind_textdomain_codeset(PACKAGE, "UTF-8");
+    } else {
+	ret = gettext(msgid);
+    }
 
-   return ret;
+    return ret;
 } 
 
 #endif  /* ENABLE_NLS */
@@ -547,13 +621,13 @@ char *iso_to_ascii (char *s)
 #ifdef ENABLE_NLS
 
 struct l2sym {
-    int l2val;   /* iso-8859-2 (or CP1250) character code */
+    int l2val;   /* ISO-8859-2 (or CP1250) character code */
     int ucs2val; /* corresponding UCS-2 code */
 };
 
 # ifndef WIN32
 
-/* iso-8859-2 */
+/* ISO-8859-2 */
 
 static struct l2sym l2table[] = { 
     { 161, 260 }, /*  A; */
@@ -615,11 +689,37 @@ static struct l2sym l2table[] = {
     { 255, 729 }  /*  '. */
 };
 
-# else
+static int l2_to_ucs (int c)
+{
+    int i, n = sizeof l2table / sizeof l2table[0];
+
+    for (i=0; i<n; i++) {
+	if (c == l2table[i].l2val) {
+	    return l2table[i].ucs2val;
+	}
+    }
+
+    return c;
+}
+
+static int ucs_to_l2 (int c)
+{
+    int i, n = sizeof l2table / sizeof l2table[0];
+
+    for (i=0; i<n; i++) {
+	if (c == l2table[i].ucs2val) {
+	    return l2table[i].l2val;
+	}
+    }
+
+    return c;
+}
+
+# endif
 
 /* Windows codepage 1250 */
 
-static struct l2sym l2table[] = { 
+static struct l2sym cptable[] = { 
     { 128, 8364 }, /*  Eu */
     { 130, 8218 }, /*  .9 */
     { 132, 8222 }, /*  :9 */
@@ -696,28 +796,30 @@ static struct l2sym l2table[] = {
     { 255,  729 }  /*  '. */
 };
 
-# endif
+# ifdef WIN32
 
-static int l2_lookup (int c)
+static int cp_to_ucs (int c)
 {
-    int i, n = sizeof l2table / sizeof l2table[0];
+    int i, n = sizeof cptable / sizeof cptable[0];
 
     for (i=0; i<n; i++) {
-	if (c == l2table[i].l2val) {
-	    return l2table[i].ucs2val;
+	if (c == cptable[i].l2val) {
+	    return cptable[i].ucs2val;
 	}
     }
 
     return c;
 }
 
-static int ucs_lookup (int c)
+# endif
+
+static int ucs_to_cp (int c)
 {
-    int i, n = sizeof l2table / sizeof l2table[0];
+    int i, n = sizeof cptable / sizeof cptable[0];
 
     for (i=0; i<n; i++) {
-	if (c == l2table[i].ucs2val) {
-	    return l2table[i].l2val;
+	if (c == cptable[i].ucs2val) {
+	    return cptable[i].l2val;
 	}
     }
 
@@ -734,7 +836,7 @@ char *sprint_l2_to_html (char *targ, const char *s, size_t len)
     while ((c = *s)) {
 # ifndef WIN32
 	if (c > 160) {
-	    sprintf(p, "&#%d;", l2_lookup(c));
+	    sprintf(p, "&#%d;", l2_to_ucs(c));
 	    p = strchr(p, ';') + 1;
 	} else if (c > 127) {
 	    sprintf(p, "&#%d;", c);
@@ -744,7 +846,7 @@ char *sprint_l2_to_html (char *targ, const char *s, size_t len)
 	}
 # else
 	if (c > 127) {
-	    sprintf(p, "&#%d;", l2_lookup(c));
+	    sprintf(p, "&#%d;", cp_to_ucs(c));
 	    p = strchr(p, ';') + 1;
 	} else {
 	    *p++ = c;
@@ -770,7 +872,11 @@ char *sprint_html_to_l2 (char *targ, const char *s)
 
     while (*s) {
 	if (sscanf(s, "&#%d;", &u)) {
-	    *p++ = ucs_lookup(u);
+# ifdef WIN32
+	    *p++ = ucs_to_cp(u);
+# else
+	    *p++ = ucs_to_l2(u);
+# endif
 	    s = strchr(s, ';') + 1;
 	} else {
 	    *p++ = *s++;
@@ -796,7 +902,11 @@ int print_as_html (const char *s, FILE *fp)
 
     while ((c = *s)) {
         if (c > 160) {
-            fprintf(fp, "&#%d;", l2_lookup(c));
+# ifdef WIN32
+	    fprintf(fp, "&#%d;", cp_to_ucs(c));
+# else
+            fprintf(fp, "&#%d;", l2_to_ucs(c));
+# endif
         } else if (c > 127) {
             fprintf(fp, "&#%d;", c);
         } else {
@@ -815,7 +925,11 @@ int print_as_locale (const char *s, FILE *fp)
 
     while (*s) {
 	if (sscanf(s, "&#%d;", &u)) {
-	    fputc(ucs_lookup(u), fp);
+# ifdef WIN32
+	    fputc(ucs_to_cp(u), fp);
+# else
+	    fputc(ucs_to_l2(u), fp);
+# endif
 	    s = strchr(s, ';') + 1;
 	} else {
 	    fputc(*s++, fp);
@@ -826,6 +940,68 @@ int print_as_locale (const char *s, FILE *fp)
     return nwrote;
 }
 
+int pprint_as_latin (PRN *prn, const char *s, int emf)
+{
+    int u, nwrote = 0;
+
+    while (*s) {
+	if (sscanf(s, "&#%d;", &u)) {
+# ifdef WIN32
+	    pputc(prn, ucs_to_cp(u));
+# else
+	    if (emf) {
+		pputc(prn, ucs_to_cp(u));
+	    } else {
+		pputc(prn, ucs_to_l2(u));
+	    }
+# endif
+	    s = strchr(s, ';') + 1;
+	} else {
+	    pputc(prn, *s++);
+	}
+	nwrote++;
+    }
+
+    return nwrote;
+}
+
+char *utf8_to_latin (const char *s)
+{
+    gsize read, wrote;
+
+    if (iso_latin_version() == 2) {
+# ifdef WIN32
+	return g_convert(s, -1, "CP1250", "UTF-8",
+			 &read, &wrote, NULL);
+# else
+	return g_convert(s, -1, "ISO-8859-2", "UTF-8",
+			 &read, &wrote, NULL);
+# endif
+    } else {
+	return g_convert(s, -1, "ISO-8859-1", "UTF-8",
+			 &read, &wrote, NULL);
+    }
+}
+
+/* allow TAB, CR, LF, FF */
+
+#define ascii_ctrl(a) (a==9 || a==10 || a==12 || a==13)
+
+int gretl_is_ascii (const char *buf)
+{
+    int a;
+
+    while (*buf) {
+	a = *buf;
+	if ((a < 32 && !(ascii_ctrl(a))) || a > 126) {
+	    return 0;
+	}
+	buf++;
+    }
+
+    return 1;
+}
+
 char *get_month_name (char *mname, int m)
 {
     struct tm mt;
@@ -847,9 +1023,7 @@ char *get_month_name (char *mname, int m)
 
 int get_utf_width (const char *str, int width)
 {
-# ifdef USE_GLIB2
     width += strlen(str) - g_utf8_strlen(str, -1);
-# endif
 
     return width;
 }
@@ -858,9 +1032,7 @@ int get_translated_width (const char *str)
 {
     int w = strlen(str);
 
-# ifdef USE_GLIB2
     w += w - g_utf8_strlen(str, -1);
-# endif
 
     return w;
 }
