@@ -43,11 +43,7 @@ static saved_list *saved_list_new (const int *list, const char *name)
     saved_list *sl = malloc(sizeof *sl);
 
     if (sl != NULL) {
-	if (gretl_executing_function()) {
-	    sl->level = gretl_function_stack_depth();
-	} else {
-	    sl->level = 0;
-	}
+	sl->level = gretl_function_depth();
 	if (list != NULL && list[0] > 0) {
 	    sl->list = gretl_list_copy(list);
 	} else {
@@ -75,13 +71,9 @@ static void free_saved_list (saved_list *sl)
 
 static saved_list *get_saved_list_by_name (const char *name)
 {
-    int fsd = 0;
+    int fsd = gretl_function_depth();
     int i;
 
-    if (gretl_executing_function()) {
-	fsd = gretl_function_stack_depth();
-    }
-
     for (i=0; i<n_lists; i++) {
 	if (!strcmp(name, list_stack[i]->name) && 
 	    fsd == list_stack[i]->level) {
@@ -103,6 +95,30 @@ int n_saved_lists (void)
     return n_lists;
 }
 
+/**
+ * max_varno_in_saved_lists:
+ *
+ * Returns: the highest ID number of a variable referenced
+ * in a saved (named) list.
+ */
+
+int max_varno_in_saved_lists (void)
+{
+    int *list;
+    int i, j, vmax = 0;
+
+    for (i=0; i<n_lists; i++) {
+	list = list_stack[i]->list;
+	for (j=1; j<=list[0]; j++) {
+	    if (list[j] > vmax) {
+		vmax = list[j];
+	    }
+	}
+    }    
+
+    return vmax;
+}
+
 /**
  * get_list_name_by_index:
  * @idx: 0-based index into array of saved lists.
@@ -162,14 +178,18 @@ static int real_remember_list (const int *list, const char *name,
 	free(orig->list);
 	orig->list = gretl_list_copy(list);
 	if (orig->list == NULL) {
-	    pprintf(prn, "Out of memory replacing list '%s'\n", name);
 	    err = E_ALLOC;
 	} else if (gretl_messages_on()) {
-	    pprintf(prn, "Replaced list '%s'\n", name);
+	    pprintf(prn, _("Replaced list '%s'\n"), name);
 	}
     } else {
 	saved_list **lstack;
 
+	err = check_varname(name);
+	if (err) {
+	    return err;
+	}
+
 	lstack = realloc(list_stack, (n_lists + 1) * sizeof *lstack);
 	if (lstack == NULL) {
 	    return E_ALLOC;
@@ -178,15 +198,14 @@ static int real_remember_list (const int *list, const char *name,
 
 	list_stack[n_lists] = saved_list_new(list, name);
 	if (list_stack[n_lists] == NULL) {
-	    pprintf(prn, "Out of memory adding list '%s'\n", name);
 	    err = E_ALLOC;
 	} else {
 	    if (gretl_messages_on()) {
 		const char *realname = list_stack[n_lists]->name;
 
-		pprintf(prn, "Added list '%s'\n", realname);
+		pprintf(prn, _("Added list '%s'\n"), realname);
 		if (strlen(realname) < strlen(name)) {
-		    pprintf(prn, "Warning: the name was truncated to %d characters\n", 
+		    pprintf(prn, _("Warning: the name was truncated to %d characters\n"), 
 			    LNAMELEN - 1);
 		}
 	    }
@@ -215,7 +234,7 @@ static int real_remember_list (const int *list, const char *name,
  * name in which case the original list is replaced.  A status
  * message is printed to @prn.
  *
- * Returns: 0 on success, %E_ALLOC on error.
+ * Returns: 0 on success, non-zero code on error.
  */
 
 int remember_list (const int *list, const char *name, PRN *prn)
@@ -224,28 +243,27 @@ int remember_list (const int *list, const char *name, PRN *prn)
 }
 
 /**
- * stack_localized_list_as:
- * @list: the list to be saved.
- * @name: the name to be given.
+ * rename_saved_list:
+ * @orig: the original name of the list.
+ * @new: the new name to be given.
  *
- * For use in user-defined functions: take a list of
- * variables that has been "localized" to function-scope
- * and save it under the given @name.
+ * Overwrites the name of a saved list.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
-int stack_localized_list_as (int *list, const char *name)
+int rename_saved_list (const char *orig, const char *new)
 {
     saved_list *sl;
-    int err;
+    int err = 0;
 
-    err = real_remember_list(list, name, 1, NULL);
-
-    if (!err) {
-	sl = list_stack[n_lists - 1];
-	sl->level += 1;
-    }
+    sl = get_saved_list_by_name(orig);
+    if (sl == NULL) {
+	err = 1;
+    } else {
+	*sl->name = '\0';
+	strncat(sl->name, new, LNAMELEN - 1);
+    } 
 
     return err;
 }
@@ -275,7 +293,6 @@ int copy_named_list_as (const char *orig, const char *new)
     } else {
 	err = real_remember_list(sl->list, new, 1, NULL);
 	if (!err) {
-	    /* for use in functions */
 	    sl = list_stack[n_lists - 1];
 	    sl->level += 1;
 	}
@@ -284,6 +301,68 @@ int copy_named_list_as (const char *orig, const char *new)
     return err;
 }
 
+/**
+ * named_list_lower_level:
+ * @name: the name of the list.
+ *
+ * If a saved list is found by the name @name, at the
+ * current level of function execution, lower its level
+ * by 1.  This is intended for use when a list is 
+ * returned by a user-defined function: it is shifted to
+ * the level of the caller.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */ 
+
+int named_list_lower_level (const char *name)
+{
+    saved_list *sl;
+    int err = 0;
+
+    sl = get_saved_list_by_name(name);
+    if (sl == NULL) {
+	err = E_DATA;
+    } else {
+	sl->level -= 1;
+    }
+
+    return err;
+}
+
+/**
+ * create_named_null_list:
+ * @name: the name to be given to the list.
+ *
+ * Creates an empty list under the given @name and adds it
+ * to the stack of saved lists at the next level of
+ * function execution.  This is intended for use when a null 
+ * list is given as an argument to a user-defined function.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int create_named_null_list (const char *name)
+{
+    saved_list *sl;
+    int *list;
+    int err;
+
+    list = gretl_null_list();
+    if (list == NULL) {
+	return E_ALLOC;
+    }
+
+    err = real_remember_list(list, name, 1, NULL);
+    if (!err) {
+	sl = list_stack[n_lists - 1];
+	sl->level += 1;
+    } else {
+	free(list);
+    }
+
+    return err;
+}
+
 /**
  * destroy_saved_lists_at_level:
  * @level: stack level of function execution.
@@ -312,6 +391,7 @@ int destroy_saved_lists_at_level (int level)
 		list_stack[j] = list_stack[j+1];
 	    }
 	    list_stack[n_lists - 1] = NULL;
+	    i--;
 	} else {
 	    nl++;
 	}
@@ -335,6 +415,100 @@ int destroy_saved_lists_at_level (int level)
     return err;
 }
 
+static int var_is_deleted (const int *dlist, int dmin, int i)
+{
+    int v = dmin + i - 1;
+
+    if (dlist != NULL) {
+	return in_gretl_list(dlist, v);
+    } else {
+	return (v >= dmin);
+    }
+}
+
+/**
+ * gretl_lists_revise:
+ * @dlist: list of variables to be deleted (or %NULL).
+ * @dmin: lowest ID number of deleted var (referenced only
+ * if @dlist is %NULL).
+ *
+ * Goes through any saved lists, adjusting the ID numbers
+ * they contain to reflect the deletion from the dataset of
+ * certain variables: those referenced in @dlist, if given, 
+ * or if @dlist is %NULL, those variables with IDs greater 
+ * than or equal to @dmin.
+ *
+ * Returns: 0 on success, non-zero code on failure.
+ */
+
+int gretl_lists_revise (const int *dlist, int dmin)
+{
+    int *list, *maplist;
+    int lmax = 0;
+    int i, j, k;
+
+    if (dlist != NULL) {
+	/* determine lowest deleted ID */
+	dmin = dlist[1];
+	for (i=2; i<=dlist[0]; i++) {
+	    if (dlist[i] > 0 && dlist[i] < dmin) {
+		dmin = dlist[i];
+	    }
+	}
+    }
+
+    /* find highest ID ref'd in any saved list */
+    for (j=0; j<n_lists; j++) {
+	list = list_stack[j]->list;
+	for (i=1; i<=list[0]; i++) {
+	    if (list[i] > lmax) {
+		lmax = list[i];
+	    }
+	}
+    }
+
+    if (lmax < dmin) {
+	/* nothing to be done */
+	return 0;
+    }
+
+    /* make mapping from old to new IDs */
+
+    maplist = gretl_list_new(lmax - dmin + 1);
+    if (maplist == NULL) {
+	return E_ALLOC;
+    }
+
+    j = dmin;
+
+    for (i=1; i<=maplist[0]; i++) {
+	if (var_is_deleted(dlist, dmin, i)) {
+	    maplist[i] = -1;
+	} else {
+	    maplist[i] = j++;
+	}
+    }
+
+    /* use mapping to revise saved lists */
+    for (j=0; j<n_lists; j++) {
+	list = list_stack[j]->list;
+	for (i=1; i<=list[0]; i++) {
+	    k = list[i] - dmin + 1;
+	    if (k >= 1) {
+		if (maplist[k] == -1) {
+		    gretl_list_delete_at_pos(list, i--);
+		} else {
+		    list[i] = maplist[k];
+		}
+	    }
+	}
+    }
+
+    free(maplist);
+
+    return 0;
+}
+
 /**
  * gretl_lists_cleanup:
  *
@@ -654,7 +828,6 @@ int *gretl_list_from_string (const char *liststr)
 		list[n++] = atoi(numstr);
 	    }
 	    s += strlen(numstr);
-
 	}
     }    
 
@@ -1091,14 +1264,16 @@ int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
 
 int gretl_list_diff (int *targ, const int *biglist, const int *sublist)
 {
-    int i, j, k = 0;
+    int i, j, k, n;
     int match, err = 0;
 
-    targ[0] = biglist[0] - sublist[0];
+    n = biglist[0] - sublist[0];
+    targ[0] = n;
 
-    if (targ[0] <= 0) {
+    if (n <= 0) {
 	err = 1;
     } else {
+	k = 1;
 	for (i=2; i<=biglist[0]; i++) {
 	    match = 0;
 	    for (j=2; j<=sublist[0]; j++) {
@@ -1108,7 +1283,11 @@ int gretl_list_diff (int *targ, const int *biglist, const int *sublist)
 		}
 	    }
 	    if (!match) {
-		targ[++k] = biglist[i];
+		if (k <= n) {
+		    targ[k++] = biglist[i];
+		} else {
+		    err = 1;
+		}
 	    }
 	}
     }
@@ -1124,39 +1303,48 @@ int gretl_list_diff (int *targ, const int *biglist, const int *sublist)
  *
  * Returns: a newly allocated list including the elements of @biglist,
  * from position @minpos onwards, that are not present in @sublist, 
- * or %NULL on failure.  It is an error if, from position @minpos on, 
- * @sublist is not a proper subset of @biglist.
+ * again from @minpos onwards, or %NULL on failure.  
  */
 
 int *gretl_list_diff_new (const int *biglist, const int *sublist,
 			  int minpos)
 {
     int *targ = NULL;
-    int i, j, n, k = 0;
+    int i, j, bi;
     int match;
 
     if (biglist == NULL || sublist == NULL) {
 	return NULL;
     }
 
-    n = biglist[0] - sublist[0];
-
-    if (n <= 0) {
+    targ = gretl_null_list();
+    if (targ == NULL) {
 	return NULL;
     }
 
-    targ = gretl_list_new(n);
-
     for (i=minpos; i<=biglist[0]; i++) {
+	bi = biglist[i];
 	match = 0;
 	for (j=minpos; j<=sublist[0]; j++) {
-	    if (sublist[j] == biglist[i]) {
+	    if (sublist[j] == bi) {
 		match = 1;
 		break;
 	    }
 	}
 	if (!match) {
-	    targ[++k] = biglist[i];
+	    /* but is this var already accounted for? */
+	    for (j=1; j<=targ[0]; j++) {
+		if (targ[j] == bi) {
+		    match = 1;
+		    break;
+		}
+	    }
+	}
+	if (!match) {
+	    targ = gretl_list_append_term(&targ, biglist[i]);
+	    if (targ == NULL) {
+		break;
+	    }
 	}
     }
 
@@ -1460,6 +1648,21 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
     return 0;
 }
 
+static int real_list_dup (const int *list, int start, int stop)
+{
+    int i, j, ret = -1;
+
+    for (i=start; i<stop && ret<0; i++) {
+	for (j=i+1; j<=stop && ret<0; j++) {
+	    if (list[i] == list[j]) {
+		ret = list[i];
+	    }
+	}
+    }
+
+    return ret;
+}
+
 /**
  * gretl_list_duplicates:
  * @list: an array of integer variable ID numbers, the first element
@@ -1476,8 +1679,9 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
 
 int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
 {
-    int i, j, start = 2;
-    int ret = -1;
+    int multi = 0;
+    int start = 2;
+    int i, ret = -1;
 
     if (ci == ARCH) {
 	start = 3;
@@ -1490,22 +1694,55 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
 	}
     } else if (ci == LAGS && list[0] > 1 && list[2] == LISTSEP) {
 	start = 3;
-    } else if (ci == TSLS || ci == AR || ci == SCATTERS || 
-	       ci == MPOLS || ci == GARCH) {
+    } else if (ci == AR || ci == SCATTERS || ci == MPOLS || ci == GARCH) {
 	for (i=2; i<list[0]; i++) {
 	    if (list[i] == LISTSEP) {
 		start = i+1;
 		break;
 	    }
 	}
-    }
-    
-    for (i=start; i<list[0] && ret < 0; i++) {
-	for (j=i+1; j<=list[0] && ret < 0; j++) {
-	    if (list[i] == list[j]) {
-		ret = list[i];
+    } else if (ci == TSLS) {
+	multi = 1;
+	for (i=2; i<list[0]; i++) {
+	    if (list[i] == LISTSEP) {
+		start = i+1;
+		break;
 	    }
 	}
+	ret = real_list_dup(list, start, list[0]);
+	if (ret == -1) {
+	    ret = real_list_dup(list, 2, start - 2);
+	}
+    } else if (ci == ARBOND) {
+	int stop = 0;
+
+	multi = 1;
+	for (i=2; i<list[0]; i++) {
+	    if (list[i] == LISTSEP) {
+		start = i;
+		break;
+	    }
+	}
+	for (i=list[0]-1; i>=2; i--) {
+	    if (list[i] == LISTSEP) {
+		stop = i;
+		break;
+	    }
+	}
+
+	if (stop == start) {
+	    ret = real_list_dup(list, start + 1, list[0]);
+	} else {
+	    ret = real_list_dup(list, start + 1, stop - 1);
+	    if (ret == -1) {
+		ret = real_list_dup(list, stop + 1, list[0]);
+	    }
+	}
+	multi = 1;
+    } 
+
+    if (!multi) {
+	ret = real_list_dup(list, start, list[0]);
     }
 
     return ret;
@@ -1517,7 +1754,8 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
  * @nvars: location for return of number of elements in full list.
  *
  * Creates a newly allocated list including all variables in the
- * dataset that are not scalars and are not hidden variables.
+ * dataset that are not scalars, are not hidden variables, and
+ * are accessible at the current level of function execution.
  * The return value is %NULL in case either (a) allocation of
  * memory failed, or (b) the resulting list would be empty.
  * The caller can distinguish between these possibilities by
@@ -1530,11 +1768,13 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
 
 int *full_var_list (const DATAINFO *pdinfo, int *nvars)
 {
+    int fsd = gretl_function_depth();
     int i, j, nv = 0;
     int *list = NULL;
 
     for (i=1; i<pdinfo->v; i++) {
-	if (var_is_series(pdinfo, i) && !var_is_hidden(pdinfo, i)) {
+	if (var_is_series(pdinfo, i) && !var_is_hidden(pdinfo, i) &&
+	    STACK_LEVEL(pdinfo, i) == fsd) {
 	    nv++;
 	}
     }
@@ -1550,7 +1790,8 @@ int *full_var_list (const DATAINFO *pdinfo, int *nvars)
     if (list != NULL) {
 	j = 1;
 	for (i=1; i<pdinfo->v; i++) {
-	    if (var_is_series(pdinfo, i) && !var_is_hidden(pdinfo, i)) {
+	    if (var_is_series(pdinfo, i) && !var_is_hidden(pdinfo, i) &&
+		STACK_LEVEL(pdinfo, i) == fsd) {
 		list[j++] = i;
 	    }
 	}
@@ -1613,6 +1854,81 @@ int gretl_list_is_consecutive (const int *list)
     return ret;
 }
 
+/**
+ * gretl_list_build:
+ * @s: string list specification.
+ * @pdinfo: dataset information.
+ * @err: location to receive error code
+ *
+ * Builds a list based on the specification in @s, which may include
+ * the ID numbers of variables, the names of variables, and/or the
+ * names of previously defined lists (all separated by spaces).
+ *
+ * Returns: the constructed list, or %NULL on failure.
+ */
+
+int *gretl_list_build (const char *s, const DATAINFO *pdinfo, int *err)
+{
+    char test[32];
+    int *list = NULL;
+    int *nlist;
+    int i, v, len, nf;
+
+    list = gretl_null_list();
+    if (list == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    nf = count_fields(s);
+    
+    for (i=0; i<nf && !*err; i++) {
+	s += strspn(s, " ");
+	len = strcspn(s, " ");
+	if (len > 31) {
+	    *err = E_PARSE;
+	} else {
+	    *test = 0;
+	    strncat(test, s, len);
+
+	    /* valid elements: integers, varnames, named lists */
+
+	    if (isdigit(*test)) {
+		v = positive_int_from_string(test);
+		if (v >= 0) {
+		    list = gretl_list_append_term(&list, v);
+		} else {
+		    *err = E_PARSE;
+		}
+	    } else {
+		v = varindex(pdinfo, test);
+		if (v < pdinfo->v) {
+		    list = gretl_list_append_term(&list, v);
+		} else {
+		    nlist = get_list_by_name(test);
+		    if (nlist != NULL) {
+			*err = gretl_list_add_list(&list, nlist);
+		    } else {
+			*err = E_UNKVAR;
+		    }
+		}
+	    }
+
+	    if (list == NULL) {
+		*err = E_ALLOC;
+	    }
+	}
+	s += len;
+    }
+
+    if (*err) {
+	free(list);
+	list = NULL;
+    }
+
+    return list;
+}
+
 int load_user_lists_file (const char *fname)
 {
     xmlDocPtr doc = NULL;
