@@ -57,6 +57,8 @@ int gretl_list_diff (int *targ, const int *biglist, const int *sublist);
 
 int *gretl_list_diff_new (const int *biglist, const int *sublist, int minpos);
 
+int *gretl_list_build (const char *s, const DATAINFO *pdinfo, int *err);
+
 int gretl_list_add_list (int **targ, const int *src);
 
 int gretl_list_insert_list (int **targ, const int *src, int pos);
@@ -88,18 +90,26 @@ int *full_var_list (const DATAINFO *pdinfo, int *nvars);
 
 int n_saved_lists (void);
 
+int max_varno_in_saved_lists (void);
+
 const char *get_list_name_by_index (int idx);
 
 int *get_list_by_name (const char *name);
 
 int remember_list (const int *list, const char *name, PRN *prn);
 
-int stack_localized_list_as (int *list, const char *name);
-
 int copy_named_list_as (const char *orig, const char *new);
 
+int named_list_lower_level (const char *name);
+
+int rename_saved_list (const char *orig, const char *new); 
+
+int create_named_null_list (const char *name);
+
 int destroy_saved_lists_at_level (int level);
 
+int gretl_lists_revise (const int *dlist, int dmin);
+
 void gretl_lists_cleanup (void);
 
 int load_user_lists_file (const char *fname);
