@@ -22,44 +22,6 @@
 
 #include "objstack.h"
 
-typedef enum {
-    M_ESS = 1,    /* error sum of squares */
-    M_T,          /* observations used */
-    M_RSQ,        /* R-squared */
-    M_SIGMA,      /* standard error of residuals */
-    M_DF,         /* degrees of freedom */
-    M_NCOEFF,     /* total number of estimated coefficients */
-    M_LNL,        /* log-likelihood */
-    M_AIC,        /* Akaike info criterion */
-    M_BIC,        /* Bayesian info criterion */
-    M_HQC,        /* Hannan-Quinn criterion */
-    M_TRSQ,       /* T * R-squared, last model */
-    M_SCALAR_MAX, /* -- separator -- */
-    M_COEFF_S,    /* single coefficient */
-    M_SE_S,       /* single standard error */
-    M_VCV_S,      /* single covariance matrix entry */
-    M_RHO_S,      /* single rho array entry */
-    M_ELEM_MAX,   /* -- separator -- */
-    M_UHAT,       /* residuals */
-    M_YHAT,       /* fitted values */
-    M_AHAT,       /* per-unit intercepts in panel model */
-    M_H,          /* GARCH predicted variances */
-    M_SERIES_MAX, /* -- separator -- */
-    M_COEFF,      /* parameter estimates */
-    M_SE,         /* parameter standard errors */
-    M_VCV,        /* parameter covariance matrix */
-    M_RHO,        /* autoregressive coefficients */
-    M_JALPHA,     /* Johansen's alpha */
-    M_JBETA,      /* Johansen's beta */
-    M_JVBETA,     /* Covariance matrix for Johansen's normalised beta */
-    M_MAX         /* sentinel */
-} ModelDataIndex;
-
-#define model_data_is_scalar(i) (i > 0 && i < M_SCALAR_MAX)
-#define model_data_is_scalar_element(i) (i > M_SCALAR_MAX && i < M_ELEM_MAX)
-#define model_data_is_series(i) (i > M_ELEM_MAX && i < M_SERIES_MAX)
-#define model_data_is_matrix(i) (i > M_ELEM_MAX && i != M_SERIES_MAX)
-
 typedef enum {
     MODEL_DATA_NONE,
     MODEL_DATA_INT,
@@ -86,7 +48,9 @@ typedef enum {
 typedef struct CoeffIntervals_ CoeffIntervals;
 
 struct CoeffIntervals_ {
+    int asy;
     int ncoeff;
+    double t;
     char **names;
     double *coeff;
     double *maxerr;
@@ -130,8 +94,10 @@ struct CoeffIntervals_ {
                       c == PWE || \
                       c == WLS)
 
-#define ASYMPTOTIC_MODEL(c) (c == ARMA || \
+#define ASYMPTOTIC_MODEL(c) (c == ARBOND || \
+                             c == ARMA || \
                              c == GARCH || \
+                             c == GMM || \
                              c == LOGIT || \
                              c == MLE || \
                              c == POISSON || \
@@ -171,6 +137,7 @@ typedef enum {
     GRETL_TEST_PANEL_HAUSMAN,
     GRETL_TEST_PANEL_F,
     GRETL_TEST_PANEL_BP,
+    GRETL_TEST_PANEL_TIMEDUM,
     GRETL_TEST_MAX
 } ModelTestType;
 
@@ -180,16 +147,18 @@ int attach_model_tests_from_xml (MODEL *pmod, xmlNodePtr node);
 
 MODEL *gretl_model_new (void);
 
-MODEL *gretl_model_new_protected (void);
-
 void gretl_model_init (MODEL *pmod);
 
 int gretl_model_allocate_storage (MODEL *pmod);
 
 MODEL **gretl_model_array_new (int n);
 
+MODEL **allocate_working_models (int n);
+
 void gretl_model_array_destroy (MODEL **models, int n);
 
+void destroy_working_models (MODEL **models, int n);
+
 void gretl_model_smpl_init (MODEL *pmod, const DATAINFO *pdinfo);
 
 void impose_model_smpl (const MODEL *pmod, DATAINFO *pdinfo);
@@ -238,7 +207,7 @@ char *gretl_model_get_param_name (const MODEL *pmod,
 
 int gretl_model_get_param_number (const MODEL *pmod, 
 				  const DATAINFO *pdinfo,
-				  char *pname);
+				  const char *s);
 
 void free_coeff_intervals (CoeffIntervals *cf);
 
@@ -286,7 +255,7 @@ int is_model_cmd (const char *line);
 
 int is_quiet_model_test (int ci, gretlopt opt);
 
-int command_ok_for_model (int test_ci, int model_ci);
+int command_ok_for_model (int test_ci, gretlopt opt, int model_ci);
 
 int model_test_ok (int ci, gretlopt opt, const MODEL *pmod, 
 		   const DATAINFO *pdinfo);
@@ -326,7 +295,8 @@ void model_test_set_crit_and_alpha (ModelTest *test,
 
 void gretl_model_test_print (const MODEL *pmod, int i, PRN *prn);
 void gretl_model_print_last_test (const MODEL *pmod, PRN *prn);
-void gretl_model_test_print_direct (const ModelTest *test, PRN *prn);
+void gretl_model_test_print_direct (const ModelTest *test, int heading,
+				    PRN *prn);
 
 void gretl_model_destroy_tests (MODEL *pmod);
 
@@ -337,7 +307,9 @@ int highest_numbered_var_in_model (const MODEL *pmod,
 
 int mle_criteria (MODEL *pmod, int addk);
 
-double coeff_pval (const MODEL *pmod, double x, int df);
+double coeff_pval (int ci, double x, int df);
+
+int gretl_model_allocate_params (MODEL *pmod, int k);
 
 int gretl_model_add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
 				   int yno, int p, int q, int P, int Q, 
@@ -356,8 +328,6 @@ void gretl_model_set_name (MODEL *pmod, const char *name);
 
 const char *gretl_model_get_name (const MODEL *pmod);
 
-int gretl_model_data_index (const char *s);
-
 double gretl_model_get_scalar (const MODEL *pmod, ModelDataIndex idx, 
 			       int *err);
 
@@ -368,6 +338,10 @@ gretl_model_get_series (const MODEL *pmod, const DATAINFO *pdinfo,
 gretl_matrix *gretl_model_get_matrix (MODEL *pmod, ModelDataIndex idx, 
 				      int *err);
 
+double 
+gretl_model_get_data_element (MODEL *pmod, int idx, const char *s,
+			      const DATAINFO *pdinfo, int *err);
+
 int gretl_model_serialize (const MODEL *pmod, SavedObjectFlags flags,
 			   FILE *fp);
 
