@@ -21,21 +21,22 @@
 
 #include "libgretl.h"
 #include "libset.h"
+#include "gretl_string_table.h"
 
 #include <unistd.h>
 
 #ifdef WIN32
 # include <windows.h>
 #else
+# include <sys/stat.h>
 # include <dirent.h>
+# include <errno.h>
 #endif
 
-#ifdef USE_GLIB2
-# include <glib.h>
-# if (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 6)
-#  ifdef WIN32
-#   define USE_G_FOPEN
-#  endif
+#include <glib.h>
+#if (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 6)
+# ifdef WIN32
+#  define USE_G_FOPEN
 # endif
 #endif
 
@@ -108,6 +109,45 @@ gzFile gretl_gzopen (const char *filename, const char *mode)
     return fz;
 }
 
+#ifdef WIN32
+
+int gretl_mkdir (const char *path)
+{
+    DIR *test;
+    int done;
+
+    test = win32_opendir(path);
+    if (test != NULL) {
+	closedir(test);
+	return 0;
+    }
+
+    done = CreateDirectory(path, NULL);
+    
+    return !done;
+}
+
+#else
+
+int gretl_mkdir (const char *path)
+{
+    int err = 0;
+    extern int errno;
+
+    errno = 0;
+
+    if (mkdir(path, 0755)) {
+	if (errno != EEXIST) { 
+	    fprintf(stderr, "%s: %s\n", path, strerror(errno));
+	    err = 1;
+	}
+    }
+
+    return err;
+}
+
+#endif
+
 int gretl_is_xml_file (const char *fname)
 {
     gzFile fz;
@@ -355,20 +395,32 @@ static char *search_dir (char *fname, const char *topdir, int code)
     return NULL;
 }
 
-static int path_is_absolute (const char *fname)
-{
-    int ret = 0;
-
 #ifdef WIN32
-    if (fname[1] == ':') ret = 1; /* drive letter? */
+# define fslash(c) (c == '/' || c == '\\')
+#else
+# define fslash(c) (c == '/')
 #endif
 
-    /* we'll count as absolute paths specified using "." */
-    if (*fname == '.' || *fname == SLASH) {
-	ret = 1;
+static int dotpath (const char *fname)
+{
+    if (fname[0] == '.') {
+	if (fslash(fname[1])) {
+	    return 1;
+	} else if (fname[1] == '.' && fslash(fname[2])) {
+	    return 1;
+	}
     }
 
-    return ret;
+    return 0;
+}
+
+/* note: for our purposes we count filenames beginning with "./" or
+   "../" as absolute 
+*/
+
+int gretl_path_is_absolute (const char *fname)
+{
+    return g_path_is_absolute(fname) || dotpath(fname);
 }
 
 static void make_path_absolute (char *fname, const char *orig)
@@ -416,49 +468,47 @@ char *addpath (char *fname, PATHS *ppaths, int script)
     test = gretl_fopen(fname, "r");
     if (test != NULL) { 
 	fclose(test); 
-	if (!path_is_absolute(fname)) {
+	if (!gretl_path_is_absolute(fname)) {
 	    make_path_absolute(fname, orig);
 	}
 	return fname;
-    } else if (path_is_absolute(fname)) {  
+    } else if (gretl_path_is_absolute(fname)) {  
 	/* unable to open file as given: if the path was absolute, fail */
 	return NULL;
     }
 
-    if (ppaths == NULL) {
-	return NULL;
-    }
-
-    /* try looking where script was found */
-    if (*ppaths->currdir) {
-	if ((fname = search_dir(fname, ppaths->currdir, CURRENT_DIR))) {
-	    return fname;
-	}
-    }
-
-    fname = tmp;
-    strcpy(fname, orig);
-
-    if (script) {
-	/* for a script, try system script dir (and subdirs) */
-	if ((fname = search_dir(fname, ppaths->scriptdir, SCRIPT_SEARCH))) { 
-	    return fname;
-	} else {
-	    char fndir[MAXLEN];
-
-	    fname = tmp;
-	    strcpy(fname, orig);
-	    sprintf(fndir, "%sfunctions", ppaths->gretldir);
-	    if ((fname = search_dir(fname, fndir, FUNCS_SEARCH))) { 
+    if (ppaths != NULL) {
+	/* try looking where script was found */
+	if (*ppaths->currdir != '\0') {
+	    if ((fname = search_dir(fname, ppaths->currdir, CURRENT_DIR))) {
 		return fname;
 	    }
 	}
-    } else {
-	/* for a data file, try system data dir (and subdirs) */
-	if ((fname = search_dir(fname, ppaths->datadir, DATA_SEARCH))) { 
-	    return fname;
-	}
-    } 
+
+	fname = tmp;
+	strcpy(fname, orig);
+
+	if (script) {
+	    /* for a script, try system script dir (and subdirs) */
+	    if ((fname = search_dir(fname, ppaths->scriptdir, SCRIPT_SEARCH))) { 
+		return fname;
+	    } else {
+		char fndir[MAXLEN];
+
+		fname = tmp;
+		strcpy(fname, orig);
+		sprintf(fndir, "%sfunctions", ppaths->gretldir);
+		if ((fname = search_dir(fname, fndir, FUNCS_SEARCH))) { 
+		    return fname;
+		}
+	    }
+	} else {
+	    /* for a data file, try system data dir (and subdirs) */
+	    if ((fname = search_dir(fname, ppaths->datadir, DATA_SEARCH))) { 
+		return fname;
+	    }
+	} 
+    }
 
     /* or try looking in user's dir (and subdirs) */
     fname = tmp;
@@ -523,8 +573,8 @@ static int get_quoted_filename (const char *line, char *fname)
 
 static int substitute_homedir (char *fname)
 {
-    int err = 0;
     char *homedir = getenv("HOME");
+    int err = 0;
 
     if (homedir != NULL) {
 	int len = strlen(fname);
@@ -548,10 +598,9 @@ static int substitute_homedir (char *fname)
  * getopenfile:
  * @line: command line (e.g. "open foo").
  * @fname: filename to be filled out.
- * @ppaths: path information struct.
- * @setpath: if non-zero, set @ppaths->currdir based on the file
- * that is found (if any).
- * @script: if non-zero, suppose the file is a command script.
+ * @ppaths: pointer to paths information struct.
+ * @opt: if includes %OPT_W, treat as web filename and don't
+ * try to add path, if %OPT_S, treat as a script.
  * 
  * Elementary path-searching: try adding various paths to the given
  * @fname and see if it can be opened.
@@ -560,14 +609,23 @@ static int substitute_homedir (char *fname)
  */
 
 int getopenfile (const char *line, char *fname, PATHS *ppaths,
-		 int setpath, int script)
+		 gretlopt opt)
 {
+    int script = (opt & OPT_S)? 1 : 0;
     char *fullname;
 
-    /* get the initial filename off the command line */
-    if (get_quoted_filename(line, fname)) return 0; 
+    if (get_quoted_filename(line, fname)) {
+	/* if the filename was quoted, we'll leave it as is */
+	return 0; 
+    }
 
-    if (sscanf(line, "%*s %s", fname) != 1) return 1;
+    if (sscanf(line, "%*s %s", fname) != 1) {
+	return E_PARSE;
+    }
+
+    if (opt & OPT_W) {
+	return 0;
+    }
 
     /* handle tilde == HOME */
     if (*fname == '~') {
@@ -577,7 +635,7 @@ int getopenfile (const char *line, char *fname, PATHS *ppaths,
     /* try a basic path search on this filename */
     fullname = addpath(fname, ppaths, script);
 
-    if (fullname != NULL && setpath) {
+    if (ppaths != NULL && fullname != NULL && script) {
 	int n, spos = slashpos(fname);
 
 	if (spos) {
@@ -606,6 +664,8 @@ struct INTERNAL_PATHS {
     char libpath[MAXLEN];
     char x12a[MAXLEN];
     char x12adir[MAXLEN];
+    char tramo[MAXLEN];
+    char tramodir[MAXLEN];
     char pngfont[128];
     unsigned char status;
 };
@@ -630,20 +690,104 @@ static void set_gretl_libpath (const char *path)
     } else {
 	sprintf(gretl_paths.libpath, "%s/lib/gretl%s", path, sfx);
     }
-#endif /* WIN32 */
+#endif /* !WIN32 */
 }
 
-static void copy_paths_to_internal (const PATHS *paths)
+#if defined(HAVE_X12A) || defined(HAVE_TRAMO)
+
+static int set_tramo_x12a_dirs (PATHS *ppaths, int baddir)
 {
-    strcpy(gretl_paths.userdir, paths->userdir);
-    strcpy(gretl_paths.gnuplot, paths->gnuplot);
-    strcpy(gretl_paths.x12a, paths->x12a);
-    strcpy(gretl_paths.x12adir, paths->x12adir);
-    strcpy(gretl_paths.pngfont, paths->pngfont);
+    char dirname[MAXLEN];
+    size_t n;
+    int err = 0;
+
+    if (baddir) {
+# ifdef HAVE_TRAMO
+	*ppaths->tramodir = '\0';
+# endif
+# ifdef HAVE_X12A
+	*ppaths->x12adir = '\0';
+# endif
+	return baddir;
+    }
+
+    strcpy(dirname, ppaths->userdir);
+    n = strlen(dirname);
+
+    if (n > 0 && (dirname[n-1] == '\\' || dirname[n-1] == '/')) {
+	dirname[n-1] = '\0';
+    }
+
+# ifdef HAVE_X12A
+    build_path(ppaths->x12adir, ppaths->userdir, "x12arima", NULL);
+    err = gretl_mkdir(ppaths->x12adir);
+    if (err) {
+	*ppaths->x12adir = '\0';
+    }
+# endif
+
+# ifdef HAVE_TRAMO
+    build_path(ppaths->tramodir, ppaths->userdir, "tramo", NULL);
+    if (gretl_mkdir(ppaths->tramodir)) {
+	*ppaths->tramodir = '\0';
+	return E_FOPEN;
+    }
+
+    sprintf(dirname, "%s%coutput", ppaths->tramodir, SLASH);
+    gretl_mkdir(dirname);
+
+    sprintf(dirname, "%s%cgraph", ppaths->tramodir, SLASH);
+    if (gretl_mkdir(dirname)) {
+	*ppaths->tramodir = '\0';
+	return E_FOPEN;
+    }
+
+    sprintf(dirname, "%s%cgraph%cacf", ppaths->tramodir, SLASH, SLASH);
+    gretl_mkdir(dirname);
+    sprintf(dirname, "%s%cgraph%cfilters", ppaths->tramodir, SLASH, SLASH);
+    gretl_mkdir(dirname);
+    sprintf(dirname, "%s%cgraph%cforecast", ppaths->tramodir, SLASH, SLASH);
+    gretl_mkdir(dirname);
+    sprintf(dirname, "%s%cgraph%cseries", ppaths->tramodir, SLASH, SLASH);
+    gretl_mkdir(dirname);
+    sprintf(dirname, "%s%cgraph%cspectra", ppaths->tramodir, SLASH, SLASH);
+    gretl_mkdir(dirname);
+# endif
+
+    return err;
+}
+
+#endif /* x12a || tramo */
+
+static void copy_paths_to_internal (PATHS *paths)
+{
+    strcpy(gretl_paths.userdir,  paths->userdir);
+    strcpy(gretl_paths.gnuplot,  paths->gnuplot);
+    strcpy(gretl_paths.x12a,     paths->x12a);
+    strcpy(gretl_paths.x12adir,  paths->x12adir);
+    strcpy(gretl_paths.tramo,    paths->tramo);
+    strcpy(gretl_paths.tramodir, paths->tramodir);
+    strcpy(gretl_paths.pngfont,  paths->pngfont);
+
+    gretl_insert_builtin_string("gretldir",  paths->gretldir);
+    gretl_insert_builtin_string("userdir",   paths->userdir);
+    gretl_insert_builtin_string("gnuplot",   paths->gnuplot);
+    gretl_insert_builtin_string("x12a",      paths->x12a);
+    gretl_insert_builtin_string("x12adir",   paths->x12adir);
+    gretl_insert_builtin_string("tramo",     paths->tramo);
+    gretl_insert_builtin_string("tramodir",  paths->tramodir);
 }
 
 const char *gretl_lib_path (void)
 {
+    if (*gretl_paths.libpath == '\0') {
+	char *epath = getenv("GRETL_PLUGIN_PATH");
+
+	if (epath != NULL) {
+	    strncat(gretl_paths.libpath, epath, MAXLEN - 1);
+	}
+    }
+
     return gretl_paths.libpath;
 }
 
@@ -652,12 +796,93 @@ const char *gretl_user_dir (void)
     return gretl_paths.userdir;
 }
 
-void set_gretl_user_dir (const char *path, PATHS *ppaths)
+#ifdef WIN32
+
+static void correct_blank_userdir (PATHS *paths)
 {
-    strcpy(gretl_paths.userdir, path);
-    ensure_slash(gretl_paths.userdir);
-    strcpy(ppaths->userdir, path);
+    char *home = appdata_path();
+
+    if (home != NULL) {
+	sprintf(paths->userdir, "%s\\gretl\\", home);
+	free(home);
+    } 
+}
+
+#else
+
+static void correct_blank_userdir (PATHS *paths)
+{
+    char *home = getenv("HOME");
+
+    if (home != NULL) {
+	sprintf(paths->userdir, "%s/gretl/", home);
+    } 
+}
+
+#endif
+
+static int validate_userdir (const char *dirname)
+{
+    int err = 0;
+
+    if (*dirname == '\0') {
+	strcpy(gretl_errmsg, _("User directory is not set"));
+	return E_DATA;
+    }
+
+    err = gretl_mkdir(dirname);
+    if (err) {
+	sprintf(gretl_errmsg, _("Couldn't create directory '%s'"), dirname);
+    }
+
+    if (!err) {
+	/* ensure the directory is writable */
+	char *testname;
+	FILE *fp;
+
+	testname = g_strdup_printf("%s%cwrite.chk", dirname, SLASH);
+	if (testname != NULL) {
+	    fp = gretl_fopen(testname, "w");
+	    if (fp == NULL) {
+		sprintf(gretl_errmsg, _("Couldn't write to '%s': "
+					"gretl will not work properly!"), 
+			dirname);
+		err = 1;
+	    } else {
+		fclose(fp);
+		remove(testname);
+	    }
+	    g_free(testname);
+	}
+    }
+
+    return err;
+}
+
+int set_gretl_user_dir (const char *path, PATHS *ppaths)
+{
+    int err = validate_userdir(path);
+
+    if (err) {
+	return err;
+    }
+
+    if (path != ppaths->userdir) {
+	strcpy(ppaths->userdir, path);
+    }
+
     ensure_slash(ppaths->userdir);
+    set_tramo_x12a_dirs(ppaths, 0);
+
+    strcpy(gretl_paths.userdir, ppaths->userdir);
+    strcpy(gretl_paths.x12adir, ppaths->x12adir);
+    strcpy(gretl_paths.tramodir, ppaths->tramodir);
+
+    gretl_insert_builtin_string("userdir", ppaths->userdir);
+    gretl_insert_builtin_string("x12adir",  ppaths->x12adir);
+    gretl_insert_builtin_string("tramodir", ppaths->tramodir);
+
+    return 0;
 }
 
 const char *gretl_gnuplot_path (void)
@@ -678,6 +903,11 @@ char *set_gretl_plotfile (const char *fname)
     return gretl_paths.plotfile;
 }
 
+const char *gretl_tramo (void)
+{
+    return gretl_paths.tramo;
+}
+
 const char *gretl_x12_arima (void)
 {
     return gretl_paths.x12a;
@@ -734,15 +964,15 @@ void show_paths (const PATHS *ppaths)
 
 #ifdef WIN32
 
-int set_paths (PATHS *ppaths, gretlopt opt)
+int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 {
     char envstr[MAXLEN];
+    int err = 0;
 
     if (opt & OPT_D) {
 	/* set defaults */
-	char *home;
+	char *home = getenv("GRETL_HOME");
 
-	home = getenv("GRETL_HOME");
 	if (home != NULL) {
 	    strcpy(ppaths->gretldir, home);
 	    ensure_slash(ppaths->gretldir);
@@ -754,7 +984,7 @@ int set_paths (PATHS *ppaths, gretlopt opt)
 	strcpy(ppaths->ratsbase, "f:\\"); 
 
 	strcpy(ppaths->x12a, "c:\\userdata\\x12arima\\x12a.exe");
-	strcpy(ppaths->x12adir, "c:\\userdata\\x12arima");
+	strcpy(ppaths->tramo, "c:\\userdata\\tramo\\tramo.exe");
 
 	if (opt & OPT_X) {
 	    strcpy(ppaths->dbhost, "ricardo.ecn.wfu.edu");
@@ -762,13 +992,19 @@ int set_paths (PATHS *ppaths, gretlopt opt)
 	    ppaths->dbhost[0] = '\0';
 	}
 
+	shelldir_init();
 	ppaths->currdir[0] = '\0';
-
 	*gretl_paths.plotfile = '\0';
-
 	strcpy(ppaths->pngfont, "verdana 8");
     } else {
 	ensure_slash(ppaths->gretldir);
+	if (*ppaths->userdir == '\0') {
+	    correct_blank_userdir(ppaths);
+	}
+	err = validate_userdir(ppaths->userdir);
+#if defined(HAVE_X12A) || defined(HAVE_TRAMO)
+	err = set_tramo_x12a_dirs(ppaths, err);
+#endif
     }
 
     sprintf(ppaths->datadir, "%sdata\\", ppaths->gretldir);
@@ -796,22 +1032,44 @@ int set_paths (PATHS *ppaths, gretlopt opt)
     putenv(envstr);
 
     ensure_slash(ppaths->userdir);
-
     set_gretl_libpath(ppaths->gretldir);
-
     copy_paths_to_internal(ppaths);
 
-    return 0;
+    return err;
 }
 
 #else /* not Windows */
 
-int set_paths (PATHS *ppaths, gretlopt opt)
+static void check_gretldir (PATHS *ppaths)
 {
+    char *epath = getenv("GRETL_HOME");
+
+    ensure_slash(ppaths->gretldir);
+
+    if (epath != NULL && strcmp(epath, ppaths->gretldir)) {
+	/* which one is right? */
+	char test[FILENAME_MAX];
+	FILE *fp;
+
+	sprintf(test, "%sCOPYING", epath);
+	fp = gretl_fopen(test, "r");
+	if (fp != NULL) {
+	    *ppaths->gretldir = '\0';
+	    strncat(ppaths->gretldir, epath, MAXLEN - 2);
+	    ensure_slash(ppaths->gretldir);
+	    fclose(fp);
+	}
+    }
+}
+
+int gretl_set_paths (PATHS *ppaths, gretlopt opt)
+{
+    int err = 0;
+
     if (opt & OPT_D) {
-	char *home;
+	/* defaults */
+	char *home = getenv("GRETL_HOME");
 
-	home = getenv("GRETL_HOME");
 	if (home != NULL) {
 	    strcpy(ppaths->gretldir, home);
 	    ensure_slash(ppaths->gretldir);
@@ -831,8 +1089,8 @@ int set_paths (PATHS *ppaths, gretlopt opt)
 
 	strcpy(ppaths->gnuplot, "gnuplot");
 	strcpy(ppaths->pngfont, "Vera 9");
-
 	ppaths->currdir[0] = '\0';	
+	shelldir_init();
 
 	/* try to set a default userdir */
 	home = getenv("HOME");
@@ -843,14 +1101,14 @@ int set_paths (PATHS *ppaths, gretlopt opt)
 	    *ppaths->userdir = '\0';
 	}
 
-#ifdef HAVE_X12A
-	strcpy(ppaths->x12a, "x12a");
-	sprintf(ppaths->x12adir, "%sx12arima", ppaths->userdir);
-#endif
-
 	*gretl_paths.plotfile = '\0';
     } else {
-	ensure_slash(ppaths->gretldir);
+	/* check validity of gretldir, userdir */
+	check_gretldir(ppaths);
+	if (*ppaths->userdir == '\0') {
+	    correct_blank_userdir(ppaths);
+	}
+	err = validate_userdir(ppaths->userdir);
     }
 
     sprintf(ppaths->datadir, "%sdata/", ppaths->gretldir);
@@ -872,13 +1130,47 @@ int set_paths (PATHS *ppaths, gretlopt opt)
 	sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretlcli.hlp"));
     }
 
-    ensure_slash(ppaths->userdir);
+    if (getenv("GTKSOURCEVIEW_LANGUAGE_DIR") == NULL) {
+	char envstr[MAXLEN];
 
+	sprintf(envstr, "GTKSOURCEVIEW_LANGUAGE_DIR=%sgtksourceview",
+		ppaths->gretldir);
+	putenv(envstr);
+    }
+
+    ensure_slash(ppaths->userdir);
     set_gretl_libpath(ppaths->gretldir);
 
+#if defined(HAVE_X12A) || defined(HAVE_TRAMO)
+    if ((!(opt & OPT_D) || !(opt & OPT_X))) {
+	err = set_tramo_x12a_dirs(ppaths, err);
+    }
+#endif
+
     copy_paths_to_internal(ppaths);
 
-    return 0;
+    return err;
 }
 
 #endif /* win32 versus unix */
+
+/* for writing a file, name given by user: if the path is not
+   absolute, switch to the gretl user dir (for a plain filename and
+   use_cwd not set) or to the current "shelldir" (for a filename
+   beginning with '.', or if use_cwd is set).
+*/
+
+void gretl_maybe_switch_dir (const char *fname)
+{
+    if (!g_path_is_absolute(fname)) {
+	if (dotpath(fname) || get_use_cwd()) {
+	    char *sdir = get_shelldir();
+
+	    if (sdir != NULL) {
+		chdir(sdir);
+	    }
+	} else {
+	    chdir(gretl_paths.userdir);
+	}
+    }
+}
