@@ -21,7 +21,9 @@
 #include "system.h"
 #include "var.h"
 #include "objstack.h"
+#include "matrix_extra.h"
 #include "gretl_restrict.h"
+#include "bootstrap.h"
 
 #define RDEBUG 0
 
@@ -31,22 +33,27 @@ struct restriction_ {
     int nterms;      /* number of terms in restriction */
     double *mult;    /* array of numerical multipliers on coeffs */
     int *eq;         /* array of equation numbers (for cross-equation case) */
-    int *coeff;      /* array of coeff numbers */
+    int *bnum;       /* array of coeff numbers */
     double rhs;      /* numerical value on right-hand side */
 };
 
 struct restriction_set_ {
     int k;                        /* number of restrictions (rows) */
     int cross;                    /* includes cross-equation restrictions? */
+    gretl_matrix *R;              /* LHS restriction matrix */
+    gretl_matrix *q;              /* RHS restriction matrix */
     char *mask;                   /* selection mask for coeffs */
     restriction **restrictions;
     void *obj;
     GretlObjType type;
     gretlopt opt;
+    double test;
+    double pval;
+    double bsum;
+    double bsd;
+    int code;
 };
 
-static void destroy_restriction_set (gretl_restriction_set *rset);
-
 static int check_R_matrix (const gretl_matrix *R)
 {
     gretl_matrix *m;
@@ -54,11 +61,13 @@ static int check_R_matrix (const gretl_matrix *R)
     int err = 0;
 
     m = gretl_matrix_alloc(k, k);
-    if (m == NULL) return E_ALLOC;
+    if (m == NULL) {
+	return E_ALLOC;
+    }
 
     gretl_matrix_multiply_mod(R, GRETL_MOD_NONE,
 			      R, GRETL_MOD_TRANSPOSE,
-			      m);
+			      m, GRETL_MOD_NONE);
 
     err = gretl_invert_general_matrix(m);
     
@@ -73,7 +82,7 @@ get_R_vecm_column (const gretl_restriction_set *rset, int i, int j)
     const restriction *r = rset->restrictions[i];
     GRETL_VAR *var = rset->obj;
     int nb = gretl_VECM_n_beta(var);
-    int col = r->coeff[j];
+    int col = r->bnum[j];
     int k;
 
     if (r->eq != NULL) {
@@ -91,7 +100,7 @@ get_R_sys_column (const gretl_restriction_set *rset, int i, int j)
     const restriction *r = rset->restrictions[i];
     gretl_equation_system *sys = rset->obj;
     const int *list;
-    int col = r->coeff[j];
+    int col = r->bnum[j];
     int k;
 
     for (k=0; k<r->eq[j]; k++) {
@@ -119,7 +128,7 @@ static double get_restriction_param (const restriction *r, int k)
     int i;    
 
     for (i=0; i<r->nterms; i++) {
-	if (r->coeff[i] == k) {
+	if (r->bnum[i] == k) {
 	    x = r->mult[i];
 	    break;
 	}
@@ -138,10 +147,14 @@ static int R_n_columns (gretl_restriction_set *rset)
 	cols = system_n_indep_vars(rset->obj);
     } else {
 	MODEL *pmod = rset->obj;
-
-	for (i=0; i<pmod->ncoeff; i++) {
-	    if (rset->mask[i]) {
-		cols++;
+	
+	if (rset->mask == NULL) {
+	    cols = pmod->ncoeff;
+	} else {
+	    for (i=0; i<pmod->ncoeff; i++) {
+		if (rset->mask[i]) {
+		    cols++;
+		}
 	    }
 	}
     }
@@ -150,19 +163,16 @@ static int R_n_columns (gretl_restriction_set *rset)
 }
 
 static int 
-restriction_set_form_full_matrices (gretl_restriction_set *rset,
-				    gretl_matrix **Rin,
-				    gretl_vector **qin)
+restriction_set_form_full_matrices (gretl_restriction_set *rset)
 {
     MODEL *pmod;
-    gretl_matrix *R;
-    gretl_vector *q;
+    gretl_matrix *R = NULL;
+    gretl_vector *q = NULL;
     restriction *r;
     double x;
     int i, j, k;
 
-    if (rset->type != GRETL_OBJ_EQN ||
-	rset->obj == NULL) {
+    if (rset->type != GRETL_OBJ_EQN || rset->obj == NULL) {
 	return 1;
     }
 
@@ -194,38 +204,34 @@ restriction_set_form_full_matrices (gretl_restriction_set *rset,
 	gretl_vector_set(q, i, r->rhs);
     }
 
-    *Rin = R;
-    *qin = q;
+    rset->R = R;
+    rset->q = q;
 
     return 0;
 }
 
 static int 
-restriction_set_form_matrices (gretl_restriction_set *rset,
-			       gretl_matrix **Rin,
-			       gretl_vector **qin)
+restriction_set_form_matrices (gretl_restriction_set *rset)
 {
     MODEL *pmod = NULL;
-    gretl_matrix *R;
-    gretl_vector *q;
+    gretl_matrix *R = NULL;
+    gretl_vector *q = NULL;
     restriction *r;
     double x;
     int col, i, j;
+    int err = 0;
 
-    R = gretl_matrix_alloc(rset->k, R_n_columns(rset));
+    R = gretl_zero_matrix_new(rset->k, R_n_columns(rset));
     if (R == NULL) {
 	return E_ALLOC;
     }
 
-    q = gretl_column_vector_alloc(rset->k);
+    q = gretl_zero_matrix_new(rset->k, 1);
     if (q == NULL) {
 	gretl_matrix_free(R);
 	return E_ALLOC;
     }
 
-    gretl_matrix_zero(R);
-    gretl_matrix_zero(q);
-
     if (rset->type == GRETL_OBJ_EQN) {
 	pmod = rset->obj;
     }
@@ -256,21 +262,24 @@ restriction_set_form_matrices (gretl_restriction_set *rset,
 #endif
 
     if (rset->type == GRETL_OBJ_VAR) {
-	GRETL_VAR *var = rset->obj;
+	gretl_matrix *D = gretl_matrix_right_nullspace(R, &err);
+
+	if (!err) {
+	    GRETL_VAR *var = rset->obj;
 
-	gretl_matrix *D = gretl_matrix_right_nullspace(R);
 #if RDEBUG
-	gretl_matrix_print(D, "D");
+	    gretl_matrix_print(D, "D");
 #endif
-	gretl_VAR_attach_restrictions(var, D);
+	    gretl_VAR_attach_restrictions(var, D);
+	}
 	gretl_matrix_free(R);
 	gretl_matrix_free(q);
     } else {
-	*Rin = R;
-	*qin = q;
+	rset->R = R;
+	rset->q = q;
     }
 
-    return 0;
+    return err;
 }
 
 /* Make a mask with 1s in positions in the array of coeffs where
@@ -284,8 +293,7 @@ static int restriction_set_make_mask (gretl_restriction_set *rset)
     restriction *r;
     int i, j;
 
-    if (rset->type != GRETL_OBJ_EQN ||
-	rset->obj == NULL) {
+    if (rset->type != GRETL_OBJ_EQN || rset->obj == NULL) {
 	return 1;
     }
 
@@ -300,7 +308,7 @@ static int restriction_set_make_mask (gretl_restriction_set *rset)
     for (i=0; i<rset->k; i++) {
 	r = rset->restrictions[i];
 	for (j=0; j<r->nterms; j++) {
-	    rset->mask[r->coeff[j]] = 1;
+	    rset->mask[r->bnum[j]] = 1;
 	}	
     }
 
@@ -320,7 +328,115 @@ static int count_ops (const char *p)
     return n;
 }
 
-static int parse_b_bit (const char *s, int *eq, int *bnum)
+/* Given the dataset position of a variable as the identifier for a
+   parameter, try to retrieve the corresponding 0-based coefficient
+   number in the model to be restricted.  We don't yet attempt this
+   for anything other than single-equation models.
+*/
+
+static int 
+bnum_from_vnum (gretl_restriction_set *r, int v, const DATAINFO *pdinfo)
+{
+    const MODEL *pmod;
+    int k;
+
+    if (r->type != GRETL_OBJ_EQN || r->obj == NULL) {
+	return -1;
+    }
+
+    pmod = r->obj;
+
+    k = gretl_model_get_param_number(pmod, pdinfo, pdinfo->varname[v]);
+
+#if RDEBUG
+    fprintf(stderr, "bnum_from_vnum: vnum = %d (%s) -> bnum = %d (coeff = %g)\n", 
+	    v, pdinfo->varname[v], k, pmod->coeff[k]);
+#endif
+
+    /* convert to 1-based for compatibility with numbers read directly:
+       the index will be converted to 0-base below */
+
+    return k + 1; 
+}
+
+/* Pick apart strings of the form "b[X]" or "b[X,Y]".  If the ",Y" is
+   present the "X" element must be an equation number, and the "Y" may
+   be a coefficient number or the name of a variable.  If the ",Y" is
+   not present, "X" may be a coefficient number or the name of a
+   variable.  This function is actually fed the string in question at
+   an offset of 1 beyond the "[".  In parsing, we skip any white space
+   in the string.
+*/
+
+static int pick_apart (gretl_restriction_set *r, const char *s, 
+		       int *eq, int *bnum,
+		       const DATAINFO *pdinfo)
+{
+    char s1[16] = {0};
+    char s2[16] = {0};
+    char *targ = s1;
+    int i, j, k;
+    int vnum = -1;
+
+#if RDEBUG
+    fprintf(stderr, "pick_apart: looking at '%s'\n", s);
+#endif
+
+    *eq = *bnum = -1;
+
+    k = haschar(']', s);
+    if (k <= 0 || k > 30) {
+	return E_PARSE;
+    }
+
+    j = 0;
+    for (i=0; i<k; i++) {
+	if (s[i] == ',') {
+	    targ = s2;
+	    j = 0;
+	} else if (!isspace(s[i])) {
+	    if (j == 15) {
+		return E_PARSE;
+	    }
+	    targ[j++] = s[i];
+	}
+    }
+
+#if RDEBUG
+    fprintf(stderr, " s1 = '%s', s2 = '%s'\n", s1, s2);
+#endif
+
+    if (targ == s2) {
+	/* got a comma separator: [eqn,bnum] */
+	*eq = positive_int_from_string(s1);
+	if (*eq <= 0) {
+	    return E_PARSE;
+	}
+	if (isdigit(*s2)) {
+	    *bnum = positive_int_from_string(s2);
+	} else if (pdinfo != NULL) {
+	    vnum = varindex(pdinfo, s2);
+	}
+    } else {
+	/* only one field: [bnum] */
+	*eq = 1;
+	if (isdigit(*s1)) {
+	    *bnum = positive_int_from_string(s1);
+	} else if (pdinfo != NULL) {
+	    vnum = varindex(pdinfo, s1);
+	}	    
+    }
+
+    if (pdinfo != NULL && vnum >= 0 && vnum < pdinfo->v) {
+	*bnum = bnum_from_vnum(r, vnum, pdinfo);
+    }
+
+    return 0;
+}
+
+static int parse_b_bit (gretl_restriction_set *r, const char *s, 
+			int *eq, int *bnum,
+			const DATAINFO *pdinfo)
 {
     int err = E_PARSE;
 
@@ -328,12 +444,7 @@ static int parse_b_bit (const char *s, int *eq, int *bnum)
 	sscanf(s, "%d", bnum);
 	err = 0;
     } else if (*s == '[') {
-	if (sscanf(s, "[%d,%d]", eq, bnum) == 2) {
-	    err = 0;
-	} else if (sscanf(s, "[%d]", bnum) == 1) {
-	    *eq = 1;
-	    err = 0;
-	}	
+	err = pick_apart(r, s + 1, eq, bnum, pdinfo);
     }
 
     if (*bnum < 1) {
@@ -345,7 +456,6 @@ static int parse_b_bit (const char *s, int *eq, int *bnum)
     }
 
     if (*eq < 1) {
-	fprintf(stderr, "borked HERE\n");
 	sprintf(gretl_errmsg, _("Equation number (%d) is out of range"), 
 		*eq);
 	err = 1;
@@ -357,7 +467,8 @@ static int parse_b_bit (const char *s, int *eq, int *bnum)
 }
 
 static int 
-parse_coeff_chunk (const char *s, double *x, int *eq, int *bnum)
+parse_coeff_chunk (gretl_restriction_set *r, const char *s, double *x, 
+		   int *eq, int *bnum, const DATAINFO *pdinfo)
 {
     const char *s0 = s;
     int err = E_PARSE;
@@ -368,7 +479,7 @@ parse_coeff_chunk (const char *s, double *x, int *eq, int *bnum)
 
     if (*s == 'b') {
 	s++;
-	err = parse_b_bit(s, eq, bnum);
+	err = parse_b_bit(r, s, eq, bnum, pdinfo);
 	*x = 1.0;
     } else if (sscanf(s, "%lf", x)) {
 	s += strspn(s, " ");
@@ -376,7 +487,7 @@ parse_coeff_chunk (const char *s, double *x, int *eq, int *bnum)
 	s += strspn(s, " *");
 	if (*s == 'b') {
 	    s++;
-	    err = parse_b_bit(s, eq, bnum);
+	    err = parse_b_bit(r, s, eq, bnum, pdinfo);
 	}
     }
 
@@ -385,8 +496,8 @@ parse_coeff_chunk (const char *s, double *x, int *eq, int *bnum)
     } 
 
 #if RDEBUG
-    printf("parse_coeff_chunk: x=%g, eq=%d, bnum=%d\n", 
-	   *x, *eq, *bnum);
+    fprintf(stderr, "parse_coeff_chunk: x=%g, eq=%d, bnum=%d\n", 
+	    *x, *eq, *bnum);
 #endif
 
     return err;
@@ -398,11 +509,11 @@ static void destroy_restriction (restriction *r)
 
     free(r->mult);
     free(r->eq);
-    free(r->coeff);
+    free(r->bnum);
     free(r);
 }
 
-static void destroy_restriction_set (gretl_restriction_set *rset)
+void destroy_restriction_set (gretl_restriction_set *rset)
 {
     int i;
 
@@ -412,6 +523,10 @@ static void destroy_restriction_set (gretl_restriction_set *rset)
 
     free(rset->restrictions);
     free(rset->mask);
+    
+    gretl_matrix_free(rset->R);
+    gretl_matrix_free(rset->q);
+
     free(rset);
 }
 
@@ -421,22 +536,24 @@ static restriction *restriction_new (int n, int cross)
     int i;
 
     r = malloc(sizeof *r);
-    if (r == NULL) return NULL;
+    if (r == NULL) {
+	return NULL;
+    }
 
     r->mult = NULL;
     r->eq = NULL;
-    r->coeff = NULL;
+    r->bnum = NULL;
 	
     r->mult = malloc(n * sizeof *r->mult);
-    r->coeff = malloc(n * sizeof *r->coeff);
-    if (r->mult == NULL || r->coeff == NULL) {
+    r->bnum = malloc(n * sizeof *r->bnum);
+    if (r->mult == NULL || r->bnum == NULL) {
 	destroy_restriction(r);
 	return NULL;
     }
 
     for (i=0; i<n; i++) {
 	r->mult[i] = 0.0;
-	r->coeff[i] = 0;
+	r->bnum[i] = 0;
     }
 
     if (cross) {
@@ -501,7 +618,7 @@ static void print_restriction (const gretl_restriction_set *rset,
     int i, k;
 
     for (i=0; i<r->nterms; i++) {
-	k = r->coeff[i];
+	k = r->bnum[i];
 	print_mult(r->mult[i], i == 0, prn);
 	if (rset->cross) {
 	    pprintf(prn, "b[%d,%d]", r->eq[i] + 1, k + 1);
@@ -547,7 +664,7 @@ add_term_to_restriction (restriction *r, double mult, int eq, int bnum, int i)
     int j;
 
     for (j=0; j<i; j++) {
-	if (bnum == r->coeff[j] && (r->eq == NULL || eq == r->eq[j])) {
+	if (bnum == r->bnum[j] && (r->eq == NULL || eq == r->eq[j])) {
 	    /* additional reference to a previously referenced coeff */
 	    r->mult[j] += mult;
 	    r->nterms -= 1;
@@ -556,7 +673,7 @@ add_term_to_restriction (restriction *r, double mult, int eq, int bnum, int i)
     }
 
     r->mult[i] = mult;
-    r->coeff[i] = bnum;
+    r->bnum[i] = bnum;
 
     if (r->eq != NULL) {
 	r->eq[i] = eq;
@@ -578,10 +695,18 @@ real_restriction_set_start (void *ptr, GretlObjType type,
     rset->type = type;
     rset->opt = opt;
 
+    rset->test = NADBL;
+    rset->pval = NADBL;
+    rset->bsum = NADBL;
+    rset->bsd = NADBL;
+
     rset->k = 0;
+    rset->R = NULL;
+    rset->q = NULL;
     rset->mask = NULL;
     rset->restrictions = NULL;
     rset->cross = 0;
+    rset->code = GRETL_STAT_NONE;
 
     if (rset->type == GRETL_OBJ_SYS) {
 	rset->cross = 1;
@@ -650,6 +775,7 @@ static int bnum_out_of_bounds (const gretl_restriction_set *rset,
 static int 
 real_restriction_set_parse_line (gretl_restriction_set *rset, 
 				 const char *line,
+				 const DATAINFO *pdinfo,
 				 int first)
 {
     const char *p = line;
@@ -658,7 +784,7 @@ real_restriction_set_parse_line (gretl_restriction_set *rset,
     int i, nt, err = 0;
 
 #if RDEBUG
-    printf("parse restriction line: got '%s'\n", line);
+    fprintf(stderr, "parse restriction line: got '%s'\n", line);
 #endif
 
     if (!strncmp(p, "restrict", 8)) {
@@ -681,7 +807,7 @@ real_restriction_set_parse_line (gretl_restriction_set *rset,
     nt = 1 + count_ops(p);
 
 #if RDEBUG
-    printf("restriction line: assuming %d terms\n", nt);
+    fprintf(stderr, "restriction line: assuming %d terms\n", nt);
 #endif
 
     r = augment_restriction_set(rset, nt);
@@ -707,10 +833,10 @@ real_restriction_set_parse_line (gretl_restriction_set *rset,
 	p += len;
 
 #if RDEBUG
-	printf(" working on chunk %d, '%s'\n", i, chunk);
+	fprintf(stderr, " working on chunk %d, '%s'\n", i, chunk);
 #endif
 
-	err = parse_coeff_chunk(chunk, &mult, &eq, &bnum);
+	err = parse_coeff_chunk(rset, chunk, &mult, &eq, &bnum, pdinfo);
 	if (err) {
 	    break;
 	} else if (bnum_out_of_bounds(rset, bnum, eq)) {
@@ -748,7 +874,8 @@ real_restriction_set_parse_line (gretl_restriction_set *rset,
 }
 
 int 
-restriction_set_parse_line (gretl_restriction_set *rset, const char *line)
+restriction_set_parse_line (gretl_restriction_set *rset, const char *line,
+			    const DATAINFO *pdinfo)
 {
     int nx = 0;
 
@@ -769,7 +896,7 @@ restriction_set_parse_line (gretl_restriction_set *rset, const char *line)
 	return 1;
     }
 
-    return real_restriction_set_parse_line(rset, line, 0);
+    return real_restriction_set_parse_line(rset, line, pdinfo, 0);
 }
 
 /* set-up for a set of restrictions for a VAR (vecm, actually) */
@@ -785,9 +912,9 @@ var_restriction_set_start (const char *line, GRETL_VAR *var)
 	return NULL;
     }
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
-    if (real_restriction_set_parse_line(rset, line, 1)) {
+    if (real_restriction_set_parse_line(rset, line, NULL, 1)) {
 	if (*gretl_errmsg == '\0') {
 	    sprintf(gretl_errmsg, _("parse error in '%s'\n"), line);
 	}
@@ -811,7 +938,7 @@ cross_restriction_set_start (const char *line, gretl_equation_system *sys)
 	return NULL;
     }
 
-    if (real_restriction_set_parse_line(rset, line, 1)) {
+    if (real_restriction_set_parse_line(rset, line, NULL, 1)) {
 	sprintf(gretl_errmsg, _("parse error in '%s'\n"), line);
 	return NULL;
     }
@@ -822,17 +949,17 @@ cross_restriction_set_start (const char *line, gretl_equation_system *sys)
 /* set-up for a set of restrictions on a single equation */
 
 gretl_restriction_set *
-eqn_restriction_set_start (const char *line, MODEL *pmod)
+eqn_restriction_set_start (const char *line, MODEL *pmod, gretlopt opt)
 {
     gretl_restriction_set *rset;
 
-    rset = real_restriction_set_start(pmod, GRETL_OBJ_EQN, OPT_NONE);
+    rset = real_restriction_set_start(pmod, GRETL_OBJ_EQN, opt);
     if (rset == NULL) {
 	strcpy(gretl_errmsg, _("Out of memory!"));
 	return NULL;
     }
 
-    if (real_restriction_set_parse_line(rset, line, 1)) {
+    if (real_restriction_set_parse_line(rset, line, NULL, 1)) {
 	sprintf(gretl_errmsg, _("parse error in '%s'\n"), line);
 	return NULL;
     }
@@ -869,13 +996,13 @@ restriction_set_start (const char *line, gretlopt opt, int *err)
 
     if (ptr == NULL) {
 	*err = E_DATA;
-	return NULL;
+	goto bailout;
     }
 
     if (type != GRETL_OBJ_EQN && type != GRETL_OBJ_SYS &&
 	type != GRETL_OBJ_VAR) {
 	*err = E_DATA;
-	return NULL;
+	goto bailout;
     }
 
     rset = real_restriction_set_start(ptr, type, opt);
@@ -884,104 +1011,57 @@ restriction_set_start (const char *line, gretlopt opt, int *err)
     }
 
     if (!*err && name == NULL) {
-	*err = real_restriction_set_parse_line(rset, line, 1);
+	*err = real_restriction_set_parse_line(rset, line, NULL, 1);
 	if (*err) {
 	    rset = NULL;
 	    if (*err == E_PARSE) {
 		sprintf(gretl_errmsg, _("parse error in '%s'\n"), line);
 	    }
 	}
-    }	
+    }
+
+ bailout:
+
+    free(name);
 
     return rset;
 }
 
-static void print_pval_str (double pval, char *str)
-{
-    if (pval < .00001) {
-	sprintf(str, "<%.5f", 0.00001);
-    } else {
-	sprintf(str, "%.5f", pval);
-    }
-}
-
-static int print_coeff (const MODEL *pmod, int i,
+static int 
+print_restricted_coeff (const MODEL *pmod, int i,
 			double coeff, double sderr, int k,
 			const DATAINFO *pdinfo, 
 			PRN *prn)
 {
-    int do_pval = 1;
-    double t, pvalue = 999.0;
+    model_coeff mc;
     int gotnan = 0;
-    char varname[24];
 
-    gretl_model_get_param_name(pmod, pdinfo, i, varname);
-    pprintf(prn, "  %-15s ", varname);
-    
-    if (isnan(coeff) || na(coeff)) {
-	pprintf(prn, "%*s", UTF_WIDTH(_("undefined"), 17), _("undefined"));
+    model_coeff_init(&mc);
+
+    if (xna(coeff)) {
 	gotnan = 1;
-    } else {
-	gretl_print_value(coeff, prn);
     }
 
-    if (isnan(sderr) || na(sderr)) {
-	pprintf(prn, "%*s\n", UTF_WIDTH(_("undefined"), 16), _("undefined"));
-	return 1;
+    mc.b = coeff;
+    mc.se = sderr;
+
+    if (!xna(coeff) && !xna(sderr) && sderr > 0) {
+	mc.tval = coeff / sderr;
+	mc.pval = coeff_pval(pmod->ci, mc.tval, pmod->dfd + k);
     }
 
-    gretl_print_value(sderr, prn); 
+    gretl_model_get_param_name(pmod, pdinfo, i, mc.name);
 
-    if (sderr > 0.0) {
-	t = coeff / sderr;
-	if (fabs(t) >= 1000.0) {
-	    char numstr[9];
-
-	    sprintf(numstr, "%#8.2G", t);
-	    pprintf(prn, " %8s", numstr);
-	} else {
-	    pprintf(prn, " %7.3f", t);
-	}
-
-	if (do_pval) {
-	    char pvalstr[16];
-	    int dfd = pmod->dfd + k;
-
-	    pvalue = coeff_pval(pmod, t, dfd);
-	    print_pval_str(pvalue, pvalstr);
-	    pprintf(prn, "%*s", UTF_WIDTH(pvalstr, 10), pvalstr);
-	}
-    } else if (do_pval) { 
-	do_pval = 0;
-	pprintf(prn, "     %*s", UTF_WIDTH(_("undefined"), 10), _("undefined"));
-    }
-
-    if (do_pval) {
-	if (pvalue < 0.01) {
-	    pputs(prn, " ***");
-	} else if (pvalue < 0.05) {
-	    pputs(prn, " **");
-	} else if (pvalue < 0.10) {
-	    pputs(prn, " *");
-	}
-    } 
-
-    pputc(prn, '\n');
+    print_coeff(&mc, prn);
 
     return gotnan;
 }
 
 static void coeff_header (const MODEL *pmod, PRN *prn)
 {
-    int use_param = pmod->ci == NLS || pmod->ci == MLE;
+    int use_param = pmod->ci == NLS || pmod->ci == MLE || pmod->ci == GMM;
 
-    if (use_param) {
-	pputs(prn, _("      PARAMETER       ESTIMATE          STDERROR"
-		     "      T STAT   P-VALUE\n\n"));
-    } else {
-	pputs(prn, _("      VARIABLE       COEFFICIENT        STDERROR"
-		     "      T STAT   P-VALUE\n\n"));
-    }
+    print_coeff_heading(use_param, prn);
 }
 
 static int 
@@ -992,8 +1072,6 @@ do_restricted_estimates (gretl_restriction_set *rset,
     MODEL *pmod = rset->obj;
     gretl_matrix *X = NULL;
     gretl_matrix *y = NULL;
-    gretl_matrix *R = NULL;
-    gretl_matrix *q = NULL;
     gretl_matrix *b = NULL;
     gretl_matrix *S = NULL;
     int *xlist = NULL;
@@ -1013,7 +1091,7 @@ do_restricted_estimates (gretl_restriction_set *rset,
 	goto bailout;
     }
 
-    err = restriction_set_form_full_matrices(rset, &R, &q);
+    err = restriction_set_form_full_matrices(rset);
     if (err) {
 	goto bailout;
     }
@@ -1038,11 +1116,12 @@ do_restricted_estimates (gretl_restriction_set *rset,
     }
 
 #if RDEBUG
-    gretl_matrix_print(R, "R");
-    gretl_matrix_print(q, "q");
+    gretl_matrix_print(rset->R, "R");
+    gretl_matrix_print(rset->q, "q");
 #endif
 
-    err = gretl_matrix_restricted_ols(y, X, R, q, b, S, &s2);
+    err = gretl_matrix_restricted_ols(y, X, rset->R, rset->q, 
+				      b, S, NULL, &s2);
 
     if (!err) {
 	double v, coeff, se;
@@ -1053,7 +1132,7 @@ do_restricted_estimates (gretl_restriction_set *rset,
 	    coeff = gretl_vector_get(b, i);
 	    v = gretl_matrix_get(S, i, i);
 	    se = (v > 1.0e-16)? sqrt(v) : 0.0;
-	    print_coeff(pmod, i, coeff, se, rset->k, pdinfo, prn);
+	    print_restricted_coeff(pmod, i, coeff, se, rset->k, pdinfo, prn);
 	}
 	pputc(prn, '\n');
 	pprintf(prn, "  %s = %.*g\n", _("Standard error of residuals"), 
@@ -1064,8 +1143,6 @@ do_restricted_estimates (gretl_restriction_set *rset,
     
     gretl_matrix_free(X);
     gretl_matrix_free(y);
-    gretl_matrix_free(R);
-    gretl_matrix_free(q);
     gretl_matrix_free(b);
     gretl_matrix_free(S);
 
@@ -1074,38 +1151,72 @@ do_restricted_estimates (gretl_restriction_set *rset,
     return err;
 }
 
-/* execute the test, for a single equation */
+/* print result, single equation */
 
-static int test_restriction_set (gretl_restriction_set *rset, 
-				 const double **Z,
-				 const DATAINFO *pdinfo,
-				 PRN *prn)
+static void 
+restriction_set_print_result (gretl_restriction_set *rset, 
+			      const double **Z, const DATAINFO *pdinfo,
+			      PRN *prn)
+{
+    MODEL *pmod = rset->obj;
+    int robust, asym;
+
+    robust = gretl_model_get_int(pmod, "robust");
+    asym = ASYMPTOTIC_MODEL(pmod->ci);
+
+    if (asym) {
+	rset->code = GRETL_STAT_WALD_CHISQ;
+	rset->pval = chisq_cdf_comp(rset->test, rset->k);
+	pprintf(prn, "\n%s: %s(%d) = %g, ", _("Test statistic"), 
+		(robust)? _("Robust chi^2"): "chi^2",
+		rset->k, rset->test);
+    } else {
+	rset->code = GRETL_STAT_F;
+	rset->test /= rset->k;
+	rset->pval = f_cdf_comp(rset->test, rset->k, pmod->dfd);
+	pprintf(prn, "\n%s: %s(%d, %d) = %g, ", _("Test statistic"), 
+		(robust)? _("Robust F"): "F",
+		rset->k, pmod->dfd, rset->test);
+    }
+
+    pprintf(prn, _("with p-value = %g\n"), rset->pval);
+    pputc(prn, '\n');
+
+    if (!(rset->opt & OPT_C)) {
+	record_test_result(rset->test, rset->pval, _("restriction"));
+    }
+
+    if (pmod != NULL && Z != NULL && !(rset->opt & OPT_Q) 
+	&& pmod->ci == OLS) {
+	do_restricted_estimates(rset, Z, pdinfo, prn);
+    }
+}
+
+/* execute the test, for a single equation */
+
+static int 
+test_restriction_set (gretl_restriction_set *rset, PRN *prn)
 {
     MODEL *pmod = rset->obj;
-    gretl_matrix *R;
-    gretl_vector *q;
     gretl_matrix *vcv = NULL;
     gretl_vector *b = NULL;
     gretl_vector *br = NULL;
-    gretl_matrix *Rv = NULL;
-    double test_stat, pval;
-    int err, robust, freeRv = 1;
+    gretl_matrix *RvR = NULL;
+    int err, freeRvR = 1;
 
-    int asym = ASYMPTOTIC_MODEL(pmod->ci);
+    gretl_error_clear();
 
-    *gretl_errmsg = '\0';
-
-    err = restriction_set_form_matrices(rset, &R, &q);
+    err = restriction_set_form_matrices(rset);
     if (err) {
 	return err;
     }
 
 #if RDEBUG
-    gretl_matrix_print(R, "R matrix");
-    gretl_matrix_print(q, "q vector");
+    gretl_matrix_print(rset->R, "R matrix");
+    gretl_matrix_print(rset->q, "q vector");
 #endif
 
-    if ((err = check_R_matrix(R))) {
+    if ((err = check_R_matrix(rset->R))) {
 	if (err == E_SINGULAR) {
 	    pputs(prn, _("Matrix inversion failed:\n"
 			 " restrictions may be inconsistent or redundant\n"));
@@ -1133,7 +1244,7 @@ static int test_restriction_set (gretl_restriction_set *rset,
     gretl_matrix_print(b, "coeff vector");
 #endif  
 
-    err = gretl_matrix_multiply(R, b, br);
+    err = gretl_matrix_multiply(rset->R, b, br);
     if (err) {
 	fprintf(stderr, "Failed: gretl_matrix_multiply(R, b, br)\n");
 	goto bailout;
@@ -1143,75 +1254,138 @@ static int test_restriction_set (gretl_restriction_set *rset,
     gretl_matrix_print(br, "br");
 #endif  
 
-    if (!gretl_is_zero_matrix(q)) {
-	err = gretl_matrix_subtract_from(br, q);
+    if (rset->opt & OPT_C) {
+	rset->bsum = br->val[0];
+    }
+
+    if (!gretl_is_zero_matrix(rset->q)) {
+	err = gretl_matrix_subtract_from(br, rset->q);
 	if (err) {
 	    fprintf(stderr, "Failed: gretl_matrix_subtract_from(br, q)\n");
 	    goto bailout;
 	}
     }
 
-    if (gretl_is_identity_matrix(R)) {
+    if (gretl_is_identity_matrix(rset->R)) {
 #if RDEBUG
 	fprintf(stderr, "R is identity matrix: taking shortcut\n");
 #endif  
-	Rv = vcv;
-	freeRv = 0;
+	RvR = vcv;
+	freeRvR = 0;
     } else {
-	Rv = gretl_matrix_A_X_A(R, GRETL_MOD_NONE, vcv, &err);
-	if (err) goto bailout;
+	RvR = gretl_matrix_alloc(rset->R->rows, rset->R->rows);
+	if (RvR == NULL) {
+	    err = E_ALLOC;
+	    goto bailout;
+	}
+	gretl_matrix_qform(rset->R, GRETL_MOD_NONE, vcv,
+			   RvR, GRETL_MOD_NONE);
 #if RDEBUG
-	gretl_matrix_print(Rv, "Rv");
+	gretl_matrix_print(RvR, "RvR");
 #endif  
+	if (rset->opt & OPT_C) {
+	    rset->bsd = sqrt(RvR->val[0]);
+	}
     }
 
-    err = gretl_invert_symmetric_matrix(Rv);
+    err = gretl_invert_symmetric_matrix(RvR);
     if (err) {
 	pputs(prn, _("Matrix inversion failed:\n"
 		     " restrictions may be inconsistent or redundant\n"));
 	goto bailout;
     }
     
-    test_stat = gretl_scalar_b_X_b(br, GRETL_MOD_TRANSPOSE, Rv, &err);
+    rset->test = gretl_scalar_qform(br, RvR, &err);
     if (err) {
 	pputs(prn, _("Failed to compute test statistic\n"));
 	goto bailout;
     }
 
-    robust = gretl_model_get_int(pmod, "robust");
-
-    if (asym) {
-	pval = chisq_cdf_comp(test_stat, rset->k);
-	pprintf(prn, "\n%s: %s(%d) = %g, ", _("Test statistic"), 
-		(robust)? _("Robust chi^2"): "chi^2",
-		rset->k, test_stat);
-    } else {
-	test_stat /= rset->k;
-	pval = f_cdf_comp(test_stat, rset->k, pmod->dfd);
-	pprintf(prn, "\n%s: %s(%d, %d) = %g, ", _("Test statistic"), 
-		(robust)? _("Robust F"): "F",
-		rset->k, pmod->dfd, test_stat);
-    }
-
-    pprintf(prn, _("with p-value = %g\n"), pval);
-    pputc(prn, '\n');
-
-    record_test_result(test_stat, pval, _("restriction"));
-
-    if (pmod != NULL && !(rset->opt & OPT_Q) && pmod->ci == OLS) {
-	do_restricted_estimates(rset, Z, pdinfo, prn);
-    }
-
  bailout:
 
-    gretl_matrix_free(R);
-    gretl_vector_free(q);
     gretl_matrix_free(vcv);
     gretl_vector_free(b);
     gretl_vector_free(br);
     
-    if (freeRv) {
-	gretl_matrix_free(Rv);
+    if (freeRvR) {
+	gretl_matrix_free(RvR);
+    }
+
+    return err;
+}
+
+static int rset_expand_R (gretl_restriction_set *rset, int k)
+{
+    gretl_matrix *R = NULL;
+    double rij;
+    int i, j, jj;
+
+    R = gretl_zero_matrix_new(rset->k, k);
+    if (R == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=0; i<rset->k; i++) {
+	jj = 0;
+	for (j=0; j<k; j++) {
+	    if (rset->mask[j]) {
+		rij = gretl_matrix_get(rset->R, i, jj);
+		gretl_matrix_set(R, i, j, rij);
+		jj++;
+	    }
+	}
+    }
+
+    gretl_matrix_free(rset->R);
+    rset->R = R;
+
+    return 0;
+}
+
+static int do_single_equation_test (gretl_restriction_set *rset,
+				    const double **Z,
+				    const DATAINFO *pdinfo,
+				    PRN *prn)
+{
+    int err, done = 0;
+
+    if (rset->opt & OPT_B) {
+	MODEL *pmod = rset->obj;
+
+	if (!bootstrap_ok(pmod->ci)) {
+	    pputs(prn, "Sorry, the bootstrap option is not supported for this test");
+	} else {
+	    restriction *r = rset->restrictions[0];
+
+	    if (rset->k == 1 && r->nterms == 1 && r->rhs == 0) {
+		/* a simple zero restriction */
+		gretlopt bopt = OPT_P | OPT_R;
+		int B = 0;
+
+		gretl_restriction_get_boot_params(&B, &bopt);
+		err = bootstrap_analysis(pmod, r->bnum[0], B, Z, pdinfo, 
+					 bopt, prn);
+		done = 1;
+	    } else {
+		/* a more complex restriction */
+		err = test_restriction_set(rset, prn);
+		if (!err) {
+		    rset->test /= rset->k;
+		    rset_expand_R(rset, pmod->ncoeff);
+		    err = bootstrap_test_restriction(pmod, rset->R, rset->q,
+						     rset->test, rset->k, Z, 
+						     pdinfo, prn);
+		}
+	    }
+	    done = 1;
+	}
+    }
+
+    if (!done) {
+	err = test_restriction_set(rset, prn);
+	if (!err) {
+	    restriction_set_print_result(rset, Z, pdinfo, prn);
+	}
     }
 
     return err;
@@ -1237,22 +1411,20 @@ gretl_restriction_set_finalize (gretl_restriction_set *rset,
 
     if (rset->type == GRETL_OBJ_VAR) {
 	/* vecm */
-	err = restriction_set_form_matrices(rset, NULL, NULL);
+	err = restriction_set_form_matrices(rset);
 	if (!err) {
 	    print_restriction_set(rset, pdinfo, prn);
-	    gretl_VECM_test_beta(rset->obj, prn);
+	    gretl_VECM_test_beta(rset->obj, pdinfo, prn);
 	}
+	destroy_restriction_set(rset);
     } else if (rset->type == GRETL_OBJ_SYS) {
 	/* simultaneous equations system */
-	gretl_matrix *R;
-	gretl_matrix *q;
-
 #if RDEBUG
 	print_restriction_set(rset, pdinfo, prn);
 #endif
-	err = restriction_set_form_matrices(rset, &R, &q);
+	err = restriction_set_form_matrices(rset);
 	if (!err) {
-	    err = check_R_matrix(R);
+	    err = check_R_matrix(rset->R);
 	    if (err == E_SINGULAR) {
 		pputs(prn, _("Matrix inversion failed:\n"
 			     " restrictions may be inconsistent or redundant\n"));
@@ -1261,19 +1433,151 @@ gretl_restriction_set_finalize (gretl_restriction_set *rset,
 	    }
 	}	
 	if (!err) {
-	    system_set_restriction_matrices(rset->obj, R, q);
+	    system_set_restriction_matrices(rset->obj, rset->R, rset->q);
+	    rset->R = NULL;
+	    rset->q = NULL;
 	    destroy_restriction_set(rset);
 	}
     } else {
-	/* single model */
+	/* single-equation model */
 	err = restriction_set_make_mask(rset);
 	if (!err) {
 	    print_restriction_set(rset, pdinfo, prn);
-	    test_restriction_set(rset, Z, pdinfo, prn);
-	    destroy_restriction_set(rset);
+	    err = do_single_equation_test(rset, Z, pdinfo, prn);
+	    if (!(rset->opt & OPT_C)) {
+		destroy_restriction_set(rset);
+	    }
 	}
     }	
 
     return err;
 }
 
+/**
+ * gretl_sum_test:
+ * @list: list of variables to use.
+ * @pmod: pointer to model.
+ * @pdinfo: information on the data set.
+ * @prn: gretl printing struct.
+ * 
+ * Calculates the sum of the coefficients, relative to the given model, 
+ * for the variables given in @list.  Prints this estimate along 
+ * with its standard error.
+ * 
+ * Returns: 0 on successful completion, error code on error.
+ */
+
+int 
+gretl_sum_test (const int *list, MODEL *pmod, const DATAINFO *pdinfo,
+		PRN *prn)
+{
+    gretl_restriction_set *r;
+    char line[MAXLEN];
+    char bstr[24];
+    int i, len, err = 0;
+
+    if (list[0] < 2) {
+	pprintf(prn, _("Invalid input\n"));
+	return E_DATA;
+    }
+
+    if (!command_ok_for_model(COEFFSUM, 0, pmod->ci)) {
+	return E_NOTIMP;
+    }
+
+    r = real_restriction_set_start(pmod, GRETL_OBJ_EQN, OPT_Q | OPT_C);
+    if (r == NULL) {
+	return 1;
+    }
+
+    *line = '\0';
+    len = 0;
+
+    for (i=1; i<=list[0]; i++) {
+	sprintf(bstr, "b[%s]", pdinfo->varname[list[i]]);
+	len += strlen(bstr) + 4;
+	if (len >= MAXLEN - 1) {
+	    err = E_PARSE;
+	    break;
+	}
+	strcat(line, bstr);
+	if (i < list[0]) {
+	    strcat(line, " + ");
+	} else {
+	    strcat(line, " = 0");
+	}
+    }
+
+    if (!err) {
+	err = real_restriction_set_parse_line(r, line, pdinfo, 1); 
+    }
+
+    if (!err) {
+	err = gretl_restriction_set_finalize(r, NULL, pdinfo, NULL);
+    }
+
+    if (!err) {
+	double test;
+
+	pprintf(prn, "\n%s: ", _("Variables"));
+
+	for (i=1; i<=list[0]; i++) {
+	    pprintf(prn, "%s ", pdinfo->varname[list[i]]);
+	}
+
+	pprintf(prn, "\n   %s = %g\n", _("Sum of coefficients"), r->bsum);
+
+	if (r->code == GRETL_STAT_F) {
+	    pprintf(prn, "   %s = %g\n", _("Standard error"), r->bsd);
+	    test = sqrt(r->test);
+	    if (r->bsum < 0) {
+		test = -test;
+	    }
+	    pprintf(prn, "   t(%d) = %g ", pmod->dfd, test);
+	    pprintf(prn, _("with p-value = %g\n"), r->pval);
+	    record_test_result(test, r->pval, _("sum")); 
+	} else if (r->code == GRETL_STAT_WALD_CHISQ) {
+	    pprintf(prn, "   %s = %g\n", _("Standard error"), r->bsd);
+	    test = sqrt(r->test);
+	    if (r->bsum < 0) {
+		test = -test;
+	    }
+	    r->pval = normal_pvalue_2(test);
+	    pprintf(prn, "   z = %g ", test);
+	    pprintf(prn, _("with p-value = %g\n"), r->pval);
+	    record_test_result(test, r->pval, _("sum")); 
+	}	    
+
+	destroy_restriction_set(r);
+    }
+
+    return err;
+}
+
+static int restrict_B;
+static gretlopt rboot_opt;
+
+int gretl_restriction_set_boot_params (int B, gretlopt opt)
+{
+    int err = 0;
+
+    rboot_opt = opt;
+
+    if (B > 0) {
+	restrict_B = B;
+    } else {
+	err = E_DATA;
+    }
+
+    return err;
+}
+
+void gretl_restriction_get_boot_params (int *pB, gretlopt *popt)
+{
+    *pB = restrict_B;
+    *popt |= rboot_opt;
+
+    /* these are ad hoc values */
+    restrict_B = 0;
+    rboot_opt = OPT_NONE;
+}
