@@ -32,16 +32,26 @@ gretl_restriction_set *
 var_restriction_set_start (const char *line, GRETL_VAR *var);
 
 gretl_restriction_set *
-eqn_restriction_set_start (const char *line, MODEL *pmod);
+eqn_restriction_set_start (const char *line, MODEL *pmod, gretlopt opt);
 
 int 
-restriction_set_parse_line (gretl_restriction_set *rset, const char *line);
+restriction_set_parse_line (gretl_restriction_set *rset, const char *line,
+			    const DATAINFO *pdinfo);
 
 int
 gretl_restriction_set_finalize (gretl_restriction_set *rset, 
 				const double **Z, const DATAINFO *pdinfo,
 				PRN *prn);
 
-#endif
+void destroy_restriction_set (gretl_restriction_set *rset);
+
+int gretl_sum_test (const int *list, MODEL *pmod, const DATAINFO *pdinfo,
+		    PRN *prn);
+
+int gretl_restriction_set_boot_params (int B, gretlopt opt);
+
+void gretl_restriction_get_boot_params (int *pB, gretlopt *popt);
+
+#endif /* GRETL_RESTRICT_H */
 
 
