@@ -34,4 +34,18 @@ int gretl_string_table_print (gretl_string_table *st, DATAINFO *pdinfo,
 
 void gretl_string_table_destroy (gretl_string_table *st);
 
+void gretl_insert_builtin_string (const char *name, const char *s);
+
+int save_named_string (const char *name, const char *s, PRN *prn);
+
+char *get_named_string (const char *name);
+
+int process_string_command (const char *line, PRN *prn);
+
+int substitute_named_strings (char *line);
+
+int string_is_defined (const char *sname);
+
+void saved_strings_cleanup (void);
+
 #endif /* GRETL_STRING_TABLE_H */
