@@ -18,21 +18,24 @@
  */
 
 #include "libgretl.h"
+#include "monte_carlo.h"
 #include "gretl_func.h"
 #include "objstack.h"
 #include "cmd_private.h"
 #include "libset.h"
 #include "usermat.h"
 #include "gretl_panel.h"
+#include "loop_private.h"
+#include "texprint.h"
+#include "gretl_string_table.h"
 
 #include <errno.h>
+#include <time.h>
 
 #ifndef WIN32
 # include <signal.h>
-# ifdef USE_GLIB2
-#  include <glib.h>
-#  define USE_GSPAWN
-# endif
+# include <glib.h>
+# define USE_GSPAWN
 #endif
 
 /**
@@ -261,33 +264,92 @@ int gretl_isunits (int t1, int t2, const double *x)
     return ret;
 }
 
+#define FEWVALS 8
+
+static int few_vals (int t1, int t2, const double *x)
+{
+    double test[FEWVALS];
+    int match;
+    int i, t, n = 0;
+
+    for (t=t1; t<=t2; t++) {
+	if (!na(x[t])) {
+	    match = 0;
+	    for (i=0; i<n; i++) {
+		if (x[t] == test[i]) {
+		    match = 1;
+		    break;
+		}
+	    }
+	    if (!match) {
+		if (n == FEWVALS) {
+		    n++;
+		    break;
+		}
+		test[n++] = x[t];
+	    }
+	}
+    }
+
+    return n;
+}
+
  /**
  * gretl_isdiscrete:
  * @x: data series to examine.
  * @t1: starting observation.
  * @t2: ending observation. 
  *
- * Returns: 1 if @x contains only values whose decimal part is 
- * zero in the given sample range and 0 otherwise.  Missing 
- * values are skipped, but numbers that cannot be represented
- * as integers as rejected as "not discrete".
+ * Checks the variable @x over the range @t1 to @t2 for discreteness.
+ * This is a heuristic whose components are (a) whether the values
+ * are "fairly round" (multiples of 0.25) or not, and, if test (a) is
+ * passed, (b) whether the variable takes on only "few" distinct
+ * values.
+ * 
+ * Returns: 0 if test (a) is not passed or the number of distinct values
+ * is > 8; else 1 if the number of distinct values is <= 8; else 2 if 
+ * the number of distinct values is <= 4.  A return of 1 is supposed
+ * to indicate that it's "reasonable" to treat @x as discrete, while
+ * a return of 2 indicates that it's probably ureasonable _not_ to
+ * treat @x as discrete, for the purpose of drawing up a frequency
+ * distribution.
  */
 
 int gretl_isdiscrete (int t1, int t2, const double *x)
 {
-    int t, ret = 1;
+    int t, n = 0, d = 1;
+    double r;
 
     for (t=t1; t<=t2; t++) {
 	if (na(x[t])) {
 	    continue;
 	}
-	if (!ok_int(x[t]) || x[t] != floor(x[t])) {
-	    ret = 0;
+	n++;
+	if (!ok_int(x[t])) {
+	    d = 0;
 	    break;
 	}
+	r = x[t] - floor(x[t]);
+	if (r != 0.0 && r != 0.25 && r != 0.5 && r != 0.75) {
+	    d = 0;
+	    break;
+	}	    
+    }
+
+    if (n == 0) {
+	d = 0;
+    }
+
+    if (d) {
+	n = few_vals(t1, t2, x);
+	if (n > FEWVALS) {
+	    d = 0;
+	} else if (n < 5) {
+	    d = 2;
+	}
     }
 
-    return ret;
+    return d;
 }
 
 /**
@@ -390,6 +452,26 @@ int count_distinct_values (const double *x, int n)
     return c;
 }
 
+/**
+ * count_distinct_int_values:
+ * @x: sorted array of ints.
+ * @n: number of elements in array.
+ *
+ * Returns: the number of distinct values in array @x,
+ * provided that @x is already sorted.
+ */
+
+int count_distinct_int_values (const int *x, int n)
+{
+    int i, c = 1;
+
+    for (i=1; i<n; i++) {
+	if (x[i] != x[i-1]) c++;
+    }
+
+    return c;
+}
+
 /**
  * rearrange_id_array:
  * @x: sorted array of doubles.
@@ -450,14 +532,27 @@ void printlist (const int *list, const char *msg)
     fputc('\n', stderr);
 }
 
-/* Compute model selection criteria */
+/**
+ * gretl_calculate_criteria:
+ * @ess: error sum of squares.
+ * @n: number of observations.
+ * @k: number of parameters estimated.
+ * @ll: pointer to recieve loglikelihood.
+ * @aic: pointer to recieve Akaike criterion.
+ * @bic: pointer to recieve Schwartz Bayesian criterion.
+ * @hqc: pointer to recieve Hannan-Quinn criterion.
+ *
+ * Calculates model selection criteria based on @ess, @nobs and
+ * @k, for a model estimated via least squares.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
 
 int gretl_calculate_criteria (double ess, int n, int k,
-			      double *pll, double *aic, double *bic,
+			      double *ll, double *aic, double *bic,
 			      double *hqc)
 {
-    double ll;
-    double c[3];
+    double lnl, c[3];
     int err = 0;
 
     if (na(ess) || ess <= 0.0 || k < 1 || n <= k) {
@@ -467,25 +562,25 @@ int gretl_calculate_criteria (double ess, int n, int k,
 
 	errno = 0;
 
-	ll = -.5 * n * log(ess);
+	lnl = -.5 * n * log(ess);
 
 	if (errno == EDOM || errno == ERANGE) {
 	    err = 1;
 	} else {
-	    ll += -.5 * n * (ln2pi1 - log((double) n));
-	    c[0] = -2.0 * ll + 2 * k;
-	    c[1] = -2.0 * ll + k * log(n);
-	    c[2] = -2.0 * ll + 2 * k * log(log(n));
+	    lnl += -.5 * n * (ln2pi1 - log((double) n));
+	    c[0] = -2.0 * lnl + 2 * k;
+	    c[1] = -2.0 * lnl + k * log(n);
+	    c[2] = -2.0 * lnl + 2 * k * log(log(n));
 	}
     }
 
     if (err) {
-	*pll = NADBL;
+	*ll = NADBL;
 	*aic = NADBL;
 	*bic = NADBL;
 	*hqc = NADBL;
     } else {
-	*pll = ll;
+	*ll = lnl;
 	*aic = c[0];
 	*bic = c[1];
 	*hqc = c[2];
@@ -494,6 +589,16 @@ int gretl_calculate_criteria (double ess, int n, int k,
     return err;
 }
 
+/**
+ * ls_criteria:
+ * @pmod: pointer to gretl model structure.
+ *
+ * Fills out the model selection criteria members of @pmod, using
+ * gretl_calculate_criteria().
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
 int ls_criteria (MODEL *pmod)
 {
     double ll, aic, bic, hqc;
@@ -510,6 +615,19 @@ int ls_criteria (MODEL *pmod)
     return err;
 }
 
+/**
+ * gretl_print_criteria:
+ * @ess: error sum of squares.
+ * @nobs: number of observations.
+ * @ncoeff: number of parameters estimated.
+ * @prn: printing struct.
+ *
+ * Prints the values of the model selection criteria AIC, 
+ * BIC and HQC for the given parameters.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
 int gretl_print_criteria (double ess, int nobs, int ncoeff, PRN *prn)
 {
     double ll, aic, bic, hqc;
@@ -545,6 +663,19 @@ real_format_obs (char *obs, int maj, int min, int pd, char sep)
     return obs;
 }
 
+/**
+ * format_obs:
+ * @obs: target string (should be of length #OBSLEN).
+ * @maj: major period (e.g. year).
+ * @min: minor period (e.g. quarter, month).
+ * @pd: data frequency.
+ *
+ * Prints to @obs the gretl-type date string representing 
+ * the observation given by @maj, @min and @pd.
+ *
+ * Returns: @obs.
+ */
+
 char *format_obs (char *obs, int maj, int min, int pd)
 {
     return real_format_obs(obs, maj, min, pd, ':');
@@ -639,7 +770,7 @@ catch_setobs_errors (const char *stobs, int pd, int n, int min, gretlopt opt)
 /**
  * set_obs:
  * @line: command line.
- * @pZ: pointer to data array.
+ * @Z: data array.
  * @pdinfo: data information struct.
  * @opt: %OPT_S for stacked time-series, %OPT_C for stacked cross-section,
  * %OPT_T for time series, %OPT_X for cross section, %OPT_P to set
@@ -651,19 +782,20 @@ catch_setobs_errors (const char *stobs, int pd, int n, int min, gretlopt opt)
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo, 
+int set_obs (const char *line, double **Z, DATAINFO *pdinfo, 
 	     gretlopt opt)
 {
     char stobs[OBSLEN];
     int structure = STRUCTURE_UNKNOWN;
+    double sd0 = pdinfo->sd0;
     int pd, dated = 0;
     int err = 0;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     if (opt & OPT_R) {
 	/* restructure panel: "hidden" option */
-	return switch_panel_orientation(*pZ, pdinfo);
+	return switch_panel_orientation(Z, pdinfo);
     }    
 
     if (!strcmp(line, "setobs")) {
@@ -672,7 +804,7 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (opt & OPT_P) {
-	return set_panel_structure_from_line(line, *pZ, pdinfo);
+	return set_panel_structure_from_line(line, Z, pdinfo);
     }
 
     /* now we get down to business */
@@ -724,7 +856,7 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
 		return 1;
 	    }
 
-	    pdinfo->sd0 = ed0;
+	    sd0 = ed0;
 	    structure = TIME_SERIES;
 
 	    /* replace any existing markers with date strings */
@@ -735,7 +867,7 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
 	}
     } else if (structure == TIME_SERIES && pd == 10) {
 	/* decennial data */
-	pdinfo->sd0 = (double) atoi(stobs);
+	sd0 = (double) atoi(stobs);
     } else {
 	int maj = 0, min = 0;
 
@@ -747,7 +879,7 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
 	/* catch undated daily or weekly data */
 	if ((pd == 5 || pd == 6 || pd == 7 || pd == 52)  
 	    && min == 0 && opt != OPT_X && opt != OPT_S && opt != OPT_C) {
-	    pdinfo->structure = TIME_SERIES;
+	    structure = TIME_SERIES;
 	} else {
 	    if (catch_setobs_errors(stobs, pd, pdinfo->n, min, opt)) {
 		return 1;
@@ -771,11 +903,12 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	/* for non-calendar data */
-	pdinfo->sd0 = dot_atof(stobs);
+	sd0 = dot_atof(stobs);
     }
 
     pdinfo->pd = pd;
     pdinfo->structure = structure;
+    pdinfo->sd0 = sd0;
 
     if (pdinfo->structure != STACKED_TIME_SERIES &&
 	pdinfo->structure != STACKED_CROSS_SECTION &&
@@ -792,7 +925,7 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
        stacked time series form
     */
     if (pdinfo->structure == STACKED_CROSS_SECTION) {
-	err = switch_panel_orientation(*pZ, pdinfo);
+	err = switch_panel_orientation(Z, pdinfo);
     }	
 
     if (!err && pdinfo->structure == STACKED_TIME_SERIES) {
@@ -842,13 +975,13 @@ int gretl_int_from_string (const char *s, const double **Z,
 	return n;
     } else if (Z == NULL || pdinfo == NULL) {
 	*err = E_DATA;
+    } else if (test[1] == '\0' && is_active_index_loop_char(*test)) {
+	n = loop_scalar_read(*s);
     } else {
 	int v = varindex(pdinfo, s);
 	double x;
 
-	if (v == INDEXNUM) {
-	    n = loop_scalar_read(*s);
-	} else if (v >= pdinfo->v) {
+	if (v >= pdinfo->v) {
 	    *err = E_UNKVAR;
 	} else if (var_is_series(pdinfo, v)) {
 	    *err = E_TYPES;
@@ -903,13 +1036,13 @@ int positive_int_from_string (const char *s)
 
 int varnum_from_string (const char *str, DATAINFO *pdinfo)
 {
-    int varno = positive_int_from_string(str);
+    int v = positive_int_from_string(str);
 
-    if (varno <= 0 || varno >= pdinfo->v) {
-	varno = -1;
+    if (v <= 0 || v >= pdinfo->v) {
+	v = -1;
     } 
     
-    return varno;
+    return v;
 }
 
 /**
@@ -931,6 +1064,10 @@ int rename_var_by_id (const char *idstr, const char *vname,
 	return E_DATA;
     }
 
+    if (var_is_const(pdinfo, v)) {
+	return overwrite_err(pdinfo, v);
+    }
+
     /* should be pre-checked for validity of varname and
        non-duplication (see interact.c under RENAME)
     */
@@ -970,6 +1107,27 @@ double *copyvec (const double *src, int n)
     return targ;
 }
 
+/**
+ * doubles_array_free:
+ * @X: 2-dimensional array of doubles.
+ * @m: number of sub-arrays.
+ *
+ * Frees a 2-dimensional array of doubles, first freeing
+ * each sub-array.
+ */
+
+void doubles_array_free (double **X, int m)
+{
+    int i;
+
+    if (X != NULL) {
+	for (i=0; i<m; i++) {
+	    free(X[i]);
+	}
+	free(X);
+    }
+}
+
 /**
  * doubles_array_new:
  * @m: number of sub-arrays.
@@ -984,7 +1142,7 @@ double *copyvec (const double *src, int n)
 double **doubles_array_new (int m, int n)
 {
     double **X;
-    int i, j;
+    int i;
 
     X = malloc(m * sizeof *X);
 
@@ -992,13 +1150,14 @@ double **doubles_array_new (int m, int n)
 	return X;
     }
 
+    for (i=0; i<m; i++) {
+	X[i] = NULL;
+    }
+
     for (i=0; i<m; i++) {
 	X[i] = malloc(n * sizeof **X);
 	if (X[i] == NULL) {
-	    for (j=0; j<i; j++) {
-		free(X[j]);
-	    }
-	    free(X);
+	    doubles_array_free(X, m);
 	    X = NULL;
 	    break;
 	}
@@ -1007,12 +1166,88 @@ double **doubles_array_new (int m, int n)
     return X;
 }
 
+/**
+ * data_array_from_model:
+ * @pmod: reference model.
+ * @Z: main data array.
+ * @missv: should equal 1 if there are missing values to be
+ * skipped, else 0.
+ *
+ * Constructs a dataset containing all the variables referenced in
+ * @pmod.  The arrays start at the correct sample offset for @pmod,
+ * and are contiguous.  If @missvals equals 0, this is done by creating
+ * a set of pointers into the main dataset, but if there are missing
+ * values to be handled, the sub-arrays are newly allocated and purged
+ * of NAs.
+ *
+ * Returns: two-dimensional array, or %NULL on failure.
+ */
+
+double **data_array_from_model (const MODEL *pmod, double **Z, int missv)
+{
+    double **X;
+    int nv = pmod->list[0];
+    int offset = pmod->t1;
+    int v, i;
+
+    if (missv) {
+	X = doubles_array_new(nv, pmod->nobs);
+    } else {
+	X = malloc(nv * sizeof *X);
+    }
+
+    if (X == NULL) return NULL;
+
+    if (missv) {
+	int t, s;
+
+	for (t=0; t<pmod->nobs; t++) {
+	    X[0][t] = 1.0;
+	}
+
+	for (i=1; i<nv; i++) {
+	    v = (i == 1)? pmod->list[1] : pmod->list[i + 1];
+	    s = 0;
+	    for (t=pmod->t1; t<=pmod->t2; t++) {
+		if (!na(pmod->uhat[t])) {
+		    X[i][s++] = Z[v][t];
+		}
+	    }
+	}
+    } else {
+	/* constant in slot 0 */
+	X[0] = Z[0] + offset;
+
+	/* dependent var in slot 1 */
+	X[1] = Z[pmod->list[1]] + offset;
+
+	/* independent vars in slots 2, 3, ... */
+	for (i=2; i<nv; i++) {
+	    v = pmod->list[i + 1];
+	    X[i] = Z[v] + offset;
+	}
+    }
+
+    return X;
+}
+
+/**
+ * re_estimate:
+ * @model_spec: estimation command line.
+ * @tmpmod: pointer to model to recieve results.
+ * @pZ: pointer to data array.
+ * @pdinfo: dataset information.
+ *
+ *
+ * Returns: 0 on success, non-zero on failure.
+ */
+
 int re_estimate (char *model_spec, MODEL *tmpmod, 
 		 double ***pZ, DATAINFO *pdinfo) 
 {
     CMD cmd;
     double rho = 0.0;
-    int err;
+    int err = 0;
 
     if (gretl_cmd_init(&cmd)) {
 	return 1;
@@ -1044,7 +1279,8 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
 	break;
     case LOGIT:
     case PROBIT:
-	*tmpmod = logit_probit(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt);
+	*tmpmod = logit_probit(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt, 
+			       NULL);
 	break;
     case TOBIT:
 	*tmpmod = tobit_model(cmd.list, pZ, pdinfo, NULL);
@@ -1104,7 +1340,7 @@ int gretl_spawn (char *cmdline)
     int ok, status;
     int ret = 0;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     signal(SIGCHLD, SIG_DFL);
 
@@ -1123,6 +1359,7 @@ int gretl_spawn (char *cmdline)
 	fprintf(stderr, "stderr: '%s'\n", errout);
 	if (!font_not_found(errout)) {
 	    strcpy(gretl_errmsg, errout);
+	    fprintf(stderr, "gretl_errmsg: '%s'\n", gretl_errmsg);
 	    ret = 1;
 	}
     } else if (status != 0) {
@@ -1201,7 +1438,26 @@ int gretl_copy_file (const char *src, const char *dest)
     fclose(destfd);
 
     return 0;
-}    
+}  
+
+/* timer */  
+
+static clock_t tim0;
+
+static void gretl_stopwatch_init (void)
+{
+    tim0 = clock();
+}
+
+double gretl_stopwatch (void)
+{
+    clock_t tim1 = clock();
+    double x = (double) (tim1 - tim0) / CLOCKS_PER_SEC;
+
+    tim0 = tim1;
+
+    return x;
+} 
 
 /* library init and cleanup functions */
 
@@ -1209,23 +1465,31 @@ void libgretl_init (void)
 {
     libset_init();
     gretl_rand_init();
+    gretl_stopwatch_init();
     set_gretl_tex_preamble(); 
 }
 
+void libgretl_session_cleanup (void)
+{
+    gretl_saved_objects_cleanup();
+    gretl_transforms_cleanup();
+    gretl_lists_cleanup();
+    destroy_user_matrices();
+    gretl_plotx(NULL);
+}
+
 void libgretl_cleanup (void)
 {
     const char *p;
 
+    libgretl_session_cleanup();
+
     gretl_rand_free();
     gretl_functions_cleanup();
-    gretl_saved_objects_cleanup();
-    gretl_transforms_cleanup();
     libset_cleanup();
-    gretl_lists_cleanup();
     gretl_command_hash_cleanup();
-    destroy_user_matrices();
+    saved_strings_cleanup();
     lapack_mem_free();
-    gretl_plotx(NULL);
 
     p = strstr(gretl_plotfile(), "gpttmp");
     if (p != NULL) {
@@ -1291,3 +1555,4 @@ double get_last_pvalue (char *blurb)
 {
     return record_or_get_test_result(0, 0, blurb, GET_TEST_PVAL);
 }
+
