@@ -22,13 +22,6 @@
 
 #include "libgretl.h"
 
-typedef enum {
-    C_AIC,
-    C_BIC,
-    C_HQC,
-    C_MAX
-} model_selection_criteria;
-
 #include <float.h>
 #include <limits.h>
 
@@ -41,6 +34,8 @@ typedef enum {
 
 void libgretl_init (void);
 
+void libgretl_session_cleanup (void);
+
 void libgretl_cleanup (void);
 
 double date (int nt, int pd, const double sd0);
@@ -63,7 +58,7 @@ int true_const (int v, const double **Z, const DATAINFO *pdinfo);
 
 char *format_obs (char *obs, int maj, int min, int pd);
 
-int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo, 
+int set_obs (const char *line, double **Z, DATAINFO *pdinfo, 
 	     gretlopt opt);
 
 /* sorting and comparison */
@@ -74,6 +69,8 @@ int gretl_inverse_compare_doubles (const void *a, const void *b);
 
 int count_distinct_values (const double *x, int n);
 
+int count_distinct_int_values (const int *x, int n);
+
 int rearrange_id_array (double *x, int m, int n);
 
 int gretl_compare_ints (const void *a, const void *b);
@@ -97,8 +94,13 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
 
 double *copyvec (const double *src, int n);
 
+void doubles_array_free (double **X, int m);
+
 double **doubles_array_new (int m, int n);
 
+double **data_array_from_model (const MODEL *pmod, double **Z, 
+				int missv);
+
 int ijton (int i, int j, int nrows);
 
 int ztox (int i, double *px, const double **Z, const DATAINFO *pdinfo);
@@ -113,7 +115,7 @@ int gretl_spawn (char *cmdline);
 
 /* model selection criteria */
 
-int gretl_calculate_criteria (double ess, int nobs, int ncoeff,
+int gretl_calculate_criteria (double ess, int n, int k,
 			      double *ll, double *aic, double *bic, 
 			      double *hqc);
 
@@ -129,4 +131,8 @@ double get_last_test_statistic (char *blurb);
 
 double get_last_pvalue (char *blurb);
 
+/* timer */
+
+double gretl_stopwatch (void);
+
 #endif /* GRETL_UTILS_H */
