@@ -22,17 +22,25 @@
 
 #ifdef WIN32
 
+#include <sys/types.h>
+#include <dirent.h>
 #include <windows.h>
 
+DIR *win32_opendir (const char *dname);
+
 int read_reg_val (HKEY tree, const char *base,
 		  char *keyname, char *keyval);
 
+int read_reg_val_with_fallback (HKEY tree0, HKEY tree1, 
+				const char *base, char *keyname, 
+				char *keyval);
+
 int write_reg_val (HKEY tree, const char *base,
 		   const char *keyname, const char *keyval);
 
 void cli_read_registry (char *callname, PATHS *ppaths);
 
-void win_show_error (DWORD dw);
+void win_show_last_error (void);
 
 int winfork (char *cmdline, const char *dir, int wshow,
 	     DWORD flags);
@@ -41,6 +49,10 @@ int gretl_spawn (char *cmdline);
 
 char *desktop_path (void);
 
+char *appdata_path (void);
+
+char *mydocs_path (void);
+
 int gretl_shell (const char *arg);
 
 #endif /* WIN32 */
