@@ -20,6 +20,8 @@
 #include "libgretl.h"
 #include "gretl_xml.h"
 #include "gretl_panel.h"
+#include "gretl_func.h"
+#include "usermat.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -740,6 +742,15 @@ int *gretl_xml_node_get_list (xmlNodePtr node, xmlDocPtr doc, int *err)
     return list;
 }
 
+static const char *skip_to_next (const char *s)
+{
+    s += strspn(s, " \r\n");
+    s += strcspn(s, " \r\n");
+    s += strspn(s, " \r\n");
+
+    return s;
+}
+
 static void *gretl_xml_get_array (xmlNodePtr node, xmlDocPtr doc,
 				  ModelDataType type,
 				  int *nelem, int *err)
@@ -789,12 +800,10 @@ static void *gretl_xml_get_array (xmlNodePtr node, xmlDocPtr doc,
 			} else if (type == MODEL_DATA_CMPLX_ARRAY) {
 			    if (sscanf(p, "%lf %lf", &cvals[i].r, &cvals[i].i) != 2) {
 				*err = E_DATA;
-			    }
-			}			    
-			/* skip to next number */
-			p += strspn(p, " \r\n");
-			p += strcspn(p, " \r\n");
-			p += strspn(p, " \r\n");
+			    } 
+			    p = skip_to_next(p);
+			}
+			p = skip_to_next(p);
 		    }
 		    free(tmp);
 		}
@@ -868,6 +877,66 @@ cmplx *gretl_xml_get_cmplx_array (xmlNodePtr node, xmlDocPtr doc,
 			       nelem, err);
 }
 
+/**
+ * gretl_xml_get_strings_array:
+ * @node: XML node pointer.
+ * @doc: XML document pointer.
+ * @nelem: location to receive number of elements in array.
+ * @err: location to receive error code.
+ * 
+ * Returns: allocated array of cmplx (complex numbers) read from 
+ * @node, or %NULL on failure.
+ */
+
+char **gretl_xml_get_strings_array (xmlNodePtr node, xmlDocPtr doc,
+				    int *nelem, int *err)
+{
+    xmlChar *tmp = xmlGetProp(node, (XUC) "count");
+    char **S = NULL;
+    const char *p;
+    int i, n = 0;
+
+    if (tmp == NULL) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    n = atoi((const char *) tmp);
+    free(tmp);
+
+    if (n > 0) {
+	S = strings_array_new(n);
+	if (S == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    tmp = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+	    if (tmp == NULL) {
+		*err = E_DATA;
+	    } else {
+		p = (const char *) tmp;
+		for (i=0; i<n; i++) {
+		    S[i] = gretl_word_strdup(p, &p);
+		    if (S[i] == NULL) {
+			*err = E_ALLOC;
+		    }
+		}
+		free(tmp);
+	    }
+	}
+    }
+
+    if (S != NULL && *err) {
+	free_strings_array(S, n);
+	S = NULL;
+    }
+
+    if (!*err) {
+	*nelem = n;
+    }
+
+    return S;
+}
+
 /**
  * gretl_xml_get_matrix:
  * @node: XML node pointer.
@@ -1080,6 +1149,115 @@ static int query_print_panel_obs (const DATAINFO *pdinfo)
     return !bal;
 }
 
+/**
+ * gretl_write_matrix_as_gdt:
+ * @fname: name of file to write.
+ * @X: matrix, variable in columns.
+ * @varnames: column names.
+ * @labels: descriptive labels for the variables, or %NULL.
+ * 
+ * Write out a .gdt data file containing the elements of
+ * of the given matrix.
+ * 
+ * Returns: 0 on successful completion, non-zero on error.
+ */
+
+int gretl_write_matrix_as_gdt (const char *fname, 
+			       const gretl_matrix *X,
+			       const char **varnames, 
+			       const char **labels)
+{
+    gzFile *fz = Z_NULL;
+    char datname[MAXLEN];
+    void *handle = NULL;
+    char *xmlbuf = NULL;
+    int (*show_progress) (long, long, int) = NULL;
+    long sz = 0L;
+    int T = X->rows;
+    int k = X->cols;
+    int i, t, err = 0;
+
+    fz = gretl_gzopen(fname, "wb");
+
+    if (fz == Z_NULL) {
+	sprintf(gretl_errmsg, _("Couldn't open %s for writing"), fname);
+	return 1;
+    }
+
+    sz = (T * k * sizeof(double));
+    if (sz > 100000) {
+	fprintf(stderr, I_("Writing %ld Kbytes of data\n"), sz / 1024);
+    } else {
+	sz = 0L;
+    }
+
+    if (sz) {
+	show_progress = get_plugin_function("show_progress", &handle);
+	if (show_progress == NULL) {
+	    sz = 0L;
+	}
+    }
+
+    if (sz) (*show_progress)(0, sz, SP_SAVE_INIT); 
+
+    simple_fname(datname, fname);
+    xmlbuf = gretl_xml_encode(datname);
+    if (xmlbuf == NULL) {
+	err = 1;
+	goto cleanup;
+    }
+
+    gzprintf(fz, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+	     "<!DOCTYPE gretldata SYSTEM \"gretldata.dtd\">\n\n"
+	     "<gretldata name=\"%s\" frequency=\"1\" "
+	     "startobs=\"1\" endobs=\"%d\" type=\"cross-section\">\n", 
+	     datname, T);
+
+    free(xmlbuf);
+
+    gretl_push_c_numeric_locale();
+
+    gzprintf(fz, "<variables count=\"%d\">\n", k);
+
+    for (i=0; i<k; i++) {
+	gzprintf(fz, "<variable name=\"%s\"", varnames[i]);
+	if (labels != NULL && labels[i] != NULL) {
+	    gzprintf(fz, "\n label=\"%s\"", labels[i]);
+	}
+	gzputs(fz, "\n/>\n");
+    }
+
+    gzputs(fz, "</variables>\n");
+
+    gzprintf(fz, "<observations count=\"%d\" labels=\"false\">\n", T);
+
+    for (t=0; t<T; t++) {
+	gzputs(fz, "<obs>");
+	for (i=0; i<k; i++) {
+	    gzprintf(fz, "%.12g ", gretl_matrix_get(X, t, i));
+	}
+	gzputs(fz, "</obs>\n");
+	if (sz && t && (t % 50 == 0)) { 
+	    (*show_progress) (50, T, SP_NONE);
+	}
+    }
+
+    gzputs(fz, "</observations>\n</gretldata>\n");
+
+ cleanup: 
+
+    gretl_pop_c_numeric_locale();
+
+    if (sz) {
+	(*show_progress)(0, T, SP_FINISH);
+	close_plugin(handle);
+    } 
+
+    gzclose(fz);
+
+    return err;
+}
+
 /**
  * gretl_write_gdt:
  * @fname: name of file to write.
@@ -1108,11 +1286,12 @@ int gretl_write_gdt (const char *fname, const int *list,
     char startdate[OBSLEN], enddate[OBSLEN];
     char datname[MAXLEN], freqstr[32];
     char numstr[128];
-    char *xmlbuf = NULL;
+    char xmlbuf[256];
     void *handle = NULL;
     int (*show_progress) (long, long, int) = NULL;
     long sz = 0L;
     int i, t, v, nvars;
+    int uerr = 0;
     int err = 0;
 
     if (gz) {
@@ -1173,10 +1352,9 @@ int gretl_write_gdt (const char *fname, const int *list,
     ntodate_full(enddate, pdinfo->t2, pdinfo);
 
     simple_fname(datname, fname);
-    xmlbuf = gretl_xml_encode(datname);
-    if (xmlbuf == NULL) {
-	err = 1;
-	goto cleanup;
+    uerr = gretl_xml_encode_to_buf(xmlbuf, datname, sizeof xmlbuf);
+    if (uerr) {
+	strcpy(xmlbuf, "unknown");
     }
 
     if (custom_time_series(pdinfo)) {
@@ -1190,17 +1368,15 @@ int gretl_write_gdt (const char *fname, const int *list,
 		 "<!DOCTYPE gretldata SYSTEM \"gretldata.dtd\">\n\n"
 		 "<gretldata name=\"%s\" frequency=\"%s\" "
 		 "startobs=\"%s\" endobs=\"%s\" ", 
-		 datname, freqstr, startdate, enddate);
+		 xmlbuf, freqstr, startdate, enddate);
     } else {
 	fprintf(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
 		"<!DOCTYPE gretldata SYSTEM \"gretldata.dtd\">\n\n"
 		"<gretldata name=\"%s\" frequency=\"%s\" "
 		"startobs=\"%s\" endobs=\"%s\" ", 
-		datname, freqstr, startdate, enddate);
+		xmlbuf, freqstr, startdate, enddate);
     }
 
-    free(xmlbuf);
-
     if (gz) {
 	gzprintf(fz, "type=\"%s\">\n", data_structure_string(pdinfo->structure));
     } else {
@@ -1209,22 +1385,20 @@ int gretl_write_gdt (const char *fname, const int *list,
 
     /* deal with description, if any */
     if (pdinfo->descrip != NULL) {
-	xmlbuf = gretl_xml_encode(pdinfo->descrip);
-	if (xmlbuf == NULL) {
+	char *dbuf = gretl_xml_encode(pdinfo->descrip);
+
+	if (dbuf == NULL) {
 	    err = 1;
 	    goto cleanup;
 	} else {
 	    if (gz) {
 		gzputs(fz, "<description>");
-		gzputs(fz, xmlbuf);
+		gzputs(fz, dbuf);
 		gzputs(fz, "</description>\n");
 	    } else {
-		fprintf(fp, "<description>%s</description>\n", xmlbuf);
+		fprintf(fp, "<description>%s</description>\n", dbuf);
 	    }
-	    free(xmlbuf);
-#ifdef XML_DEBUG
-	    fprintf(stderr, "xmlbuf encoded buffer freed\n");
-#endif
+	    free(dbuf);
 	}
     }
 
@@ -1239,18 +1413,12 @@ int gretl_write_gdt (const char *fname, const int *list,
 
     for (i=1; i<=nvars; i++) {
 	v = savenum(list, i);
-	xmlbuf = gretl_xml_encode(pdinfo->varname[v]);
+	gretl_xml_encode_to_buf(xmlbuf, pdinfo->varname[v], sizeof xmlbuf);
 
-	if (xmlbuf == NULL) {
-	    err = 1;
-	    goto cleanup;
+	if (gz) {
+	    gzprintf(fz, "<variable name=\"%s\"", xmlbuf);
 	} else {
-	    if (gz) {
-		gzprintf(fz, "<variable name=\"%s\"", xmlbuf);
-	    } else {
-		fprintf(fp, "<variable name=\"%s\"", xmlbuf);
-	    }
-	    free(xmlbuf);
+	    fprintf(fp, "<variable name=\"%s\"", xmlbuf);
 	}
 
 	if (var_is_scalar(pdinfo, v) && !na(Z[v][0])) {
@@ -1265,32 +1433,24 @@ int gretl_write_gdt (const char *fname, const int *list,
 	}
 
 	if (*VARLABEL(pdinfo, v)) {
-	    xmlbuf = gretl_xml_encode(VARLABEL(pdinfo, v));
-	    if (xmlbuf == NULL) {
-		err = 1;
-		goto cleanup;
-	    } else {
+	    uerr = gretl_xml_encode_to_buf(xmlbuf, VARLABEL(pdinfo, v), sizeof xmlbuf);
+	    if (!uerr) {
 		if (gz) {
 		    gzprintf(fz, "\n label=\"%s\"", xmlbuf);
 		} else {
 		    fprintf(fp, "\n label=\"%s\"", xmlbuf);
 		}
-		free(xmlbuf);
 	    }
 	} 
 
 	if (*DISPLAYNAME(pdinfo, v)) {
-	    xmlbuf = gretl_xml_encode(DISPLAYNAME(pdinfo, v));
-	    if (xmlbuf == NULL) {
-		err = 1;
-		goto cleanup;
-	    } else {
+	    uerr = gretl_xml_encode_to_buf(xmlbuf, DISPLAYNAME(pdinfo, v), sizeof xmlbuf);
+	    if (!uerr) {
 		if (gz) {
 		    gzprintf(fz, "\n displayname=\"%s\"", xmlbuf);
 		} else {
 		    fprintf(fp, "\n displayname=\"%s\"", xmlbuf);
 		}
-		free(xmlbuf);
 	    }
 	} 
 
@@ -1332,10 +1492,13 @@ int gretl_write_gdt (const char *fname, const int *list,
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	alt_puts("<obs", fp, fz);
 	if (pdinfo->markers && pdinfo->S != NULL) {
-	    if (gz) {
-		gzprintf(fz, " label=\"%s\"", pdinfo->S[t]);
-	    } else {
-		fprintf(fp, " label=\"%s\"", pdinfo->S[t]);
+	    uerr = gretl_xml_encode_to_buf(xmlbuf, pdinfo->S[t], sizeof xmlbuf);
+	    if (!uerr) {
+		if (gz) {
+		    gzprintf(fz, " label=\"%s\"", xmlbuf);
+		} else {
+		    fprintf(fp, " label=\"%s\"", xmlbuf);
+		}
 	    }
 	} 
 	if (panelobs) {
@@ -1460,7 +1623,7 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 	    }
 	    tmp = xmlGetProp(cur, (XUC) "displayname");
 	    if (tmp != NULL) {
-		transcribe_string(DISPLAYNAME(pdinfo, i), (char *) tmp, MAXDISP);
+		var_set_display_name(pdinfo, i, (char *) tmp);
 		free(tmp);
 	    }
 	    tmp = xmlGetProp(cur, (XUC) "compact-method");
@@ -1510,7 +1673,7 @@ static int process_values (double **Z, DATAINFO *pdinfo, int t, char *s)
     double x;
     int i, err = 0;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     for (i=1; i<pdinfo->v && !err; i++) {
 	if (var_is_scalar(pdinfo, i)) {
@@ -1870,8 +2033,8 @@ static void data_read_message (const char *fname, DATAINFO *pdinfo, PRN *prn)
  * @ppdinfo: pointer to data information struct.
  * @fname: name of file to try.
  * @ppaths: path information struct.
- * @data_status: DATA_NONE: no datafile currently open; DATA_CLEAR: datafile
- * is open, should be cleared; DATA_APPEND: add to current dataset.
+ * @ocode: %DATA_NONE: no datafile currently open; %DATA_CLEAR: datafile
+ * is open, should be cleared; %DATA_APPEND: add to current dataset.
  * @prn: where messages should be written.
  * @gui: should = 1 if the function is launched from the GUI, else 0.
  * 
@@ -1882,7 +2045,7 @@ static void data_read_message (const char *fname, DATAINFO *pdinfo, PRN *prn)
  */
 
 int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
-		    PATHS *ppaths, int data_status, PRN *prn, int gui) 
+		    PATHS *ppaths, DataOpenCode ocode, PRN *prn, int gui) 
 {
     DATAINFO *tmpdinfo;
     double **tmpZ = NULL;
@@ -1892,7 +2055,7 @@ int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
     int caldata = 0;
     long fsz, progress = 0L;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     check_for_console(prn);
 
@@ -2012,7 +2175,7 @@ int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
 
     data_read_message(fname, tmpdinfo, prn);
 
-    if (data_status == DATA_APPEND) {
+    if (ocode == DATA_APPEND) {
 	err = merge_data(pZ, *ppdinfo, tmpZ, tmpdinfo, prn);
 	if (err) {
 	    tmpZ = NULL;
@@ -2024,7 +2187,7 @@ int gretl_read_gdt (double ***pZ, DATAINFO **ppdinfo, char *fname,
 	    strcpy(ppaths->datfile, fname);
 	}
 	free_Z(*pZ, *ppdinfo);
-	if (data_status == DATA_CLEAR) {
+	if (ocode == DATA_CLEAR) {
 	    clear_datainfo(*ppdinfo, CLEAR_FULL);
 	}
 	free(*ppdinfo);
@@ -2080,7 +2243,7 @@ char *gretl_get_gdt_description (const char *fname)
     xmlNodePtr cur;
     xmlChar *buf = NULL;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     LIBXML_TEST_VERSION
 	xmlKeepBlanksDefault(0);
@@ -2119,3 +2282,93 @@ char *gretl_get_gdt_description (const char *fname)
     return (char *) buf;
 }
 
+static char *gretl_xml_get_doc_type (const char *fname, int *err)
+{
+    xmlDocPtr doc;
+    xmlNodePtr node;
+    char *ret = NULL;
+
+    doc = gretl_xmlParseFile(fname);
+
+    if (doc == NULL) {
+	sprintf(gretl_errmsg, _("xmlParseFile failed on %s"), fname);
+	*err = 1;
+    } else {
+	node = xmlDocGetRootElement(doc);
+	if (node == NULL) {
+	    sprintf(gretl_errmsg, _("%s: empty document"), fname);
+	    *err = 1;
+	} else {
+	    ret = gretl_strdup((char *) node->name);
+	    if (ret == NULL) {
+		*err = 1;
+	    }
+	}
+    }
+
+    if (doc != NULL) {
+	xmlFreeDoc(doc);
+	xmlCleanupParser();
+    }
+
+    return ret;
+}
+
+int load_user_matrix_file (const char *fname) 
+{
+    xmlDocPtr doc = NULL;
+    xmlNodePtr cur = NULL;
+    gretl_matrix *m;
+    char *name;
+    int err = 0;
+
+    xmlKeepBlanksDefault(0);
+
+    err = gretl_xml_open_doc_root(fname, "gretl-matrices", &doc, &cur);
+    if (err) {
+	return err;
+    }
+
+    cur = cur->xmlChildrenNode;
+    while (cur != NULL && !err) {
+        if (!xmlStrcmp(cur->name, (XUC) "gretl-matrix")) {
+	    name = (char *) xmlGetProp(cur, (XUC) "name");
+	    if (name == NULL) {
+		err = 1;
+	    } else {
+		m = gretl_xml_get_matrix(cur, doc, &err);
+		if (m != NULL) {
+		    err = user_matrix_add(m, name);
+		}
+		free(name);
+	    }
+	}
+	cur = cur->next;
+    }
+
+    if (doc != NULL) {
+	xmlFreeDoc(doc);
+	xmlCleanupParser();
+    }
+
+    return err;
+}
+
+int load_user_XML_file (const char *fname)
+{
+    char *rootname = NULL;
+    int err = 0;
+
+    rootname = gretl_xml_get_doc_type(fname, &err);
+
+    if (!strcmp(rootname, "gretl-functions")) {
+	err = load_user_function_file(fname);
+    } else if (!strcmp(rootname, "gretl-matrices")) {
+	err = load_user_matrix_file(fname);
+    }
+
+    free(rootname);
+
+    return err;
+}
+
