@@ -26,23 +26,25 @@
 #define CMD_NULL    -1
 #define CMD_COMMENT -2
 
-typedef struct _CMD CMD;
+typedef struct CMD_ CMD;
+typedef struct ExecState_ ExecState;
 
 typedef enum {
     CMD_BATCH_MODE     = 1 << 0,
-    CMD_ECHO_TO_STDOUT = 1 << 1,
-    CMD_STACKING       = 1 << 2
+    CMD_STACKING       = 1 << 1,
+    CMD_RECORDING      = 1 << 2,
+    CMD_CLI            = 1 << 3
 } CmdEchoFlags;
 
 typedef enum {
-    OPT_BATCH = 1,
-    OPT_HELP,
-    OPT_PVALS,
-    OPT_VERSION,
-    OPT_RUNIT,
-    OPT_DBOPEN,
-    OPT_WEBDB,
-    OPT_DUMP
+    OPT_BATCH   = 1 << 0,
+    OPT_HELP    = 1 << 1,
+    OPT_VERSION = 1 << 2,
+    OPT_RUNIT   = 1 << 3,
+    OPT_DBOPEN  = 1 << 4,
+    OPT_WEBDB   = 1 << 5,
+    OPT_DUMP    = 1 << 6,
+    OPT_DEBUG   = 1 << 7
 } ProgramOptions;
 
 typedef enum {
@@ -50,8 +52,17 @@ typedef enum {
     BASQUE
 } ForcedLangs;
 
-#define HIDDEN_COMMAND(c) (c == FNCALL || \
+typedef enum {
+    CONSOLE_EXEC      = 1 << 0,
+    SCRIPT_EXEC       = 1 << 1,
+    SESSION_EXEC      = 1 << 2,
+    INCLUDE_EXEC      = 1 << 3,
+    FUNCTION_EXEC     = 1 << 4
+} ExecFlags;
+
+#define HIDDEN_COMMAND(c) (c == ADDTO || \
                            c == FUNCERR || \
+                           c == OMITFROM || \
                            c == REMEMBER)
     
 /* functions follow */
@@ -81,7 +92,8 @@ int get_command_index (char *line, CMD *cmd, const DATAINFO *pdinfo);
 
 int command_number (const char *cmd);
 
-int help (const char *cmdword, const char *helpfile, PRN *prn);
+int cli_help (const char *cmdword, const char *helpfile, 
+	      int locale, PRN *prn);
 
 int parseopt (const char **argv, int argc, char *fname, 
 	      int *force_lang);
@@ -93,9 +105,9 @@ void echo_cmd (const CMD *cmd, const DATAINFO *pdinfo, const char *line,
 
 void echo_function_call (const char *line, unsigned char flags, PRN *prn);
 
-int simple_commands (CMD *cmd, const char *line, 
-		     double ***pZ, DATAINFO *pdinfo,
-		     PRN *prn);
+void safe_print_line (const char *line, int *plen, PRN *prn);
+
+int gretl_cmd_exec (ExecState *s, double ***pZ, DATAINFO **ppdinfo);
 
 int call_pca_plugin (VMatrix *corrmat, double ***pZ,
 		     DATAINFO *pdinfo, gretlopt *pflag,
@@ -103,8 +115,6 @@ int call_pca_plugin (VMatrix *corrmat, double ***pZ,
 
 int ready_for_command (const char *line);
 
-void safe_print_line (const char *line, PRN *prn);
-
 #endif /* INTERACT_H */
 
 
