@@ -1,5 +1,7 @@
 /* bootstrapped confidence intervals for impulse response functions */
 
+#include "matrix_extra.h"
+
 #if BDEBUG
 # define BOOT_ITERS 5
 #else
@@ -206,33 +208,19 @@ static int irf_boot_init (irfboot *boot, const GRETL_VAR *var,
 	    goto bailout;
 	}
 	boot->E = gretl_matrix_alloc(boot->n, boot->neqns);
-	if (boot->E == NULL) {
-	    err = E_ALLOC;
-	    goto bailout;
-	}
 	boot->S = gretl_matrix_alloc(boot->neqns, boot->neqns);
-	if (boot->S == NULL) {
+	if (boot->E == NULL || boot->S == NULL) {
 	    err = E_ALLOC;
 	    goto bailout;
-	}
+	}	    
     }
 
     boot->rE = gretl_matrix_alloc(boot->n, boot->neqns);
-    if (boot->rE == NULL) {
-	err = E_ALLOC;
-	goto bailout;
-    }
-
     boot->resp = gretl_matrix_alloc(boot->horizon, BOOT_ITERS);
-    if (boot->resp == NULL) {
-	err = E_ALLOC;
-	goto bailout;
-    }
-
     boot->sample = malloc(boot->n * sizeof *boot->sample);
-    if (boot->sample == NULL) {
+    if (boot->rE == NULL || boot->resp == NULL || 
+	boot->sample == NULL) {
 	err = E_ALLOC;
-	goto bailout;
     }
 
  bailout:
@@ -314,7 +302,7 @@ re_estimate_VECM (irfboot *boot, GRETL_VAR *jvar, int targ, int shock,
     static void *handle = NULL;
     int err = 0;
 
-    *gretl_errmsg = 0;
+    gretl_error_clear();
 
     if (iter == 0) {
 	/* first round: open the Johansen plugin */
@@ -373,7 +361,7 @@ static int re_estimate_VAR (irfboot *boot, int targ, int shock, int iter)
     if (!err) {    
 	gretl_matrix_multiply_mod(boot->E, GRETL_MOD_TRANSPOSE,
 				  boot->E, GRETL_MOD_NONE,
-				  boot->S);
+				  boot->S, GRETL_MOD_NONE);
 	gretl_matrix_divide_by_scalar(boot->S, boot->n);
 	err = gretl_VAR_do_error_decomp(boot->S, boot->C);
     }
