@@ -73,7 +73,6 @@ struct kalman_ {
     gretl_matrix *VE;
     gretl_matrix *PHV;
     gretl_matrix *Ax;
-    gretl_matrix *Tmprn;
     gretl_matrix *Tmpnn;
     gretl_matrix *Tmprr;
     gretl_matrix *Tmprr_2a;
@@ -102,7 +101,6 @@ void kalman_free (kalman *K)
     gretl_matrix_free(K->PHV);
     gretl_matrix_free(K->Ax);
 
-    gretl_matrix_free(K->Tmprn);
     gretl_matrix_free(K->Tmpnn);
     gretl_matrix_free(K->Tmprr);
     gretl_matrix_free(K->Tmprr_2a);
@@ -313,7 +311,6 @@ kalman *kalman_new (const gretl_matrix *S, const gretl_matrix *P,
     K->PHV = NULL;
     K->Ax = NULL;
 
-    K->Tmprn = NULL;
     K->Tmpnn = NULL;
     K->Tmprr = NULL;
     K->Tmprr_2a = NULL;
@@ -370,7 +367,6 @@ kalman *kalman_new (const gretl_matrix *S, const gretl_matrix *P,
     K->Ax = gretl_matrix_alloc(K->n, 1);
 
     /* will hold various intermediate products */
-    K->Tmprn = gretl_matrix_alloc(K->r, K->n);
     K->Tmpnn = gretl_matrix_alloc(K->n, K->n);
     K->Tmprr = gretl_matrix_alloc(K->r, K->r);
     K->Tmprr_2a = gretl_matrix_alloc(K->r, K->r);
@@ -382,8 +378,8 @@ kalman *kalman_new (const gretl_matrix *S, const gretl_matrix *P,
 	K->PH == NULL || K->HPH == NULL ||
 	K->FPH == NULL || K->V == NULL || K->VE == NULL ||
 	K->PHV == NULL || K->Ax == NULL ||
-	K->Tmprn == NULL || K->Tmpnn == NULL ||
-	K->Tmprr == NULL || K->Tmprr_2a == NULL || K->Tmprr_2b == NULL) {
+	K->Tmpnn == NULL || K->Tmprr == NULL || 
+	K->Tmprr_2a == NULL || K->Tmprr_2b == NULL) {
 	*err = E_ALLOC;
 	kalman_free(K);
 	K = NULL;
@@ -410,76 +406,6 @@ matrix_diff (const gretl_matrix *a, const gretl_matrix *b)
     return 0;
 }
 
-/* matrix multiplication, a * b, with no checks */
-
-static void fast_multiply (const gretl_matrix *a, const gretl_matrix *b,
-			   gretl_matrix *c)
-{
-    double x;
-    int i, j, k;
-    int aidx, bidx;
-
-    for (i=0; i<a->rows; i++) {
-	for (j=0; j<b->cols; j++) {
-	    x = 0.0;
-	    aidx = i;
-	    bidx = j * b->rows;
-	    for (k=0; k<a->cols; k++) {
-		x += a->val[aidx] * b->val[bidx];
-		aidx += a->rows;
-		bidx++;
-	    }
-	    c->val[mdx(c,i,j)] = x;
-	}
-    }
-}
-
-/* matrix multiplication, a' * b, with no checks */
-
-static void fast_A_prime_B (const gretl_matrix *a, const gretl_matrix *b,
-			    gretl_matrix *c)
-{
-    double x;
-    int i, j, k;
-    int aidx, bidx;
-
-    for (i=0; i<a->cols; i++) {
-	for (j=0; j<b->cols; j++) {
-	    x = 0.0;
-	    aidx = i * a->rows;
-	    bidx = j * b->rows;
-	    for (k=0; k<a->rows; k++) {
-		x += a->val[aidx++] * b->val[bidx++];
-	    }
-	    c->val[mdx(c,i,j)] = x;
-	}
-    }
-}
-
-/* matrix multiplication, a * b', with no checks */
-
-static void fast_A_B_prime (const gretl_matrix *a, const gretl_matrix *b,
-			    gretl_matrix *c)
-{
-    double x;
-    int i, j, k;
-    int aidx, bidx;
-
-    for (i=0; i<a->rows; i++) {
-	for (j=0; j<b->rows; j++) {
-	    x = 0.0;
-	    aidx = i;
-	    bidx = j;
-	    for (k=0; k<a->cols; k++) {
-		x += a->val[aidx] * b->val[bidx];
-		aidx += a->rows;
-		bidx += b->rows;
-	    }
-	    c->val[mdx(c,i,j)] = x;
-	}
-    }
-}
-
 /* below: if postmult is non-zero, we're post-multiplying by the
    transpose of F */
 
@@ -489,16 +415,14 @@ static int multiply_by_F (kalman *K, const gretl_matrix *A,
     int ret = 0;
 
     if (gretl_is_zero_matrix(A)) {
-	int i, n = B->rows * B->cols;
-
-	for (i=0; i<n; i++) {
-	    B->val[i] = 0.0;
-	}
+	gretl_matrix_zero(B);
     } else if (K->nonshift == K->r) {
 	if (postmult) {
-	    fast_A_B_prime(A, K->F, B);
+	    gretl_matrix_multiply_mod(A, GRETL_MOD_NONE,
+				      K->F, GRETL_MOD_TRANSPOSE,
+				      B, GRETL_MOD_NONE);
 	} else {
-	    fast_multiply(K->F, A, B);
+	    gretl_matrix_multiply(K->F, A, B);
 	}
     } else { 
 	int i, j, c;
@@ -506,6 +430,7 @@ static int multiply_by_F (kalman *K, const gretl_matrix *A,
 	int r2 = K->r - r1;
 	gretl_matrix *topF = K->Tmprr_2a;
 	gretl_matrix *top = K->Tmprr_2b;
+	double x;
 
 	if (postmult) {
 	    /* if post-multiplying by F', "top" actually means "left" */
@@ -521,24 +446,24 @@ static int multiply_by_F (kalman *K, const gretl_matrix *A,
 	    /* copy from F to topF */
 	    for (i=0; i<r1; i++) {
 		for (j=0; j<K->r; j++) {
-		    topF->val[mdx(topF, j, i)] = 
-			K->F->val[mdx(K->F, i, j)];
+		    x = gretl_matrix_get(K->F, i, j);
+		    gretl_matrix_set(topF, j, i, x);
 		}
 	    }
 
-	    fast_multiply(A, topF, top);
+	    gretl_matrix_multiply(A, topF, top);
 
 	    for (i=0; i<c; i++) {
 		for (j=0; j<r1; j++) {
-		    B->val[mdx(B, i, j)] = 
-			top->val[mdx(top, i, j)];
+		    x = gretl_matrix_get(top, i, j);
+		    gretl_matrix_set(B, i, j, x);
 		}
 	    }
 
 	    for (i=0; i<c; i++) {
 		for (j=0; j<r2; j++) {
-		    B->val[mdx(B, i, j + r1)] = 
-			A->val[mdx(A, i, j)];
+		    x = gretl_matrix_get(A, i, j);
+		    gretl_matrix_set(B, i, j + r1, x);
 		}
 	    }
 
@@ -555,24 +480,24 @@ static int multiply_by_F (kalman *K, const gretl_matrix *A,
 
 	    for (i=0; i<r1; i++) {
 		for (j=0; j<K->r; j++) {
-		    topF->val[mdx(topF, i, j)] = 
-			K->F->val[mdx(K->F, i, j)];
+		    x = gretl_matrix_get(K->F, i, j);
+		    gretl_matrix_set(topF, i, j, x);
 		}
 	    }
 
-	    fast_multiply(topF, A, top);
-	    
+	    gretl_matrix_multiply(topF, A, top);
+
 	    for (i=0; i<r1; i++) {
 		for (j=0; j<c; j++) {
-		    B->val[mdx(B, i, j)] =
-			top->val[mdx(top, i, j)];
+		    x = gretl_matrix_get(top, i, j);
+		    gretl_matrix_set(B, i, j, x);
 		}
 	    }
 
 	    for (i=0; i<r2; i++) {
 		for (j=0; j<c; j++) {
-		    B->val[mdx(B, i + r1, j)] =
-			A->val[mdx(A, i, j)];
+		    x = gretl_matrix_get(A, i, j);
+		    gretl_matrix_set(B, i + r1, j, x);
 		}
 	    }
 	}
@@ -631,15 +556,18 @@ static int kalman_iter_1 (kalman *K, double *llt)
 
     /* form E = y - A'x - H'S */
     err += gretl_matrix_subtract_from(K->e, K->Ax);
-    fast_A_prime_B(K->H, K->S0, K->Tmpnn);
-    err += gretl_matrix_subtract_from(K->e, K->Tmpnn);
+    gretl_matrix_multiply_mod(K->H, GRETL_MOD_TRANSPOSE,
+			      K->S0, GRETL_MOD_NONE,
+			      K->e, GRETL_MOD_DECUMULATE);
     
     /* form (H'PH + R)^{-1} * (y - Ax - H'S) = "VE" */
-    fast_multiply(K->V, K->e, K->VE);
+    gretl_matrix_multiply(K->V, K->e, K->VE);
 
     if (llt != NULL) {
 	/* form (y - Ax - H'S)' * (H'PH + R)^{-1} * (y - Ax - H'S) */
-	fast_A_prime_B(K->e, K->VE, K->Tmpnn);
+	gretl_matrix_multiply_mod(K->e, GRETL_MOD_TRANSPOSE,
+				  K->VE, GRETL_MOD_NONE,
+				  K->Tmpnn, GRETL_MOD_NONE);
 
 	/* contribution to log-likelihood of the above -- see Hamilton
 	   (1994) equation [13.4.1] page 385.
@@ -650,11 +578,11 @@ static int kalman_iter_1 (kalman *K, double *llt)
     /* form FPH */
     err += multiply_by_F(K, K->PH, K->FPH, 0);
 
-    /* form FPH * (H'PH + R)^{-1} * (y - A'x - H'S) */
-    fast_multiply(K->FPH, K->VE, K->Tmprn);
-
-    /* complete calculation of S+ */
-    err += gretl_matrix_add_to(K->S1, K->Tmprn);
+    /* form FPH * (H'PH + R)^{-1} * (y - A'x - H'S) 
+       and add to S+ */
+    gretl_matrix_multiply_mod(K->FPH, GRETL_MOD_NONE,
+			      K->VE, GRETL_MOD_NONE,
+			      K->S1, GRETL_MOD_CUMULATE);
 
     return err;
 }
@@ -666,31 +594,25 @@ static int kalman_iter_1 (kalman *K, double *llt)
 
 static int kalman_iter_2 (kalman *K)
 {
-    gretl_matrix *PHV = K->Tmprn;
-    gretl_matrix *HP;
     int err = 0;
 
     /* form P - PH(H'PH + R)^{-1}H'P */
-    fast_multiply(K->PH, K->V, PHV);
-    HP = gretl_matrix_reuse(K->PH, K->n, K->r);
-    fast_A_prime_B(K->H, K->P0, HP);
-    fast_multiply(PHV, HP, K->Tmprr);
-    gretl_matrix_subtract_from(K->P0, K->Tmprr);
+    err += gretl_matrix_qform(K->PH, GRETL_MOD_NONE, K->V,
+			      K->P0, GRETL_MOD_DECUMULATE);
 
     /* pre-multiply by F, post-multiply by F' */
-    err += multiply_by_F(K, K->P0, K->Tmprr, 0);
-    err += multiply_by_F(K, K->Tmprr, K->P1, 1);
-
-    /* add Q */
-    if (arma_ll(K)) {
-	K->P1->val[0] += K->Q->val[0];
+    if (K->nonshift == K->r) {
+	gretl_matrix_qform(K->F, GRETL_MOD_NONE, K->P0,
+			   K->P1, GRETL_MOD_NONE);
     } else {
-	err += gretl_matrix_add_to(K->P1, K->Q);
+	err += multiply_by_F(K, K->P0, K->Tmprr, 0);
+	err += multiply_by_F(K, K->Tmprr, K->P1, 1);
+	gretl_matrix_xtr_symmetric(K->P1);
     }
 
-    /* put K->PH back the way we found it */
-    gretl_matrix_reuse(K->PH, K->r, K->n);
-    
+    /* add Q */
+    err += gretl_matrix_add_to(K->P1, K->Q);
+
     return err;
 }
 
@@ -740,11 +662,11 @@ static void kalman_set_Ax (kalman *K, int t)
 	axi = 0.0;
 	/* case j == 0 */
 	if (K->ifc) {
-	    axi += K->A->val[mdx(K->A, 0, i)]; /* \times 1.0, implicitly */
+	    axi += gretl_matrix_get(K->A, 0, i); /* \times 1.0, implicitly */
 	}
 	for (j=1; j<K->k; j++) {
-	    aji = K->A->val[mdx(K->A, j, i)];
-	    xjt = K->x->val[mdx(K->x, t, j - 1)];
+	    aji = gretl_matrix_get(K->A, j, i);
+	    xjt = gretl_matrix_get(K->x, t, j - 1);
 	    axi += aji * xjt;
 	}
 	gretl_vector_set(K->Ax, i, axi);
@@ -761,7 +683,7 @@ kalman_initialize_error (kalman *K, int t)
     int i;
 
     for (i=0; i<K->n; i++) {
-	K->e->val[i] = K->y->val[mdx(K->y, t, i)];
+	K->e->val[i] = gretl_matrix_get(K->y, t, i);
     }
 }
 
@@ -772,10 +694,12 @@ kalman_initialize_error (kalman *K, int t)
 static void
 kalman_record_error (kalman *K, int t)
 {
+    double eti;
     int i;
 
     for (i=0; i<K->n; i++) {
-	K->E->val[mdx(K->E, t, i)] = K->e->val[i]; /* K->e is a row vector */
+	eti = gretl_vector_get(K->e, i); /* K->e is a row vector */
+	gretl_matrix_set(K->E, t, i, eti);
     }
 }
 
@@ -786,8 +710,7 @@ static int kalman_incr_S (kalman *K)
     double x;
     int err = 0;
 
-    x = gretl_scalar_b_X_b(K->e, GRETL_MOD_TRANSPOSE,
-			   K->V, &err);
+    x = gretl_scalar_qform(K->e, K->V, &err);
     if (!err) {
 	K->SSRw += x;
     }
@@ -855,7 +778,7 @@ int kalman_forecast (kalman *K)
 	}	
 
 	/* initial matrix calculations */
-	fast_multiply(K->P0, K->H, K->PH);
+	gretl_matrix_multiply(K->P0, K->H, K->PH);
 	if (arma_ll(K)) {
 	    K->HPH->val[0] = 0.0;
 	    for (i=0; i<K->r; i++) {
@@ -864,7 +787,9 @@ int kalman_forecast (kalman *K)
 	    ldet = log(K->HPH->val[0]);
 	    K->V->val[0] = 1.0 / K->HPH->val[0];
 	} else {
-	    fast_A_prime_B(K->H, K->PH, K->HPH);
+	    gretl_matrix_multiply_mod(K->H, GRETL_MOD_TRANSPOSE,
+				      K->PH, GRETL_MOD_NONE,
+				      K->HPH, GRETL_MOD_NONE);
 	    if (K->R != NULL) {
 		gretl_matrix_add_to(K->HPH, K->R);
 	    }
