@@ -48,9 +48,11 @@
 #endif
 
 #ifdef ENABLE_NLS
+# define G_(String) gp_gettext (String)
 # define I_(String) iso_gettext (String) 
 # define M_(String) maybe_iso_gettext (String)
 #else
+# define G_(String) String
 # define I_(String) String
 # define M_(String) String
 #endif /* ENABLE_NLS */
@@ -77,7 +79,11 @@
 #define OBSLEN    16  /* space allocated for obs strings (including termination) */
 
 #ifndef M_PI
-# define M_PI 3.14159265358979323846
+# define M_PI 3.1415926535897932384626432
+#endif
+
+#ifndef M_2PI
+# define M_2PI 6.2831853071795864769252864
 #endif
 
 #define LN_2_PI       1.837877066409345
@@ -89,6 +95,19 @@
 /* numbers smaller than the given limit will print as zero */
 #define screen_zero(x)  ((fabs(x) > 1.0e-13)? x : 0.0)
 
+typedef enum {
+    ARG_NONE = 0,
+    ARG_SCALAR,
+    ARG_SERIES,
+    ARG_LIST,
+    ARG_MATRIX,
+    ARG_BOOL,
+    ARG_INT,
+    ARG_REF_SCALAR,
+    ARG_REF_SERIES,
+    ARG_REF_MATRIX
+} GretlArgType;
+
 enum ts_codes {
     CROSS_SECTION,
     TIME_SERIES,
@@ -171,12 +190,7 @@ typedef enum {
     OP_LT  = '<',
     OP_NEQ = 21,
     OP_GTE = 22,
-    OP_LTE = 23,
-    /* matrix operators */
-    OP_DOTMULT = 24,
-    OP_DOTDIV  = 25,
-    OP_DOTPOW  = 26,
-    OP_KRON    = 27,
+    OP_LTE = 23
 } GretlOp;
 
 typedef enum {
@@ -187,29 +201,11 @@ typedef enum {
 } DistCode;
 
 typedef enum {
-    T_NONE,
-    T_LOG,
-    T_EXP, 
-    T_SIN, 
-    T_COS,
-    T_TAN,
-    T_ATAN,
-    T_INT, 
-    T_ABS, 
-    T_SQRT, 
-    T_DNORM,
-    T_CNORM,
-    T_QNORM,
-    T_GAMMA,
-    T_LNGAMMA,
-    T_UNIFORM, 
-    T_NORMAL, 
-    T_CHISQ, 
-    T_STUDENT,
-    T_LOG10,
-    T_LOG2,
-    T_MATHMAX
-} GretlMathFunc;
+    C_AIC,
+    C_BIC,
+    C_HQC,
+    C_MAX
+} ModelSelCriteria;
 
 #ifndef CMPLX
 typedef struct _cmplx cmplx;
@@ -236,12 +232,13 @@ typedef struct GRETL_VAR_ GRETL_VAR;
 
 typedef struct model_data_item_ model_data_item;
 typedef struct ModelTest_ ModelTest;
-typedef struct _gretl_equation_system gretl_equation_system;
+typedef struct gretl_equation_system_ gretl_equation_system;
 
 /* information on individual variable */
 struct VARINFO_ {
     char label[MAXLABEL];
     char display_name[MAXDISP];
+    char gp_display_name[MAXDISP];
     int flags;
     char compact_method;
     char stack_level;
@@ -305,6 +302,8 @@ struct PATHS_ {
     char gnuplot[MAXLEN];
     char x12a[MAXLEN];
     char x12adir[MAXLEN];
+    char tramo[MAXLEN];
+    char tramodir[MAXLEN];
     char dbhost[32];
     char pngfont[128];
 };
@@ -312,7 +311,7 @@ struct PATHS_ {
 struct VMatrix_ {
     int ci;
     int dim;
-    int t1, t2;
+    int t1, t2, n;
     char **names;
     double *vec;
     int *list;
@@ -364,13 +363,14 @@ struct MODEL_ {
     double fstt;                 /* F-statistic */
     double lnL;                  /* log-likelihood */
     double ybar, sdy;            /* mean and std. dev. of dependent var. */
-    double criterion[3];         /* array of model selection statistics */
+    double criterion[C_MAX];     /* array of model selection statistics */
     double dw, rho;              /* Durbin-Watson stat. and estimated 1st
 				    order autocorrelation coefficient */
     ARINFO *arinfo;              /* pointer to struct to hold special info for 
 				    autoregressive model */ 
     int errcode;                 /* Error code in case of failure */
     char *name;                  /* for use in GUI */
+    char *depvar;                /* name of dependent var in special cases */
     int nparams;                 /* number of named model parameters */
     char **params;               /* for named model parameters */
     int ntests;                  /* number of attached test results */
@@ -392,7 +392,8 @@ struct MODEL_ {
 #include "interact.h"
 #include "dataset.h"
 #include "estimate.h"
-#include "generate.h"
+#include "genmain.h"
+#include "genfuncs.h"
 #include "compare.h"
 #include "gretl_intl.h"
 #include "gretl_list.h"
@@ -407,10 +408,8 @@ struct MODEL_ {
 #include "describe.h"
 #include "printout.h"
 #include "modelprint.h"
-#include "texprint.h"
 #include "graphing.h"
 #include "random.h"
-#include "monte_carlo.h"
 #include "nonparam.h"
 #include "options.h"
 #include "discrete.h"
