@@ -23,13 +23,24 @@
 #include "libset.h"
 #include "usermat.h"
 
+#include <unistd.h>
+#include <errno.h>
+
 #define PDEBUG 0
 
 enum {
     AUTO_LAG_STOCK_WATSON,
-    AUTO_LAG_WOOLDRIDGE
+    AUTO_LAG_WOOLDRIDGE,
+    AUTO_LAG_NEWEYWEST
 };
 
+enum {
+    STATE_USE_CWD        = 1 << 0,  /* store: use current dir as default */
+    STATE_ECHO_ON        = 1 << 1,  /* echoing commands or not */
+    STATE_MSGS_ON        = 1 << 2,  /* emitting non-error messages or not */
+    STATE_FORCE_DECPOINT = 1 << 3   /* override locale decimal separator */
+};    
+
 /* for values that really want a non-negative integer */
 #define UNSET_INT -1
 #define is_unset(i) (i == UNSET_INT)
@@ -41,6 +52,10 @@ struct robust_opts {
     int user_lag;
     int hc_version;
     int force_hc;
+    int hkern;
+    int prewhite;
+    int pcse;
+    double qsband;
 };
 
 struct garch_opts {
@@ -53,36 +68,31 @@ struct bkbp_opts {
     int periods[2];
 };
 
-struct sample_info {
-    int t1;
-    int t2;
-    char submode;
-    char *submask;
-};
-    
-struct bhhh_opts {
+struct max_opts {
     double toler;
     int maxiter;
 };
 
 struct set_vars_ {
-    int use_qr;                 /* use QR decomposition? */
+    int flags;
+    int use_qr;                 /* use QR decomposition by default? */
+    int halt_on_err;            /* halt cli program on script error? */
     unsigned int seed;          /* for PRNG */
-    int halt_on_error;          /* halt cli program on script error? */
     int shell_ok;               /* shell commands permitted? */
     double hp_lambda;           /* for Hodrick-Prescott filter */
     int horizon;                /* for VAR impulse responses */ 
+    int bootrep;                /* bootstrap replications */
     double nls_toler;           /* NLS convergence criterion */
-    int gretl_echo;             /* echoing commands or not */
-    int gretl_msgs;             /* emitting non-error messages or not */
     char delim;                 /* delimiter for CSV data export */
     int longdigits;             /* digits for printing data in long form */
+    int max_verbose;            /* verbose output from maximizer? */
     gretl_matrix *initvals;     /* for parameter initialization */
     struct robust_opts ropts;   /* robust standard error options */
     struct garch_opts gopts;    /* GARCH covariance matrix */
     struct bkbp_opts bkopts;    /* Baxter-King filter */
-    struct sample_info sinfo;   /* record of dataset sample state */
-    struct bhhh_opts maxopts;   /* options for BHHH maximisation */
+    struct max_opts bhhh_opts;  /* options for BHHH maximization */
+    struct max_opts bfgs_opts;  /* options for BFGS maximization */
+    char shelldir[MAXLEN];      /* working dir for shell commands */
 };
 
 /* global state */
@@ -91,9 +101,13 @@ set_vars *state;
 static void robust_opts_init (struct robust_opts *opts)
 {
     opts->auto_lag = AUTO_LAG_STOCK_WATSON;
-    opts->user_lag = 0;
+    opts->user_lag = UNSET_INT;
     opts->hc_version = 0;
     opts->force_hc = 0; 
+    opts->hkern = KERNEL_BARTLETT;
+    opts->prewhite = 0;
+    opts->pcse = 0;
+    opts->qsband = NADBL;
 }
 
 static void robust_opts_copy (struct robust_opts *opts)
@@ -102,6 +116,24 @@ static void robust_opts_copy (struct robust_opts *opts)
     opts->user_lag = state->ropts.user_lag;
     opts->hc_version = state->ropts.hc_version;
     opts->force_hc = state->ropts.force_hc; 
+    opts->hkern = state->ropts.hkern; 
+    opts->prewhite = state->ropts.prewhite;
+    opts->pcse = state->ropts.pcse;
+    opts->qsband = state->ropts.qsband;
+}
+
+static const char *hac_kernel_string (void)
+{
+    switch (state->ropts.hkern) {
+    case KERNEL_BARTLETT:
+	return "bartlett";
+    case KERNEL_PARZEN:
+	return "parzen";
+    case KERNEL_QS:
+	return "quadratic-spectral";
+    default:
+	return "";
+    }
 }
 
 static void garch_opts_init (struct garch_opts *opts)
@@ -130,16 +162,22 @@ static void bkbp_opts_copy (struct bkbp_opts *opts)
     opts->periods[1] = state->bkopts.periods[1];
 }
 
-static void bhhh_opts_init (struct bhhh_opts *opts)
+static void max_opts_init (struct max_opts *opts)
 {
     opts->toler = NADBL;
     opts->maxiter = 500;
 }
 
-static void bhhh_opts_copy (struct bhhh_opts *opts)
+static void bhhh_opts_copy (struct max_opts *opts)
 {
-    opts->toler = state->maxopts.toler;
-    opts->maxiter = state->maxopts.maxiter;
+    opts->toler = state->bhhh_opts.toler;
+    opts->maxiter = state->bhhh_opts.maxiter;
+}
+
+static void bfgs_opts_copy (struct max_opts *opts)
+{
+    opts->toler = state->bfgs_opts.toler;
+    opts->maxiter = state->bfgs_opts.maxiter;
 }
 
 static void print_initvals (const gretl_matrix *ivals, PRN *prn)
@@ -151,62 +189,51 @@ static void print_initvals (const gretl_matrix *ivals, PRN *prn)
     }
 }
 
-static void sample_info_init (struct sample_info *sinfo)
-{
-    sinfo->t1 = UNSET_INT;
-    sinfo->t2 = UNSET_INT;
-    sinfo->submask = NULL;
-    sinfo->submode = 0;
-}
-
-#define sinfo_is_set(s) (s.t1 != UNSET_INT && s.t2 != UNSET_INT)
+/* check_for_state() returns non-zero if the program options
+   state is not readable */
 
 static int check_for_state (void) 
 {
-    if (state != NULL) {
+    if (state == NULL) { /* was "!=" ?? */
 	return libset_init();
     } else {
+#if PDEBUG > 1
+	fprintf(stderr, "check_for_state: state = %p\n", (void *) state);
+#endif
 	return 0;
     }
 }
 
-static void state_vars_copy (set_vars *sv, const DATAINFO *pdinfo)
+static int flag_to_bool (set_vars *sv, int flag)
+{
+    return (sv->flags & flag)? 1 : 0;
+}
+
+static void state_vars_copy (set_vars *sv)
 {
 #if PDEBUG
-    fprintf(stderr, "state_vars_copy called: pdinfo=%p\n", (void *) pdinfo);
+    fprintf(stderr, "state_vars_copy() called\n");
 #endif
+    sv->flags = state->flags;
     sv->use_qr = state->use_qr;
+    sv->halt_on_err = state->halt_on_err;
     sv->seed = state->seed;
-    sv->halt_on_error = state->halt_on_error;
     sv->shell_ok = state->shell_ok;
     sv->hp_lambda = state->hp_lambda;
     sv->horizon = state->horizon;
+    sv->bootrep = state->bootrep;
     sv->nls_toler = state->nls_toler;
-    sv->gretl_echo = state->gretl_echo; 
-    sv->gretl_msgs = state->gretl_msgs; 
     sv->delim = state->delim; 
     sv->longdigits = state->longdigits; 
+    sv->max_verbose = state->max_verbose;
     sv->initvals = gretl_matrix_copy(state->initvals);
+    strcpy(sv->shelldir, state->shelldir);
 
     robust_opts_copy(&sv->ropts);
     garch_opts_copy(&sv->gopts);
     bkbp_opts_copy(&sv->bkopts);
-    bhhh_opts_copy(&sv->maxopts);
-
-    if (pdinfo != NULL) {
-	sv->sinfo.t1 = pdinfo->t1;
-	sv->sinfo.t2 = pdinfo->t2;
-	sv->sinfo.submode = pdinfo->submode;
-	sv->sinfo.submask = copy_datainfo_submask(pdinfo);
-#if PDEBUG
-	fprintf(stderr, " sinfo.t1=%d, sinfo.t2=%d, "
-		"sinfo.submode=%d, sinfo.submask=%p\n", sv->sinfo.t1,
-		sv->sinfo.t2, sv->sinfo.submode,
-		(void *) sv->sinfo.submask);
-#endif
-    } else {
-	sample_info_init(&sv->sinfo);
-    }
+    bhhh_opts_copy(&sv->bhhh_opts);
+    bfgs_opts_copy(&sv->bfgs_opts);
 }
 
 static void state_vars_init (set_vars *sv)
@@ -214,24 +241,26 @@ static void state_vars_init (set_vars *sv)
 #if PDEBUG
     fprintf(stderr, "state_vars_init called\n");
 #endif
+    sv->flags = STATE_ECHO_ON | STATE_MSGS_ON;
     sv->use_qr = UNSET_INT; 
     sv->seed = 0;
-    sv->halt_on_error = UNSET_INT;
+    sv->halt_on_err = UNSET_INT;
     sv->shell_ok = 0;
     sv->hp_lambda = NADBL;
     sv->horizon = UNSET_INT;
+    sv->bootrep = 1000;
     sv->nls_toler = NADBL;
-    sv->gretl_echo = 1; 
-    sv->gretl_msgs = 1; 
     sv->delim = UNSET_INT;
     sv->longdigits = 10;
+    sv->max_verbose = 0;
     sv->initvals = NULL;
+    *sv->shelldir = '\0';
 
     robust_opts_init(&sv->ropts);
     garch_opts_init(&sv->gopts);
     bkbp_opts_init(&sv->bkopts);
-    bhhh_opts_init(&sv->maxopts);
-    sample_info_init(&sv->sinfo);
+    max_opts_init(&sv->bhhh_opts);
+    max_opts_init(&sv->bfgs_opts);
 }
 
 int get_hc_version (void)
@@ -362,7 +391,7 @@ double get_bhhh_toler (void)
 	return 1.0;
     }
 
-    return state->maxopts.toler;
+    return state->bhhh_opts.toler;
 }
 
 int get_bhhh_maxiter (void)
@@ -371,10 +400,37 @@ int get_bhhh_maxiter (void)
 	return 0;
     }
 
-    return state->maxopts.maxiter;
+    return state->bhhh_opts.maxiter;
 }
 
-int set_bhhh_toler (double tol)
+double get_bfgs_toler (void)
+{
+    if (check_for_state()) {
+	return 1.0;
+    }
+
+    if (na(state->bfgs_opts.toler)) {
+	state->bfgs_opts.toler = get_default_nls_toler();
+    }
+
+    return state->bfgs_opts.toler;
+}
+
+int get_bfgs_maxiter (void)
+{
+    if (check_for_state()) {
+	return 0;
+    }
+
+    return state->bfgs_opts.maxiter;
+}
+
+enum {
+    BHHH,
+    BFGS
+};
+
+static int real_set_toler (double tol, int c)
 {
     int err = 0;
 
@@ -384,14 +440,26 @@ int set_bhhh_toler (double tol)
 
     if (tol <= 0.0) {
 	err = 1;
+    } else if (c == BFGS) {
+	state->bfgs_opts.toler = tol;
     } else {
-	state->maxopts.toler = tol;
+	state->bhhh_opts.toler = tol;
     }
 
     return err;
 }
 
-int set_bhhh_maxiter (int n)
+int set_bhhh_toler (double tol)
+{
+    return real_set_toler(tol, BHHH);
+}
+
+int set_bfgs_toler (double tol)
+{
+    return real_set_toler(tol, BFGS);
+}
+
+static int real_set_maxiter (int n, int c)
 {
     int err = 0;
 
@@ -401,30 +469,52 @@ int set_bhhh_maxiter (int n)
 
     if (n < 1) {
 	err = 1;
+    } else if (c == BFGS) {
+	state->bfgs_opts.maxiter = n;
     } else {
-	state->maxopts.maxiter = n;
+	state->bhhh_opts.maxiter = n;
     }
 
     return err;
 }
 
+int set_bhhh_maxiter (int n)
+{
+    return real_set_maxiter(n, BHHH);
+}
+
+int set_bfgs_maxiter (int n)
+{
+    return real_set_maxiter(n, BFGS);
+}
+
 int set_long_digits (int n)
 {
-    int err = 0;
-
     if (check_for_state()) {
 	return E_ALLOC;
     }
 
     if (n < 1 || n > 20) {
-	err = 1;
+	return 1;
     } else {
 	state->longdigits = n;
+	return 0;
     }
-
-    return err;
 }
 
+int set_max_verbose (int n)
+{
+    if (check_for_state()) {
+	return E_ALLOC;
+    }
+
+    if (n < 0) {
+	return 1;
+    } else {
+	state->max_verbose = n;
+	return 0;
+    }
+}
 
 int get_VAR_horizon (void)
 {
@@ -435,6 +525,15 @@ int get_VAR_horizon (void)
     }
 }
 
+int get_bootstrap_replications (void)
+{
+    if (check_for_state()) {
+	return 0;
+    } else {
+	return state->bootrep;
+    }
+}
+
 double get_nls_toler (void)
 {
     if (check_for_state()) {
@@ -572,26 +671,36 @@ int get_force_hc (void)
 
 void set_gretl_echo (int e)
 {
-    check_for_state();
-    state->gretl_echo = e;
+    if (check_for_state()) return;
+
+    if (e) {
+	state->flags |= STATE_ECHO_ON;
+    } else {
+	state->flags &= ~STATE_ECHO_ON;
+    }
 }
 
 int gretl_echo_on (void)
 {
-    check_for_state();
-    return state->gretl_echo;
+    if (check_for_state()) return 1;
+    return flag_to_bool(state, STATE_ECHO_ON);
 }
 
 void set_gretl_messages (int e)
 {
-    check_for_state();
-    state->gretl_msgs = e;
+    if (check_for_state()) return;
+
+    if (e) {
+	state->flags |= STATE_MSGS_ON;
+    } else {
+	state->flags &= ~STATE_MSGS_ON;
+    }
 }
 
 int gretl_messages_on (void)
 {
-    check_for_state();
-    return state->gretl_msgs;
+    if (check_for_state()) return 1;
+    return flag_to_bool(state, STATE_MSGS_ON);
 }
 
 char get_csv_delim (const DATAINFO *pdinfo)
@@ -610,37 +719,132 @@ int get_long_digits (void)
     return state->longdigits;
 }
 
+int get_max_verbose (void)
+{
+    check_for_state();
+    return state->max_verbose;
+}
+
 const gretl_matrix *get_init_vals (void)
 {
-    return state->initvals;
-}    
+    const gretl_matrix *m = NULL;
 
-int get_hac_lag (int m)
+    check_for_state();
+    m = state->initvals;
+    state->initvals = NULL;
+
+    return m;
+}   
+
+char *get_shelldir (void)
+{
+    check_for_state();
+
+    if (state != NULL && *state->shelldir != '\0') {
+	return state->shelldir;
+    } else {
+	return NULL;
+    }
+} 
+
+int get_hac_lag (int T)
 {
     check_for_state();
 
     /* Variants of Newey-West */
 
-    if (state->ropts.user_lag != 0 && state->ropts.user_lag < m - 2) {
+    if (state->ropts.user_lag >= 0 && state->ropts.user_lag < T - 2) {
 	/* FIXME upper limit? */
 	return state->ropts.user_lag;
-    }
-
-    if (state->ropts.auto_lag == AUTO_LAG_STOCK_WATSON) {
-	return 0.75 * pow(m, 1.0 / 3.0);
     } else if (state->ropts.auto_lag == AUTO_LAG_WOOLDRIDGE) {
-	return 4.0 * pow(m / 100.0, 2.0 / 9.0);
+	return 4.0 * pow(T / 100.0, 2.0 / 9.0);
+    } else {
+	/* Stock-Watson default */
+	return 0.75 * pow(T, 1.0 / 3.0);
     }
-
-    /* fallback -- should not be reached */
-    return 0.75 * pow(m, 1.0 / 3.0);
 }
 
-static char *get_hac_lag_string (void)
+/* prewhitening implies nw3, but not vice versa */
+
+int data_based_hac_bandwidth (void)
+{
+    if (is_unset(state->ropts.user_lag)) {
+	if (state->ropts.auto_lag == AUTO_LAG_NEWEYWEST ||
+	    state->ropts.prewhite) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+int get_hac_kernel (void)
 {
     check_for_state();
 
-    if (state->ropts.user_lag > 0 && state->ropts.user_lag < 1000) {
+    return state->ropts.hkern;
+}
+
+void set_hac_kernel (int k)
+{
+    check_for_state();
+
+    if (k >= KERNEL_BARTLETT && k <= KERNEL_QS) {
+	state->ropts.hkern = k;
+    }
+}
+
+int get_hac_prewhiten (void)
+{
+    check_for_state();
+
+    return state->ropts.prewhite;
+}
+
+void set_hac_prewhiten (int w)
+{
+    check_for_state();
+    state->ropts.prewhite = (w != 0);
+}
+
+int get_panel_beck_katz (void)
+{
+    check_for_state();
+
+    return state->ropts.pcse;
+}
+
+void set_panel_beck_katz (int b)
+{
+    check_for_state();
+    state->ropts.pcse = (b != 0);
+}
+
+double get_qs_bandwidth (void)
+{
+    check_for_state();
+
+    if (!na(state->ropts.qsband) && state->ropts.qsband > 0) {
+	return state->ropts.qsband;
+    } else {
+	/* what's a sensible default here? */
+	return 2.0;
+    }
+}
+
+void set_qs_bandwidth (double w)
+{
+    check_for_state();
+    if (w >= 0.0) {
+	state->ropts.qsband = w;
+    }
+}
+
+static const char *get_hac_lag_string (void)
+{
+    check_for_state();
+
+    if (state->ropts.user_lag >= 0 && state->ropts.user_lag < 1000) {
 	static char lagstr[6];
 
 	sprintf(lagstr, "%d", state->ropts.user_lag);
@@ -652,6 +856,81 @@ static char *get_hac_lag_string (void)
     }
 }
 
+int libset_numeric_string (const char *s, int *pi, double *px)
+{
+    char *test;
+    int ret = 1;
+
+    if (s == NULL || *s == '\0' ||
+	!strcmp(s, "inf") || !strcmp(s, "nan")) {
+	return 0;
+    }
+
+    errno = 0;
+
+    if (px != NULL) {
+	*px = strtod(s, &test);
+	if (*test != '\0' || errno == ERANGE) {
+	    ret = 0;
+	}
+    } else {
+	long li = strtol(s, &test, 10);
+
+	if (*test != '\0' || errno == ERANGE) {
+	    ret = 0;
+	} else {
+	    *pi = (int) li;
+	}
+    }
+
+    return ret;
+}
+
+/* all the libset ints/doubles are non-negative */
+
+static int libset_get_scalar (const char *s, double **Z,
+			      DATAINFO *pdinfo,
+			      int *pi, double *px)
+{
+    double x = NADBL;
+    int v, err = 0;
+
+    if (libset_numeric_string(s, pi, px)) {
+	if (pi != NULL && *pi < 0) {
+	    err = E_DATA;
+	} else if (px != NULL && *px < 0.0) {
+	    err = E_DATA;
+	}
+	return err;
+    }
+
+    v = varindex(pdinfo, s);
+
+    if (v >= pdinfo->v) {
+	return E_UNKVAR;
+    } else if (var_is_series(pdinfo, v)) {
+	return E_DATATYPE;
+    } else {
+	x = Z[v][0];
+    }
+
+    if (x < 0.0) {
+	return E_DATA;
+    }
+
+    if (px != NULL) {
+	*px = x;
+    } else if (pi != NULL) {
+	if (na(x) || fabs(x) > (double) INT_MAX) {
+	    err = E_DATA;
+	} else {
+	    *pi = (int) x;
+	}
+    }
+
+    return err;
+}
+
 static int parse_hc_variant (const char *s)
 {
     int err = 1;
@@ -710,6 +989,17 @@ void set_tseries_hccme (const char *s)
     free(scpy);
 }
 
+void set_panel_hccme (const char *s)
+{
+    check_for_state();
+
+    if (!strcmp(s, "Arellano")) {
+	state->ropts.pcse = 0;
+    } else if (!strcmp(s, "PCSE")) {
+	state->ropts.pcse = 1;
+    }
+}
+
 void set_garch_robust_vcv (const char *s)
 {
     char *scpy = gretl_strdup(s);
@@ -758,16 +1048,20 @@ static int set_line_width (const char *s0, const char *s1,
 }
 
 static int set_bkbp_limits (const char *s0, const char *s1,
+			    double **Z, DATAINFO *pdinfo,
 			    PRN *prn)
 {
     int p0, p1;
+    int err = 0;
 
-    if (!isdigit((unsigned char) *s0) || !isdigit((unsigned char) *s1)) {
-	return 1;
+    err = libset_get_scalar(s0, Z, pdinfo, &p0, NULL);
+    if (!err) {
+	err = libset_get_scalar(s1, Z, pdinfo, &p1, NULL);
     }
 
-    p0 = atoi(s0);
-    p1 = atoi(s1);
+    if (err) {
+	return err;
+    }
 
     if (p1 < p0) {
 	/* 2nd entry should be bigger than 1st one */
@@ -798,7 +1092,7 @@ static int set_initvals (const char *s, const DATAINFO *pdinfo, PRN *prn)
 	gretl_matrix_free(state->initvals);
 	state->initvals = NULL;
     } else {
-	m = get_matrix_by_name(mname, pdinfo);
+	m = get_matrix_by_name(mname);
 	if (m == NULL) {
 	    pprintf(prn, _("'%s': no such matrix\n"), mname);
 	    err = E_DATA;
@@ -813,6 +1107,42 @@ static int set_initvals (const char *s, const DATAINFO *pdinfo, PRN *prn)
     return err;
 }
 
+void shelldir_init (void)
+{
+    char *test = getcwd(state->shelldir, MAXLEN);
+
+    if (test == NULL) {
+	*state->shelldir = '\0';
+    }
+}
+
+static int set_shelldir (const char *s)
+{
+    int err = 0;
+
+    /* skip past "set shelldir" and space */
+    s += 12;
+    s += strspn(s, " ");
+
+    if (*s == '\0') {
+	*state->shelldir = '\0';
+    } else if (*s == '"') {
+	int len = haschar('"', s + 1);
+
+	if (len <= 0 || len >= MAXLEN) {
+	    err = E_PARSE;
+	} else {
+	    *state->shelldir = '\0';
+	    strncat(state->shelldir, s + 1, len);
+	}
+    } else {
+	*state->shelldir = '\0';
+	strncat(state->shelldir, s, MAXLEN - 1);
+    }
+
+    return err;
+}
+
 static int parse_set_plotfile (const char *s)
 {
     char *fname;
@@ -883,17 +1213,29 @@ static int display_settings (PRN *prn)
     pputs(prn, "Variables that can be set using \"set\" (do \"help set\""
 	  " for details):\n");
 
-    pprintf(prn, " echo = %d\n", state->gretl_echo);
+    pprintf(prn, " echo = %d\n", flag_to_bool(state, STATE_ECHO_ON));
+    pprintf(prn, " messages = %d\n", flag_to_bool(state, STATE_MSGS_ON));
 
     ival = get_use_qr(); /* checks env */
     pprintf(prn, " qr = %d\n", state->use_qr);
 
+    pprintf(prn, " use_cwd = %d\n", flag_to_bool(state, STATE_USE_CWD));
+    pprintf(prn, " force_decpoint = %d\n", flag_to_bool(state, STATE_FORCE_DECPOINT));
+
     uval = get_gretl_random_seed();
     pprintf(prn, " seed = %u\n", uval);
 
     pprintf(prn, " hac_lag = %s\n", get_hac_lag_string());
     pprintf(prn, " hc_version = %d\n", state->ropts.hc_version);
     pprintf(prn, " force_hc = %d\n", state->ropts.force_hc);
+    pprintf(prn, " hac_kernel = %s\n", hac_kernel_string());
+    pprintf(prn, " hac_prewhiten = %d\n", state->ropts.prewhite);
+    if (na(state->ropts.qsband)) {
+	pputs(prn, " qs_bandwidth: auto\n");
+    } else {
+	pprintf(prn, " qs_bandwidth = %g\n", state->ropts.qsband);
+    }
+    pprintf(prn, " pcse = %d\n", state->ropts.pcse);
 
     pprintf(prn, " garch_vcv = %s\n", garch_vcv_string());
 
@@ -923,6 +1265,8 @@ static int display_settings (PRN *prn)
 	pprintf(prn, " horizon = %d\n", state->horizon);
     }
 
+    pprintf(prn, " bootrep = %d\n", state->bootrep);
+
     pprintf(prn, " nls_toler = %g\n", get_nls_toler());
 
     dval = get_bhhh_toler();
@@ -931,18 +1275,31 @@ static int display_settings (PRN *prn)
     } else {
 	pprintf(prn, " bhhh_toler = %g\n", dval);
     }
-
     pprintf(prn, " bhhh_maxiter = %d\n", get_bhhh_maxiter());
-    pprintf(prn, " messages = %d\n", state->gretl_msgs);
 
-    ival =  get_halt_on_error(); /* checks env */
-    pprintf(prn, " halt_on_error = %d\n", state->halt_on_error);
+    dval = get_bfgs_toler();
+    if (na(dval)) {
+	pputs(prn, " bfgs_toler = default\n");
+    } else {
+	pprintf(prn, " bfgs_toler = %g\n", dval);
+    }
+    pprintf(prn, " bfgs_maxiter = %d\n", get_bfgs_maxiter());
 
-    pprintf(prn, " shell_ok = %d\n", state->shell_ok);
+    ival = get_halt_on_error(); /* checks env */
+    pprintf(prn, " halt_on_error = %d\n", state->halt_on_err);
+
+    pprintf(prn, " shell_ok = %d\n", get_shell_ok());
     pprintf(prn, " csv_delim = %s\n", arg_from_delim(state->delim));
     pprintf(prn, " longdigits = %d\n", state->longdigits);
+    pprintf(prn, " max_verbose = %d\n", state->max_verbose);
     print_initvals(state->initvals, prn);
 
+    if (*state->shelldir) {
+	pprintf(prn, " shelldir = '%s'\n", state->shelldir);
+    } else {
+	pputs(prn, " shelldir = unset\n");
+    }
+
     return 0;
 }
 
@@ -952,10 +1309,12 @@ static int display_settings (PRN *prn)
 #define boolean_off(s) (!strcmp(s, "off") || !strcmp(s, "0") || \
                         !strcmp(s, "false"))
 
-int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
+int execute_set_line (const char *line, double **Z, DATAINFO *pdinfo, 
+		      PRN *prn)
 {
     char setobj[16], setarg[16], setarg2[16];
-    int nw, err = E_PARSE;
+    int k, nw, err = E_PARSE;
+    double x;
 
     check_for_state();
 
@@ -973,12 +1332,17 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 	    return parse_set_plotfile(line);
 	} else if (!strcmp(setobj, "initvals")) {
 	    return set_initvals(line, pdinfo, prn);
+	} else if (!strcmp(setobj, "shelldir")) {
+	    return set_shelldir(line);
 	}
     }
 
     if (nw == 1) {
 	if (!strcmp(setobj, "echo")) {
-	    state->gretl_echo = 1;
+	    state->flags |= STATE_ECHO_ON;
+	    err = 0;
+	} else if (!strcmp(setobj, "stopwatch")) {
+	    gretl_stopwatch();
 	    err = 0;
 	}
     } else if (nw == 2) {
@@ -987,18 +1351,18 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 	/* set command echo on/off */
 	if (!strcmp(setobj, "echo")) {
 	    if (boolean_off(setarg)) {
-		state->gretl_echo = 0;
+		state->flags &= ~STATE_ECHO_ON;
 		err = 0;
 	    } else if (boolean_on(setarg)) {
-		state->gretl_echo = 1;
+		state->flags |= STATE_ECHO_ON;
 		err = 0;
 	    }
 	} else if (!strcmp(setobj, "messages")) {
 	    if (boolean_off(setarg)) {
-		state->gretl_msgs = 0;
+		state->flags &= ~STATE_MSGS_ON;
 		err = 0;
 	    } else if (boolean_on(setarg)) {
-		state->gretl_msgs = 1;
+		state->flags |= STATE_MSGS_ON;
 		err = 0;
 	    }
 	} else if (!strcmp(setobj, "csv_delim")) {
@@ -1012,11 +1376,16 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 	    /* set max lag for HAC estimation */
 	    if (!strcmp(setarg, "nw1")) {
 		state->ropts.auto_lag = AUTO_LAG_STOCK_WATSON;
-		state->ropts.user_lag = 0;
+		state->ropts.user_lag = UNSET_INT;
 		err = 0;
 	    } else if (!strcmp(setarg, "nw2")) {
 		state->ropts.auto_lag = AUTO_LAG_WOOLDRIDGE;
-		state->ropts.user_lag = 0;
+		state->ropts.user_lag = UNSET_INT;
+		err = 0;
+	    } else if (!strcmp(setarg, "nw3") ||
+		       !strcmp(setarg, "auto")) {
+		state->ropts.auto_lag = AUTO_LAG_NEWEYWEST;
+		state->ropts.user_lag = UNSET_INT;
 		err = 0;
 	    } else if (isdigit(*setarg)) {
 		state->ropts.user_lag = atoi(setarg);
@@ -1034,6 +1403,38 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 		set_force_hc(0);
 		err = 0;
 	    }
+	} else if (!strcmp(setobj, "hac_kernel")) {
+	    if (!strcmp(setarg, "bartlett")) {
+		state->ropts.hkern = KERNEL_BARTLETT;
+		err = 0;
+	    } else if (!strcmp(setarg, "parzen")) {
+		state->ropts.hkern = KERNEL_PARZEN;
+		err = 0;
+	    } else if (!strcmp(setarg, "qs")) {
+		state->ropts.hkern = KERNEL_QS;
+		err = 0;
+	    }
+	} else if (!strcmp(setobj, "hac_prewhiten")) {
+	    if (boolean_on(setarg)) { 
+		set_hac_prewhiten(1);
+		err = 0;
+	    } else if (boolean_off(setarg)) { 
+		set_hac_prewhiten(0);
+		err = 0;
+	    }
+	} else if (!strcmp(setobj, "pcse")) {
+	    if (boolean_on(setarg)) { 
+		set_panel_beck_katz(1);
+		err = 0;
+	    } else if (boolean_off(setarg)) { 
+		set_panel_beck_katz(0);
+		err = 0;
+	    }	    
+	} else if (!strcmp(setobj, "qs_bandwidth")) {
+	    err = libset_get_scalar(setarg, Z, pdinfo, NULL, &x);
+	    if (!err) {
+		state->ropts.qsband = x;
+	    }	    
 	} else if (!strcmp(setobj, "garch_vcv")) {
 	    /* set GARCH VCV variant */
 	    err = set_garch_vcv_variant(setarg);
@@ -1046,44 +1447,59 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 		state->use_qr = 0;
 		err = 0;
 	    }
-	} else if (!strcmp(setobj, "halt_on_error")) {
+	} else if (!strcmp(setobj, "use_cwd")) {
 	    if (boolean_on(setarg)) {
-		state->halt_on_error = 1;
+		state->flags |= STATE_USE_CWD;
 		err = 0;
 	    } else if (boolean_off(setarg)) {
-		state->halt_on_error = 0;
+		state->flags &= ~STATE_USE_CWD;
+		err = 0;
+	    }
+	} else if (!strcmp(setobj, "force_decpoint")) {
+	    if (boolean_on(setarg)) {
+		state->flags |= STATE_FORCE_DECPOINT;
+		err = 0;
+	    } else if (boolean_off(setarg)) {
+		state->flags &= ~STATE_FORCE_DECPOINT;
+		err = 0;
+	    }
+	} else if (!strcmp(setobj, "halt_on_error")) {
+	    if (boolean_on(setarg)) {
+		state->halt_on_err = 1;
+		err = 0;
+	    } else if (boolean_off(setarg)) {
+		state->halt_on_err = 0;
 		err = 0;
 	    }
 	} else if (!strcmp(setobj, "shell_ok")) {
 	    pprintf(prn, "You can only set this variable via the gretl GUI\n");
 	} else if (!strcmp(setobj, "seed")) {
 	    /* seed for PRNG */
-	    if (isdigit(*setarg)) {
-		int k = atoi(setarg);
-
+	    err = libset_get_scalar(setarg, Z, pdinfo, &k, NULL);
+	    if (!err) {
 		gretl_rand_set_seed((unsigned int) k);
 		pprintf(prn, 
 			_("Pseudo-random number generator seeded with %d\n"), k);
 		state->seed = k;
-		err = 0;
-
 	    }
 	} else if (!strcmp(setobj, "hp_lambda")) {
 	    /* Hodrick-Prescott filter parameter */
 	    if (!strcmp(setarg, "auto")) {
 		state->hp_lambda = NADBL;
 		err = 0;
-	    } else if (check_atof(setarg) == 0) {
-		state->hp_lambda = atof(setarg);
-		err = 0;
+	    } else {
+		err = libset_get_scalar(setarg, Z, pdinfo, NULL, &x);
+		if (!err) {
+		    state->hp_lambda = x;
+		}
 	    }
 	} else if (!strcmp(setobj, "bkbp_k")) {
 	    /* Baxter-King approximation order */
-	    if (isdigit(*setarg)) {
-		state->bkopts.k = atoi(setarg);
+	    err = libset_get_scalar(setarg, Z, pdinfo, &k, NULL);
+	    if (!err) {
+		state->bkopts.k = k;
 		pprintf(prn, 
 			_("Baxter-King approximation = %d\n"), state->bkopts.k);
-		err = 0;
 	    }
 	} else if (!strcmp(setobj, "horizon")) {
 	    /* horizon for VAR impulse responses */
@@ -1091,42 +1507,71 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 		state->horizon = UNSET_INT;
 		err = 0;
 	    } else {
-		state->horizon = atoi(setarg);
-		if (state->horizon > 0) {
-		    err = 0;
+		err = libset_get_scalar(setarg, Z, pdinfo, &k, NULL);
+		if (!err) {
+		    state->horizon = k;
 		} else {
 		    state->horizon = UNSET_INT;
 		}
-	    }	    
+	    }
+	} else if (!strcmp(setobj, "bootrep")) {
+	    err = libset_get_scalar(setarg, Z, pdinfo, &k, NULL);
+	    if (!err) {
+		err = (k <= 0);
+	    }
+	    if (!err) {
+		state->bootrep = k;
+	    } 
 	} else if (!strcmp(setobj, "nls_toler")) {
-	    double tol;
-
-	    if (sscanf(setarg, "%lf", &tol)) {
-		err = set_nls_toler(tol);
+	    err = libset_get_scalar(setarg, Z, pdinfo, NULL, &x);
+	    if (!err) {
+		err = set_nls_toler(x);
 	    }
 	} else if (!strcmp(setobj, "bhhh_toler")) {
 	    /* Tolerance for BHHH (ARMA, Tobit) */
-	    double tol;
-
 	    if (!strcmp(setarg, "default")) {
 		set_bhhh_toler(NADBL);
 		err = 0;
-	    } else if (sscanf(setarg, "%lf", &tol)) {
-		err = set_bhhh_toler(tol);
+	    } else {
+		err = libset_get_scalar(setarg, Z, pdinfo, NULL, &x);
+		if (!err) {
+		    err = set_bhhh_toler(x);
+		}
 	    }
 	} else if (!strcmp(setobj, "bhhh_maxiter")) {
 	    /* Maximum iterations for BHHH (ARMA, Tobit) */
 	    if (isdigit(*setarg)) {
 		err = set_bhhh_maxiter(atoi(setarg));
 	    }
-	} else if (!strcmp(setobj, "longdigits")) {
-	    if (isdigit(*setarg)) {
-		err = set_long_digits(atoi(setarg));
+	} else if (!strcmp(setobj, "bfgs_toler")) {
+	    if (!strcmp(setarg, "default")) {
+		set_bfgs_toler(NADBL);
+		err = 0;
+	    } else {
+		err = libset_get_scalar(setarg, Z, pdinfo, NULL, &x);
+		if (!err) {
+		    err = set_bfgs_toler(x);
+		}
 	    }
-	} 
+	} else if (!strcmp(setobj, "bfgs_maxiter")) {
+	    err = libset_get_scalar(setarg, Z, pdinfo, &k, NULL);
+	    if (!err) {
+		err = set_bfgs_maxiter(k);
+	    }
+	} else if (!strcmp(setobj, "longdigits")) {
+	    err = libset_get_scalar(setarg, Z, pdinfo, &k, NULL);
+	    if (!err) {
+		err = set_long_digits(k);
+	    }
+	} else if (!strcmp(setobj, "max_verbose")) {
+	    err = libset_get_scalar(setarg, Z, pdinfo, &k, NULL);
+	    if (!err) {
+		err = set_max_verbose(k);
+	    }
+	} 	    
     } else if (nw == 3) {
 	if (!strcmp(setobj, "bkbp_limits")) {
-	    err = set_bkbp_limits(setarg, setarg2, prn);
+	    err = set_bkbp_limits(setarg, setarg2, Z, pdinfo, prn);
 	} else if (!strcmp(setobj, "linewidth")) {
 	    err = set_line_width(setarg, setarg2, pdinfo, prn);
 	}
@@ -1140,12 +1585,21 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 void set_use_qr (int set)
 {
     check_for_state();
-    state->use_qr = set;
+
+    if (state != NULL) {
+	state->use_qr = set;
+    }
 }
 
 int get_use_qr (void)
 {
-    check_for_state();
+    if (check_for_state()) {
+	return 0;
+    }
+
+#if PDEBUG
+    fprintf(stderr, "get_use_qr: state = %p\n", (void *) state);
+#endif
 
     if (is_unset(state->use_qr)) {
 	char *s = getenv("GRETL_USE_QR");
@@ -1160,21 +1614,50 @@ int get_use_qr (void)
     return state->use_qr;
 }
 
-int get_halt_on_error (void)
+void set_use_cwd (int set)
 {
     check_for_state();
 
-    if (is_unset(state->halt_on_error)) {
+    if (state != NULL) {
+	if (set) {
+	    state->flags |= STATE_USE_CWD;
+	} else {
+	    state->flags &= ~STATE_USE_CWD;
+	}
+    }
+}
+
+int get_use_cwd (void)
+{
+    if (check_for_state()) {
+	return 0;
+    }
+
+    return flag_to_bool(state, STATE_USE_CWD);
+}
+
+int get_halt_on_error (void)
+{
+    if (check_for_state()) {
+	return 1;
+    }
+
+#if PDEBUG
+    fprintf(stderr, "get_halt_on_error: state = %p, halt_on_err = %d\n",
+	    (void *) state, state->halt_on_err);
+#endif
+
+    if (is_unset(state->halt_on_err)) {
 	char *s = getenv("GRETL_KEEP_GOING");
 
 	if (s != NULL && *s != '\0' && *s != '0') {
-	    state->halt_on_error = 0;
+	    state->halt_on_err = 0;
 	} else {
-	    state->halt_on_error = 1;
+	    state->halt_on_err = 1;
 	}
     } 
 
-    return state->halt_on_error;
+    return state->halt_on_err;
 }
 
 #ifndef WIN32
@@ -1214,17 +1697,6 @@ int get_shell_ok (void)
     return state->shell_ok;
 }
 
-static void update_sample_info (set_vars *sv, const DATAINFO *pdinfo)
-{
-    if (pdinfo != NULL) {
-	sv->sinfo.t1 = pdinfo->t1;
-	sv->sinfo.t2 = pdinfo->t2;
-	sv->sinfo.submode = pdinfo->submode;
-	free(sv->sinfo.submask);
-	sv->sinfo.submask = copy_datainfo_submask(pdinfo);
-    }	 
-}
-
 /* Mechanism for pushing and popping program state for user-defined
    functions. push_program_state() is used when a function starts
    execution: the function gets a copy of the current program state,
@@ -1232,10 +1704,10 @@ static void update_sample_info (set_vars *sv, const DATAINFO *pdinfo)
    function exits.
 */
 
-int n_states;
+static int n_states;
 static set_vars **state_stack;
 
-int push_program_state (const DATAINFO *pdinfo)
+int push_program_state (void)
 {
     set_vars **sstack;
     set_vars *newstate;
@@ -1243,13 +1715,7 @@ int push_program_state (const DATAINFO *pdinfo)
     int err = 0;
 
 #if PDEBUG
-    fprintf(stderr, "push_program_state: ns=%d\n", ns);
-    if (pdinfo != NULL) {
-	fprintf(stderr, " pdinfo=%p, t1=%d, t2=%d\n",
-		(void *) pdinfo, pdinfo->t1, pdinfo->t2);
-    } else {
-	fprintf(stderr, " pdinfo is NULL\n");
-    }
+    fprintf(stderr, "push_program_state: n_states = %d\n", ns);
 #endif
 
     newstate = malloc(sizeof *newstate);
@@ -1269,10 +1735,8 @@ int push_program_state (const DATAINFO *pdinfo)
 	    /* set all defaults */
 	    state_vars_init(newstate);
 	} else {
-	    /* set sample on existing state */
-	    update_sample_info(state, pdinfo);
 	    /* copy existing state */
-	    state_vars_copy(newstate, pdinfo);
+	    state_vars_copy(newstate);
 	}
 	state_stack = sstack;
 	state = state_stack[ns] = newstate;
@@ -1288,32 +1752,8 @@ int push_program_state (const DATAINFO *pdinfo)
     return err;
 }
 
-static int 
-subsampled_at_this_level (const DATAINFO *pdinfo, set_vars *sv)
-{
-    int ret = 0;
-
-    if (!complex_subsampled()) {
-	return 0;
-    }
-
-    if (pdinfo->submask != NULL) {
-	if (sv->sinfo.submask == NULL) {
-	    ret = 1;
-	} else if (submask_cmp(pdinfo->submask, sv->sinfo.submask)) {
-	    ret = 1;
-	}
-    } 
-
-    return ret;
-}
-
 static void free_state (set_vars *sv)
 {
-    if (sv->sinfo.submask != NULL) {
-	free(sv->sinfo.submask);
-    }
-
     if (sv->initvals != NULL) {
 	gretl_matrix_free(sv->initvals);
     }
@@ -1321,23 +1761,18 @@ static void free_state (set_vars *sv)
     free(sv);
 }
 
-/* Called when a new-style function exits: restores the program state
-   that was in force when the function started executing.  This may
-   involve putting the sample back the way it was.  But if the dataset
-   was complex subsampled either before or after function execution,
-   we will not (currently) mess with the sample state.  That case
-   requires more careful thought.
+/* Called when a user-defined function exits: restores the program
+   state that was in force when the function started executing.
 */
 
-int pop_program_state (double ***pZ, DATAINFO **ppdinfo)
+int pop_program_state (void)
 {
     set_vars **sstack;
     int ns = n_states;
     int err = 0;
 
 #if PDEBUG
-    fprintf(stderr, "pop_program_state called: ns=%d, pdinfo=%p\n",
-	    ns, (void *) *ppdinfo);
+    fprintf(stderr, "pop_program_state called: ns=%d\n", ns);
 #endif
 
     if (ns < 2) {
@@ -1361,31 +1796,6 @@ int pop_program_state (double ***pZ, DATAINFO **ppdinfo)
     fprintf(stderr, " state is now state_stack[%d]\n", ns - 2);
 #endif
 
-    if (!err && *ppdinfo != NULL && sinfo_is_set(state->sinfo)) {
-	if (subsampled_at_this_level(*ppdinfo, state)) {
-#if PDEBUG
-	    fprintf(stderr, " undoing current sub-sampling\n");
-#endif
-	    restore_full_sample(pZ, ppdinfo);
-	    if (state->sinfo.submask != NULL) {
-#if PDEBUG
-		fprintf(stderr, " reimposing outer sample restriction\n");
-#endif
-		restrict_sample_from_mask(state->sinfo.submask, state->sinfo.submode, 
-					  pZ, ppdinfo);
-	    }
-	}
-	(*ppdinfo)->t1 = state->sinfo.t1;
-	(*ppdinfo)->t2 = state->sinfo.t2;
-    }
-
-#if PDEBUG
-    if (*ppdinfo != NULL) {
-	fprintf(stderr, " err=%d, pdinfo->t1=%d, pdinfo->t2=%d\n",
-		err, (*ppdinfo)->t1, (*ppdinfo)->t2);
-    } 
-#endif
-
     return err;
 }
 
@@ -1401,7 +1811,7 @@ int libset_init (void)
 #endif
 
     if (!done) {
-	err = push_program_state(NULL);
+	err = push_program_state();
 	done = 1;
     }
 
@@ -1412,6 +1822,10 @@ void libset_cleanup (void)
 {
     int i;
 
+#if PDEBUG
+    fprintf(stderr, "libset_cleanup called\n");
+#endif
+
     for (i=0; i<n_states; i++) {
 	free_state(state_stack[i]);
     }
@@ -1421,18 +1835,6 @@ void libset_cleanup (void)
     n_states = 0;
 }
 
-int libset_restore_state_zero (double ***pZ, DATAINFO **ppdinfo)
-{
-    int i, ns = n_states;
-    int err = 0;
-
-    for (i=ns; i>=2 && !err; i--) {
-	err = pop_program_state(pZ, ppdinfo);
-    }
-
-    return err;
-}
-
 /* switches for looping, batch mode, and pausing between screens of
    output: these depend on the state of the program calling libgretl,
    but are not user-settable
