@@ -24,12 +24,16 @@
 #ifndef LOOP_PRIVATE_H
 #define LOOP_PRIVATE_H
 
-enum ifcodes {
+enum {
     SET_FALSE,
     SET_TRUE,
     SET_ELSE,
+    SET_ELIF,
     SET_ENDIF,
     IS_FALSE,
+    DOINDENT,
+    UNINDENT,
+    GETINDENT,
     RELAX
 };
 
