@@ -20,7 +20,7 @@
 #ifndef MISSING_PRIVATE_H
 #define MISSING_PRIVATE_H
 
-#define missing_masked(m,t) (m != NULL && m[t] == '1')
+#define masked(m,t) (m != NULL && m[t] == '1')
 #define has_missing_obs(m)  ((m)->missmask != NULL)
 
 void set_reference_missmask (const MODEL *pmod);
