@@ -28,11 +28,10 @@ int gretl_execute_loop (void);
 
 int ok_in_loop (int ci);
 
-int gretl_loop_append_line (char *line, int ci, gretlopt opt,
-			    double ***pZ, DATAINFO *pdinfo);
+int gretl_loop_append_line (ExecState *s, double ***pZ, 
+			    DATAINFO *pdinfo);
 
-int gretl_loop_exec (char *line, double ***pZ, DATAINFO **ppdinfo, 
-		     MODEL **models, PRN *prn);
+int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO **ppdinfo); 
 
 int model_is_in_loop (const MODEL *pmod);
 
