@@ -18,11 +18,11 @@
  */
 
 #include "libgretl.h"
+#include "monte_carlo.h"
 #include "var.h"
 #include "johansen.h"
 #include "system.h"
 #include "objstack.h"
-#include "genrfuncs.h"
 #include "usermat.h"
 #include "modelspec.h"
 
@@ -136,7 +136,6 @@ static void gretl_object_destroy (void *ptr, GretlObjType type)
     }
 }
 
-#if ODEBUG
 static int gretl_object_get_refcount (void *ptr, GretlObjType type)
 {
     int rc = -999;
@@ -164,7 +163,6 @@ static int gretl_object_get_refcount (void *ptr, GretlObjType type)
 
     return rc;
 }    
-#endif
 
 /* The stuff below: Note that we can't "protect" a model (against
    deletion) simply by setting its refcount to some special value,
@@ -173,29 +171,68 @@ static int gretl_object_get_refcount (void *ptr, GretlObjType type)
    "protected species" list.
 */
 
-#define NPROT 4
+static MODEL **protected_models;
+static int n_prot;
 
-static MODEL *protected_models[NPROT] = {
-    NULL, NULL, NULL, NULL
-};
-
-void gretl_model_protect (MODEL *pmod)
+int gretl_model_protect (MODEL *pmod)
 {
-    int i;
+    MODEL **prmod;
+    int err = 0;
 
-    for (i=0; i<NPROT; i++) {
-	if (protected_models[i] == NULL) {
-	    protected_models[i] = pmod;
+    prmod = realloc(protected_models, (n_prot + 1) * sizeof *prmod);
+
+    if (prmod == NULL) {
+	fprintf(stderr, "gretl_model_protect: out of memory!\n");
+	err = E_ALLOC;
+    } else {
+	protected_models = prmod;
+	protected_models[n_prot++] = pmod;
+    }
+
+    return err;
+}
+
+static int gretl_model_unprotect (MODEL *pmod)
+{
+    MODEL **prmod;
+    int match = 0;
+    int i, j, err = 0;
+
+    for (i=0; i<n_prot; i++) {
+	if (protected_models[i] == pmod) {
+	    match = 1;
+	    for (j=i; j<n_prot-1; j++) {
+		protected_models[j] = protected_models[j+1];
+	    }
 	    break;
 	}
     }
+
+    if (match) {
+	if (n_prot == 1) {
+	    free(protected_models);
+	    protected_models = NULL;
+	    n_prot = 0;
+	} else {
+	    prmod = realloc(protected_models, (n_prot - 1) * sizeof *prmod);
+	    if (prmod == NULL) {
+		fprintf(stderr, "gretl_model_unprotect: out of memory!\n");
+		err = E_ALLOC;
+	    } else {
+		protected_models = prmod;
+		n_prot--;
+	    }
+	}
+    }
+
+    return err;
 }
 
 static int model_is_protected (MODEL *pmod)
 {
     int i, prot = 0;
 
-    for (i=0; i<NPROT; i++) {
+    for (i=0; i<n_prot; i++) {
 	if (pmod == protected_models[i]) {
 	    prot = 1;
 	    break;
@@ -271,7 +308,7 @@ void gretl_object_unref (void *ptr, GretlObjType type)
 
 /**
  * set_as_last_model:
- * @ptr: pointer to gretl obejct (e.g. #MODEL).
+ * @ptr: pointer to gretl object (e.g. #MODEL).
  * @type: type of object.
  *
  * Puts @ptr in place as the "last model" (which will be
@@ -293,9 +330,11 @@ void set_as_last_model (void *ptr, GretlObjType type)
 	gretl_object_unref(last_model.ptr, last_model.type);
     }
 
-    last_model.ptr = ptr;
-    last_model.type = type;
-    gretl_object_ref(ptr, type);
+    if (last_model.ptr != ptr || last_model.type != type) {
+	last_model.ptr = ptr;
+	last_model.type = type;
+	gretl_object_ref(ptr, type);
+    }
 
 #if ODEBUG
     fprintf(stderr, " refcount on \"last_model\" = %d\n",
@@ -348,6 +387,38 @@ const char *gretl_object_get_name (void *p, GretlObjType type)
     return NULL;
 }
 
+static stacker *
+get_stacker_by_name (const char *oname, GretlObjType type, int *onum)
+{
+    stacker *s = NULL;
+    const char *test;
+    int i;
+
+    if (oname == NULL) {
+	return NULL;
+    }
+
+    for (i=0; i<n_obj; i++) {
+	if (type == GRETL_OBJ_ANY || type == ostack[i].type) {
+	    test = gretl_object_get_name(ostack[i].ptr, ostack[i].type);
+	    if (!strcmp(oname, test)) {
+		if (onum != NULL) {
+		    *onum = i;
+		}
+		s = &ostack[i];
+		break;
+	    }
+	}
+    }
+
+#if ODEBUG
+    fprintf(stderr, "get_stacker_by_name: name='%s', type=%d: got s=%p\n",
+	    oname, type, (void *) s);
+#endif
+    
+    return s;
+}
+
 static void *
 get_object_by_name (const char *oname, GretlObjType type, int *onum)
 {
@@ -423,6 +494,23 @@ gretl_get_object_and_type (const char *name, void **pp, GretlObjType *type)
     return err;
 }
 
+MODEL *get_model_by_ID (int ID) 
+{
+    MODEL *pmod;
+    int i;
+
+    for (i=0; i<n_obj; i++) {
+	if (ostack[i].type == GRETL_OBJ_EQN) {
+	    pmod = (MODEL *) ostack[i].ptr;
+	    if (pmod->ID == ID) {
+		return pmod;
+	    }
+	}
+    }
+
+    return NULL;
+}
+
 MODEL *get_model_by_name (const char *mname)
 {
     return get_object_by_name(mname, GRETL_OBJ_EQN, NULL);
@@ -515,6 +603,8 @@ void remove_model_from_stack (MODEL *pmod)
 	last_model.ptr = NULL;
 	last_model.type = 0;
     }
+
+    gretl_model_unprotect(pmod);
 }
 
 static int object_stack_index (const void *p)
@@ -549,6 +639,11 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
     stacker *orig;
     int onum, err = 0;
 
+#if ODEBUG
+    fprintf(stderr, "real_stack_object: on entry, p=%p, type=%d, name='%s'\n", 
+	    (void *) p, type, name);
+#endif
+
     if (p == NULL) {
 	return 1;
     }
@@ -567,7 +662,7 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
 	return err;
     }
 
-    orig = get_object_by_name(name, type, &onum);
+    orig = get_stacker_by_name(name, type, &onum);
 
     if (orig != NULL) {
 	/* replace existing object of same name */
@@ -739,37 +834,8 @@ static double real_get_obj_scalar (void *p, GretlObjType type, int idx)
     return x;
 }
 
-/* retrieve from an object some value that is stored on the object in
-   the form of a scalar element of an array */
-
-static double 
-real_get_obj_scalar_element (void *p, GretlObjType type, int idx, 
-			     const char *key, const DATAINFO *pdinfo, 
-			     int *err)
-{
-    double x = INVALID_STAT;
-    
-    if (idx <= 0) {
-	*err = 1;
-	return x;
-    }
-
-    if (type == GRETL_OBJ_EQN) {
-	MODEL *pmod = (MODEL *) p;
-
-	x = get_model_data_element(pmod, idx, key, pdinfo, err);
-	if (*err) {
-	    x = INVALID_STAT;
-	}
-    } 
-
-    /* FIXME add accessors for systems, VARs */
-
-    return x;
-}
-
 static double *
-real_get_obj_series (void *p, GretlObjType type, int idx, const char *key, 
+real_get_obj_series (void *p, GretlObjType type, int idx,
 		     const DATAINFO *pdinfo, int *err)
 {
     double *x = NULL;
@@ -783,15 +849,7 @@ real_get_obj_series (void *p, GretlObjType type, int idx, const char *key,
 	MODEL *pmod = (MODEL *) p;
 
 	x = gretl_model_get_series(pmod, pdinfo, idx, err);
-    } else if (type == GRETL_OBJ_SYS) {
-	gretl_equation_system *sys = (gretl_equation_system *) p;
-
-	x = gretl_equation_system_get_series(sys, pdinfo, idx, key, err);
-    } else if (type == GRETL_OBJ_VAR) {
-	GRETL_VAR *var = (GRETL_VAR *) p;
-
-	x = gretl_VAR_get_series(var, pdinfo, idx, key, err);
-    }
+    } 
 
     return x;
 }
@@ -837,6 +895,15 @@ void unset_genr_model (void)
     genr_model.ptr = NULL;
 }
 
+void *get_genr_model (GretlObjType *type)
+{
+    if (type != NULL) {
+	*type = genr_model.type;
+    }
+
+    return genr_model.ptr;
+}
+
 static stacker *find_smatch (const char *oname)
 {
     stacker *smatch = NULL;
@@ -863,17 +930,28 @@ static stacker *find_smatch (const char *oname)
     return smatch;
 }
 
-double saved_object_get_scalar (const char *oname, const char *key,
-				int *err)
+/* retrieve the type of a named model, or of the last model
+   if @name is NULL */
+
+GretlObjType gretl_model_get_type (const char *name)
+{
+    stacker *smatch = find_smatch(name);
+
+    if (smatch == NULL) {
+	return GRETL_OBJ_UNKNOWN;
+    } else {
+	return smatch->type;
+    }
+}
+
+double saved_object_get_scalar (const char *oname, int idx, int *err)
 {
     double ret = INVALID_STAT;
     stacker *smatch;
-    int idx;
 
     smatch = find_smatch(oname);
 
     if (smatch != NULL) {
-	idx = gretl_model_data_index(key);
 	ret = real_get_obj_scalar(smatch->ptr, smatch->type, idx);
     }
 
@@ -884,46 +962,18 @@ double saved_object_get_scalar (const char *oname, const char *key,
     return ret;
 }
 
-double saved_object_get_scalar_element (const char *oname, const char *key,
-					const DATAINFO *pdinfo, int *err)
-{
-    double ret = INVALID_STAT;
-    stacker *smatch;
-    int idx;
-
-#if ODEBUG
-    fprintf(stderr, "saved_object_get_scalar_element: oname='%s', key='%s'\n",
-	    oname, key);
-#endif
-
-    smatch = find_smatch(oname);
-
-    if (smatch != NULL) {
-	idx = gretl_model_data_index(key);
-	ret = real_get_obj_scalar_element(smatch->ptr, smatch->type, idx, 
-					  key, pdinfo, err);
-    }
-
-    if (ret == INVALID_STAT && !*err) {
-	*err = E_BADSTAT;
-    }
-
-    return ret;
-}
-
-double *saved_object_get_series (const char *oname, const char *key, 
-				 const DATAINFO *pdinfo, int *err)
+double *saved_object_get_series (const char *oname, int idx,
+				 const DATAINFO *pdinfo, 
+				 int *err)
 {
     double *x = NULL;
     stacker *smatch;
-    int idx;
 
     smatch = find_smatch(oname);
 
     if (smatch != NULL) {
-	idx = gretl_model_data_index(key);
 	x = real_get_obj_series(smatch->ptr, smatch->type, idx, 
-				key, pdinfo, err);
+				pdinfo, err);
     }
 
     if (x == NULL && !*err) {
@@ -934,28 +984,15 @@ double *saved_object_get_series (const char *oname, const char *key,
 }
 
 gretl_matrix *
-saved_object_get_matrix (const char *oname, const char *key,
-			 double ***pZ, DATAINFO *pdinfo,
-			 int *err)
+saved_object_get_matrix (const char *oname, int idx, int *err)
 {
     gretl_matrix *M = NULL;
-    const char *mspec = NULL;
     stacker *smatch;
-    int idx;
 
     smatch = find_smatch(oname);
 
     if (smatch != NULL) {
-	idx = gretl_model_data_index(key);
 	M = real_get_obj_matrix(smatch->ptr, smatch->type, idx, err);
-	mspec = strchr(key, '[');
-    }
-
-    if (M != NULL && mspec != NULL) {
-	gretl_matrix *S = matrix_get_submatrix(M, mspec, pZ, pdinfo, err);
-	
-	gretl_matrix_free(M);
-	M = S;
     }
 
     if (M == NULL && !*err) {
@@ -965,19 +1002,18 @@ saved_object_get_matrix (const char *oname, const char *key,
     return M;
 }
 
-int 
-saved_object_print_scalar (const char *oname, const char *key, PRN *prn)
+static int varchar_spn_with_space (const char *s)
 {
-    int err = 0;
-    double val = saved_object_get_scalar(oname, key, &err);
+    const char *varchars = "abcdefghijklmnopqrstuvwxyz"
+	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+	"0123456789_ ";
+    int ret = 0;
 
-    if (err) {
-	pprintf(prn, _("%s: no data for '%s'\n"), oname, key);
-    } else {
-	pprintf(prn, "%s: %s = %.8g\n", oname, key + 1, val);
+    if (isalpha(*s)) {
+	ret = strspn(s, varchars);
     }
 
-    return err;
+    return ret;
 }
 
 #define OPDEBUG 0
@@ -989,7 +1025,6 @@ int parse_object_command (const char *s, char *name, char **cmd)
 {
     int len, start = 0;
     int quoted = 0;
-    const char *p;
     int err = 0;
 
     *name = 0;
@@ -1002,32 +1037,30 @@ int parse_object_command (const char *s, char *name, char **cmd)
 
     /* skip an opening quote */
     if (*s == '"') {
-	s++;
 	quoted = 1;
+	s++;
     }
 
-    p = s;
-
-    /* crawl to end of (possibly quoted) "name" */
-    len = 0;
-    while (*s) {
-	if (*s == '"') {
-	    quoted = 0;
-	    len--;
-	}
-	if (!quoted && (isspace(*s) || *s == '.')) break;
-	s++; len++;
+    if (quoted) {
+	len = varchar_spn_with_space(s);
+    } else {
+	len = gretl_varchar_spn(s);
     }
 
     if (len == 0) {
 	return 0;
-    }    
+    } 
 
     if (len > MAXSAVENAME - 1) {
 	len = MAXSAVENAME - 1;
     }
 
-    strncat(name, p, len);
+    strncat(name, s, len);
+    s += len;
+
+    if (quoted && *s == '"') {
+	s++;
+    }
 
     /* is an object command embedded? */
     if (*s == '.') {
@@ -1167,7 +1200,14 @@ int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, PRN *prn)
     }
 
     if (type == GRETL_OBJ_EQN) {
-	err = model_error_dist(ptr, pZ, pdinfo, prn);
+	MODEL *pmod = ptr;
+
+	if ((pmod->ci == LOGIT || pmod->ci == PROBIT) &&
+	    gretl_model_get_int(pmod, "ordered")) {
+	    err = E_NOTIMP;
+	} else {
+	    err = model_error_dist(ptr, pZ, pdinfo, prn);
+	}
     } else if (type == GRETL_OBJ_SYS) {
 	err = system_normality_test(ptr, prn);
     } else if (type == GRETL_OBJ_VAR) {
@@ -1185,8 +1225,10 @@ void gretl_saved_objects_cleanup (void)
 
     for (i=0; i<n_obj; i++) {
 	if (ostack[i].ptr == last_model.ptr) {
-	    /* don't double-free! */
-	    last_model.ptr = NULL;
+	    if (gretl_object_get_refcount(ostack[i].ptr, ostack[i].type) == 1) {
+		/* don't double-free! */
+		last_model.ptr = NULL;
+	    }
 	}
 #if ODEBUG
 	fprintf(stderr, "gretl_saved_objects_cleanup:\n"
@@ -1203,8 +1245,8 @@ void gretl_saved_objects_cleanup (void)
     n_vars = 0;
 
     if (last_model.ptr != NULL) {
-	if (last_model.type != GRETL_OBJ_EQN || !
-	    model_is_protected(last_model.ptr)) {
+	if (last_model.type != GRETL_OBJ_EQN || 
+	    !model_is_protected(last_model.ptr)) {
 #if ODEBUG
 	    fprintf(stderr, "gretl_saved_objects_cleanup:\n"
 		    " calling gretl_object_destroy on last_model\n");
