@@ -36,6 +36,7 @@ typedef enum {
     GRETL_OBJ_GRAPH,
     GRETL_OBJ_PLOT,
     GRETL_OBJ_TEXT,
+    GRETL_OBJ_MATRIX,
     GRETL_OBJ_UNKNOWN,
     GRETL_OBJ_MAX
 } GretlObjType;
@@ -62,12 +63,16 @@ enum {
 
 void set_as_last_model (void *ptr, GretlObjType type);
 
-void gretl_model_protect (MODEL *pmod);
+int gretl_model_protect (MODEL *pmod);
 
 void *get_last_model (GretlObjType *type);
 
+void *get_genr_model (GretlObjType *type);
+
 MODEL *get_model_by_name (const char *mname);
 
+MODEL *get_model_by_ID (int ID);
+
 GRETL_VAR *get_VAR_by_name (const char *vname);
 
 GRETL_VAR *get_VECM_by_name (const char *vname);
@@ -79,6 +84,8 @@ void *gretl_get_object_by_name (const char *name);
 int gretl_get_object_and_type (const char *name, void **pp, 
 			       GretlObjType *type);
 
+GretlObjType gretl_model_get_type (const char *name);
+
 int object_is_on_stack (const void *ptr);
 
 int gretl_stack_object (void *ptr, GretlObjType type);
@@ -95,20 +102,19 @@ void gretl_object_ref (void *ptr, GretlObjType type);
 
 void gretl_object_unref (void *ptr, GretlObjType type);
 
-double saved_object_get_scalar (const char *oname, const char *key, int *err);
+double saved_object_get_scalar (const char *oname, int idx, int *err);
 
 int saved_object_print_scalar (const char *oname, const char *key, PRN *prn);
 
 double saved_object_get_scalar_element (const char *oname, const char *key,
 					const DATAINFO *pdinfo, int *err);
 
-double *saved_object_get_series (const char *oname, const char *key, 
-				 const DATAINFO *pdinfo, int *err);
+double *saved_object_get_series (const char *oname, int idx, 
+				 const DATAINFO *pdinfo, 
+				 int *err);
 
 gretl_matrix *
-saved_object_get_matrix (const char *oname, const char *key,
-			 double ***pZ, DATAINFO *pdinfo,
-			 int *err);
+saved_object_get_matrix (const char *oname, int idx, int *err);
 
 int gretl_object_rename (void *p, GretlObjType type, const char *oname);
 
