@@ -22,9 +22,11 @@
 /* model commands plus ADD and OMIT */
 #define vcv_opt_ok(c) (c == ADD || \
                        c == AR || \
+                       c == ARBOND || \
                        c == ARMA || \
                        c == CORC || \
                        c == GARCH || \
+                       c == GMM || \
                        c == HCCM || \
                        c == HILU || \
                        c == HSK || \
@@ -76,8 +78,13 @@ struct gretl_option gretl_opts[] = {
     { ADF,      OPT_V, "verbose" },
     { ADF,      OPT_Q, "quiet" },
     { ADF,      OPT_F, "difference" },
+    { ADF,      OPT_E, "test-down" },
+    { ARBOND,   OPT_A, "asymptotic" },
+    { ARBOND,   OPT_D, "time-dummies" },
+    { ARBOND,   OPT_T, "two-step" },
     { ARMA,     OPT_C, "conditional" },
     { ARMA,     OPT_N, "nc" },    
+    { ARMA,     OPT_Q, "quiet" },
     { ARMA,     OPT_V, "verbose" },
     { ARMA,     OPT_X, "x-12-arima" },
     { BXPLOT,   OPT_O, "notches" },
@@ -88,9 +95,19 @@ struct gretl_option gretl_opts[] = {
     { COINT2,   OPT_A, "crt" },
     { COINT2,   OPT_D, "seasonals" },
     { COINT2,   OPT_N, "nc" },
+    { COINT2,   OPT_Q, "quiet" },
     { COINT2,   OPT_R, "rc" },
     { COINT2,   OPT_T, "ct" },
+    { COINT2,   OPT_V, "verbose" },
+    { CORR,     OPT_U, "uniform" },
+    { CUSUM,    OPT_R, "squares" },
+    { DIFFTEST, OPT_G, "sign" },
+    { DIFFTEST, OPT_R, "rank-sum" },
+    { DIFFTEST, OPT_I, "signed-rank" },
+    { DIFFTEST, OPT_V, "verbose" },
     { DISCRETE, OPT_R, "reverse" },
+    { DUMMIFY,  OPT_F, "drop-first" },
+    { DUMMIFY,  OPT_L, "drop-last" },
     { EQNPRINT, OPT_O, "complete" },
     { EQNPRINT, OPT_T, "t-ratios" },
     { TABPRINT, OPT_O, "complete" },
@@ -107,19 +124,20 @@ struct gretl_option gretl_opts[] = {
     { GARCH,    OPT_A, "arma-init" },    
     { GARCH,    OPT_R, "robust" },
     { GARCH,    OPT_V, "verbose" },
+    { GMM,      OPT_I, "iterate" },
+    { GMM,      OPT_T, "two-step" },
+    { GMM,      OPT_V, "verbose" },
     { GNUPLOT,  OPT_O, "with-lines" },
+    { GNUPLOT,  OPT_I, "inverse-fit" },
+    { GNUPLOT,  OPT_L, "loess-fit" },
+    { GNUPLOT,  OPT_Q, "quadratic-fit" },
+    { GNUPLOT,  OPT_N, "linear-fit" },
     { GNUPLOT,  OPT_M, "with-impulses" },
     { GNUPLOT,  OPT_S, "suppress-fitted" },
     { GNUPLOT,  OPT_T, "time-series" },
     { GNUPLOT,  OPT_Z, "dummy" },
     { GRAPH,    OPT_O, "tall" },
     { HILU,     OPT_B, "no-corc" },
-    { IMPORT,   OPT_B, "box1" },
-    { IMPORT,   OPT_O, "octave" },
-#if 0 /* not ready */
-    { INFO,     OPT_V, "variables" },
-    { INFO,     OPT_T, "tests" },
-#endif
     { KPSS,     OPT_T, "trend" },
     { KPSS,     OPT_V, "verbose" },
     { KPSS,     OPT_Q, "quiet" },
@@ -130,8 +148,10 @@ struct gretl_option gretl_opts[] = {
     { LMTEST,   OPT_L, "logs" },
     { LMTEST,   OPT_S, "squares" }, 
     { LMTEST,   OPT_P, "panel" },
+    { LMTEST,   OPT_Q, "quiet" },
     { LMTEST,   OPT_W, "white" },
     { LOGIT,    OPT_R, "robust" },
+    { LOGIT,    OPT_V, "verbose" },
     { LOOP,     OPT_P, "progressive" },
     { LOOP,     OPT_Q, "quiet" },
     { LOOP,     OPT_V, "verbose" },
@@ -155,18 +175,26 @@ struct gretl_option gretl_opts[] = {
     { OLS,      OPT_S, "simple-print" },
     { OMIT,     OPT_B, "both" },
     { OMIT,     OPT_I, "silent" },
+    { OMIT,     OPT_P, "bootstrap" },
     { OMIT,     OPT_Q, "quiet" },
     { OMIT,     OPT_T, "inst" },
     { OMIT,     OPT_W, "wald" },
     { OMITFROM, OPT_B, "both" },
+    { OMITFROM, OPT_P, "bootstrap" },
     { OMITFROM, OPT_Q, "quiet" },
     { OMITFROM, OPT_T, "inst" },
+    { OMITFROM, OPT_W, "wald" },
+    { OPEN,     OPT_B, "box1" },
+    { OPEN,     OPT_O, "octave" },
+    { OPEN,     OPT_W, "www" },
     { OUTFILE,  OPT_A, "append" },
     { OUTFILE,  OPT_C, "close" },
     { OUTFILE,  OPT_W, "write" },
     { PANEL,    OPT_B, "between" },
+    { PANEL,    OPT_D, "time-dummies" },
     { PANEL,    OPT_H, "hausman-reg" },
     { PANEL,    OPT_O, "vcv" },
+    { PANEL,    OPT_P, "pooled" },
     { PANEL,    OPT_Q, "quiet" },
     { PANEL,    OPT_R, "robust" },
     { PANEL,    OPT_S, "silent" },
@@ -178,13 +206,16 @@ struct gretl_option gretl_opts[] = {
     { PCA,      OPT_A, "save-all" },
     { PCA,      OPT_O, "save" },
     { PERGM,    OPT_O, "bartlett" },
+    { PERGM,    OPT_L, "log" },
     { PLOT,     OPT_O, "one-scale" },
     { PRINT,    OPT_O, "byobs" },
     { PRINT,    OPT_L, "long" },
     { PRINT,    OPT_N, "no-dates" },
     { PRINT,    OPT_T, "ten" },
     { PROBIT,   OPT_R, "robust" },
+    { PROBIT,   OPT_V, "verbose" },
     { QUIT,     OPT_X, "exit" },
+    { RESTRICT, OPT_B, "bootstrap" },
     { RESTRICT, OPT_Q, "quiet" },
     { SCATTERS, OPT_L, "with-lines" },
     { SETINFO,  OPT_C, "continuous" },
@@ -240,7 +271,6 @@ struct gretl_option gretl_opts[] = {
     { WLS,      OPT_Q, "quiet" },
     { XTAB,     OPT_C, "column" },
     { XTAB,     OPT_R, "row" },
-    { XTAB,     OPT_X, "chi-square" },
     { XTAB,     OPT_Z, "zeros" },
     { 0,        0L,    NULL }
 };
@@ -552,7 +582,7 @@ gretlopt get_gretl_options (char *line, int *err)
     char cmdword[9] = {0};
     int ci, myerr = 0;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     if (err != NULL) {
 	*err = 0;
@@ -575,17 +605,19 @@ gretlopt get_gretl_options (char *line, int *err)
 	ci = NLS;
     } else if (strstr(line, "end mle")) {
 	ci = MLE;
+    } else if (strstr(line, "end gmm")) {
+	ci = GMM;
     } else {
 	ci = gretl_command_number(cmdword);
     }
 
     /* some commands do not take a "flag", and "-%c" may have
        some other meaning */
-    if (ci == 0 || ci == GENR) {
+    if (ci == 0 || ci == GENR || ci == PRINTF) {
 	return oflags;
     }
 
-    if (ci != SETINFO) {
+    if (ci != SETINFO && ci != TABPRINT && ci != EQNPRINT) {
 	/* try for short-form options (e.g. "-o") */
 	opt = get_short_opts(line, ci, &myerr);
 	if (!myerr && opt) {
@@ -633,11 +665,6 @@ const char *print_flags (gretlopt oflags, int ci)
 	return flagstr;
     }
 
-    if (ci == FNCALL) {
-	/* no options for user-defined functions, yet */
-	return flagstr;
-    }
-
     if (ci == QUIT || ci == GENR) {
 	/* any option flags are "hidden" */
 	return flagstr;
@@ -675,3 +702,29 @@ int check_for_loop_only_options (int ci, gretlopt opt, PRN *prn)
 
     return ret;
 }
+
+/**
+ * incompatible_options:
+ * @opt: option flags to be tested.
+ * @test: bitwise OR of flags that are incompatible in context.
+ * 
+ * Returns: %E_BADOPT if @opt contains more than one of the flags
+ * in @test, otherwise 0.
+ */
+
+int incompatible_options (gretlopt opt, gretlopt test)
+{
+    int optcount = 0;
+    gretlopt o;
+
+    for (o=OPT_A; o<=OPT_Z; o=o<<1) {
+	if ((opt & o) && (test & o)) {
+	    optcount++;
+	    if (optcount > 1) {
+		return E_BADOPT;
+	    }
+	}
+    }
+
+    return 0;
+}
