@@ -55,7 +55,9 @@ enum {
     P_EVIEWS_IMPORT,
     P_STATA_IMPORT,
     P_JMULTI_IMPORT,
-    P_ZIPFILE
+    P_ZIPFILE,
+    P_OPROBIT,
+    P_ARBOND
 } plugin_codes;
 
 struct plugin_info {
@@ -97,7 +99,9 @@ struct plugin_info plugins[] = {
     { P_EVIEWS_IMPORT,   "eviews_import" },
     { P_STATA_IMPORT,    "stata_import" },
     { P_JMULTI_IMPORT,   "jmulti_import" },
-    { P_ZIPFILE,         "gretlzip" }
+    { P_ZIPFILE,         "gretlzip" },
+    { P_OPROBIT,         "oprobit" },
+    { P_ARBOND,          "arbond" }
 };  
 
 struct plugin_function plugin_functions[] = { 
@@ -142,10 +146,8 @@ struct plugin_function plugin_functions[] = {
     { "range_mean_graph", P_RANGE_MEAN },
 
     /* statistical tables */
-    { "norm_lookup",  P_STATS_TABLES },
-    { "t_lookup",     P_STATS_TABLES },
-    { "chisq_lookup", P_STATS_TABLES },
-    { "dw_lookup",    P_STATS_TABLES },
+    { "dw_lookup",       P_STATS_TABLES },
+    { "rank_sum_lookup", P_STATS_TABLES },
 
     /* SUR, 3SLS, FIML */
     { "system_estimate", P_SYSEST },
@@ -162,6 +164,7 @@ struct plugin_function plugin_functions[] = {
     { "tobit_estimate",    P_TOBIT },
     { "garch_model",       P_GARCH },
     { "poisson_estimate",  P_POISSON },
+    { "ordered_estimate",  P_OPROBIT },
 
     /* audio graphs etc */
     { "midi_play_graph",   P_AUDIO },
@@ -171,7 +174,8 @@ struct plugin_function plugin_functions[] = {
     { "mackinnon_pvalue",  P_URCDIST },
 
     /* kernel density estimation */
-    { "kernel_density",    P_KERNEL },
+    { "kernel_density",        P_KERNEL },
+    { "array_kernel_density",  P_KERNEL },
 
     /* Hurst exponent estimation */
     { "hurst_exponent",    P_FRACTAL },
@@ -184,6 +188,9 @@ struct plugin_function plugin_functions[] = {
     { "gretl_unzip_file",   P_ZIPFILE},
     { "gretl_is_zipfile",   P_ZIPFILE},
 
+    /* Arellano-Bond estimation */
+    { "arbond_estimate",    P_ARBOND},
+
     /* sentinel */
     { NULL, 0 }
 };
