@@ -40,8 +40,6 @@ void text_print_model_confints (const CoeffIntervals *cf, PRN *prn);
 
 void print_freq (const FreqDist *freq, PRN *prn);
 
-void printcorr (const VMatrix *corrmat, PRN *prn);
-
 void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn);
 
 void print_smpl (const DATAINFO *pdinfo, 
@@ -63,7 +61,8 @@ void maybe_list_vars (const DATAINFO *pdinfo, PRN *prn);
 
 int get_printdata_blocks (void);
 
-int printdata (const int *list, const double **Z, const DATAINFO *pdinfo, 
+int printdata (const int *list, const char *mstr,
+	       const double **Z, const DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn);
 
 int print_data_sorted (const int *list, const int *obsvec, 
@@ -82,6 +81,10 @@ int text_print_forecast (const FITRESID *fr,
 			 double ***pZ, DATAINFO *pdinfo, 
 			 gretlopt opt, PRN *prn);
 
+void print_iter_info (int iter, double crit, int type, int k, 
+		      const double *b, const double *g, 
+		      double sl, PRN *prn);
+
 void text_print_vmatrix (VMatrix *vmat, PRN *prn);
 
 void gretl_print_fullwidth_double (double x, int digits, PRN *prn);
@@ -99,7 +102,7 @@ void gretl_printxn (double x, int n, PRN *prn);
 int do_printf (const char *line, double ***pZ, 
 	       DATAINFO *pdinfo, PRN *prn);
 
-int generate_obs_markers (double ***pZ, DATAINFO *pdinfo, char *s);
+int generate_obs_markers (const char *s, double ***pZ, DATAINFO *pdinfo);
 
 int in_usa (void);
 
@@ -107,6 +110,8 @@ char *bufgets (char *s, size_t size, const char *buf);
 
 void bufgets_init (const char *buf);
 
+void bufgets_finalize (const char *buf);
+
 void scroll_pause (void);
 
 int scroll_pause_or_quit (void);
