@@ -38,6 +38,8 @@ double t_cdf_comp (double x, int df);
 
 double t_pvalue_2 (double x, int df);
 
+double t_critval (double a, int df);
+
 double chisq_cdf (double x, int df);
 
 double chisq_cdf_comp (double x, int df);
@@ -52,26 +54,37 @@ double f_critval (double a, int dfn, int dfd);
 
 double normal_cdf (double x);
 
+double normal_cdf_inverse (double x);
+
 double normal_pdf (double x);
 
+double normal_critval (double a);
+
 double log_normal_pdf (double x);
 
+double bvnorm_cdf (double a, double b, double rho);
+
 double gamma_cdf_comp (double s1, double s2, double x, int control);
 
 double tcrit95 (int df);
 
 double rhocrit95 (int n);
 
-double batch_pvalue (const char *str, 
-		     const double **Z, const DATAINFO *pdinfo, 
-                     PRN *prn, gretlopt opt);
+double cephes_gamma (double x);
 
-void interact_pvalue (void);
+double cephes_lgamma (double x); 
 
-int print_critical (const char *line, PRN *prn);
+double gretl_get_pvalue (char st, const double *p);
 
-double genr_get_critical (const char *line, const double **Z, 
-			  const DATAINFO *pdinfo);
+double gretl_get_cdf (char st, double *p);
+
+double gretl_get_critval (char st, double *p);
+
+int batch_pvalue (const char *str, 
+		  double ***pZ, DATAINFO *pdinfo, 
+		  PRN *prn);
+
+void print_pvalue (char st, double *p, double pv, PRN *prn);
 
 #endif /* PVALUES_H */
 
