@@ -20,11 +20,16 @@
 #ifndef QR_ESTIMATE_H
 #define QR_ESTIMATE_H
 
-int gretl_qr_regress (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
-		      gretlopt opts);
+int gretl_qr_regress (MODEL *pmod, const double **Z, DATAINFO *pdinfo,
+		      gretlopt opt);
 
-int qr_tsls_vcv (MODEL *pmod, const double **Z, gretlopt opts);
+int qr_tsls_vcv (MODEL *pmod, const double **Z, const DATAINFO *pdinfo,
+		 gretlopt opt);
 
-int redundant_var (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, int **droplist);
+double hac_weight (int kern, int h, int i);
+
+double qs_hac_weight (double bt, int i);
+
+int newey_west_bandwidth (const gretl_matrix *f, int kern, int *h, double *bt);
 
 #endif  /* QR_ESTIMATE_H */
