@@ -22,16 +22,9 @@
 #include "libgretl.h"
 
 #include <time.h>
+#include <glib.h>
 
-#ifdef USE_GLIB2
-# include <glib.h>
-#else
-# include "mt19937ar.c"
-#endif
-
-#ifdef USE_GLIB2
 static GRand *gretl_rand;
-#endif
 
 static unsigned int useed;
 
@@ -49,12 +42,8 @@ unsigned int get_gretl_random_seed (void)
 void gretl_rand_init (void)
 {
     useed = time(NULL);
-#ifdef USE_GLIB2
     gretl_rand = g_rand_new();
     gretl_rand_set_seed((guint32) useed);
-#else
-    init_genrand(useed);
-#endif
 }
 
 /**
@@ -74,23 +63,80 @@ void gretl_rand_set_seed (unsigned int seed)
     } else {
 	useed = seed;
     }
-#ifdef USE_GLIB2
     g_rand_set_seed(gretl_rand, useed);
+}
+
+#define gretl_one_uniform() (g_rand_double_range(gretl_rand, 0, 1))
+
+/**
+ * gretl_one_snormal:
+ *
+ */
+
+double gretl_one_snormal (void) 
+{
+    double x, y, z;
+
+ tryagain:
+    x = gretl_one_uniform();
+    y = gretl_one_uniform();
+    z = sqrt(-2. * log(x));
+    if (isnan(z) || isinf(z)) {
+	goto tryagain;
+    }
+
+    return (z * cos(M_2PI * y));
+}
+
+/**
+ * gretl_two_snormals:
+ *
+ */
+
+#define MOD_POLAR 0
+
+void gretl_two_snormals (double *z1, double *z2) 
+{
+    double x, y, z;
+
+#if MOD_POLAR
+ tryagain:
+    x = 2 * gretl_one_uniform() - 1;
+    y = 2 * gretl_one_uniform() - 1;
+    z = x*x + y*y;
+    if (z >= 1) {
+	goto tryagain;
+    }
+
+    z = sqrt(-2 * log(z)/z);
+    *z1 = z * x;
+    *z2 = z * y;
 #else
-    init_genrand(useed);
+ tryagain:
+    x = gretl_one_uniform();
+    y = gretl_one_uniform();
+    z = sqrt(-2 * log(x));
+    if (isnan(z) || isinf(z)) {
+	goto tryagain;
+    }
+
+    y *= M_2PI;
+
+    *z1 = z * cos(y);
+    *z2 = z * sin(y);
 #endif
 }
 
 /**
  * gretl_uniform_dist_minmax:
- * @a: target array
- * @t1: start of the fill range
- * @t2: end of the fill range
+ * @a: target array.
+ * @t1: start of the fill range.
+ * @t2: end of the fill range.
  * @min: lower bound of range.
  * @min: upper bound of range.
  *
  * Fill the selected subset of array @a with pseudo-random drawings
- * from the uniform distribution on @min-@max, using the Mersenne
+ * from the uniform distribution on @min to @max, using the Mersenne
  * Twister.
  *
  * Returns: 0 on success, 1 on invalid input.
@@ -109,11 +155,7 @@ int gretl_uniform_dist_minmax (double *a, int t1, int t2,
     }
 
     for (t=t1; t<=t2; t++) {
-#ifdef USE_GLIB2
 	a[t] = g_rand_double_range(gretl_rand, min, max);
-#else
-	a[t] = min + genrand_int32() * (max / 4294967296.0);
-#endif 
     }
 
     return 0;
@@ -135,11 +177,7 @@ void gretl_uniform_dist (double *a, int t1, int t2)
     int t;
 
     for (t=t1; t<=t2; t++) {
-#ifdef USE_GLIB2
-	a[t] = g_rand_double_range(gretl_rand, 0.0, 1.0);
-#else
-	a[t] = genrand_int32() * (1.0 / 4294967296.0);
-#endif 
+	a[t] = gretl_one_uniform();
     }
 }
 
@@ -157,24 +195,24 @@ void gretl_uniform_dist (double *a, int t1, int t2)
 
 void gretl_normal_dist (double *a, int t1, int t2) 
 {
-    double x, y, z;
+#if 1
+    double z1, z2;
     int t;
 
     for (t=t1; t<=t2; t++) {
-    tryagain:
-#ifdef USE_GLIB2
-	x = g_rand_double(gretl_rand);
-	y = g_rand_double(gretl_rand);
-#else
-	x = genrand_real2();
-	y = genrand_real2();
-#endif
-	z = sqrt(-2. * log(x));
-	if (isnan(z) || isinf(z)) {
-	    goto tryagain;
+	gretl_two_snormals(&z1, &z2);
+	a[t] = z1;
+	if (t < t2) {
+	    a[++t] = z2;
 	}
-	a[t] = z * cos(2. * M_PI * y);
     }
+#else
+    int t;
+    
+    for (t=t1; t<=t2; t++) {
+	a[t] = gretl_one_snormal();
+    }
+#endif
 }
 
 /**
@@ -234,7 +272,7 @@ gretl_normal_dist_with_params (double *a, int t1, int t2,
 
 int gretl_chisq_dist (double *a, int t1, int t2, int v) 
 {
-    double x, y, z;
+    double z;
     int i, t;
 
     if (v < 1) {
@@ -244,19 +282,7 @@ int gretl_chisq_dist (double *a, int t1, int t2, int v)
     for (t=t1; t<=t2; t++) {
 	a[t] = 0.0;
 	for (i=0; i<v; i++) {
-	tryagain:
-#ifdef USE_GLIB2
-	    x = g_rand_double(gretl_rand);
-	    y = g_rand_double(gretl_rand);
-#else
-	    x = genrand_real2();
-	    y = genrand_real2();
-#endif
-	    z = sqrt(-2. * log(x));
-	    if (isnan(z) || isinf(z)) {
-		goto tryagain;
-	    }
-	    z *= cos(2. * M_PI * y);
+	    z = gretl_one_snormal();
 	    a[t] += z * z;
 	}
     }
@@ -306,6 +332,96 @@ int gretl_t_dist (double *a, int t1, int t2, int v)
     return 0;
 }
 
+/**
+ * gretl_binomial_dist:
+ * @a: target array.
+ * @t1: start of the fill range.
+ * @t2: end of the fill range.
+ * @n: number of trials.
+ * @p: success probability per trial.
+ *
+ * Fill the selected range of array @a with pseudo-random drawings
+ * from the binomial distribution with parameters @n and @p.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_binomial_dist (double *a, int t1, int t2, int n, double p) 
+{
+    double *b;
+    int i, t;
+
+    if (n < 1 || p <= 0 || p >= 1) {
+	return E_INVARG;
+    }
+
+    b = malloc(n * sizeof *b);
+    if (b == NULL) {
+	return E_ALLOC;
+    }
+
+    for (t=t1; t<=t2; t++) {
+	a[t] = 0.0;
+	gretl_uniform_dist(b, 0, n - 1);
+	for (i=0; i<n; i++) {
+	    if (b[i] <= p) {
+		a[t] += 1;
+	    }
+	}
+    }
+
+    free(b);
+
+    return 0;
+}
+
+/* Poisson rv with mean m */
+
+static double genpois (double m)
+{
+    double x;
+
+    if (m > 200) {
+	x = (m + 0.5) + sqrt(m) * gretl_one_snormal();
+	x = floor(x);
+    } else {
+	int y = 0;
+
+	x = exp(m) * gretl_one_uniform();
+	while (x > 1) {
+	    y++;
+	    x *= gretl_one_uniform();
+	}
+	x = (double) y;
+    }
+
+    return x;
+}
+
+/**
+ * gretl_poisson_dist:
+ * @a: target array.
+ * @t1: start of the fill range.
+ * @t2: end of the fill range.
+ * @m: mean (see below).
+ * @vec: should be 1 if @m is an array, else 0.
+ *
+ * Fill the selected range of array @a with pseudo-random drawings
+ * from the Poisson distribution with a mean determined by 
+ * @m, which can either be a pointer to a scalar, or an array
+ * of length greater than or equal to @t2 + 1.  
+ */
+
+void gretl_poisson_dist (double *a, int t1, int t2, double *m,
+			 int vec) 
+{
+    int t;
+
+    for (t=t1; t<=t2; t++) {
+	a[t] = (vec)? genpois(m[t]) : genpois(*m);
+    }
+}
+
 /**
  * gretl_rand_int_max:
  * @max: the maximum value (open)
@@ -316,11 +432,7 @@ int gretl_t_dist (double *a, int t1, int t2, int v)
 
 unsigned int gretl_rand_int_max (unsigned int max)
 {
-#ifdef USE_GLIB2
     return g_rand_int_range(gretl_rand, 0, max);
-#else
-    return genrand_int32() * (max / 4294967296.0);
-#endif
 }
 
 /**
@@ -332,11 +444,7 @@ unsigned int gretl_rand_int_max (unsigned int max)
 
 unsigned int gretl_rand_int (void)
 {
-#ifdef USE_GLIB2
     return g_rand_int(gretl_rand);
-#else
-    return genrand_int32();
-#endif
 }
 
 /**
@@ -347,11 +455,7 @@ unsigned int gretl_rand_int (void)
 
 void gretl_rand_free (void)
 {
-#ifdef USE_GLIB2
     g_rand_free(gretl_rand);
-#else
-    return;
-#endif
 }
 
 
