@@ -22,6 +22,8 @@
 
 void gretl_rand_set_seed (unsigned int seed);
 
+double gretl_one_snormal (void);
+
 void gretl_uniform_dist (double *a, int t1, int t2);
 
 int gretl_uniform_dist_minmax (double *a, int t1, int t2,
@@ -36,6 +38,11 @@ int gretl_chisq_dist (double *a, int t1, int t2, int v);
 
 int gretl_t_dist (double *a, int t1, int t2, int v);
 
+int gretl_binomial_dist (double *a, int t1, int t2, int n, double p);
+
+void gretl_poisson_dist (double *a, int t1, int t2, double *m,
+			 int vec);
+
 unsigned int gretl_rand_int_max (unsigned int max);
 
 unsigned int gretl_rand_int (void);
