@@ -23,12 +23,7 @@
 
 #include <errno.h>
 #include <time.h>
-
-#if defined(USE_GLIB2)
-# include <glib.h>
-#else
-# include <fnmatch.h>
-#endif
+#include <glib.h>
 
 /**
  * string_is_blank:
@@ -408,6 +403,93 @@ char *gretl_strndup (const char *src, size_t n)
     return targ;
 }
 
+/**
+ * gretl_strdup_printf:
+ * @template: as in printf().
+ * @Varargs: arguments to be printed.
+ *
+ * Print the arguments according to @format.
+ * 
+ * Returns: allocated result of the printing, or %NULL on failure.
+ */
+
+char *gretl_strdup_printf (const char *template, ...)
+{
+    va_list args;
+    int plen, bsize = 2048;
+    char *buf;
+
+    buf = malloc(bsize);
+    if (buf == NULL) {
+	return NULL;
+    }
+
+    memset(buf, 0, 1);
+
+    va_start(args, template);
+    plen = vsnprintf(buf, bsize, template, args);
+    va_end(args);
+
+    if (plen >= bsize) {
+	fputs("gretl_strdup_printf warning: string was truncated\n",
+	      stderr);
+    }
+
+    return buf;
+}
+
+/**
+ * gretl_str_expand:
+ * @orig: pointer to the base string.
+ * @add: the string to be added.
+ * @sep: string to be interpolated, or %NULL.
+ *
+ * Creates a newly allocated string built by concatenating
+ * @orig and @add, with @sep interpolated unless @sep is
+ * %NULL, and replaces the content of @orig with the new string.
+ * As a special case, if @orig is %NULL, or if the content of
+ * @orig is %NULL, we just duplicate @add.
+ *
+ * Returns: the reallocated string, or %NULL on failure.  In case
+ * of failure the content of @orig is freed, if @orig is not %NULL,
+ * to avoid memory leakage.
+ */
+
+char *gretl_str_expand (char **orig, const char *add, const char *sep)
+{
+    char *targ;
+    int n;
+
+    if (add == NULL) {
+	return NULL;
+    }
+
+    if (orig == NULL || *orig == NULL) {
+	return gretl_strdup(add);
+    }
+
+    n = strlen(*orig);
+    if (sep != NULL) {
+	n += strlen(sep);
+    }
+    n += strlen(add) + 1;
+
+    targ = realloc(*orig, n);
+    if (targ == NULL) {
+	free(*orig);
+	*orig = NULL;
+	return NULL;
+    }
+
+    if (sep != NULL) {
+	strcat(targ, sep);
+    }
+    strcat(targ, add);
+    *orig = targ;
+
+    return targ;
+}
+
 #define is_word_char(c) (isalnum((unsigned char) c) || c == '_')
 
 /**
@@ -464,6 +546,52 @@ char *gretl_word_strdup (const char *src, const char **ptr)
     return targ;
 }
 
+/**
+ * gretl_string_split:
+ * @s: the source string.
+ * @n: location to receive the number of substrings.
+ *
+ * Parses @s into a set of zero or more substrings, separated
+ * by one or more spaces, and creates an array of those substrings. 
+ * On sucessful exit, @n holds the number of substrings. 
+ *
+ * Returns: the allocated array or %NULL in case of failure.
+ */
+
+char **gretl_string_split (const char *s, int *n)
+{
+    int i, k, m = count_fields(s);
+    char *word;
+    char **S;
+
+    *n = 0;
+
+    if (m == 0) {
+	return NULL;
+    }
+
+    S = strings_array_new(m);
+    if (S == NULL) {
+	return NULL;
+    }
+
+    for (i=0; i<m; i++) {
+	s += strspn(s, " ");
+	k = strcspn(s, " ");
+	word = gretl_strndup(s, k);
+	if (word == NULL) {
+	    free_strings_array(S, m);
+	    return NULL;
+	}
+	S[i] = word;
+	s += k;
+    }
+
+    *n = m;
+
+    return S;
+}
+
 /**
  * gretl_double_from_string:
  * @s: the source string.
@@ -756,7 +884,7 @@ int top_n_tail (char *str)
 	    shift_string_left(str, i);
 	}
 
-	/* then replace backslash, if present */
+	/* replace backslash, if present */
 	len = strlen(str);
 	if (str[len - 1] == '\\') {
 	    str[len - 1] = ' ';
@@ -887,30 +1015,34 @@ char *tailstrip (char *str)
 
 char *compress_spaces (char *s)
 {
+    int i = 0, inquote = 0;
     char *p, *q;
 
     if (s == NULL || *s == 0) {
 	return s;
     }
 
-    if (strchr(s, '"') != NULL) {
-	/* don't mess with literals */
-	return s;
-    }
-
     p = q = s;
 
     while (*s) {
-	if (*s == '\t') *s = ' '; /* trash tabs */
-	if (*s == ' ') {
-	    p = s + 1;
-	    if (*p == 0) break;
-	    while (*p == ' ') p++;
-	    if (p - s > 1) {
-		memmove(s + 1, p, strlen(p) + 1);
+	if (*s == '"' && (i == 0 || *(s-1) != '\\')) {
+	    inquote = !inquote;
+	}
+	if (!inquote) {
+	    if (*s == '\t') {
+		*s = ' '; /* trash tabs */
+	    }
+	    if (*s == ' ') {
+		p = s + 1;
+		if (*p == 0) break;
+		while (*p == ' ') p++;
+		if (p - s > 1) {
+		    memmove(s + 1, p, strlen(p) + 1);
+		}
 	    }
 	}
 	s++;
+	i++;
     }
 
     return q;
@@ -985,6 +1117,40 @@ char **strings_array_new (int nstrs)
     return s;
 }
 
+/**
+ * strings_array_add:
+ * @pS: pointer to strings array.
+ * @n: location of present number of strings in array.
+ * @p: string to add to array.
+ *
+ * Allocates storage for an extra member of @S and adds a
+ * copy of string @p in the last position.  On success,
+ * the content of @n is incremented by 1.
+ * 
+ * Returns: 0 on success, %E_ALLOC on failure.
+ */
+
+int strings_array_add (char ***pS, int *n, const char *p)
+{
+    char **Tmp;
+    int m = *n;
+
+    Tmp = realloc(*pS, (m + 1) * sizeof *Tmp);
+    if (Tmp == NULL) {
+	return E_ALLOC;
+    }
+
+    *pS = Tmp;
+    Tmp[m] = gretl_strdup(p);
+
+    if (Tmp[m] == NULL) {
+	return E_ALLOC;
+    } else {
+	*n += 1;
+	return 0;
+    }
+}
+
 /**
  * strings_array_new_with_length:
  * @nstrs: number of strings in array.
@@ -1208,7 +1374,7 @@ void modify_date_for_csv (char *s, int pd)
     if (pd == 4) {
 	charsub(s, ':', 'Q');
     } else {
-	charsub(s, ':', '-');
+	charsub(s, ':', 'M');
     }
 }
 
@@ -1232,18 +1398,8 @@ void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn)
 	ntodate_full(tmp, t, pdinfo);
 	if (quarterly_or_monthly(pdinfo)) {
 	    modify_date_for_csv(tmp, pdinfo->pd);
-	    if (pdinfo->delim == ',') {
-		pprintf(prn, "\"%s\"%c", tmp, pdinfo->delim);
-	    } else {
-		pprintf(prn, "%s%c", tmp, pdinfo->delim);
-	    }
-	} else {
-	    if (pdinfo->delim == ',') {
-		pprintf(prn, "\"'%s\"%c", tmp, pdinfo->delim);
-	    } else {
-		pprintf(prn, "%s%c", tmp, pdinfo->delim);
-	    }
 	}
+	pprintf(prn, "%s%c", tmp, pdinfo->delim);
     }
 }
 	
@@ -1293,41 +1449,33 @@ int gretl_xml_validate (const char *s)
  * gretl_xml_encode:
  * @str: NUL-terminated source string.
  *
- * Returns: an allocated re-write of @buf, with characters that are
+ * Returns: an allocated re-write of @str, with characters that are
  * special in XML encoded as character entities.  See also
  * gretl_xml_validate().
  */
 
 char *gretl_xml_encode (const char *str)
 {
-    char *xmlstr, *p;
+    char *targ, *p;
     const char *s = str;
-    size_t sz = strlen(str) + 1;
-
-#ifdef XML_DEBUG
-    fprintf(stderr, "gretl_xml_encode: original buffer size=%d\n", sz);
-#endif
+    int len = strlen(s) + 1;
 
     while (*s) {
-	if (*s == '&') sz += 4;
-	else if (*s == '<') sz += 3;
-	else if (*s == '>') sz += 3;
-	else if (*s == '"') sz += 5;
+	if (*s == '&') len += 4;
+	else if (*s == '<') len += 3;
+	else if (*s == '>') len += 3;
+	else if (*s == '"') len += 5;
 	s++;
     }
 
-    xmlstr = malloc(sz);
-    if (xmlstr == NULL) {
+    targ = malloc(len);
+    if (targ == NULL) {
 	sprintf(gretl_errmsg, _("out of memory in XML encoding"));
 	return NULL;
     }
 
-#ifdef XML_DEBUG
-    fprintf(stderr, "gretl_xml_encode: malloc'd xmlstr at size %d\n", sz);
-#endif
-
     s = str;
-    p = xmlstr;
+    p = targ;
     
     while (*s) {
 	if (*s == '&') {
@@ -1348,13 +1496,81 @@ char *gretl_xml_encode (const char *str)
 	s++;
     }
 
-    xmlstr[sz-1] = '\0';
+    targ[len-1] = '\0';
 
 #ifdef XML_DEBUG
-    fprintf(stderr, "done gretl_xml_encode: xmlstr='%s'\n", xmlstr);
+    fprintf(stderr, "done gretl_xml_encode: targ='%s'\n", targ);
 #endif
 
-    return xmlstr;
+    return targ;
+}
+
+/**
+ * gretl_xml_encode_to_buf:
+ * @targ: target buffer.
+ * @src: NUL-terminated source string.
+ * @n: size of @targ in bytes.
+ *
+ * Writes into @targ a version of @src in which characters that are
+ * special in XML are encoded as character entities.  See also
+ * gretl_xml_encode() for the case where the encoding of @src is
+ * of unknown size at compile time.
+ *
+ * Returns: 0 on success or 1 on error.  An error occurs if (a) the 
+ * encoded version of @src is longer than @n bytes (allowing for NUL 
+ * termination), or (b) @src does not validate as UTF-8.  On error
+ * the conversion is not done.  
+ */
+
+int gretl_xml_encode_to_buf (char *targ, const char *src, int n)
+{
+    const char *s = src;
+    int len = strlen(s) + 1;
+
+    if (!g_utf8_validate(src, -1, NULL)) {
+	fprintf(stderr, "gretl_xml_encode_to_buf: source not UTF-8\n");
+	return 1;
+    }
+
+    while (*s) {
+	if (*s == '&') len += 4;
+	else if (*s == '<') len += 3;
+	else if (*s == '>') len += 3;
+	else if (*s == '"') len += 5;
+	s++;
+    }
+
+    *targ = '\0';
+    
+    if (len > n) {
+	fprintf(stderr, "gretl_xml_encode_to_buf: buffer too small\n");
+	return 1;
+    }
+
+    s = src;
+    
+    while (*s) {
+	if (*s == '&') {
+	    strcpy(targ, "&amp;");
+	    targ += 5;
+	} else if (*s == '<') {
+	    strcpy(targ, "&lt;");
+	    targ += 4;
+	} else if (*s == '>') {
+	    strcpy(targ, "&gt;");
+	    targ += 4;
+	} else if (*s == '"') {
+	    strcpy(targ, "&quot;");
+	    targ += 6;
+	} else {
+	    *targ++ = *s;
+	}
+	s++;
+    }
+
+    *targ = '\0';
+
+    return 0;
 }
 
 static char x2c (char *s) 
@@ -1554,8 +1770,6 @@ const char *path_last_element (const char *path)
     return p;
 }
 
-#if defined(USE_GLIB2)
-
 int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
 {
     GPatternSpec *pspec;
@@ -1590,39 +1804,6 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
     return list;
 }
 
-#elif defined(HAVE_FNMATCH_H) 
-
-int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
-{
-    int *list = NULL;
-    int i, n = 0;
-
-    for (i=1; i<pdinfo->v; i++) { 
-	if (var_is_series(pdinfo, i) &&
-	    fnmatch(pattern, pdinfo->varname[i], 0) == 0) {
-	    n++;
-	}
-    }
-
-    if (n > 0) {
-	list = malloc((n + 1) * sizeof *list);
-	if (list != NULL) {
-	    int j = 1;
-
-	    list[0] = n;
-	    for (i=1; i<pdinfo->v; i++) { 
-		if (var_is_series(pdinfo, i) &&
-		    fnmatch(pattern, pdinfo->varname[i], 0) == 0) {
-		    list[j++] = i;
-		}
-	    }
-	}
-    }
-
-    return list;
-}
-
-#endif
     
 
 
