@@ -69,8 +69,14 @@ char *gretl_strdup (const char *src);
 
 char *gretl_strndup (const char *src, size_t n);
 
+char *gretl_strdup_printf (const char *template, ...);
+
 char *gretl_word_strdup (const char *src, const char **ptr);
 
+char **gretl_string_split (const char *s, int *n);
+
+char *gretl_str_expand (char **orig, const char *add, const char *sep);
+
 double gretl_double_from_string (const char *s, const char **ptr);
 
 char *charsub (char *str, char find, char repl);
@@ -101,6 +107,8 @@ char *safecpy (char *targ, const char *src, int n);
 
 char **strings_array_new (int nstrs);
 
+int strings_array_add (char ***pS, int *n, const char *p);
+
 char **strings_array_new_with_length (int nstrs, int len);
 
 char **strings_array_dup (char **strs, int n);
@@ -125,6 +133,8 @@ int gretl_xml_validate (const char *s);
 
 char *gretl_xml_encode (const char *str);
 
+int gretl_xml_encode_to_buf (char *targ, const char *src, int n);
+
 void unescape_url (char *url);
 
 char *make_varname_unique (char *vname, int v, DATAINFO *pdinfo);
@@ -138,8 +148,6 @@ char *build_path (char *targ, const char *dirname, const char *fname,
 
 const char *path_last_element (const char *path);
 
-#if defined(USE_GLIB2) || defined (HAVE_FNMATCH_H)
 int *varname_match_list (const DATAINFO *pdinfo, const char *pattern);
-#endif
 
 #endif /* STRUTILS_H */
