@@ -23,6 +23,7 @@
 #include "libset.h"
 #include "gretl_func.h"
 #include "gretl_panel.h"
+#include "cmd_private.h"
 
 #define SUBDEBUG 0
 
@@ -51,6 +52,10 @@ static const DATAINFO *peerinfo;
 
 #define SUBMASK_SENTINEL 127
 
+static int 
+restore_sample_from_subinfo (double ***pZ, DATAINFO **ppdinfo,
+			     ExecState *state);
+
 /* accessors for full dataset, when sub-sampled */
 
 double ***fetch_full_Z (void)
@@ -60,6 +65,9 @@ double ***fetch_full_Z (void)
 
 void reset_full_Z (double ***pZ)
 {
+#if SUBDEBUG
+    fprintf(stderr, "reset_full_Z -> %p\n", (void *) *pZ);
+#endif    
     fullZ = *pZ;
 }
 
@@ -95,10 +103,11 @@ static int get_submask_length (const char *s)
 
 char *copy_subsample_mask (const char *src)
 {
-    int n = get_submask_length(src);
     char *ret = NULL;
 
     if (src != NULL) {
+	int n = get_submask_length(src);
+
 	ret = malloc(n * sizeof *ret);
 	if (ret != NULL) {
 	    memcpy(ret, src, n);
@@ -195,15 +204,26 @@ void maybe_free_full_dataset (const DATAINFO *pdinfo)
     }
 }
 
-static void relink_full_dataset (double ***pZ, DATAINFO **ppdinfo)
+/* we do this on "restore full sample" */
+
+static void 
+relink_to_full_dataset (double ***pZ, DATAINFO **ppdinfo, int nullit)
 {
     *pZ = fullZ;
     *ppdinfo = fullinfo;
     (*ppdinfo)->submode = 0;
 
-    fullZ = NULL;
-    fullinfo = NULL;
-    peerinfo = NULL;
+#if SUBDEBUG
+    fprintf(stderr, "relink_to_full_dataset: fullZ = %p, fullinfo = %p\n",
+	    (void *) fullZ, (void *) fullinfo);
+#endif
+
+    if (nullit) {
+	fullZ = NULL;
+	fullinfo = NULL;
+	peerinfo = NULL;
+	fullZ = 0;
+    }
 }
 
 /* sync malloced elements of the fullinfo struct that might
@@ -236,6 +256,10 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 	/* sync, in case anything has moved */
 	sync_dataset_elements(pdinfo);
 
+	if (pmod->submask != NULL) {
+	    free(pmod->submask);
+	}
+
 	pmod->submask = copy_subsample_mask(pdinfo->submask);
 	if (pmod->submask == NULL) {
 	    err = E_ALLOC;
@@ -245,26 +269,31 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
     return err;
 }
 
-/* apparatus for updating full dataset when restoring full sample
-   after sub-sampling */
+/* Apparatus for updating full dataset when restoring full sample
+   after sub-sampling.  
+*/
 
 static void
 update_full_data_values (const double **subZ, const DATAINFO *pdinfo)
 {
-    int i, t;
+    int i, s, t;
 
-    for (i=1; i<fullinfo->v; i++) {
-	int subt = 0;
+#if SUBDEBUG
+    fprintf(stderr, "update_full_data_values: fullZ=%p, subZ=%p, pdinfo=%p\n",
+	    (void *) fullZ, (void *) subZ, (void *) pdinfo);
+#endif
 
+    for (i=1; i<fullinfo->v && i<pdinfo->v; i++) {
+	s = 0;
 	if (var_is_scalar(pdinfo, i)) {
 	    fullZ[i][0] = subZ[i][0];
 	} else {
 	    for (t=0; t<fullinfo->n; t++) {
 		if (pdinfo->submask[t] == 1) {
-		    fullZ[i][t] = subZ[i][subt++];
+		    fullZ[i][t] = subZ[i][s++];
 		} else if (pdinfo->submask[t] == 'p') {
 		    /* skip panel padding (?) */
-		    subt++;
+		    s++;
 		}
 	    }
 	}
@@ -301,7 +330,7 @@ static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
     int V0 = fullinfo->v;
     int N = fullinfo->n;
     double **newZ = NULL;
-    int i, t, subt;
+    int i, t, s;
     int err = 0;
 
     if (V1 <= V0) {
@@ -312,6 +341,13 @@ static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
 	return E_NOMERGE;
     }
 
+#if SUBDEBUG
+    fprintf(stderr, "add_new_vars_to_full:\n");
+    fprintf(stderr, " V1 = pdinfo->v = %d; V0 = fullinfo->v = %d\n",
+	    V1, V0);
+    fprintf(stderr, " Z = %p, fullZ = %p\n", (void *) Z, (void *) fullZ);
+#endif
+
     /* allocate expanded data array */
     newZ = realloc(fullZ, V1 * sizeof *fullZ);
 
@@ -336,27 +372,18 @@ static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
 	return E_ALLOC;
     }
 
+#if SUBDEBUG
+    fprintf(stderr, "After expansion, fullZ = %p (%d vars)\n", (void *) fullZ,
+	    pdinfo->v);
+#endif
+
     for (i=V0; i<pdinfo->v; i++) {
 	if (var_is_scalar(pdinfo, i)) {
-	   fullZ[i][0] = Z[i][0]; 
-	}
-    }
-
-    subt = 0;
-
-    for (t=0; t<N; t++) {
-	if (pdinfo->submask[t]) {
-	    for (i=V0; i<pdinfo->v; i++) {
-		if (var_is_series(pdinfo, i)) {
-		    fullZ[i][t] = Z[i][subt];
-		}
-	    }
-	    subt++;
+	    fullZ[i][0] = Z[i][0]; 
 	} else {
-	    for (i=V0; i<pdinfo->v; i++) { 
-		if (var_is_series(pdinfo, i)) {
-		    fullZ[i][t] = NADBL;
-		}
+	    s = 0;
+	    for (t=0; t<N; t++) {
+		fullZ[i][t] = (pdinfo->submask[t])? Z[i][s++] : NADBL;
 	    }
 	}
     }
@@ -397,9 +424,41 @@ static char *make_current_sample_mask (DATAINFO *pdinfo)
     return currmask;
 }
 
+static void destroy_subsampled_datainfo (DATAINFO *pdinfo)
+{
+    clear_datainfo(pdinfo, CLEAR_SUBSAMPLE);
+#if SUBDEBUG
+    fprintf(stderr, "destroy_subsampled_datainfo: freeing at %p\n", 
+	    (void *) pdinfo);
+#endif
+    free(pdinfo);
+}
+
+/* marker for the case where restore_full_sample is called as
+   a preliminary for imposing a restriction */
+
+static int restricting;
+
+static void flag_restricting_on (void)
+{
+    restricting = 1;
+}
+
+static void flag_restricting_off (void)
+{
+    restricting = 0;
+}
+
+static int doing_restrict (void)
+{
+    return restricting;
+}
+
 /* restore_full_sample: 
  * @pZ: pointer to data array.  
- * @ppdinfo: address of data info pointer. 
+ * @ppdinfo: address of data info pointer.
+ * @state: structure representing program execution state,
+ * or %NULL. 
  *
  * Restore the full data range, undoing any sub-sampling that was
  * previously performed (either by shifting the endpoints of the
@@ -408,30 +467,32 @@ static char *make_current_sample_mask (DATAINFO *pdinfo)
  * Returns: 0 on success, non-zero error code on failure.
  */
 
-int restore_full_sample (double ***pZ, DATAINFO **ppdinfo)
+int restore_full_sample (double ***pZ, DATAINFO **ppdinfo,
+			 ExecState *state)
 {
+    DATAINFO *pdinfo = *ppdinfo;
     int err = 0;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
-#if SUBDEBUG
-    fprintf(stderr, "restore_full_sample: pZ=%p, ppdinfo=%p\n",
-	    (void *) pZ, (void *) ppdinfo);
-    fprintf(stderr, "*pZ=%p, *ppdinfo=%p\n",
-	    (void *) *pZ, (void *) *ppdinfo);
-#endif
-
-    /* simple case: merely resetting the sample starting
-       and ending points to full range */
     if (!complex_subsampled()) {
-	(*ppdinfo)->t1 = 0;
-	(*ppdinfo)->t2 = (*ppdinfo)->n - 1;
+	if (pdinfo->t1 != 0 || pdinfo->t2 != pdinfo->n - 1) {
+	    pdinfo->t1 = 0;
+	    pdinfo->t2 = pdinfo->n - 1;
 #if SUBDEBUG
-	fprintf(stderr, " reset t1 and t2; all done\n");
+	    fprintf(stderr, "restore_full_sample: reset t1 and t2\n");
 #endif
+	}
 	return 0;
     }
 
+#if SUBDEBUG
+    fprintf(stderr, "\nrestore_full_sample: pZ=%p, ppdinfo=%p\n",
+	    (void *) pZ, (void *) ppdinfo);
+    fprintf(stderr, " *pZ=%p, *ppdinfo=%p, state=%p\n",
+	    (void *) *pZ, (void *) pdinfo, (void *) state);
+#endif
+
     /* beyond this point we are doing a non-trivial restoration
        of a stored full dataset, which has previously been
        subsampled, e.g., by some boolean criterion 
@@ -439,23 +500,23 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo)
 
     /* reattach malloc'd elements of subsampled dataset,
        which may have moved */
-    sync_dataset_elements(*ppdinfo);
+    sync_dataset_elements(pdinfo);
 
     if (!err) {
 	/* update values for pre-existing series, which may have been
 	   modified via genr etc */
-	update_full_data_values((const double **) *pZ, *ppdinfo);
+	update_full_data_values((const double **) *pZ, pdinfo);
 
 	/* if case markers were added when subsampled, carry them back */
-	update_case_markers(*ppdinfo);
+	update_case_markers(pdinfo);
 
 	/* delete any newly added hidden vars */
-	err = dataset_destroy_hidden_variables(pZ, *ppdinfo, fullinfo->v);
+	err = dataset_destroy_hidden_variables(pZ, pdinfo, fullinfo->v);
 
 	/* in case any new vars were added when subsampled, try to merge
 	   them into the full dataset */
 	if (!err) {
-	    err = add_new_vars_to_full((const double **) *pZ, *ppdinfo);
+	    err = add_new_vars_to_full((const double **) *pZ, pdinfo);
 	}
     }
 
@@ -466,11 +527,27 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo)
 		_("Missing sub-sample information; can't merge data\n"));
     }
 
-    free_Z(*pZ, *ppdinfo);
-    clear_datainfo(*ppdinfo, CLEAR_SUBSAMPLE);
-    free(*ppdinfo);
+    /* destroy sub-sampled data array */
+    free_Z(*pZ, pdinfo);
 
-    relink_full_dataset(pZ, ppdinfo);
+    if (state != NULL && state->subinfo != NULL) {
+	if (doing_restrict()) {
+	    /* preliminary to sub-sampling inside a function */
+	    if (*ppdinfo != state->subinfo) {
+		destroy_subsampled_datainfo(*ppdinfo);
+	    }
+	    relink_to_full_dataset(pZ, ppdinfo, 0);
+	} else {
+	    /* inside a function, where the dataset was sub-sampled
+	       on entry to the function: restoring the "full sample" in
+	       effect means restoring the outer sub-sample.
+	    */
+	    err = restore_sample_from_subinfo(pZ, ppdinfo, state);
+	} 
+    } else {
+	destroy_subsampled_datainfo(*ppdinfo);
+	relink_to_full_dataset(pZ, ppdinfo, 1);
+    }
 
     return err;
 }
@@ -545,23 +622,22 @@ static int mask_from_temp_dummy (const char *line,
 				 char *mask)
 {
     char formula[MAXLEN];
-    int dnum, err;
+    double *x;
+    int err = 0;
 
-    /* + 4 to skip the command word "smpl" */
-    sprintf(formula, "__tmpmsk=%s", line + 4);
+    *formula = '\0';
+    strncat(formula, line + 4, MAXLEN - 1);
 
-    err = generate(formula, pZ, pdinfo, OPT_P, NULL);
-    if (err) {
-	return err;
+    x = generate_series(formula, pZ, pdinfo, &err);
+
+    if (!err) {
+	err = copy_dummy_to_mask(mask, x, pdinfo->n);
+	if (err) {
+	    sprintf(gretl_errmsg, _("'%s' is not a dummy variable"), "mask");
+	}
     }
 
-    dnum = varindex(pdinfo, "tmpmsk");
-    err = copy_dummy_to_mask(mask, (*pZ)[dnum], pdinfo->n);
-    if (err) {
-	sprintf(gretl_errmsg, _("'%s' is not a dummy variable"), "mask");
-    }
-
-    dataset_drop_variable(dnum, pZ, pdinfo);
+    free(x);
 
     return err;
 }
@@ -700,12 +776,22 @@ static int make_random_mask (const char *oldmask, const char *line,
     return err;
 }
 
-static void backup_full_dataset (double ***pZ, DATAINFO **ppdinfo,
+static void backup_full_dataset (double **Z, DATAINFO *pdinfo,
 				 const DATAINFO *newinfo)
 {
-    fullZ = *pZ;
-    fullinfo = *ppdinfo;
-    peerinfo = newinfo;
+    fullZ = Z;
+
+    if (pdinfo != NULL) {
+	fullinfo = pdinfo;
+    } 
+    if (newinfo != NULL) {
+	peerinfo = newinfo;
+    }
+
+#if SUBDEBUG
+    fprintf(stderr, "backup_full_dataset: fullZ = %p, fullinfo = %p\n",
+	    (void *) fullZ, (void *) fullinfo);
+#endif
 }
 
 int complex_subsampled (void)
@@ -724,42 +810,61 @@ int get_full_length_n (void)
     return n;
 }
 
-/* when subsampling cases from a time series, if the resulting dataset
-   is still a time series without internal "holes" then automatically
-   set a time series interpretation of the reduced dataset
+/* When sub-sampling on some boolean criterion, check to see if we can
+   meet the criterion by simply adjusting the endpoints of the sample
+   range: life will be simpler if that is so.
 */
 
-static void
-maybe_reconstitute_time_series (const DATAINFO *pdinfo,
-				const char *mask,
-				DATAINFO *subinfo)
+static int mask_contiguous (const char *mask,
+			    const DATAINFO *pdinfo,
+			    int *pt1, int *pt2)
 {
-    int missing = 1, switches = 0;
-    int t, t1 = 0, ts_ok = 1;
+    int t, t1 = 0, t2 = pdinfo->n - 1;
+    int contig = 1;
 
     for (t=0; t<pdinfo->n; t++) {
-	if (missing && mask[t] == 1) {
-	    t1 = t;
-	    switches++;
-	    missing = 0;
-	} else if (!missing && mask[t] == 0) {
-	    switches++;
-	    missing = 1;
-	}
-	if (switches > 2) {
-	    ts_ok = 0;
+	if (mask[t] == 0) {
+	    t1++;
+	} else {
 	    break;
 	}
     }
 
-    if (ts_ok) {
-	char line[32];
-	char stobs[OBSLEN];
+    for (t=pdinfo->n - 1; t>=0; t--) {
+	if (mask[t] == 0) {
+	    t2--;
+	} else {
+	    break;
+	}
+    }
 
-	ntodate(stobs, t1, pdinfo);
-	sprintf(line, "setobs %d %s", pdinfo->pd, stobs);
-	set_obs(line, NULL, subinfo, OPT_NONE);
-    } 
+    for (t=t1; t<=t2; t++) {
+	if (mask[t] == 0) {
+	    /* there's a hole inside the range */
+	    contig = 0;
+	    break;
+	}
+    }
+
+    if (contig && dataset_is_panel(pdinfo)) {
+	int n = t2 - t1 + 1;
+
+	/* sample must leave a whole number of panel units; moreover,
+	   to retain "panelness" this number must be greater than 1 
+	*/
+	if (t1 % pdinfo->pd != 0 || n % pdinfo->pd != 0) {
+	    contig = 0;
+	} else if (n == pdinfo->pd) {
+	    contig = 0;
+	}
+    }
+
+    if (contig) {
+	*pt1 = t1;
+	*pt2 = t2;
+    }
+
+    return contig;
 }
 
 static void 
@@ -770,7 +875,8 @@ copy_data_to_subsample (double **subZ, DATAINFO *subinfo,
     int i, t, st;
 
 #if SUBDEBUG
-    fprintf(stderr, "copying data to subsample\n");
+    fprintf(stderr, "copy_data_to_subsample: subinfo = %p, pdinfo = %p\n",
+	    (void *) subinfo, (void *) pdinfo);
 #endif
 
     /* copy data values */
@@ -793,14 +899,14 @@ copy_data_to_subsample (double **subZ, DATAINFO *subinfo,
     }
 
     /* copy observation markers, if any */
-   if (pdinfo->markers && subinfo->markers) {
-       st = 0;
-       for (t=0; t<pdinfo->n; t++) {
-	   if (mask == NULL || mask[t] == 1 || mask[t] == 'p') {
-	       strcpy(subinfo->S[st++], pdinfo->S[t]);
-	   }
-       }
-   }
+    if (pdinfo->markers && subinfo->markers) {
+	st = 0;
+	for (t=0; t<pdinfo->n; t++) {
+	    if (mask == NULL || mask[t] == 1 || mask[t] == 'p') {
+		strcpy(subinfo->S[st++], pdinfo->S[t]);
+	    }
+	}
+    }
 
     strcpy(subinfo->stobs, "1");
     sprintf(subinfo->endobs, "%d", subinfo->n);
@@ -937,35 +1043,40 @@ make_restriction_mask (int mode, const char *line, const int *list,
     return err;
 }
 
+/* this is also used in session.c, when re-establishing
+   a previously sub-sampled data state on reopening 
+   a saved session */
+
 int 
 restrict_sample_from_mask (char *mask, int mode, 
-			   double ***pZ, DATAINFO **ppdinfo)
+			   double ***pZ, DATAINFO **ppdinfo,
+			   ExecState *state)
 {
     double **subZ = NULL;
-    DATAINFO *subinfo = NULL;
+    DATAINFO *pdinfo = *ppdinfo;
+    DATAINFO *subinfo;
     int err = 0;
 
-    /* allocate new datainfo */
     subinfo = datainfo_new();
     if (subinfo == NULL) {
 	return E_ALLOC;
     }
 
-    subinfo->n = count_selected_cases(mask, *ppdinfo);
-    subinfo->v = (*ppdinfo)->v;
+    subinfo->n = count_selected_cases(mask, pdinfo);
+    subinfo->v = pdinfo->v;
 
 #if SUBDEBUG
-    fprintf(stderr, "count_selected_cases: %d obs in subsample\n",
-	    subinfo->n);
+    fprintf(stderr, "restrict_sample_from_mask: new subinfo=%p, %d obs in subsample\n",
+	    (void *) subinfo, subinfo->n);
 #endif
 
-    if (dataset_is_panel(*ppdinfo)) {
+    if (dataset_is_panel(pdinfo)) {
 	/* are we able to reconstitute a panel? */
-	int np, n = count_panel_units(mask, *ppdinfo);
+	int np, n = count_panel_units(mask, pdinfo);
 
 	if (n > 1 && subinfo->n > n) {
 	    /* add padding rows if need be */
-	    np = make_panel_submask(mask, *ppdinfo, &err);
+	    np = make_panel_submask(mask, pdinfo, &err);
 	    if (err) {
 		free(subinfo);
 		return err;
@@ -982,11 +1093,16 @@ restrict_sample_from_mask (char *mask, int mode,
 	return E_ALLOC;
     }
 
+#if SUBDEBUG
+    fprintf(stderr, "incoming *pZ at %p, new subZ is at %p\n", 
+	    (void *) *pZ, (void *) subZ);
+#endif
+
     /* link (don't copy) varnames and descriptions, since these are
        not dependent on the series length */
-    subinfo->varname = (*ppdinfo)->varname;
-    subinfo->varinfo = (*ppdinfo)->varinfo;
-    subinfo->descrip = (*ppdinfo)->descrip;
+    subinfo->varname = pdinfo->varname;
+    subinfo->varinfo = pdinfo->varinfo;
+    subinfo->descrip = pdinfo->descrip;
 
     if (subinfo->structure == STACKED_TIME_SERIES) {
 #if SUBDEBUG
@@ -1002,7 +1118,7 @@ restrict_sample_from_mask (char *mask, int mode,
     }
 
     /* set up case markers? */
-    if ((*ppdinfo)->markers) {
+    if (pdinfo->markers) {
 	err = dataset_allocate_obs_markers(subinfo);
 	if (err) {
 	    free_Z(subZ, subinfo);
@@ -1013,22 +1129,18 @@ restrict_sample_from_mask (char *mask, int mode,
 
     /* copy across data (and case markers, if any) */
     copy_data_to_subsample(subZ, subinfo, (const double **) *pZ, 
-			   *ppdinfo, mask);
+			   pdinfo, mask);
 
-    if (dataset_is_time_series(*ppdinfo)) {
-	if (mode == SUBSAMPLE_USE_DUMMY || 
-	    mode == SUBSAMPLE_BOOLEAN ||
-	    mode == SUBSAMPLE_DROP_MISSING) {
-	    maybe_reconstitute_time_series(*ppdinfo, mask, subinfo);
-	}
+    if (state != NULL && state->subinfo != NULL) {
+	backup_full_dataset(*pZ, pdinfo, NULL);
+    } else {
+	backup_full_dataset(*pZ, pdinfo, subinfo);
     }
 
-    /* save state */
-    backup_full_dataset(pZ, ppdinfo, subinfo);
     subinfo->submode = mode;
     subinfo->submask = copy_subsample_mask(mask);
 
-    /* and switch pointers */
+    /* switch pointers */
     *pZ = subZ;
     *ppdinfo = subinfo;
 
@@ -1039,6 +1151,7 @@ restrict_sample_from_mask (char *mask, int mode,
  * @line: command line (or %NULL).  
  * @pZ: pointer to original data array.  
  * @ppdinfo: address of original data info pointer. 
+ * @state: structure representing program state (or %NULL).
  * @list: list of variables in case of OPT_M (or %NULL).  
  * @opt: option flags.
  * @prn: printing apparatus.
@@ -1061,13 +1174,22 @@ restrict_sample_from_mask (char *mask, int mode,
 
 int restrict_sample (const char *line, const int *list,
 		     double ***pZ, DATAINFO **ppdinfo, 
-		     gretlopt opt, PRN *prn)
+		     ExecState *state, gretlopt opt, 
+		     PRN *prn)
 {
     char *oldmask = NULL;
     char *mask = NULL;
+    int free_oldmask = 0;
     int mode, err = 0;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
+
+#if SUBDEBUG
+    fprintf(stderr, "\nrestrict_sample: '%s'\n", line);
+    fprintf(stderr, " ppdinfo=%p, *ppdinfo=%p, pZ=%p, *pZ=%p, state=%p\n", 
+	    (void *) ppdinfo, (void *) *ppdinfo, (void *) pZ, (void *) *pZ,
+	    (void *) state);
+#endif
 
     mode = get_restriction_mode(opt);
     if (mode == SUBSAMPLE_UNKNOWN) {
@@ -1081,11 +1203,16 @@ int restrict_sample (const char *line, const int *list,
 	if (oldmask == NULL) {
 	    return E_ALLOC;
 	}
+	free_oldmask = 1;
+    } else if (state != NULL && state->subinfo != NULL) {
+	/* subsampling within a function */
+	oldmask = state->subinfo->submask;
     }
 
-    /* We must first restore the full data range, for housekeeping
-       purposes */
-    err = restore_full_sample(pZ, ppdinfo);
+    /* restore the full data range, for housekeeping purposes */
+    flag_restricting_on();
+    err = restore_full_sample(pZ, ppdinfo, state);
+    flag_restricting_off();
     if (err) {
 	return err;
     }
@@ -1096,20 +1223,121 @@ int restrict_sample (const char *line, const int *list,
     }
 
     if (!err && mask != NULL) {
-	err = restrict_sample_from_mask(mask, mode, pZ, ppdinfo);
+	int t1 = 0, t2 = 0;
+	int contig = 0;
+
+	if (mode != SUBSAMPLE_RANDOM &&
+	    (dataset_is_time_series(*ppdinfo) ||
+	     dataset_is_panel(*ppdinfo))) {
+	    contig = mask_contiguous(mask, *ppdinfo, &t1, &t2);
+	}
+
+	if (contig) {
+	    (*ppdinfo)->t1 = t1;
+	    (*ppdinfo)->t2 = t2;
+	} else {
+	    err = restrict_sample_from_mask(mask, mode, pZ, ppdinfo,
+					    state);
+	}
     }
 
     free(mask);
-    free(oldmask);
+    if (free_oldmask) {
+	free(oldmask);
+    }
 
     return err;
 }
 
+/* For use when exiting a function: re-establish the sub-sample that
+   was in force on entry, which is recorded in subinfo */
+
+static int 
+restore_sample_from_subinfo (double ***pZ, DATAINFO **ppdinfo,
+			     ExecState *state)
+{
+    double **subZ = NULL;
+    DATAINFO *pdinfo = *ppdinfo;
+    DATAINFO *subinfo = state->subinfo;
+    char *mask = subinfo->submask;
+    int i, s, t;
+
+#if SUBDEBUG
+    fprintf(stderr, "restore_sample_from_subinfo:\n"
+	    " subinfo=%p, n=%d, v=%d\n"
+	    " *ppdinfo=%p, n=%d, v=%d\n",
+	    (void *) subinfo, subinfo->n, subinfo->v,
+	    (void *) pdinfo, pdinfo->n, pdinfo->v);
+#endif
+
+    /* allow for added variables */
+    subinfo->v = pdinfo->v;
+
+    /* set up the sub-sampled Z */
+    if (allocate_Z(&subZ, subinfo)) {
+	return E_ALLOC;
+    }
+
+    /* re-link varnames, etc. */
+    subinfo->varname = pdinfo->varname;
+    subinfo->varinfo = pdinfo->varinfo;
+    subinfo->descrip = pdinfo->descrip;
+
+    /* copy data values */
+    for (i=1; i<fullinfo->v; i++) {
+	if (var_is_series(fullinfo, i)) {
+	    s = 0;
+	    for (t=0; t<fullinfo->n; t++) {
+		if (mask == NULL) {
+		    subZ[i][s++] = fullZ[i][t];
+		} else if (mask[t] == 1) {
+		    subZ[i][s++] = fullZ[i][t];
+		} else if (mask[t] == 'p') {
+		    /* panel padding */
+		    subZ[i][s++] = NADBL;
+		}
+	    }
+	} else {
+	    subZ[i][0] = fullZ[i][0];
+	}
+    }
+
+    if (*ppdinfo != subinfo) {
+	destroy_subsampled_datainfo(*ppdinfo);
+    }
+
+    /* switch pointers */
+    *pZ = subZ;
+    *ppdinfo = subinfo;
+
+#if SUBDEBUG
+    fprintf(stderr, " set *pZ = %p, *ppdinfo = %p\n", (void *) subZ, 
+	    (void *) subinfo);
+#endif
+
+    return 0;    
+}
+
 enum {
     SMPL_T1,
     SMPL_T2
 };
 
+static int panel_round (const DATAINFO *pdinfo, int t, int code)
+{
+    if (code == SMPL_T1) {
+	while ((t + 1) % pdinfo->paninfo->Tmax != 1) {
+	    t++;
+	}
+    } else {
+	while ((t + 1) % pdinfo->paninfo->Tmax != 0) {
+	    t--;
+	}
+    }
+
+    return t;
+}
+
 static int 
 get_sample_limit (char *s, const double **Z, DATAINFO *pdinfo,
 		  int code)
@@ -1117,7 +1345,7 @@ get_sample_limit (char *s, const double **Z, DATAINFO *pdinfo,
     int v, ret = -1;
 
     if (*s == '-' || *s == '+') {
-	/* incremental form */
+	/* increment/decrement form */
 	int incr = 0;
 
 	if (isdigit((unsigned char) s[1])) {
@@ -1131,6 +1359,9 @@ get_sample_limit (char *s, const double **Z, DATAINFO *pdinfo,
 		}
 	    }
 	}
+	if (dataset_is_panel(pdinfo)) {
+	    incr *= pdinfo->paninfo->Tmax;
+	}
 	if (code == SMPL_T1) {
 	    ret = pdinfo->t1 + incr;
 	} else {
@@ -1139,6 +1370,9 @@ get_sample_limit (char *s, const double **Z, DATAINFO *pdinfo,
     } else {
 	/* absolute form */
 	ret = get_t_from_obs_string(s, Z, pdinfo);
+	if (dataset_is_panel(pdinfo)) {
+	    ret = panel_round(pdinfo, ret, code);
+	}
     }
 
     return ret;
@@ -1149,7 +1383,7 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
     int nf, new_t1 = pdinfo->t1, new_t2 = pdinfo->t2;
     char cmd[5], newstart[OBSLEN], newstop[OBSLEN];
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     nf = count_fields(line);
 
@@ -1168,6 +1402,7 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
     }
 
     if (nf == 1) {
+	/* no-op, just print the current sample */
 	return 0;
     }
 	
