@@ -332,7 +332,9 @@ void gretl_equation_system_destroy (gretl_equation_system *sys)
 {
     int i;
 
-    if (sys == NULL || sys->lists == NULL) return;
+    if (sys == NULL || sys->lists == NULL) {
+	return;
+    }
 
     sys->refcount -= 1;
     if (sys->refcount > 0) {
@@ -613,12 +615,10 @@ system_print_F_test (const gretl_equation_system *sys,
     const gretl_matrix *R = sys->R;
     const gretl_matrix *q = sys->q;
     int Rrows = gretl_matrix_rows(R);
-    int vcols = gretl_matrix_cols(vcv);
     int dfu = system_get_dfu(sys);
     int dfn = gretl_matrix_rows(R);
 
     gretl_matrix *Rbq = NULL;
-    gretl_matrix *Rv = NULL;
     gretl_matrix *RvR = NULL;
 
     double F = NADBL;
@@ -630,10 +630,9 @@ system_print_F_test (const gretl_equation_system *sys,
     }
 
     Rbq = gretl_matrix_alloc(Rrows, 1);
-    Rv = gretl_matrix_alloc(Rrows, vcols);
     RvR = gretl_matrix_alloc(Rrows, Rrows);
 
-    if (Rbq == NULL || Rv == NULL || RvR == NULL) {
+    if (Rbq == NULL || RvR == NULL) {
 	pputs(prn, "Out of memory in F test\n");
 	goto bailout;
     }
@@ -641,10 +640,8 @@ system_print_F_test (const gretl_equation_system *sys,
     gretl_matrix_multiply(R, b, Rbq);
     gretl_matrix_subtract_from(Rbq, q);
 
-    gretl_matrix_multiply(R, vcv, Rv);
-    gretl_matrix_multiply_mod(Rv, GRETL_MOD_NONE,
-			      R, GRETL_MOD_TRANSPOSE,
-			      RvR);
+    gretl_matrix_qform(R, GRETL_MOD_NONE, vcv,
+		       RvR, GRETL_MOD_NONE);
 
     err = gretl_invert_symmetric_matrix(RvR);
     if (err) {
@@ -652,7 +649,7 @@ system_print_F_test (const gretl_equation_system *sys,
 	goto bailout;
     }
 
-    F = gretl_scalar_b_X_b(Rbq, GRETL_MOD_TRANSPOSE, RvR, &err);
+    F = gretl_scalar_qform(Rbq, RvR, &err);
     if (err) {
 	pputs(prn, "Matrix multiplication failed in F test\n");
 	goto bailout;
@@ -668,7 +665,6 @@ system_print_F_test (const gretl_equation_system *sys,
  bailout:
     
     gretl_matrix_free(Rbq);
-    gretl_matrix_free(Rv);
     gretl_matrix_free(RvR);
 }
 
@@ -787,11 +783,11 @@ static int estimate_with_test (gretl_equation_system *sys,
 static void 
 adjust_sys_flags_for_method (gretl_equation_system *sys, int method)
 {
-    char old_flags = sys->flags;
+    char oldflags = sys->flags;
 
     sys->flags = 0;
 
-    if (old_flags & GRETL_SYS_ITERATE) {
+    if (oldflags & GRETL_SYS_ITERATE) {
 	/* the iterate option is only available for WLS, SUR or 3SLS */
 	if (sys->method == SYS_WLS || sys->method == SYS_SUR ||
 	    sys->method == SYS_3SLS) {
@@ -802,12 +798,12 @@ adjust_sys_flags_for_method (gretl_equation_system *sys, int method)
     /* by default, apply a df correction for single-equation methods */
     if (sys->method == SYS_OLS || sys->method == SYS_WLS ||
 	sys->method == SYS_TSLS || sys->method == SYS_LIML) {
-	if (old_flags & GRETL_SYSTEM_DFCORR) {
+	if (oldflags & GRETL_SYSTEM_DFCORR) {
 	    sys->flags |= GRETL_SYSTEM_DFCORR;
 	}
     } 
 
-    if (old_flags & GRETL_SYS_VCV_GEOMEAN) {
+    if (oldflags & GRETL_SYS_VCV_GEOMEAN) {
 	sys->flags |= GRETL_SYS_VCV_GEOMEAN;
     }    
 }
@@ -815,6 +811,8 @@ adjust_sys_flags_for_method (gretl_equation_system *sys, int method)
 static void 
 set_sys_flags_from_opt (gretl_equation_system *sys, gretlopt opt)
 {
+    char oldflags = sys->flags;
+
     sys->flags = 0;
 
     if (opt & OPT_T) {
@@ -836,6 +834,14 @@ set_sys_flags_from_opt (gretl_equation_system *sys, gretlopt opt)
     if (opt & OPT_M) {
 	sys->flags |= GRETL_SYS_VCV_GEOMEAN;
     } 
+
+    if (oldflags & GRETL_SYSTEM_SAVE_UHAT) {
+	sys->flags |= GRETL_SYSTEM_SAVE_UHAT;
+    }
+
+    if (oldflags & GRETL_SYSTEM_SAVE_YHAT) {
+	sys->flags |= GRETL_SYSTEM_SAVE_YHAT;
+    }
 }
 
 /**
@@ -863,7 +869,7 @@ gretl_equation_system_estimate (gretl_equation_system *sys,
 		       double ***, DATAINFO *, gretlopt, PRN *);
     int stest = 0;
 
-    *gretl_errmsg = 0;
+    gretl_error_clear();
 
     if (opt == OPT_UNSET) {
 	opt = OPT_NONE;
@@ -943,7 +949,7 @@ int gretl_equation_system_finalize (gretl_equation_system *sys,
     gretlopt opt = OPT_NONE;
     int err = 0;
 
-    *gretl_errmsg = 0;
+    gretl_error_clear();
 
     if (sys == NULL) {
 	strcpy(gretl_errmsg, _(nosystem));
@@ -1172,18 +1178,18 @@ void make_system_data_info (gretl_equation_system *sys, int eqn,
     }
 }
 
-int gretl_system_add_resids_to_dataset (const char *sysname, int eqnum,
+int gretl_system_add_resids_to_dataset (gretl_equation_system *sys, int eqnum,
 					double ***pZ, DATAINFO *pdinfo)
 {
-    gretl_equation_system *sys;
     int v, t;
 
-    sys = get_equation_system_by_name(sysname);
-    if (sys == NULL || sys->uhat == NULL) {
-	return 1;
+    if (sys->uhat == NULL) {
+	return E_DATA;
     }
 
-    if (dataset_add_series(1, pZ, pdinfo)) return E_ALLOC;
+    if (dataset_add_series(1, pZ, pdinfo)) {
+	return E_ALLOC;
+    }
 
     v = pdinfo->v - 1;
 
@@ -1655,7 +1661,7 @@ system_parse_line (gretl_equation_system *sys, const char *line,
 {
     int err = 0;
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     if (strncmp(line, "identity", 8) == 0) {
 	err = add_identity_to_sys(sys, line + 8, pdinfo);
@@ -2074,3 +2080,23 @@ int gretl_system_serialize (gretl_equation_system *sys, SavedObjectFlags flags,
     return err;
 }
 
+void gretl_system_set_save_flag (gretl_equation_system *sys)
+{
+    sys->flags |= GRETL_SYS_SAVEIT;
+}
+
+void gretl_system_unset_save_flag (gretl_equation_system *sys)
+{
+    sys->flags &= ~GRETL_SYS_SAVEIT;
+}
+
+int gretl_system_save_flag_set (gretl_equation_system *sys)
+{
+    if (sys == NULL) {
+	return 0;
+    } else if (sys->flags & GRETL_SYS_SAVEIT) {
+	return 1;
+    } else {
+	return 0;
+    }
+}
