@@ -21,6 +21,31 @@
 
 #include "libgretl.h"
 #include "johansen.h"
+#include "texprint.h"
+
+static char colspec[4][8];
+static int use_custom;
+static int use_pdf;
+
+int tex_using_custom_tabular (void)
+{
+    return use_custom;
+}
+
+const char *tex_column_format (int i)
+{
+    if (i >= 0 && i < 4) {
+	return colspec[i];
+    } else {
+	return "";
+    }
+}
+
+void set_tex_use_pdf (const char *prog)
+{
+    use_pdf = strstr(prog, "pdf") != NULL ||
+	strstr(prog, "PDF") != NULL;
+}
 
 #define tex_screen_zero(x)  ((fabs(x) > 1.0e-17)? x : 0.0)
 
@@ -79,6 +104,37 @@ static void tex_print_math_float (double x, PRN *prn)
     pprintf(prn, "$%s$", number);
 }
 
+/**
+ * tex_float_string:
+ * @x: value to be printed.
+ * @targ: target string.
+ *
+ * Prints the value @x into @targ with @prec digits of precision.
+ * The result is intended for use in a non-math environment: if
+ * @x is negative, the minus sign is put into math mode.
+ * 
+ * Returns: @targ.
+ */
+
+char *tex_float_string (double x, int prec, char *targ)
+{
+    int offset = 0;
+
+    *targ = '\0';
+
+    x = tex_screen_zero(x);
+    if (x < 0.0) {
+	strcat(targ, "$-$");
+	x = fabs(x);
+	offset = 3;
+    }
+
+    sprintf(targ + offset, "%#.*g", prec, x);
+    tex_modify_exponent(targ);
+
+    return targ;
+}
+
 /**
  * tex_escape:
  * @targ: target string (must be pre-allocated)
@@ -116,19 +172,19 @@ static void cut_extra_zero (char *numstr)
     numstr[s + p + GRETL_DIGITS] = '\0';
 }
 
-void tex_dcolumn_double (double xx, char *numstr)
+void tex_dcolumn_double (double x, char *numstr)
 {
     double a;
 
-    if (na(xx)) {
+    if (na(x)) {
 	strcpy(numstr, "\\multicolumn{1}{c}{}");
 	return;
     }
 
-    xx = screen_zero(xx);
-    a = fabs(xx);
+    x = screen_zero(x);
+    a = fabs(x);
 
-    sprintf(numstr, "%#.*g", GRETL_DIGITS, xx);
+    sprintf(numstr, "%#.*g", GRETL_DIGITS, x);
 
     if (a != 0.0 && (a >= UPPER_F_LIMIT || a < LOWER_F_LIMIT)) {
 	int expon;
@@ -137,7 +193,7 @@ void tex_dcolumn_double (double xx, char *numstr)
 	p = strchr(numstr, 'e');
 	expon = atoi(p + 2);
 	strcpy(p, "\\mbox{e");
-	sprintf(exponstr, "%s%02d}", (xx > 10)? "+" : "-", expon);
+	sprintf(exponstr, "%s%02d}", (x > 10)? "+" : "-", expon);
 	strcat(numstr, exponstr);
     } else {
 	cut_extra_zero(numstr);
@@ -189,6 +245,29 @@ static int tex_greek_param (char *targ, const char *src)
     return (*targ != 0);
 }
 
+static void tex_arbond_coeff_name (char *targ, const char *src,
+				   int inmath)
+{
+    char vname[VNAMELEN], vnesc[16];
+    int lag;
+
+    if (sscanf(src, "D%10[^(](%d)", vname, &lag) == 2) {
+	tex_escape(vnesc, vname);
+	if (!inmath) {
+	    sprintf(targ, "$\\Delta \\mbox{\\rm %s}_{%d}$", vnesc, lag);
+	} else {
+	    sprintf(targ, "\\Delta \\mbox{\\rm %s}_{%d}", vnesc, lag);
+	}
+    } else {
+	tex_escape(vnesc, src);
+	if (inmath) {
+	    sprintf(targ, "\\mbox{%s}", vnesc);
+	} else {
+	    strcpy(targ, vnesc);
+	}
+    }
+}
+
 static void tex_garch_coeff_name (char *targ, const char *src,
 				  int inmath)
 {
@@ -198,9 +277,9 @@ static void tex_garch_coeff_name (char *targ, const char *src,
     if (sscanf(src, "%15[^(](%d)", vname, &lag) == 2) {
 	/* e.g. "alpha(0)" */
 	if (!inmath) {
-	    sprintf(targ, "$\\%s_%d$", vname, lag);
+	    sprintf(targ, "$\\%s_{%d}$", vname, lag);
 	} else {
-	    sprintf(targ, "\\%s_%d", vname, lag);
+	    sprintf(targ, "\\%s_{%d}", vname, lag);
 	}
     } else {
 	/* regular variable name */
@@ -338,83 +417,257 @@ static void tex_vecm_varname (char *s, const DATAINFO *pdinfo, int v)
     }
 }
 
-int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-		     int i, PRN *prn)
+static int tex_print_coeff_custom (const model_coeff *mc, PRN *prn)
 {
-    char tmp[32], coeff[64], sderr[64], tratio[64], pval[64];
+    char fmt[12];
 
-    if (isnan(pmod->coeff[i]) || na(pmod->coeff[i])) {
-	sprintf(coeff, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
-    } else {
-	tex_dcolumn_double(pmod->coeff[i], coeff);
-    }
+    pprintf(prn, "%s & ", mc->name);
 
-    if (isnan(pmod->sderr[i]) || na(pmod->sderr[i])) {
-	sprintf(sderr, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
-	sprintf(tratio, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
-	sprintf(pval, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
-    } else {
-	tex_dcolumn_double(pmod->sderr[i], sderr);
-	sprintf(tratio, "%.4f", pmod->coeff[i] / pmod->sderr[i]);
-	sprintf(pval, "%.4f", coeff_pval(pmod, pmod->coeff[i] / pmod->sderr[i], 
-					 pmod->dfd));
-    }    
-
-    *tmp = 0;
-    if (pmod->aux == AUX_ARCH) {
-	tex_make_cname(tmp, pdinfo->varname[pmod->list[i+2]]);
-    } else if (pmod->ci == NLS) {
-	if (!tex_greek_param(tmp, pmod->params[i+1])) {
-	    tex_escape(tmp, pmod->params[i+1]);
+    if (colspec[0][0]) {
+	/* coefficient */
+	if (na(mc->b)) {
+	    pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	} else {
+	    sprintf(fmt, "$%s$", colspec[0]);
+	    pprintf(prn, fmt, mc->b);
 	}
-    } else if (pmod->ci == ARMA) {
-	tex_arma_coeff_name(tmp, pmod->params[i+1], 0);
-    } else if (pmod->ci == GARCH) {
-	tex_garch_coeff_name(tmp, pmod->params[i+1], 0);
-    } else if (pmod->ci == VAR) {
-	tex_lagname(tmp, pdinfo, pmod->list[i+2]);
-    } else if (pmod->aux == AUX_VECM) {
-	tex_vecm_varname(tmp, pdinfo, pmod->list[i+2]);
-    } else if (pmod->ci == MPOLS && pmod->params != NULL) {
-	tex_mp_coeff_name(tmp, pmod->params[i], 0);
-    } else {
-	tex_escape(tmp, pdinfo->varname[pmod->list[i+2]]);
     }
-	
-    if (pmod->ci != LOGIT && pmod->ci != PROBIT) {
-	pprintf(prn, "%s &\n"
-		"  %s &\n"
-		"    %s &\n"
-		"      %s &\n"
-		"        %s \\\\\n",  
-		tmp,
-		coeff,
-		sderr,
-		tratio,
-		pval);	
-    } else { 
-	/* LOGIT, PROBIT */
-	double *slopes = gretl_model_get_data(pmod, "slopes");
-	char slope[32];
 
-	if (pmod->list[i+2]) {
-	    tex_dcolumn_double(slopes[i], slope);
-	}
-	pprintf(prn, "%s &\n"
-		"  %s &\n"
-		"    %s &\n"
-		"      %s &\n"
-		"        %s \\\\\n",  
-		tmp,
-		coeff,
-		sderr,
-		tratio,
-		(pmod->list[i+2])? slope : "");
+    if (!colspec[1][0] && !colspec[2][0] && !colspec[3][0]) {
+	pputs(prn, " \\\\\n");
+	return 0;
     }
 
+    if (colspec[1][0]) {
+	if (colspec[0][0]) {
+	    pputs(prn, " & ");
+	}
+	/* standard error */
+	if (na(mc->se)) {
+	    pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	} else {
+	    pprintf(prn, colspec[1], mc->se);
+	}
+    }
+
+    if (!colspec[2][0] && !colspec[3][0]) {
+	pputs(prn, " \\\\\n");
+	return 0;
+    }
+
+    if (colspec[2][0]) {
+	if (colspec[0][0] || colspec[1][0]) {
+	    pputs(prn, " & ");
+	}
+	/* t-ratio */
+	if (na(mc->tval)) {
+	    pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	} else {
+	    sprintf(fmt, "$%s$", colspec[2]);
+	    pprintf(prn, fmt, mc->tval);
+	}
+    } 
+
+    if (colspec[3][0]) {
+	if (colspec[0][0] || colspec[1][0] || colspec[2][0]) {
+	    pputs(prn, " & ");
+	}
+	/* p-value */
+	if (na(mc->pval)) {
+	    pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	} else {
+	    pprintf(prn, colspec[3], mc->pval);
+	}
+    }  
+
+    pputs(prn, " \\\\\n");
+
     return 0;
 }
 
+void make_tex_coeff_name (const MODEL *pmod, const DATAINFO *pdinfo, int i,
+			  char *name)
+{
+    int j = i + 2;
+
+    if (pmod->aux == AUX_ARCH) {
+	tex_make_cname(name, pdinfo->varname[pmod->list[i+2]]);
+    } else if (pmod->ci == NLS) {
+	if (!tex_greek_param(name, pmod->params[i])) {
+	    tex_escape(name, pmod->params[i]);
+	}
+    } else if (pmod->ci == ARMA) {
+	tex_arma_coeff_name(name, pmod->params[i], 0);
+    } else if (pmod->ci == GARCH) {
+	tex_garch_coeff_name(name, pmod->params[i], 0);
+    } else if (pmod->ci == VAR) {
+	tex_lagname(name, pdinfo, pmod->list[j]);
+    } else if (pmod->aux == AUX_VECM) {
+	tex_vecm_varname(name, pdinfo, pmod->list[j]);
+    } else if (pmod->ci == MPOLS && pmod->params != NULL) {
+	tex_mp_coeff_name(name, pmod->params[i], 0);
+    } else if ((pmod->ci == PROBIT || pmod->ci == LOGIT) &&
+	       pmod->params != NULL) {
+	tex_escape(name, pmod->params[i]);
+    } else if (pmod->ci == PANEL || pmod->ci == MLE ||
+	       pmod->ci == GMM) {
+	tex_escape(name, pmod->params[i]);
+    } else if (pmod->ci == ARBOND) {
+	tex_arbond_coeff_name(name, pmod->params[i], 0);
+    } else {
+	tex_escape(name, pdinfo->varname[pmod->list[j]]);
+    }
+}
+
+void tex_print_coeff (const model_coeff *mc, PRN *prn)
+{
+    char bstr[64], sestr[64], tstr[64], col4[64];
+
+    if (use_custom) {
+	tex_print_coeff_custom(mc, prn);
+	return;
+    }
+
+    if (na(mc->b)) {
+	sprintf(bstr, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+    } else {
+	tex_dcolumn_double(mc->b, bstr);
+    }
+
+    if (na(mc->se)) {
+	sprintf(sestr, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+    } else {
+	tex_dcolumn_double(mc->se, sestr);
+    }
+
+    if (na(mc->tval)) {
+	sprintf(tstr, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+    } else {
+	sprintf(tstr, "%.4f", mc->tval);
+    }
+
+    *col4 = '\0';
+
+    if (!na(mc->slope)) {
+	tex_dcolumn_double(mc->slope, col4);
+    } else if (mc->show_pval) {
+	if (!na(mc->pval)) {
+	    sprintf(col4, "%.4f", mc->pval);
+	}
+    }
+
+    pprintf(prn, "%s &\n"
+	    "  %s &\n"
+	    "    %s &\n"
+	    "      %s &\n"
+	    "        %s \\\\\n",  
+	    mc->name,
+	    bstr,
+	    sestr,
+	    tstr,
+	    col4);	    
+}
+
+void tex_custom_coeff_table_start (const char *col1, const char *col2,
+				   PRN *prn)
+{
+    int i, ncols = 0;
+
+    for (i=0; i<4; i++) {
+	if (colspec[i][0]) ncols++;
+    }
+
+    pputs(prn, "\\vspace{1em}\n\n"
+	  "\\begin{tabular}{l");
+
+    for (i=0; i<ncols; i++) {
+	pputs(prn, "r");
+    }
+
+    pputs(prn, "}\n");
+
+    pprintf(prn, "\\multicolumn{1}{c}{%s} &\n", I_(col1));
+
+    if (colspec[0][0]) {
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_(col2));
+    }
+
+    if (!colspec[1][0] && !colspec[2][0] && !colspec[3][0]) {
+	pputs(prn, " \\\\\n");
+	return;
+    }
+
+    if (colspec[1][0]) {
+	if (colspec[0][0]) {
+	    pputs(prn, " &\n");
+	}
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_("Std.\\ Error"));
+    }
+
+    if (!colspec[2][0] && !colspec[3][0]) {
+	pputs(prn, " \\\\\n");
+	return;
+    }
+
+    if (colspec[2][0]) {
+	if (colspec[0][0] || colspec[1][0]) {
+	    pputs(prn, " &\n");
+	}
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_("$t$-statistic"));
+    }
+
+    if (colspec[3][0]) {
+	if (colspec[0][0] || colspec[1][0] || colspec[2][0]) {
+	    pputs(prn, " &\n");
+	}
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_("p-value"));
+    }
+
+    pputs(prn, " \\\\\n");
+}
+
+void tex_coeff_table_start (const char *col1, const char *col2,
+			    int binary, PRN *prn)
+{
+
+    char pt;
+
+    if (use_custom) {
+	tex_custom_coeff_table_start(col1, col2, prn);
+	return;
+    }
+
+    pt = get_local_decpoint();
+
+    pprintf(prn, "\\vspace{1em}\n\n"
+	    "\\begin{tabular*}{\\textwidth}"
+	    "{@{\\extracolsep{\\fill}}\n"
+	    "l%% col 1: varname\n"
+	    "  D{%c}{%c}{-1}%% col 2: coeff\n"
+	    "    D{%c}{%c}{-1}%% col 3: sderr\n"
+	    "      D{%c}{%c}{-1}%% col 4: t-stat\n"
+	    "        D{%c}{%c}{4}}%% col 5: p-value (or slope)\n"
+	    "%s &\n"
+	    "  \\multicolumn{1}{c}{%s} &\n"
+	    "    \\multicolumn{1}{c}{%s} &\n"
+	    "      \\multicolumn{1}{c}{%s} &\n"
+	    "        \\multicolumn{1}{c}{%s%s} \\\\[1ex]\n",
+	    pt, pt, pt, pt, pt, pt, pt, pt, I_(col1),
+	    I_(col2), I_("Std.\\ Error"), 
+	    I_("$t$-statistic"), 
+	    (binary)? I_("Slope"): I_("p-value"),
+	    (binary)? "$^*$" : "");
+}
+
+void tex_coeff_table_end (PRN *prn)
+{
+    if (use_custom) {
+	pputs(prn, "\\end{tabular}\n\n");
+    } else {
+	pputs(prn, "\\end{tabular*}\n\n");
+    }
+}
+
 void tex_print_VECM_omega (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *prn)
 {
     char vname[48];
@@ -561,6 +814,8 @@ static PRN *make_tex_prn (int ID, char *texfile,
     if (*texfile == '\0') {
 	sprintf(texfile, "%s%s_%d.tex", gretl_user_dir(),
 		(eqn)? "equation" : "model", ID);
+    } else {
+	gretl_maybe_switch_dir(texfile);
     }
 
     prn = gretl_print_new_with_filename(texfile);
@@ -626,7 +881,45 @@ void set_gretl_tex_preamble (void)
     }
 }
 
-void gretl_tex_preamble (PRN *prn, int ams)
+static void landscape_modify_line (char *line)
+{
+    char *p, *rem;
+
+    if (strstr(line, "landscape")) {
+	return;
+    }
+
+    p = strstr(line, "documentclass");
+
+    if (p != NULL) {
+	if (*(p + 13) == '[') {
+	    p = strchr(p, ']');
+	    if (p != NULL) {
+		rem = gretl_strdup(p);
+		if (rem != NULL) {
+		    sprintf(p, ",landscape%s", rem);
+		    free(rem);
+		}
+	    }
+	} else {
+	    p += 13;
+	    rem = gretl_strdup(p); 
+	    if (rem != NULL) {
+		sprintf(p, "[landscape]%s", rem);
+		free(rem);
+	    }
+	}
+    }
+}
+
+static int tex_use_utf;
+
+void set_tex_use_utf (int s)
+{
+    tex_use_utf = s;
+}
+
+void gretl_tex_preamble (PRN *prn, int fmt)
 {
     FILE *fp = NULL;
     int userfile = 0;
@@ -634,9 +927,15 @@ void gretl_tex_preamble (PRN *prn, int ams)
     if (tex_preamble_file[0] != '\0') {
 	fp = gretl_fopen(tex_preamble_file, "r");
 	if (fp != NULL) {
-	    char line[128];
+	    char line[256];
+
+	    /* FIXME model table: longtable and geom packages */
 
 	    while (fgets(line, sizeof line, fp)) {
+		if (strstr(line, "documentclass") && 
+		    (fmt & GRETL_FORMAT_LANDSCAPE)) {
+		    landscape_modify_line(line);
+		}
 		pputs(prn, line);
 	    }
 	    userfile = 1;
@@ -645,13 +944,37 @@ void gretl_tex_preamble (PRN *prn, int ams)
     }
 
     if (!userfile) {
-	pputs(prn, "\\documentclass[11pt]{article}\n");
+	pputs(prn, "\\documentclass");
+	
+	if (fmt & GRETL_FORMAT_MODELTAB) {
+	    if (fmt & GRETL_FORMAT_LANDSCAPE) {
+		pputs(prn, "[landscape]");
+	    }
+	} else if (fmt & GRETL_FORMAT_LANDSCAPE) {
+	    pputs(prn, "[11pt,landscape]");
+	} else {
+	    pputs(prn, "[11pt]");
+	}
+
+	pputs(prn, "{article}\n");
 
 #ifdef ENABLE_NLS
-	pputs(prn, "\\usepackage[latin1]{inputenc}\n\n");
+	if (tex_use_utf) {
+	    pputs(prn, "\\usepackage{ucs}\n");
+	    pputs(prn, "\\usepackage[utf8x]{inputenc}\n");
+	} else {
+	    pputs(prn, "\\usepackage[latin1]{inputenc}\n");
+	}
 #endif
-	if (ams) {
+	if (fmt & GRETL_FORMAT_EQN) {
 	    pputs(prn, "\\usepackage{amsmath}\n\n");
+	} else if (fmt & GRETL_FORMAT_MODELTAB) {
+	    pputs(prn, "\\usepackage{longtable}\n");
+	    if (use_pdf) {
+		pputs(prn, "\\usepackage[margin=2cm,pdftex]{geometry}\n\n");
+	    } else {
+		pputs(prn, "\\usepackage[margin=2cm,dvips]{geometry}\n\n");
+	    }
 	} else {
 	    pputs(prn, "\\usepackage{dcolumn,longtable}\n\n");
 	}
@@ -697,7 +1020,7 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
     split = (nc > MAXCOEFF);
 
     if (opt & OPT_S) {
-	gretl_tex_preamble(prn, 1);
+	gretl_tex_preamble(prn, GRETL_FORMAT_EQN);
     } else{
 	pputs(prn, "%%% the following needs the amsmath LaTeX package\n\n");
     }
@@ -710,18 +1033,23 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 
     /* dependent variable */
     *tmp = '\0';
-    if (pmod->ci == POISSON) {
+    if (pmod->depvar != NULL) {
+	tex_escape(tmp, pmod->depvar);
+    } else if (pmod->ci == POISSON) {
 	char vname[32];
 
 	tex_escape(vname, pdinfo->varname[pmod->list[1]]);
 	sprintf(tmp, "log(%s)", vname);
-    } else if (pmod->ci == ARMA || pmod->ci == GARCH) {
-	tex_escape(tmp, pdinfo->varname[pmod->list[4]]);
     } else {
-	tex_escape(tmp, pdinfo->varname[pmod->list[1]]);
+	i = gretl_model_get_depvar(pmod);
+	tex_escape(tmp, pdinfo->varname[i]);
     }
 
-    pprintf(prn, "\\widehat{\\rm %s} %s= \n", tmp, (split? "&" : ""));
+    if (pmod->ci == ARBOND) {
+	pprintf(prn, "\\widehat{\\Delta \\rm %s} %s= \n", tmp, (split? "&" : ""));
+    } else {
+	pprintf(prn, "\\widehat{\\rm %s} %s= \n", tmp, (split? "&" : ""));
+    }
 
     if (pmod->ci == GARCH) {
 	nc -= (1 + pmod->list[1] + pmod->list[2]);
@@ -751,17 +1079,32 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	if (i > 0 || pmod->ifc == 0) {
 	    pputs(prn, "\\,");
 	    if (pmod->ci == ARMA) {
-		cchars += strlen(pmod->params[i+1]);
-		tex_arma_coeff_name(tmp, pmod->params[i+1], 1);
+		cchars += strlen(pmod->params[i]);
+		tex_arma_coeff_name(tmp, pmod->params[i], 1);
 		pputs(prn, tmp);
 	    } else if (pmod->ci == GARCH) {
-		cchars += strlen(pmod->params[i+1]);
-		tex_garch_coeff_name(tmp, pmod->params[i+1], 1);
+		cchars += strlen(pmod->params[i]);
+		tex_garch_coeff_name(tmp, pmod->params[i], 1);
 		pputs(prn, tmp);
+	    } else if (pmod->ci == PANEL) {
+		cchars += strlen(pmod->params[i]);
+		tex_escape(tmp, pmod->params[i]);
+		pprintf(prn, "\\mbox{%s}", tmp);
+	    } else if (pmod->ci == ARBOND) {
+		if (strcmp(pmod->params[i], "const")) {
+		    cchars += strlen(pmod->params[i]);
+		    tex_arbond_coeff_name(tmp, pmod->params[i], 1);
+		    pputs(prn, tmp);
+		}
 	    } else if (pmod->ci == MPOLS && pmod->params != NULL) {
 		cchars += strlen(pmod->params[i]);
 		tex_mp_coeff_name(tmp, pmod->params[i], 1);
 		pputs(prn, tmp);
+	    } else if ((pmod->ci == PROBIT || pmod->ci == LOGIT) &&
+		       pmod->params != NULL) {
+		cchars += strlen(pmod->params[i]);
+		tex_escape(tmp, pmod->params[i]);
+		pprintf(prn, "\\mbox{%s}", tmp);
 	    } else if (offvar > 0 && i == nc - 1) {
 		cchars += strlen(pdinfo->varname[offvar]);
 		tex_escape(tmp, pdinfo->varname[offvar]);
@@ -832,21 +1175,20 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	}
 
 	pputs(prn, "\\notag \\\\\n");
-    }	    
+    }
+
+    pprintf(prn, "T = %d ", pmod->nobs);
 
     /* additional info (R^2 etc) */
     if (pmod->ci == LAD) { 
 	sprintf(tmp, "%g", pmod->rho);
 	tex_modify_exponent(tmp);
-	pprintf(prn, "T = %d \\quad \\sum |\\hat{u}_t| = %s",
-		pmod->nobs, tmp);
+	pprintf(prn, "\\quad \\sum |\\hat{u}_t| = %s", tmp);
     } else {
 	if (!na(pmod->adjrsq)) {
-	    pprintf(prn, "T = %d \\quad \\bar{R}^2 = %.4f ",
-		    pmod->nobs, pmod->adjrsq);
+	    pprintf(prn, "\\quad \\bar{R}^2 = %.4f ", pmod->adjrsq);
 	} else if (!na(pmod->lnL)) {
-	    pprintf(prn, "T = %d \\quad \\mbox{ln}L = %.4f ",
-		    pmod->nobs, pmod->lnL);
+	    pprintf(prn, "\\quad \\mbox{ln}L = %.4f ", pmod->lnL);
 	}
 
 	if (pmod->ci != LOGIT && pmod->ci != PROBIT && !na(pmod->fstt)) {
@@ -938,8 +1280,8 @@ int tex_print_model (MODEL *pmod, const DATAINFO *pdinfo,
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int texprint (MODEL *pmod, const DATAINFO *pdinfo,
-	      char *texfile, gretlopt opt)
+int texprint (MODEL *pmod, const DATAINFO *pdinfo, char *texfile, 
+	      gretlopt opt)
 {
     PRN *prn;
     int eqn = (opt & OPT_E);
@@ -977,3 +1319,112 @@ void tex_print_obs_marker (int t, const DATAINFO *pdinfo, PRN *prn)
 	pprintf(prn, "%8s ", tmp);
     }
 }
+
+static int check_colspec (const char *s)
+{
+    const char *ok = "eEfgG";
+    int w = 0, p = 0;
+    char c = 0;
+    int err = 1;
+
+    /* blank is OK */
+    if (*s == '\0') {
+	return 0;
+    }
+
+    if (*s != '%') {
+	return 1;
+    }
+
+    s++;
+
+    if (*s == '#') { /* OK */
+	s++;
+    }
+
+    if (sscanf(s, "%d.%d%c", &w, &p, &c) == 3) {
+	if (w != 0 && p > 0 && strchr(ok, c)) {
+	    err = 0;
+	}
+    } else if (sscanf(s, "%d%c", &w, &c) == 2) {
+	if (w != 0 && strchr(ok, c)) {
+	    err = 0;
+	}
+    } else if (sscanf(s, ".%d%c", &p, &c) == 2) {
+	if (p > 0 && strchr(ok, c)) {
+	    err = 0;
+	}
+    } else if (sscanf(s, "%c", &c) == 1) {
+	if (strchr(ok, c)) {
+	    err = 0;
+	}
+    } 
+
+    return err;
+}
+
+/**
+ * set_tex_param_format:
+ * @s: stylized format string.
+ *
+ * Sets the format with which parameters will be printed, when
+ * producing TeX tabular output.
+ */
+
+void set_tex_param_format (const char *s)
+{
+    const char *p = s;
+    int i, n = 0;
+    int err = 0;
+
+    if (s == NULL) {
+	use_custom = 0;
+	return;
+    }
+
+    for (i=0; i<4; i++) {
+	colspec[i][0] = '\0';
+    }
+
+    i = 0;
+
+    while (i < 4) {
+	if (*s == '|' || *s == '\0') {
+	    if (n > 7) {
+		n = 7;
+	    }
+	    strncat(colspec[i], p, n);
+	    fprintf(stderr, "spec %d = '%s'\n", i, colspec[i]);
+	    err = check_colspec(colspec[i]);
+	    if (err || *s == '\0') {
+		break;
+	    }
+	    p = s + 1;
+	    i++;
+	    n = 0;
+	} else {
+	    n++;
+	}
+	s++;
+    }
+
+    if (!err) {
+	/* all columns can't be blank */
+	n = 0;
+	for (i=0; i<4; i++) {
+	    if (colspec[i][0] != '\0') n++;
+	}
+	if (n == 0) {
+	    err = 1;
+	}
+    }
+
+    if (err) {
+	for (i=0; i<4; i++) {
+	    colspec[i][0] = '\0';
+	}
+	use_custom = 0;
+    } else {
+	use_custom = 1;
+    }
+}
