@@ -28,6 +28,10 @@ enum {
     VARS_DIFFER
 } varcomp_codes;
 
+enum {
+    INVERSE = NC + 1
+};
+
 static int check_vals (const double *x, const double *y, int n)
 {
     int ret = VARS_IDENTICAL;
@@ -100,7 +104,10 @@ make_transform_varname (char *vname, const char *orig, int ci,
 	strcpy(vname, "D");
 	strncat(vname, orig, len - strlen(ext) - 1);
 	strcat(vname, ext);
-    }	
+    } else if (ci == INVERSE) {
+	strcpy(vname, "i_");
+	strncat(vname, orig, len - 2);
+    }
 
     return 0;
 }
@@ -127,6 +134,8 @@ make_transform_label (char *label, const char *parent,
 	} else {
 	    sprintf(label, "= %s(t + %d)", parent, -lag);
 	}
+    } else if (ci == INVERSE) {
+	sprintf(label, "= 1/%s", parent);
     } else {
 	err = 1;
     }
@@ -152,6 +161,37 @@ int is_standard_lag (int v, const DATAINFO *pdinfo, int *parent)
     return ret;
 }
 
+int is_dummy_child (int v, const DATAINFO *pdinfo, int *parent)
+{
+    const char *test = VARLABEL(pdinfo, v);
+    char vname[VNAMELEN];
+    double val;
+    int pv = pdinfo->v;
+    int i = 0, ret = 0;
+
+    if (sscanf(test, _("dummy for %s = %lf"), vname, &val) == 2 ||
+	sscanf(test, "dummy for %s = %lf", vname, &val) == 2) {
+	pv = varindex(pdinfo, vname);
+    } else if (!strncmp(pdinfo->varname[v], "dt_", 3)) {
+	if (sscanf(pdinfo->varname[v] + 3, "%d", &i) && i > 1) {
+	    pv = varindex(pdinfo, "dt_1");
+	}
+    } else if (!strncmp(pdinfo->varname[v], "du_", 3)) {
+	if (sscanf(pdinfo->varname[v] + 3, "%d", &i) && i > 1) {
+	    pv = varindex(pdinfo, "du_1");
+	}
+    }	
+
+    if (pv < pdinfo->v) {
+	*parent = pv;
+	ret = 1;
+    } else {
+	*parent = 0;
+    }
+
+    return ret;
+}
+
 static void make_xp_varname (char *vname, const char *v1, 
 			     const char *v2, int len)
 {
@@ -268,14 +308,8 @@ static int get_log (int v, double *logvec, const double **Z,
 
     for (t=pdinfo->t1; t<=pdinfo->t2 && !err; t++) {
 	xx = (var_is_series(pdinfo, v))? Z[v][t] : Z[v][0];
-	if (na(xx)) {
+	if (na(xx) || xx <= 0.0) {
 	    logvec[t] = NADBL;
-	} else if (xx <= 0.0) { /* ?? treatment of 0 arg */
-	    sprintf(gretl_errmsg, 
-		    _("Log error: Variable '%s', obs %d,"
-		      " value = %g\n"), pdinfo->varname[v],
-		    t+1, xx);
-	    err = 1;
 	} else {
 	    logvec[t] = log(xx); 
 	}
@@ -339,6 +373,26 @@ static int get_xpx (int vi, int vj, double *xvec, const double **Z,
     return 0;
 }
 
+/* write reciprocal into xvec */
+
+static int get_inverse (int v, double *xvec, const double **Z, 
+			const DATAINFO *pdinfo)
+{
+    double xt;
+    int t;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	xt = (var_is_series(pdinfo, v))? Z[v][t] : Z[v][0];
+	if (na(xt) || xt == 0.0) {
+	    xvec[t] = NADBL;
+	} else {
+	    xvec[t] = 1.0 / xt;
+	}
+    }
+
+    return 0;
+}
+
 /* write dummy for (v == value) into xvec */
 
 static int get_discdum (int v, double val, double *xvec, const double **Z, 
@@ -463,6 +517,8 @@ static int get_transform (int ci, int v, int aux, double x,
     } else if (ci == DUMMIFY) {
 	/* "x" = value for dummy */
 	err = get_discdum(v, x, vx, (const double **) *pZ, pdinfo);
+    } else if (ci == INVERSE) {
+	err = get_inverse(v, vx, (const double **) *pZ, pdinfo);
     }
 
     if (err) {
@@ -554,6 +610,23 @@ int loggenr (int v, double ***pZ, DATAINFO *pdinfo)
     return get_transform(LOGS, v, 0, 0.0, pZ, pdinfo, VNAMELEN - 3);
 }
 
+/**
+ * invgenr: 
+ * @v: ID number in dataset of source variable.
+ * @pZ: pointer to data array.
+ * @pdinfo: information on dataset.
+ *
+ * Creates the reciprocal of variable @v if this variable does
+ * not already exist.
+ *
+ * Returns: the ID number of the reciprocal, or -1 on error.
+ */
+
+int invgenr (int v, double ***pZ, DATAINFO *pdinfo)
+{
+    return get_transform(INVERSE, v, 0, 0.0, pZ, pdinfo, VNAMELEN - 3);
+}
+
 /**
  * diffgenr: 
  * @v: ID number in dataset of source variable.
@@ -929,21 +1002,47 @@ int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 
 #define DUMDEBUG 0
 
+static int dummify_candidate (int v, double **Z, DATAINFO *pdinfo)
+{
+    if (var_is_discrete(pdinfo, v)) {
+	/* pre-approved */
+	return 1;
+    }
+
+    if (v == 0 || var_is_scalar(pdinfo, v)) {
+	return 0;
+    }  
+
+    if (gretl_isdiscrete(0, pdinfo->n - 1, Z[v])) {
+	return 1;
+    }
+
+    return 0;
+}
+
 /**
  * list_dumgenr:
  * @plist: pointer to list of variables to process; on exit
  * the list holds the ID numbers of the generated dummies.
  * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
+ * @opt: can be %OPT_F to drop the first value, %OPT_L to drop
+ * the last, or %OPT_NONE.
  *
- * Generates and adds to the data set dummy variables for the
- * distinct values of the variables given in the list pointed to
- * by @plist. The latter must have already been marked as discrete.
+ * For each of the variables given in the list to which @plist
+ * points, generates and adds to the data set k dummy variables 
+ * coding for the k distinct values of the variable in question.
+ * All these variables must have already been marked as discrete.
+ * If the %OPT_F or %OPT_L option is given, either the first or
+ * the last value of each variable is taken as the "base" and is
+ * not given a dummy encoding (that is, only k - 1 dummies are
+ * added for each variable).
  *
  * Returns: 0 on success, error code on error.
 */
 
-int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo)
+int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
+		  gretlopt opt)
 {
     int *list = *plist;
     int *tmplist = NULL;
@@ -968,10 +1067,11 @@ int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo)
     for (i=1; i<=list[0] && !err; i++) {
 	int vi = list[i];
 	int nuniq, tnum;
+	int jmin, jmax;
 	double xt;
 
-	if (vi == 0 || var_is_scalar(pdinfo, vi) || !var_is_discrete(pdinfo, vi)) {
-	    continue; 
+	if (!dummify_candidate(vi, *pZ, pdinfo)) {
+	    continue;
 	}
 
 	n = 0;
@@ -995,11 +1095,17 @@ int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo)
 
 	rearrange_id_array(x, nuniq, n);
 
+	jmin = (opt & OPT_F)? 1 : 0;
+	jmax = (opt & OPT_L)? nuniq - 1 : nuniq;
+
 #if DUMDEBUG 
-	fprintf(stderr, "variable %d has %d distinct values:\n", vi, nuniq);
+	fprintf(stderr, "variable %d has %d distinct values\n", vi, nuniq);
+	if (opt & OPT_F) fprintf(stderr, "skipping lowest value\n");
+	if (opt & OPT_L) fprintf(stderr, "skipping highest value\n");
+	fprintf(stderr, "jmin = %d, jmax = %d\n", jmin, jmax);
 #endif
 
-	for (j=0; j<nuniq && !err; j++) {
+	for (j=jmin; j<jmax && !err; j++) {
 	    tnum = get_transform(DUMMIFY, vi, j+1, x[j], pZ, pdinfo, startlen);
 #if DUMDEBUG   
 	    fprintf(stderr, "VALUE = %g, tnum = %d\n", x[j], tnum);
@@ -1016,6 +1122,7 @@ int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo)
     }
 
     if (!err && tmplist[0] == 0) {
+	strcpy(gretl_errmsg, _("dummify: no suitable variables were found"));
 	err = E_DATA;
     }
 
