@@ -24,12 +24,16 @@ int default_lag_order (const DATAINFO *pdinfo);
 
 int is_standard_lag (int v, const DATAINFO *pdinfo, int *parent);
 
+int is_dummy_child (int v, const DATAINFO *pdinfo, int *parent);
+
 int diffgenr (int v, int ci, double ***pZ, DATAINFO *pdinfo);
 
 int laggenr (int v, int lag, double ***pZ, DATAINFO *pdinfo);
 
 int loggenr (int v, double ***pZ, DATAINFO *pdinfo);
 
+int invgenr (int v, double ***pZ, DATAINFO *pdinfo);
+
 int xpxgenr (int vi, int vj, double ***pZ, DATAINFO *pdinfo);
 
 int list_diffgenr (int *list, int ci, double ***pZ, DATAINFO *pdinfo);
@@ -41,7 +45,8 @@ int list_loggenr (int *list, double ***pZ, DATAINFO *pdinfo);
 int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo, 
 		  gretlopt opt);
 
-int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo);
+int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
+		  gretlopt opt);
 
 int list_makediscrete (const int *list, DATAINFO *pdinfo, gretlopt opt);
 
