@@ -23,6 +23,7 @@
 #include "qr_estimate.h"
 #include "libset.h"
 #include "estim_private.h"
+#include "matrix_extra.h"
 
 #define TDEBUG 0
 
@@ -33,8 +34,7 @@ static void tsls_omitzero (int *list, const double **Z, int t1, int t2)
     for (i=2; i<=list[0]; i++) {
         v = list[i];
         if (gretl_iszero(t1, t2, Z[v])) {
-	    gretl_list_delete_at_pos(list, i);
-	    i--;
+	    gretl_list_delete_at_pos(list, i--);
 	}
     }
 }
@@ -430,6 +430,11 @@ tsls_sargan_test (MODEL *tsls_model, int Orank, int *instlist,
 	return 0;
     }
 
+    if (tsls_model->list[0] == 2 && tsls_model->list[2] == 0) {
+	/* degenerate model with const only */
+	return 0;
+    }
+
     err = dataset_add_series(1, pZ, pdinfo);
     if (err) {
 	return err;
@@ -544,7 +549,7 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
     int rank, ndrop = 0;
     int *droplist = NULL;
     double test;
-    int i, k, v;
+    int i, j, k;
 
     Q = gretl_matrix_data_subset(instlist, Z, t1, t2, mask);
     if (Q == NULL) {
@@ -565,13 +570,15 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 	goto bailout;
     }
 
-    rank = gretl_matrix_QR_rank(R, NULL, err);
+    rank = gretl_check_QR_rank(R, err);
+
 #if TDEBUG
     fprintf(stderr, "tsls_Q: k = Q->cols = %d, rank = %d\n", k, rank);
     if (rank == 0) {
 	gretl_matrix_print(R, "R");
     }
 #endif
+
     if (*err) {
 	goto bailout;
     } else if (rank < k) {
@@ -585,17 +592,19 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 	    droplist[0] = 0;
 	}
 
+	j = 1;
 	for (i=0; i<k; i++) {
-	    v = instlist[i+1];
 	    test = gretl_matrix_get(R, i, i);
 	    if (fabs(test) < R_DIAG_MIN) {
 		if (droplist != NULL) {
 		    droplist[0] += 1;
-		    droplist[droplist[0]] = v;
+		    droplist[droplist[0]] = instlist[j];
 		}	    
-		fprintf(stderr, "tsls_Q: dropping redundant instrument %d\n", v);
-		gretl_list_delete_at_pos(instlist, i+1);
+		fprintf(stderr, "tsls_Q: dropping redundant instrument %d\n", 
+			instlist[j]);
+		gretl_list_delete_at_pos(instlist, j--);
 	    }
+	    j++;
 	}
 
 	k = instlist[0];
@@ -790,43 +799,66 @@ reglist_remove_redundant_vars (const MODEL *tmod, int *s2list, int *reglist)
     return 0;
 }
 
-static void compute_first_stage_F (MODEL *pmod, int v, int fitv, 
-				   gretl_matrix *Q, 
-				   double **Z, DATAINFO *pdinfo)
+static int 
+compute_first_stage_F (MODEL *pmod, int v, const int *reglist, 
+		       const int *instlist, double ***pZ, 
+		       DATAINFO *pdinfo, gretlopt opt)
 {
-    int n = 0;
-    int k = gretl_matrix_cols(Q);
-    int ifc = pmod->ifc;
-    double essr = 0.0, essu = 0.0;
-    double x, F, ybar = 0.0;
-    int t, dfn, dfd;
+    MODEL mod1;
+    gretlopt myopt;
+    int *list1 = NULL;
+    int *flist = NULL;
+    double F;
+    int i, vi;
+    int err = 0;
 
-    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	if (!na(Z[v][t])) {
-	    ybar += Z[v][t];
-	    n++;
+    gretl_model_init(&mod1);
+
+    list1 = gretl_list_append_term(&list1, v);
+    if (list1 == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=1; i<=instlist[0]; i++) {
+	vi = instlist[i];
+	list1 = gretl_list_append_term(&list1, vi);
+	if (list1 == NULL) {
+	    err = E_ALLOC;
+	    break;
+	}
+	if (!in_gretl_list(reglist, vi)) {
+	    flist = gretl_list_append_term(&flist, vi);
+	    if (flist == NULL) {
+		err = E_ALLOC;
+		break;
+	    }
 	}
     }
 
-    ybar /= n;
+    if (!err) {
+	myopt = (opt & OPT_R)? (OPT_A | OPT_R) : OPT_A;
+	mod1 = lsq(list1, pZ, pdinfo, OLS, myopt);
+	err = mod1.errcode;
+    }
 
-    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	if (!na(Z[v][t])) {
-	    x = Z[v][t] - ybar;
-	    essr += x * x;
-	    x = Z[v][t] - Z[fitv][t];
-	    essu += x * x;
+    if (!err) {
+	F = robust_omit_F(flist, &mod1);
+	if (na(F)) {
+	    err = 1;
 	}
-    }  
+    }
 
-    dfd = n - k;
-    dfn = k - ifc;
+    if (!err) {
+	gretl_model_set_double(pmod, "stage1-F", F);
+	gretl_model_set_int(pmod, "stage1-dfn", flist[0]);
+	gretl_model_set_int(pmod, "stage1-dfd", mod1.dfd);
+    }
+    
+    clear_model(&mod1);
+    free(list1);
+    free(flist);
 
-    F = ((essr - essu) / dfn) / (essu / dfd);
-
-    gretl_model_set_double(pmod, "stage1-F", F);
-    gretl_model_set_int(pmod, "stage1-dfn", dfn);
-    gretl_model_set_int(pmod, "stage1-dfd", dfd);
+    return err;
 }
 
 static int 
@@ -967,7 +999,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     /* initialize model in case we bail out early on error */
     gretl_model_init(&tsls);
 
-    *gretl_errmsg = '\0';
+    gretl_error_clear();
 
     pos = gretl_list_separator_position(list);
     if (pos == 0) {
@@ -1119,7 +1151,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (ci == TSLS && hatlist[0] == 1) {
-	compute_first_stage_F(&tsls, ev, hatlist[1], Q, *pZ, pdinfo);
+	compute_first_stage_F(&tsls, ev, reglist, instlist, pZ, pdinfo, opt);
     } 
 
     if (tsls.list[0] < s2list[0]) {
@@ -1135,7 +1167,7 @@ MODEL tsls_func (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
     /* compute standard errors */
     if ((opt & OPT_R) || get_use_qr()) {
 	/* QR decomp in force, or robust standard errors called for */
-	qr_tsls_vcv(&tsls, (const double **) *pZ, opt);
+	qr_tsls_vcv(&tsls, (const double **) *pZ, pdinfo, opt);
 	if (tsls.errcode) {
 	    goto bailout;
 	}	
