@@ -421,7 +421,7 @@ gretl_VAR_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
 	add_VAR_fcast_variance(var, F, nf, t1 - t0);
     }
 
-    gretl_matrix_set_int(F, t0);
+    gretl_matrix_set_t1(F, t0);
     var->F = F;
 
 #if 0
@@ -556,7 +556,7 @@ gretl_VECM_add_forecast (GRETL_VAR *var, int t0, int t1, int t2,
 	add_VAR_fcast_variance(var, F, nf, t1 - t0);
     }
 
-    gretl_matrix_set_int(F, t0);
+    gretl_matrix_set_t1(F, t0);
     var->F = F;
 
 #if 0
@@ -573,7 +573,7 @@ gretl_VAR_get_forecast_matrix (GRETL_VAR *var, int t0, int t1, int t2,
 {
     if (var->F != NULL) {
 	int ncols, nf = t2 - t0 + 1;
-	int ft1 = gretl_matrix_get_int(var->F);
+	int ft1 = gretl_matrix_get_t1(var->F);
 
 	ncols = (opt & OPT_S)? var->neqns: 2 * var->neqns;
 
@@ -623,7 +623,7 @@ int gretl_VAR_autocorrelation_test (GRETL_VAR *var, int order,
     int i, err = 0;
 
     for (i=0; i<var->neqns && !err; i++) {
-	pprintf(prn, "Equation %d:\n", i + 1);
+	pprintf(prn, "%s %d:\n", _("Equation"), i + 1);
 	err = autocorr_test(var->models[i], order, pZ, pdinfo,
 			    OPT_Q | OPT_S, prn);
 	gretl_model_test_print(var->models[i], 0, prn);
@@ -634,14 +634,13 @@ int gretl_VAR_autocorrelation_test (GRETL_VAR *var, int order,
 }
 
 int gretl_VAR_arch_test (GRETL_VAR *var, int order, 
-			 double ***pZ, DATAINFO *pdinfo,
-			 PRN *prn)
+			 DATAINFO *pdinfo, PRN *prn)
 {
     int i, err = 0;
 
     for (i=0; i<var->neqns && !err; i++) {
-	pprintf(prn, "Equation %d:\n", i + 1);
-	err = arch_test_simple(var->models[i], order, pZ, pdinfo, OPT_NONE, prn);
+	pprintf(prn, "%s %d:\n", _("Equation"), i + 1);
+	err = arch_test(var->models[i], order, pdinfo, OPT_NONE, prn);
     }
 
     return err;
@@ -931,14 +930,14 @@ gretl_VAR_get_fcast_decomp (GRETL_VAR *var, int targ, int periods)
 		/* calculate initial variances */
 		err = gretl_matrix_multiply_mod(idx, GRETL_MOD_NONE,
 						var->C, GRETL_MOD_TRANSPOSE,
-						ctmp);
+						ctmp, GRETL_MOD_NONE);
 		err = gretl_matrix_multiply(var->C, ctmp, cic);
 		gretl_matrix_copy_values(vt, cic);
 	    } else {
 		/* calculate further variances */
 		err = gretl_matrix_multiply_mod(vt, GRETL_MOD_NONE,
 						var->A, GRETL_MOD_TRANSPOSE,
-						vtmp);
+						vtmp, GRETL_MOD_NONE);
 		err = gretl_matrix_multiply(var->A, vtmp, vt);
 		gretl_matrix_add_to(vt, cic);
 	    }
@@ -1292,46 +1291,28 @@ static int add_model_data_to_var (GRETL_VAR *var, const MODEL *pmod, int k)
 static int VAR_add_roots (GRETL_VAR *var)
 {
     gretl_matrix *CompForm = NULL;
-    double *eigA = NULL;
-    double x, y;
-    int i, np, err = 0;
+    int err = 0;
 
     if (var->A == NULL) {
 	return 1;
     }
 
-    np = gretl_matrix_rows(var->A);
+    var->lambda = NULL;
 
-    var->lambda = gretl_matrix_alloc(np, 2);
-    if (var->lambda == NULL) {
-        err = E_ALLOC;
-    }
-
-    if (!err) {
-	CompForm = gretl_matrix_copy(var->A);
-	if (CompForm == NULL) {
-	    err = E_ALLOC;
-	}
+    CompForm = gretl_matrix_copy(var->A);
+    if (CompForm == NULL) {
+	err = E_ALLOC;
     }
 
     /* save eigenvalues of companion form matrix */
     if (!err) {
-        eigA = gretl_general_matrix_eigenvals(CompForm, 0, &err);
-	if (!err) {
-	    for (i=0; i<np; i++) {
-		x = eigA[i];
-		y = eigA[np + i];
-		gretl_matrix_set(var->lambda, i, 0, x);
-		gretl_matrix_set(var->lambda, i, 1, y);
-	    }
+        var->lambda = gretl_general_matrix_eigenvals(CompForm, 0, &err);
 #if 0
-	    gretl_matrix_print(var->A, "Companion form matrix");
-	    gretl_matrix_print(var->lambda, "Eigenvalues");
+	gretl_matrix_print(var->A, "Companion form matrix");
+	gretl_matrix_print(var->lambda, "Eigenvalues");
 #endif
-	}
     }
 
-    free(eigA);
     gretl_matrix_free(CompForm);
 
     if (err) {
@@ -1364,7 +1345,7 @@ static double gretl_VAR_ldet (GRETL_VAR *var, int *err)
     } else {
 	gretl_matrix_multiply_mod(var->F, GRETL_MOD_TRANSPOSE,
 				  var->F, GRETL_MOD_NONE,
-				  S);
+				  S, GRETL_MOD_NONE);
 	gretl_matrix_divide_by_scalar(S, var->T);
 	ldet = gretl_vcv_log_determinant(S);
 	if (na(ldet)) {
@@ -1420,7 +1401,7 @@ static void gretl_VAR_print_lagsel (gretl_matrix *lltab,
     pprintf(prn, _("VAR system, maximum lag order %d"), maxlag);
     pputs(prn, "\n\n");
 
-    pputs(prn, ("The asterisks below indicate the best (that is, minimized) values\n"
+    pputs(prn, _("The asterisks below indicate the best (that is, minimized) values\n"
 	  "of the respective information criteria, AIC = Akaike criterion,\n"
 	  "BIC = Schwartz Bayesian criterion and HQC = Hannan-Quinn criterion."));
     pputs(prn, "\n\n");
@@ -1680,7 +1661,7 @@ static int VAR_add_variance_matrix (GRETL_VAR *var)
     if (!err) {
 	gretl_matrix_multiply_mod(var->E, GRETL_MOD_TRANSPOSE,
 				  var->E, GRETL_MOD_NONE,
-				  var->S);
+				  var->S, GRETL_MOD_NONE);
 	gretl_matrix_divide_by_scalar(var->S, var->T);
     }
 
@@ -2091,24 +2072,25 @@ transcribe_data_as_uhat (int v, const double **Z, gretl_matrix *u,
     }
 }
 
-int gretl_VECM_test_beta (GRETL_VAR *vecm, PRN *prn)
+int gretl_VECM_test_beta (GRETL_VAR *vecm, const DATAINFO *pdinfo, 
+			  PRN *prn)
 {
     void *handle = NULL;
-    int (*vecm_beta_test) (GRETL_VAR *, PRN *);
+    int (*vecm_beta_test) (GRETL_VAR *, const DATAINFO *, PRN *);
     int err = 0;
 
     if (vecm->jinfo == NULL || vecm->jinfo->D == NULL) {
 	return E_DATA;
     }    
 
-    *gretl_errmsg = 0;
+    gretl_error_clear();
     
     vecm_beta_test = get_plugin_function("vecm_beta_test", &handle);
     
     if (vecm_beta_test == NULL) {
 	err = 1;
     } else {
-	err = (* vecm_beta_test) (vecm, prn);
+	err = (* vecm_beta_test) (vecm, pdinfo, prn);
 	close_plugin(handle);
     }
     
@@ -2116,20 +2098,21 @@ int gretl_VECM_test_beta (GRETL_VAR *vecm, PRN *prn)
 }
 
 static int 
-johansen_complete (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo, PRN *prn)
+johansen_complete (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo, 
+		   gretlopt opt, PRN *prn)
 {
     void *handle = NULL;
-    int (*johansen) (GRETL_VAR *, double ***, DATAINFO *, PRN *);
+    int (*johansen) (GRETL_VAR *, double ***, DATAINFO *, gretlopt, PRN *);
     int err = 0;
 
-    *gretl_errmsg = 0;
+    gretl_error_clear();
     
     johansen = get_plugin_function("johansen_analysis", &handle);
 
     if (johansen == NULL) {
 	err = 1;
     } else {
-	err = (* johansen) (jvar, pZ, pdinfo, prn);
+	err = (* johansen) (jvar, pZ, pdinfo, opt, prn);
 	close_plugin(handle);
     }
     
@@ -2531,17 +2514,11 @@ johansen_VAR_prepare (int order, int rank, const int *list, const int *exolist,
 
     if (seasonals) {
 	if (pdinfo->pd > 1) {
-#if 0
-	    /* Center the dummies only if there's a constant in the VAR? 
-	       This seems to be a bad idea.
-	    */
-	    int flag = (jcode(jvar) >= J_UNREST_CONST)? 1 : -1;
-#else
-	    int flag = 1;
-#endif
+	    int center = 1;
+
 	    jvar->jinfo->seasonals = pdinfo->pd - 1;
 	    nexo += pdinfo->pd - 1;
-	    di0 = dummy(pZ, pdinfo, flag);
+	    di0 = dummy(pZ, pdinfo, center);
 	    if (di0 == 0) {
 		jvar->err = E_ALLOC;
 	    }
@@ -2689,13 +2666,13 @@ johansen_driver (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
     if (!jvar->err) {
 	gretl_matrix_multiply_mod(jvar->jinfo->u, GRETL_MOD_NONE,
 				  jvar->jinfo->u, GRETL_MOD_TRANSPOSE,
-				  jvar->jinfo->Suu);
+				  jvar->jinfo->Suu, GRETL_MOD_NONE);
 	gretl_matrix_multiply_mod(jvar->jinfo->v, GRETL_MOD_NONE,
 				  jvar->jinfo->v, GRETL_MOD_TRANSPOSE,
-				  jvar->jinfo->Svv);
+				  jvar->jinfo->Svv, GRETL_MOD_NONE);
 	gretl_matrix_multiply_mod(jvar->jinfo->u, GRETL_MOD_NONE,
 				  jvar->jinfo->v, GRETL_MOD_TRANSPOSE,
-				  jvar->jinfo->Suv);
+				  jvar->jinfo->Suv, GRETL_MOD_NONE);
 
 	gretl_matrix_divide_by_scalar(jvar->jinfo->Suu, jvar->T);
 	gretl_matrix_divide_by_scalar(jvar->jinfo->Svv, jvar->T);
@@ -2735,7 +2712,7 @@ johansen_driver (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
 	   by the special code in irfboot.c
 	*/
 	if (!jvar->err && !(opt & OPT_B)) {
-	    jvar->err = johansen_complete(jvar, pZ, pdinfo, prn);
+	    jvar->err = johansen_complete(jvar, pZ, pdinfo, opt, prn);
 	}
     } 
 
@@ -2801,8 +2778,8 @@ GRETL_VAR *johansen_test (int order, const int *list, double ***pZ, DATAINFO *pd
  * @opt: %OPT_A: include constant plus restricted trend; %OPT_D:
  * include centered seasonals; %OPT_N: no constant; %OPT_R:
  * restricted constant; %OPT_T: constant and unrestricted trend
- * (note: default "case" is unrestricted constant).
- * %OPT_V: produce verbose results.
+ * (note: default "case" is unrestricted constant);
+ * %OPT_V: produce verbose results; %OPT_Q: just print the tests.
  * @prn: gretl printing struct.
  *
  * Carries out the Johansen test for cointegration and prints the
@@ -2812,8 +2789,8 @@ GRETL_VAR *johansen_test (int order, const int *list, double ***pZ, DATAINFO *pd
  * Returns: 0 on success, non-zero code on error.
  */
 
-int johansen_test_simple (int order, const int *list, double ***pZ, DATAINFO *pdinfo,
-			  gretlopt opt, PRN *prn)
+int johansen_test_simple (int order, const int *list, double ***pZ, 
+			  DATAINFO *pdinfo, gretlopt opt, PRN *prn)
 {
     GRETL_VAR *jvar;
     int err;
@@ -2847,9 +2824,9 @@ int johansen_test_simple (int order, const int *list, double ***pZ, DATAINFO *pd
  * the VECM system or %NULL on failure.
  */
 
-GRETL_VAR *vecm (int order, int rank, int *list, 
-		 double ***pZ, DATAINFO *pdinfo,
-		 gretlopt opt, PRN *prn, int *err)
+GRETL_VAR *gretl_VECM (int order, int rank, int *list, 
+		       double ***pZ, DATAINFO *pdinfo,
+		       gretlopt opt, PRN *prn, int *err)
 {
     GRETL_VAR *jvar = NULL;
     int *endo_list = NULL, *exo_list = NULL;
@@ -2896,6 +2873,10 @@ int gretl_VAR_attach_restrictions (GRETL_VAR *var, gretl_matrix *D)
 	return 1;
     }
 
+    if (var->jinfo->D != NULL) {
+	gretl_matrix_free(var->jinfo->D);
+    }
+
     var->jinfo->D = D;
 
     return 0;
@@ -3090,6 +3071,10 @@ double *gretl_VAR_get_series (const GRETL_VAR *var, const DATAINFO *pdinfo,
     return x;    
 }
 
+#define vecm_matrix(i) (i == M_JALPHA || i == M_JBETA || \
+                        i == M_JVBETA || i == M_JS00 || \
+                        i == M_JS11 || i == M_JS01)
+
 gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx, 
 				    int *err)
 {
@@ -3107,7 +3092,7 @@ gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx,
 	src = var->A;
     } else if (idx == M_VCV) {
 	src = var->S;
-    } else if (idx == M_JALPHA || idx == M_JBETA || idx == M_JVBETA ) {
+    } else if (vecm_matrix(idx)) {
 	if (var->jinfo != NULL) {
 	    switch (idx) {
 	    case M_JALPHA: 
@@ -3119,6 +3104,15 @@ gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx,
 	    case M_JVBETA: 
 		src = var->jinfo->Bvar;
 		break;
+	    case M_JS00:
+		src = var->jinfo->Suu;
+		break;
+	    case M_JS11:
+		src = var->jinfo->Svv;
+		break;
+	    case M_JS01:
+		src = var->jinfo->Suv;
+		break;
 	    }
 	}
     }
@@ -3156,6 +3150,101 @@ static GRETL_VAR *gretl_VAR_rebuilder_new (void)
     return var;
 }
 
+static int VAR_retrieve_jinfo (xmlNodePtr node, xmlDocPtr doc,
+			       GRETL_VAR *var)
+{
+    xmlNodePtr cur = node->xmlChildrenNode;
+    JohansenInfo *jinfo;
+    int *list = NULL;
+    int ID, code, rank;
+    int seas, nexo;
+    int got = 0;
+    int err = 0;
+
+    got += gretl_xml_get_prop_as_int(node, "ID", &ID);
+    got += gretl_xml_get_prop_as_int(node, "code", &code);
+    got += gretl_xml_get_prop_as_int(node, "rank", &rank);
+    got += gretl_xml_get_prop_as_int(node, "seasonals", &seas);
+    got += gretl_xml_get_prop_as_int(node, "nexo", &nexo);
+
+    if (got != 5) {
+	return E_DATA;
+    }
+
+    got = 0;
+
+    while (cur != NULL && !err) {
+	if (!xmlStrcmp(cur->name, (XUC) "list")) {
+	    list = gretl_xml_node_get_list(cur, doc, &err);
+	    got = 1;
+	    break;
+	} 
+	cur = cur->next;
+    }
+
+    if (!err && !got) {
+	err = E_DATA;
+    }
+
+    if (err) {
+	return err;
+    }
+
+    jinfo = johansen_info_new(list, NULL, rank, OPT_NONE);
+    if (jinfo == NULL) {
+	return E_ALLOC;
+    }
+
+    jinfo->ID = ID;
+    jinfo->code = code;
+    jinfo->rank = rank;
+    jinfo->seasonals = seas;
+    jinfo->nexo = nexo;
+
+    cur = node->xmlChildrenNode;
+
+    while (cur != NULL && !err) {
+	if (!xmlStrcmp(cur->name, (XUC) "difflist")) {
+	    jinfo->difflist = gretl_xml_node_get_list(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "exolist")) {
+	    jinfo->exolist = gretl_xml_node_get_list(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "biglist")) {
+	    jinfo->biglist = gretl_xml_node_get_list(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "levels_list")) {
+	    jinfo->levels_list = gretl_xml_node_get_list(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "varlist")) {
+	    jinfo->varlist = gretl_xml_node_get_list(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "u")) {
+	    jinfo->u = gretl_xml_get_matrix(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "v")) {
+	    jinfo->v = gretl_xml_get_matrix(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "Suu")) {
+	    jinfo->Suu = gretl_xml_get_matrix(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "Suv")) {
+	    jinfo->Suv = gretl_xml_get_matrix(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "Beta")) {
+	    jinfo->Beta = gretl_xml_get_matrix(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "Alpha")) {
+	    jinfo->Alpha = gretl_xml_get_matrix(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "Bse")) {
+	    jinfo->Bse = gretl_xml_get_matrix(cur, doc, &err);
+	} else if (!xmlStrcmp(cur->name, (XUC) "D")) {
+	    jinfo->D = gretl_xml_get_matrix(cur, doc, &err);
+	}
+	cur = cur->next;
+    }
+
+    if (err) {
+	johansen_info_free(jinfo);
+    } else {
+	var->jinfo = jinfo;
+    }
+
+    fprintf(stderr, "VAR_retrieve_jinfo: err = %d\n", err);
+
+    return err;
+}
+
 static int VAR_retrieve_equations (xmlNodePtr node, xmlDocPtr doc,
 				   MODEL **models, int neqns)
 {
@@ -3226,6 +3315,8 @@ GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
 	    var->Ivals = gretl_xml_get_double_array(cur, doc, &n, err);
 	} else if (!xmlStrcmp(cur->name, (XUC) "equations")) {
 	    *err = VAR_retrieve_equations(cur, doc, var->models, var->neqns);
+	} else if (!xmlStrcmp(cur->name, (XUC) "gretl-johansen")) {
+	    *err = VAR_retrieve_jinfo(cur, doc, var);
 	}
 	cur = cur->next;
     } 
