@@ -29,9 +29,9 @@ GRETL_VAR *gretl_VAR (int order, int *list,
 		      double ***pZ, DATAINFO *pdinfo,
 		      gretlopt opt, PRN *prn, int *err);
 
-GRETL_VAR *vecm (int order, int rank, int *list, 
-		 double ***pZ, DATAINFO *pdinfo,
-		 gretlopt opt, PRN *prn, int *err);
+GRETL_VAR *gretl_VECM (int order, int rank, int *list, 
+		       double ***pZ, DATAINFO *pdinfo,
+		       gretlopt opt, PRN *prn, int *err);
 
 const gretl_matrix *
 gretl_VAR_get_forecast_matrix (GRETL_VAR *var, int t1, int t2, int pre_obs,
@@ -51,8 +51,7 @@ int gretl_VAR_autocorrelation_test (GRETL_VAR *var, int order,
 				    PRN *prn);
 
 int gretl_VAR_arch_test (GRETL_VAR *var, int order, 
-			 double ***pZ, DATAINFO *pdinfo,
-			 PRN *prn);
+			 DATAINFO *pdinfo, PRN *prn);
 
 int gretl_VAR_normality_test (const GRETL_VAR *var, PRN *prn);
 
@@ -100,7 +99,8 @@ int gretl_VAR_get_highest_variable (const GRETL_VAR *var,
 
 int gretl_VECM_n_beta (const GRETL_VAR *vecm);
 
-int gretl_VECM_test_beta (GRETL_VAR *vecm, PRN *prn);
+int gretl_VECM_test_beta (GRETL_VAR *vecm, const DATAINFO *pdinfo, 
+			  PRN *prn);
 
 int gretl_VECM_rank (const GRETL_VAR *vecm);
 
