@@ -28,6 +28,7 @@
 #include "bhhh_max.h"
 #include "libset.h"
 #include "kalman.h"
+#include "matrix_extra.h"
 
 #define ARMA_DEBUG 0
 
@@ -606,7 +607,7 @@ static void write_big_theta (const double *theta,
     }
 
     for (i=1; i<=qmax; i++) {
-	gretl_vector_set(H, i, mc[i]);
+	H->val[i] = mc[i];
     }    
 }
 
@@ -852,7 +853,7 @@ static int arma_OPG_vcv (MODEL *pmod, kalman *K, double *b,
 
     gretl_matrix_multiply_mod(G, GRETL_MOD_NONE,
 			      G, GRETL_MOD_TRANSPOSE,
-			      V);
+			      V, GRETL_MOD_NONE);
 
     err = gretl_invert_symmetric_matrix(V);
 
@@ -1234,13 +1235,13 @@ static int kalman_arma (const int *alist, double *coeff,
 	}
 	kalman_use_ARMA_ll(K);
 	err = BFGS_max(b, ainfo->nc, maxit, reltol, 
-		       &fncount, &grcount, kalman_arma_ll, 
+		       &fncount, &grcount, kalman_arma_ll, C_LOGLIK,
 		       NULL, K, (prn != NULL)? OPT_V : OPT_NONE, prn);
 	if (err) {
 	    fprintf(stderr, "BFGS_max returned %d\n", err);
 	} else if (1 || getenv("ARMA_HESSIAN")) { /* Hello! Testing */
 	    kalman_do_ma_check = 0;
-	    hess = numerical_hessian(b, ainfo->nc, kalman_arma_ll, K);
+	    hess = numerical_hessian(b, ainfo->nc, kalman_arma_ll, K, &err);
 	    kalman_do_ma_check = 1;
 	}
     }
@@ -1380,18 +1381,18 @@ static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
 #endif
     char fnstr[MAXLINE];
     char term[32];
-    nls_spec *spec;
+    nlspec *spec;
     int *plist = NULL;
     int v, oldv = pdinfo->v;
     int nparam;
     int i, j, k, err = 0;
 
-    spec = nls_spec_new(NLS, pdinfo);
+    spec = nlspec_new(NLS, pdinfo);
     if (spec == NULL) {
 	return E_ALLOC;
     }
 
-    nls_spec_set_t1_t2(spec, 0, ainfo->t2 - ainfo->t1); /* ?? */
+    nlspec_set_t1_t2(spec, 0, ainfo->t2 - ainfo->t1); /* ?? */
 
     nparam = ainfo->ifc + ainfo->p + ainfo->P + ainfo->nexo;
 
@@ -1472,15 +1473,15 @@ static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
     fprintf(stderr, "initting using NLS spec:\n %s\n", fnstr);
 #endif
 
-    err = nls_spec_set_regression_function(spec, fnstr, pdinfo);
+    err = nlspec_set_regression_function(spec, fnstr, pdinfo);
 
     if (!err) {
-	err = nls_spec_add_param_list(spec, plist, (const double **) *pZ,
-				      pdinfo);
+	err = nlspec_add_param_list(spec, plist, (const double **) *pZ,
+				    pdinfo);
     }
 
     if (!err) {
-	*amod = model_from_nls_spec(spec, pZ, pdinfo, nlsopt, prn);
+	*amod = model_from_nlspec(spec, pZ, pdinfo, nlsopt, prn);
 	err = amod->errcode;
 #if ARMA_DEBUG
 	if (!err) {
@@ -1490,9 +1491,9 @@ static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
 #endif
     }
 
-    bailout:
+ bailout:
 
-    nls_spec_destroy(spec);
+    nlspec_destroy(spec);
     free(plist);
 
     return err;
