@@ -17,25 +17,22 @@
  *
  */
 
+#include "../cephes/mconf.h"
+
 #include "libgretl.h"
 #include "bhhh_max.h"
 #include "libset.h"
 
-#include "../cephes/mconf.h"
-
 #include <glib.h>
 
 #ifdef WIN32
 # include <windows.h>
 # include <io.h>
 #else
-# if GLIB_CHECK_VERSION(2,0,0)
-#  define GLIB2
-#  include <signal.h>
-# endif /* GLIB_CHECK_VERSION */
+# include <signal.h>
 #endif
 
-#ifdef GLIB2
+#ifndef WIN32
 
 static int tramo_x12a_spawn (const char *workdir, const char *fmt, ...)
 {
@@ -110,7 +107,7 @@ static int tramo_x12a_spawn (const char *workdir, const char *fmt, ...)
     return ret;
 }
 
-#endif
+#endif /* !WIN32 */
 
 #include "arma_common.c"
 
@@ -127,7 +124,7 @@ static int add_unique_output_file (MODEL *pmod, const char *path)
     err = rename(fname, unique);
     if (!err) {
 	gretl_model_set_data(pmod, "x12a_output", g_strdup(unique),
-			     strlen(fname) + 1);
+			     MODEL_DATA_STRING, strlen(fname) + 1);
     } 
 
     return err;
@@ -162,11 +159,18 @@ static int x12_date_to_n (const char *s, const DATAINFO *pdinfo)
 {
     char date[12] = {0};
 
+    if (dated_daily_data(pdinfo)) {
+	/* FIXME? */
+	return atoi(s);
+    }
+
     if (pdinfo->pd > 1) {
-	if (strlen(s) <= 4) {
-	    strncat(date, s, 2);
+	int len = strlen(s);
+
+	if (len <= 4) {
+	    strncat(date, s, len - 2);
 	    strcat(date, ":");
-	    strncat(date, s + 2, 4);
+	    strcat(date, s + len - 2);
 	} else {
 	    strncat(date, s, 4);
 	    strcat(date, ":");
@@ -185,12 +189,13 @@ static int get_ll_stats (const char *fname, MODEL *pmod)
 {
     FILE *fp;
     char line[80], statname[12];
+    int nobs = 0, nefobs = 0;
     double x;
 
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
 	fprintf(stderr, "Couldn't read from '%s'\n", fname);
-	return 1;
+	return E_FOPEN;
     }
 
     pmod->sigma = NADBL;
@@ -199,7 +204,8 @@ static int get_ll_stats (const char *fname, MODEL *pmod)
 
     while (fgets(line, sizeof line, fp)) {
 	if (sscanf(line, "%11s %lf", statname, &x) == 2) {
-	    if (!strcmp(statname, "nobs")) pmod->nobs = (int) x;
+	    if (!strcmp(statname, "nobs")) nobs = (int) x;
+	    else if (!strcmp(statname, "nefobs")) nefobs = (int) x;
 	    else if (!strcmp(statname, "var")) pmod->sigma = sqrt(x);
 	    else if (!strcmp(statname, "lnlkhd")) pmod->lnL = x;
 	    else if (!strcmp(statname, "aic")) pmod->criterion[C_AIC] = x;
@@ -212,6 +218,13 @@ static int get_ll_stats (const char *fname, MODEL *pmod)
 
     fclose(fp);
 
+    if (nobs != nefobs && nefobs > 0) {
+	pmod->nobs = nefobs;
+	pmod->t1 += nobs - nefobs;
+    } else {
+	pmod->nobs = nobs;
+    }
+
     return 0;
 }
 
@@ -239,7 +252,7 @@ static int get_roots (const char *fname, MODEL *pmod,
     if (fp == NULL) {
 	fprintf(stderr, "Couldn't read from '%s'\n", fname);
 	free(roots);
-	return 1;
+	return E_FOPEN;
     }
 
     gretl_push_c_numeric_locale();
@@ -262,13 +275,15 @@ static int get_roots (const char *fname, MODEL *pmod,
     fclose(fp);
 
     if (i != nr) {
+	fprintf(stderr, "Error reading '%s'\n", fname);
 	free(roots);
 	roots = NULL;
-	err = 1;
+	err = E_DATA;
     }
 
     if (roots != NULL) {
-	gretl_model_set_data(pmod, "roots", roots, nr * sizeof *roots);
+	gretl_model_set_data(pmod, "roots", roots, MODEL_DATA_CMPLX_ARRAY,
+			     nr * sizeof *roots);
     }
 
     return err;
@@ -336,11 +351,11 @@ static int get_x12a_vcv (const char *fname, MODEL *pmod, int nc)
 static int 
 get_estimates (const char *fname, MODEL *pmod, struct arma_info *ainfo)
 {
-    double *ar_coeff = pmod->coeff + arma_has_const(ainfo);
+    double *ar_coeff = pmod->coeff + ainfo->ifc;
     double *ma_coeff = ar_coeff + ainfo->p + ainfo->P;
     double *x_coeff = ma_coeff + ainfo->q + ainfo->Q;
 
-    double *ar_sderr = pmod->sderr + arma_has_const(ainfo);
+    double *ar_sderr = pmod->sderr + ainfo->ifc;
     double *ma_sderr = ar_sderr + ainfo->p + ainfo->P;
     double *x_sderr = ma_sderr + ainfo->q + ainfo->Q;
 
@@ -354,7 +369,7 @@ get_estimates (const char *fname, MODEL *pmod, struct arma_info *ainfo)
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
 	fprintf(stderr, "Couldn't read from '%s'\n", fname);
-	return 1;
+	return E_FOPEN;
     }
 
     for (i=0; i<ainfo->nc; i++) {
@@ -400,35 +415,12 @@ get_estimates (const char *fname, MODEL *pmod, struct arma_info *ainfo)
 
     for (i=0; i<ainfo->nc; i++) {
 	if (na(pmod->coeff[i]) || na(pmod->sderr[i])) {
-	    err = 1;
+	    fprintf(stderr, "Error reading '%s'\n", fname);
+	    err = E_DATA;
 	    break;
 	}
     }
 
-    if (!err) {
-	double narfac = 1.0;
-	double sarfac = 1.0;
-	double arfac;
-
-	for (i=1; i<=ainfo->p; i++) {
-	    narfac -= pmod->coeff[i];
-	}
-
-	for (i=1; i<=ainfo->P; i++) {
-	    sarfac -= pmod->coeff[i+ainfo->p];
-	}
-
-	arfac = narfac * sarfac;
-
-	pmod->coeff[0] *= arfac;
-	pmod->sderr[0] *= arfac;
-
-	for (i=0; i<ainfo->r; i++) {
-	    x_coeff[i] *= arfac;
-	    x_sderr[i] *= arfac;
-	}
-    }
-
     return err;
 }
 
@@ -461,7 +453,7 @@ get_uhat (const char *fname, MODEL *pmod, const DATAINFO *pdinfo)
 	    if (t >= 0 && t < pdinfo->n) {
 		pmod->uhat[t] = x;
 		nobs++;
-	    }
+	    } 
 	}
     }
 
@@ -470,6 +462,7 @@ get_uhat (const char *fname, MODEL *pmod, const DATAINFO *pdinfo)
     fclose(fp);
 
     if (nobs == 0) {
+	fprintf(stderr, "Error reading '%s'\n", fname);
 	err = E_DATA;
     }
 
@@ -528,14 +521,10 @@ populate_arma_model (MODEL *pmod, const int *list, const char *path,
 #endif
 
     if (err) {
-	fprintf(stderr, "problem getting model info\n");
-	pmod->errcode = E_FOPEN;
+	fprintf(stderr, "problem reading X-12-ARIMA model info\n");
+	pmod->errcode = err;
     } else {
-	write_arma_model_stats(pmod, NULL, list, Z, NULL, ainfo);
-	gretl_model_add_arma_varnames(pmod, pdinfo, ainfo->yno,
-				      ainfo->p, ainfo->q, 
-				      ainfo->P, ainfo->Q,
-				      ainfo->r);
+	write_arma_model_stats(pmod, list, ainfo, Z, pdinfo);
     }
 }
 
@@ -549,7 +538,11 @@ output_series_to_spc (const int *list, const double **Z,
 
     for (t=t1; t<=t2; t++) {
 	for (i=1; i<=list[0]; i++) {
-	    fprintf(fp, "%g ", Z[list[i]][t]);
+	    if (na(Z[list[i]][t])) {
+		fputs("-9999.0 ", fp);
+	    } else {
+		fprintf(fp, "%g ", Z[list[i]][t]);
+	    }
 	}
 	fputc('\n', fp);
     }
@@ -564,7 +557,7 @@ arma_info_get_x_list (struct arma_info *ainfo, const int *alist)
     int start = arma_list_y_position(ainfo);
     int i;
 
-    xlist = gretl_list_new(ainfo->r);
+    xlist = gretl_list_new(ainfo->nexo);
 
     if (xlist != NULL) {
 	for (i=1; i<=xlist[0]; i++) {
@@ -578,11 +571,20 @@ arma_info_get_x_list (struct arma_info *ainfo, const int *alist)
 static void 
 make_x12a_date_string (int t, const DATAINFO *pdinfo, char *str)
 {
-    double dx = date(t, pdinfo->pd, pdinfo->sd0);
-    int yr = (int) dx;
-    int subper = 0;
+    double dx;
+    int yr, subper = 0;
     char *s;
 
+    /* daily data: for now we'll try just numbering the observations
+       consecutively */
+    if (dated_daily_data(pdinfo)) {
+	sprintf(str, "%d", t + 1);
+	return;
+    } 
+
+    dx = date(t, pdinfo->pd, pdinfo->sd0);
+    yr = (int) dx;
+
     sprintf(str, "%g", dx);
     s = strchr(str, '.');
 
@@ -599,6 +601,9 @@ make_x12a_date_string (int t, const DATAINFO *pdinfo, char *str)
     }    
 }
 
+#define MAXOBS 720
+#define MAXFCAST 60
+
 static int write_spc_file (const char *fname, 
 			   const double **Z, const DATAINFO *pdinfo,
 			   struct arma_info *ainfo, const int *alist,
@@ -608,10 +613,11 @@ static int write_spc_file (const char *fname,
     int *xlist = NULL;
     FILE *fp;
     char datestr[12];
+    int nfcast = 0;
     int tmax;
     int i;
 
-    if (ainfo->r > 0) {
+    if (ainfo->nexo > 0) {
 	xlist = arma_info_get_x_list(ainfo, alist);
 	if (xlist == NULL) {
 	    return E_ALLOC;
@@ -628,18 +634,40 @@ static int write_spc_file (const char *fname,
 
     make_x12a_date_string(ainfo->t1, pdinfo, datestr);
 
-    fprintf(fp, "series {\n period = %d\n title = \"%s\"\n", pdinfo->pd, 
-	    pdinfo->varname[ainfo->yno]);
+    if (dated_daily_data(pdinfo)) {
+	/* FIXME? */
+	fprintf(fp, "series {\n period = 1\n title = \"%s\"\n",  
+		pdinfo->varname[ainfo->yno]);
+    } else {
+	fprintf(fp, "series {\n period = %d\n title = \"%s\"\n", pdinfo->pd, 
+		pdinfo->varname[ainfo->yno]);
+    }	
     fprintf(fp, " start = %s\n", datestr);
 
     ylist[0] = 1;
     ylist[1] = ainfo->yno;
 
+    tmax = ainfo->t2;
+
     if ((opt & OPT_F) && ainfo->t2 < pdinfo->n - 1) {
+	int nobs;
+
+	/* FIXME ensure we don't print any NAs */
+
 	tmax = pdinfo->n - 1;
-    } else {
-	tmax = ainfo->t2;
-    }
+	nobs = tmax - ainfo->t1 + 1;
+	if (nobs > MAXOBS) {
+	    tmax -= nobs - MAXOBS;
+	}
+	nfcast = tmax - ainfo->t2;
+	if (nfcast > MAXFCAST) {
+	    tmax -= nfcast - MAXFCAST;
+	    nfcast -= nfcast - MAXFCAST;
+	}
+#if 0
+	fprintf(stderr, "x12a: doing forecast: nfcast = %d\n", nfcast);
+#endif
+    } 
 
     output_series_to_spc(ylist, Z, ainfo->t1, tmax, fp);
 
@@ -652,10 +680,10 @@ static int write_spc_file (const char *fname,
 
     /* regression specification */
     fputs("Regression {\n", fp);
-    if (arma_has_const(ainfo)) {
+    if (ainfo->ifc) {
 	fputs(" variables = (const)\n", fp);
     }
-    if (ainfo->r > 0) {
+    if (ainfo->nexo > 0) {
 	fputs(" user = ( ", fp);
 	for (i=1; i<=xlist[0]; i++) {
 	    fprintf(fp, "%s ", pdinfo->varname[xlist[i]]);
@@ -664,7 +692,7 @@ static int write_spc_file (const char *fname,
 	output_series_to_spc(xlist, Z, ainfo->t1, tmax, fp);
 	free(xlist);
     }
-    fputs("\n}\n", fp);
+    fputs("}\n", fp);
 
     /* arima specification */
     if (ainfo->P > 0 || ainfo->Q > 0) {
@@ -692,9 +720,9 @@ static int write_spc_file (const char *fname,
 
     fputs("}\n", fp);
 
-    if (opt & OPT_F) {
+    if (nfcast > 0) {
 	fputs("forecast {\n save = (ftr)\n", fp);
-	fprintf(fp, " maxlead = %d\n}\n", tmax - ainfo->t2);
+	fprintf(fp, " maxlead = %d\n}\n", nfcast);
     }
 
     gretl_pop_c_numeric_locale();
@@ -704,6 +732,23 @@ static int write_spc_file (const char *fname,
     return 0;
 }
 
+static void delete_old_files (const char *path)
+{
+    const char *exts[] = {
+	"acm", "itr", "lkf", "lks", "mdl", "est", "rts",
+	"rcm", "rsd", "ftr", "err", "log", "out", NULL
+    };
+    char old[MAXLEN];
+    int i, n = strlen(path);
+
+    for (i=0; exts[i] != NULL; i++) {
+	*old = '\0';
+	strncat(old, path, n - 3);
+	strcat(old, exts[i]);
+	remove(old);
+    }
+}
+
 MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 		      gretlopt opt, PRN *prn)
 {
@@ -713,11 +758,11 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
     char yname[VNAMELEN], path[MAXLEN];
     int *alist = NULL;
     PRN *aprn = NULL;
-#ifndef GLIB2
-    char cmd[MAXLEN];
-#endif
     MODEL armod;
     struct arma_info ainfo;
+#ifdef WIN32
+    char *cmd;
+#endif
     int err = 0;
 
     if (verbose) {
@@ -725,7 +770,7 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
     } 
 
     if (pdinfo->t2 < pdinfo->n - 1) {
-	/* FIXME this is temporary */
+	/* FIXME this is temporary (OPT_F -> generate forecast) */
 	opt |= OPT_F;
     }
 
@@ -765,17 +810,17 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
     sprintf(path, "%s%c%s.spc", workdir, SLASH, yname);
     write_spc_file(path, Z, pdinfo, &ainfo, alist, opt);
 
+    /* remove any files from on old run, in case of error */
+    delete_old_files(path);
+
     /* run the program */
-#if defined(WIN32)
-    sprintf(cmd, "\"%s\" %s -r -p -q", prog, yname);
+#ifdef WIN32
+    cmd = g_strdup_printf("\"%s\" %s -r -p -q", prog, yname);
     err = winfork(cmd, workdir, SW_SHOWMINIMIZED, 
 		  CREATE_NEW_CONSOLE | HIGH_PRIORITY_CLASS);
-#elif defined(GLIB2)
-    err = tramo_x12a_spawn(workdir, prog, yname, "-r", "-p", "-q", "-n", NULL);
+    g_free(cmd);
 #else
-    sprintf(cmd, "cd \"%s\" && \"%s\" %s -r -p -q -n >/dev/null", 
-	    workdir, prog, yname);
-    err = gretl_spawn(cmd);
+    err = tramo_x12a_spawn(workdir, prog, yname, "-r", "-p", "-q", "-n", NULL);
 #endif
 
     if (!err) {
@@ -788,12 +833,15 @@ MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo,
 	    print_iterations(path, aprn);
 	}
 	if (!armod.errcode) {
-	    int xcode = (opt & OPT_C)? 2 : 1;
+	    int acode = ARMA_X12A;
 
 	    if (gretl_in_gui_mode()) {
 		add_unique_output_file(&armod, path);
 	    }
-	    gretl_model_set_int(&armod, "arma_by_x12a", xcode);
+	    if (!(opt & OPT_C)) {
+		acode |= ARMA_EXACT;
+	    }
+	    gretl_model_set_int(&armod, "arma_flags", acode);
 	}	
     } else {
 	armod.errcode = E_UNSPEC;
