@@ -87,36 +87,6 @@ garch_full_hessian (int t1, int t2,
 		    int q, int p, double *h,
 		    double **dhdp, double *zt);
 
-static void free_2d_array (double **A, int k)
-{
-    int i;
-
-    if (A == NULL) return;
-
-    for (i=0; i<k; i++) free(A[i]);
-    free(A);
-}
-
-static double **allocate_2d_array (int k, int T)
-{
-    double **A;
-    int i, j;
-
-    A = malloc(k * sizeof *A);
-    if (A == NULL) return NULL;
-
-    for (i=0; i<k; i++) {
-	A[i] = malloc(T * sizeof **A);
-	if (A[i] == NULL) {
-	    for (j=0; j<i; j++) free(A[j]);
-	    free(A);
-	    return NULL;
-	}
-    }
-
-    return A;
-}
-
 static int vs_allocate (double ***pdhdp, double ***pg, 
 			double **pparam, double **pgrad, 
 			double **pc, double **paux, double **pvch,
@@ -150,12 +120,12 @@ static int vs_allocate (double ***pdhdp, double ***pg,
 	goto bailout;
     }
 
-    D = allocate_2d_array(np, T);
+    D = doubles_array_new(np, T);
     if (D == NULL) {
 	goto bailout;
     }
 
-    G = allocate_2d_array(nrc, T);
+    G = doubles_array_new(nrc, T);
     if (G == NULL) {
 	goto bailout;
     }
@@ -185,8 +155,8 @@ static int vs_allocate (double ***pdhdp, double ***pg,
     free(vch);
     free(parpre);
     free(yhat);
-    free_2d_array(D, np);
-    free_2d_array(G, nrc);
+    doubles_array_free(D, np);
+    doubles_array_free(G, nrc);
 
     return 1;
 }
@@ -196,8 +166,8 @@ static void vs_free (double **dhdp, int np, double **g, int nrc,
 		     double *c, double *aux, double *vcv,
 		     double *parpre, double *yhat)
 {
-    free_2d_array(dhdp, np);
-    free_2d_array(g, nrc);
+    doubles_array_free(dhdp, np);
+    doubles_array_free(g, nrc);
     free(param);
     free(grad);
     free(c);
@@ -207,7 +177,7 @@ static void vs_free (double **dhdp, int np, double **g, int nrc,
     free(yhat);
 }
 
-static void print_iter_info (int iter, double *theta, int m, double ll,
+static void garch_iter_info (int iter, double *theta, int m, double ll,
 			     int hess, PRN *prn)
 {
     int i;
@@ -252,7 +222,7 @@ static void copy_coeff (const double *c, int nc, double *b)
 {
     int i;
 
-    for (i = 0; i < nc; ++i) {
+    for (i=0; i<nc; i++) {
 	b[i] = c[i];
     }
 } 
@@ -271,19 +241,19 @@ int ols_ (int t1, int t2, const double **X, int nx,
 
     copy_coeff(c, nc, b);
 
-    for (t = t1; t <= t2; ++t) {
+    for (t=t1; t<=t2; t++) {
 	amax[t] = get_yhat(X, nx, t, b);
     }
 
-    for (i = 0; i < nc; ++i) {
+    for (i=0; i<nc; i++) {
 	aux[i] = 0.0;
-	for (j = 0; j < nc; ++j) {
+	for (j=0; j<nc; j++) {
 	    vc[i + j * nc] = 0.0;
 	}
     }
 
-    for (t = t1; t <= t2; ++t) {
-	for (k = 0; k < nc; ++k) {
+    for (t=t1; t<=t2; t++) {
+	for (k=0; k<nc; k++) {
 	    oldc = c[k];
 	    deltc = relinc;
 	    if (oldc != 0.0) {
@@ -299,11 +269,11 @@ int ols_ (int t1, int t2, const double **X, int nx,
 	}
 	copy_coeff(c, nc, b);
 
-	/* cumulates all the w'z into diagonal blocks of vc 
+	/* cumulates all the w'z into diagonal blocks of vc, 
 	   and w'y into elements of aux */
-	for (i = 0; i < nc; ++i) {
+	for (i=0; i<nc; i++) {
 	    aux[i] += g[i][t] * y[t];
-	    for (j = 0; j < nc; ++j) {
+	    for (j=0; j<nc; j++) {
 		vc[i + j * nc] += g[i][t] * g[j][t];
 	    }
 	}
@@ -313,11 +283,9 @@ int ols_ (int t1, int t2, const double **X, int nx,
 
     if (err == 0) {
 	/* compute coefficients */
-	for (i = 0; i < nc; ++i) {
+	for (i=0; i<nc; i++) {
 	    c[i] = 0.0;
-	}
-	for (i = 0; i < nc; ++i) {
-	    for (j = 0; j < nc; ++j) {
+	    for (j=0; j<nc; j++) {
 		c[i] += vc[i + j * nc] * aux[j];
 	    }
 	}
@@ -326,8 +294,8 @@ int ols_ (int t1, int t2, const double **X, int nx,
 	fputs("OLS: matrix is singular, initial coefficients are unchanged\n",
 	      stderr);
 
-	for (i = 0; i < nc; ++i) {
-	    for (j = 0; j < nc; ++j) {
+	for (i=0; i<nc; i++) {
+	    for (j=0; j<nc; j++) {
 		vc[i + j * nc] = 0.0;
 	    }
 	}
@@ -457,9 +425,7 @@ make_garch_vcv (int t1, int t2,
 		vcv[k] = vcr[k];
 	    }
 	}
-    }
-
-    else if (vopt == VCV_QML) {
+    } else if (vopt == VCV_QML) {
 	vcr = robust_vcv(vch, vco, nparam);
 	if (vcr == NULL) {
 	    err = 1;
@@ -468,9 +434,7 @@ make_garch_vcv (int t1, int t2,
 	for (k=0; k<np2; k++) {
 	    vcv[k] = vcr[k];
 	}	
-    }	
-	
-    else if (vopt == VCV_HESSIAN) {
+    } else if (vopt == VCV_HESSIAN) {
 	for (k=0; k<np2; k++) {
 	    vcv[k] = vch[k];
 	}
@@ -502,13 +466,13 @@ make_garch_vcv (int t1, int t2,
    res2:  vector of 0's on input, squared resids on output (not needed)
    res:   vector of 0's on input, resids on output
    h:     null pointer on input, conditional variances on output
-   y:     on input, vector with dep. var., not needed on output
+   y:     on input, vector with dep. var.; not needed on output
    amax:  vector; element 0 holds the garch intercept; 1 and 2 the
           arch & garch orders; from 3 onwards, the arch & garch 
           parameters
    b:     0 on input, holds vector of coefficient for the conditional
           mean on output
-   scale: double used to scale dep. var.
+   scale: double used to scale dependent variable
    iters: int, 0 on input, holds number of iterations on output
    prn:   print handle for info on iterations and other diagnostic output
 
@@ -524,7 +488,7 @@ int garch_estimate (int t1, int t2, int nobs,
     int count, conv = 0;
     int i, q, p, nparam;
 
-    double zt[6];   /* max value of (1 + q + p) */
+    double zt[6];   /* 6 == max value of (1 + q + p) */
 
     double pappo, tol1, tol2;
     double sumgra; 
@@ -577,7 +541,7 @@ int garch_estimate (int t1, int t2, int nobs,
     ols_(t1, t2, X, nx, c, nc, y, amax, aux, b, g);
 
 #ifdef FFDEBUG
-    for (i = 0; i < nc; i++) {
+    for (i=0; i<nc; i++) {
 	fprintf(stderr, "after ols g[%d] = %.9g\n", i, g[i]);
     } 
 #endif    
@@ -592,7 +556,7 @@ int garch_estimate (int t1, int t2, int nobs,
 		      X, nx, t1, t2, param, b, &a0, 
 		      q, p, h);
 
-	print_iter_info(it1, param, nparam, ll, 0, prn);
+	garch_iter_info(it1, param, nparam, ll, 0, prn);
 
 	/* store previous coefficients */
 	for (i=0; i<nparam; i++) {
@@ -657,7 +621,7 @@ int garch_estimate (int t1, int t2, int nobs,
 			   grad, param, nparam, b, &a0,
 			   q, p, h, dhdp, zt);
 
-	print_iter_info(ittot++, param, nparam, ll, 1, prn);
+	garch_iter_info(ittot++, param, nparam, ll, 1, prn);
 
 	s1 = 0.0;
 	s2 = 0.0;
@@ -807,12 +771,12 @@ garch_ll (double *c, int nc, double *res2,
 	res2[t] = h[t] = uncvar;
     }
 
-    for (t = t1; t <= t2; ++t) {
+    for (t=t1; t<=t2; t++) {
 	h[t] = *a0;
-	for (i = 1; i <= q; ++i) {
+	for (i=1; i<=q; i++) {
 	    h[t] += res2[t-i] * alpha[i-1];
 	}
-	for (i = 1; i <= p; ++i) {
+	for (i=1; i<=p; i++) {
 	    h[t] += h[t-i] * beta[i-1];
 	}
 	/* arbitrary */
@@ -822,7 +786,7 @@ garch_ll (double *c, int nc, double *res2,
     }
 
     ll = 0.0;
-    for (t = t1; t <= t2; ++t) {
+    for (t=t1; t<=t2; t++) {
 	double hts = h[t] * gscale * gscale;
 
 	ll -= 0.5 * log(hts) + 0.5 * res2[t] / h[t] + LN_SQRT_2_PI;
@@ -840,25 +804,25 @@ garch_ll (double *c, int nc, double *res2,
      sum).
  */
 
-static void check_ht (double *param, int np)
+static void check_ht (double *b, int np)
 {
-    int i;
     double sum = 0.0;
+    int i;
 
-    if (param[0] <= 0.0) {
-	param[0] = SMALL_HT;
+    if (b[0] <= 0.0) {
+	b[0] = SMALL_HT;
     }
 
     for (i=1; i<np; i++) {
-	if (param[i] < 0.0) {
-	    param[i] = 0.0;
+	if (b[i] < 0.0) {
+	    b[i] = 0.0;
 	}
-	sum += param[i];
+	sum += b[i];
     }
 
     if (sum > 1.0) {
 	for (i=1; i<np; i++) {
-	    param[i] /= sum;
+	    b[i] /= sum;
 	}
     }
 } 
@@ -910,7 +874,7 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 	++(*count);
     }
 
-    for (i = 0; i < nc; ++i) {
+    for (i=0; i<nc; i++) {
 	c[i] = param[i];
     }
 
@@ -920,33 +884,33 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
        residuals.
      */
 
-    for (k = 1; k <= p; k++) {
-	for (i = 0; i < nvparm; ++i) {
+    for (k=1; k<=p; k++) {
+	for (i=0; i<nvparm; i++) {
 	    dhdp[nc+i][t1-k] = 0.0;
 	    if (H != NULL) { /* hessian only */
-		for (j = 0; j < nvparm; ++j) {
+		for (j=0; j<nvparm; j++) {
 		    H[nc+i][nc+j][k] = 0.0;
 		}
 	    }
 	}
     }
 
-    for (t = t1; t <= t2; ++t) {
+    for (t=t1; t<=t2; t++) {
 	/* fill in zt at time t (see p. 401) */
 	zt[0] = 1.0;
-	for (i = 1; i <= q; ++i) {
+	for (i=1; i<=q; i++) {
 	    zt[i] = res2[t-i];
 	}
-	for (i = 1; i <= p; ++i) {
+	for (i=1; i<=p; i++) {
 	    zt[q+i] = h[t-i];
 	}
 
 	/* Fill in dhtdp at time t, part relative to variance parameters
 	   (eq. 7, p. 402) 
 	*/
-	for (i = 0; i < nvparm; ++i) {
+	for (i=0; i<nvparm; i++) {
 	    dhdp[nc+i][t] = zt[i];
-	    for (j = 1; j <= p; ++j) {
+	    for (j=1; j<=p; j++) {
 		dhdp[nc+i][t] += dhdp[nc+i][t-j] * beta[j-1];
 	    }
 	}
@@ -959,47 +923,47 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
     */
 
     /* building blocks for pre-sample terms */
-    for (i = 0; i < nc; ++i) {
+    for (i=0; i<nc; i++) {
 	asum2[i] = 0.0;
-	for (t = t1; t <= t2; t++) {
+	for (t=t1; t<=t2; t++) {
 	    asum2[i] -= res[t] * 2.0 * g[i][t];
 	}
 	asum2[i] /= n; 
     }   
 
     /* pre-sample range */
-    for (t = t1-lag; t < t1; ++t) {
-	for (i = 0; i < nc; ++i) {
+    for (t=t1-lag; t<t1; t++) {
+	for (i=0; i<nc; i++) {
 	    dhdp[i][t] = asum2[i];
 	}
     }
 
     /* actual sample range */
-    for (t = t1; t <= t2; ++t) {
-	for (i = 0; i < nc; i++) {
+    for (t=t1; t<=t2; t++) {
+	for (i=0; i<nc; i++) {
 	    dhdp[i][t] = 0.0;
-	    for (j = 1; j <= q; ++j) {
+	    for (j=1; j<=q; j++) {
 		if (t - q < t1) {
 		    dhdp[i][t] += alpha[j-1] * asum2[i];
 		} else {
 		    dhdp[i][t] -= alpha[j-1] * 2.0 * g[i][t-j] * res[t-j];
 		}
 	    }
-	    for (j = 1; j <= p; ++j) {
+	    for (j=1; j<=p; j++) {
 		dhdp[i][t] += dhdp[i][t-j] * beta[j-1];
 	    }
 	}
     }
 
     /* Initialize gradient and vcv */
-    for (i = 0; i < nparam; i++) {
+    for (i=0; i<nparam; i++) {
 	grad[i] = 0.0;
-	for (j = 0; j < nparam; j++) {
+	for (j=0; j<nparam; j++) {
 	    vcv[vix(i,j)] = 0.0;
 	}
     }
 
-    for (t = t1; t <= t2; ++t) {
+    for (t=t1; t<=t2; t++) {
 	double r_h = res[t] / h[t];
 	double r2_h = res[t] * r_h;
 	double aa, bb;
@@ -1007,7 +971,7 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 	/* 
 	   First part, relative to regression coefficients (eq. 10, p. 402) 
  	*/
-	for (i = 0; i < nc; ++i) {
+	for (i=0; i<nc; i++) {
 	    aa = r_h * g[i][t] + .5 / h[t] * dhdp[i][t] * (r2_h - 1.0);
 	    grad[i] += aa;
 	    if (code == VCV_OP) {
@@ -1028,7 +992,7 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 	/* 
 	   Second part, relative to variance parameters (eq. 6, p. 401) 
 	*/
-	for (i = 0; i < nvparm; ++i) {
+	for (i=0; i<nvparm; i++) {
 	    int nci = nc + i;
 
 	    aa = .5 / h[t] * dhdp[nci][t] * (r2_h - 1.0);
@@ -1045,15 +1009,15 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
     }
 
     if (code == VCV_IM) {
-	for (t = t1; t <= t2; ++t) {
+	for (t=t1; t<=t2; t++) {
 	    double ht2 = h[t] * h[t];
 
 	    /* Part relative to the coefficients (eq. 30, p. 406).
 	       Since we take the expected value, only the first two terms
 	       remain.
 	    */
-	    for (i = 0; i < nc; ++i) {
-		for (j = 0; j < nc; ++j) {
+	    for (i=0; i<nc; i++) {
+		for (j=0; j<nc; j++) {
 		    vcv[vix(i,j)] = vcv[vix(i,j)] 
 			- g[i][t] * g[j][t] / h[t] 
 			- .5 * dhdp[i][t] * dhdp[j][t] / ht2;
@@ -1064,8 +1028,8 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 	       Since we take the expected value, only the second term
 	       remains.
 	    */
-	    for (i = nc; i < nparam; ++i) {
-		for (j = nc; j < nparam; ++j) {
+	    for (i=nc; i<nparam; i++) {
+		for (j=nc; j<nparam; j++) {
 		    vcv[vix(i,j)] -= .5 * dhdp[i][t] * dhdp[j][t] / ht2;
 		}
 	    }
@@ -1079,21 +1043,22 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 
     /* Initial values in dhdpdp (here, "H") are 2/t x'x, and zero for
        off-diagonal (mixed) blocks. */
-    for (k = 0; k < lag; ++k) {
-	for (i = 0; i < nc; ++i) {
-	    for (j = 0; j < nc; ++j) {
+
+    for (k=0; k<lag; k++) {
+	for (i=0; i<nc; i++) {
+	    for (j=0; j<nc; j++) {
 		H[i][j][k+1] = 0.; 
 	    }
 	}
-	for (t = t1; t <= t2; ++t) {
-	    for (i = 0; i < nc; ++i) {
-		for (j = 0; j < nc; ++j) {
+	for (t=t1; t<=t2; t++) {
+	    for (i=0; i<nc; i++) {
+		for (j=0; j<nc; j++) {
 		    H[i][j][k+1] += 2.0 * g[i][t] * g[j][t] / n;
 		}
 	    }
 	}
-	for (i = 0; i < nc; ++i) {
-	    for (j = 0; j < nvparm; ++j) {
+	for (i=0; i<nc; i++) {
+	    for (j=0; j<nvparm; j++) {
 		/* mod. by AC: zero _all_ mixed entries */
 		H[i][nc+j][k+1] = H[nc+j][i][k+1] = 0.0; 
 	    }
@@ -1102,14 +1067,14 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 
     /* Now we fill out the full Hessian */
 
-    for (t = t1; t <= t2; ++t) {
+    for (t=t1; t<=t2; ++t) {
 	double r_h = res[t] / h[t];
 	double r2_h = r_h * res[t];
 	double r2_h3 = r2_h / (h[t] * h[t]);
 	double u_h2 = 1.0 / (h[t] * h[t]);
 
-	for (i = 0; i < nparam; ++i) {
-	    for (j = 0; j < nparam; ++j) {
+	for (i=0; i<nparam; i++) {
+	    for (j=0; j<nparam; j++) {
 		H[i][j][0] = 0.0; 
 	    }
 	}
@@ -1118,9 +1083,9 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 	    goto L90;
 	}
 
-	for (k = 1; k <= q; ++k) {
-	    for (i = 0; i < nc; ++i) {
-		for (j = 0; j < nc; ++j) {
+	for (k=1; k<=q; k++) {
+	    for (i=0; i<nc; i++) {
+		for (j=0; j<nc; j++) {
 		    if (t - q < t1) {
 			H[i][j][0] += H[i][j][q] * alpha[k-1];
 		    } else {
@@ -1131,30 +1096,30 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 	    }
 	}
 
-	for (k = 1; k <= p; ++k) {
-	    for (i = 0; i < nc; ++i) {
-		for (j = 0; j < nc; ++j) {
+	for (k=1; k<=p; k++) {
+	    for (i=0; i<nc; i++) {
+		for (j=0; j<nc; j++) {
 		    H[i][j][0] += H[i][j][k] * beta[k-1];
 		}
 	    }
 	}
 
-	for (i = 0; i < nc; ++i) {
-	    for (k = 1; k <= q; ++k) {
+	for (i=0; i<nc; i++) {
+	    for (k=1; k<=q; k++) {
 		if (t - q < t1) {
 		    H[i][nc+k][0] += asum2[i];
 		} else {
 		    H[i][nc+k][0] -= 2.0 * g[i][t-k] * res[t-k];
 		}
 	    }
-	    for (k = 1; k <= p; ++k) {
+	    for (k=1; k<=p; k++) {
 		H[i][nc+q+k][0] += dhdp[i][t-k];
 	    }
 	}
 
-	for (k = 1; k <= p; ++k) { 
-	    for (i = 0; i < nc; ++i) {
-		for (j = 0; j < nvparm; ++j) {
+	for (k=1; k<=p; k++) { 
+	    for (i=0; i<nc; i++) {
+		for (j=0; j<nvparm; j++) {
 		    H[i][nc+j][0] += H[i][nc+j][k] * beta[k-1];
 		}
 	    }
@@ -1165,8 +1130,9 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 	   Since we take the expected value, only the first two terms
 	   remain.
  	*/
-	for (i = 0; i < nc; ++i) {
-	    for (j = 0; j < nc; ++j) {
+
+	for (i=0; i<nc; i++) {
+	    for (j=0; j<nc; j++) {
 		vcv[vix(i,j)] = vcv[vix(i,j)] 
 		    - g[i][t] * g[j][t] / h[t] 
 		    - .5 * r2_h3 * dhdp[i][t] * dhdp[j][t] 
@@ -1182,20 +1148,21 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 	   Since we take the expected value, only the second term
 	   remains.
  	*/
+
 	if (p > 0) {
-	    for (i = 0; i < nvparm; ++i) {
-		for (j = 1; j <= p; ++j) {
+	    for (i=0; i<nvparm; i++) {
+		for (j=1; j<=p; j++) {
 		    H[nc+i][nc+q+j][0] += dhdp[nc+i][t-j];
 		}
 	    }
-	    for (i = 1; i <= p; ++i) {
-		for (j = 0; j < nvparm; ++j) {
+	    for (i=1; i<=p; i++) {
+		for (j=0; j<nvparm; j++) {
 		    H[nc+q+i][nc+j][0] += dhdp[nc+j][t-i];
 		}
 	    }
-	    for (k = 1; k <= p; ++k) {
-		for (i = 0; i < nvparm; ++i) {
-		    for (j = 0; j < nvparm; ++j) { 
+	    for (k=1; k<=p; k++) {
+		for (i=0; i<nvparm; i++) {
+		    for (j=0; j<nvparm; j++) { 
 			H[nc+i][nc+j][0] += H[nc+i][nc+j][k] * beta[k-1];
 #ifdef FDEBUG
 			if ((t==t1 || t==t2)) {
@@ -1210,8 +1177,8 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 	    }
 	}
 
-	for (i = nc; i < nparam; ++i) {
-	    for (j = nc; j < nparam; ++j) {
+	for (i=nc; i<nparam; i++) {
+	    for (j=nc; j<nparam; j++) {
 		vcv[vix(i,j)] = vcv[vix(i,j)]
 		    + .5 * u_h2 * dhdp[i][t] * dhdp[j][t] 
 		    - r2_h3 * dhdp[i][t] * dhdp[j][t] 
@@ -1226,8 +1193,8 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 	}
 
 	/* top-right mixed part (eq. 17, p. 403) */
-	for (i = 0; i < nc; ++i) {
-	    for (j = 0; j < nvparm; ++j) {
+	for (i=0; i<nc; i++) {
+	    for (j=0; j<nvparm; j++) {
 		vcv[vix(i,nc+j)] = vcv[vix(i,nc+j)]
 		    - g[i][t] * r_h * dhdp[nc+j][t] / h[t] 
 		    - .5 * (r2_h - 1.0) * dhdp[nc+j][t] * dhdp[i][t] / (h[t] * h[t]) 
@@ -1239,11 +1206,11 @@ static int vcv_setup (int t1, int t2, double *c, int nc,
 	}
 
 	/* before quitting time t, tidy up dhdpdp */
-	for (k = 0; k < lag; ++k) { 
-	    for (i = 0; i < nparam; ++i) {
-		for (j = 0; j < nparam; ++j) {
+	for (k=0; k<lag; k++) { 
+	    for (i=0; i<nparam; i++) {
+		for (j=0; j<nparam; j++) {
 #ifdef FDEBUG
-		    if (t<5) {
+		    if (t < 5) {
 			fprintf(stderr, "t=%d: setting H(%d,%d,%d) to H(%d,%d,%d)=%g\n",
 				t,i,j,lag-k,i,j,lag-k-1, H[i][j][lag-k-1]);
 		    }
@@ -1677,11 +1644,20 @@ static double ***allocate_dhdpdp (int np, int p, int q)
     int i, j, lag = (p > q)? p : q;
 
     H = malloc(np * sizeof *H);
+    if (H == NULL) {
+	return NULL;
+    }
     
     for (i=0; i<np; i++) {
 	H[i] = malloc(np * sizeof **H);
+	if (H[i] == NULL) {
+	    return NULL;
+	}
 	for (j=0; j<np; j++) {
 	    H[i][j] = malloc((lag + 1) * sizeof ***H);
+	    if (H[i][j] == NULL) {
+		return NULL;
+	    }
 	}
     }
 
