@@ -210,7 +210,7 @@ over_identification_test (fiml_system *fsys, double ***pZ, DATAINFO *pdinfo)
 
     err = gretl_matrix_multiply_mod(uru, GRETL_MOD_TRANSPOSE,
 				    uru, GRETL_MOD_NONE,
-				    urv);
+				    urv, GRETL_MOD_NONE);
 
     if (err) {
 	goto bailout;
@@ -285,7 +285,7 @@ fiml_form_sigma_and_psi (fiml_system *fsys, const double **Z, int t1)
 
     err = gretl_matrix_multiply_mod(fsys->uhat, GRETL_MOD_TRANSPOSE,
 				    fsys->uhat, GRETL_MOD_NONE,
-				    fsys->sigma);
+				    fsys->sigma, GRETL_MOD_NONE);
 
     gretl_matrix_divide_by_scalar(fsys->sigma, fsys->n);
 
@@ -842,7 +842,7 @@ static int fiml_get_std_errs (fiml_system *fsys, const gretl_matrix *R)
     
     if (R != NULL) {
 	err = gretl_matrix_restricted_ols(fsys->arty, fsys->artx, R, NULL,
-					  fsys->artb, vcv, NULL);
+					  fsys->artb, vcv, NULL, NULL);
     } else {
 	err = gretl_matrix_svd_ols(fsys->arty, fsys->artx, fsys->artb, 
 				   vcv, NULL, NULL);
@@ -953,7 +953,7 @@ int fiml_driver (gretl_equation_system *sys, double ***pZ,
 	/* run artificial regression (ETM, equation 12.86) */
 	if (R != NULL) {
 	    err = gretl_matrix_restricted_ols(fsys->arty, fsys->artx, R, NULL,
-					      fsys->artb, NULL, NULL);
+					      fsys->artb, NULL, NULL, NULL);
 	} else {
 #if 0
 	    err = gretl_matrix_svd_ols(fsys->arty, fsys->artx, fsys->artb, 
