@@ -26,6 +26,7 @@
 #include "var.h"
 
 #include "fcp.h"
+#include "mod_garch.h"
 
 #undef VPARM_DEBUG
 
@@ -40,35 +41,19 @@ static void add_garch_varnames (MODEL *pmod, const DATAINFO *pdinfo,
     int p = list[1];
     int q = list[2];
     int r = list[0] - 4;
-    int i, j, np = 3 + p + q + r;
+    int i, j, np = 2 + p + q + r;
 
     free(pmod->list);
     pmod->list = gretl_list_copy(list);
 
-    pmod->params = malloc(np * sizeof pmod->params);
-    if (pmod->params == NULL) {
-	pmod->errcode = E_ALLOC;
+    gretl_model_allocate_params(pmod, np);
+    if (pmod->errcode) {
 	return;
     }
 
-    pmod->nparams = np;
+    strcpy(pmod->params[0], pdinfo->varname[0]);
 
-    for (i=0; i<np; i++) {
-	pmod->params[i] = malloc(VNAMELEN);
-	if (pmod->params[i] == NULL) {
-	    for (j=0; j<i; j++) free(pmod->params[j]);
-	    free(pmod->params);
-	    pmod->params = NULL;
-	    pmod->nparams = 0;
-	    pmod->errcode = E_ALLOC;
-	    return;
-	}
-    }
-
-    strcpy(pmod->params[0], pdinfo->varname[pmod->list[4]]);
-    strcpy(pmod->params[1], pdinfo->varname[0]);
-
-    j = 2;
+    j = 1;
     for (i=0; i<r; i++) {
 	if (pmod->list[5+i] > 0) {
 	    strcpy(pmod->params[j++], pdinfo->varname[pmod->list[5+i]]);
@@ -80,6 +65,7 @@ static void add_garch_varnames (MODEL *pmod, const DATAINFO *pdinfo,
     for (i=0; i<q; i++) {
 	sprintf(pmod->params[j++], "alpha(%d)", i + 1);
     }
+
     for (i=0; i<p; i++) {
 	sprintf(pmod->params[j++], "beta(%d)", i + 1);
     }
@@ -198,8 +184,8 @@ static int make_garch_dataset (const int *list, double **Z,
 			       double **py, double ***pX)
 {
     double *y = NULL, **X = NULL;
-    int i, k = 0, t;
-    int xnum, ynum = list[4];
+    int vx, vy = list[4];
+    int i, k, s, t;
 
     /* If pad > 0 we have to create a newly allocated, padded
        dataset.  Otherwise we can use a virtual dataset, made
@@ -208,25 +194,22 @@ static int make_garch_dataset (const int *list, double **Z,
 
     if (pad > 0) {
 	y = malloc(bign * sizeof *y);
-	if (y == NULL) return 1;
+	if (y == NULL) {
+	    return E_ALLOC;
+	}
+	*py = y;
     } 
 
     if (nx > 0) {
-	X = malloc(nx * sizeof *X);
-	if (X == NULL) goto bailout;
-
-	if (pad > 0) {
-	    for (i=0; i<nx; i++) {
-		X[i] = malloc(bign * sizeof **X);
-		if (X[i] == NULL) {
-		    for (t=0; t<i; t++) {
-			free(X[t]);
-		    }
-		    free(X);
-		    goto bailout;
-		}
-	    } 
-	}  
+	if (pad) {
+	    X = doubles_array_new(nx, bign);
+	} else {
+	    X = malloc(nx * sizeof *X);
+	}
+	if (X == NULL) {
+	    free(y);
+	    return E_ALLOC;
+	}
     }
 
     if (pad > 0) {
@@ -238,35 +221,34 @@ static int make_garch_dataset (const int *list, double **Z,
 		    X[i][t] = 0.0;
 		}
 	    } else {
-		y[t] = Z[ynum][t-pad];
-		if (nx > 0) k = 5;
+		s = t - pad;
+		y[t] = Z[vy][s];
+		k = 5;
 		for (i=0; i<nx; i++) {
-		    xnum = list[k++]; 
-		    if (xnum == 0) xnum = list[k++];
-		    X[i][t] = Z[xnum][t-pad];
+		    vx = list[k++]; 
+		    if (vx == 0) {
+			vx = list[k++];
+		    }
+		    X[i][t] = Z[vx][s];
 		}
 	    }
 	}
-	*py = y;
     } else {
 	/* build virtual dataset */
-	*py = Z[ynum];
-	if (nx > 0) k = 5;
+	*py = Z[vy];
+	k = 5;
 	for (i=0; i<nx; i++) {
-	    xnum = list[k++]; 
-	    if (xnum == 0) xnum = list[k++];
-	    X[i] = Z[xnum];
+	    vx = list[k++]; 
+	    if (vx == 0) {
+		vx = list[k++];
+	    }
+	    X[i] = Z[vx];
 	}
     }
 
     *pX = X;
 
     return 0;
-
- bailout:
-
-    free(y);
-    return E_ALLOC;
 }
 
 static int get_vopt (int robust)
@@ -291,6 +273,77 @@ static int get_vopt (int robust)
     return vopt;
 }
 
+static void garch_print_init (const double *coeff, int k,
+			      const double *a, int p, int q,
+			      int manual, PRN *prn)
+{
+    int i;
+
+    pputc(prn, '\n');
+
+    if (manual) {
+	pputs(prn, "Manual initialization of parameters");
+    } else {
+	pputs(prn, "Automatic initialization of parameters");
+    }
+
+    pputs(prn, "\n\n Regression coefficients:\n");
+
+    for (i=0; i<k; i++) {
+	pprintf(prn, "  theta[%d] = %g\n", i, coeff[i]);
+    }
+
+    pputs(prn, "\n Variance parameters:\n");
+
+    pprintf(prn, "  alpha[0] = %g\n", a[0]);
+    for (i=0; i<p; i++) {
+	pprintf(prn, "  alpha[%d] = %g\n", i+1, a[i+3]);
+    }
+    for (i=0; i<q; i++) {
+	pprintf(prn, "   beta[%d] = %g\n", i, a[i+3+p]);
+    }
+
+    pputc(prn, '\n');
+}
+
+/* pick up any manually set initial values (if these
+   have been set via "set initvals") */
+
+static int garch_manual_init (double *a, int p, int q, 
+			      double *coeff, double *b,
+			      int k, PRN *prn)
+{
+    const gretl_matrix *m = get_init_vals();
+    int mlen = gretl_vector_get_length(m);
+    int n = p + q + 1 + k;
+    int i, j;
+
+    if (mlen != n) {
+	if (mlen > 0) {
+	    fprintf(stderr, "Number of initvals = %d, but we want %d "
+		    "values for GARCH\n", mlen, n);
+	}
+	return 0;
+    }
+
+    /* coefficients on regressors */
+    for (i=0; i<k; i++) {
+	coeff[i] = m->val[i];
+	b[i] = 0.0;
+    }
+
+
+    /* variance parameters */
+    a[0] = m->val[i++];
+    for (j=0; j<p+q; j++) {
+	a[j+3] = m->val[i++];
+    }
+
+    garch_print_init(coeff, k, a, p, q, 1, prn);
+
+    return 1;
+}
+
 int do_fcp (const int *list, double **Z, double scale,
 	    const DATAINFO *pdinfo, MODEL *pmod,
 	    PRN *prn, gretlopt opt)
@@ -306,9 +359,10 @@ int do_fcp (const int *list, double **Z, double scale,
     double *res = NULL, *res2 = NULL;
     double *coeff = NULL, *b = NULL;
     double *vcv = NULL;
-    int err = 0, iters = 0;
+    int fnc = 0, grc = 0, iters = 0;
     int nobs, maxlag, bign, pad = 0;
     int i, nx, nparam, vopt;
+    int err = 0;
 
     vopt = get_vopt(opt & OPT_R);
 
@@ -335,6 +389,7 @@ int do_fcp (const int *list, double **Z, double scale,
 	err = E_ALLOC;
 	goto bailout;
     }
+
     for (i=0; i<bign; i++) {
 	res2[i] = res[i] = amax[i] = 0.0;
     }   
@@ -351,6 +406,7 @@ int do_fcp (const int *list, double **Z, double scale,
 	err = E_ALLOC;
 	goto bailout;
     }
+
     for (i=0; i<nparam * nparam; i++) {
 	vcv[i] = 0.0;
     } 
@@ -361,26 +417,47 @@ int do_fcp (const int *list, double **Z, double scale,
 	goto bailout;
     }
 
-    /* initial coefficients from OLS */
-    for (i=0; i<ncoeff; i++) {
-	coeff[i] = pmod->coeff[i];
-	b[i] = 0.0;
-    }
-
-    /* for compatibility with FCP... */
+    /* a bit odd, but for compatibility with FCP... */
     amax[1] = q;
     amax[2] = p; 
 
-    /* initialize variance parameters */
-    amax[0] = vparm_init[0];
-    for (i=0; i<p+q; i++) {
-	amax[i+3] = vparm_init[i+1];
+    if (!garch_manual_init(amax, p, q, coeff, b, ncoeff, prn)) {
+
+	/* initial coefficients from OLS */
+	for (i=0; i<ncoeff; i++) {
+	    coeff[i] = pmod->coeff[i];
+	    b[i] = 0.0;
+	}
+
+	/* initialize variance parameters */
+	amax[0] = vparm_init[0];
+	for (i=0; i<p+q; i++) {
+	    amax[i+3] = vparm_init[i+1];
+	}
+
+	if (opt & OPT_V) {
+	    garch_print_init(coeff, ncoeff, amax, p, q, 0, prn);
+	}
     }
 
+#ifdef USE_FCP
     err = garch_estimate(t1 + pad, t2 + pad, bign, 
 			 (const double **) X, nx, coeff, ncoeff, 
 			 vcv, res2, res, h, y, amax, b, scale, &iters,
 			 prn, vopt);
+#else
+    if (getenv("FCP_GARCH") != NULL) {
+	err = garch_estimate(t1 + pad, t2 + pad, bign, 
+			     (const double **) X, nx, coeff, ncoeff, 
+			     vcv, res2, res, h, y, amax, b, scale, &iters,
+			     prn, vopt);
+    } else {
+	err = garch_estimate_mod(t1 + pad, t2 + pad, bign, 
+				 (const double **) X, nx, coeff, ncoeff, 
+				 vcv, res2, res, h, y, amax, b, scale, &fnc,
+				 &grc, prn, vopt);
+    }
+#endif
 
     if (err != 0) {
 	pmod->errcode = err;
@@ -404,7 +481,12 @@ int do_fcp (const int *list, double **Z, double scale,
 	write_garch_stats(pmod, (const double **) Z, scale, pdinfo, 
 			  list, amax, nparam, pad, res, h);
 	make_packed_vcv(pmod, vcv, nparam, ncoeff, scale);
-	gretl_model_set_int(pmod, "iters", iters);
+	if (iters > 0) {
+	    gretl_model_set_int(pmod, "iters", iters);
+	} else if (fnc > 0) {
+	    gretl_model_set_int(pmod, "fncount", fnc);
+	    gretl_model_set_int(pmod, "grcount", grc);
+	}
 	gretl_model_set_int(pmod, "ml_vcv", vopt);
     }
 
@@ -419,18 +501,9 @@ int do_fcp (const int *list, double **Z, double scale,
     free(vcv); 
 
     if (pad > 0) {
-	/* don't free y if it's just a pointer into Z */
 	free(y);
-    }
-    
-    if (X != NULL) {
-	if (pad > 0) {
-	    /* don't free the X[i] if they're just pointers into
-	       the original data matrix */
-	    for (i=0; i<nx; i++) {
-		free(X[i]);
-	    }
-	}
+	doubles_array_free(X, nx);
+    } else {
 	free(X);
     }
 
@@ -552,7 +625,7 @@ garch_init_by_arma (const MODEL *pmod, const int *garchlist,
     /* dep var is squared OLS residual */
     list[4] = v;
 
-    amod = arma(list, (const double **) *pZ, pdinfo, OPT_NONE, NULL);
+    amod = arma(list, (const double **) *pZ, pdinfo, OPT_C, NULL);
     if (amod.errcode) {
 	err = amod.errcode;
 	goto bailout;
@@ -587,7 +660,7 @@ static int *get_garch_list (const int *list, const double **Z,
 
     *err = 0;
 
-    /* rule out pure AR in variance */
+    /* rule out pure AR in variance (FIXME, why?) */
     if (p > 0 && q == 0) {
 	gretl_errmsg_set(_("Error in garch command"));
 	*err = E_DATA;
@@ -595,7 +668,7 @@ static int *get_garch_list (const int *list, const double **Z,
     }
 
     /* rule out excessive total GARCH terms */
-    else if (p + q > 5) {
+    if (p + q > 5) {
 	gretl_errmsg_set(_("Error in garch command"));
 	*err = E_DATA;
 	return NULL;
@@ -624,7 +697,7 @@ static int *get_garch_list (const int *list, const double **Z,
     return glist;
 }
 
-/* make regresson list for initial OLS */
+/* make regression list for initial OLS */
 
 static int *make_ols_list (const int *list)
 {
@@ -699,52 +772,51 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 			  pdinfo, &err);
     if (err) {
 	model.errcode = err;
+	return model;
     }
 
-    if (!err) {
-	ols_list = make_ols_list(list);
-	if (ols_list == NULL) {
-	    err = model.errcode = E_ALLOC;
-	}
+    ols_list = make_ols_list(list);
+    if (ols_list == NULL) {
+	model.errcode = E_ALLOC;
+	return model;
     }
 
     /* run initial OLS */
-    if (!err) {
-	model = lsq(ols_list, pZ, pdinfo, OLS, OPT_A | OPT_M);
-	if (model.errcode) {
-	    err = model.errcode;
-	}
+    model = lsq(ols_list, pZ, pdinfo, OLS, OPT_A | OPT_M);
+    if (model.errcode) {
+	goto bailout;
     }
 
+#if 0
+    printmodel(&model, pdinfo, OPT_NONE, prn);
+#endif
+
 #if GARCH_AUTOCORR_TEST
     /* pretest the residuals for autocorrelation */
-    if (!err && prn != NULL) {
+    if (prn != NULL) {
 	garch_pretest(&model, pZ, pdinfo, &LMF, &pvF);
     }
 #endif
 
 #if GARCH_SCALE_SIGMA
-    if (!err) {
-	yno = ols_list[1];
-	scale = model.sigma;
-	for (t=0; t<pdinfo->n; t++) {
-	    if (!na((*pZ)[yno][t])) {
-		(*pZ)[yno][t] /= scale;
-	    }
+    yno = ols_list[1];
+    scale = model.sigma;
+    for (t=0; t<pdinfo->n; t++) {
+	if (!na((*pZ)[yno][t])) {
+	    (*pZ)[yno][t] /= scale;
 	}
-	for (t=0; t<model.ncoeff; t++) {
-	    model.coeff[t] *= scale;
-	}
-	model.ess /= scale * scale;
-	model.sigma = 1.0;
-    } 
+    }
+    for (t=0; t<model.ncoeff; t++) {
+	model.coeff[t] /= scale;
+    }
+    model.ess /= scale * scale;
+    model.sigma = 1.0;
 #endif 
 
     /* default variance parameter initialization */
-    vparm_init[0] = model.sigma * model.sigma;
-    for (t=1; t<VPARM_MAX; t++) {
-	vparm_init[t] = 0.1;
-    }
+    vparm_init[1] = 0.2;
+    vparm_init[list[2]+1] = 0.7;
+    vparm_init[0] = model.sigma * model.sigma * 0.1;
 
     if (opt & OPT_A) {
 	/* "--arma-init": try initializing params via ARMA */
@@ -752,9 +824,7 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 				      pZ, pdinfo);
     }
 
-    if (!err) {
-	do_fcp(list, *pZ, scale, pdinfo, &model, prn, opt); 
-    }
+    do_fcp(list, *pZ, scale, pdinfo, &model, prn, opt); 
 
     if (scale != 1.0) {
 	/* undo scaling of dependent variable */
@@ -765,6 +835,8 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
+ bailout:
+
     free(ols_list);
     free(list);
 
