@@ -34,7 +34,7 @@ static void set_all_missing (double **Z, DATAINFO *pdinfo)
 
 static void invalid_varname (PRN *prn)
 {
-    pputs(prn, get_gretl_errmsg());
+    pputs(prn, gretl_errmsg_get());
     pputs(prn, _("\nPlease rename this variable and try again"));
 }
 
@@ -335,7 +335,8 @@ static int obs_column_heading (const char *label)
 	    lower(test);
 	    if (strncmp(test, "obs", 3) == 0 ||
 		strcmp(test, "date") == 0 ||
-		strcmp(test, "year") == 0) {
+		strcmp(test, "year") == 0 ||
+		strcmp(test, "yr") == 0) {
 		ret = 1;
 	    }
 	    g_free(test);
