@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) by Allin Cottrell
+ *  Copyright (c) by Allin Cottrell and Riccardo "Jack" Lucchetti
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,15 +17,13 @@
  *
  */
 
-/* A collaborative effort between Allin Cottrell and Riccardo "Jack"
-   Lucchetti */
-
 #include "libgretl.h"
 #include "pvalues.h"
 #include "gretl_matrix.h"
 #include "var.h"
 
 #define JDEBUG 0
+#define OLDEIG 0
 
 /* 
    Critical values for Johansen's likelihood ratio tests
@@ -159,55 +157,18 @@ gamma_par_asymp (double tracetest, double lmaxtest, JohansenCode det,
     return 0;
 }
 
-/* normalize the eigenvectors, based on the Svv matrix */
+/* Remove a possible excess zero from the end of a floating point
+   number printed to the given precision p: are we working around
+   a bug in the C library?
+*/
 
-static int johansen_normalize (JohansenInfo *jv, gretl_matrix *evecs)
+static void fix_xstr (char *s, int p)
 {
-    gretl_matrix *a = NULL, *b = NULL;
-    double x, den;
-    int nv = gretl_matrix_rows(jv->Svv);
-    int i, j, h;
-    int err = 0;
+    int n = strlen(s);
 
-    if (jv->rank > 0) {
-	h = jv->rank;
-    } else {
-	h = nv;
+    if (n > p && strspn(s + n - p, "0") == p) {
+	s[n-1] = 0;
     }
-
-    a = gretl_column_vector_alloc(nv);
-    b = gretl_column_vector_alloc(nv);
-
-    if (a == NULL || b == NULL) {
-	gretl_matrix_free(a);
-	gretl_matrix_free(b);
-	return E_ALLOC;
-    }
-
-    for (j=0; j<h; j++) {
-	/* select column from evecs */
-	for (i=0; i<nv; i++) {
-	    x = gretl_matrix_get(evecs, i, j);
-	    gretl_vector_set(a, i, x);
-	}
-
-	/* find value of appropriate denominator */
-	gretl_matrix_multiply(jv->Svv, a, b);
-	den = gretl_vector_dot_product(a, b, &err);
-
-	if (!err) {
-	    den = sqrt(den);
-	    for (i=0; i<nv; i++) {
-		x = gretl_matrix_get(evecs, i, j);
-		gretl_matrix_set(evecs, i, j, x / den);
-	    }
-	} 
-    }
-
-    gretl_matrix_free(a);
-    gretl_matrix_free(b);
-
-    return err;
 }
 
 enum {
@@ -224,8 +185,9 @@ static void print_beta_or_alpha (JohansenInfo *jv, int k,
 {
     gretl_matrix *c = (which == PRINT_BETA)? jv->Beta : jv->Alpha;
     int rows = gretl_matrix_rows(c);
+    char xstr[32];
     int i, j;
-    double x;
+    double x, y;
 
     if (rescale) {
 	pprintf(prn, "\n%s\n", (which == PRINT_BETA)? 
@@ -246,16 +208,21 @@ static void print_beta_or_alpha (JohansenInfo *jv, int k,
 	    pprintf(prn, "%-10s", "trend");
 	}
 	for (j=0; j<k; j++) {
+	    x = gretl_matrix_get(c, i, j);
 	    if (rescale) {
-		x = gretl_matrix_get(jv->Beta, j, j);
+		y = gretl_matrix_get(jv->Beta, j, j);
 		if (which == PRINT_BETA) {
-		    pprintf(prn, "%#12.5g ", gretl_matrix_get(c, i, j) / x);
+		    x /= y;
 		} else {
-		    pprintf(prn, "%#12.5g ", gretl_matrix_get(c, i, j) * x);
+		    x *= y;
 		}
-	    } else {
-		pprintf(prn, "%#12.5g ", gretl_matrix_get(c, i, j));
 	    }
+	    if (x == -0.0) {
+		x = 0.0;
+	    }
+	    sprintf(xstr, "%#.5g", x);
+	    fix_xstr(xstr, 5);
+	    pprintf(prn, "%12s ", xstr);
 	}
 	pputc(prn, '\n');
     }
@@ -290,11 +257,9 @@ static int compute_alpha (JohansenInfo *jv, int n)
     } 
 
     if (!err) {
-	gretl_matrix_multiply(jv->Svv, jv->Beta, tmp1);
-	gretl_matrix_multiply_mod(jv->Beta, GRETL_MOD_TRANSPOSE,
-				  tmp1, GRETL_MOD_NONE,
-				  tmp2);
-	err = gretl_invert_general_matrix(tmp2);
+	gretl_matrix_qform(jv->Beta, GRETL_MOD_TRANSPOSE, jv->Svv,
+			   tmp2, GRETL_MOD_NONE);
+	err = gretl_invert_symmetric_matrix(tmp2);
     }
 
     if (!err) {
@@ -366,7 +331,7 @@ static int compute_omega (GRETL_VAR *vecm)
 
     gretl_matrix_multiply_mod(vecm->E, GRETL_MOD_TRANSPOSE,
 			      vecm->E, GRETL_MOD_NONE,
-			      vecm->S);
+			      vecm->S, GRETL_MOD_NONE);
 
     gretl_matrix_divide_by_scalar(vecm->S, vecm->T);
     
@@ -521,7 +486,7 @@ static int form_Pi (GRETL_VAR *vecm, const gretl_matrix *Alpha,
     if (!err) {
 	gretl_matrix_multiply_mod(Alpha, GRETL_MOD_NONE,
 				  Beta, GRETL_MOD_TRANSPOSE,
-				  Pi);
+				  Pi, GRETL_MOD_NONE);
     }
 
     if (freeit) {
@@ -729,7 +694,7 @@ compute_long_run_matrix (JohansenInfo *jv, int n, const DATAINFO *pdinfo, PRN *p
     if (!err) {
 	gretl_matrix_multiply_mod(jv->Beta, GRETL_MOD_NONE,
 				  jv->Beta, GRETL_MOD_TRANSPOSE,
-				  tmp);
+				  tmp, GRETL_MOD_NONE);
 
 	/* \Zeta_0 = \Sigma_{uv} A A' */
 	gretl_matrix_multiply(jv->Suv, tmp, Z0);
@@ -748,14 +713,14 @@ compute_long_run_matrix (JohansenInfo *jv, int n, const DATAINFO *pdinfo, PRN *p
 */
 
 static int
-print_beta_and_alpha (JohansenInfo *jv, double *evals, int h,
+print_beta_and_alpha (JohansenInfo *jv, gretl_matrix *evals, int h,
 		      const DATAINFO *pdinfo, PRN *prn)
 {
     int i, err = 0;
 
     pprintf(prn, "\n%s", _("eigenvalue"));
     for (i=0; i<h; i++) {
-	pprintf(prn, "%#12.5g ", evals[i]);
+	pprintf(prn, "%#12.5g ", evals->val[i]);
     }
     pputc(prn, '\n');
 
@@ -844,31 +809,30 @@ static int beta_variance (GRETL_VAR *vecm)
     gretl_matrix *HSH = NULL;
 
     int r = jrank(vecm);
+    int m = gretl_matrix_cols(vecm->jinfo->Alpha);
     int n = gretl_matrix_rows(vecm->jinfo->Beta);
     int i, j, k, err = 0;
 
     double x;
 
-    /* initial allocations */
     O = gretl_matrix_copy(vecm->S);
+    aOa = gretl_matrix_alloc(m, m);
     HSH = gretl_matrix_alloc(n - r, n - r);
 
-    if (O == NULL || HSH == NULL) {
+    if (O == NULL || aOa == NULL || HSH == NULL) {
 	err = E_ALLOC;
 	goto bailout;
     }
 
     /* compute \alpha' \Omega^{-1} \alpha */
+
     err = gretl_invert_symmetric_matrix(O);
     if (err) {
 	goto bailout;
     }
 
-    aOa = gretl_matrix_A_X_A(vecm->jinfo->Alpha, GRETL_MOD_TRANSPOSE, O, &err);
-    if (aOa == NULL) {
-	err = E_ALLOC;
-	goto bailout;
-    }    
+    gretl_matrix_qform(vecm->jinfo->Alpha, GRETL_MOD_TRANSPOSE, O,
+		       aOa, GRETL_MOD_NONE);
 
 #if JDEBUG
     gretl_matrix_print(vecm->S, "vecm->S");
@@ -933,7 +897,7 @@ static int beta_variance (GRETL_VAR *vecm)
     return err;
 }
 
-static int johansen_ll_calc (GRETL_VAR *jvar, const double *eigvals)
+static int johansen_ll_calc (GRETL_VAR *jvar, const gretl_matrix *eigvals)
 {
     gretl_matrix *Suu;
     double ldet, T_2 = (double) jvar->T / 2.0;
@@ -950,7 +914,7 @@ static int johansen_ll_calc (GRETL_VAR *jvar, const double *eigvals)
 	ldet = gretl_matrix_log_determinant(Suu, &err);
 	jvar->ll = - T_2 * n * (1.0 + LN_2_PI) - T_2 * ldet;
 	for (i=0; i<h; i++) {
-	    jvar->ll -= T_2 * log(1.0 - eigvals[i]); 
+	    jvar->ll -= T_2 * log(1.0 - eigvals->val[i]); 
 	}
 	gretl_matrix_free(Suu);
     }
@@ -984,28 +948,28 @@ static int vecm_ll_stats (GRETL_VAR *vecm)
 }
 
 static int 
-compute_coint_test (GRETL_VAR *jvar, const double *eigvals, PRN *prn)
+compute_coint_test (GRETL_VAR *jvar, const gretl_matrix *evals, PRN *prn)
 {
     int T = jvar->T;
     int n = jvar->neqns;
     double cumeig = 0.0;
-    double *lambdamax = NULL;
+    double *lmax = NULL;
     double *trace = NULL;
     double pvals[2];
     int i;
 
     trace = malloc(n * sizeof *trace);
-    lambdamax = malloc(n * sizeof *lambdamax);
+    lmax = malloc(n * sizeof *lmax);
 
-    if (trace == NULL || lambdamax == NULL) {
+    if (trace == NULL || lmax == NULL) {
 	free(trace);
-	free(lambdamax);
+	free(lmax);
 	return E_ALLOC;
     }
 
     for (i=n-1; i>=0; i--){
-	lambdamax[i] = -T * log(1.0 - eigvals[i]); 
-	cumeig += lambdamax[i];
+	lmax[i] = -T * log(1.0 - evals->val[i]); 
+	cumeig += lmax[i];
 	trace[i] = cumeig; 
     }
 
@@ -1016,28 +980,62 @@ compute_coint_test (GRETL_VAR *jvar, const double *eigvals, PRN *prn)
 	    _("Lmax test"), _("p-value"));	
 
     for (i=0; i<n; i++) {
-	gamma_par_asymp(trace[i], lambdamax[i], jcode(jvar), n - i, pvals);
+	gamma_par_asymp(trace[i], lmax[i], jcode(jvar), n - i, pvals);
 	pprintf(prn, "%4d%#11.5g%#11.5g [%6.4f]%#11.5g [%6.4f]\n", 
-		i, eigvals[i], trace[i], pvals[0], lambdamax[i], pvals[1]);
+		i, evals->val[i], trace[i], pvals[0], lmax[i], pvals[1]);
     }
     pputc(prn, '\n');
 
-    free(lambdamax);
+    free(lmax);
     free(trace);
 
     return 0;
 }
 
+static int johansen_get_eigenvalues (gretl_matrix *Suu,
+				     const gretl_matrix *Suv,
+				     const gretl_matrix *Svv,
+				     gretl_matrix *M,
+				     gretl_matrix **evals,
+				     int rank)
+{
+    gretl_matrix *Tmp = NULL;
+    int n = Svv->cols;
+    int err;
+
+    err = gretl_invert_symmetric_matrix(Suu);
+    if (err) {
+	return err;
+    }
+
+    Tmp = gretl_matrix_alloc(n, n);
+    if (Tmp == NULL) {
+	return E_ALLOC;
+    }
+
+    gretl_matrix_qform(Suv, GRETL_MOD_TRANSPOSE, 
+		       Suu, Tmp, GRETL_MOD_NONE);
+
+    *evals = gretl_gensymm_eigenvals(Tmp, Svv, M, &err);
+
+    if (!err) {
+	err = gretl_symmetric_eigen_sort(*evals, M, rank);
+    }
+
+    gretl_matrix_free(Tmp);
+
+    return err;
+}
+
 /* Public entry point for both cointegration test and VECM
    estimation */
 
-int johansen_analysis (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo, PRN *prn)
+int johansen_analysis (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo, 
+		       gretlopt opt, PRN *prn)
 {
     gretl_matrix *M = NULL;
-    gretl_matrix *TmpL = NULL, *TmpR = NULL;
-    gretl_matrix *Suu = NULL, *Svv = NULL;
-
-    double *eigvals = NULL;
+    gretl_matrix *Suu = NULL;
+    gretl_matrix *evals = NULL;
 
     int n = jvar->neqns;
     int nv = gretl_matrix_cols(jvar->jinfo->Svv);
@@ -1048,80 +1046,41 @@ int johansen_analysis (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo, PRN *prn
     fprintf(stderr, "\n*** starting johansen_analysis()\n\n");
 #endif
 
-    TmpL = gretl_matrix_alloc(nv, n);
-    TmpR = gretl_matrix_alloc(nv, nv);
     M = gretl_matrix_alloc(nv, nv);
 
-    /* We want to preserve Svv and Suu, so copy them before
-       inverting */
-    Svv = gretl_matrix_copy(jvar->jinfo->Svv);
+    /* We want to preserve Suu, so copy before processing */
     Suu = gretl_matrix_copy(jvar->jinfo->Suu);
 
-    if (Suu == NULL || Svv == NULL || TmpL == NULL || 
-	TmpR == NULL || M == NULL) {
-	err = 1;
-	goto eigenvals_bailout;
+    if (Suu == NULL || M == NULL) {
+	err = E_ALLOC;
+	goto bailout;
     }
 
-    if (nv > n) {
-	gretl_matrix_reuse(TmpR, n, nv);
-    }
-
-    /* calculate Suu^{-1} Suv */
-    err = gretl_invert_general_matrix(Suu);
-    if (!err) {
-	err = gretl_matrix_multiply(Suu, jvar->jinfo->Suv, TmpR);
-    }
-
-    /* calculate Svv^{-1} Suv' */
-    if (!err) {
-	err = gretl_invert_general_matrix(Svv);
-    }
-    if (!err) {
-	err = gretl_matrix_multiply_mod(Svv, GRETL_MOD_NONE,
-					jvar->jinfo->Suv, GRETL_MOD_TRANSPOSE, 
-					TmpL);
-    }
-
-    if (!err) {
-	err = gretl_matrix_multiply(TmpL, TmpR, M);
-    }
-
-    if (err) {
-	goto eigenvals_bailout;
-    }
-
-    eigvals = gretl_general_matrix_eigenvals(M, 1, &err);
+    err = johansen_get_eigenvalues(Suu, jvar->jinfo->Suv, jvar->jinfo->Svv,
+				   M, &evals, rank);
     if (err) {
 	pputs(prn, _("Failed to find eigenvalues\n"));
-    } else {
-	err = gretl_eigen_sort(eigvals, M, rank);
-    }
+	goto bailout;
+    } 
 
 #if JDEBUG
     gretl_matrix_print(M, "raw eigenvector(s)");
 #endif
 
     if (!err) {
-	johansen_ll_calc(jvar, eigvals);
-
-	if (rank == 0) {
-	    compute_coint_test(jvar, eigvals, prn);
-	} 
-
-	/* normalize the eigenvectors */
-	johansen_normalize(jvar->jinfo, M);
-#if JDEBUG
-	gretl_matrix_print(M, "normalized tmpR");
-#endif
+	johansen_ll_calc(jvar, evals);
 
 	if (rank == 0) {
+	    compute_coint_test(jvar, evals, prn);
 	    /* just running cointegration test */
+	    if (opt & OPT_Q) {
+		goto bailout;
+	    }
 	    jvar->jinfo->Beta = M;
 	    M = NULL;
 	    err = compute_alpha(jvar->jinfo, n);
 	    if (!err) {
-		print_beta_and_alpha(jvar->jinfo, eigvals, n, pdinfo, prn);
+		print_beta_and_alpha(jvar->jinfo, evals, n, pdinfo, prn);
 		compute_long_run_matrix(jvar->jinfo, n, pdinfo, prn);
 	    }
 	} else {
@@ -1153,15 +1112,11 @@ int johansen_analysis (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo, PRN *prn
 	}
     } 
 
- eigenvals_bailout:    
+ bailout:    
 
-    gretl_matrix_free(TmpL);
-    gretl_matrix_free(TmpR);
     gretl_matrix_free(M);
-    gretl_matrix_free(Svv);
     gretl_matrix_free(Suu);
-
-    free(eigvals);
+    gretl_matrix_free(evals);
 
     return err;
 }
@@ -1177,13 +1132,7 @@ johansen_bootstrap_round (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
 			  int iter)
 {
     gretl_matrix *M = NULL;
-    gretl_matrix *TmpL = NULL;
-    gretl_matrix *TmpR = NULL;
-    gretl_matrix *Svv = NULL;
-
-    double *eigvals = NULL;
-
-    int n = jvar->neqns;
+    gretl_matrix *evals = NULL;
     int nv = gretl_matrix_cols(jvar->jinfo->Svv);
     int err = 0;
 
@@ -1191,49 +1140,17 @@ johansen_bootstrap_round (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
     fprintf(stderr, "\n*** starting johansen_bootstrap_round()\n\n");
 #endif
 
-    TmpL = gretl_matrix_alloc(nv, n);
-    TmpR = gretl_matrix_alloc(nv, nv);
     M = gretl_matrix_alloc(nv, nv);
-    Svv = gretl_matrix_copy(jvar->jinfo->Svv);
-
-    if (TmpL == NULL || TmpR == NULL || M == NULL || Svv == NULL) {
-	err = 1;
-	goto eigenvals_bailout;
-    }
-
-    if (nv > n) {
-	gretl_matrix_reuse(TmpR, n, nv);
-    }    
-
-    /* calculate Suu^{-1} Suv */
-    err = gretl_invert_general_matrix(jvar->jinfo->Suu);
-    if (!err) {
-	err = gretl_matrix_multiply(jvar->jinfo->Suu, jvar->jinfo->Suv, TmpR);
-    }
-
-    /* calculate Svv^{-1} Suv' */
-    if (!err) {
-	err = gretl_invert_general_matrix(Svv);
-    }
-    if (!err) {
-	err = gretl_matrix_multiply_mod(Svv, GRETL_MOD_NONE,
-					jvar->jinfo->Suv, GRETL_MOD_TRANSPOSE, 
-					TmpL);
-    }
-
-    if (!err) {
-	err = gretl_matrix_multiply(TmpL, TmpR, M);
+    if (M == NULL) {
+	err = E_ALLOC;
+	goto bailout;
     }
 
+    err = johansen_get_eigenvalues(jvar->jinfo->Suu, jvar->jinfo->Suv, 
+				   jvar->jinfo->Svv, M, &evals, 
+				   jrank(jvar));
     if (err) {
-	goto eigenvals_bailout;
-    }    
-
-    if (!err) {
-	eigvals = gretl_general_matrix_eigenvals(M, 1, &err);
-	if (!err) {
-	    err = gretl_eigen_sort(eigvals, M, jrank(jvar));
-	}
+	goto bailout;
     }
 
 #if JDEBUG
@@ -1241,10 +1158,6 @@ johansen_bootstrap_round (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
 #endif
 
     if (!err) {
-	johansen_normalize(jvar->jinfo, M); 
-#if JDEBUG
-	gretl_matrix_print(M, "normalized M");
-#endif
 	if (jvar->jinfo->Beta == NULL) {
 	    jvar->jinfo->Beta = gretl_matrix_copy(M);
 	} else {
@@ -1264,20 +1177,16 @@ johansen_bootstrap_round (GRETL_VAR *jvar, double ***pZ, DATAINFO *pdinfo,
 	}
     } 
 
- eigenvals_bailout:    
+ bailout:    
 
-    gretl_matrix_free(TmpL);
-    gretl_matrix_free(TmpR);
     gretl_matrix_free(M);
-    gretl_matrix_free(Svv);
-
-    free(eigvals);
+    gretl_matrix_free(evals);
 
     return err;
 }
 
 static int
-johansen_LR_calc (GRETL_VAR *jvar, const double *eigvals, PRN *prn)
+johansen_LR_calc (GRETL_VAR *jvar, const gretl_matrix *evals, PRN *prn)
 {
     gretl_matrix *Suu;
     double llr = 0.0;
@@ -1299,8 +1208,8 @@ johansen_LR_calc (GRETL_VAR *jvar, const double *eigvals, PRN *prn)
     if (!err) {
 	llr = - T_2 * n * (1.0 + LN_2_PI) - T_2 * ldet;
 	for (i=0; i<h; i++) {
-	    pprintf(prn, "eigenvalue %d = %g\n", i+1, eigvals[i]);
-	    llr -= T_2 * log(1.0 - eigvals[i]); 
+	    pprintf(prn, "eigenvalue %d = %g\n", i+1, evals->val[i]);
+	    llr -= T_2 * log(1.0 - evals->val[i]); 
 	}
 	pputc(prn, '\n');
     }
@@ -1331,52 +1240,35 @@ johansen_LR_calc (GRETL_VAR *jvar, const double *eigvals, PRN *prn)
    restricted constant or trend
 */
 
-int vecm_beta_test (GRETL_VAR *jvar, PRN *prn)
+int vecm_beta_test (GRETL_VAR *jvar, const DATAINFO *pdinfo, PRN *prn)
 {
     gretl_matrix *M = NULL;
-    gretl_matrix *TmpL = NULL;
-    gretl_matrix *TmpR = NULL;
     gretl_matrix *Svv = NULL;
     gretl_matrix *Suv = NULL;
     gretl_matrix *Suu = NULL;
-
-    double *eigvals = NULL;
+    gretl_matrix *evals = NULL;
 
     int n = jvar->neqns;
-    int nv = gretl_matrix_cols(jvar->jinfo->Svv);
     int m = gretl_matrix_cols(jvar->jinfo->D);
+    int rank = jrank(jvar);
     int err = 0;
 
     M = gretl_matrix_alloc(m, m);
-    TmpL = gretl_matrix_alloc(m, n); /* FIXME n or nv? */
-    TmpR = gretl_matrix_alloc(nv, nv);
     Svv = gretl_matrix_alloc(m, m);
     Suv = gretl_matrix_alloc(n, m);
     Suu = gretl_matrix_copy(jvar->jinfo->Suu);
 
-    if (TmpL == NULL || TmpR == NULL || M == NULL || 
-	Svv == NULL || Suv == NULL || Suu == NULL) {
+    if (M == NULL || Svv == NULL || Suv == NULL || Suu == NULL) {
 	err = E_ALLOC;
 	goto bailout;
     }
 
     pputs(prn, "\nTest of restrictions on cointegrating relations\n\n");
 
-    gretl_matrix_print_to_prn(jvar->jinfo->D, "Restriction matrix, D", prn);
+    /* calculate Svv <- D' Svv D */
+    gretl_matrix_qform(jvar->jinfo->D, GRETL_MOD_TRANSPOSE,
+		       jvar->jinfo->Svv, Svv, GRETL_MOD_NONE);
 
-    /* calculate D'Svv */
-    gretl_matrix_reuse(TmpR, m, nv);
-    err = gretl_matrix_multiply_mod(jvar->jinfo->D, GRETL_MOD_TRANSPOSE,
-				    jvar->jinfo->Svv, GRETL_MOD_NONE, 
-				    TmpR);
-
-    if (!err) {
-	/* Svv <- D'SvvD */
-	err = gretl_matrix_multiply(TmpR, jvar->jinfo->D, Svv);
-    }
-
-    gretl_matrix_print_to_prn(Svv, "D'SvvD", prn);
-    
     if (!err) {
 	/* Suv <- SuvD */
 	err = gretl_matrix_multiply(jvar->jinfo->Suv, 
@@ -1384,54 +1276,37 @@ int vecm_beta_test (GRETL_VAR *jvar, PRN *prn)
 				    Suv);
     }
 
-    gretl_matrix_print_to_prn(Suv, "SuvD", prn);
+    err = johansen_get_eigenvalues(Suu, Suv, Svv, M, &evals, rank);
 
     if (!err) {
-	/* calculate Suu^{-1} Suv */
-	err = gretl_invert_general_matrix(Suu);
-	if (!err) {
-	    gretl_matrix_reuse(TmpR, n, m);
-	    err = gretl_matrix_multiply(Suu, Suv, TmpR);
-	}
-    }
-
-    /* calculate Svv^{-1} Suv' */
-    if (!err) {
-	err = gretl_invert_general_matrix(Svv);
-	if (!err) {
-	    err = gretl_matrix_multiply_mod(Svv, GRETL_MOD_NONE,
-					    Suv, GRETL_MOD_TRANSPOSE, 
-					    TmpL);
-	}
-    }
-
-    if (!err) {
-	err = gretl_matrix_multiply(TmpL, TmpR, M);
-    }
-
-    gretl_matrix_print_to_prn(M, "M", prn);
-
-    if (!err) {
-	eigvals = gretl_general_matrix_eigenvals(M, 1, &err);
-	if (!err) {
-	    err = gretl_eigen_sort(eigvals, M, jrank(jvar));
-	}
-    }
-
-    if (!err) {
-	johansen_LR_calc(jvar, eigvals, prn);
+	johansen_LR_calc(jvar, evals, prn);
     } 
 
+    if (!err) {
+	gretl_matrix_multiply_mod(jvar->jinfo->D, GRETL_MOD_NONE,
+				  M, GRETL_MOD_NONE,
+				  jvar->jinfo->Beta, GRETL_MOD_NONE);
+	err = compute_alpha(jvar->jinfo, n);
+    }
+
+    if (!err) {
+	err = phillips_normalize_beta(jvar); 
+    }
+
+    if (!err) {
+	print_beta_or_alpha(jvar->jinfo, rank, pdinfo, prn, PRINT_BETA, 0);
+	print_beta_or_alpha(jvar->jinfo, rank, pdinfo, prn, PRINT_ALPHA, 0);
+	pputc(prn, '\n');
+	compute_long_run_matrix(jvar->jinfo, n, pdinfo, prn);
+    }
+
  bailout:    
 
     gretl_matrix_free(M);
-    gretl_matrix_free(TmpL);
-    gretl_matrix_free(TmpR);
     gretl_matrix_free(Svv);
     gretl_matrix_free(Suv);
     gretl_matrix_free(Suu);
-
-    free(eigvals);
+    gretl_matrix_free(evals);
 
     return err;
 }
