@@ -179,6 +179,24 @@ unsigned char leverage_data_dialog (void)
     return flag;
 }
 
+static void 
+leverage_x_range (int t1, int t2, const double *x, FILE *fp)
+{
+    double xrange, xmin;
+    double xmin0 = x[t1];
+    double xmax = x[t2];
+
+    xrange = xmax - xmin0;
+    xmin = xmin0 - xrange * .025;
+    xmax += xrange * .025;
+
+    if (xmin0 >= 0.0 && xmin < 0.0) {
+	xmin = 0.0;
+    }
+
+    fprintf(fp, "set xrange [%.7g:%.7g]\n", xmin, xmax);
+}
+
 static int leverage_plot (const MODEL *pmod, gretl_matrix *S,
 			  double ***pZ, DATAINFO *pdinfo)
 {
@@ -205,16 +223,19 @@ static int leverage_plot (const MODEL *pmod, gretl_matrix *S,
     fputs("set size 1.0,1.0\nset multiplot\nset size 1.0,0.48\n", fp);
     fputs("set xzeroaxis\n", fp);
     fputs("set nokey\n", fp); 
-    if (obs == NULL) { 
+
+    if (obs != NULL) {
+	leverage_x_range(pmod->t1, pmod->t2, obs, fp);
+    } else {
 	fprintf(fp, "set xrange [%g:%g]\n", 
 		pmod->t1 + 0.5, pmod->t2 + 1.5);
-    }
+    } 
 
     /* upper plot: leverage factor */
     fputs("set origin 0.0,0.50\n", fp);
     fputs("set missing '?'\n", fp);
     fputs("set yrange [0:1]\n", fp);
-    fprintf(fp, "set title '%s'\n", I_("leverage"));
+    fprintf(fp, "set title '%s'\n", G_("leverage"));
     fputs("plot \\\n'-' using 1:2 w impulses\n", fp);
 
     for (t=pmod->t1; t<=pmod->t2; t++) {
@@ -240,7 +261,7 @@ static int leverage_plot (const MODEL *pmod, gretl_matrix *S,
     fputs("set origin 0.0,0.0\n", fp);
     fputs("set missing '?'\n", fp);
     fputs("set yrange [*:*]\n", fp);
-    fprintf(fp, "set title '%s'\n", I_("influence")); 
+    fprintf(fp, "set title '%s'\n", G_("influence")); 
     fputs("plot \\\n'-' using 1:2 w impulses\n", fp);
 
     for (t=pmod->t1; t<=pmod->t2; t++) {
@@ -261,6 +282,7 @@ static int leverage_plot (const MODEL *pmod, gretl_matrix *S,
 	}
     }
     fputs("e\n", fp);
+
     fputs("set nomultiplot\n", fp);
 
     gretl_pop_c_numeric_locale();
@@ -533,7 +555,7 @@ gretl_matrix *model_leverage (const MODEL *pmod, double ***pZ,
     }
 
     if (S != NULL) {
-	gretl_matrix_set_int(S, pmod->t1);
+	gretl_matrix_set_t1(S, pmod->t1);
     }
 
     return S;    
