@@ -681,7 +681,8 @@ save_and_print_results (gretl_equation_system *sys,
     if (system_save_uhat(sys)) {
 	j = pdinfo->v;
 	err = dataset_add_series(m, pZ, pdinfo);
-    }
+    } 
+
     if (system_save_yhat(sys)) {
 	if (j == 0) {
 	    j = pdinfo->v;
@@ -1146,7 +1147,7 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 
 	    err = gretl_matrix_multiply_mod(Xi, GRETL_MOD_TRANSPOSE,
 					    Xk, GRETL_MOD_NONE, 
-					    M);
+					    M, GRETL_MOD_NONE);
 
 	    if (rtsls || (single_equation && method != SYS_WLS)) {
 		sij = 1.0;
