@@ -682,7 +682,7 @@ int find_coeff_number (const MODEL *pmod, const char *param)
     }
 
     for (i=0; i<pmod->ncoeff; i++) {
-	if (!strcmp(param, pmod->params[i+1])) {
+	if (!strcmp(param, pmod->params[i])) {
 	    return i;
 	}
     }
@@ -807,13 +807,9 @@ static int real_run_check (int round, PRN *prn)
 	*pmod = nls(&Z, datainfo, OPT_NONE, prn);
 
 	if (pmod->errcode) {
-	    char errtext[128];
-
 	    err = pmod->errcode;
-	    if (get_errmsg(err, errtext, NULL) != NULL) {
-		fprintf(stderr, "%s: ERROR: model error %d (%s)\n", 
-			tester.datname, err, errtext);
-	    }
+	    fprintf(stderr, "%s: ERROR: model error %d\n", 
+		    tester.datname, err);
 	    errmsg(err, prn);
 	} else {
 	    if (verbose) {
@@ -890,7 +886,7 @@ static int run_gretl_nls_check (void)
     if (verbose) {
 	prn = gretl_print_new(GRETL_PRINT_STDOUT);
     } else {
-	prn = gretl_print_new(GRETL_PRINT_NULL);
+	prn = NULL;
     }
 
     err1 = real_run_check(1, prn);
