@@ -9,45 +9,79 @@
 
 #ifdef WIN32
 # include <windows.h>
-# include <winsock.h>
+# include <shellapi.h>
 #endif
 
-#ifdef USE_GTK
-# include <gtk/gtk.h>
-#endif
+#define ERRLEN 256
+#define MAXLEN 512
+#define UBUFSIZE 8192
+#define E_ALLOC 15
+#define E_DATA 2
 
 #include "updater.h"
-#include "webget.h"
 
 FILE *flg;
 int logit;
 int debug;
 
-#define MAXLEN 512
-
 enum {
     UPDATER_DEFAULT,
     UPDATER_GET_LISTING,
     UPDATER_GET_FILE
 } program_opts;
 
-static char errbuf[256];
 static char get_fname[48];
+static char gretl_errmsg[ERRLEN];
 static int ask_before_download = 1;
 static time_t filedate;
 static int argcount;
 static int prog_opt;
 
-#ifdef USE_GTK
-static GtkWidget *mainwin;
-static GtkWidget *mainlabel;
-static char infobuf[256];
-static void put_text_on_window (void);
-#else
-# define GTK_RESPONSE_ACCEPT IDYES
-# define GTK_RESPONSE_NO     IDNO
+#define STANDALONE
+#include "gretl_www.c"
+
+#ifndef WIN32
+# define IDYES 1
 #endif
 
+#ifndef TRUE
+# define TRUE 1
+#endif
+
+/* Replication (or stubs) for functionality usually provided
+   by libgretl (which we don't want to depend on in this context)
+*/
+
+FILE *gretl_fopen (const char *filename, const char *mode)
+{
+    FILE *fp = NULL;
+
+    fp = fopen(filename, mode);
+
+    return fp;
+}
+
+int show_progress (long res, long expected, int flag)
+{
+    return 0;
+}
+
+char *gretl_strdup (const char *src)
+{
+    char *targ = NULL;
+
+    if (src != NULL) {
+	targ = malloc(strlen(src) + 1);
+	if (targ != NULL) {
+	    strcpy(targ, src);
+	}
+    }
+
+    return targ;
+}
+
+/* end replication/stubs */
+
 static void getout (int err)
 {
     if (logit) {
@@ -75,6 +109,35 @@ enum {
 
 static int gretl_run_status;
 
+static void ws_cleanup (void)
+{
+    WSACleanup();
+}
+
+int ws_startup (void)
+{
+    WORD requested;
+    WSADATA data;
+
+    requested = MAKEWORD(1, 1);
+
+    if (WSAStartup(requested, &data)) {
+	fprintf(stderr, "Couldn't find usable socket driver\n");
+	return 1;
+    }
+
+    if (LOBYTE (requested) < 1 || (LOBYTE (requested) == 1 &&
+				   HIBYTE (requested) < 1)) {
+	fprintf(stderr, "Couldn't find usable socket driver\n");
+	WSACleanup();
+	return 1;
+    }
+
+    atexit(ws_cleanup);
+
+    return 0;
+}
+
 static int win_error (void)
 {
     LPVOID msg;
@@ -130,56 +193,65 @@ static int create_child_process (char *prog)
 
 static int yes_no_dialog (const char *msg)
 {
-    int ret;
-
-    ret = MessageBox(NULL, msg, "gretl updater", 
-		     MB_YESNO | MB_ICONQUESTION);
-
-    if (ret == IDYES) {
-	return GTK_RESPONSE_ACCEPT;
-    } else {
-	return GTK_RESPONSE_NO;
-    }
+    return MessageBox(NULL, msg, "gretl updater", 
+		      MB_YESNO | MB_ICONQUESTION);
 }
 
 static int msgbox (const char *msg, int err)
 {
-    int ret;
-
     if (err) {
-        ret = MessageBox(NULL, msg, "gretl updater", MB_OK | MB_ICONERROR);
+        return MessageBox(NULL, msg, "gretl updater", MB_OK | MB_ICONERROR);
     } else {
-        ret = MessageBox(NULL, msg, "gretl updater", MB_OK | MB_ICONINFORMATION);
+        return MessageBox(NULL, msg, "gretl updater", MB_OK | MB_ICONINFORMATION);
     }
-
-    return ret;
 }
 
-static void ws_cleanup (void)
+static int read_reg_val (HKEY tree, char *keyname, char *keyval)
 {
-    WSACleanup();
-}
+    unsigned long datalen = MAXLEN;
+    int error = 0;
+    HKEY regkey;
 
-static int ws_startup (void)
-{
-    WORD requested;
-    WSADATA data;
-
-    requested = MAKEWORD(1, 1);
-
-    if (WSAStartup(requested, &data)) {
-        errbox("Couldn't find usable socket driver");
+    if (RegOpenKeyEx(
+                     tree,                        /* handle to open key */
+                     "Software\\gretl",           /* subkey name */
+                     0,                           /* reserved */
+                     KEY_READ,                    /* access mask */
+                     &regkey                      /* key handle */
+                     ) != ERROR_SUCCESS) {
+        fprintf(stderr, "couldn't open registry\n");
         return 1;
     }
 
-    if (LOBYTE (requested) < 1 || (LOBYTE (requested) == 1 &&
-                                   HIBYTE (requested) < 1)) {
-        errbox("Couldn't find usable socket driver");
-        WSACleanup();
-        return 1;
+    if (RegQueryValueEx(
+                        regkey,
+                        keyname,
+                        NULL,
+                        NULL,
+                        keyval,
+                        &datalen
+                        ) != ERROR_SUCCESS) {
+        error = 1;
+    }
+
+    RegCloseKey(regkey);
+
+    return error;
+}
+
+static void read_proxy_info (int *use_proxy, char *dbproxy) 
+{
+    char val[128];
+
+    if (read_reg_val(HKEY_CURRENT_USER, "useproxy", val) == 0) {
+	if (!strcmp(val, "true") || !strcmp(val, "1")) {
+	    *use_proxy = 1;
+	}
+    }
+
+    if (use_proxy && read_reg_val(HKEY_CURRENT_USER, "dbproxy", val) == 0) {
+        strncat(dbproxy, val, 20);
     }
-    atexit(ws_cleanup);
-    return 0;
 }
 
 BOOL CALLBACK record_gretl_running (HWND hw, LPARAM lp)
@@ -211,59 +283,27 @@ static int check_for_gretl (void)
     return gretl_run_status;
 }
 
-#else /* ! WIN32 */
+#else /* not WIN32 */
 
-gint yes_no_dialog (char *msg)
+static int yes_no_dialog (const char *msg)
 {
-    GtkWidget *dialog, *label, *hbox;
-    int ret;
+    printf("%s\n", msg);
 
-    dialog = gtk_dialog_new_with_buttons ("gretl updater",
-					  NULL,
-					  GTK_DIALOG_MODAL | 
-					  GTK_DIALOG_DESTROY_WITH_PARENT,
-					  GTK_STOCK_YES,
-					  GTK_RESPONSE_ACCEPT,
-					  GTK_STOCK_NO,
-					  GTK_RESPONSE_NO,
-					  NULL);
-    
-    label = gtk_label_new (msg);
-    gtk_widget_show(label);
-    hbox = gtk_hbox_new(FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 10);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
-		       hbox, FALSE, FALSE, 10);
-
-    ret = gtk_dialog_run(GTK_DIALOG(dialog));
-					  
-    gtk_widget_destroy(dialog);
-
-    switch (ret) {
-    case GTK_RESPONSE_ACCEPT: 
-	return ret;
-    default: 
-	return GTK_RESPONSE_NO;
-    }
+    return 0;
 }
 
 static int msgbox (const char *msg, int err)
 {
-    GtkWidget *dialog;
+    if (err) {
+	printf("Error: %s\n", msg);
+    } else {
+	printf("Info: %s\n", msg);
+    }
 
-    dialog = gtk_message_dialog_new (NULL, 
-				     GTK_DIALOG_DESTROY_WITH_PARENT,
-				     (err)? GTK_MESSAGE_ERROR : GTK_MESSAGE_INFO,
-				     GTK_BUTTONS_CLOSE,
-				     msg);
-    gtk_dialog_run (GTK_DIALOG (dialog));
-    gtk_widget_destroy (dialog);
-    
     return 0;
 }
 
-#endif /* WIN32 vs ! WIN32 */
+#endif /* WIN32 or not */
 
 static char *get_size_string (size_t fsize)
 {
@@ -280,15 +320,6 @@ static char *get_size_string (size_t fsize)
     return sizestr;
 }
 
-#ifdef USE_GTK
-
-static void put_text_on_window (void)
-{
-    gtk_label_set_text(GTK_LABEL(mainlabel), infobuf);
-}
-
-#endif
-
 int errbox (const char *msg) 
 {
     return msgbox(msg, 1);
@@ -299,7 +330,36 @@ int infobox (const char *msg)
     return msgbox(msg, 0);
 }
 
-void listerr (char *buf, char *fname)
+static int files_query (char **getbuf, time_t filedate)
+{
+    int use_proxy = 0;
+    char dbproxy[21] = {0};
+
+#ifdef WIN32
+    read_proxy_info(&use_proxy, dbproxy);
+#endif
+
+    gretl_www_init(RICARDO, dbproxy, use_proxy);
+
+    return get_update_info(getbuf, filedate, QUERY_SILENT); 
+}
+
+static int get_remote_file (const char *fname)
+{
+    int use_proxy = 0;
+    char dbproxy[21] = {0};
+
+#ifdef WIN32
+    read_proxy_info(&use_proxy, dbproxy);
+#endif
+
+    gretl_www_init(RICARDO, dbproxy, use_proxy);
+
+    return retrieve_url(RICARDO, GRAB_FILE, fname, NULL, SAVE_TO_FILE, 
+			fname, NULL);
+}
+
+static void listerr (const char *fname)
 {
     char mybuf[256];
 
@@ -317,15 +377,15 @@ void listerr (char *buf, char *fname)
 	}
     }
 
-    if (*buf != '\0') {
+    if (*gretl_errmsg != '\0') {
 	strcat(mybuf, "\n");
-	strcat(mybuf, buf);
+	strcat(mybuf, gretl_errmsg);
     }
 
     errbox(mybuf);
 }
 
-void usage (char *prog)
+static void usage (char *prog)
 {
     char usebuf[1024];
 
@@ -338,7 +398,7 @@ void usage (char *prog)
     getout(0);
 }
 
-time_t get_time_from_stamp_file (const char *fname)
+static time_t get_time_from_stamp_file (const char *fname)
      /* E.g. Sun Mar 16 13:50:52 EST 2003 */
 {
     FILE *fp;
@@ -403,31 +463,28 @@ static int real_program (void)
     if (argcount == 1) {
 	/* no arguments: a default update */
 
-#ifdef USE_GTK
-	strcpy(infobuf, "Looking for gretl updates...");
-	put_text_on_window();
-#endif
-
 	if (logit) {
 	    fputs("doing default update (argcount = 1)\n", flg);
 	}
 
-	getbuf = mymalloc(GRETL_BUFSIZE);
+	getbuf = malloc(UBUFSIZE);
 	if (getbuf == NULL) return 1;
 
-	clear(getbuf, GRETL_BUFSIZE);
+	memset(getbuf, 0, UBUFSIZE);
 
 	if (logit) {
 	    fputs("getbuf allocated OK\n", flg);
 	}
 
-	err = files_query(&getbuf, errbuf, filedate);
+	err = files_query(&getbuf, filedate);
 	if (err) {
-	   listerr(errbuf, NULL);
+	   listerr(NULL);
 	   getout(1);
 	}
 
-	if (logit) fputs("call to files_query: success\n", flg);
+	if (logit) {
+	    fputs("call to files_query: success\n", flg);
+	}
 
 	i = 0;
 	while ((line = strtok((i)? NULL: getbuf, "\n"))) {
@@ -455,26 +512,20 @@ static int real_program (void)
 		sprintf(query, "An update file is available%s.\n"
 			"Get it now?", get_size_string(fsize));
 		resp = yes_no_dialog(query);
-		if (resp != GTK_RESPONSE_ACCEPT) break;
+		if (resp != IDYES) break;
 	    }
 
-#ifdef USE_GTK
-	    sprintf(infobuf, "Downloading %s", get_fname);
-	    put_text_on_window();    
-	    while (gtk_events_pending()) gtk_main_iteration();
-#endif
-	    
 	    if (logit) {
 		fprintf(flg, "trying to get '%s'\n", get_fname);
 	    }
 
-	    err = get_remote_file(get_fname, errbuf);
+	    err = get_remote_file(get_fname);
 
 	    if (logit) {
 		fprintf(flg, "get_remote_file() returned %d\n", err);
 	    }	    
 	    if (err) {
-		listerr(errbuf, get_fname);
+		listerr(get_fname);
 		return 1;
 	    }
 
@@ -495,14 +546,15 @@ static int real_program (void)
 		err = 1;
 	    }
 	}
-    }
-
-    else if (prog_opt == UPDATER_GET_LISTING) { /* get listing */
-	getbuf = malloc(8192); 
-	clear(getbuf, 8192);
-	err = files_query(&getbuf, errbuf, filedate);
+    } else if (prog_opt == UPDATER_GET_LISTING) { /* get listing */
+	getbuf = malloc(UBUFSIZE); 
+	if (getbuf == NULL) {
+	    getout(1);
+	}
+	memset(getbuf, 0, UBUFSIZE);
+	err = files_query(&getbuf, filedate);
 	if (err) {
-	    listerr(errbuf, NULL);
+	    listerr(NULL);
 	    free(getbuf);
 	    getout(1);
 	}
@@ -510,17 +562,17 @@ static int real_program (void)
 	    infobox(getbuf);
 	}
 	free(getbuf);
-    }
-    
-    else if (prog_opt == UPDATER_GET_FILE) { /* get a specified file */
-	err = get_remote_file(get_fname, errbuf);
+    } else if (prog_opt == UPDATER_GET_FILE) { /* get a specified file */
+	err = get_remote_file(get_fname);
 	if (err) {
-	    listerr(errbuf, get_fname);
+	    listerr(get_fname);
 	    getout(1);
 	}
 	tarerr = untgz(get_fname);
 	remerr = remove(get_fname);
-	if (!err && !tarerr && !remerr) unpack_ok = 1;
+	if (!err && !tarerr && !remerr) {
+	    unpack_ok = 1;
+	}
     }
 
 #ifdef WIN32
@@ -528,7 +580,7 @@ static int real_program (void)
 	int resp;
 
 	resp = yes_no_dialog("gretl update succeeded.\r\nStart gretl now?");
-	if (resp == GTK_RESPONSE_ACCEPT) {
+	if (resp == IDYES) {
 	    create_child_process("gretlw32.exe");
 	}
     }
@@ -540,70 +592,10 @@ static int real_program (void)
 
     getout(err);
 
-#ifdef USE_GTK
-    gtk_main_quit();
-#endif
-
     return TRUE;
 }
 
-#ifdef USE_GTK
-
-static gint cancel_callback (void)
-{
-    gtk_main_quit();
-    return FALSE;
-}
-
-static void create_main_gtk_window (int warn)
-{
-    GtkWidget *main_vbox;
-    GtkWidget *buttonbox;
-    GtkWidget *tmp;
-    int mainwin_width = 300;
-    int mainwin_height = 100;
-
-    mainwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title(GTK_WINDOW(mainwin), "gretl updater");
-    gtk_window_set_default_size(GTK_WINDOW(mainwin), 
-				mainwin_width, mainwin_height);
-    
-    main_vbox = gtk_vbox_new(FALSE, 4);
-    gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 8);
-    gtk_container_add(GTK_CONTAINER(mainwin), main_vbox);
-
-    if (warn) {
-	mainlabel = gtk_label_new("If gretl is running, please close it "
-				  "down before running the updater.\n\n"
-				  "Press OK to proceed or Cancel to quit");
-    } else {
-	mainlabel = gtk_label_new("Press OK to proceed or Cancel to quit");
-    }
-	
-    gtk_box_pack_start(GTK_BOX(main_vbox), mainlabel, TRUE, TRUE, 0);
-
-    buttonbox = gtk_hbox_new(TRUE, 4);
-    gtk_container_set_border_width(GTK_CONTAINER(buttonbox), 8);
-    gtk_box_pack_start(GTK_BOX(main_vbox), buttonbox, FALSE, FALSE, 0);
-
-    /* Create an "OK" button */
-    tmp = gtk_button_new_from_stock(GTK_STOCK_OK);
-    GTK_WIDGET_SET_FLAGS(tmp, GTK_CAN_DEFAULT);
-    gtk_container_add(GTK_CONTAINER(buttonbox), tmp);
-    g_signal_connect (G_OBJECT(tmp), "clicked", 
-		      G_CALLBACK(real_program), NULL);
-    gtk_widget_grab_default (tmp);
-
-    /* Create a "Cancel" button */
-    tmp = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-    gtk_container_add(GTK_CONTAINER(buttonbox), tmp);
-    g_signal_connect (G_OBJECT(tmp), "clicked", 
-		      G_CALLBACK(cancel_callback), NULL);
-    
-    gtk_widget_show_all(mainwin);
-}
-
-#else 
+#ifdef WIN32 
 
 static void win32_start (int warn)
 {
@@ -625,19 +617,9 @@ static void win32_start (int warn)
     }
 }
 
-int show_progress (long res, long expected, int flag)
-{
-    return 0;
-}
-
 #endif
 
-#ifdef WIN32
 int main (int argc, char *argv[])
-#else
-int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, 
-		    LPSTR CmdLine, int Show)
-#endif
 {
     const char *testfile = "gretl.stamp";
     int warn = 1;
@@ -677,7 +659,7 @@ int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
     }    
 
 #ifdef WIN32
-    if (read_reg_val(HKEY_CLASSES_ROOT, "gretldir", gretldir)) {
+    if (read_reg_val(HKEY_LOCAL_MACHINE, "gretldir", gretldir)) {
 	errbox("Couldn't get the path to the gretl installation\n"
 	       "from the Windows registry");
 	exit(EXIT_FAILURE);
@@ -698,7 +680,9 @@ int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
 	}
     }
 
-    if (ws_startup()) exit(EXIT_FAILURE);
+    if (ws_startup()) {
+	exit(EXIT_FAILURE);
+    }
 #endif
 
     flg = fopen("updater.log", "w");
@@ -715,16 +699,12 @@ int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
     filedate = get_time_from_stamp_file(testfile);
 
     if (filedate == (time_t) 0) {
-	sprintf(errbuf, "Couldn't get time-stamp from file '%s'", testfile);
-	errbox(errbuf);
+	sprintf(gretl_errmsg, "Couldn't get time-stamp from file '%s'", testfile);
+	errbox(gretl_errmsg);
 	getout(1);
     } 
 
-#ifdef USE_GTK
-    gtk_init(&argc, &argv);
-    create_main_gtk_window(warn);
-    gtk_main();
-#else
+#ifdef WIN32
     win32_start(warn);
 #endif
 
